SystemOrganization addCategory: #'System-Announcements'!
SystemOrganization addCategory: #'System-Applications'!
SystemOrganization addCategory: #'System-Change Notification'!
SystemOrganization addCategory: #'System-Changes'!
SystemOrganization addCategory: #'System-Clipboard'!
SystemOrganization addCategory: #'System-CommandLine'!
SystemOrganization addCategory: #'System-FilePackage'!
SystemOrganization addCategory: #'System-FileRegistry'!
SystemOrganization addCategory: #'System-Finalization'!
SystemOrganization addCategory: #'System-Hashing-Core'!
SystemOrganization addCategory: #'System-Hashing-DSA'!
SystemOrganization addCategory: #'System-Hashing-MD5'!
SystemOrganization addCategory: #'System-Hashing-SHA1'!
SystemOrganization addCategory: #'System-History'!
SystemOrganization addCategory: #'System-Installers'!
SystemOrganization addCategory: #'System-Localization'!
SystemOrganization addCategory: #'System-Object Events'!
SystemOrganization addCategory: #'System-Platforms'!
SystemOrganization addCategory: #'System-Serial Port'!
SystemOrganization addCategory: #'System-Settings-Browser'!
SystemOrganization addCategory: #'System-Settings-Core'!
SystemOrganization addCategory: #'System-Settings-Filter'!
SystemOrganization addCategory: #'System-Sound'!
SystemOrganization addCategory: #'System-Support'!
SystemOrganization addCategory: #'System-Tools'!
SystemOrganization addCategory: #'System-PragmaCollector'!

!FileStream methodsFor: '*System-Changes' stamp: 'StephaneDucasse 3/23/2010 22:42'!
fileIntoNewChangeSet
	"File all of my contents into a new change set." 

	self readOnly.
	ChangeSet newChangesFromStream: self named: (self localName)
! !

IdentityDictionary subclass: #SystemDictionary
	instanceVariableNames: 'cachedClassNames cachedNonClassNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!SystemDictionary commentStamp: 'nice 3/6/2010 21:56' prior: 0!
I represent a special dictionary used as global namespace for class names :

	Smalltalk globals classNames.

and for traits too:

	Smalltalk globals traitNames.
	
and a few other globals:

	(Smalltalk globals keys reject: [:k | (Smalltalk globals at: k) isBehavior])
			collect: [:k | k -> (Smalltalk globals at: k) class].

As the above example let you guess, the global namespace of Smalltalk system is accessed through:

	Smalltalk globals.!

!SystemDictionary class methodsFor: 'instance creation' stamp: ''!
withOrganizer: anOrganizer
	| dictionary |
	dictionary := self new.
	dictionary organization: anOrganizer. "update too the back pointer"
	^dictionary.
	! !

!SystemDictionary methodsFor: 'classes and traits' stamp: 'MarcusDenker 4/21/2011 14:54'!
allClasses  
	"Return all the class defines in the Smalltalk SystemDictionary"

	^ self classNames collect: [:name | self at: name]! !

!SystemDictionary methodsFor: 'classes and traits' stamp: 'MarcusDenker 4/21/2011 13:51'!
allClassesAndTraits
	"Return all the classes and traits defined in the Smalltalk SystemDictionary"

	^ self classAndTraitNames collect: [:each | self at: each]! !

!SystemDictionary methodsFor: 'classes and traits' stamp: 'MarcusDenker 4/30/2011 07:47'!
allClassesAndTraitsDo: aBlock
	"Evaluate the argument, aBlock, for each class and trait in the system."
	
	^self classAndTraitNames do: [:each | aBlock value: (self at: each)]
	
	
! !

!SystemDictionary methodsFor: 'classes and traits' stamp: 'MarcusDenker 4/30/2011 10:51'!
allClassesDo: aBlock
	"Evaluate the argument, aBlock, for each class in the system."

	^self classNames do: [:name | aBlock value: (self at: name)]! !

!SystemDictionary methodsFor: 'classes and traits' stamp: ''!
allTraits
	"Return all traits defined in the Smalltalk SystemDictionary"

	^ self traitNames collect: [:each | self at: each]! !

!SystemDictionary methodsFor: 'classes and traits' stamp: 'MarcusDenker 4/30/2011 10:51'!
allTraitsDo: aBlock
	"Evaluate the argument, aBlock, for each trait in the system."

	^self traitNames do: [:name | aBlock value: (self at: name)]! !

!SystemDictionary methodsFor: 'dictionary access' stamp: ''!
associationOrUndeclaredAt: key 
	"return an association or install in undeclared.  Used for mating up ImageSegments."

	^ self associationAt: key ifAbsent: [
		Undeclared at: key put: nil.
		Undeclared associationAt: key]! !

!SystemDictionary methodsFor: 'dictionary access' stamp: ''!
at: aKey put: anObject 
	"Override from Dictionary to check Undeclared and fix up
	references to undeclared variables."
	| index element |
	(self includesKey: aKey) ifFalse: 
		[self declare: aKey from: Undeclared.
		self flushClassNameCache].
	super at: aKey put: anObject.
	^ anObject! !

!SystemDictionary methodsFor: 'accessing' stamp: ''!
bindingOf: varName
	"SystemDictionaries includes symbols only"
	
	^ super bindingOf: varName asSymbol! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'MarcusDenker 4/20/2011 13:44'!
classAndTraitNames
	"Answer a sorted collection of all class and trait (not including class-traits) names. The performance of 
	this algorithm is O(n) if the classNames are already cached, otherwise O(n*log(n)) where n is self size."

	| classNames traitNames result temporary |
	classNames := self classNames.
	traitNames := self traitNames.
	temporary := Array new: classNames size + traitNames size.
	result := temporary shallowCopy.
	temporary
		replaceFrom: 1
		to: classNames size
		with: classNames
		startingAt: 1;
		replaceFrom: classNames size + 1
		to: temporary size
		with: traitNames
		startingAt: 1;
		mergeFirst: 1
		middle: classNames size
		last: temporary size
		into: result
		by: nil.
	^result! !

!SystemDictionary methodsFor: 'classes and traits' stamp: ''!
classNamed: className 
	^self classOrTraitNamed: className.! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'ul 10/8/2010 21:22'!
classNames
	"Answer a sorted collection of all class names. Use the return value of #fillCaches to avoid concurrency issues."
	
	^cachedClassNames ifNil: [ self fillCaches at: 1 ]! !

!SystemDictionary methodsFor: 'classes and traits' stamp: 'md 3/12/2011 15:28'!
classOrTraitNamed: aString
	"aString is either a class or trait name or a class or trait name followed by ' class' or 'classTrait' respectively.
	Answer the class or metaclass it names."

	| meta baseName |
	(aString endsWith: ' class')
		ifTrue: [ 
			meta := true.
			baseName := aString copyFrom: 1 to: aString size - 6 ]
		ifFalse: [ 
			(aString endsWith: ' classTrait')
				ifTrue: [ 
					meta := true.
					baseName := aString copyFrom: 1 to: aString size - 11 ]
				ifFalse: [ 
					meta := false.
					baseName := aString ] ].
	^self at: baseName asSymbol ifPresent:
            [ :global |
               (global isBehavior or: [global isTrait])ifTrue:
                       [ meta
                               ifFalse: [ global ]
                               ifTrue: [ global classSide ]]]! !

!SystemDictionary methodsFor: 'accessing' stamp: ''!
environment
	"for conversion from SmalltalkImage to SystemDictionary"
	
	^ self ! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'StephaneDucasse 6/6/2011 21:13'!
fillCaches
	"Fill cachedClassNames and cachedNonClassNames. Return an array with the calculated values."

	| classNames nonClassNames |
	classNames := OrderedCollection new: self size.
	nonClassNames := OrderedCollection new.
	self keysAndValuesDo: [ :key :value |
		"The key == value name test below addresses two separate issues:
			1) Obsolete classes, where key = #Foo and value name = 'AnObsoleteFoo'
			2) Aliases, i.e., Smalltalk at: #OtherName put: aClass"
		((value isKindOf: (self class environment at: #Class)) and: [ key == value name ])
			ifTrue: [ classNames add: key ]
			ifFalse: [ nonClassNames add: key ] ].
		"The expression (self class environment at: #Class) deserves some explanation.
		For bootstrapping we need to have a different Class when executing value isKindOf: Class :
		In such expression Class represents a kind of fixed point: the class of the class of the system. 
		When bootstrapping we want the class Class of the current kernel defined in the current namespace.
		Since the current namespace should contains the class that describes itself as well as a new Class class.
		we are done :). - StephaneDucasse"
		
	classNames sort.
	cachedNonClassNames := nonClassNames sort.
	cachedClassNames := classNames.
	^{ classNames. nonClassNames }! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'ul 10/8/2010 21:23'!
flushClassNameCache
	"Smalltalk flushClassNameCache"
	"Force recomputation of the cached list of class names and non-class names."

	cachedClassNames := cachedNonClassNames := nil! !

!SystemDictionary methodsFor: 'classes and traits' stamp: 'EstebanLorenzano 7/27/2012 16:30'!
forgetClass: aClass logged: aBool 
	"Delete the class, aClass, from the system.
	Note that this doesn't do everything required to dispose of a class - to do that use Class>>removeFromSystem."

	aBool ifTrue: [SystemAnnouncer uniqueInstance classRemoved: aClass fromCategory: aClass category].		
	self organization removeElement: aClass name.
	Smalltalk removeFromStartUpList: aClass.
	Smalltalk removeFromShutDownList: aClass.
	self removeKey: aClass name ifAbsent: [].
	self flushClassNameCache! !

!SystemDictionary methodsFor: 'testing' stamp: 'MarcusDenker 11/30/2010 11:10'!
hasBindingThatBeginsWith: aString
	"Use the cached class and non-class names for better performance."

	| name searchBlock |
	searchBlock := [ :element |
		(element beginsWith: aString)
			ifTrue: [ 0 ]
			ifFalse: [
				aString < element
					ifTrue: [ -1 ]
					ifFalse: [ 1 ] ] ].
	name := self classNames 
		findBinary: searchBlock
		ifNone: [nil].
	name ifNotNil: [ ^true ].
	name := self nonClassNames 
		findBinary: searchBlock
		ifNone: [nil].
	^name notNil! !

!SystemDictionary methodsFor: 'classes and traits' stamp: ''!
hasClassNamed: aString
	"Answer whether there is a class of the given name, but don't intern aString if it's not alrady interned."

	Symbol hasInterned: aString ifTrue: 
		[:aSymbol | ^ (self at: aSymbol ifAbsent: [nil]) isKindOf: Class].
	^ false! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'ul 10/8/2010 21:22'!
nonClassNames
	"Answer a sorted collection of all non-class names. Use the return value of #fillCaches to avoid concurrency issues."
	
	^cachedNonClassNames ifNil: [ self fillCaches at: 2 ]! !

!SystemDictionary methodsFor: 'accessing' stamp: ''!
organization
	"Return the organizer for the receiver"
	
	^ self at: #SystemOrganization! !

!SystemDictionary methodsFor: 'accessing' stamp: ''!
organization: anOrganization
	"Return the organizer for the receiver"
	
	anOrganization environment: self.
	^self at: #SystemOrganization put: anOrganization! !

!SystemDictionary methodsFor: 'accessing' stamp: ''!
poolUsers
	"Answer a dictionary of pool name -> classes that refer to it.
	Also includes any globally know dictionaries (such as
	Smalltalk, Undeclared etc) which although not strictly
	accurate is potentially useful information"
	"Smalltalk poolUsers"
	| poolUsers |
	poolUsers := Dictionary new.
	self keys
		do: [:k | | pool refs | "yes, using isKindOf: is tacky but for reflective code like
			this it is very useful. If you really object you can:-
			a) go boil your head.
			b) provide a better answer.
			your choice."
			(((pool := self at: k) isKindOf: Dictionary)
					or: [pool isKindOf: SharedPool class])
				ifTrue: [refs := self systemNavigation allClasses
								select: [:c | c sharedPools identityIncludes: pool]
								thenCollect: [:c | c name].
					refs
						add: (self systemNavigation
								allReferencesTo: (self associationAt: k)).
					poolUsers at: k put: refs]].
	^ poolUsers! !

!SystemDictionary methodsFor: 'printing' stamp: ''!
printElementsOn: aStream
	aStream nextPutAll:'(lots of globals)'! !

!SystemDictionary methodsFor: 'classes and traits' stamp: 'StephaneDucasse 11/7/2011 22:44'!
removeClassNamed: aName
	"Invoked from fileouts:  if there is currently a class in the system named aName, then remove it.  If anything untoward happens, report it in the Transcript.  "

	| oldClass |
	(oldClass := self at: aName asSymbol ifAbsent: [nil]) == nil
		ifTrue:
			[self crTrace: 'Removal of class named ', aName, ' ignored because ', aName, ' does not exist.'.
			^ self].

	oldClass removeFromSystem! !

!SystemDictionary methodsFor: 'dictionary access' stamp: ''!
removeKey: key ifAbsent: aBlock
	"Remove key (and its associated value) from the receiver. If key is not in
	the receiver, answer the result of evaluating aBlock. Otherwise, answer
	the value externally named by key."
	
	self flushClassNameCache.
	^super removeKey: key ifAbsent: aBlock! !

!SystemDictionary methodsFor: 'renaming' stamp: ''!
renameClass: aClass as: newName 
	"Rename the class, aClass, to have the title newName."

	^self renameClass: aClass from: aClass name to: newName! !

!SystemDictionary methodsFor: 'renaming' stamp: ''!
renameClass: aClass from: oldName 
	"Rename the class, aClass, to have the title newName."

	^self renameClass: aClass from: oldName to: aClass name! !

!SystemDictionary methodsFor: 'renaming' stamp: 'EstebanLorenzano 7/27/2012 16:30'!
renameClass: aClass from: oldName to: newName
	"Rename the class, aClass, to have the title newName."

	| oldref category |
	category := self organization categoryOfElement: oldName.
	self organization classify: newName under: category.
	self organization removeElement: oldName.
	oldref := self associationAt: oldName.
	self removeKey: oldName.
	oldref key: newName.
	self add: oldref.  "Old association preserves old refs"
	Smalltalk renamedClass: aClass from: oldName to: newName.
	self flushClassNameCache.
	SystemAnnouncer uniqueInstance classRenamed: aClass from: oldName to: newName inCategory: category! !

!SystemDictionary methodsFor: 'renaming' stamp: 'StephaneDucasse 11/7/2011 22:44'!
renameClassNamed: oldName as: newName
	"Invoked from fileouts:  if there is currently a class in the system named oldName, then rename it to newName.  If anything untoward happens, report it in the Transcript.  "

	| oldClass |
	(oldClass := self at: oldName asSymbol ifAbsent: [nil]) == nil
		ifTrue:
			[self  crTrace: 'Class-rename for ', oldName, ' ignored because ', oldName, ' does not exist.'.
			^ self].

	oldClass rename: newName! !

!SystemDictionary methodsFor: 'class and trait names' stamp: 'MarcusDenker 4/30/2011 08:25'!
traitNames
	"Answer a SortedCollection of all traits (not including class-traits) names."
	
	^self nonClassNames select: [ :name |
			self
				at: name
				ifPresent: [ :global | (global isTrait) and: [global isObsolete not]]
				ifAbsent: [ false ] ]! !

!SystemDictionary methodsFor: 'copying' stamp: ''!
veryDeepCopyWith: deepCopier
	"Return self.  I can't be copied.  Do not record me."! !

WeakActionSequence variableSubclass: #WeakActionSequenceTrappingErrors
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!WeakActionSequenceTrappingErrors methodsFor: 'converting' stamp: 'rw 7/20/2003 16:03'!
asActionSequenceTrappingErrors

	^self! !

!WeakActionSequenceTrappingErrors methodsFor: 'evaluating' stamp: 'rw 8/6/2003 12:09'!
value
	"Do the same as my parent, but make sure that all actions that do not  
	give errors are evaluated before resignaling the ones that gave errors  
	(giving the chance to clients to handle them)."

	^self valueStartingFrom: 1! !

!WeakActionSequenceTrappingErrors methodsFor: 'evaluating' stamp: 'nk 9/6/2004 08:22'!
valueStartingFrom: startIndex
	"Do the same as my parent, but make sure that all actions that do not 
	give errors are evaluated before resignaling the ones that gave errors 
	(giving the chance to clients to handle them)."

	"Note: I currently trap Halt,Error so that I am sure to get a Halt event in case of a Halt. This is being fixed in the exception system - when the fix is done it will be enough to capture only Error."

	| each answer |
	startIndex to: self size do: [:index |
		each := self at: index.
		each isReceiverOrAnyArgumentGarbage ifFalse: [
			[answer := each value]
				on: Halt, Error
				do: [:exc | 
						self valueStartingFrom: index + 1.
						exc pass]]].
	^ answer! !

!WeakActionSequenceTrappingErrors methodsFor: 'evaluating' stamp: 'rw 8/6/2003 12:07'!
valueWithArguments: anArray 
	"Do the same as my parent, but make sure that all actions that do not 
	give errors are evaluated before resignaling the ones that gave errors 
	(giving the chance to clients to handle them)."

	^self valueWithArguments: anArray startingFrom: 1! !

!WeakActionSequenceTrappingErrors methodsFor: 'evaluating' stamp: 'nk 9/6/2004 08:22'!
valueWithArguments: anArray startingFrom: startIndex
	"Do the same as my parent, but make sure that all actions that do not 
	give errors are evaluated before resignaling the ones that gave errors 
	(giving the chance to clients to handle them)."

	"Note: I currently trap Halt,Error so that I am sure to get a Halt event in case of a Halt. This is being fixed in the exception system - when the fix is done it will be enough to capture only Error."

	| each answer |
	startIndex to: self size do: [:index |
		each := self at: index.
		each isReceiverOrAnyArgumentGarbage ifFalse: [
			[answer := each valueWithArguments: anArray]
				on: Halt, Error
				do: [:exc | 
						self valueWithArguments: anArray startingFrom: index + 1.
						exc pass]]].
	^ answer! !

!GZipWriteStream class methodsFor: '*System-FileRegistry' stamp: 'st 9/18/2004 23:44'!
fileReaderServicesForFile: fullName suffix: suffix
	"Don't offer to compress already-compressed files
	sjc 3-May 2003-added jpeg extension"

	^({ 'gz' . 'sar' . 'zip' . 'gif' . 'jpg' . 'jpeg'. 'pr'. 'png'} includes: suffix)
		ifTrue: [ #() ]
		ifFalse: [ self services ]
! !

!GZipWriteStream class methodsFor: '*System-FileRegistry' stamp: 'tbn 8/11/2010 10:33'!
serviceCompressFile

	^ FileModifyingSimpleServiceEntry 
				provider: self 
				label: 'Compress file'
				selector: #compressFile:
				description: 'Compress file using gzip compression, making a new file'! !

!GZipWriteStream class methodsFor: '*System-FileRegistry' stamp: 'nk 11/26/2002 13:10'!
services
	^ { self serviceCompressFile }! !

!Number class methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 3/18/2009 15:09'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForNumber! !

!ByteArray methodsFor: '*System-Hashing-Core' stamp: 'rww 4/11/2004 14:48'!
asByteArrayOfSize: size
	"
		'34523' asByteArray asByteArrayOfSize: 100.

	(((
		| repeats bytes | 
		repeats := 1000000.
		bytes := '123456789123456789123456789123456789123456789123456789' asByteArray.
		 [repeats timesRepeat: (bytes asByteArrayOfSize: 1024) ] timeToRun.
	)))"

	| bytes |
	size < self size
		ifTrue: [^ self error: 'bytearray bigger than ', size asString].
	bytes := self asByteArray.
	^ (ByteArray new: (size - bytes size)), bytes
! !

!ByteArray methodsFor: '*System-Hashing-Core' stamp: 'SvenVanCaekenberghe 4/18/2011 20:40'!
asInteger
	"Convert me to an Integer, network byte order, most significant byte first, big endian"
	
	| integer |
	integer := 0.
	self withIndexDo: [ :each :index | 
		integer := integer + (each bitShift: (self size - index) * 8) ].
	^ integer! !

!ByteArray methodsFor: '*System-Hashing-Core' stamp: 'StephaneDucasse 10/17/2009 17:15'!
bitXor: aByteArray 
	| answer |
	answer := self copy.
	1 
		to: (self size min: aByteArray size)
		do: 
			[ :each | 
			answer 
				at: each
				put: ((self at: each) bitXor: (aByteArray at: each)) ].
	^ answer! !

!ByteArray methodsFor: '*System-Hashing-Core' stamp: 'cmm 2/21/2006 00:05'!
destroy
	1 to: self size do: 
		[ : x |
		self at: x put: 0 ]! !

!Behavior methodsFor: '*System-Support' stamp: 'StephaneDucasse 4/30/2011 21:36'!
allCallsOn
	"Answer a SortedCollection of all the methods that refer to me by name or as part of an association in a global dict."
	
	^ (self allCallsOnIn: self systemNavigation)
! !

!Behavior methodsFor: '*System-Support' stamp: 'StephaneDucasse 4/30/2011 21:35'!
allCallsOn: aSymbol
	"Answer a SortedCollection of all the methods that call on aSymbol."

	^ self  systemNavigation allCallsOn: aSymbol from: self .
	! !

!Behavior methodsFor: '*System-Support' stamp: 'StephaneDucasse 4/30/2011 21:35'!
allCallsOnIn: aSystemNavigation
	"Answer a SortedCollection of all the methods that refer to me by name or as part of an association in a global dict."

	^ (aSystemNavigation 
		allReferencesTo: (self environment associationAt: self theNonMetaClass name)), 
		(aSystemNavigation allCallsOn: self theNonMetaClass name)	! !

!Behavior methodsFor: '*System-Support' stamp: 'MarcusDenker 7/12/2012 17:58'!
allUnsentMessages
	"Answer an array of all the messages defined by the receiver that are not sent anywhere in the system."

	^ SystemNavigation new allUnsentMessagesIn: self selectors! !

!Behavior methodsFor: '*System-Support' stamp: 'AlexandreBergel 9/12/2011 11:17'!
referencedClasses
	"Return the set of classes that are directly referenced by my methods"

	| answer |
	answer := Set new.
	self methods do: [ :cm |
		answer addAll: 
			( cm literals select: [ :l | l isKindOf: Association ] thenCollect: #value ) ].
		
	^ answer! !

!GZipReadStream class methodsFor: '*System-FileRegistry' stamp: 'lr 3/14/2010 21:13'!
fileReaderServicesForFile: fullName suffix: suffix
	| services |
	suffix = 'gz' | (suffix = '*')
		ifFalse: [ ^ #() ].
	services := OrderedCollection new.
	suffix = '*' | (fullName asLowercase endsWith: '.cs.gz') | (fullName asLowercase endsWith: '.mcs.gz')
		ifTrue: [ 
			services add: self serviceFileIn.
			(Smalltalk globals includesKey: #ChangeSorter)
				ifTrue: [ services add: self serviceFileIntoNewChangeSet ] ].
	services addAll: self services.
	^ services! !

!GZipReadStream class methodsFor: '*System-FileRegistry' stamp: 'tbn 8/11/2010 10:29'!
serviceDecompressToFile

	^ FileModifyingSimpleServiceEntry 
				provider: self 
				label: 'Decompress to file'
				selector: #saveContents:
				description: 'Decompress to file'! !

!GZipReadStream class methodsFor: '*System-FileRegistry' stamp: 'tbn 8/11/2010 10:19'!
serviceFileIn
	"Answer a service for filing in an entire file"

	^ SimpleServiceEntry 
		provider: self 
		label: 'FileIn entire file'
		selector: #fileIn:
		description: 'File in the entire decompressed contents of the file, which is expected to contain Smalltalk code in fileout ("chunk") format'
		buttonLabel: 'Filein'

! !

!GZipReadStream class methodsFor: '*System-FileRegistry' stamp: 'tbn 8/11/2010 10:19'!
serviceFileIntoNewChangeSet
	"Answer a service for filing in an entire file"
	^ SimpleServiceEntry
		provider: self
		label: 'Install into new change set'
		selector: #fileIntoNewChangeSet:
		description: 'Install the decompressed contents of the file as a body of code in the image: create a new change set and file-in the selected file into it'
		buttonLabel: 'Install'! !

!GZipReadStream class methodsFor: '*System-FileRegistry' stamp: 'tbn 8/11/2010 10:19'!
serviceViewDecompress

	^ SimpleServiceEntry 
				provider: self 
				label: 'View decompressed'
				selector: #viewContents:
				description: 'View decompressed' 
! !

!GZipReadStream class methodsFor: '*System-FileRegistry' stamp: 'sd 2/1/2002 22:16'!
services

	^ Array 
		with: self serviceViewDecompress
		with: self serviceDecompressToFile
	! !

!Association methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 3/24/2009 23:20'!
settingFixedDomainValueNodeFrom: aSettingNode 
	^ aSettingNode fixedDomainValueNodeForAssociation: self! !

!Integer methodsFor: '*System-Hashing-Core' stamp: 'PeterHugossonMiller 9/3/2009 10:00'!
asArray

	| stream |
	stream := Array new writeStream.
	self digitLength to: 1 by: -1 do: [:digitIndex |
		stream nextPut: (self digitAt: digitIndex)].
	^ stream contents
! !

!Integer methodsFor: '*System-Hashing-Core' stamp: 'PeterHugossonMiller 9/3/2009 10:01'!
asByteArray

	| stream |
	stream := ByteArray new writeStream.
	self digitLength to: 1 by: -1 do: [:digitIndex |
		stream nextPut: (self digitAt: digitIndex)].
	^ stream contents
! !

!Integer methodsFor: '*System-Hashing-Core' stamp: 'StephaneDucasse 10/17/2009 17:15'!
asByteArrayOfSize: aSize 
	"Answer a ByteArray of aSize with my value, most-significant byte first."
	| answer digitPos |
	aSize < self digitLength ifTrue: [ self error: 'number to large for byte array' ].
	answer := ByteArray new: aSize.
	digitPos := 1.
	aSize 
		to: aSize - self digitLength + 1
		by: -1
		do: 
			[ :pos | 
			answer 
				at: pos
				put: (self digitAt: digitPos).
			digitPos := digitPos + 1 ].
	^ answer! !

!FilePath class methodsFor: '*system-settings' stamp: 'AlainPlantec 9/3/2010 17:28'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode  inputWidgetForFileOrDirectoryWithAction: #chooseFilePath.! !

OrderedCollection subclass: #ObjectFinalizerCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Finalization'!

!ObjectFinalizerCollection commentStamp: 'ul 2/26/2010 14:23' prior: 0!
An ObjectFinalizerCollection is simple collection which is intended to hold ObjectFinalizers and be used by WeakRegistry.!

!ObjectFinalizerCollection methodsFor: 'finalization' stamp: 'ul 2/25/2010 16:53'!
finalize
	"Finalize all elements in this collection. The elements are expected to be ObjectFinalizers, but can be any object which understands #finalize."

	self do: #finalize! !

!ObjectFinalizerCollection methodsFor: 'testing' stamp: 'Igor.Stasenko 5/25/2010 04:58'!
hasMultipleExecutors
	^ true! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'PeterHugossonMiller 9/3/2009 10:20'!
backChunk
	"Answer the contents of the receiver back to the previous terminator character.  Doubled terminators indicate an embedded terminator character."
	| terminator out ch |
	terminator := $!!.
	out := (String new: 1000) writeStream.
	[(ch := self back) == nil] whileFalse: [
		(ch == terminator) ifTrue: [
			self oldPeekBack == terminator ifTrue: [
				self oldBack.  "skip doubled terminator"
			] ifFalse: [
				^ out contents reversed  "we're done!!"
			].
		].
		out nextPut: ch.
	].
	^ out contents reversed! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'PeterHugossonMiller 9/3/2009 10:43'!
basicNextChunk
	"Answer the contents of the receiver, up to the next terminator character. Doubled terminators indicate an embedded terminator character."
	| terminator out ch |
	terminator := $!!.
	out := (String new: 1000) writeStream.
	self skipSeparators.
	[(ch := self next) == nil] whileFalse: [
		(ch == terminator) ifTrue: [
			self peek == terminator ifTrue: [
				self next.  "skip doubled terminator"
			] ifFalse: [
				^ out contents  "terminator is not doubled; we're done!!"
			].
		].
		out nextPut: ch.
	].
	^ out contents! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'sd 5/23/2003 14:40'!
checkForPreamble: chunk
	((chunk beginsWith: '"Change Set:') and: [ChangeSet current preambleString == nil])
		ifTrue: [ChangeSet current preambleString: chunk].
	((chunk beginsWith: '"Postscript:') and: [ChangeSet current postscriptString == nil])
		ifTrue: [ChangeSet current postscriptString: chunk].
							
! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'StephaneDucasse 9/12/2010 09:45'!
copyMethodChunkFrom: aStream
	"Copy the next chunk from aStream (must be different from the receiver)."
	
	self nextChunkPut: aStream nextChunkText! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'StephaneDucasse 9/12/2010 09:45'!
copyMethodChunkFrom: aStream at: pos
	"Copy the next chunk from aStream (must be different from the receiver)."
	
	aStream position: pos.
	self nextChunkPut: aStream nextChunkText.! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'SergeStinckwich 7/31/2009 15:25'!
copyPreamble: preamble from: aStream at: pos 
	"Look for a changeStamp for this method by peeking backward.
	Write a method preamble, with that stamp if found."
	| terminator last50 stamp i |
	terminator := $!!.

	"Look back to find stamp in old preamble, such as...
	Polygon methodsFor: 'private' stamp: 'di 6/25/97 21:42' prior: 34957598!! "
	aStream position: pos.
	aStream backChunk.	"to beginning of method"
	last50 := aStream backChunk.	"to get preamble"
	aStream position: pos.
	stamp := String new.
	(i := last50 
		findLastOccurrenceOfString: 'stamp:'
		startingAt: 1) > 0 ifTrue: 
		[ stamp := (last50 
			copyFrom: i + 8
			to: last50 size) copyUpTo: $' ].

	"Write the new preamble, with old stamp if any."
	self
		cr;
		nextPut: terminator.
	self nextChunkPut: (String streamContents: 
			[ :strm | 
			strm nextPutAll: preamble.
			stamp size > 0 ifTrue: 
				[ strm
					nextPutAll: ' stamp: ';
					print: stamp ] ]).
	self cr! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'ar 4/12/2005 17:34'!
decodeString: string andRuns: runsRaw

	| strm runLength runValues newString index |
	strm := ReadStream on: runsRaw from: 1 to: runsRaw size.
	(strm peekFor: $( ) ifFalse: [^ nil].
	runLength := OrderedCollection new.
	[strm skipSeparators.
	 strm peekFor: $)] whileFalse: 
		[runLength add: (Number readFrom: strm)].

	runValues := OrderedCollection new.
	[strm atEnd not] whileTrue: 
		[runValues add: (Number readFrom: strm).
		strm next.].

	newString := WideString new: string size.
	index := 1.
	runLength with: runValues do: [:length :leadingChar |
		index to: index + length - 1 do: [:pos |
			newString at: pos put: (Character leadingChar: leadingChar code: (string at: pos) charCode).
		].
		index := index + length.
	].

	^ newString.
! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'SeanDeNigris 6/21/2012 08:39'!
fileInFor: client announcing: announcement
	"This is special for reading expressions from text that has been formatted 
	with exclamation delimitors. The expressions are read and passed to the 
	Compiler. Answer the result of compilation.  Put up a progress report with
     the given announcement as the title.
	Does NOT handle preambles or postscripts specially."
	| val |
	announcement displayProgressFrom: 0 to: self size
		during:
		[:bar |
		[self atEnd]
			whileFalse: 
				[bar current: self position.
				self skipSeparators.
				[ | chunk | val := (self peekFor: $!!) ifTrue: [
						(self class evaluatorClass evaluate: self nextChunk for: client logged: false) scanFrom: self
					] ifFalse: [
						chunk := self nextChunk.
						self checkForPreamble: chunk.
						self class evaluatorClass evaluate: chunk for: client logged: true ].
				] on: InMidstOfFileinNotification
				  do: [ :ex | ex resume: true].
				self atEnd ifFalse: [ self skipStyleChunk ]].
		self close].
	"Note:  The main purpose of this banner is to flush the changes file."
	Smalltalk logChange: '----End fileIn of ' , self name , '----'.
	^ val! !

!PositionableStream methodsFor: '*System-Changes' stamp: ''!
header
	"If the stream requires a standard header, override this message.  See HtmlFileStream"! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'PeterHugossonMiller 9/3/2009 10:43'!
nextChunk
	"Answer the contents of the receiver, up to the next terminator character. Doubled terminators indicate an embedded terminator character."
	| terminator out ch |
	terminator := $!!.
	out := (String new: 1000) writeStream.
	self skipSeparators.
	[(ch := self next) == nil] whileFalse: [
		(ch == terminator) ifTrue: [
			self peek == terminator ifTrue: [
				self next.  "skip doubled terminator"
			] ifFalse: [
				^ self parseLangTagFor: out contents  "terminator is not doubled; we're done!!"
			].
		].
		out nextPut: ch.
	].
	^ self parseLangTagFor: out contents.
! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'sumim 11/20/2003 18:13'!
nextChunkText
	"Deliver the next chunk as a Text.  Decode the following ]style[ chunk if present.  Position at start of next real chunk."
	| string runsRaw strm runs peek pos |
	"Read the plain text"
	string := self nextChunk.
	
	"Test for ]style[ tag"
	pos := self position.
	peek := self skipSeparatorsAndPeekNext.
	peek = $] ifFalse: [self position: pos. ^ string asText].  "no tag"
	(self upTo: $[) = ']style' ifFalse: [self position: pos. ^ string asText].  "different tag"

	"Read and decode the style chunk"
	runsRaw := self basicNextChunk.	"style encoding"
	strm := ReadStream on: runsRaw from: 1 to: runsRaw size.
	runs := RunArray scanFrom: strm.

	^ Text basicNew setString: string setRunsChecking: runs.
! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'sumim 11/20/2003 18:11'!
parseLangTagFor: aString

	| string peek runsRaw pos |
	string := aString.
	"Test for ]lang[ tag"
	pos := self position.
	peek := self skipSeparatorsAndPeekNext.
	peek = $] ifFalse: [self position: pos. ^ string].  "no tag"
	(self upTo: $[) = ']lang' ifTrue: [
		runsRaw := self basicNextChunk.
		string := self decodeString: aString andRuns: runsRaw
	] ifFalse: [
		self position: pos
	].
	^ string.
! !

!PositionableStream methodsFor: '*System-Changes' stamp: 'tk 12/29/97 12:37'!
skipStyleChunk
	"Get to the start of the next chunk that is not a style for the previous chunk"

	| pos |
	pos := self position.
	self skipSeparators.
	self peek == $] 
		ifTrue: [(self upTo: $[) = ']text' 	"old -- no longer needed"
				"now positioned past the open bracket"
			ifFalse: [self nextChunk]]	"absorb ]style[ and its whole chunk"
				
		ifFalse: [self position: pos]	"leave untouched"
! !

!PositionableStream methodsFor: '*System-Changes' stamp: ''!
trailer
	"If the stream requires a standard trailer, override this message.  See HtmlFileStream"! !

!PositionableStream methodsFor: '*System-Changes' stamp: ''!
verbatim: aString
	"Do not attempt to translate the characters.  Use to override nextPutAll:"
	^ self nextPutAll: aString! !

!CodeImporter methodsFor: '*System-CommandLine' stamp: 'CamilloBruni 6/14/2012 23:26'!
evaluate
	" stripped down version of evaluateDeclarations"
	| value |
	
	self codeDeclarations do: [ :decl | value := decl import ].
	self flushChangesFile.
	
	^value! !

BasicClassOrganizer subclass: #PseudoClassOrganizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-FilePackage'!

!PseudoClassOrganizer methodsFor: 'comment accessing' stamp: 'NS 4/6/2004 16:44'!
classComment
	"Answer the comment associated with the object that refers to the receiver."
	classComment == nil ifTrue: [^''].
	^classComment! !

!PseudoClassOrganizer methodsFor: 'comment accessing' stamp: 'NS 4/6/2004 16:44'!
classComment: aChangeRecord
	classComment := aChangeRecord! !

!PseudoClassOrganizer methodsFor: 'accessing' stamp: 'NS 4/6/2004 12:27'!
setDefaultList: aCollection
	super setDefaultList: aCollection.
	self classComment: nil.! !

MorphTreeNodeModel subclass: #SettingNode
	instanceVariableNames: 'parentName pragma contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Browser'!

!SettingNode commentStamp: 'alain.plantec 4/20/2009 21:42' prior: 0!
A SettingTreeNode is used by a SettingCollector as a node for setting trees. Built trees can be viewed by a SettingBrowser

Instance Variables
	contents:		<Collection of SettingTreeNode>
	parentNode:		<SettingTreeNode or nil>

contents
	- the children

parentNode
	- the parent or nil if the node is a root
!

!SettingNode methodsFor: 'comparing' stamp: 'alain.plantec 3/3/2009 22:46'!
<= other
	^ self asString <= other asString! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 07:33'!
allChildren
	^ self model childrenOf: self.
! !

!SettingNode methodsFor: 'menu' stamp: 'StephaneDucasse 10/15/2011 20:58'!
browseDeclaration
	"open a browser on the method which defines myself"
	Smalltalk tools browser
		fullOnClass: self methodClass class selector: self selector! !

!SettingNode methodsFor: 'checking' stamp: 'AlainPlantec 12/13/2009 21:59'!
checkForUnknownTarget
	self parentNode ifNotNil: [:p | item target ifNil: [item target: p item target].
		item targetSelector ifNil: [item targetSelector: p item targetSelector]].
	self allChildren do: [:c | c checkForUnknownTarget]! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 08:52'!
contents
	(self settingDeclaration hasValue
			and: [self settingValue = false])
		ifTrue: [^ OrderedCollection new]
		ifFalse: [^ contents 
				ifNil: [contents := self allChildren.
					self settingDeclaration ordering
						ifTrue: [contents := contents asArray sort: self model sortBlock].
					contents := self model retainedNodesFromList: contents asOrderedCollection]]! !

!SettingNode methodsFor: 'menu' stamp: 'alain.plantec 4/3/2009 07:59'!
deeplyDetect: aBlock 
	(aBlock value: self) ifTrue: [^ self].
	self contents do: [:sub | (sub deeplyDetect: aBlock) ifNotNil: [:found | ^ found]].
	^ nil! !

!SettingNode methodsFor: 'menu' stamp: 'alain.plantec 4/3/2009 07:57'!
deeplySelect: aBlock in: aCollection
	(aBlock value: self) ifTrue: [aCollection add: self].
	self contents do: [:sub | sub deeplySelect: aBlock in: aCollection].
	^ aCollection! !

!SettingNode methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 16:06'!
description
	^ self settingDeclaration description! !

!SettingNode methodsFor: 'menu' stamp: 'AlainPlantec 11/24/2009 15:24'!
exploreItem
	self settingDeclaration explore! !

!SettingNode methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 16:38'!
icon
	^ self settingDeclaration ifNotNil: [:d | d icon ] ! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/11/2009 09:46'!
inputMorphFor: aContainer
	^ self settingDeclaration 
		ifNotNil: [:v | v inputMorphFor: aContainer]! !

!SettingNode methodsFor: 'menu' stamp: 'AlainPlantec 11/24/2009 15:24'!
inspectItem
	self settingDeclaration inspect! !

!SettingNode methodsFor: 'testing' stamp: 'alain.plantec 3/17/2009 09:45'!
isEmpty
	^ self contents isEmpty! !

!SettingNode methodsFor: 'menu' stamp: 'AlainPlantec 2/25/2011 17:07'!
keyStroke: anEvent from: aTreeMorph
	| c |
	c := anEvent keyCharacter.
	c = $b
		ifTrue: [self menuBrowse. ^ true].
	c = $i
		ifTrue: [self inspectItem. ^ true].
	c = $I
		ifTrue: [self exploreItem. ^ true].
	^ false
! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 14:13'!
labelMorphFor: aContainer
	^ self settingDeclaration 
		ifNotNil: [:v | v labelMorphFor: aContainer]! !

!SettingNode methodsFor: 'menu' stamp: 'AlainPlantec 3/10/2010 07:32'!
menu: menu shifted: b 
	menu
		add: 'Browse (b)' translated
		target: self
		selector: #browseDeclaration.
! !

!SettingNode methodsFor: 'menu' stamp: 'alain.plantec 3/23/2009 10:11'!
menuBrowse
	self browseDeclaration! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/3/2009 14:44'!
methodClass
	^ self pragma ifNotNil: [self pragma methodClass theNonMetaClass]! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/9/2009 07:52'!
order
	^ item order! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/9/2009 07:52'!
order: anInteger
	item order: anInteger! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/3/2009 14:46'!
package
	
	^ self methodClass ifNotNil: [:mc | PackageOrganizer default mostSpecificPackageOfClass: mc ifNone: []]! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 09:26'!
parent: aSymbol
	self parentName: aSymbol! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 09:27'!
parentName
	^ parentName
	! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 09:27'!
parentName: aSymbol
	parentName := aSymbol.
	! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 07:51'!
parentNode
	^ self model nodeNamed: self parentName! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 14:51'!
pragma
	^ pragma! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 07:38'!
pragma: aPragma
	pragma := aPragma! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:50'!
receiverPackage
	^ self settingReceiver
		ifNotNil: [PackageOrganizer default
				mostSpecificPackageOfClass: self settingReceiver class
				ifNone: []] ! !

!SettingNode methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 5/16/2011 16:14'!
selector
	^ self pragma 
		ifNil: ['']
		ifNotNil: [:p | p selector]! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 07:11'!
settingDeclaration
	^ self item! !

!SettingNode methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:50'!
settingReceiver
	^ item settingReceiver! !

!SettingNode methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 23:40'!
settingValue
	^ self settingDeclaration realValue
! !

!SettingNode methodsFor: 'updating' stamp: 'AlainPlantec 10/30/2011 17:19'!
updateAccordingTo: aFilter 
	(aFilter keepSettingTreeNode: self) 
		ifFalse: [
			self contents do: [ :n | n updateAccordingTo: aFilter].
			self contents removeAllSuchThat: [ :n | (aFilter keepSettingTreeNode: n) not  and: [n isEmpty]]]
		! !

!String class methodsFor: '*system-settings-browser' stamp: 'alain.plantec 3/18/2009 15:02'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForString! !

!Boolean class methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 3/18/2009 14:48'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForBoolean! !

!FileDirectory class methodsFor: '*system-settings' stamp: 'AlainPlantec 9/3/2010 17:32'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForFileOrDirectoryWithAction: #chooseFileDirectory.! !

MorphTreeModel subclass: #SettingBrowser
	instanceVariableNames: 'viewedPackages roots status searchedText treeHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Browser'!

SettingBrowser class
	instanceVariableNames: 'searchedTextList regexpSearch'!

!SettingBrowser commentStamp: 'SeanDeNigris 11/30/2011 15:19' prior: 0!
<help>

!! The User interface
A SettingBrowser allows the browsing as well as the editing of setting values.
For each setting, a label and an input widget allowing the change of the setting value are shown.

!!!! Browsing standards settings
In order to open a setting browser one can also use the expression below:
--------------------
SettingBrowser open
--------------------
It opens a SettingBrowser for all settings declared with the standard 'systemsettings' pragma keyword. 
To see how these settings are declared in the image, you can browse all senders of #systemsettings:
--------------
SystemNavigation new  browseAllCallsOn: #systemsettings
--------------

!!!! Browsing application specific settings
For specific applications, other pragma keywords can be used. These keywords can be passed as follow:
--------------------
(SettingBrowser forKeywords: #('blob')) open.
--------------------
Then, only settings which are declared with the keyword 'blob' are viewed. 
Here is an example of such a setting declared in the class side of a class BlobSettings 
--------------------
BlobSettings class>>blobSettingOn: aBuilder
	<blob>
	(aBuilder group: #blobEditing) 
		label: 'Editing' translated;
		parent: #blobBrowsing; 
		description: 'All settings concerned with blob editing' translated;
		with: [	
			(aBuilder setting: #color)	...
--------------------
The expression 'SettingBrowser open' is then equivalent to the '(SettingBrowser forKeywords: #('systemsettings')) open' expression.

!!!! Filtering
The SettingBrowser has a filtering functionality to limitate the number of settings. You can enter a token in the input field of the toolbar in order to show only settings that are matching the token.
Two filtering modes are allowed:  with a substring or with a regular expression.
- with a substring: only settings which name, description, pragma class or method selector includes this substring are viewed; 
- with a regular expression: the same as with a substring except that the input is used as a regular expression. This filtering is applied if the Regexp checkbox is checked.

!! How to declare a setting
A setting is declared with a method class side. This kind of method takes a builder as argument and a standard setting is tagged with the <systemsettings> pragma. The builder argument serves as a facade for the declaration.

!!!!A simple boolean setting example

Let's start with a setting declaration example:
CodeHolderSystemSettings class>>caseSensitiveFindsSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #caseSensitiveFinds) 
		label: 'Case sensitive search' translated;
		description: 'If true, then the "find" command in text will always make its searches in a case-sensitive fashion' translated;
		setSelector: #caseSensitiveFinds:;
		getSelector: #caseSensitiveFinds;
		target: TextEditor;
		parent: #codeEditing.

For this setting to be declared, we make the asumption that we have TextEditor class>>caseSensitiveFinds and TextEditor class>>caseSensitiveFinds: methods in order to change the preference value. 
To declare a setting, just send #setting: to the builder with its identifier, a Symbol passed as argument. It creates a setting node. Then you can set the label, the description with #label: and #description sent to the newly created setting node. You also have to set the selectors for setting and getting the preference value as well as the target to which these accessors are sent  (often a class). This is done by sending respectively, #setSelector:, #getSelector: and #target: to the setting node.
Because all settings are organized in trees we need a way to indicate what is the position of the setting node in the overall setting trees list. In fact it can be done two ways. The first way is to use the #parent: message (A second possibility is to declare a subtree in one method, it is explained later in this documentation).The #parent: message is send for non root settings. #parent takes the identifier of the parent setting as argument.

You may notice that in this example,  if we don't take into account the $: at the end of the setting accessor, the getting and the setting accessors are the same. This is often the case. You can simply set the setter and the getter by sending the #selector: message to the setting node. Thus the declaration is simplified as follow:
CodeHolderSystemSettings class>>caseSensitiveFindsSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #caseSensitiveFinds) 
		label: 'Case sensitive search' translated;
		description: 'If true, then the "find" command in text will always make its searches in a case-sensitive fashion' translated;
		selector: #caseSensitiveFinds;
		target: TextEditor;
		parent: #codeEditing.

You may also notice that the identifier of the setting is then the same as the selector. In this case, you can omit to set the selector because by default, the identifier is used as the selector for getting the preference value and the identifier concatenated with a $: at the end is used as the setting selector. Thus the declaration is simplified again as follow:
CodeHolderSystemSettings class>>caseSensitiveFindsSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #caseSensitiveFinds) 
		label: 'Case sensitive search' translated;
		description: 'If true, then the "find" command in text will always make its searches in a case-sensitive fashion' translated;
		target: TextEditor;
		parent: #codeEditing.

Finally, if you decide to declare a setting directly in the class which implements the selector, the target information can be also omitted because the class in which the setting is declared is used by default. Thus the simplest declaration can be:
TextEditor class>>caseSensitiveFindsSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #caseSensitiveFinds) 
		label: 'Case sensitive search' translated;
		description: 'If true, then the "find" command in text will always make its searches in a case-sensitive fashion' translated;
		parent: #codeEditing.

!!!!Declaring a subtree in one single method
Directly declaring a sub-tree of settings in one method is also possible. Then, typically, a root group is declared for the application settings and the children settings themselves are also declared within the same method as in the example below in which #formatCommentWithStatements and #indentString are directly declared as children of the #configurableFormatter setting:

RBConfigurableFormatter class>>settingsOn: aBuilder
	<systemsettings>	
	(aBuilder group: #configurableFormatter)
		target: self;
		parent: #refactoring;
		label: 'Configurable Formatter' translated;
		description: 'Settings related to the formatter' translated;
		with: [
			(aBuilder setting: #formatCommentWithStatements)
				label: 'Format comment with statements' translated.
			(aBuilder setting: #indentString)
				label: 'Indent string' translated]

!!!!Optional setting
Any setting can have children. In the case where a boolean setting is used as a parent, then, its children are chown only if the parent preference value is true.

!!!!Range setting
You send the #range: message to the builder instead of the #setting: message. In addition, you send the #range: message to the setting with an interval as argument in order to indicate the valid range.
screenMarginSettingOn: aBuilder
	<systemsettings>
	(aBuilder range: #fullscreenMargin)
		target: SystemWindow;
		parent: #windows;
		label: 'Fullscreen margin' translated;
		description: 'Specify the amount of space that is let around a windows when it''s opened fullscreen' translated;
		range: (-5 to: 100).

!!!!List setting
For this kind of setting, the SettingBrowser will show a drop list. Here is an example for the window position strategy. Notice that the setting is declared by sending the #pickOne message to the builder. Notice also that the list of valid values are given by sending the #domainValues: message to the setting. The argument is an array of association. Each association key is the label that is shown and the corresponding association value gives the value that will be assigned to the preference.
windowPositionStrategySettingsOn: aBuilder
	<systemsettings>
	(aBuilder pickOne: #usedStrategy) 
		label: 'Window position strategy' translated;
		target: RealEstateAgent;
		domainValues: {
			'Reverse Stagger' translated -> #staggerFor:initialExtent:world:. 
			'Cascade' translated -> #cascadeFor:initialExtent:world:. 
			'Standard' translated -> #standardFor:initialExtent:world:};

!!!!Launcher
A launcher is a particular setting. It allows to launch a script directly from the setting browser. Imagine that you have changed some settings and that you need to evaluate a script in order to update some other objets. It can be used also to configurate globally a package of the entire image.
As an example, in order to use True Type Fonts, the system must be updated by collecting all the available TT fonts. This can be done by evaluating the following expression:
-------------
FreeTypeFontProvider current updateFromSystem
-------------
In order to be able to launch this script from the setting browser, you have to declare a launcher. For example, look-at how the script for the TT fonts is declared in GraphicFontSettings class >> #standardFontsSettingsOn:.

GraphicFontSettings class >> #standardFontsSettingsOn:
	<systemsettings>
	(aBuilder group: #standardFonts)
		...
		(aBuilder launcher: #updateFromSystem)
				order: 1; 
				target: FreeTypeFontProvider;
				targetSelector: #current;
				description: 'Update available fonts by scanning the current system';
				script: #updateFromSystem;
				label: 'Update fonts from system' translated.

Notice that you send #launcher: to the builder in order to create the setting node, then you send #script: to the created node with the selector of the script passed as argument.
</help>

Implementation details
See also SettingNode, SettingTree, SettingNodeBuilder and SettingTreeBuilder classes.

Instance Variables
	roots:		<Collection of SettingTreeNode>
	searchedText:		<String>
	status:		<WriteStream>
	collector:		<SettingCollector>

roots
	- the roots of the viewed setting trees

searchedText
	- the string used as a filter token or regexp in order to select viewed settings

status
	- a WriteStream used in order to store useful information which are shown when no current item is selected

collector
	- The SettingCollector which has the responsibility to collect settings and to build viewed trees
!

SettingBrowser class
	instanceVariableNames: 'searchedTextList regexpSearch'!

!SettingBrowser class methodsFor: 'private helpers' stamp: 'AlainPlantec 9/17/2011 21:29'!
allSettingsState
	"
	Workspace openContents: self allSettingsState label: 'Current Setting assignments' 
	"
	| declList |
	declList := (self currentNodeList select: [ :node | node item hasValue])
		collect: [ :node | node item ].
	^ String streamContents: [:s | declList do: [:d | d settingStoreOn: s] ]! !

!SettingBrowser class methodsFor: 'opening' stamp: 'AlainPantec 1/7/2011 08:19'!
browse: aCollectionOfKeywords
	"open a SettingBrowser view which is able to browse all settings 
	that are declared with a pragma keyword which is at least
	one of the string contained in aCollectionOfKeywords
	example:
	(SettingBrowser browse: #('systemsettings'))
	browse all settings which make use of the pragma keyword 'systemsettings'
	
	Note that the default setting pragma keyword is 'systemsettings'
	"
	(self forKeywords: aCollectionOfKeywords) open! !

!SettingBrowser class methodsFor: 'private helpers' stamp: 'AlainPlantec 9/14/2011 16:10'!
currentNodeList
	^ (SettingTree new  acceptableKeywords: self settingsKeywords) nodeList! !

!SettingBrowser class methodsFor: 'opening' stamp: 'AlainPantec 1/7/2011 08:19'!
forKeywords: aCollectionOfKeywords
	"create a new SettingBrowser initialized with a set of pragma keywords"
	^ self new acceptableKeywords: aCollectionOfKeywords! !

!SettingBrowser class methodsFor: 'accessing' stamp: 'AlainPlantec 9/17/2011 18:32'!
initialExtent
	"Initial extent of the SettingBrowser window"
	^ 900@600
		! !

!SettingBrowser class methodsFor: 'world menu' stamp: 'FernandoOlivero 4/12/2011 10:15'!
menuCommandOn: aBuilder
	<worldMenu>
	(aBuilder item: #'Settings')
		parent: #System;
		order: 1;
		icon:  self theme  smallConfigurationIcon;
		action: [SettingBrowser open]; 
		help: 'Opens a SystemSettingBrowser which allows you to alter many system settings'! !

!SettingBrowser class methodsFor: 'opening' stamp: 'alain.plantec 4/13/2009 14:33'!
open
	"Open a SettingBrowser view"
	^ self new open! !

!SettingBrowser class methodsFor: 'updating' stamp: 'AlainPlantec 11/30/2009 10:22'!
refreshAllSettingBrowsers
	self allSubInstances do: [:sb | sb updateList]! !

!SettingBrowser class methodsFor: 'accessing' stamp: 'AlainPantec 1/7/2011 08:15'!
regexpSearch
	"Is the regexp filtering used"
	^ regexpSearch ifNil: [regexpSearch := false]! !

!SettingBrowser class methodsFor: 'accessing' stamp: 'AlainPantec 1/7/2011 08:16'!
regexpSearch: aBoolean
	"Set to true if the regexp filtering is to be used"

	regexpSearch := aBoolean! !

!SettingBrowser class methodsFor: 'accessing' stamp: 'AlainPantec 1/7/2011 08:16'!
searchedTextList
	"Keep a list of previously entered filtering tokens"
	^ searchedTextList ifNil: [searchedTextList := OrderedCollection new]! !

!SettingBrowser class methodsFor: 'accessing' stamp: 'AlainPlantec 9/14/2011 16:05'!
settingsKeywords 
	^ ((PragmaCollector new filter: [:prg | prg keyword = #settingPragmaProcessor]) reset) collect: [:p | p method selector].
! !

!SettingBrowser class methodsFor: 'private helpers' stamp: 'AlainPlantec 9/17/2011 21:29'!
viewAllSettings
	"self viewAllSettings"
	Workspace openContents: self allSettingsState label: 'Current Setting assignments' 
! !

!SettingBrowser class methodsFor: 'help system' stamp: 'AlainPlantec 1/5/2011 13:03'!
wikiStyleHelp
	<wikiStyleHelp: #'The setting browser'>
	^ self comment! !

!SettingBrowser methodsFor: 'menu' stamp: 'AlainPlantec 1/29/2010 21:22'!
about
	self inform: 'Here you can browse, query and change all system settings' translated! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 11:44'!
acceptableKeywords: aCollectionOfRegExps 
	self treeHolder acceptableKeywords: aCollectionOfRegExps ! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/23/2009 15:03'!
addToSearchedTextList: aString
	(self searchedTextList includes: aString) 
		ifFalse: [self searchedTextList size > 5 ifTrue: [self searchedTextList removeFirst].
		self searchedTextList add: aString]! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/2/2009 14:12'!
addToStatus: aText 
	self status nextPutAll: aText.
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 11:05'!
allPackagesWithSettings
	^( self treeHolder nodeList collect: [:n | n receiverPackage]) asSet! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 1/30/2010 10:18'!
applyFilters
	roots := nil.
	self searchedText
		ifNotEmpty: [self updateAccordingTo: self textFilter].
	self viewedPackages
		ifNotEmpty: [ | filter |
			filter := SettingPackageFilter new
			packages: self viewedPackages.
			self updateAccordingTo: filter].
	self updateList
! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 10/7/2011 10:08'!
buildWindowIn: aWindow 
	| statusView toolBar treeMorph toolBarY gap menuBar packageListView statusViewHeight |
	treeMorph := self treeMorphIn: aWindow.
	statusView := self statusViewIn: aWindow.
	statusViewHeight := 100.
	toolBar := self toolBarIn: aWindow.
	gap := 3.
	packageListView := self packageListViewIn: aWindow.
	toolBarY := toolBar minExtent y + gap.
	aWindow 
		addMorph: toolBar
		fullFrame: (LayoutFrame 
				fractions: (0 @ 0 corner: 1 @ 0)
				offsets: (0 @ (gap ) corner: 0 @ (toolBarY + gap))).
	aWindow 
		addMorph: treeMorph
		fullFrame: (LayoutFrame 
				fractions: (0 @ 0 corner: 1 @ 0.99)
				offsets: (0 @ (toolBarY + gap + gap) corner: 0 @ statusViewHeight negated)).
	aWindow 
		addMorph: statusView
		fullFrame: (LayoutFrame 
				fractions: (0 @ 0.99 corner: 0.75 @ 1)
				offsets: (0 @ statusViewHeight negated corner: 0 @ 0)).
	aWindow 
		addMorph: packageListView
		fullFrame: (LayoutFrame 
				fractions: (0.75 @ 0.99 corner: 1 @ 1)
				offsets: (0 @ statusViewHeight negated corner: 0 @ 0)).
	^ aWindow! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 1/29/2010 21:08'!
changePackageSet: aSet 
	aSet ~= self viewedPackages
		ifTrue: [self viewedPackages: aSet.
			self applyFilters].
	^ true! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 1/1/2012 22:43'!
changeSearchedText: aString 
	self searchedText: aString.
	self applyFilters.
	^ true ! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 10/11/2011 14:59'!
choosePackagesIn: aWindow 
	| result list |
	list := MorphTreeModel new
				beCheckList;
				autoMultiSelection: true;
				rootItems: (self allPackagesWithSettings asArray sort: [:a :b | a packageName < b packageName]);
				 wrapBlockOrSelector: #packageName.
	result := list openDialogWindowIn: aWindow title: 'Choose packages' selectedtems: self viewedPackages.
	result ifNotNil: [self changePackageSet: result]! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 9/17/2011 21:33'!
closePathFrom: aSetting 
	(self settingNodeOfDeclaration: aSetting)
		ifNotNil: [:node | self changed: {#rootNodes. #closeItemPath}, (node path collect: [:n | n withoutListWrapper])]! !

!SettingBrowser methodsFor: 'menu' stamp: 'AlainPlantec 12/1/2009 15:51'!
doubleClick
	self selectedNode ifNotNil: [:node | node browseDeclaration]! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 13:51'!
getDescription
	^ (self selectedNode)
		ifNil: [self status contents ifEmpty: ['Hit return in text fields to accept the input' translated]]
		ifNotNil: [:sel | (Text string: sel item label attribute: TextEmphasis bold), String cr, sel item description]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/30/2010 10:10'!
getViewedPackageNames
	^ Text
		streamContents: [:s | self viewedPackages
				ifEmpty: [s nextPutAll: 'All packages' translated]
				ifNotEmpty: [:selections | 
					(selections asSet = self allPackagesWithSettings asSet)
						ifTrue: [s nextPutAll: 'All packages']
						ifFalse: [selections
								do: [:package | 
									s nextPutAll: package packageName.
									s nextPut: Character space]]]]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'IgorStasenko 4/26/2011 15:57'!
helpDeclaringAndManagingSettings
	"Open up a workspace with explanatory info in it about the Setting Browser"
	Smalltalk tools workspace new
		contents: "SettingManager comment;" 'To be documented here later';
		openLabel: 'Declaring and managing settings' translated! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 21:23'!
helpMenuIn: aWindow
	| helpMenu |
	helpMenu := aWindow newMenu.
	helpMenu
		addToggle: 'About' translated
		target: self
		selector: #about.
	helpMenu lastItem font: aWindow theme menuFont.
	helpMenu
		addToggle: 'Declaring and managing settings' translated
		target: self
		selector: #helpDeclaringAndManagingSettings.
	helpMenu lastItem font: aWindow theme menuFont.
	^ helpMenu
! !

!SettingBrowser methodsFor: 'user interface' stamp: 'alain.plantec 9/21/2009 09:40'!
initialExtent
	^ self class initialExtent! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 12/1/2009 09:10'!
isRendered
	| renderingMorphs |
	renderingMorphs := SystemWindow allSubInstances select: [:sw | sw model = self].
	^ renderingMorphs anySatisfy: [:m | m owner notNil]! !

!SettingBrowser methodsFor: 'menu' stamp: 'AlainPlantec 10/30/2011 17:03'!
keyStroke: anEvent from: aTreeMorph
	| c |
	c := anEvent keyCharacter.
	c = $a
		ifTrue: [^ self expandAll].
	c = $A
		ifTrue: [^ self collapseAll]. 
	self selectedNode
		ifNotNil: [:current | current keyStroke: anEvent from: aTreeMorph.
			current settingDeclaration hasDefault
				ifTrue: [c = $d
						ifTrue: [^ self menuSetToDefault: current]].
			current settingDeclaration hasEditableList
				ifTrue: [c = $e
						ifTrue: [^ self menuEmptyList: current]]]! !

!SettingBrowser methodsFor: 'menu' stamp: 'AlainPlantec 10/30/2011 17:06'!
menu: menu shifted: b 
	"Set up the menu to apply to the receiver's, 
	honoring the #shifted boolean"
	menu
		add: 'Expand all (a)' translated
		target: self
		selector: #expandAll. 
	menu
		add: 'Collapse all (A)' translated
		target: self 
		selector: #collapseAll.
	self selectedNode
		ifNotNil: [:current | 
			menu
				add: 'Expand all from here' translated
				target: self
				selector: #expandAllFromNode:
				argument: current.
			menu addLine.
			current menu: menu shifted: b].
	menu addLine.
	self selectedNode
		ifNotNil: [:current | 
			current settingDeclaration hasDefault
				ifTrue: [menu
						add: 'Set to default (d)' translated
						target: self
						selector: #menuSetToDefault:
						argument: current].
			current settingDeclaration hasEditableList
				ifTrue: [menu
						add: 'Empty list (e)' translated
						target: self
						selector: #menuEmptyList:
						argument: current]].
	^ menu! !

!SettingBrowser methodsFor: 'menu' stamp: 'alain.plantec 10/19/2009 14:49'!
menuEmptyList: aTreeNode
	| v |
	(v := aTreeNode settingDeclaration) hasEditableList 
		ifTrue: [v emptyList. self updateList]! !

!SettingBrowser methodsFor: 'menu' stamp: 'alain.plantec 10/19/2009 14:17'!
menuSetToDefault: aTreeNode
	| v |
	(v := aTreeNode settingDeclaration) hasDefault
		ifTrue: [(self confirm: 'Set to default value ?' translated)
				ifTrue: [v setToDefault. self updateList]]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 4/29/2010 17:51'!
newTreeHolder
	^ SettingTree new  acceptableKeywords: self settingsKeywords! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 10/10/2011 11:31'!
open
	
	Cursor wait
		showWhile: [ | window |window := StandardWindow new model: self.
			window title: self windowTitle.
			self buildWindowIn: window.
			window themeChanged.
			window openInWorld.
			(window findDeeplyA: self treeMorphClass)  takeKeyboardFocus. 
			^ window]! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 9/17/2011 21:33'!
openPathFrom: aSetting
	(self settingNodeOfDeclaration: aSetting) ifNotNil: [:node |
		self changed: {#rootNodes. #openItemPath}, (node path collect: [:n | n withoutListWrapper])]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 21:10'!
packageListViewIn: aWindow
	^ aWindow
				newTextEditorFor: self
				getText: #getViewedPackageNames
				setText: #setViewedPackageNames:.
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 3/27/2009 23:04'!
pragmaErrorOccured: aPragmaError
	self setDescription: aPragmaError message, String cr! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/23/2009 15:01'!
regexpSearch
	^ self class regexpSearch! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 1/1/2012 22:31'!
regexpSearch: aBoolean
	self class regexpSearch: aBoolean.
! !

!SettingBrowser methodsFor: 'initialize-release' stamp: 'AlainPlantec 1/29/2010 21:10'!
release
	treeHolder release.
	treeHolder := nil.
	roots := nil.
	status := nil.
	searchedText := nil.
	viewedPackages := nil.
	super release.! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 2/4/2010 17:15'!
rootNodes
	^ roots ifNil: [roots := self treeHolder settingTreeRoots]
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/8/2009 08:58'!
searchedText
	^ searchedText ifNil: [searchedText := '']! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 3/23/2009 18:38'!
searchedText: aString
	searchedText := aString.
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/20/2009 09:19'!
searchedTextList
	^ self class searchedTextList! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 11/18/2009 08:28'!
selection: aSelectionHolder
	super selection: aSelectionHolder.
	self setDescription: ''.
	self changed: #getDescription
! !

!SettingBrowser methodsFor: 'menu' stamp: 'AlainPlantec 11/24/2009 11:44'!
setAllToDefault
	(self confirm: 'Set all to default value ?' translated) ifTrue: [
		self treeHolder nodeList do: [:node | node item hasDefault ifTrue: [node item setToDefault]].
		self updateList]! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/2/2009 14:12'!
setDescription: aText 
	self status reset.
	self addToStatus: aText.
	self changed: #getDescription! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 21:10'!
setViewedPackageNames: aText 
	| allViewed |
	allViewed := Set new.
	aText asString subStrings
		do: [:sub | (PackageOrganizer default
						packageNamed: sub
						ifAbsent: [])
				ifNotNil: [:pkg | allViewed add: pkg]].
	self changePackageSet: allViewed.
	self changed: #getViewedPackageNames
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 11:44'!
settingNodeOfDeclaration: aSettingDeclaration
	^ self treeHolder deeplyDetect: [:n | n settingDeclaration = aSettingDeclaration]! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 11/27/2009 22:08'!
settingValueChanged: aSetting 
	aSetting realValue = true
		ifTrue: [self openPathFrom: aSetting].
	aSetting realValue = false
		ifTrue: [self closePathFrom: aSetting].
	aSetting changed: #realValue.
	aSetting changed: #fontButtonLabel.
	aSetting changed: #index.
	aSetting changed: #content ! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 9/14/2011 16:06'!
settingsKeywords 
	^ self class settingsKeywords! !

!SettingBrowser methodsFor: 'accessing' stamp: 'alain.plantec 4/2/2009 14:11'!
status
	^ status ifNil: [status := String new writeStream]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'alain.plantec 3/23/2009 18:38'!
statusViewIn: aMorph
	| statusView |
	statusView := aMorph
				newTextEditorFor: self
				getText: #getDescription
				setText: #setDescription:.
	^ statusView
 ! !

!SettingBrowser methodsFor: 'updating' stamp: 'alain.plantec 4/23/2009 15:01'!
textFilter
	^ self regexpSearch
		ifFalse: [SettingTokenFilter new token: self searchedText]
		ifTrue: [SettingRegexpFilter new regexp: self searchedText] ! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/1/2012 22:52'!
toolBarIn: aMorph 
	| toolBar |
	toolBar := aMorph newToolbar: {
	aMorph
		newButtonFor: self
		getState: nil
		action: #expandAll
		arguments: nil
		getEnabled: nil
		label: 'Expand all' translated
		help: 'Expand all tree nodes' translated.
	aMorph newToolSpacer.
	aMorph newToolSpacer.
	aMorph newLabel: 'Search for: ' translated.
	(SearchMorph
				on: self
				list: #searchedTextList
				selected: #searchedText
				changeSelected: #changeSearchedText:
				useIndex: false 
				addToList: #addToSearchedTextList:
				class: String
				getEnabled: nil) hResizing: #spaceFill; 
				ghostText: 'Hit return to accept' translated.
	aMorph
			newCheckboxFor: self
			getSelected: #regexpSearch
			setSelected: #regexpSearch:
			getEnabled: nil
			label: 'Regexp' 
			help: 'Filter according to a regular expression' translated.
	aMorph newToolSpacer.	
	aMorph newToolSpacer.
	(aMorph
				newButtonFor: self
				getState: nil
				action: #choosePackagesIn:
				arguments: {aMorph}
				getEnabled: nil
				label: 'Choose packages' translated 
				help: 'Select packages' translated).
	aMorph newToolSpacer}.
	^ toolBar! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 11/18/2009 08:31'!
treeChanged: anAnnounce
	roots := nil.
	self selection: nil.
	self updateList! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 11:42'!
treeHolder
	^ treeHolder
		ifNil: [treeHolder := self newTreeHolder whenChangedSend: #treeChanged: to: self; yourself]! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 10/10/2011 11:31'!
treeMorphIn: aMorph 
	| treeMorph |
	treeMorph := (self treeMorphClass on: self)
				getMenuSelector: #menu:shifted:;
				keystrokeActionSelector: #keyStroke:from:;
				columns: {
					MorphTreeColumn new startWidth: 300; rowMorphGetSelector: #labelMorphFor:; yourself.
					MorphTreeColumn new rowMorphGetSelector: #inputMorphFor:;yourself
				};
				doubleClickSelector: #doubleClick;
				rowInset: 6;
				columnInset: 6;
				rowColorForEven: Color veryLightGray muchLighter;
				preferedPaneColor: Color veryLightGray lighter lighter;
				yourself.
	^ treeMorph buildContents! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 9/17/2011 21:34'!
updateAccordingTo: aFilter
	self selection: nil.
	self setDescription: ''.
	roots := nil.
	self updateList.
	[
	self rootNodes do: [:r | r updateAccordingTo: aFilter].
	self rootNodes removeAllSuchThat: [:n | (aFilter keepSettingTreeNode: n) not and: [n isEmpty]].
	]
		on: RegexSyntaxError
		do: [:ex | 
			self setDescription: 'Regular expression syntax error, see `documentation'' protocol of RxParser class for user''s guide'].
	self expandAll! !

!SettingBrowser methodsFor: 'updating' stamp: 'AlainPlantec 9/17/2011 21:34'!
updateList
	self isRendered ifTrue: [
		self treeHolder updateList.
		self changed: #rootNodes.
		self changed: #getDescription.
		self changed: #getViewedPackageNames].	
! !

!SettingBrowser methodsFor: 'updating' stamp: 'alain.plantec 4/2/2009 14:12'!
updateStatusWith: aMessage
	self addToStatus: aMessage.
	self addToStatus: Character cr.
! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 9/17/2011 21:25'!
viewedLabelOfSetting: aSettingDeclaration 
	| node |
	node := self settingNodeOfDeclaration: aSettingDeclaration.
	^ node 
		ifNil: [aSettingDeclaration label]
		ifNotNil: [(node allChildren notEmpty and: [aSettingDeclaration hasValue])
				ifTrue: [aSettingDeclaration label, '...'] 
				ifFalse: [aSettingDeclaration label]]! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 21:11'!
viewedPackages
	^ viewedPackages
		ifNil: [viewedPackages := Set new]! !

!SettingBrowser methodsFor: 'accessing' stamp: 'AlainPlantec 1/30/2010 10:18'!
viewedPackages: aCollectionOfPackagesOrNil
	viewedPackages := aCollectionOfPackagesOrNil! !

!SettingBrowser methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 21:29'!
windowTitle
	^ 'Settings browser'! !

!StrikeFont methodsFor: '*system-settings-browser' stamp: 'AlainPlantec 1/30/2010 14:48'!
settingStoreOn: aStream 
	aStream nextPut: $(.
	aStream nextPutAll: self class name.
	aStream nextPut: Character space.
	aStream nextPutAll: 'familyName: ''', self familyName, ''' pointSize: ', self pointSize asString, ' emphasized: ', self emphasis asString.
	aStream nextPut: $)
! !

!AbstractFont class methodsFor: '*system-settings-browser' stamp: 'alain.plantec 3/18/2009 14:49'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForFont! !

Announcement subclass: #PragmaCollectorReset
	instanceVariableNames: 'collector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-PragmaCollector'!

!PragmaCollectorReset commentStamp: 'alain.plantec 10/20/2009 11:14' prior: 0!
A PragmaCollectorReset is an announce which is announced by a PragamColllector when it is reset.
see PragmaCollector>>reset.

Instance Variables
	collector:		<PragmaCollector>

collector
	- the collector which is reset
!

!PragmaCollectorReset class methodsFor: 'as yet unclassified' stamp: 'alain.plantec 10/19/2009 11:45'!
collector: aCollector
	^ self new collector: aCollector! !

!PragmaCollectorReset methodsFor: 'as yet unclassified' stamp: 'alain.plantec 10/19/2009 11:45'!
collector: aCollector
	collector := aCollector! !

Announcement subclass: #SystemAnnouncement
	instanceVariableNames: 'event timeStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!SystemAnnouncement commentStamp: '<historical>' prior: 0!
The root announcement for the events coming from the Pharo system.

Currently, it wraps the event that comes from the SystemChangeNotifier.!

SystemAnnouncement subclass: #CategoryAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!CategoryAdded commentStamp: 'cyrilledelaunay 1/18/2011 12:29' prior: 0!
This announcement will be emited when adding a category using:
=> SystemOrganizer >> addCategory:!

!CategoryAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:12'!
categoryName

	^ self item! !

!CategoryAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 7/28/2010 10:34'!
item
	^ event item! !

SystemAnnouncement subclass: #CategoryRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!CategoryRemoved commentStamp: 'cyrilledelaunay 1/18/2011 12:28' prior: 0!
This announcement will be emited when removing a category using:
=> SystemOrganizer >> removeCategory:!

!CategoryRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:12'!
categoryName

	^ self  item! !

!CategoryRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:12'!
categorySelector

	^ self item asSymbol! !

SystemAnnouncement subclass: #CategoryRenamed
	instanceVariableNames: 'from to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!CategoryRenamed commentStamp: 'cyrilledelaunay 1/18/2011 12:26' prior: 0!
This announcement will be emited when renaming a category using:
=> SystemOrganizer >> renameCategory:toBe:!

!CategoryRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 7/27/2010 21:59'!
newName

	^ event newName! !

!CategoryRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 7/27/2010 21:59'!
oldName

	^ event oldName! !

!CategoryRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:14'!
oldNameSymbol

	^ event oldName asSymbol! !

SystemAnnouncement subclass: #ClassAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassAdded commentStamp: 'cyrilledelaunay 1/18/2011 11:56' prior: 0!
This announcement will be emitted when a class or a trait is added, using:
	=> Trait >> named: (the notification is done in Trait >> named:uses:category:env:)
	=> Class >> subclass:
	!

!ClassAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:15'!
itemClass

	^ self item ! !

!ClassAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:15'!
itemClassSymbol

	^ self itemClass theNonMetaClass name asSymbol! !

SystemAnnouncement subclass: #ClassCommented
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

SystemAnnouncement subclass: #ClassModifiedClassDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassModifiedClassDefinition methodsFor: 'testing' stamp: 'GuillermoPolito 6/22/2012 15:38'!
anyChanges
	self flag: 'We should get rid of this please. normally if you send a changed event, is because something changed, so this is nonsense'.
	^ self event isSuperclassModified
		| self event areInstVarsModified
			| self event areClassVarsModified
				| self event areSharedPoolsModified
					| self event isTraitCompositionModified.! !

!ClassModifiedClassDefinition methodsFor: 'testing' stamp: 'GuillermoPolito 7/20/2012 17:18'!
oldItem
	^self event oldItem! !

SystemAnnouncement subclass: #ClassRecategorized
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassRecategorized commentStamp: 'cyrilledelaunay 1/18/2011 10:42' prior: 0!
This announcement will be raised when:
- we set a category to a class (see 'Class >>category:')
- we set a category to a Trait (see Trait>>category:)

It corresponds to the RecategorizedEvent!

!ClassRecategorized methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 7/27/2010 22:38'!
itemClass 

	^ event itemClass ! !

!ClassRecategorized methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 7/28/2010 09:09'!
newCategory 

	^ event itemCategory ! !

!ClassRecategorized methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 4/24/2011 18:24'!
oldCategory 

	^ event oldCategory asSymbol! !

SystemAnnouncement subclass: #ClassRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassRemoved commentStamp: 'cyrilledelaunay 1/18/2011 11:43' prior: 0!
the annoucement will be emitted when removing a class or a trait using:  
	=> removeFromSystem!

!ClassRemoved methodsFor: 'accessing' stamp: 'GuillermoPolito 7/22/2012 16:09'!
itemClass
	"to make polymorphic the events"
	^self removedClass! !

!ClassRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 5/26/2012 16:15'!
removedClass

	^ self item theNonMetaClass! !

!ClassRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 5/26/2012 16:10'!
removedClassCategory

	"the previous definition was self item class theNonMetaClass category
	but his makes the assumption that the class is a living and well connected object"
	
	^ self event itemCategory! !

SystemAnnouncement subclass: #ClassRenamed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassRenamed commentStamp: 'cyrilledelaunay 1/18/2011 11:44' prior: 0!
the annoucement will be emitted when renaming a class or a trait using:  
	=> RenameClassRefactoring >> rename:to:
	=> class>>rename:
The corresponding event is raised in: SystemDictionary>>renameClass:from:to:!

!ClassRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:16'!
itemClass

	^ self item! !

!ClassRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:16'!
itemPackageName
	"It cannot be self itemClass package name since itemClass refer to the new class which is not in
	the current RPackageOrganizer - because we are currently in the process of rename it.
	
	It cannot be  (Smalltalk globals at: self oldName) package name because oldName class is not anymore in Smalltalk globals."
	^ self event itemCategory
	! !

!ClassRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:16'!
newName
	
	^ self event newName! !

!ClassRenamed methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:16'!
oldName
	
	^ self event oldName! !

SystemAnnouncement subclass: #ClassReorganized
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ClassReorganized commentStamp: 'cyrilledelaunay 1/18/2011 15:03' prior: 0!
This announcement corresponds to the ReorganizedEvent, which seems to (by looking at the references of ReorganizedEvent) be raised when:
- we rename a protocol (see 'renameCategory:toBe: '). if The category is not empty, SystemMethodRecategorizedAnnouncement will also be emitted
- we sort (modify the order of) protocols (see 'sortCategories')
- we add a protocol (see 'addCategory:before:')
- we remove a protocol (see 'removeCategory:' 'removeEmptyCategories')!

!ClassReorganized methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:16'!
itemClass

	^ self item! !

SystemAnnouncement subclass: #ExpressionEvaluated
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ExpressionEvaluated methodsFor: 'as yet unclassified' stamp: 'CamilleTeruel 7/30/2012 00:49'!
expression
	^ event item.! !

SystemAnnouncement subclass: #MethodAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!MethodAdded commentStamp: 'cyrilledelaunay 1/18/2011 13:06' prior: 0!
This announcement is emited when we add a method to a class or a trait using:
	=> Behavior >> compile: or TraitBehavior >> compile:!

!MethodAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 7/16/2010 13:08'!
itemClass

	^ event itemClass
	! !

!MethodAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:17'!
itemMethod

	^ self item! !

!MethodAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 7/16/2010 13:08'!
itemProtocol

	^ event itemProtocol! !

!MethodAdded methodsFor: 'accessing' stamp: 'StephaneDucasse 7/16/2010 13:08'!
itemSelector

	^ event itemSelector
	! !

SystemAnnouncement subclass: #MethodModified
	instanceVariableNames: 'oldMethod newMethod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!MethodModified commentStamp: 'BenjaminVanRyseghem 4/1/2011 16:04' prior: 0!
This announcement is emited when we RE-compile a method in a class or a trait, with: ClassDescription >> compile: or TraitDescription >> compile:. If the method is not yet registered in the class or the trait, the announcement will not be emitted.

The action of renaming a method will be handled by SystemMethodRemovedAnnouncement and SystemMethodAddedAnnouncement, since this refactoring is concretely composed by removing the old method and add a new with the new name  !

!MethodModified class methodsFor: 'instance creation' stamp: 'CamilleTeruel 7/29/2012 19:01'!
oldMethod: oldMethod newMethod: newMethod
	(oldMethod class == newMethod class) 
		ifFalse: [ self error: 'The two methods don''t belong to the same class' ].
	(oldMethod selector == newMethod selector)
		ifFalse: [ self error: 'The two methods haven''t the same selector' ].
	^ self new
		oldMethod: oldMethod;
		newMethod: newMethod;
		yourself.! !

!MethodModified methodsFor: 'temp to remove' stamp: 'CamilleTeruel 7/29/2012 19:21'!
event: aFuckingEventIWantToKill
	self oldMethod: aFuckingEventIWantToKill item.
	self newMethod: aFuckingEventIWantToKill oldItem.! !

!MethodModified methodsFor: 'temp to remove' stamp: 'CamilleTeruel 7/29/2012 19:23'!
item
	^ oldMethod ! !

!MethodModified methodsFor: 'temp to remove' stamp: 'CamilleTeruel 7/29/2012 19:23'!
itemClass

	^ newMethod methodClass.
	! !

!MethodModified methodsFor: 'temp to remove' stamp: 'CamilleTeruel 7/29/2012 19:24'!
itemProtocol

	^ newMethod protocol! !

!MethodModified methodsFor: 'temp to remove' stamp: 'CamilleTeruel 7/29/2012 19:14'!
itemSelector

	^ self selector.
	! !

!MethodModified methodsFor: 'temp to remove' stamp: 'CamilleTeruel 7/29/2012 19:13'!
newMethod

	^ newMethod ifNil: [event oldItem]! !

!MethodModified methodsFor: 'accessing' stamp: 'CamilleTeruel 7/29/2012 18:53'!
newMethod: aCompiledMethod
	newMethod := aCompiledMethod.
	! !

!MethodModified methodsFor: 'temp to remove' stamp: 'CamilleTeruel 7/29/2012 19:29'!
oldItem
	"that is stupid..."
	^ newMethod  ! !

!MethodModified methodsFor: 'temp to remove' stamp: 'CamilleTeruel 7/29/2012 19:14'!
oldMethod
	
	^ oldMethod ifNil: [self  item]! !

!MethodModified methodsFor: 'accessing' stamp: 'CamilleTeruel 7/29/2012 18:53'!
oldMethod: aCompiledMethod
	oldMethod := aCompiledMethod.
	! !

!MethodModified methodsFor: 'accessing' stamp: 'CamilleTeruel 7/29/2012 19:18'!
selector
	^ newMethod ifNil: [ event itemSelector ] ifNotNil: [ newMethod selector].! !

SystemAnnouncement subclass: #MethodRecategorized
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!MethodRecategorized commentStamp: 'cyrilledelaunay 1/18/2011 15:03' prior: 0!
This announcement will be emitted when:
	=> a non-empty protocol is renamed in a class or a trait (using ClassOrganizer >> renameCategory:toBe:). If the protocol does not contain any method, the announcement will not be emitted. Therefore, if we rename a non-empty protocol, both SystemMethodRecategorizedAnnouncement and SystemClassReorganizedAnnouncement will be emitted
	=> a method is removed from a category of a class or a trait (using ClassOrganizer >> removeElement:)
	=> a method is RE-categorized in a protocol of a trait or a class, using Organizer>>classify:under:.  If the method is classify under its current protocol, the announcement will not be emitted!

!MethodRecategorized methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 15:40'!
itemClass

	^ event itemClass
	! !

!MethodRecategorized methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 16:14'!
itemNewProtocol

	^ event itemProtocol! !

!MethodRecategorized methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 16:11'!
itemOldProtocol

	^ event oldCategory! !

!MethodRecategorized methodsFor: 'accessing' stamp: 'cyrilledelaunay 1/17/2011 15:41'!
itemSelector

	^ event itemSelector
	! !

SystemAnnouncement subclass: #MethodRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!MethodRemoved commentStamp: 'cyrilledelaunay 1/18/2011 14:43' prior: 0!
This announcement will be emitted when a method is removed from a trait or a class, 
using ClassDescription >>removeSelector: or TraitDescription >> removeSelector.
!

!MethodRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 4/13/2011 21:23'!
itemClass

	^ self event itemClass
	! !

!MethodRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 10/12/2011 15:18'!
itemMethod

	^ event item
	! !

!MethodRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 4/13/2011 21:23'!
itemProtocol

	^ self event itemProtocol! !

!MethodRemoved methodsFor: 'accessing' stamp: 'StephaneDucasse 4/13/2011 21:23'!
itemSelector

	^ self event itemSelector
	! !

!MethodRemoved methodsFor: 'accessing' stamp: 'CamilleTeruel 7/30/2012 00:50'!
methodClass
	^ self itemClass.! !

!MethodRemoved methodsFor: 'accessing' stamp: 'CamilleTeruel 7/30/2012 00:51'!
selector
	^ self itemSelector! !

SystemAnnouncement subclass: #ProtocolAdded
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ProtocolAdded commentStamp: '<historical>' prior: 0!
This class is not used, but should be used when we hook into the addition and removal of protocols. Right now, we only get ClassReorganizedAnnouncement!

SystemAnnouncement subclass: #ProtocolRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!ProtocolRemoved commentStamp: '<historical>' prior: 0!
This class is not used, but should be used when we hook into the addition and removal of protocols. Right now, we only get ClassReorganizedAnnouncement!

!SystemAnnouncement class methodsFor: 'instance creation' stamp: 'StephaneDucasse 4/13/2011 17:03'!
fromInternal: anInternalAnnouncement
	"create a new announcement from an internal announcement. Copy the variables"
	
	| instance |
	self flag: #ben. "why copy does not work?"
	instance := self new.
	1 to: self allInstVarNames size do:[:i | instance instVarAt: i put: ( anInternalAnnouncement instVarAt: i )].
	^ instance! !

!SystemAnnouncement class methodsFor: 'instance creation' stamp: 'StephaneDucasse 8/9/2011 23:05'!
new 
	
	^ super new setTimeStamp; yourself! !

!SystemAnnouncement methodsFor: 'private' stamp: 'tg 6/5/2010 12:17'!
event
	^ event! !

!SystemAnnouncement methodsFor: 'private' stamp: 'tg 6/5/2010 12:17'!
event: anObject
	event := anObject! !

!SystemAnnouncement methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 4/1/2011 17:18'!
item

	^ self event item! !

!SystemAnnouncement methodsFor: 'private' stamp: 'StephaneDucasse 8/9/2011 23:05'!
setTimeStamp

	timeStamp := TimeStamp current.! !

!SystemAnnouncement methodsFor: 'accessing' stamp: 'StephaneDucasse 8/9/2011 23:04'!
timeStamp

	^ timeStamp! !

!SystemAnnouncement methodsFor: 'accessing' stamp: 'StephaneDucasse 8/9/2011 23:04'!
timeStamp: aTimeStamp

	timeStamp := aTimeStamp! !

SystemAnnouncement subclass: #UnknownSystemChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

Announcer subclass: #SystemAnnouncer
	instanceVariableNames: 'suspended'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

SystemAnnouncer class
	instanceVariableNames: 'announcer privateAnnouncer'!

!SystemAnnouncer commentStamp: '<historical>' prior: 0!
This class plays the role of the announcer for events raised by the Pharo system.

Currently, it acts as a wrapper over the SystemChangeNotifier, but in the future it should be used for announcing the events directly from the system.. 

BEWARE: You should not need to subclass me. Think about just using me to send your announces (if you need system behavior) or using your own announcers as instance or class  variables.!

SystemAnnouncer class
	instanceVariableNames: 'announcer privateAnnouncer'!

!SystemAnnouncer class methodsFor: 'private' stamp: 'EstebanLorenzano 7/27/2012 19:39'!
announceFromEvent: aChangeEvent
	"This method creates an announcement from a given Event (subclass of AbstractEvent) and announces it.
	
	For example, from an AddedEvent that concerns a class the method creates SystemAddedClassAnnouncement and then it will announce it both from the generic announcer and from the classAnnouncer "
	| announcement |
	
	announcement := aChangeEvent asSystemAnnouncement.
	self uniqueInstance announce: announcement.! !

!SystemAnnouncer class methodsFor: 'accessing' stamp: 'EstebanLorenzano 7/27/2012 12:25'!
announcer
	"Answer a system annoncer."
	^ self uniqueInstance! !

!SystemAnnouncer class methodsFor: 'accessing' stamp: 'GuillermoPolito 4/27/2012 13:25'!
announcer: anAnnouncer
	announcer := anAnnouncer! !

!SystemAnnouncer class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 7/27/2012 14:55'!
current
	"Answer a system annoncer."
	self flag: #todo. "Remove Me"
	^ self uniqueInstance! !

!SystemAnnouncer class methodsFor: 'utilities' stamp: 'EstebanLorenzano 7/27/2012 11:16'!
defaultPreferredAnnouncementClassForEvent: aChangeEvent

	| itemKindName eventKindName announcementClassName announcementClass |
	self flag: #todo. "Remove?"
	itemKindName := aChangeEvent itemKind.
	eventKindName := (aChangeEvent class name endsWith: 'Event') 
		ifTrue: [ aChangeEvent class name copyFrom: 1 to: aChangeEvent class name size - 'Event' size ] 
		ifFalse: [ aChangeEvent class name ].
	announcementClassName := 	(
										'System', 
										itemKindName capitalized, 
										eventKindName,
										'Announcement'
									) asSymbol.
	announcementClass := Smalltalk at: announcementClassName ifAbsent: [nil].
	^ announcementClass! !

!SystemAnnouncer class methodsFor: 'initialization' stamp: 'EstebanLorenzano 7/27/2012 18:53'!
initialize
	"self initialize"
	
	"SystemAnnouncer uniqueInstance 
		notify: self ofAllSystemChangesUsing: #announceFromEvent:"! !

!SystemAnnouncer class methodsFor: 'accessing' stamp: 'EstebanLorenzano 7/27/2012 19:43'!
privateAnnouncer
	"Answer a system annoncer. Normally you should not invoke it"

	^privateAnnouncer ifNil: [ privateAnnouncer := self new ]! !

!SystemAnnouncer class methodsFor: 'accessing' stamp: 'StephaneDucasse 6/2/2012 22:07'!
privateAnnouncer: anAnnouncer
	"Answer a system annoncer."

	privateAnnouncer := anAnnouncer! !

!SystemAnnouncer class methodsFor: 'utilities' stamp: 'EstebanLorenzano 7/27/2012 19:44'!
reset
	announcer := nil.
	privateAnnouncer := nil.	! !

!SystemAnnouncer class methodsFor: 'utilities' stamp: 'EstebanLorenzano 7/27/2012 19:44'!
restoreAllNotifications
	"self restoreAllNotifications"
	
	self reset.
	RPackageOrganizer registerInterestToSystemAnnouncement.
	MCPackageManager registerForNotifications.
	MCWorkingCopy registerForNotifications.

	! !

!SystemAnnouncer class methodsFor: 'instance creation' stamp: 'EstebanLorenzano 7/27/2012 12:15'!
uniqueInstance
	"Answer a system annoncer."
	
	^ announcer ifNil: [ announcer := self new ]! !

!SystemAnnouncer class methodsFor: 'initialization' stamp: 'CamilleTeruel 7/29/2012 18:49'!
unload
	SystemAnnouncer uniqueInstance unsubscribe: self! !

!SystemAnnouncer class methodsFor: 'utilities' stamp: 'EstebanLorenzano 7/27/2012 12:18'!
unsubscribe: anObject
	"unsubscribe an object from my public/private announcers"
	self flag: #todo. "remove me"
	self uniqueInstance unsubscribe: anObject.
	self privateAnnouncer unsubscribe: anObject! !

!SystemAnnouncer methodsFor: 'announce' stamp: 'EstebanLorenzano 7/27/2012 19:50'!
announce: anAnnouncement
	self isSuspended 
		ifFalse: [ 
			self class privateAnnouncer basicAnnounce: anAnnouncement.
			self basicAnnounce: anAnnouncement ]! !

!SystemAnnouncer methodsFor: 'announce' stamp: 'EstebanLorenzano 7/27/2012 19:36'!
basicAnnounce: anAnnouncement
	super announce: anAnnouncement.! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:42'!
class: aClass oldComment: oldComment newComment: newComment oldStamp: oldStamp newStamp: newStamp
	"A class was commented in the system."

	self announce: (CommentedEvent class: aClass oldComment: oldComment newComment: newComment oldStamp: oldStamp newStamp: newStamp)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:42'!
class: aClass recategorizedFrom: oldCategory to: newCategory 
	self announce: (RecategorizedEvent 
				class: aClass
				category: newCategory
				oldCategory: oldCategory)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:42'!
classAdded: aClass inCategory: aCategoryName 
	self announce: (AddedEvent class: aClass category: aCategoryName)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:42'!
classCategoryAdded: aClassCategoryName

	self announce: (AddedEvent
					classCategory: aClassCategoryName)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:42'!
classCategoryRemoved: aClassCategoryName

	self announce: (RemovedEvent
					classCategory: aClassCategoryName)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:43'!
classCategoryRenamedFrom: anOldClassCategoryName to: aNewClassCategoryName

	self announce: (RenamedEvent
					classCategoryRenamedFrom: anOldClassCategoryName 
					to: aNewClassCategoryName)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:43'!
classCommented: aClass
	"A class with the given name was commented in the system."

	self announce: (CommentedEvent class: aClass)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:43'!
classCommented: aClass inCategory: aCategoryName 
	"A class with the given name was commented in the system."

	self announce: (CommentedEvent class: aClass category: aCategoryName)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:43'!
classDefinitionChangedFrom: oldClass to: newClass
	self announce: (ModifiedClassDefinitionEvent classDefinitionChangedFrom: oldClass to: newClass)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:43'!
classRemoved: aClass fromCategory: aCategoryName 
	self announce: (RemovedEvent class: aClass category: aCategoryName)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:43'!
classRenamed: aClass from: oldClassName to: newClassName inCategory: aCategoryName 
	self announce: (RenamedEvent 
				class: aClass
				category: aCategoryName
				oldName: oldClassName
				newName: newClassName)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:43'!
classReorganized: aClass
	self announce: (ReorganizedEvent class: aClass)! !

!SystemAnnouncer methodsFor: 'temporal for remove' stamp: 'EstebanLorenzano 7/27/2012 15:42'!
doSilently: aBlock
	self suspendAllWhile: aBlock! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'EstebanLorenzano 7/27/2012 15:39'!
evaluated: textOrStream
	^ self evaluated: textOrStream context: nil.! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:43'!
evaluated: expression context: aContext
	self announce: (DoItEvent 
				expression: expression
				context: aContext)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'EstebanLorenzano 7/27/2012 11:04'!
isSuspended
	^suspended ifNil: [ suspended := false ]! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'EstebanLorenzano 7/27/2012 15:39'!
methodAdded: aMethod

	^ self methodAdded: aMethod selector: aMethod selector inClass: aMethod methodClass ! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:43'!
methodAdded: aMethod selector: aSymbol inClass: aClass 
	"A method with the given selector was added to aClass, but not put in a protocol."

	self announce: (AddedEvent
				method: aMethod 
				selector: aSymbol
				class: aClass)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:44'!
methodAdded: aMethod selector: aSymbol inClass: aClass requestor: requestor
	"A method with the given selector was added to aClass, but not put in a protocol."

	self announce: (AddedEvent
				method: aMethod 
				selector: aSymbol
				class: aClass
				requestor: requestor)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:44'!
methodAdded: aMethod selector: aSymbol inProtocol: aCategoryName class: aClass 
	"A method with the given selector was added to aClass in protocol aCategoryName."

	self announce: (AddedEvent
				method: aMethod
				selector: aSymbol
				protocol: aCategoryName
				class: aClass)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:44'!
methodAdded: aMethod selector: aSymbol inProtocol: aCategoryName class: aClass requestor: requestor
	"A method with the given selector was added to aClass in protocol aCategoryName."

	self announce: (AddedEvent
				method: aMethod
				selector: aSymbol
				protocol: aCategoryName
				class: aClass
				requestor: requestor)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:51'!
methodChangedFrom: oldMethod to: newMethod 

	self methodChangedFrom: oldMethod 
		to: newMethod 
		selector: newMethod selector 
		inClass: newMethod methodClass 
		oldProtocol: nil 
		newProtocol: nil 
		requestor: nil! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'EstebanLorenzano 7/27/2012 15:39'!
methodChangedFrom: oldMethod to: newMethod selector: newSelector inClass: aClass

	self methodChangedFrom: oldMethod to: newMethod selector: newSelector inClass: aClass oldProtocol: nil newProtocol: nil requestor: nil! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:44'!
methodChangedFrom: oldMethod to: newMethod selector: aSymbol inClass: aClass oldProtocol: oldProtocol newProtocol: newProtocol requestor: requestor
	self announce: ((ModifiedMethodEvent
					methodChangedFrom: oldMethod
					to: newMethod
					selector: aSymbol 
					inClass: aClass
					requestor: requestor)
					oldProtocol: oldProtocol;
					newProtocol: newProtocol)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'EstebanLorenzano 7/27/2012 15:39'!
methodChangedFrom: oldMethod to: newMethod selector: aSymbol inClass: aClass requestor: requestor

	self methodChangedFrom: oldMethod to: newMethod selector: aSymbol inClass: aClass oldProtocol: nil newProtocol: nil requestor: requestor! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'EstebanLorenzano 7/27/2012 15:39'!
methodRemoved: aMethod 
	^ self methodRemoved: aMethod selector: aMethod selector class: aMethod methodClass! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:44'!
methodRemoved: aMethod selector: aSymbol class: aClass 
	"A method with the given selector was removed from the class."

	self announce: (RemovedEvent
				method: aMethod 
				selector: aSymbol
				class: aClass)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:44'!
methodRemoved: aMethod selector: aSymbol inProtocol: protocol class: aClass 
	"A method with the given selector was removed from the class."

	self announce: (RemovedEvent
				method: aMethod 
				selector: aSymbol
				protocol: protocol
				class: aClass)! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:44'!
selector: selector recategorizedFrom: oldCategory to: newCategory inClass: aClass

	self announce: (RecategorizedEvent 
				method: (aClass compiledMethodAt: selector ifAbsent: [nil])
				selector: selector
				protocol: newCategory
				class: aClass
				oldProtocol: oldCategory)! !

!SystemAnnouncer methodsFor: 'announce' stamp: 'EstebanLorenzano 7/27/2012 11:06'!
suspendAllWhile: aBlock
	| oldSuspended |
	oldSuspended := self isSuspended.
	suspended := true.
	aBlock ensure: [ suspended := oldSuspended ]! !

!SystemAnnouncer methodsFor: 'system triggers' stamp: 'CamilleTeruel 7/29/2012 18:44'!
traitDefinitionChangedFrom: oldTrait to: newTrait
	self announce: (ModifiedTraitDefinitionEvent traitDefinitionChangedFrom: oldTrait to: newTrait)! !

!SystemAnnouncer methodsFor: 'temporal for remove' stamp: 'CamilleTeruel 7/30/2012 10:21'!
trigger: announcement 
	^ self announce: announcement.! !

!Color class methodsFor: '*System-Settings-Browser' stamp: 'AlainPlantec 3/5/2010 09:33'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForColor! !

!Array methodsFor: '*System-Settings-Browser' stamp: 'AlainPlantec 1/31/2010 10:35'!
settingStoreOn: aStream 
	"Use the literal form if possible."

	self isLiteral
		ifTrue: 
			[aStream nextPut: $#; nextPut: $(.
			self do: 
				[:element | 
				element settingStoreOn: aStream.
				aStream space].
			aStream nextPut: $)]
		ifFalse: [super settingStoreOn: aStream]! !

Categorizer subclass: #SystemOrganizer
	instanceVariableNames: 'environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!SystemOrganizer commentStamp: '<historical>' prior: 0!
My instances provide an organization for the classes in the system, just as a ClassOrganizer organizes the messages within a class. The only difference is the methods for fileIn/Out.!

!SystemOrganizer class methodsFor: 'cleanup' stamp: ''!
cleanUp: agressive
	"Remove empty categories when cleaning aggressively"

	agressive ifTrue: [SystemOrganization removeEmptyCategories].
! !

!SystemOrganizer class methodsFor: 'instance creation' stamp: 'SeanDeNigris 12/2/2011 08:57'!
default
	"look in the environment that the receiver organizes for the associated organization" 
	
	^ self environment organization.! !

!SystemOrganizer methodsFor: 'accessing' stamp: 'EstebanLorenzano 7/27/2012 16:30'!
addCategory: newCategory
	| r |
	r := super addCategory: newCategory.
	SystemAnnouncer uniqueInstance classCategoryAdded: newCategory.
	^ r! !

!SystemOrganizer methodsFor: 'testing' stamp: 'LucFabresse 10/24/2010 17:42'!
categoriesMatching: matchString
	"Return all matching categories"
	
	self categories ifNil: [ ^#() ].
	^ self categories select: [:c | matchString match: c]! !

!SystemOrganizer methodsFor: 'utils' stamp: ''!
classesInCategory: category
	^ (self listAtCategoryNamed: category) collect: [ :className | self environment at: className ] ! !

!SystemOrganizer methodsFor: 'utils' stamp: ''!
commentInventory: categoryName
	"SystemOrganization commentInventory: 'Morphic*'"

	| classes commentedClasses |
	classes := OrderedCollection new.
	self categories
		withIndexCollect: [ :cat :idx | 
			(categoryName match: cat)
				ifTrue: [ classes addAll: (self listAtCategoryNumber: idx) ]
				ifFalse: [ nil ] ].
	commentedClasses := classes select: [ :catCls | (self environment at: catCls) hasComment ].
	^ 'There are ' , classes size asString , ' classes in ' , categoryName , ' of which ' , commentedClasses size asString
		, ' have comments and ' , (classes size - commentedClasses size) asString , ' do not yet have comments.'! !

!SystemOrganizer methodsFor: 'accessing' stamp: ''!
environment
	 ^ environment ifNil: [ environment := Smalltalk globals]! !

!SystemOrganizer methodsFor: 'accessing' stamp: ''!
environment: aSystemDictionary
	 environment := aSystemDictionary.
! !

!SystemOrganizer methodsFor: 'filein/out' stamp: 'SeanDeNigris 7/12/2012 08:44'!
fileOut  
	"SystemOrganization fileOut"

	| internalStream |
	internalStream := (String new: 30000) writeStream.
	internalStream nextPutAll: 'SystemOrganization changeFromCategorySpecs: #('; cr;
		print: SystemOrganization;  "ends with a cr"
		nextPutAll: ')!!'; cr.

	FileStream 
		writeSourceCodeFrom: internalStream 
		baseName: 'SystemOrganization.st' asFileReference nextVersion basename 
		isSt: true
! !

!SystemOrganizer methodsFor: 'filein/out' stamp: 'ab 06/29/2011 14:02'!
fileOutCategory: category 
	"Store on the file named category (a string) concatenated with '.st' all the 
	classes associated with the category."
	
	| internalStream |
	internalStream := (String new: 1000) writeStream.
	self fileOutCategory: category on: internalStream initializing: true.
	^ FileStream writeSourceCodeFrom: internalStream baseName: category isSt: true ! !

!SystemOrganizer methodsFor: 'filein/out' stamp: ''!
fileOutCategory: category on: aFileStream 
	"Store on the file associated with aFileStream, all the classes associated 
	with the category and any requested shared pools."
	^self fileOutCategory: category on: aFileStream initializing: true! !

!SystemOrganizer methodsFor: 'filein/out' stamp: ''!
fileOutCategory: category on: aFileStream initializing: aBool
	"Store on the file associated with aFileStream, all the traits and classes associated 
	with the category and any requested shared pools in the right order."

	| first poolSet tempClass classes traits |
	traits := self orderedTraitsIn: category.
	classes := self superclassOrder: category.
	poolSet := Set new.
	classes do:  [:class | class sharedPools do: [:eachPool | poolSet add: eachPool]].
	poolSet size > 0 ifTrue: [
		tempClass := Class new.
		tempClass shouldFileOutPools ifTrue: [
			poolSet := poolSet select: [:aPool |
				tempClass shouldFileOutPool: (Smalltalk globals keyAtIdentityValue: aPool)].
			poolSet do: [:aPool | tempClass fileOutPool: aPool onFileStream: aFileStream]]].
	first := true.
	traits, classes do: [:each | 
		first
			ifTrue: [first := false]
			ifFalse: [aFileStream cr; nextPut: Character newPage; cr].
		each
			fileOutOn: aFileStream
			moveSource: false
			toFile: 0
			initializing: false].
	aBool ifTrue: [classes do: [:cls | cls fileOutInitializerOn: aFileStream]].! !

!SystemOrganizer methodsFor: 'remove' stamp: 'StephaneDucasse 5/20/2012 17:37'!
includesCategory: aString
	"Return all matching categories"
	
	self categories ifNil: [ ^#() ].
	^ self categories includes: aString! !

!SystemOrganizer methodsFor: 'initialize-release' stamp: 'LucFabresse 10/24/2010 19:57'!
initialize
	super initialize.
	categoryArray := Array new.
	categoryStops := Array new.
	elementArray := Array new! !

!SystemOrganizer methodsFor: 'utils' stamp: ''!
orderedTraitsIn: category
	"Answer an OrderedCollection containing references to the traits in the 
	category whose name is the argument, category (a string). The traits 
	are ordered so they can be filed in." 

	| behaviors traits |
	behaviors := (self listAtCategoryNamed: category asSymbol) collect: [ :title | self environment at: title ].
	traits := behaviors reject: [ :each | each isBehavior ].
	traits := traits
		asSortedCollection: [ :t1 :t2 | (t2 traitComposition allTraits includes: t1) or: [ (t1 traitComposition allTraits includes: t2) not ] ].
	^ traits asArray! !

!SystemOrganizer methodsFor: 'remove' stamp: ''!
removeCategoriesMatching: matchString
	"Remove all matching categories with their classes"
	(self categoriesMatching: matchString) do:
		[:c | self removeSystemCategory: c]! !

!SystemOrganizer methodsFor: 'accessing' stamp: 'EstebanLorenzano 7/27/2012 16:30'!
removeCategory: cat 
	| r |
	r := super removeCategory: cat.
	SystemAnnouncer uniqueInstance classCategoryRemoved: cat.
	^ r! !

!SystemOrganizer methodsFor: 'utils' stamp: ''!
removeMissingClasses
	"Remove any class names that are no longer in the Smalltalk dictionary. Used for cleaning up after garbage collecting user-generated classes."

	"SystemOrganization removeMissingClasses"
 
	elementArray copy
		do: [ :el | 
			(self environment includesKey: el)
				ifFalse: [ self removeElement: el ] ]! !

!SystemOrganizer methodsFor: 'remove' stamp: ''!
removeSystemCategory: category
	"remove all the classes and traits associated with the category"

	 (self orderedTraitsIn: category) , (self superclassOrder: category)
		reverseDo: [:each | each removeFromSystem].

	self removeCategory: category.
! !

!SystemOrganizer methodsFor: 'accessing' stamp: 'EstebanLorenzano 7/27/2012 16:30'!
renameCategory: oldCatString toBe: newCatString
	| r |
	r := super renameCategory: oldCatString toBe: newCatString.
	SystemAnnouncer uniqueInstance 
		classCategoryRenamedFrom: oldCatString to: newCatString.
	^ r! !

!SystemOrganizer methodsFor: 'utils' stamp: 'GuillermoPolito 6/26/2012 15:58'!
superclassOrder: category
	"Answer an OrderedCollection containing references to the classes in the 
	category whose name is the argument, category (a string). The classes 
	are ordered with superclasses first so they can be filed in."

	| behaviors classes |
	behaviors := (self listAtCategoryNamed: category asSymbol) collect: [ :title | self environment at: title ].
	classes := behaviors select: [ :each | each isBehavior ].
	^ Class superclassOrder: classes ! !

!SystemOrganizer methodsFor: 'query' stamp: ''!
uncommentedClassesIn: categoryName
	"SystemOrganization uncommentedClassesIn: 'Morphic*'"

	| classes |
	classes := OrderedCollection new.
	self categories
		withIndexCollect: [ :cat :idx | 
			(categoryName match: cat)
				ifTrue: [ classes addAll: (self listAtCategoryNumber: idx) ]
				ifFalse: [ nil ] ].
	^ (classes collect: [ :clsName | self environment at: clsName ] thenSelect: [ :cls | cls hasComment not ]) asArray! !

!ZipArchive class methodsFor: '*System-FileRegistry' stamp: 'CarloTeixeira 8/3/2010 01:09'!
fileReaderServicesForFile: fullName suffix: suffix 
	"Answers a collection of services that can handle the file and suffixes"
	|  services |
	services := OrderedCollection new.
	({'zip'.'sar'.'pr'. 'mcz'. '*'} includes: suffix)
		ifTrue: [	services add: self serviceExtractAll].
	^services.! !

!ZipArchive class methodsFor: '*System-FileRegistry' stamp: 'CarloTeixeira 8/3/2010 01:08'!
serviceExtractAll
	"Answer a service for extracting files in a zip to a user-specified directory"

	^ FileModifyingSimpleServiceEntry 
		provider: self
		label: 'extract all to...'
		selector: #extractAllIn: 
		description: 'extract all files to a user-specified directory'
		buttonLabel: 'extract all'! !

!StrikeFontSet methodsFor: '*system-settings-browser' stamp: 'AlainPlantec 1/31/2010 10:45'!
settingStoreOn: aStream 
	aStream nextPut: $(.
	aStream nextPutAll: self class name.
	aStream nextPut: Character space.
	aStream nextPutAll: 'familyName: ''', self familyName, ''' size: ', self pointSize asString, ' emphasized: ', self emphasis asString.
	aStream nextPut: $)
! !

!Point class methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 4/21/2009 09:57'!
settingInputWidgetForNode: aSettingNode
	^ aSettingNode inputWidgetForPoint! !

MessageSend subclass: #ObjectFinalizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Finalization'!

!ObjectFinalizer commentStamp: 'jcg 4/27/2010 01:01' prior: 0!
Represents an action that is to be taken when a target object (not referenced by the ObjectFinalizer) is garbage-collected.!

!ObjectFinalizer methodsFor: 'finalization' stamp: 'IgorStasenko 8/17/2011 15:43'!
finalize
	"Finalize the resource associated with the receiver. This message should only be sent during the finalization process. There is NO garantuee that the resource associated with the receiver hasn't been free'd before so take care that you don't run into trouble - this all may happen with interrupt priority."
	self value! !

!ObjectFinalizer methodsFor: 'initialize' stamp: 'jcg 1/15/2010 01:36'!
receiver: anObject selector: aSymbol argument: aParameter
	"Not really necessary now that we're a subclass of MessageSend, but we keep it around because existing code might expect it."
	self 
		receiver: anObject;
		selector: aSymbol;
		arguments: (Array with: aParameter).! !

!LogicalFont methodsFor: '*system-settings-browser' stamp: 'alain.plantec 3/19/2009 10:02'!
settingStoreOn: aStream 
	aStream nextPut: $(.
	aStream nextPutAll: self class name.
	aStream nextPut: Character space.
	aStream nextPutAll: 'familyName: '.
	self familyName settingStoreOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: 'fallbackFamilyNames: '.
	self fallbackFamilyNames settingStoreOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: 'pointSize: '.
	self pointSize settingStoreOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: 'stretchValue: '.
	self stretchValue settingStoreOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: 'weightValue: '.
	self weightValue settingStoreOn: aStream.
	aStream nextPut: Character space.
	aStream nextPutAll: 'slantValue: '.
	self slantValue settingStoreOn: aStream.
	aStream nextPut: $)! !

!LogicalFont methodsFor: '*system-settings-browser' stamp: 'AlainPlantec 12/18/2009 08:55'!
withSizeIncrementedBy: anInteger
	^ LogicalFont 
		familyName: self familyName 
		fallbackFamilyNames: self fallbackFamilyNames
		pointSize: self pointSize + anInteger
		stretchValue: self stretchValue
		weightValue: self weightValue
		slantValue: self slantValue ! !

!ClassDescription methodsFor: '*System-Support' stamp: ''!
allUnreferencedClassVariables
	"Answer a list of the names of all the receiver's unreferenced class vars, including those defined in superclasses"

	^ self systemNavigation allUnreferencedClassVariablesOf: self! !

Object subclass: #AbstractEvent
	instanceVariableNames: 'item itemKind environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!AbstractEvent commentStamp: 'StephaneDucasse 2/10/2011 16:05' prior: 0!
I'm a generic event raised by the system (SystemChangeNotifier unique instance) when anything (both classes or methods) changes in the system. Each one of my children represents a specific kind of event the system can notify. 

Now I will soon fade away and be replaced by announcements. !

!AbstractEvent class methodsFor: 'accessing' stamp: 'NS 1/16/2004 14:08'!
allChangeKinds
	"AbstractEvent allChangeKinds"

	^AbstractEvent allSubclasses collect: [:cl | cl changeKind]! !

!AbstractEvent class methodsFor: 'accessing' stamp: 'bvs 7/20/2004 12:12'!
allItemKinds
	"self allItemKinds"

	^(AbstractEvent class organization listAtCategoryNamed: #'item kinds') 
		collect: [:sel | self perform: sel]! !

!AbstractEvent class methodsFor: 'item kinds' stamp: 'rw 7/9/2003 11:12'!
categoryKind

	^#category! !

!AbstractEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:08'!
changeKind
	"Return a symbol, with a : as last character, identifying the change kind."

	self subclassResponsibility! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'NS 1/19/2004 18:42'!
class: aClass
	^ self item: aClass kind: AbstractEvent classKind.! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'NS 1/19/2004 18:42'!
class: aClass category: cat 
	| instance |
	instance := self class: aClass.
	instance itemCategory: cat.
	^instance! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'ab 2/10/2005 16:32'!
classCategory: aName
	^ self item: aName kind: AbstractEvent categoryKind.! !

!AbstractEvent class methodsFor: 'item kinds' stamp: 'rw 7/9/2003 11:12'!
classKind

	^#class! !

!AbstractEvent class methodsFor: 'temporary' stamp: 'rw 7/11/2003 10:23'!
comment1

"Smalltalk organization removeElement: #ClassForTestingSystemChanges3
Smalltalk garbageCollect 
Smalltalk organizati

classify:under:


SystemChangeNotifier uniqueInstance releaseAll
SystemChangeNotifier uniqueInstance noMoreNotificationsFor: aDependent.


aDependent := SystemChangeNotifierTest new.
SystemChangeNotifier uniqueInstance
	notifyOfAllSystemChanges: aDependent
	using: #event:

SystemChangeNotifier uniqueInstance classAdded: #Foo inCategory: #FooCat



| eventSource dependentObject |
eventSource := EventManager new.
dependentObject := Object new.

register - dependentObject becomes dependent:
eventSource
	when: #anEvent send: #error to: dependentObject.

unregister dependentObject:
eventSource removeDependent: dependentObject.

[eventSource triggerEvent: #anEvent]
	on: Error
	do: [:exc | self halt: 'Should not be!!']."! !

!AbstractEvent class methodsFor: 'temporary' stamp: 'rw 7/11/2003 10:24'!
comment2

"HTTPSocket useProxyServerNamed: 'proxy.telenet.be' port: 8080
TestRunner open

--------------------
We propose two orthogonal groups to categorize each event:
(1) the 'change type':
	added, removed, modified, renamed
	+ the composite 'changed' (see below for an explanation)
(2) the 'item type':
	class, method, instance variable, pool variable, protocol, category
	+ the composite 'any' (see below for an explanation).
The list of supported events is the cross product of these two lists (see below for an explicit enumeration of the events).

Depending on the change type, certain information related to the change is always present (for adding, the new things that was added, for removals, what was removed, for renaming, the old and the new name, etc.).

Depending on the item type, information regarding the item is present (for a method, which class it belongs to). 

Certain events 'overlap', for example, a method rename triggers a class change. To capture this I impose a hierarchy on the 'item types' (just put some numbers to clearly show the idea. They don't need numbers, really. Items at a certain categories are included by items one category number higher):
level 1 category
level 2 class
level 3 instance variable, pool variable, protocol, method.

Changes propagate according to this tree: any 'added', 'removed' or 'renamed' change type in level X triggers a 'changed' change type in level X - 1. A 'modified' change type does not trigger anything special.
For example, a method additions triggers a class modification. This does not trigger a category modification.

Note that we added 'composite events': wildcards for the 'change type' ('any' - any system additions) and for the 'item type' ('Changed' - all changes related to classes), and one for 'any change systemwide' (systemChanged).

This result is this list of Events:

classAdded
classRemoved
classModified
classRenamed (?)
classChanged (composite)

methodAdded
methodRemoved
methodModified
methodRenamed (?)
methodChanged (composite)

instanceVariableAdded
instanceVariableRemoved
instanceVariableModified 
instanceVariableRenamed (?)
instanceVariableChanged (composite)

protocolAdded
protocolRemoved
protocolModified
protocolRenamed (?)
protocolChanged (composite)

poolVariableAdded
poolVariableRemoved
poolVariableModified
poolVariableRenamed (?)
poolChanged (composite)

categoryAdded
categoryRemoved
categoryModified
categeryRenamed (?)
categoryChanged (composite)

anyAdded (composite)
anyRemoved (composite)
anyModified (composite)
anyRenamed (composite)

anyChanged (composite)



To check: can we pass somehow the 'source' of the change (a browser, a file-in, something else) ? Maybe by checking the context, but should not be too expensive either... I found this useful in some of my tools, but it might be too advanced to have in general. Tools that need this can always write code to check it for them.  But is not always simple...


Utilities (for the recent methods) and ChangeSet are the two main clients at this moment.

Important: make it very explicit that the event is send synchronously (or asynchronously, would we take that route).


					category
						class
							comment
							protocol
								method
OR
				category
				Smalltalk
					class
						comment
						protocol
						method
??



						Smalltalk	category
								\	/
								class
							/	  |	\
						comment  |	protocol
								  |	/
								method

"! !

!AbstractEvent class methodsFor: 'temporary' stamp: 'rw 7/11/2003 15:43'!
comment3

"Things to consider for trapping:
ClassOrganizer>>#changeFromCategorySpecs:
	Problem: I want to trap this to send the appropriate bunch of ReCategorization events, but ClassOrganizer instances do not know where they belong to (what class, or what system); it just uses symbols. So I cannot trigger the change, because not enough information is available. This is a conceptual problem: the organization is stand-alone implementation-wise, while conceptually it belongs to a class. The clean solution could be to reroute this message to a class, but this does not work for all of the senders (that would work from the browserm but not for the file-in).

Browser>>#categorizeAllUncategorizedMethods
	Problem: should be trapped to send a ReCategorization event. However, this is model code that should not be in the Browser. Clean solution is to move it out of there to the model, and then trap it there (or reroute it to one of the trapped places).

Note: Debugger>>#contents:notifying: recompiles methods when needed, so I trapped it to get updates. However, I need to find a way to write a unit test for this. Haven't gotten around yet for doing this though...
"! !

!AbstractEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:18'!
eventSelectorBlock

	^[:itemKind :changeKind | itemKind, changeKind, 'Event:']! !

!AbstractEvent class methodsFor: 'item kinds' stamp: 'rw 7/14/2003 11:41'!
expressionKind

	^#expression! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'rw 7/9/2003 11:19'!
item: anItem kind: anItemKind

	^self basicNew item: anItem kind: anItemKind! !

!AbstractEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:19'!
itemChangeCombinations

	^self supportedKinds collect: [:itemKind | self eventSelectorBlock value: itemKind value: self changeKind]! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'NS 1/16/2004 14:19'!
method: aMethod class: aClass

	| instance |
	instance := self item: aMethod kind: self methodKind.
	instance itemClass: aClass.
	^instance! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'HenrikSperreJohansen 10/5/2010 11:11'!
method: aMethod protocol: prot class: aClass

	| instance |
	instance := self method: aMethod class: aClass.
	instance itemProtocol: prot.
	^instance! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'NS 1/27/2004 10:48'!
method: aMethod selector: aSymbol class: aClass

	| instance |
	instance := self item: aMethod kind: self methodKind.
	instance itemSelector: aSymbol.
	instance itemClass: aClass.
	^instance! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'NS 1/27/2004 10:49'!
method: aMethod selector: aSymbol class: aClass requestor: requestor

	| instance |
	instance := self method: aMethod selector: aSymbol class: aClass.
	instance itemRequestor: requestor.
	^instance! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'NS 1/27/2004 10:49'!
method: aMethod selector: aSymbol protocol: prot class: aClass

	| instance |
	instance := self method: aMethod selector: aSymbol class: aClass.
	instance itemProtocol: prot.
	^instance! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'NS 1/27/2004 10:50'!
method: aMethod selector: aSymbol protocol: prot class: aClass requestor: requestor

	| instance |
	instance := self method: aMethod selector: aSymbol protocol: prot class: aClass.
	instance itemRequestor: requestor.
	^instance! !

!AbstractEvent class methodsFor: 'item kinds' stamp: 'rw 7/9/2003 11:12'!
methodKind

	^#method! !

!AbstractEvent class methodsFor: 'instance creation' stamp: 'rw 6/30/2003 09:20'!
new
	"Override new to trigger an error, since we want to use specialized methods to create basic and higher-level events."

	^self error: 'Instances can only be created using specialized instance creation methods.'! !

!AbstractEvent class methodsFor: 'item kinds' stamp: 'rw 7/10/2003 12:36'!
protocolKind

	^#protocol! !

!AbstractEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:04'!
supportedKinds
	"All the kinds of items that this event can take. By default this is all the kinds in the system. But subclasses can override this to limit the choices. For example, the SuperChangedEvent only works with classes, and not with methods, instance variables, ..."

	^self allItemKinds! !

!AbstractEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 11:39'!
systemEvents
	"Return all the possible events in the system. Make a cross product of 
	the items and the change types."
	"self systemEvents"

	^self allSubclasses
		inject: OrderedCollection new
		into: [:allEvents :eventClass | allEvents addAll: eventClass itemChangeCombinations; yourself]! !

!AbstractEvent methodsFor: '*System-Announcements' stamp: 'EstebanLorenzano 7/27/2012 15:38'!
asAnnouncement 
	^self asSystemAnnouncement! !

!AbstractEvent methodsFor: '*System-Announcements' stamp: 'CamilleTeruel 7/29/2012 18:40'!
asSystemAnnouncement
	| itemKindName eventKindName announcementClassName announcementClass |
	itemKindName := self itemKind.
	eventKindName := (self class name endsWith: 'Event') 
		ifTrue: [ self class name copyFrom: 1 to: self class name size - 'Event' size ] 
		ifFalse: [ self class name ].
	announcementClassName := (itemKindName capitalized, eventKindName) asSymbol.
	announcementClass := Smalltalk at: announcementClassName ifAbsent: [nil].
	^ announcementClass isNil 
		ifTrue: [ UnknownSystemChange new ]
		ifFalse: [announcementClass new event: self]! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'rw 7/10/2003 12:10'!
changeKind

	^self class changeKind! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'rw 7/10/2003 12:43'!
environmentAt: anItemKind

	(self itemKind = anItemKind) ifTrue: [^self item].
	^environment at: anItemKind ifAbsent: [nil]! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'rw 7/10/2003 12:20'!
eventSelector

	^self class eventSelectorBlock value: itemKind value: self changeKind! !

!AbstractEvent methodsFor: 'testing' stamp: 'rw 6/30/2003 08:34'!
isAdded

	^false! !

!AbstractEvent methodsFor: 'testing' stamp: 'NS 1/19/2004 18:41'!
isCategoryKnown

	^self itemCategory notNil! !

!AbstractEvent methodsFor: 'testing' stamp: 'rw 7/10/2003 15:01'!
isCommented

	^false! !

!AbstractEvent methodsFor: 'testing' stamp: 'rw 7/14/2003 10:15'!
isDoIt

	^false! !

!AbstractEvent methodsFor: 'testing' stamp: 'NS 1/19/2004 15:09'!
isModified

	^false! !

!AbstractEvent methodsFor: 'testing' stamp: 'NS 1/21/2004 09:40'!
isProtocolKnown

	^self itemCategory notNil! !

!AbstractEvent methodsFor: 'testing' stamp: 'rw 7/1/2003 19:53'!
isRecategorized

	^false! !

!AbstractEvent methodsFor: 'testing' stamp: 'rw 6/30/2003 08:34'!
isRemoved

	^false! !

!AbstractEvent methodsFor: 'testing' stamp: 'rw 7/1/2003 11:35'!
isRenamed

	^false! !

!AbstractEvent methodsFor: 'testing' stamp: 'NS 1/27/2004 12:44'!
isReorganized
	^ false! !

!AbstractEvent methodsFor: 'accessing' stamp: 'rw 6/30/2003 08:22'!
item
	"Return the item that triggered the event (typically the name of a class, a category, a protocol, a method)."

	^item! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'rw 7/10/2003 12:36'!
item: anItem kind: anItemKind

	item := anItem.
	itemKind := anItemKind.
	environment := Dictionary new! !

!AbstractEvent methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:43'!
itemCategory

	^self environmentAt: self class categoryKind! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'rw 7/10/2003 12:37'!
itemCategory: aCategory

	environment at: self class categoryKind put: aCategory! !

!AbstractEvent methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:43'!
itemClass

	^self environmentAt: self class classKind! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'rw 7/10/2003 12:36'!
itemClass: aClass

	environment at: self class classKind put: aClass! !

!AbstractEvent methodsFor: 'accessing' stamp: 'rw 7/14/2003 12:10'!
itemExpression

	^self environmentAt: self class expressionKind! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'rw 7/14/2003 12:11'!
itemExpression: anExpression

	environment at: self class expressionKind put: anExpression! !

!AbstractEvent methodsFor: 'accessing' stamp: 'rw 6/30/2003 08:22'!
itemKind
	"Return the kind of the item of the event (#category, #class, #protocol, #method, ...)"

	^itemKind! !

!AbstractEvent methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:44'!
itemMethod

	^self environmentAt: self class methodKind! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'rw 7/10/2003 12:38'!
itemMethod: aMethod

	environment at: self class methodKind put: aMethod! !

!AbstractEvent methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:44'!
itemProtocol

	^self environmentAt: self class protocolKind! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'rw 7/10/2003 12:38'!
itemProtocol: aProtocol

	environment at: self class protocolKind put: aProtocol! !

!AbstractEvent methodsFor: 'accessing' stamp: 'NS 1/27/2004 10:38'!
itemRequestor

	^self environmentAt: #requestor! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'NS 1/27/2004 10:38'!
itemRequestor: requestor

	environment at: #requestor put: requestor! !

!AbstractEvent methodsFor: 'accessing' stamp: 'NS 1/27/2004 10:38'!
itemSelector

	^self environmentAt: #selector! !

!AbstractEvent methodsFor: 'private-accessing' stamp: 'NS 1/27/2004 10:39'!
itemSelector: aSymbol

	environment at: #selector put: aSymbol! !

!AbstractEvent methodsFor: 'printing' stamp: 'NS 1/19/2004 17:52'!
printOn: aStream

	self printEventKindOn: aStream.
	aStream
		nextPutAll: ' Event for item: ';
		print: self item;
		nextPutAll: ' of kind: ';
		print: self itemKind! !

!AbstractEvent methodsFor: 'triggering' stamp: 'rw 7/14/2003 17:06'!
trigger: anEventManager 
	"Trigger the event manager."

	anEventManager triggerEvent: self eventSelector with: self.! !

AbstractEvent subclass: #AddedEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!AddedEvent commentStamp: 'StephaneDucasse 2/10/2011 16:06' prior: 0!
I'm the event sent by the system when something is added.!

!AddedEvent class methodsFor: 'accessing' stamp: 'rw 7/19/2003 09:52'!
changeKind

	^#Added! !

!AddedEvent class methodsFor: 'accessing' stamp: 'NS 1/20/2004 12:22'!
supportedKinds
	"All the kinds of items that this event can take."
	
	^ Array with: self classKind with: self methodKind with: self categoryKind with: self protocolKind! !

!AddedEvent methodsFor: 'testing' stamp: 'rw 6/30/2003 08:35'!
isAdded

	^true! !

!AddedEvent methodsFor: 'printing' stamp: 'rw 6/30/2003 09:31'!
printEventKindOn: aStream

	aStream nextPutAll: 'Added'! !

AbstractEvent subclass: #CommentedEvent
	instanceVariableNames: 'oldComment newComment oldStamp newStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!CommentedEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:08'!
changeKind

	^#Commented! !

!CommentedEvent class methodsFor: 'accessing' stamp: ''!
class: aClass oldComment: oldComment newComment: newComment

	^(self class: aClass) oldComment: oldComment; newComment: newComment; yourself! !

!CommentedEvent class methodsFor: 'accessing' stamp: 'gk 8/23/2007 21:36'!
class: aClass oldComment: oldComment newComment: newComment oldStamp: oldStamp newStamp: newStamp

	^(self class: aClass)
		oldComment: oldComment;
		newComment: newComment;
		oldStamp: oldStamp;
		newStamp: newStamp; yourself! !

!CommentedEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 11:20'!
supportedKinds

	^Array with: self classKind! !

!CommentedEvent methodsFor: 'testing' stamp: 'rw 7/1/2003 11:37'!
isCommented

	^true! !

!CommentedEvent methodsFor: 'accessing' stamp: 'gk 8/15/2007 01:17'!
newComment
	^newComment! !

!CommentedEvent methodsFor: 'accessing' stamp: 'gk 8/15/2007 01:17'!
newComment: aString
	newComment := aString! !

!CommentedEvent methodsFor: 'accessing' stamp: 'gk 8/23/2007 21:37'!
newStamp
	^newStamp! !

!CommentedEvent methodsFor: 'accessing' stamp: 'gk 8/23/2007 21:37'!
newStamp: aString
	newStamp := aString! !

!CommentedEvent methodsFor: 'accessing' stamp: 'gk 8/15/2007 01:18'!
oldComment
	^oldComment! !

!CommentedEvent methodsFor: 'accessing' stamp: 'gk 8/15/2007 01:17'!
oldComment: aString
	oldComment := aString! !

!CommentedEvent methodsFor: 'accessing' stamp: 'gk 8/23/2007 21:37'!
oldStamp
	^oldStamp! !

!CommentedEvent methodsFor: 'accessing' stamp: 'gk 8/23/2007 21:37'!
oldStamp: aString
	oldStamp := aString! !

!CommentedEvent methodsFor: 'printing' stamp: 'rw 7/1/2003 11:37'!
printEventKindOn: aStream

	aStream nextPutAll: 'Commented'! !

AbstractEvent subclass: #DoItEvent
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!DoItEvent commentStamp: 'StephaneDucasse 2/10/2011 16:06' prior: 0!
I'm the event sent by the system when a doit is done.!

!DoItEvent class methodsFor: 'accessing' stamp: 'rw 7/14/2003 10:19'!
changeKind

	^#DoIt! !

!DoItEvent class methodsFor: 'instance creation' stamp: 'NS 1/19/2004 09:47'!
expression: stringOrStream context: aContext
	| instance |
	instance := self item: stringOrStream kind: AbstractEvent expressionKind.
	instance context: aContext.
	^instance! !

!DoItEvent class methodsFor: 'accessing' stamp: 'NS 1/20/2004 12:23'!
supportedKinds
	^ Array with: self expressionKind! !

!DoItEvent methodsFor: '*System-Announcements' stamp: 'EstebanLorenzano 7/27/2012 14:20'!
asSystemAnnouncement
	^ ExpressionEvaluated new 
		event: self;
		yourself! !

!DoItEvent methodsFor: 'accessing' stamp: 'rw 7/14/2003 11:29'!
context

	^context! !

!DoItEvent methodsFor: 'private-accessing' stamp: 'rw 7/14/2003 11:29'!
context: aContext

	context := aContext! !

!DoItEvent methodsFor: 'testing' stamp: 'rw 7/14/2003 10:15'!
isDoIt

	^true! !

!DoItEvent methodsFor: 'printing' stamp: 'rw 7/14/2003 10:15'!
printEventKindOn: aStream

	aStream nextPutAll: 'DoIt'! !

AbstractEvent subclass: #ModifiedEvent
	instanceVariableNames: 'oldItem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

ModifiedEvent subclass: #ModifiedClassDefinitionEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!ModifiedClassDefinitionEvent class methodsFor: 'instance creation' stamp: 'NS 1/20/2004 11:52'!
classDefinitionChangedFrom: oldClass to: newClass
	| instance |
	instance := self item: newClass kind: self classKind.
	instance oldItem: oldClass.
	^instance! !

!ModifiedClassDefinitionEvent class methodsFor: 'accessing' stamp: 'NS 1/20/2004 12:26'!
supportedKinds
	"All the kinds of items that this event can take."
	
	^ Array with: self classKind! !

!ModifiedClassDefinitionEvent methodsFor: 'testing' stamp: 'NS 1/20/2004 19:31'!
areClassVarsModified
	^ self classVarNames ~= self oldClassVarNames! !

!ModifiedClassDefinitionEvent methodsFor: 'testing' stamp: 'NS 1/20/2004 19:30'!
areInstVarsModified
	^ self instVarNames ~= self oldInstVarNames! !

!ModifiedClassDefinitionEvent methodsFor: 'testing' stamp: 'NS 1/20/2004 19:32'!
areSharedPoolsModified
	^ self sharedPools ~= self oldSharedPools! !

!ModifiedClassDefinitionEvent methodsFor: 'accessing' stamp: 'gk 8/22/2007 01:10'!
classVarNames
	^ item classVarNames! !

!ModifiedClassDefinitionEvent methodsFor: 'accessing' stamp: 'gk 8/22/2007 00:11'!
instVarNames
	^ item instVarNames! !

!ModifiedClassDefinitionEvent methodsFor: 'testing' stamp: 'NS 1/20/2004 19:29'!
isSuperclassModified
	^ item superclass ~~ oldItem superclass! !

!ModifiedClassDefinitionEvent methodsFor: 'testing' stamp: 'gk 8/21/2007 02:21'!
isTypeModified
	^ item typeOfClass ~~ oldItem typeOfClass! !

!ModifiedClassDefinitionEvent methodsFor: 'accessing' stamp: 'gk 8/22/2007 01:10'!
oldClassVarNames
	^ oldItem classVarNames! !

!ModifiedClassDefinitionEvent methodsFor: 'accessing' stamp: 'gk 8/22/2007 00:30'!
oldInstVarNames
	^ oldItem instVarNames! !

!ModifiedClassDefinitionEvent methodsFor: 'accessing' stamp: 'NS 1/20/2004 19:31'!
oldSharedPools
	^ oldItem sharedPools! !

!ModifiedClassDefinitionEvent methodsFor: 'accessing' stamp: 'NS 1/20/2004 19:28'!
oldSuperclass
	^ oldItem superclass! !

!ModifiedClassDefinitionEvent methodsFor: 'accessing' stamp: 'gk 8/21/2007 02:23'!
oldTypeOfClass
	^ oldItem typeOfClass! !

!ModifiedClassDefinitionEvent methodsFor: 'accessing' stamp: 'NS 1/20/2004 19:31'!
sharedPools
	^ item sharedPools! !

!ModifiedClassDefinitionEvent methodsFor: 'accessing' stamp: 'NS 1/20/2004 19:28'!
superclass
	^ item superclass! !

!ModifiedClassDefinitionEvent methodsFor: 'accessing' stamp: 'gk 8/21/2007 02:23'!
typeOfClass
	^ item typeOfClass! !

!ModifiedEvent class methodsFor: 'accessing' stamp: 'NS 1/19/2004 15:10'!
changeKind

	^#Modified! !

!ModifiedEvent class methodsFor: 'instance creation' stamp: 'NS 1/20/2004 19:37'!
classDefinitionChangedFrom: oldClass to: newClass
	^ ModifiedClassDefinitionEvent classDefinitionChangedFrom: oldClass to: newClass! !

!ModifiedEvent class methodsFor: 'instance creation' stamp: 'NS 1/27/2004 11:40'!
methodChangedFrom: oldMethod to: newMethod selector: aSymbol inClass: aClass
	| instance |
	instance := self method: newMethod selector: aSymbol class: aClass.
	instance oldItem: oldMethod.
	^ instance! !

!ModifiedEvent class methodsFor: 'instance creation' stamp: 'NS 1/27/2004 11:40'!
methodChangedFrom: oldMethod to: newMethod selector: aSymbol inClass: aClass requestor: requestor
	| instance |
	instance := self method: newMethod selector: aSymbol class: aClass requestor: requestor.
	instance oldItem: oldMethod.
	^ instance! !

!ModifiedEvent class methodsFor: 'accessing' stamp: 'NS 1/20/2004 12:25'!
supportedKinds
	"All the kinds of items that this event can take."
	
	^ Array with: self classKind with: self methodKind with: self categoryKind with: self protocolKind! !

!ModifiedEvent methodsFor: 'testing' stamp: 'NS 1/19/2004 15:09'!
isModified

	^true! !

!ModifiedEvent methodsFor: 'testing' stamp: 'mtf 10/9/2007 10:16'!
isProtocolModified
	^ false! !

!ModifiedEvent methodsFor: 'accessing' stamp: 'NS 1/19/2004 15:08'!
oldItem
	^ oldItem! !

!ModifiedEvent methodsFor: 'private-accessing' stamp: 'NS 1/19/2004 15:08'!
oldItem: anItem
	oldItem := anItem! !

!ModifiedEvent methodsFor: 'printing' stamp: 'NS 1/19/2004 15:10'!
printEventKindOn: aStream

	aStream nextPutAll: 'Modified'! !

!ModifiedEvent methodsFor: 'printing' stamp: 'NS 1/19/2004 17:57'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPutAll: ' oldItem: ';
		print: oldItem.! !

ModifiedEvent subclass: #ModifiedMethodEvent
	instanceVariableNames: 'oldProtocol newProtocol'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!ModifiedMethodEvent methodsFor: '*System-Announcements' stamp: 'HenrikSperreJohansen 1/13/2012 14:29'!
asSystemAnnouncement
	^ MethodModified new event: self! !

!ModifiedMethodEvent methodsFor: 'testing' stamp: 'test 8/26/2007 00:18'!
isProtocolModified
	^ self oldProtocol ~~ self newProtocol! !

!ModifiedMethodEvent methodsFor: 'accessing' stamp: 'mtf 8/25/2007 23:17'!
newProtocol
	^ newProtocol! !

!ModifiedMethodEvent methodsFor: 'accessing' stamp: 'mtf 8/25/2007 23:17'!
newProtocol: anObject
	newProtocol := anObject! !

!ModifiedMethodEvent methodsFor: 'accessing' stamp: 'mtf 8/25/2007 23:17'!
oldProtocol
	^ oldProtocol! !

!ModifiedMethodEvent methodsFor: 'accessing' stamp: 'mtf 8/25/2007 23:17'!
oldProtocol: anObject
	oldProtocol := anObject! !

ModifiedEvent subclass: #ModifiedTraitDefinitionEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!ModifiedTraitDefinitionEvent class methodsFor: 'accessing' stamp: 'al 7/18/2004 10:43'!
supportedKinds
	"All the kinds of items that this event can take."
	
	^ Array with: self classKind! !

!ModifiedTraitDefinitionEvent class methodsFor: 'instance creation' stamp: 'al 7/18/2004 10:50'!
traitDefinitionChangedFrom: oldTrait to: newTrait
	| instance |
	instance := self item: newTrait kind: self classKind.
	instance oldItem: oldTrait.
	^instance! !

!ModifiedTraitDefinitionEvent methodsFor: 'testing' stamp: 'al 7/18/2004 11:08'!
anyChanges
	^ self isTraitCompositionModified! !

!ModifiedTraitDefinitionEvent methodsFor: 'testing' stamp: 'al 7/18/2004 10:43'!
isTraitCompositionModified
	^self traitComposition printString ~= self oldTraitComposition printString! !

!ModifiedTraitDefinitionEvent methodsFor: 'accessing' stamp: 'al 7/18/2004 10:43'!
oldTraitComposition
	^ oldItem traitComposition! !

!ModifiedTraitDefinitionEvent methodsFor: 'printing' stamp: 'al 7/18/2004 10:47'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPutAll: ' TraitComposition: ';
		print: self isTraitCompositionModified! !

!ModifiedTraitDefinitionEvent methodsFor: 'accessing' stamp: 'al 7/18/2004 10:43'!
traitComposition
	^ item traitComposition! !

AbstractEvent subclass: #RecategorizedEvent
	instanceVariableNames: 'oldCategory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!RecategorizedEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:09'!
changeKind

	^#Recategorized! !

!RecategorizedEvent class methodsFor: 'instance creation' stamp: 'rw 7/9/2003 14:21'!
class: aClass category: cat oldCategory: oldName

	^(self class: aClass category: cat) oldCategory: oldName! !

!RecategorizedEvent class methodsFor: 'instance creation' stamp: 'HenrikSperreJohansen 10/5/2010 12:30'!
method: aMethod selector: selector protocol: prot class: aClass oldProtocol: oldName

	^(self method: aMethod selector: selector protocol: prot class: aClass) 
		oldCategory: oldName! !

!RecategorizedEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 11:20'!
supportedKinds

	^Array with: self classKind with: self methodKind! !

!RecategorizedEvent methodsFor: 'testing' stamp: 'rw 7/1/2003 19:51'!
isRecategorized

	^true! !

!RecategorizedEvent methodsFor: 'accessing' stamp: 'PavelKrivanek 7/17/2012 14:57'!
itemOldProtocol

	^ self oldCategory! !

!RecategorizedEvent methodsFor: 'accessing' stamp: 'rw 7/1/2003 20:08'!
oldCategory

	^oldCategory! !

!RecategorizedEvent methodsFor: 'accessing' stamp: 'rw 7/1/2003 20:08'!
oldCategory: aCategoryName

	oldCategory := aCategoryName! !

!RecategorizedEvent methodsFor: 'printing' stamp: 'rw 7/2/2003 09:12'!
printEventKindOn: aStream

	aStream nextPutAll: 'Recategorized'! !

AbstractEvent subclass: #RemovedEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!RemovedEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:09'!
changeKind

	^#Removed! !

!RemovedEvent class methodsFor: 'accessing' stamp: 'NS 1/20/2004 12:28'!
supportedKinds
	"All the kinds of items that this event can take."
	
	^ Array with: self classKind with: self methodKind with: self categoryKind with: self protocolKind! !

!RemovedEvent methodsFor: 'testing' stamp: 'rw 6/30/2003 08:35'!
isRemoved

	^true! !

!RemovedEvent methodsFor: 'printing' stamp: 'rw 6/30/2003 09:31'!
printEventKindOn: aStream

	aStream nextPutAll: 'Removed'! !

AbstractEvent subclass: #RenamedEvent
	instanceVariableNames: 'newName oldName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!RenamedEvent class methodsFor: 'accessing' stamp: 'rw 7/10/2003 12:09'!
changeKind

	^#Renamed! !

!RenamedEvent class methodsFor: 'instance creation' stamp: 'NS 1/27/2004 12:19'!
class: aClass category: cat oldName: oldName newName: newName

	^(self class: aClass category: cat) oldName: oldName; newName: newName! !

!RenamedEvent class methodsFor: 'instance creation' stamp: 'ab 2/10/2005 16:49'!
classCategoryRenamedFrom: anOldClassCategoryName to: aNewClassCategoryName

	^(self classCategory: anOldClassCategoryName) oldName: anOldClassCategoryName; newName: aNewClassCategoryName! !

!RenamedEvent class methodsFor: 'accessing' stamp: 'NS 1/20/2004 12:30'!
supportedKinds
	"All the kinds of items that this event can take."
	
	^ Array with: self classKind with: self categoryKind with: self protocolKind! !

!RenamedEvent methodsFor: 'testing' stamp: 'rw 7/1/2003 11:34'!
isRenamed

	^true! !

!RenamedEvent methodsFor: 'accessing' stamp: 'NS 1/27/2004 12:18'!
newName
	^ newName! !

!RenamedEvent methodsFor: 'accessing' stamp: 'NS 1/27/2004 12:18'!
newName: aName

	newName := aName! !

!RenamedEvent methodsFor: 'accessing' stamp: 'rw 7/1/2003 12:00'!
oldName

	^oldName! !

!RenamedEvent methodsFor: 'accessing' stamp: 'rw 7/1/2003 12:01'!
oldName: aName

	oldName := aName! !

!RenamedEvent methodsFor: 'printing' stamp: 'rw 7/1/2003 11:34'!
printEventKindOn: aStream

	aStream nextPutAll: 'Renamed'! !

AbstractEvent subclass: #ReorganizedEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Change Notification'!

!ReorganizedEvent commentStamp: 'StephaneDucasse 2/10/2011 16:05' prior: 0!
I'm the event sent by the system when a method or a class is recategorized.

To retrieve information, you can access to:
	item: the CompiledMethod or the class recategorized
	itemKind: #method for a method, #class for a class
	environment: aDictionary with new information
		for a method:
			- #class: the class of the method (it hasn't changed)
			- #protocol: the new category of the method
			- #selector: the selector of the method (it hasn't changed)
		for a class: 
			- #category: the new category
	oldCategory: the category where the item was formerly categorized!

!ReorganizedEvent class methodsFor: 'accessing' stamp: 'NS 1/27/2004 12:46'!
changeKind

	^#Reorganized! !

!ReorganizedEvent class methodsFor: 'accessing' stamp: 'NS 1/27/2004 12:46'!
supportedKinds

	^Array with: self classKind! !

!ReorganizedEvent methodsFor: '*System-Announcements' stamp: 'StephaneDucasse 7/15/2010 23:19'!
category

	^self itemCategory! !

!ReorganizedEvent methodsFor: 'testing' stamp: 'NS 1/27/2004 12:44'!
isReorganized
	^ true! !

!ReorganizedEvent methodsFor: 'printing' stamp: 'NS 1/27/2004 12:44'!
printEventKindOn: aStream

	aStream nextPutAll: 'Reorganized'! !

Object subclass: #AbstractSoundSystem
	instanceVariableNames: ''
	classVariableNames: 'SoundEnabled SoundQuickStart'
	poolDictionaries: ''
	category: 'System-Sound'!

!AbstractSoundSystem commentStamp: 'AlainPlantec 1/7/2010 21:36' prior: 0!
This is the abstract base class for a sound system. A sound system offers a small protocol for playing sounds and making beeps and works like a facade towards the rest of Squeak. A sound system is registered in the application registry SoundService and can be accessed by "SoundService default" like for example:

SoundService default playSoundNamed: 'croak'

The idea is that as much sound playing as possible should go through this facade. This way we decouple the sound system from the rest of Squeak and make it pluggable. It also is a perfect spot to check for the SoundSettings class>>soundEnabled

Two basic subclasses exist at the time of this writing, the BaseSoundSystem which represents the standard Squeak sound system, and the DummySoundSystem which is a dummy implementation that can be used when there is no sound card available, or when the base sound system isn't in the image, or when you simply don't want to use the available sound card.!

!AbstractSoundSystem class methodsFor: 'preference access' stamp: 'StephaneDucasse 1/30/2011 21:31'!
soundEnabled
	^ SoundEnabled ifNil: [SoundEnabled := false]! !

!AbstractSoundSystem class methodsFor: 'preference access' stamp: 'StephaneDucasse 1/30/2011 21:31'!
soundEnabled: aBoolean
	SoundEnabled := aBoolean! !

!AbstractSoundSystem class methodsFor: 'preference access' stamp: 'StephaneDucasse 1/30/2011 21:31'!
soundQuickStart
	^ SoundQuickStart ifNil: [SoundQuickStart := false]! !

!AbstractSoundSystem class methodsFor: 'preference access' stamp: 'StephaneDucasse 1/30/2011 21:31'!
soundQuickStart: aBoolean
	SoundQuickStart := aBoolean! !

!AbstractSoundSystem methodsFor: 'playing' stamp: 'gk 2/24/2004 23:27'!
beep
	"Make a primitive beep."

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:47'!
playSampledSound: samples rate: rate

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:50'!
playSoundNamed: soundName

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:51'!
playSoundNamed: soundName ifAbsentReadFrom: aifFileName

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:51'!
playSoundNamedOrBeep: soundName

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:52'!
randomBitsFromSoundInput: bitCount

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:52'!
sampledSoundChoices

	self subclassResponsibility! !

!AbstractSoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:53'!
shutDown
	"Default is to do nothing."! !

!AbstractSoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:56'!
soundNamed: soundName

	self subclassResponsibility! !

AbstractSoundSystem subclass: #DummySoundSystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Sound'!

!DummySoundSystem commentStamp: 'gk 2/24/2004 23:14' prior: 0!
This is a dummy sound system registered in SoundService to absorb all sound playing and to use the primitive beep instead of sampled sounds when playing a beep.!

!DummySoundSystem class methodsFor: 'initialization' stamp: 'StephaneDucasse 1/30/2011 21:32'!
initialize
	SoundService register: self.! !

!DummySoundSystem class methodsFor: 'initialization' stamp: 'gk 2/23/2004 21:08'!
unload
	SoundService registeredClasses do: [:ss |
		(ss isKindOf: self) ifTrue: [SoundService unregister: ss]].! !

!DummySoundSystem methodsFor: 'playing' stamp: 'gk 2/24/2004 23:53'!
beep
	"Make a primitive beep."

	Beeper beepPrimitive! !

!DummySoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:53'!
playSampledSound: samples rate: rate
	"Do nothing."
	! !

!DummySoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:54'!
playSoundNamed: soundName
	"Do nothing."! !

!DummySoundSystem methodsFor: 'playing' stamp: 'gk 2/23/2004 19:54'!
playSoundNamed: soundName ifAbsentReadFrom: aifFileName
	"Do nothing."! !

!DummySoundSystem methodsFor: 'playing' stamp: 'gk 4/8/2005 14:15'!
playSoundNamedOrBeep: soundName 
	"There is no sound support, so we make the beep."

	self beep! !

!DummySoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 20:48'!
randomBitsFromSoundInput: bitCount
 	"I'm not sure what the right thing to do here is."

 	self error: 'Can not provide random data.'! !

!DummySoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:54'!
sampledSoundChoices
	"No choices other than this."

	^ #('silence')! !

!DummySoundSystem methodsFor: 'misc' stamp: 'gk 2/23/2004 19:55'!
soundNamed: soundName
	"There are no sounds to look up."

	^ nil! !

Object subclass: #AnnouncementLogger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

!AnnouncementLogger commentStamp: 'StephaneDucasse 5/9/2011 17:17' prior: 0!
Use me to debug and log to the transcript annoncements.


AnnouncementLogger new subscribeTo: SystemAnnouncer announcer

To unsubscribe

AnnouncementLogger  allInstancesDo: [:each | SystemAnnouncer announcer unsubscribe: each ]!

!AnnouncementLogger methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 5/9/2011 17:11'!
logAnnouncement: ann

	Transcript show: ann printString ; cr! !

!AnnouncementLogger methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 5/9/2011 17:15'!
subscribeTo: anAnnouncer

	anAnnouncer "weak" 
		on: Announcement send: #logAnnouncement: to: self! !

Object subclass: #AppRegistry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Applications'!

AppRegistry class
	instanceVariableNames: 'registeredClasses default'!

!AppRegistry commentStamp: 'ads 4/2/2003 15:30' prior: 0!
AppRegistry is a simple little class, not much more than a wrapper around a collection. It's intended to help break dependencies between packages. For example, if you'd like to be able to send e-mail, you could use the bare-bones MailComposition class, or you could use the full-blown Celeste e-mail client. Instead of choosing one or the other, you can call "MailSender default" (where MailSender is a subclass of AppRegistry), and thus avoid creating a hard-coded dependency on either of the two mail senders.

This will only really be useful, of course, for applications that have a very simple, general, well-defined interface. Most of the time, you're probably better off just marking your package as being dependent on a specific other package, and avoiding the hassle of this whole AppRegistry thing. But for simple things like e-mail senders or web browsers, it might be useful.
!

AppRegistry class
	instanceVariableNames: 'registeredClasses default'!

!AppRegistry class methodsFor: 'registering' stamp: 'ads 3/29/2003 13:36'!
appName
	"Defaults to the class name, which is probably good enough, but you could override this in subclasses if you want to."
	^ self name! !

!AppRegistry class methodsFor: 'singleton' stamp: 'StephaneDucasse 1/28/2011 15:16'!
askForDefault
        "self askForDefault"
        self registeredClasses isEmpty ifTrue:
                [self inform: 'There are no ', self appName, ' applications registered.'.
                ^ default := nil].
        self registeredClasses size = 1 ifTrue:
                [^ default := self registeredClasses anyOne].
        ^Smalltalk globals
                at: #UIManager
                ifPresent: [:class | self chooseAppWithUI]
                ifAbsent:  [| stream |
                        stream := WriteStream with: 'You have to choose a Class by using ''#chooseApp: index:'''.
                        stream nextPut: Character cr.
                        self registeredClasses doWithIndex: [:class :index |
                                stream
                                        nextPutAll: index asString;
                                        nextPutAll: ' - ';
                                        nextPutAll: class name asString;
                                        nextPut: Character cr].
                        self inform: stream contents.
                        default := nil]! !

!AppRegistry class methodsFor: 'private' stamp: 'SeanDeNigris 9/15/2010 01:24'!
askForNewDefaultOnNextRequest

	default := nil.! !

!AppRegistry class methodsFor: 'private' stamp: 'StephaneDucasse 1/28/2011 15:16'!
chooseApp: anIndex

        ^default := self registeredClasses at: anIndex ifAbsent: [nil]! !

!AppRegistry class methodsFor: 'cleanup' stamp: 'StephaneDucasse 3/9/2010 16:16'!
cleanUp
	"AppRegistry cleanUp"

	self removeObsolete.! !

!AppRegistry class methodsFor: 'singleton' stamp: 'ads 3/29/2003 13:11'!
default
	^ default ifNil: [self askForDefault]! !

!AppRegistry class methodsFor: 'singleton' stamp: 'BenjaminVanRyseghem 2/24/2012 15:51'!
default: aClassOrNil
	"Sets my default to aClassOrNil. 
	Answers the old default."
	| oldDefault |
	oldDefault := default.
	aClassOrNil ifNotNil: [ self register: aClassOrNil ].
	default := aClassOrNil.
	^ oldDefault! !

!AppRegistry class methodsFor: 'singleton' stamp: 'nk 3/9/2004 12:35'!
defaultOrNil
	^ default! !

!AppRegistry class methodsFor: 'registering' stamp: 'SeanDeNigris 9/15/2010 01:25'!
register: aProviderClass

	(self registeredClasses includes: aProviderClass) ifTrue: [ ^ self ].
	
	self askForNewDefaultOnNextRequest.	"if you're registering a new app you probably want to use it"
	self registeredClasses add: aProviderClass.! !

!AppRegistry class methodsFor: 'registering' stamp: 'ads 3/29/2003 13:01'!
registeredClasses
	^ registeredClasses ifNil: [registeredClasses := OrderedCollection new]! !

!AppRegistry class methodsFor: 'cleanup' stamp: 'StephaneDucasse 1/30/2011 21:35'!
removeObsolete
	"AppRegistry removeObsolete"
	self registeredClasses copy do:[:cls| 
		(cls class isObsolete or:[cls isBehavior and:[cls isObsolete]]) 
			ifTrue:[self unregister: cls]].
	self subclasses do:[:cls| cls removeObsolete].! !

!AppRegistry class methodsFor: 'cleanup' stamp: 'StephaneDucasse 1/30/2011 21:38'!
reset
	"AppRegistry reset"

	registeredClasses := nil.! !

!AppRegistry class methodsFor: 'registering' stamp: 'ads 3/29/2003 13:03'!
unregister: aProviderClass
	(default = aProviderClass) ifTrue: [default := nil].
	self registeredClasses remove: aProviderClass ifAbsent: [].! !

!AppRegistry methodsFor: 'notes' stamp: 'ads 4/2/2003 15:04'!
seeClassSide
	"All the code for AppRegistry is on the class side."! !

AppRegistry subclass: #MailSender
	instanceVariableNames: ''
	classVariableNames: 'SmtpServer UserName'
	poolDictionaries: ''
	category: 'System-Applications'!

!MailSender class methodsFor: 'testing' stamp: 'dvf 5/11/2002 01:31'!
isSmtpServerSet
	^ SmtpServer notNil and: [SmtpServer notEmpty]
! !

!MailSender class methodsFor: 'communication' stamp: 'ads 5/11/2003 21:11'!
sendMessage: aMailMessage

	self default ifNotNil: [self default sendMailMessage: aMailMessage]! !

!MailSender class methodsFor: 'settings' stamp: 'rbb 3/1/2005 10:59'!
setSmtpServer
	"Set the SMTP server used to send outgoing messages via"
	SmtpServer ifNil: [SmtpServer := ''].
	SmtpServer := UIManager default
		request: 'What is your mail server for outgoing mail?'
		initialAnswer: SmtpServer.
! !

!MailSender class methodsFor: 'settings' stamp: 'md 9/29/2010 17:54'!
setSmtpServer: aString
	SmtpServer := aString! !

!MailSender class methodsFor: 'settings' stamp: 'rbb 3/1/2005 11:00'!
setUserName
	"Change the user's email name for use in composing messages."

	(UserName isNil) ifTrue: [UserName := ''].
	UserName := UIManager default
		request: 'What is your email address?\(This is the address other people will reply to you)' withCRs
		initialAnswer: UserName.
	UserName ifNotNil: [UserName := UserName]! !

!MailSender class methodsFor: 'accessing' stamp: 'dvf 5/11/2002 01:29'!
smtpServer
	"Answer the server for sending email"

	self isSmtpServerSet
		ifFalse: [self setSmtpServer].
	SmtpServer isEmpty ifTrue: [
		self error: 'no SMTP server specified' ].

	^SmtpServer! !

!MailSender class methodsFor: 'accessing' stamp: 'dvf 5/11/2002 00:49'!
userName
	"Answer the user name to be used in composing messages."

	(UserName isNil or: [UserName isEmpty])
		ifTrue: [self setUserName].

	UserName isEmpty ifTrue: [ self error: 'no user name specified' ].

	^UserName! !

AppRegistry subclass: #PrettyPrinting
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Applications'!

!PrettyPrinting class methodsFor: 'class initialization' stamp: 'damiencassou 7/30/2009 10:55'!
initialize
	"I'm a registry as well as a default provider"
	self register: self! !

!PrettyPrinting class methodsFor: 'pretty printing' stamp: 'damiencassou 7/30/2009 10:59'!
prettyPrinterClassFor: aBehavior
	|defaultPrinter|
	defaultPrinter := self default.
	^ (defaultPrinter isNil or: [defaultPrinter = self])
		ifTrue: [aBehavior compilerClass]
		ifFalse: [self default prettyPrinterClassFor: aBehavior]! !

AppRegistry subclass: #SoundService
	instanceVariableNames: ''
	classVariableNames: 'DefaultSoundEnabled'
	poolDictionaries: ''
	category: 'System-Applications'!

!SoundService commentStamp: 'AlainPlantec 1/7/2010 22:41' prior: 0!
This is the AppRegistry class for the sound system.

A sound system offers a small protocol for playing sounds and making beeps and works like a facade towards the rest of Squeak. A sound system is registered in this registry and can be accessed by "SoundService default". This way we decouple the sound system from the rest of Squeak and make it pluggable. It also is a perfect spot to check for the SoundSettings class>>soundEnabled.!

!SoundService class methodsFor: 'settings' stamp: 'StephaneDucasse 1/30/2011 19:09'!
defaultSoundEnabled
	^DefaultSoundEnabled ifNil: [false]! !

!SoundService class methodsFor: 'settings' stamp: 'GuillermoPolito 3/30/2012 11:38'!
soundEnabled
	^ self defaultSoundEnabled! !

!SoundService class methodsFor: 'settings' stamp: 'StephaneDucasse 1/30/2011 19:10'!
soundEnabled: aBoolean
	DefaultSoundEnabled := aBoolean! !

!SoundService class methodsFor: 'settings' stamp: 'HenrikSperreJohansen 1/21/2011 14:22'!
toggleSoundEnabling
     self soundEnabled: self soundEnabled not! !

Object subclass: #Author
	instanceVariableNames: 'fullName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

Author class
	instanceVariableNames: 'uniqueInstance'!

!Author commentStamp: 'MiguelCoba 7/25/2009 01:09' prior: 0!
I am responsible for the full name used to identify the current code author.!

Author class
	instanceVariableNames: 'uniqueInstance'!

!Author class methodsFor: 'utilities' stamp: ''!
changeStamp 
	"Answer a string to be pasted into source code to mark who changed it and when."
	^ Author fullName , ' ' , Date today mmddyyyy, ' ',
		((String streamContents: [:s | Time now print24: true on: s]) copyFrom: 1 to: 5)! !

!Author class methodsFor: 'utilities' stamp: ''!
changeStampPerSe
	"Answer a string to be pasted into source code to mark who changed it and when."

	^ (Author fullNamePerSe ifNil: ['.']) , ' ' , Date today mmddyyyy, ' ',
		((String streamContents: [:s | Time now print24: true on: s]) 
			copyFrom: 1 to: 5)! !

!Author class methodsFor: 'utilities' stamp: ''!
fixStamp: changeStamp 
	| parts |
	parts := changeStamp findTokens: ' '.
	(parts size > 0 and: [parts last first isLetter]) ifTrue:
		["Put initials first in all time stamps..."
		^ String streamContents:
				[:s | s nextPutAll: parts last.
				parts allButLast do: [:p | s space; nextPutAll: p]]].
	^ changeStamp! !

!Author class methodsFor: 'convenience' stamp: ''!
fullName
	^ Author uniqueInstance fullName! !

!Author class methodsFor: 'convenience' stamp: ''!
fullName: aString 
	^ Author uniqueInstance checkAndSetFullNameWith: aString! !

!Author class methodsFor: 'convenience' stamp: ''!
fullNamePerSe
	^ Author uniqueInstance fullNamePerSe! !

!Author class methodsFor: 'instance creation' stamp: ''!
new
	self error: 'Author is a singleton -- send uniqueInstance instead'! !

!Author class methodsFor: 'convenience' stamp: ''!
requestFullName
	^ Author uniqueInstance requestFullName! !

!Author class methodsFor: 'instance creation' stamp: ''!
reset
	^ uniqueInstance := nil! !

!Author class methodsFor: 'instance creation' stamp: ''!
uniqueInstance
	^ uniqueInstance ifNil: [ uniqueInstance := super new ]! !

!Author class methodsFor: 'instance creation' stamp: ''!
uniqueInstance: anInstance
	"Needed by AuthorTest to restore saved instance"
	^ uniqueInstance := anInstance! !

!Author class methodsFor: 'testing-support' stamp: ''!
useAuthor: aString during: aBlock
	^ self uniqueInstance useAuthor: aString during: aBlock! !

!Author methodsFor: 'accessing' stamp: 'Lr 12/9/2010 17:34'!
checkAndSetFullNameWith: aString 
	| s allowed |
	aString ifNil: [^self].
	s := ''.
	allowed := ($A to: $Z)
				, ($a to: $z)
				, ($0 to: $9).
	aString
		do: [:c | (allowed includes: c)
				ifTrue: [s := s , c asString]].
	self fullName: s! !

!Author methodsFor: 'accessing' stamp: ''!
fullName
	"Answer the full name to be used to identify the current code author."

	[fullName isEmptyOrNil] whileTrue: [
		self requestFullName.
		(fullName isNil or:[fullName = 'FirstnameLastname' translated]) ifTrue: [fullName := nil. ^'Anonymous']].
	^ fullName! !

!Author methodsFor: 'accessing' stamp: ''!
fullName: aString
	fullName := aString.! !

!Author methodsFor: 'accessing' stamp: ''!
fullNamePerSe

	"Answer the currently-prevailing author full name, such as it is, empty or not"

	^ fullName! !

!Author methodsFor: 'compatibility' stamp: ''!
ifUnknownAuthorUse: aString during: aBlock
	"If no author is set use the name aString while executing aBlock."
	"for compatibility with 1.0"
	
	| previous |
	fullName isEmptyOrNil ifFalse: [ ^ aBlock value ].
	fullName := aString.
	^ aBlock ensure: [ self reset ]! !

!Author methodsFor: 'initialization' stamp: ''!
initialize
	super initialize.
	fullName := ''.! !

!Author methodsFor: 'ui-requests' stamp: ''!
messagePrompt
	^
'Please type your full name. 
It will be used to sign the changes you make to the image.
Spaces, accents, dashes, underscore and similar characters are not allowed
'! !

!Author methodsFor: 'ui-requests' stamp: ''!
requestFullName
	| initialAnswer |
	initialAnswer := fullName isEmptyOrNil
				ifTrue: ['FirstnameLastname' translated]
				ifFalse: [fullName].
	self checkAndSetFullNameWith: ( UIManager default
				request: self messagePrompt
				initialAnswer: initialAnswer
				title: 'Author identification' translated)! !

!Author methodsFor: 'compatibility' stamp: ''!
reset
	fullName := ''! !

!Author methodsFor: 'testing-support' stamp: ''!
useAuthor: aString during: aBlock
	| previous |
	previous := fullName.
	fullName := aString.
	^ aBlock ensure: [ fullName := previous ]! !

Object subclass: #Beeper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

Beeper class
	instanceVariableNames: 'default'!

!Beeper commentStamp: 'gk 2/26/2004 22:44' prior: 0!
Beeper provides simple audio (or in some other way) feedback to the user.

The recommended use is "Beeper beep" to give the user the equivalence of a beep. If you want to force the beep to use the primitive in the VM for beeping, then use "Beeper beepPrimitive". In either case, if sounds are disabled there will be no beep.

The actual beeping, when you use "Beeper beep", is done by sending a #play message to a registered playable object. You can register your own playable object by invoking the class side method #setDefault: passing in an object that responds to the #play message.

The default playable object is an instance of Beeper itself which implements #play on the instance side. That implementation delegates the playing of the beep to the default SoundService.

Note that #play is introduced as a common interface between AbstractSound and Beeper.
This way we can register instances of AbstractSound as playable entities, for example:

	Beeper setDefault: (SampledSound new
						setSamples: self coffeeCupClink
						samplingRate: 12000).

Then "Beeper beep" will play the coffeeCup sound.!

Beeper class
	instanceVariableNames: 'default'!

!Beeper class methodsFor: 'beeping' stamp: ''!
beep
	"The preferred way of producing an audible feedback.
	The default playable entity (an instance of Beeper)
	also uses the pluggable SoundService
	mechanism, so it will use the primitive beep only
	if there is no other sound mechanism available."
	
	self default 
		ifNil: [self beepPrimitive]
		ifNotNil: [ self default play].
! !

!Beeper class methodsFor: 'beeping' stamp: 'HenrikSperreJohansen 1/21/2011 14:24'!
beepPrimitive
	"Make a primitive beep. Only use this if
	you want to force this to be a primitive beep.
	Otherwise use Beeper class>>beep
	since this method bypasses the current
	registered playable entity."

	SoundService soundEnabled 
		ifTrue: [self primitiveBeep]! !

!Beeper class methodsFor: 'customize' stamp: ''!
clearDefault
	"Clear the default playable.
	Will be lazily initialized in Beeper class >>default."

	default := nil! !

!Beeper class methodsFor: 'customize' stamp: ''!
default
	"When the default is not defined it is
	initialized using #newDefault."

	default isNil 
		ifTrue: [default := self newDefault ].
	^ default! !

!Beeper class methodsFor: 'customize' stamp: ''!
newDefault
	"Subclasses may override me to provide a default beep.
	This base implementation returns an instance of Beeper
	which uses the pluggable sound service."

	^ self new! !

!Beeper class methodsFor: 'private' stamp: ''!
primitiveBeep
	"Make a primitive beep. Not to be called directly.
	It is much better to use Beeper class>>beep
	or Beeper class>>beepPrimitive
	since this method bypasses the current
	registered playable entity and does not
	check SoundSettings class>>soundEnabled."

	<primitive: 140>
	self primitiveFailed! !

!Beeper class methodsFor: 'customize' stamp: ''!
setDefault: aPlayableEntity
	"Set the playable entity used when making a beep.
	The playable entity should implement the message #play."

	default := aPlayableEntity! !

!Beeper methodsFor: 'play interface' stamp: 'StephaneDucasse 2/12/2011 16:30'!
play
	"This is how the default Beeper makes a beep,
	by sending beep to the default sound service.
	The sound system will check if sounds are enabled."

	SoundService default new beep! !

Object subclass: #ChangeRecord
	instanceVariableNames: 'file position type class category meta stamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Changes'!

!ChangeRecord commentStamp: '<historical>' prior: 0!
A ChangeRecord represents a change recorded on a file in fileOut format.
It includes a type (more needs to be done here), and additional information
for certain types such as method defs which need class and category.!

!ChangeRecord methodsFor: 'access' stamp: ''!
category
	^category! !

!ChangeRecord methodsFor: 'testing' stamp: 'StephaneDucasse 7/23/2010 21:59'!
classIncludesSelector
	
	| aClass | 
	^ (aClass := self methodClass) notNil
			and: [aClass includesSelector: self methodSelector]! !

!ChangeRecord methodsFor: 'access' stamp: 'HenrikSperreJohansen 9/12/2010 10:02'!
commentClass 
	| commentClass |
	type == #classComment ifFalse: [^ nil].
	(Smalltalk includesKey: class asSymbol) ifFalse: [^ nil].
	commentClass := Smalltalk at: class asSymbol.
	^meta ifTrue: [commentClass class]
		ifFalse: [commentClass]
! !

!ChangeRecord methodsFor: 'initialization' stamp: ''!
file: f position: p type: t
	file := f.
	position := p.
	type := t! !

!ChangeRecord methodsFor: 'initialization' stamp: '6/6/97 08:48 dhhi'!
file: f position: p type: t class: c category: cat meta: m stamp: s
	self file: f position: p type: t.
	class := c.
	category := cat.
	meta := m.
	stamp := s! !

!ChangeRecord methodsFor: 'initialization' stamp: 'jb 7/1/2011 10:40'!
fileIn
	"File the receiver in.  If I represent a method or a class-comment, file the method in and make a note of it in the recent-submissions list; if I represent a do-it, then, well, do it."

	Cursor read
		showWhile: [ 
			| methodClass aSelector s |
			(methodClass := self methodClass) notNil
				ifTrue: [ 
					methodClass
						compile: self text
						classified: category
						withStamp: stamp
						notifying: nil.
					(aSelector := self methodSelector)
						ifNotNil: [ RecentMessageList uniqueInstance 
									addMethodReference: ((methodClass >> aSelector) methodReference )] ].
			type == #doIt
				ifTrue: [ 
					((s := self string) beginsWith: '----')
						ifFalse: [ self class evaluatorClass evaluate: s ] ].
			"This expression break the recentMessageSet because it hijacks compiled methods to represent classComment. In the future we should redo a complete changeset metamodel based on classDefinition, classComment and other. 
			type == #classComment
				ifTrue: [ 
					| cls |
					(cls := Smalltalk globals at: class asSymbol) comment: self text stamp: stamp.
					RecentMessageSet noteMethodSubmission: #Comment forClass: cls ]" ]! !

!ChangeRecord methodsFor: 'access' stamp: 'sumim 9/1/2003 18:27'!
fileIndex
	^ (SourceFiles collect: [ :sf | sf name]) 
		indexOf: file name ifAbsent: [^ nil].
! !

!ChangeRecord methodsFor: 'access' stamp: 'nk 1/7/2004 10:28'!
fileName
	^(file ifNotNil: [ file name ]) 
			ifNil: [ '<no file>' ]! !

!ChangeRecord methodsFor: 'access' stamp: 'sw 10/20/2002 02:53'!
fileOutOn: aFileStream
	"File the receiver out on the given file stream"

	| aString |
	type == #method
		ifTrue:
			[aFileStream nextPut: $!!.
			aString :=  class asString
							, (meta ifTrue: [' class methodsFor: ']
									ifFalse: [' methodsFor: '])
							, category asString printString.
			stamp ifNotNil:
				[aString := aString, ' stamp: ''', stamp, ''''].
			aFileStream nextChunkPut: aString.
			aFileStream cr].

	type == #preamble ifTrue: [aFileStream nextPut: $!!].

	type == #classComment
		ifTrue:
			[aFileStream nextPut: $!!.
			aFileStream nextChunkPut: class asString, ' commentStamp: ', stamp storeString.
			aFileStream cr].

	aFileStream nextChunkPut: self string.
	type == #method ifTrue: [aFileStream nextChunkPut: ' '].
	aFileStream cr! !

!ChangeRecord methodsFor: 'access' stamp: ''!
isMetaClassChange
	^meta! !

!ChangeRecord methodsFor: 'testing' stamp: 'StephaneDucasse 1/7/2011 17:22'!
isMethodDefinedInImage
	"answer whether the method represented by the receiver is present in the image"
	
	^ self type = #method
		and: [self classIncludesSelector]! !

!ChangeRecord methodsFor: 'testing' stamp: 'StephaneDucasse 7/23/2010 22:40'!
isMethodNotDefinedInImage
	"answer whether the method represented by the receiver is not present in the image.
	pay attention is it not just isMethodDefinedInImage not"
	
	| aClass |
	^ self type = #method and:
			[(aClass := self methodClass) isNil or:
				[(aClass includesSelector: self methodSelector) not]]! !

!ChangeRecord methodsFor: 'testing' stamp: 'StephaneDucasse 7/23/2010 22:52'!
isUnchangedMethod

	^ self isMethodDefinedInImage 
		and: [self string withBlanksCondensed 
				= (self methodClass sourceCodeAt: self methodSelector) asString withBlanksCondensed ]! !

!ChangeRecord methodsFor: 'access' stamp: 'StephaneDucasse 7/23/2010 21:45'!
methodClass
	| methodClass |
	type == #method
		ifFalse: [ ^ nil ].
	(Smalltalk globals includesKey: class asSymbol)
		ifFalse: [ ^ nil ].
	methodClass := Smalltalk globals at: class asSymbol.
	^ meta
		ifTrue: [ methodClass class ]
		ifFalse: [ methodClass ]! !

!ChangeRecord methodsFor: 'access' stamp: ''!
methodClassName
	^class! !

!ChangeRecord methodsFor: 'access' stamp: 'lr 3/14/2010 21:13'!
methodSelector
	^ type == #method
		ifTrue: [ (Smalltalk globals at: class ifAbsent: [ Object ]) parserClass new parseSelector: self string ]! !

!ChangeRecord methodsFor: 'access' stamp: 'sumim 9/2/2003 14:07'!
position
	^ position! !

!ChangeRecord methodsFor: 'access' stamp: 'sumim 9/2/2003 13:33'!
prior
	| currFile preamble prevPos tokens prevFileIndex |
	currFile := file readOnlyCopy.
	currFile position: (0 max: position - 150).
	[currFile position < (position - 1)] whileTrue: [preamble := currFile nextChunk].
	currFile close.
	prevPos := nil.
	(preamble findString: 'methodsFor:' startingAt: 1) > 0
		ifTrue: [tokens := Scanner new scanTokens: preamble]
		ifFalse: [tokens := Array new].
	((tokens size between: 7 and: 8)
	and: [(tokens at: tokens size - 5) == #methodsFor:]) ifTrue: [
		(tokens at: tokens size - 3) == #stamp:
		ifTrue: [
			prevPos := tokens last.
			prevFileIndex := SourceFiles fileIndexFromSourcePointer: prevPos.
			prevPos := SourceFiles filePositionFromSourcePointer: prevPos]
		ifFalse: [
			prevPos := tokens at: tokens size - 2.
			prevFileIndex := tokens last].
		(prevPos = 0 or: [prevFileIndex = 0]) ifTrue: [prevPos := nil]].
	prevPos ifNil: [^ nil].
	^ {prevFileIndex. prevPos. 
		SourceFiles sourcePointerFromFileIndex: prevFileIndex andPosition: prevPos}! !

!ChangeRecord methodsFor: 'access' stamp: '6/6/97 08:56 dhhi'!
stamp
	^ stamp! !

!ChangeRecord methodsFor: 'access' stamp: 'tk 9/7/2000 15:09'!
stamp: threePartString

	stamp := threePartString! !

!ChangeRecord methodsFor: 'access' stamp: 'di 1/13/98 16:57'!
string 
	| string |
	file openReadOnly.
	file position: position.
	string := file nextChunk.
	file close.
	^ string! !

!ChangeRecord methodsFor: 'access' stamp: 'tk 6/23/1999 08:20'!
text
	| text |
	^ file ifNil: ['']
		ifNotNil: [
			file openReadOnly.
			file position: position.
			text := file nextChunkText.
			file close.
			text]! !

!ChangeRecord methodsFor: 'access' stamp: 'nice 1/5/2010 15:59'!
timeStamp
	"Answer a TimeStamp that corresponds to my (text) stamp"
	| tokens |
	tokens := self stamp findTokens: Character separators.
	^ tokens size > 2
		ifTrue: [[ | time date |date := Date
						fromString: (tokens at: tokens size - 1).
			time := Time fromString: tokens last.
			TimeStamp date: date time: time]
				on: Error
				do: [:ex | ex
						return: (TimeStamp fromSeconds: 0)]]
		ifFalse: [TimeStamp fromSeconds: 0]! !

!ChangeRecord methodsFor: 'access' stamp: ''!
type
	^ type! !

Object subclass: #ChangeSet
	instanceVariableNames: 'name preamble postscript changeRecords structures superclasses'
	classVariableNames: 'AllChangeSets DefaultChangeSetDirectoryName MustCheckForSlips PreviousSet'
	poolDictionaries: ''
	category: 'System-Changes'!

ChangeSet class
	instanceVariableNames: 'current'!

!ChangeSet commentStamp: 'VeronicaUquillas 6/11/2010 12:36' prior: 0!
ChangeSets keep track of the changes made to a system, so they can be written on a file as source code (a "fileOut"). 

This implementation of ChangeSet is capable of remembering and manipulating methods for which the classes are not present in the system.  However at the present time, this capability is not used in normal rearranging and fileOuts, but only for invoking and revoking associated with isolation layers.

For isolated projects (see Project class comment), the changeSet binding is semi-permanent.  Every project exists in an isolation layer defined by its closest enclosing parent (or itself) that is isolated.  If a project is not isolated, then changes reported to its designated changeSet must also be reported to the permanent changeSet for that layer, designated in the isolated project.  This ensures that that outer project will be able to revert all changes upon exit.

Note that only certain changes may be reverted.  Classes may not be added, removed, renamed or reshaped except in the layer in which they are defined because these operations on non-local classes are not revertable.

If a Squeak Project is established as being isolated, then its associated changeSet will be declared to be revertable.  In this case all changes stored can be reverted.  The changeSet associated with an isolated project is tied to that project, and cannot be edited in a changeSorter.
------

name - a String used to name the changeSet, and thus any associated project or fileOut.

preamble and postscript:  two strings that serve as prefix (useful for documentation) and suffix (useful for doits) to the fileout of the changeSet.

revertable - a Boolean
If this variable is true, then all of the changes recorded by this changeSet can be reverted.

changeRecords -  Dictionary {class name -> a ClassChangeRecord}.
These classChangeRecords (qv) remember all of the system changes.

structures -    Dictionary {#Rectangle -> #(<classVersionInteger> 'origin' 'corner')}.
Of  the names of the instances variables before any changes for all classes in classChanges, and all of their superclasses.  In the same format used in SmartRefStream.  Inst var names are strings.  

superclasses -    Dictionary {#Rectangle -> #Object}.
Of all classes in classChanges, and all of their superclasses.

Structures and superclasses save the instance variable names of this class and all of its superclasses.  Later we can tell how it changed and write a conversion method.  The conversion method is used when old format objects are brought in from the disk from ImageSegment files (.extSeg) or SmartRefStream files (.obj .morph .bo .sp).

NOTE:  It should be fairly simple, by adding a bit more information to the classChangeRecords, to reconstruct the information now stored in 'structures' and 'superclasses'.  This would be a welcome simplification.
!

ChangeSet class
	instanceVariableNames: 'current'!

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:20'!
allChangeSetNames
	^ self allChangeSets collect: [:c | c name]! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:10'!
allChangeSets
	"Return the list of all current ChangeSets"

	^ AllChangeSets! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:17'!
allChangeSets: aCollection
	"Return the list of all current ChangeSets"

	AllChangeSets := aCollection.! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:22'!
allChangeSetsWithClass: class selector: selector
	class ifNil: [^ #()].
	^ self allChangeSets select: 
		[:cs | (cs atSelector: selector class: class) ~~ #none]! !

!ChangeSet class methodsFor: 'services' stamp: 'StephaneDucasse 3/23/2010 21:27'!
assuredChangeSetNamed: aName
	"Answer a change set of the given name.  If one already exists, answer that, else create a new one and answer it."

	| existing |
	^ (existing := self named: aName)
		ifNotNil: [existing]
		ifNil: [self basicNewChangeSet: aName]! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:14'!
basicNewChangeSet: newName
	| newSet |
	newName ifNil: [^ nil].
	(self named: newName) ifNotNil:
		[self inform: 'Sorry that name is already used'.
		^ nil].
	newSet := self basicNewNamed: newName.
	AllChangeSets add: newSet.
	^ newSet! !

!ChangeSet class methodsFor: 'instance creation' stamp: 'di 4/6/2001 09:43'!
basicNewNamed: aName

	^ (self basicNew name: aName) initialize! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:11'!
changeSetsNamedSuchThat: nameBlock
	"(ChangeSet changeSetsNamedSuchThat:
		[:name | name first isDigit and: [name initialInteger >= 373]])
		do: [:cs | AllChangeSets remove: cs wither]"

	^ AllChangeSets select: [:aChangeSet | nameBlock value: aChangeSet name]! !

!ChangeSet class methodsFor: 'cleanup' stamp: 'MarcusDenker 2/22/2012 12:54'!
cleanUp: aggressive
	"Only delete change sets when being aggressive"

	aggressive ifTrue: [ 
			ChangeSet removeChangeSetsNamedSuchThat: [ :each | true ].
			ChangeSet resetCurrentToNewUnnamedChangeSet.].! !

!ChangeSet class methodsFor: 'services' stamp: 'ar 7/15/2005 21:37'!
countOfChangeSetsWithClass: aClass andSelector: aSelector
	"Answer how many change sets record a change for the given class and selector"

	^ (self allChangeSetsWithClass: aClass selector: aSelector) size! !

!ChangeSet class methodsFor: 'current changeset' stamp: 'IgorStasenko 3/6/2011 18:16'!
current
	"return the current changeset assure first that we have a named changeset."
	
	current isMoribund
		ifTrue: [(self newChanges: (self assuredChangeSetNamed: 'Unnamed'))] .
	^ current! !

!ChangeSet class methodsFor: 'defaults' stamp: 'CamilloBruni 5/7/2012 01:08'!
defaultChangeSetDirectory
	"Answer the directory in which to store ChangeSets. 
	Answer the default directory if the preferred directory doesn't exist."
	| dir directoryName |
	directoryName := self defaultChangeSetDirectoryName.
	dir := directoryName isEmptyOrNil
		ifTrue: [ FileSystem workingDirectory ]
		ifFalse: [ FileSystem workingDirectory / directoryName ].
	dir exists
		ifTrue: [^ dir].
	^ FileSystem workingDirectory! !

!ChangeSet class methodsFor: 'defaults' stamp: 'StephaneDucasse 6/11/2012 17:53'!
defaultChangeSetDirectory: dirOrName 
	"Set the Preference for storing change sets to the given directory or name (possibly relative).
	Rewrite directory names below the default directory as relative names.
	If dirOrName is an empty string, use the default directory."

	"ChangeSet defaultChangeSetDirectory: 'changeSets'"

	| dirName defaultFullName |
	
	dirName := dirOrName isString
				ifTrue: [ dirOrName asFileReference fullName ]
				ifFalse: [dirOrName fullName].
				
	defaultFullName := FileSystem workingDirectory fullName.
	dirName = defaultFullName
		ifTrue: [dirName := '']
		ifFalse: [
			(dirName beginsWith: defaultFullName , FileSystem disk delimiter asString)
				ifTrue: [dirName := dirName copyFrom: defaultFullName size + 2 to: dirName size]].
	self defaultChangeSetDirectoryName: dirName! !

!ChangeSet class methodsFor: 'settings' stamp: 'StephaneDucasse 6/11/2012 18:04'!
defaultChangeSetDirectoryName
	^ DefaultChangeSetDirectoryName ifNil: [DefaultChangeSetDirectoryName := '.']! !

!ChangeSet class methodsFor: 'settings' stamp: 'AlainPlantec 12/18/2009 14:14'!
defaultChangeSetDirectoryName: aName
	DefaultChangeSetDirectoryName  := aName! !

!ChangeSet class methodsFor: 'defaults' stamp: 'dgd 9/6/2003 19:56'!
defaultName
	^ self uniqueNameLike: 'Unnamed' translated! !

!ChangeSet class methodsFor: 'services' stamp: 'ar 7/15/2005 21:38'!
doesAnyChangeSetHaveClass: aClass andSelector: aSelector
	"Answer whether any known change set bears a change for the given class and selector"

	^ (self countOfChangeSetsWithClass: aClass andSelector: aSelector) > 0! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'StephaneDucasse 3/29/2010 16:06'!
existingOrNewChangeSetNamed: aName

	| newSet |
	^(self named: aName) ifNil: [
		newSet := self basicNewNamed: aName.
		AllChangeSets add: newSet.
		newSet]! !

!ChangeSet class methodsFor: 'services' stamp: 'CamilloBruni 5/7/2012 02:10'!
fileIntoNewChangeSet: fullName
	"File in all of the contents of the currently selected file, if any, into a new change set." 

	| fn ff |
	fullName ifNil: [^ Beeper beep].
	
	fn := (Smalltalk hasClassNamed: #GZipReadStream) 
		ifTrue: [(Smalltalk classNamed:  #GZipReadStream) uncompressedFileName: fullName]
		ifFalse: [fullName].
	[ ff := FileStream readOnlyFileNamed: fn.
	self newChangesFromStream: ff named: (fn  asFileReference basename)]
		ensure: [ff ifNotNil: [ff close]]! !

!ChangeSet class methodsFor: 'filein/out' stamp: 'StephaneDucasse 3/31/2010 16:57'!
fileOutChangeSetsNamed: nameList
	"File out the list of change sets whose names are provided"
     "ChangesOrganizer fileOutChangeSetsNamed: #('New Changes' 'miscTidies-sw')"

	| notFound empty infoString |
	notFound := OrderedCollection new.
	empty := OrderedCollection new.
	nameList do:
		[:aName | | aChangeSet | 
			(aChangeSet := self named: aName)
				ifNotNil: [aChangeSet isEmpty
							ifTrue: [empty add: aName]
							ifFalse: [aChangeSet fileOut]]
				ifNil: [notFound add: aName]].

	infoString := (nameList size - notFound size) printString, ' change set(s) filed out'.
	notFound size > 0 ifTrue:
		[infoString := infoString, '

', notFound size printString, ' change set(s) not found:'.
		notFound do:
			[:aName | infoString := infoString, '
', aName]].
	empty size > 0 ifTrue:
		[infoString := infoString, '
', empty size printString, ' change set(s) were empty:'.
		empty do:
			[:aName | infoString := infoString, '
', aName]].

	self inform: infoString! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'nice 1/5/2010 15:59'!
gatherChangeSets		"ChangeSet gatherChangeSets"
	"Collect any change sets created in other projects"
	| allChangeSets |
	allChangeSets := AllChangeSets asSet.
	ChangeSet allSubInstances do: [:each | | obsolete |
		(allChangeSets includes: each) == (obsolete := each isMoribund) ifTrue:[
			obsolete
				ifTrue: ["Was included and is obsolete."
						AllChangeSets remove: each]
				ifFalse: ["Was not included and is not obsolete."
						AllChangeSets add: each]]].
	^ AllChangeSets! !

!ChangeSet class methodsFor: 'initialization' stamp: 'MarcusDenker 8/25/2010 21:02'!
initialize
	"ChangeSet initialize"

	AllChangeSets isNil
		ifTrue: [ AllChangeSets := OrderedCollection new ].
	self gatherChangeSets.
	FileServices registerFileReader: self! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:29'!
mostRecentChangeSetWithChangeForClass: class selector: selector
	| hits |
	hits := self allChangeSets select: 
		[:cs | (cs atSelector: selector class: class) ~~ #none].
	hits isEmpty ifTrue: [^ 'not in any change set'].
	^ 'recent cs: ', hits last name! !

!ChangeSet class methodsFor: 'settings' stamp: 'AlainPlantec 12/7/2009 10:58'!
mustCheckForSlips
	^ MustCheckForSlips ifNil: [MustCheckForSlips := true]! !

!ChangeSet class methodsFor: 'settings' stamp: 'AlainPlantec 12/7/2009 10:58'!
mustCheckForSlips: aBoolean
	MustCheckForSlips := aBoolean! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:11'!
named: aName
	"Return the change set of the given name, or nil if none found.  1/22/96 sw"

	^ AllChangeSets
			detect: [:aChangeSet | aChangeSet name = aName]
			ifNone: [nil]! !

!ChangeSet class methodsFor: 'instance creation' stamp: 'ar 7/16/2005 15:17'!
new
	"All current changeSets must be registered in the AllChangeSets collection.
	Due to a quirk of history, this is maintained as class variable of ChangeSorter."

	^ self basicNewChangeSet: ChangeSet defaultName! !

!ChangeSet class methodsFor: 'services' stamp: 'ar 7/15/2005 21:31'!
newChangeSet
	"Prompt the user for a name, and establish a new change set of
	that name (if ok), making it the current changeset.  Return nil
	of not ok, else return the actual changeset."

	| newName newSet |
	newName := UIManager default
		request: 'Please name the new change set:'
		initialAnswer: ChangeSet defaultName.
	newName isEmptyOrNil ifTrue:
		[^ nil].
	newSet := self basicNewChangeSet: newName.
	newSet ifNotNil:
		[self  newChanges: newSet].
	^ newSet! !

!ChangeSet class methodsFor: 'services' stamp: 'CamilloBruni 5/4/2012 21:29'!
newChangeSet: aName
	"Makes a new change set called aName, add author full name to try to
	ensure a unique change set name."

	| newName |
	newName := aName , '.' , Author fullName.
	^ self basicNewChangeSet: newName! !

!ChangeSet class methodsFor: 'current changeset' stamp: 'GuillermoPolito 6/16/2012 17:27'!
newChanges: aChangeSet 
	"Set the system ChangeSet to be the argument, aChangeSet.  Tell the current project that aChangeSet is now its change set.  When called from Project enter:, the setChangeSet: call is redundant but harmless; when called from code that changes the current-change-set from within a project, it's vital"

	SystemAnnouncer current unsubscribe: current.
	current := aChangeSet.
	
	SystemAnnouncer current weak
		on: ClassRemoved send: #classRemoved: to: aChangeSet;
		on: ClassAdded send: #classAdded: to: aChangeSet;
		on: ClassCommented send: #classCommented: to: aChangeSet;
		on: ClassRenamed send: #classRenamed: to: aChangeSet;
		on: ClassReorganized send: #classReorganized: to: aChangeSet;
		on: ClassRecategorized send: #classRecategorized: to: aChangeSet;
		on: ClassModifiedClassDefinition send: #classModified: to: aChangeSet.
	
	SystemAnnouncer current weak
		on: MethodAdded send: #methodAdded: to: aChangeSet;
		on: MethodModified send: #methodModified: to: aChangeSet;
		on: MethodRemoved send: #methodRemoved: to: aChangeSet;
		on: MethodRecategorized  send: #methodRecategorized: to: aChangeSet.! !

!ChangeSet class methodsFor: 'services' stamp: 'GuillermoPolito 5/5/2012 23:11'!
newChangesFromStream: aStream named: aName
	"File in the code from the stream into a new change set whose
	name is derived from aName. Leave the 'current change set'
	unchanged. Return the new change set or nil on failure."

	| oldChanges newName newSet |
	oldChanges := ChangeSet current.
	PreviousSet := oldChanges name. 		"so a Bumper update can find it"
	newName := aName withoutPeriodSuffix.
	newSet := self basicNewChangeSet: newName.
	[ | newStream |newSet ifNotNil:[
		(aStream respondsTo: #converter:) ifFalse: [
			newStream := MultiByteBinaryOrTextStream with: (aStream contentsOfEntireFile).
			newStream reset.] ifTrue: [newStream := aStream].
		self newChanges: newSet.
		newStream setConverterForCode.
		CodeImporter evaluateReadStream: newStream readStream.
		"Transcript cr; show: 'File ', aName, ' successfully filed in to change set ', newName"].
	aStream close] ensure: [self newChanges: oldChanges].
	PreviousSet := nil.
	^ newSet! !

!ChangeSet class methodsFor: 'current changeset' stamp: 'sd 5/22/2003 22:18'!
noChanges 
	"Initialize the system ChangeSet."

	current initialize! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:13'!
promoteToTop: aChangeSet
	"Make aChangeSet the first in the list from now on"

	AllChangeSets remove: aChangeSet ifAbsent: [^ self].
	AllChangeSets add: aChangeSet! !

!ChangeSet class methodsFor: 'defaults' stamp: 'StephaneDucasse 6/11/2012 18:04'!
promptForDefaultChangeSetDirectoryIfNecessary
	"Check the Preference (if any), and prompt the
	user to change it if necessary.
	The default if the Preference is unset is the current
	directory. Answer the directory."
	"ChangeSet
	promptForDefaultChangeSetDirectoryIfNecessary"
	| choice directoryPath dir message |
	directoryPath := self defaultChangeSetDirectoryName.
	[dir := directoryPath asFileReference.
	dir exists] 
		whileFalse: [message := 'The preferred change set directory' translated , ' (''{1}'') ' , 'does not exist.' translated , '
' , 'Create it or use the default directory' translated , ' ({2})?' format: {directoryPath. FileSystem workingDirectory fullName}.
			choice := UIManager default
						chooseFrom: (#('Create directory' 'Use default directory and forget preference' 'Choose another directory' )
								collect: [:ea | ea translated])
						message: message.
			choice = 1
				ifTrue: [dir ensureDirectory ].
			choice = 3
				ifTrue: [dir := UIManager default chooseDirectory.
					directoryPath := dir
								ifNil: ['']
								ifNotNil: [dir pathName]]].
	self defaultChangeSetDirectory: directoryPath.
	^ dir! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:13'!
removeChangeSet: aChangeSet
	"Remove the given changeSet.  Caller must assure that it's cool to do this"

	AllChangeSets remove: aChangeSet ifAbsent: [].
	aChangeSet wither
! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:26'!
removeChangeSetsNamedSuchThat: nameBlock
	(self changeSetsNamedSuchThat: nameBlock)
		do: [:cs | self removeChangeSet: cs]! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'StephaneDucasse 3/23/2010 22:28'!
removeEmptyUnnamedChangeSets
	"Remove all change sets that are empty, whose names start with Unnamed."
	"ChangeSet removeEmptyUnnamedChangeSets"
	
	| toGo |
	(toGo := (self changeSetsNamedSuchThat: [:csName | csName beginsWith: 'Unnamed'])
		select: [:cs | cs isEmpty and: [cs okayToRemoveInforming: false]])
		do: [:cs | self removeChangeSet: cs].
	self inform: toGo size printString, ' change set(s) removed.'! !

!ChangeSet class methodsFor: 'services' stamp: 'StephaneDucasse 3/30/2010 23:03'!
reorderChangeSets
	"Change the order of the change sets to something more convenient:
		First come all numbered updates.
		Next come all remaining changesets"

	"self reorderChangeSets"
	
	| newMid newTail |
	newMid := OrderedCollection new.
	newTail := OrderedCollection new.
	self allChangeSets do:
		[:aChangeSet |
			aChangeSet isNumbered
						ifTrue: [newMid add: aChangeSet]
						ifFalse: [newTail add: aChangeSet]].
	self  allChangeSets: newMid, newTail.
	! !

!ChangeSet class methodsFor: 'current changeset' stamp: 'sd 9/8/2006 21:05'!
resetCurrentToNewUnnamedChangeSet
 
	current := self new.  
	self newChanges: current ! !

!ChangeSet class methodsFor: 'scanning' stamp: 'ar 7/16/2005 15:12'!
scanCategory: file
	"Scan anything that involves more than one chunk; method name is historical only"
	| itemPosition item tokens stamp isComment anIndex |
	itemPosition := file position.
	item := file nextChunk.

	isComment := (item includesSubString: 'commentStamp:').
	(isComment or: [item includesSubString: 'methodsFor:']) ifFalse:
		["Maybe a preamble, but not one we recognize; bail out with the preamble trick"
		^{(ChangeRecord new file: file position: itemPosition type: #preamble)}].

	tokens := Scanner new scanTokens: item.
	tokens size >= 3 ifTrue:
		[stamp := ''.
		anIndex := tokens indexOf: #stamp: ifAbsent: [nil].
		anIndex ifNotNil: [stamp := tokens at: (anIndex + 1)].

		tokens second == #methodsFor:
			ifTrue: [^ self scanFile: file category: tokens third class: tokens first
							meta: false stamp: stamp].
		tokens third == #methodsFor:
			ifTrue: [^ self scanFile: file category: tokens fourth class: tokens first
							meta: true stamp: stamp]].

		tokens second == #commentStamp:
			ifTrue:
				[stamp := tokens third.
				item := (ChangeRecord new file: file position: file position type: #classComment
										class: tokens first category: nil meta: false stamp: stamp).
				file nextChunk.
				file skipStyleChunk.
				^Array with: item].
	^#()! !

!ChangeSet class methodsFor: 'scanning' stamp: 'ar 7/16/2005 15:11'!
scanFile: file category: cat class: class meta: meta stamp: stamp
	| itemPosition method items |
	items := OrderedCollection new.
	[itemPosition := file position.
	method := file nextChunk.
	file skipStyleChunk.
	method size > 0] whileTrue:[
		items add: (ChangeRecord new file: file position: itemPosition type: #method
							class: class category: cat meta: meta stamp: stamp)].
	^items! !

!ChangeSet class methodsFor: 'scanning' stamp: 'SeanDeNigris 6/21/2012 08:45'!
scanFile: file from: startPosition to: stopPosition
	| changeList |
	changeList := OrderedCollection new.
	file position: startPosition.
'Scanning ', file localName, '...'
	displayProgressFrom: startPosition to: stopPosition
	during: [:bar | | prevChar itemPosition item |
	[file position < stopPosition] whileTrue:[
		bar current: file position.
		[file atEnd not and: [file peek isSeparator]]
			whileTrue: [prevChar := file next].
		(file peekFor: $!!) ifTrue:[
			(prevChar = Character cr or: [prevChar = Character lf])
				ifTrue: [changeList addAll: (self scanCategory: file)].
		] ifFalse:[
			itemPosition := file position.
			item := file nextChunk.
			file skipStyleChunk.
			item size > 0 ifTrue:[
				changeList add: (ChangeRecord new file: file position: itemPosition type: #doIt).
			].
		].
	]].
	^changeList! !

!ChangeSet class methodsFor: 'scanning' stamp: 'StephaneDucasse 5/28/2011 13:32'!
scanVersionsOf: method class: class meta: meta category: cat selector: selector
	| position prevPos prevFileIndex preamble tokens sourceFilesCopy stamp changeList file |
	changeList := OrderedCollection new.
	position := method filePosition.
	sourceFilesCopy := SourceFiles collect:[:x | x ifNotNil:[x readOnlyCopy]].
	method fileIndex = 0 ifTrue: [^ nil].
	file := sourceFilesCopy at: method fileIndex.
	[position notNil & file notNil] whileTrue:[
		file position: (0 max: position-150).  "Skip back to before the preamble"
		preamble := method getPreambleFrom: file at: (0 max: position - 3).
		"Preamble is likely a linked method preamble, if we're in
			a changes file (not the sources file).  Try to parse it
			for prior source position and file index"
		prevPos := nil.
		stamp := ''.
		(preamble findString: 'methodsFor:' startingAt: 1) > 0
			ifTrue: [tokens := Scanner new scanTokens: preamble]
			ifFalse: [tokens := Array new  "ie cant be back ref"].
		((tokens size between: 7 and: 8)
			and: [(tokens at: tokens size-5) = #methodsFor:]) ifTrue:[
				(tokens at: tokens size-3) = #stamp: ifTrue:[
					"New format gives change stamp and unified prior pointer"
					stamp := tokens at: tokens size-2.
					prevPos := tokens last.
					prevFileIndex := sourceFilesCopy fileIndexFromSourcePointer: prevPos.
					prevPos := sourceFilesCopy filePositionFromSourcePointer: prevPos.
				] ifFalse: ["Old format gives no stamp; prior pointer in two parts"
					prevPos := tokens at: tokens size-2.
					prevFileIndex := tokens last.
				].
				(prevPos = 0 or: [prevFileIndex = 0]) ifTrue: [prevPos := nil]
			].
		((tokens size between: 5 and: 6)
			and: [(tokens at: tokens size-3) = #methodsFor:]) ifTrue:[
				(tokens at: tokens size-1) = #stamp: ifTrue: [
					"New format gives change stamp and unified prior pointer"
					stamp := tokens at: tokens size.
			]
		].
 		changeList add: (ChangeRecord new file: file position: position type: #method
						class: class name category: cat meta: meta stamp: stamp).
		position := prevPos.
		prevPos notNil ifTrue:[file := sourceFilesCopy at: prevFileIndex].
	].
	sourceFilesCopy do: [:x | x ifNotNil:[x close]].
	^changeList! !

!ChangeSet class methodsFor: 'enumerating' stamp: 'ar 7/15/2005 21:13'!
secondaryChangeSet
	"Answer a likely change set to use as the second initial one in a Dual Change Sorter.  "

	AllChangeSets size = 1 ifTrue: [^ AllChangeSets first].
	AllChangeSets last == ChangeSet current
		ifTrue: 	[^ AllChangeSets at: (AllChangeSets size - 1)]
		ifFalse:	[^ AllChangeSets last]! !

!ChangeSet class methodsFor: 'filein/out' stamp: 'SteveFreeman 7/17/2010 12:06'!
superclassOrder: classes
	^ Class superclassOrder: classes! !

!ChangeSet class methodsFor: 'filein/out' stamp: 'al 7/18/2004 18:45'!
traitsOrder: aCollection 
	"Answer an OrderedCollection. The traits 
	are ordered so they can be filed in."

	|  traits |
	traits := aCollection asSortedCollection: [:t1 :t2 |
		(t1 isBaseTrait and: [t1 classTrait == t2]) or: [
			(t2 traitComposition allTraits includes: t1) or: [
				(t1 traitComposition allTraits includes: t2) not]]].
	^traits asArray! !

!ChangeSet class methodsFor: 'defaults' stamp: 'lr 3/12/2010 22:16'!
uniqueNameLike: aString 
	| try index |
	(self named: aString)
		ifNil: [ ^ aString ].
	index := 1.
	[ try := aString , index printString.
	  (self named: try)
		ifNil: [ ^ try ].
	  index := index + 1 ] repeat
! !

!ChangeSet methodsFor: 'moving changes' stamp: 'lr 3/14/2010 21:13'!
absorbClass: className from: otherChangeSet
	"Absorb into the receiver all the changes found in the class in the other change set.
	*** Classes renamed in otherChangeSet may have problems"

	| cls |
	(self changeRecorderFor: className) assimilateAllChangesIn: (otherChangeSet changeRecorderFor: className).
	(cls := Smalltalk globals classNamed: className) ifNotNil: [ self absorbStructureOfClass: cls from: otherChangeSet ]! !

!ChangeSet methodsFor: 'moving changes' stamp: 'di 3/23/2000 11:52'!
absorbMethod: selector class: aClass from: aChangeSet
	"Absorb into the receiver all the changes for the method in the class in the other change set."

	| info |
	info := aChangeSet methodChanges at: aClass name ifAbsent: [Dictionary new].
	self atSelector: selector class: aClass put: (info at: selector).

! !

!ChangeSet methodsFor: 'moving changes' stamp: 'sw 1/30/2001 15:41'!
absorbStructureOfClass: aClass from: otherChangeSet
	"Absorb into the receiver all the structure and superclass info in the other change set.  Used to write conversion methods."

	| sup next |
	otherChangeSet structures ifNil: [^ self].
	(otherChangeSet structures includesKey: aClass name) ifFalse: [^ self].
	structures ifNil:
		[structures := Dictionary new.
		superclasses := Dictionary new].
	sup := aClass name.
	[(structures includesKey: sup) 
		ifTrue: ["use what is here" true]
		ifFalse: [self flag: #noteToDan.  "sw 1/30/2001 13:57 emergency workaround -- a case arose where the otherChangeSet's structures did not have the key, and it gummed up the works."
				(otherChangeSet structures includesKey: sup) ifTrue:
					[structures at: sup put: (otherChangeSet structures at: sup)].
				next := otherChangeSet superclasses at: sup.
				superclasses at: sup put: next.
				(sup := next) = 'nil']
	] whileFalse.


! !

!ChangeSet methodsFor: 'change logging' stamp: 'VeronicaUquillas 6/11/2010 11:36'!
addClass: class 
	"Include indication that a new class was created."

	class wantsChangeSetLogging ifFalse: [^ self].
	self atClass: class add: #new.
	self atClass: class add: #change.
	self addCoherency: class name! !

!ChangeSet methodsFor: 'private' stamp: 'di 3/23/2000 08:37'!
addCoherency: className
	"SqR!! 19980923: If I recreate the class then don't remove it"

	(self changeRecorderFor: className)
		checkCoherence.
"
	classRemoves remove: className ifAbsent: [].
	(classChanges includesKey: className) ifTrue:
		[(classChanges at: className) remove: #remove ifAbsent: []]
"! !

!ChangeSet methodsFor: 'method changes' stamp: 'sw 12/28/2000 18:08'!
adoptSelector: aSelector forClass: aClass
	"Adopt the given selector/class combination as a change in the receiver"

	self noteNewMethod: (aClass methodDictionary at: aSelector)
			forClass: aClass selector: aSelector priorMethod: nil! !

!ChangeSet methodsFor: 'moving changes' stamp: 'di 4/4/2000 11:21'!
assimilateAllChangesFoundIn: otherChangeSet
	"Make all changes in otherChangeSet take effect on self as if they happened just now."

	otherChangeSet changedClassNames do:
		[:className | self absorbClass: className from: otherChangeSet]
! !

!ChangeSet methodsFor: 'filein/out' stamp: 'MarcusDenker 8/25/2010 21:02'!
assurePostscriptExists
	"Make sure there is a StringHolder holding the postscript.  "

	"NOTE: FileIn recognizes the postscript by the line with Postscript: on it"

	postscript isNil
		ifTrue: [ 
			postscript := '"Postscript:
Leave the line above, and replace the rest of this comment by a useful one.
Executable statements should follow this comment, and should
be separated by periods, with no exclamation points (!!).
Be sure to put any further comments in double-quotes, like this one."
' ]! !

!ChangeSet methodsFor: 'filein/out' stamp: 'ar 7/16/2005 18:02'!
assurePreambleExists
	"Make sure there is a StringHolder holding the preamble; if it's found to have reverted to empty contents, put up the template"

	(preamble isEmptyOrNil)
		ifTrue: [preamble := self preambleTemplate]! !

!ChangeSet methodsFor: 'private' stamp: 'di 3/28/2000 14:40'!
atClass: class add: changeType

	(self changeRecorderFor: class)
		noteChangeType: changeType fromClass: class! !

!ChangeSet methodsFor: 'private' stamp: 'di 4/1/2000 12:00'!
atClass: class includes: changeType

	^(changeRecords at: class name ifAbsent: [^false])
		includesChangeType: changeType! !

!ChangeSet methodsFor: 'private' stamp: 'di 4/1/2000 12:00'!
atSelector: selector class: class

	^ (changeRecords at: class name ifAbsent: [^ #none])
		atSelector: selector ifAbsent: [^ #none]! !

!ChangeSet methodsFor: 'method changes' stamp: 'Alexandre Bergel 11/21/2009 10:36'!
atSelector: selector class: class put: changeType

	(self changeRecorderFor: class) atSelector: selector put: changeType.
! !

!ChangeSet methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/8/2012 14:42'!
author
	| author |
	self assurePreambleExists.
	author := self preambleString lineNumber: 3.
	author := author copyFrom: 8 to: author size. "Strip the 'Author:' prefix. Ugly ugly."	
	^ author trimBoth! !

!ChangeSet methodsFor: 'change logging' stamp: 'VeronicaUquillas 6/11/2010 11:37'!
changeClass: class from: oldClass
	"Remember that a class definition has been changed.  Record the original structure, so that a conversion method can be built."

	class wantsChangeSetLogging ifFalse: [^ self]. 
	class isMeta 
		ifFalse: [self atClass: class add: #change]	"normal"
		ifTrue: [((self classChangeAt: class theNonMetaClass name) includes: #add) 
			ifTrue: [self atClass: class add: #add] 	"When a class is defined, the metaclass
				is not recorded, even though it was added.  A further change is
				really just part of the original add."
			ifFalse: [self atClass: class add: #change]].
	self addCoherency: class name.
	(self changeRecorderFor: class) notePriorDefinition: oldClass.
	self noteClassStructure: oldClass! !

!ChangeSet methodsFor: 'private' stamp: 'MarcusDenker 8/25/2010 20:55'!
changeRecorderFor: class

	| cname |
	(class isString)
		ifTrue: [ cname := class ]
		ifFalse: [ cname := class name ].

	"Later this will init the changeRecords so according to whether they should be revertable."
	^ changeRecords at: cname
			ifAbsent: [^ changeRecords at: cname
							put: (ClassChangeRecord new initFor: cname revertable: false)]! !

!ChangeSet methodsFor: 'class changes' stamp: 'di 4/1/2000 12:00'!
changedClassNames
	"Answer a OrderedCollection of the names of changed or edited classes.
	DOES include removed classes.  Sort alphabetically."

	^ changeRecords keysSortedSafely ! !

!ChangeSet methodsFor: 'class changes' stamp: 'lr 3/14/2010 21:13'!
changedClasses
	"Answer an OrderedCollection of changed or edited classes.
	Does not include removed classes.  Sort alphabetically by name."

	"Much faster to sort names first, then convert back to classes.  Because metaclasses reconstruct their name at every comparison in the sorted collection.
	8/91 sw chgd to filter out non-existent classes (triggered by problems with class-renames"

	^ self changedClassNames
		collect: [ :className | Smalltalk globals classNamed: className ]
		thenSelect: [ :aClass | aClass notNil ]! !

!ChangeSet methodsFor: 'method changes' stamp: 'sd 9/22/2011 17:55'!
changedMessageList
	"Used by a message set browser to access the list view information."

	| messageList |
	messageList := OrderedCollection new.
	changeRecords associationsDo: [:clAssoc | | className classIsMeta |
		className := clAssoc key asSymbol.
		classIsMeta := (className findTokens: ' ') size > 1.

		(clAssoc value allChangeTypes includes: #comment) ifTrue:
			[messageList add:
				(RGCommentDefinition new
					parentName: className) asActive ].

		clAssoc value methodChangeTypes associationsDo: [:mAssoc |
			(#(remove addedThenRemoved) includes: mAssoc value) ifFalse:
				[messageList add:
					((RGMethodDefinition named: mAssoc key)
						parentName: className;
						isMetaSide: classIsMeta) asActive ]]].
	^ messageList asArray sort! !

!ChangeSet methodsFor: 'filein/out' stamp: 'MiguelCoba 7/25/2009 02:16'!
checkForAlienAuthorship
	"Check to see if there are any methods in the receiver that have author full name other than that of the current author, and open a browser on all found"

	| aList fullName |
	(fullName := Author fullNamePerSe) ifNil: [^ self inform: 'No author full name set in this image'].
	(aList := self methodsWithInitialsOtherThan: fullName) size > 0
		ifFalse:
			[^ self inform: 'All methods in "', self name, '"
have authoring stamps which start with "', fullName, '"']
		ifTrue:
			[self systemNavigation  browseMessageList: aList name: 'methods in "', self name, '" whose authoring stamps do not start with "', fullName, '"']! !

!ChangeSet methodsFor: 'filein/out' stamp: 'MiguelCoba 7/25/2009 02:17'!
checkForAnyAlienAuthorship
	"Check to see if there are any versions of any methods in the receiver that have author full name other than that of the current author, and open a browser on all found"

	| aList fullName |
	(fullName := Author fullNamePerSe) ifNil: [^ self inform: 'No author full name set in this image'].
	(aList := self methodsWithAnyInitialsOtherThan: fullName) size > 0
		ifFalse: [^ self inform: 'All versions of all methods in "', self name, '"
have authoring stamps which start with "', fullName, '"']
		ifTrue:
			[self systemNavigation  browseMessageList: aList name: 'methods in "', self name, '" with any authoring stamps not starting with "', fullName, '"']! !

!ChangeSet methodsFor: 'filein/out' stamp: 'StephaneDucasse 5/23/2011 22:45'!
checkForSlips
	"Return a collection of method refs with possible debugging code in them."
	| slips |
	slips := OrderedCollection new.
	self changedClasses do:
		[:aClass |
		(self methodChangesAtClass: aClass name) associationsDo: 
				[:mAssoc | | method | (#(remove addedThenRemoved) includes: mAssoc value) ifFalse:
					[method := aClass compiledMethodAt: mAssoc key ifAbsent: [nil].
					method ifNotNil:
						[method hasReportableSlip
							ifTrue: [ slips add: (aClass >> mAssoc key) methodReference ]]]]].
	^ slips! !

!ChangeSet methodsFor: 'filein/out' stamp: 'sd 4/16/2003 09:16'!
checkForUnclassifiedMethods
	"Open a message list browser on all methods in the current change set that have not been categorized,"

	| aList |
	(aList := self methodsWithoutClassifications) size > 0
		ifFalse:
			[^ self inform: 'All methods in "', self name, '"
are categorized.']
		ifTrue:
			[self systemNavigation  browseMessageList: aList name: 'methods in "', self name, '" which have not been categorized']! !

!ChangeSet methodsFor: 'filein/out' stamp: 'sd 4/16/2003 09:16'!
checkForUncommentedMethods
	| aList |
	"Check to see if there are any methods in the receiver that have no comments, and open a browser on all found"

	(aList := self methodsWithoutComments) size > 0
		ifFalse:
			[^ self inform: 'All methods in "', self name, '" have comments']
		ifTrue:
			[self systemNavigation  browseMessageList: aList name: 'methods in "', self name, '" that lack comments']! !

!ChangeSet methodsFor: 'filein/out' stamp: 'StephaneDucasse 8/1/2011 17:15'!
checkForUnsentMessages
	"Check the change set for unsent messages, and if any are found, open 
	up a message-list browser on them"
	| nameLine allChangedSelectors augList unsent |
	nameLine := '"' , self name , '"'.
	allChangedSelectors := Set new.
	(augList := self changedMessageList)
		do: [:each | each isValid
				ifTrue: [allChangedSelectors add: each selector]].
	unsent := self systemNavigation allUnsentMessagesIn: allChangedSelectors.
	unsent size = 0
		ifTrue: [^ self inform: 'There are no unsent 
messages in change set
' , nameLine].
	self systemNavigation
		browseMessageList: (augList
				select: [:each | unsent includes: each selector])
		name: 'Unsent messages in ' , nameLine! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 6/16/2012 17:20'!
classAdded: anEvent
	self addClass: anEvent item! !

!ChangeSet methodsFor: 'class changes' stamp: 'di 4/1/2000 12:00'!
classChangeAt: className
	"Return what we know about class changes to this class."

	^ (changeRecords at: className ifAbsent: [^ Set new])
		allChangeTypes! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 6/16/2012 17:21'!
classCommented: anEvent
	self commentClass: anEvent item! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 7/20/2012 17:18'!
classModified: anEvent
	self flag: 'We should get rid of this please. normally if you send a changed event, is because something changed, so this is nonsense'.
	anEvent anyChanges ifTrue: [
		self changeClass: anEvent item from: anEvent oldItem
	]! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 7/23/2012 10:15'!
classRecategorized: anEvent
	self changeClass: anEvent item from: anEvent item! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 6/16/2012 17:19'!
classRemoved: anEvent
	self noteRemovalOf: anEvent item.! !

!ChangeSet methodsFor: 'accessing' stamp: 'di 4/1/2000 12:00'!
classRemoves

	^ changeRecords keys select:
		[:className | (changeRecords at: className) isClassRemoval]! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 6/16/2012 17:21'!
classRenamed: anEvent
	self renameClass: anEvent item from: anEvent oldName to: anEvent newName! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 6/16/2012 17:21'!
classReorganized: anEvent
	self reorganizeClass: anEvent item! !

!ChangeSet methodsFor: 'initialization' stamp: 'di 4/1/2000 12:00'!
clear 
	"Reset the receiver to be empty.  "

	changeRecords := Dictionary new.
	preamble := nil.
	postscript := nil! !

!ChangeSet methodsFor: 'class changes' stamp: 'NS 1/26/2004 09:46'!
commentClass: class 
	"Include indication that a class comment has been changed."

	class wantsChangeSetLogging ifFalse: [^ self].
	self atClass: class add: #comment! !

!ChangeSet methodsFor: 'class changes' stamp: 'nk 6/26/2002 12:30'!
containsClass: aClass
	^ self changedClasses includes: aClass! !

!ChangeSet methodsFor: 'filein/out' stamp: 'nk 10/15/2003 09:55'!
defaultChangeSetDirectory
	^self class defaultChangeSetDirectory! !

!ChangeSet methodsFor: 'accessing' stamp: 'ar 7/16/2005 18:59'!
editPostscript
	"edit the receiver's postscript, in a separate window.  "
	self assurePostscriptExists.
	UIManager default
		edit: self postscript 
		label: 'Postscript for ChangeSet named ', name
		accept:[:aString| self postscript: aString].! !

!ChangeSet methodsFor: 'moving changes' stamp: 'ar 7/16/2005 18:59'!
editPreamble
	"edit the receiver's preamble, in a separate window.  "
	self assurePreambleExists.
	UIManager default
		edit: self preamble 
		label: 'Preamble for ChangeSet named ', name
		accept:[:aString| self preamble: aString]! !

!ChangeSet methodsFor: 'change logging' stamp: 'CamilleTeruel 7/29/2012 18:33'!
event: anEvent
	"Hook for SystemChangeNotifier"
	self halt.
	anEvent itemKind = #class ifTrue: [
		anEvent isRemoved 
			ifTrue: [self noteRemovalOf: anEvent item].
		anEvent isAdded 
			ifTrue: [self addClass: anEvent item].
		anEvent isModified 
			ifTrue: [anEvent anyChanges ifTrue: [self changeClass: anEvent item from: anEvent oldItem]].
		anEvent isCommented 
			ifTrue: [self commentClass: anEvent item].
		anEvent isRenamed 
			ifTrue: [self renameClass: anEvent item from: anEvent oldName to: anEvent newName].
		anEvent isReorganized
			ifTrue: [self reorganizeClass: anEvent item].
		anEvent isRecategorized
			ifTrue: [self changeClass: anEvent item from: anEvent item].
	].

	anEvent itemKind = #method ifTrue: [
		anEvent isAdded
			ifTrue: [self noteNewMethod: anEvent item forClass: anEvent itemClass selector: anEvent itemSelector priorMethod: nil].
		anEvent isModified
			ifTrue: [self noteNewMethod: anEvent item forClass: anEvent itemClass selector: anEvent itemSelector priorMethod: anEvent oldItem].
		anEvent isRemoved
			ifTrue: [self removeSelector: anEvent itemSelector class: anEvent itemClass priorMethod: anEvent item lastMethodInfo: {anEvent item sourcePointer. anEvent itemProtocol}].
		anEvent isRecategorized
			ifTrue: [self reorganizeClass: anEvent itemClass].
	].! !

!ChangeSet methodsFor: 'moving changes' stamp: 'di 4/4/2000 11:49'!
expungeEmptyClassChangeEntries

	changeRecords keysAndValuesRemove:
		[:className :classRecord | classRecord hasNoChanges]! !

!ChangeSet methodsFor: 'class changes' stamp: 'PeterHugossonMiller 9/3/2009 00:18'!
fatDefForClass: class 
	| newDef oldDef oldStrm newStrm outStrm oldVars newVars addedVars |
	class isBehavior ifFalse: [ ^ class definition ].
	newDef := class definition.
	oldDef := (self changeRecorderFor: class) priorDefinition.
	oldDef ifNil: [ ^ newDef ].
	oldDef = newDef ifTrue: [ ^ newDef ].
	oldStrm := oldDef readStream.
	newStrm := newDef readStream.
	outStrm := (String new: newDef size * 2) writeStream.

	"Merge inst vars from old and new defs..."
	oldStrm
		upToAll: 'instanceVariableNames';
		upTo: $'.
	outStrm
		nextPutAll: (newStrm upToAll: 'instanceVariableNames');
		nextPutAll: 'instanceVariableNames:'.
	newStrm peek = $: ifTrue: [ newStrm next ].	"may or may not be there, but already written"
	outStrm
		nextPutAll: (newStrm upTo: $');
		nextPut: $'.
	oldVars := (oldStrm upTo: $') findTokens: Character separators.
	newVars := (newStrm upTo: $') findTokens: Character separators.
	addedVars := oldVars asSet
		addAll: newVars;
		removeAll: oldVars;
		asOrderedCollection.
	oldVars , addedVars do: 
		[ :var | 
		outStrm
			nextPutAll: var;
			space ].
	outStrm nextPut: $'.
	class isMeta ifFalse: 
		[ "Merge class vars from old and new defs..."
		oldStrm
			upToAll: 'classVariableNames:';
			upTo: $'.
		outStrm
			nextPutAll: (newStrm upToAll: 'classVariableNames:');
			nextPutAll: 'classVariableNames:';
			nextPutAll: (newStrm upTo: $');
			nextPut: $'.
		oldVars := (oldStrm upTo: $') findTokens: Character separators.
		newVars := (newStrm upTo: $') findTokens: Character separators.
		addedVars := oldVars asSet
			addAll: newVars;
			removeAll: oldVars;
			asOrderedCollection.
		oldVars , addedVars do: 
			[ :var | 
			outStrm
				nextPutAll: var;
				space ].
		outStrm nextPut: $' ].
	outStrm nextPutAll: newStrm upToEnd.
	^ outStrm contents! !

!ChangeSet methodsFor: 'filein/out' stamp: 'StephaneDucasse 6/11/2012 18:09'!
fileOut
	"File out the receiver, to a file whose name is a function of the  
	change-set name and a unique numeric tag."
	| slips nameToUse |
	ChangeSet promptForDefaultChangeSetDirectoryIfNecessary.
	nameToUse := (self defaultChangeSetDirectory / self name , 'cs') nextVersion basename.
	Cursor write showWhile: [ | internalStream |
			internalStream := (String new: 10000) writeStream.
			internalStream header; timeStamp.
			self fileOutPreambleOn: internalStream.
			self fileOutOn: internalStream.
			self fileOutPostscriptOn: internalStream.
			internalStream trailer.

			FileStream writeSourceCodeFrom: internalStream baseName: (nameToUse copyFrom: 1 to: nameToUse size - 3) isSt: false.
	].
	self class  mustCheckForSlips
		ifFalse: [^ self].
	slips := self checkForSlips.
	(slips size > 0
			and: [self confirm: 'Methods in this fileOut have halts
or references to the Transcript
or other ''slips'' in them.
Would you like to browse them?' translated])
		ifTrue: [self systemNavigation browseMessageList: slips name: 'Possible slips in ' , name]! !

!ChangeSet methodsFor: 'filein/out' stamp: 'di 3/28/2000 09:35'!
fileOutChangesFor: class on: stream 
	"Write out all the method changes for this class."

	| changes |
	changes := Set new.
	(self methodChangesAtClass: class name) associationsDo: 
		[:mAssoc | (mAssoc value = #remove or: [mAssoc value = #addedThenRemoved])
			ifFalse: [changes add: mAssoc key]].
	changes isEmpty ifFalse: 
		[class fileOutChangedMessages: changes on: stream.
		stream cr]! !

!ChangeSet methodsFor: 'private' stamp: 'al 7/22/2008 21:36'!
fileOutClassDefinition: class on: stream 
	"Write out class definition for the given class on the given stream, if the class definition was added or changed."

	(self atClass: class includes: #rename) ifTrue:
		[stream nextChunkPut: 'Smalltalk renameClassNamed: #', (self oldNameFor: class), ' as: #', class name; cr].

	(self atClass: class includes: #change) ifTrue: [ "fat definition only needed for changes"
		stream nextChunkPut: (self fatDefForClass: class); cr.
		DeepCopier new checkClass: class.	"If veryDeepCopy weakly copies some inst 
			vars in this class, warn author when new ones are added." 
	] ifFalse: [
		(self atClass: class includes: #add) ifTrue: [ "use current definition for add"
			stream nextChunkPut: class definition; cr.
			DeepCopier new checkClass: class.	"If veryDeepCopy weakly copies some inst 
				vars in this class, warn author when new ones are added." 
		].
	].

	(self atClass: class includes: #comment) ifTrue:
		[class theNonMetaClass organization putCommentOnFile: stream numbered: 0 moveSource: false forClass: class theNonMetaClass.
		stream cr].

! !

!ChangeSet methodsFor: 'filein/out' stamp: 'dvf 9/27/2005 19:04'!
fileOutOn: stream 
	"Write out all the changes the receiver knows about"

	| classList traits classes traitList list |
	(self isEmpty and: [stream isKindOf: FileStream])
		ifTrue: [self inform: 'Warning: no changes to file out'].
		
	traits := self changedClasses reject: [:each | each isBehavior].
	classes := self changedClasses select: [:each | each isBehavior].
	traitList := self class traitsOrder: traits asOrderedCollection.
	classList := self class superclassOrder: classes asOrderedCollection.
	list := OrderedCollection new
		addAll: traitList;
		addAll: classList;
		yourself.
	
	"First put out rename, max classDef and comment changes."
	list do: [:aClass | self fileOutClassDefinition: aClass on: stream].

	"Then put out all the method changes"
	list do: [:aClass | self fileOutChangesFor: aClass on: stream].

	"Finally put out removals, final class defs and reorganization if any"
	list reverseDo: [:aClass | self fileOutPSFor: aClass on: stream].

	self classRemoves asSortedCollection do:
		[:aClassName | stream nextChunkPut: 'Smalltalk removeClassNamed: #', aClassName; cr].! !

!ChangeSet methodsFor: 'filein/out' stamp: 'nice 1/5/2010 15:59'!
fileOutPSFor: class on: stream 
	"Write out removals and initialization for this class."

	| dict classRecord currentDef |
	classRecord := changeRecords at: class name ifAbsent: [^ self].
	dict := classRecord methodChangeTypes.
	dict keysSortedSafely do:
		[:key | | changeType | changeType := dict at: key.
		(#(remove addedThenRemoved) includes: changeType)
			ifTrue: [stream nextChunkPut: class name,
						' removeSelector: ', key storeString; cr]
			ifFalse: [(key = #initialize and: [class isMeta]) ifTrue:
						[stream nextChunkPut: class soleInstance name, ' initialize'; cr]]].
	((classRecord includesChangeType: #change)
		and: [(currentDef := class definition) ~= (self fatDefForClass: class)]) ifTrue:
		[stream nextChunkPut: currentDef; cr ].
	(classRecord includesChangeType: #reorganize) ifTrue:
		[class fileOutOrganizationOn: stream.
		stream cr]! !

!ChangeSet methodsFor: 'filein/out' stamp: 'MarcusDenker 8/25/2010 21:02'!
fileOutPostscriptOn: stream
	"If the receiver has a postscript, put it out onto the stream.  "

	| aString |
	aString := self postscriptString.
	(aString notNil and: [ aString size > 0 ])
		ifTrue: [ 
			stream nextChunkPut: aString.	"surroundedBySingleQuotes"
			stream
				cr;
				cr ]! !

!ChangeSet methodsFor: 'filein/out' stamp: 'MarcusDenker 8/25/2010 21:02'!
fileOutPreambleOn: stream
	"If the receiver has a preamble, put it out onto the stream.  "

	| aString |
	aString := self preambleString.
	(aString notNil and: [ aString size > 0 ])
		ifTrue: [ 
			stream nextChunkPut: aString.	"surroundedBySingleQuotes"
			stream
				cr;
				cr ]! !

!ChangeSet methodsFor: 'moving changes' stamp: 'di 4/4/2000 12:40'!
forgetAllChangesFoundIn: otherChangeSet
	"Remove from the receiver all method changes found in aChangeSet. The intention is facilitate the process of factoring a large set of changes into disjoint change sets.  To use:  in a change sorter, copy over all the changes you want into some new change set, then use the subtract-other-side feature to subtract those changes from the larger change set, and continue in this manner."

	otherChangeSet == self ifTrue: [^ self].
	otherChangeSet changedClassNames do:
		[:className | self forgetChangesForClass: className in: otherChangeSet].
	self expungeEmptyClassChangeEntries.

"  Old code...
	aChangeSet changedClassNames do: 
		[:className |
			(cls := Smalltalk classNamed: className) ~~ nil ifTrue:
				[itsMethodChanges := aChangeSet methodChanges at: className 
						ifAbsent: [Dictionary new].
				itsMethodChanges associationsDo: [:assoc | 
					self forgetChange: assoc value forSelector: assoc key class: cls].
				myClassChange := self classChangeAt: className.
				myClassChange size > 0 ifTrue:
					[(aChangeSet classChangeAt: className) do:
						[:aChange | myClassChange remove: aChange ifAbsent: []]].
				self noteClassForgotten: className]].

	aChangeSet classRemoves do:
		[:className | (recorder := changeRecords at: className ifAbsent: [])
			ifNotNil: [recorder forgetClassRemoval]].
	self expungeEmptyClassChangeEntries
"
! !

!ChangeSet methodsFor: 'moving changes' stamp: 'di 4/4/2000 12:04'!
forgetChangesForClass: className in: otherChangeSet
	"See forgetAllChangesFoundIn:.  Used in culling changeSets."

	(self changeRecorderFor: className)
			forgetChangesIn: (otherChangeSet changeRecorderFor: className).
	self noteClassForgotten: className
! !

!ChangeSet methodsFor: 'method changes' stamp: 'sw 4/19/2001 19:45'!
hasAnyChangeForSelector: aSelector
	"Answer whether the receiver has any change under the given selector, whether it be add, change, or remove, for any class"

	changeRecords do:
		[:aRecord | (aRecord changedSelectors  includes: aSelector)
			ifTrue:	[^ true]].
	^ false! !

!ChangeSet methodsFor: 'accessing' stamp: 'sw 6/29/1999 14:44'!
hasPostscript
	^ postscript notNil! !

!ChangeSet methodsFor: 'moving changes' stamp: 'sw 3/5/1999 19:27'!
hasPreamble
	^ preamble notNil! !

!ChangeSet methodsFor: 'initialization' stamp: 'MarcusDenker 8/25/2010 20:55'!
initialize 
	"Initialize the receiver to be empty."

	super initialize.
	name ifNil: [^ self error: 'All changeSets must be registered, as in ChangeSorter newChangeSet'].
	self clear.
! !

!ChangeSet methodsFor: 'testing' stamp: 'RAA 10/19/2000 13:17'!
isEmpty
	"Answer whether the receiver contains any elements."
	
	changeRecords ifNil: [^true].
	^ changeRecords isEmpty ! !

!ChangeSet methodsFor: 'initialization' stamp: 'MarcusDenker 8/25/2010 21:02'!
isMoribund
	"Answer whether the receiver is obsolete and about to die; part of an effort to get such guys cleared out from the change sorter.  2/7/96 sw"

	^ name isNil! !

!ChangeSet methodsFor: 'testing' stamp: 'StephaneDucasse 3/30/2010 23:02'!
isNumbered
	"Answer whether a change set is numbered"

	^  self name startsWithDigit! !

!ChangeSet methodsFor: 'filein/out' stamp: 'StephaneDucasse 5/28/2011 13:32'!
lookForSlips
	"Scan the receiver for changes that the user may regard as slips to be remedied"

	| slips nameLine msg |
	nameLine := '
"', self name, '"
'.
	(slips := self checkForSlips) size = 0 ifTrue:
		[^ self inform: 'No slips detected in change set', nameLine].

	msg := slips size = 1
		ifTrue:
			[ 'One method in change set', nameLine, 
'has a halt, reference to the Transcript,
and/or some other ''slip'' in it.
Would you like to browse it? ?']
		ifFalse:
			[ slips size printString,
' methods in change set', nameLine, 'have halts or references to the
Transcript or other ''slips'' in them.
Would you like to browse them?'].

	(UIManager default  chooseFrom: #('Ignore' 'Browse slips') title: msg) = 2
		ifTrue: [self systemNavigation  browseMessageList: slips
							name: 'Possible slips in ', name]! !

!ChangeSet methodsFor: 'method changes' stamp: 'StephaneDucasse 8/1/2011 17:15'!
messageListForChangesWhich: aBlock ifNone: ifEmptyBlock

	| answer |
	answer := self changedMessageList select: [ :each |
		aBlock value: each actualClass value: each selector ].
	answer isEmpty ifTrue: [^ifEmptyBlock value].
	^answer
! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 6/16/2012 17:22'!
methodAdded: anEvent
	self noteNewMethod: anEvent item forClass: anEvent itemClass selector: anEvent itemSelector priorMethod: nil! !

!ChangeSet methodsFor: 'accessing' stamp: 'nice 1/5/2010 15:59'!
methodChanges

	| methodChangeDict |
	methodChangeDict := Dictionary new.
	changeRecords associationsDo:
		[:assn | | changeTypes |
		changeTypes := assn value methodChangeTypes.
		changeTypes isEmpty ifFalse: [methodChangeDict at: assn key put: changeTypes]].
	^ methodChangeDict! !

!ChangeSet methodsFor: 'method changes' stamp: 'di 4/1/2000 12:00'!
methodChangesAtClass: className
	"Return an old-style dictionary of method change types."

	^(changeRecords at: className ifAbsent: [^ Dictionary new])
		methodChangeTypes! !

!ChangeSet methodsFor: 'accessing' stamp: 'di 3/29/2000 16:22'!
methodInfoFromRemoval: classAndSelector

	^ (self changeRecorderFor: classAndSelector first)
		infoFromRemoval: classAndSelector last! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 6/16/2012 17:23'!
methodModified: anEvent
	self noteNewMethod: anEvent item forClass: anEvent itemClass selector: anEvent itemSelector priorMethod: anEvent oldItem! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 6/16/2012 17:23'!
methodRecategorized: anEvent
	self reorganizeClass: anEvent itemClass! !

!ChangeSet methodsFor: 'change logging' stamp: 'GuillermoPolito 6/16/2012 17:23'!
methodRemoved: anEvent
	self removeSelector: anEvent itemSelector class: anEvent itemClass priorMethod: anEvent item lastMethodInfo: {anEvent item sourcePointer. anEvent itemProtocol}! !

!ChangeSet methodsFor: 'moving changes' stamp: 'nice 1/5/2010 15:59'!
methodsWithAnyInitialsOtherThan: myInits
	"Return a collection of method refs whose author appears to be different from the given one, even historically"
	| slips |
	slips := Set new.
	self changedClasses do: [:aClass |
		(self methodChangesAtClass: aClass name) associationsDo: [ :mAssoc | | method |
			(#(remove addedThenRemoved) includes: mAssoc value) ifFalse:
				[method := aClass compiledMethodAt: mAssoc key ifAbsent: [nil].
				method ifNotNil: [
					(aClass changeRecordsAt: mAssoc key) do: [ :chg | | aTimeStamp |
						aTimeStamp := chg stamp.
						(aTimeStamp notNil and: [(aTimeStamp beginsWith: myInits) not])
							ifTrue: [slips add: aClass name , ' ' , mAssoc key]]]]]].
	^ slips! !

!ChangeSet methodsFor: 'moving changes' stamp: 'MarcusDenker 2/20/2010 03:15'!
methodsWithInitialsOtherThan: myInits
	"Return a collection of method refs whose author appears to be different from the given one"
	| slips |
	slips := OrderedCollection new.
	self changedClasses do:
		[:aClass |
		(self methodChangesAtClass: aClass name) associationsDo: 
				[:mAssoc | | method aTimeStamp | (#(remove addedThenRemoved) includes: mAssoc value) ifFalse:
					[method := aClass compiledMethodAt: mAssoc key ifAbsent: [nil].
					method ifNotNil:
						[((aTimeStamp :=  method timeStamp) notNil and:
							[(aTimeStamp beginsWith: myInits) not])
								ifTrue: [slips add: aClass name , ' ' , mAssoc key]]]]].
	^ slips

	"Smalltalk browseMessageList: (ChangeSet current methodsWithInitialsOtherThan: 'sw') name: 'authoring problems'"! !

!ChangeSet methodsFor: 'testing' stamp: 'nice 1/5/2010 15:59'!
methodsWithoutClassifications
	"Return a collection representing methods in the receiver which have not been categorized"

	| slips notClassified |

	notClassified := {'as yet unclassified' asSymbol. #all}.
	slips := OrderedCollection new.
	self changedClasses do:
		[:aClass |
		(self methodChangesAtClass: aClass name) associationsDo: 
				[:mAssoc | | aSelector | (aClass includesSelector:  (aSelector := mAssoc key)) ifTrue:
						[(notClassified includes: (aClass organization categoryOfElement: aSelector))
								ifTrue: [slips add: aClass name , ' ' , aSelector]]]].
	^ slips

	"Smalltalk browseMessageList: (ChangeSet current methodsWithoutClassifications) name: 'unclassified methods'"! !

!ChangeSet methodsFor: 'moving changes' stamp: 'nice 10/20/2009 20:55'!
methodsWithoutComments
	"Return a collection representing methods in the receiver which have no precode comments"

	| slips |
	slips := OrderedCollection new.
	self changedClasses do:
		[:aClass |
		(self methodChangesAtClass: aClass name) associationsDo: 
				[:mAssoc | (#(remove addedThenRemoved) includes: mAssoc value) ifFalse:
					[(aClass includesSelector:  mAssoc key) ifTrue:
						[(aClass firstPrecodeCommentFor: mAssoc key) isEmptyOrNil
								ifTrue: [slips add: aClass name , ' ' , mAssoc key]]]]].
	^ slips

	"Smalltalk browseMessageList: (ChangeSet current methodsWithoutComments) name: 'methods lacking comments'"! !

!ChangeSet methodsFor: 'accessing' stamp: 'MarcusDenker 8/25/2010 21:02'!
name
	"The name of this changeSet.
	 2/7/96 sw: If name is nil, we've got garbage.  Help to identify."

	^ name isNil
		ifTrue: [ '<no name -- garbage?>' ]
		ifFalse: [ name ]! !

!ChangeSet methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

!ChangeSet methodsFor: 'class changes' stamp: 'lr 3/14/2010 21:13'!
noteClassForgotten: className
	"Remove from structures if class is not a superclass of some other one we are remembering"

	structures ifNil: [ ^ self ].
	Smalltalk globals
		at: className
		ifPresent: [ :cls | 
			cls subclasses
				do: [ :sub | 
					(structures includesKey: sub)
						ifTrue: [ ^ self ] ] ].	"No delete"
	structures removeKey: className ifAbsent: [  ]! !

!ChangeSet methodsFor: 'class changes' stamp: 'dvf 9/27/2005 19:05'!
noteClassStructure: aClass
	"Save the instance variable names of this class and all of its superclasses.  Later we can tell how it changed and write a conversion method.  The conversion method is used when old format objects are brought in from the disk from ImageSegment files (.extSeg) or SmartRefStream files (.obj .morph .bo .sp)."

	| clsName |
	aClass isBehavior ifFalse: [^ self].
	
	structures ifNil: [structures := Dictionary new.
				superclasses := Dictionary new].
	clsName := (aClass name asLowercase beginsWith: 'anobsolete') 
		ifTrue: [(aClass name copyFrom: 11 to: aClass name size) asSymbol]
		ifFalse: [aClass name].
	(structures includesKey: clsName) ifFalse: [
		structures at: clsName put: 
			((Array with: aClass classVersion), (aClass allInstVarNames)).
		superclasses at: clsName put: aClass superclass name].
	"up the superclass chain"
	aClass superclass ifNotNil: [self noteClassStructure: aClass superclass].
! !

!ChangeSet methodsFor: 'change logging' stamp: 'VeronicaUquillas 6/11/2010 11:37'!
noteNewMethod: newMethod forClass: class selector: selector priorMethod: methodOrNil

	class wantsChangeSetLogging ifFalse: [^ self].
	(self changeRecorderFor: class)
		noteNewMethod: newMethod selector: selector priorMethod: methodOrNil
! !

!ChangeSet methodsFor: 'class changes' stamp: 'NS 1/19/2004 17:49'!
noteRemovalOf: class
	"The class is about to be removed from the system.
	Adjust the receiver to reflect that fact."

	class wantsChangeSetLogging ifFalse: [^ self].
	(self changeRecorderFor: class)
		noteChangeType: #remove fromClass: class.
	changeRecords removeKey: class class name ifAbsent: [].! !

!ChangeSet methodsFor: 'testing' stamp: 'sw 8/3/1998 16:25'!
okayToRemove
	^ self okayToRemoveInforming: true! !

!ChangeSet methodsFor: 'testing' stamp: 'stephane.ducasse 7/10/2009 16:44'!
okayToRemoveInforming: aBoolean
	"Answer whether it is okay to remove the receiver.  If aBoolean is true, inform the receiver if it is not okay"

	| aName |
	aName := self name.
	self == self class current ifTrue:
		[aBoolean ifTrue: [self inform: 'Cannot remove "', aName, '"
because it is the 
current change set.'].
		^ false].
	^ true
! !

!ChangeSet methodsFor: 'private' stamp: 'di 4/1/2000 12:00'!
oldNameFor: class

	^ (changeRecords at: class name) priorName! !

!ChangeSet methodsFor: 'filein/out' stamp: 'ar 7/16/2005 18:03'!
postscript
	"Answer the string representing the postscript.  "
	^postscript ifNotNil:[postscript isString ifTrue:[postscript] ifFalse:[postscript contents asString]]! !

!ChangeSet methodsFor: 'filein/out' stamp: 'ar 7/16/2005 18:03'!
postscript: aString
	"Answer the string representing the postscript.  "
	postscript := aString! !

!ChangeSet methodsFor: 'accessing' stamp: 'ar 7/16/2005 18:04'!
postscriptHasDependents
	^false! !

!ChangeSet methodsFor: 'filein/out' stamp: 'ar 7/16/2005 18:03'!
postscriptString
	"Answer the string representing the postscript.  "
	^self postscript! !

!ChangeSet methodsFor: 'filein/out' stamp: 'ar 7/16/2005 18:03'!
postscriptString: aString
	"Establish aString as the new contents of the postscript.  "
	self postscript: aString! !

!ChangeSet methodsFor: 'filein/out' stamp: 'ar 7/16/2005 17:55'!
preamble
	"Answer the string representing the preamble"
	^preamble ifNotNil:[preamble isString ifTrue:[preamble] ifFalse:[preamble contents asString]]! !

!ChangeSet methodsFor: 'filein/out' stamp: 'ar 7/16/2005 18:00'!
preamble: aString
	"Establish aString as the new contents of the preamble.  "

	preamble := aString! !

!ChangeSet methodsFor: 'filein/out' stamp: 'ar 7/16/2005 18:00'!
preambleString
	"Answer the string representing the preamble"

	^self preamble! !

!ChangeSet methodsFor: 'filein/out' stamp: 'ar 7/16/2005 18:00'!
preambleString: aString
	"Establish aString as the new contents of the preamble.  "
	self preamble: aString.! !

!ChangeSet methodsFor: 'filein/out' stamp: 'AlainPlantec 12/18/2009 14:11'!
preambleTemplate
	"Answer a string that will form the default contents for a change set's preamble.
	Just a first stab at what the content should be."

	^ String streamContents: [:strm |
		strm nextPutAll: '"Change Set:'.  "NOTE: fileIn recognizes preambles by this string."
		strm tab;tab; nextPutAll: self name.
		strm cr; nextPutAll: 'Date:'; tab; tab; tab; nextPutAll: Date today printString.
		strm cr; nextPutAll: 'Author:'; tab; tab; tab; nextPutAll:  Author fullName.
		strm cr; cr; nextPutAll: '<your descriptive text goes here>"']
"ChangeSet current preambleTemplate"! !

!ChangeSet methodsFor: 'accessing' stamp: 'StephaneDucasse 5/13/2010 11:33'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: ' named ', self name! !

!ChangeSet methodsFor: 'moving changes' stamp: 'di 4/1/2000 12:00'!
removeClassAndMetaClassChanges: class
	"Remove all memory of changes associated with this class and its metaclass.  7/18/96 sw"

	changeRecords removeKey: class name ifAbsent: [].
	changeRecords removeKey: class class name ifAbsent: [].
! !

!ChangeSet methodsFor: 'moving changes' stamp: 'yo 8/30/2002 13:59'!
removeClassChanges: class
	"Remove all memory of changes associated with this class"
	| cname |
	(class isString)
		ifTrue: [ cname := class ]
		ifFalse: [ cname := class name ].

	changeRecords removeKey: cname ifAbsent: [].
	self noteClassForgotten: cname.! !

!ChangeSet methodsFor: 'accessing' stamp: 'sw 6/29/1999 14:48'!
removePostscript
	postscript := nil! !

!ChangeSet methodsFor: 'moving changes' stamp: 'sw 3/5/1999 19:32'!
removePreamble
	preamble := nil! !

!ChangeSet methodsFor: 'change logging' stamp: 'VeronicaUquillas 6/11/2010 11:37'!
removeSelector: selector class: class priorMethod: priorMethod lastMethodInfo: info
	"Include indication that a method has been forgotten.
	info is a pair of the source code pointer and message category
	for the method that was removed."

	class wantsChangeSetLogging ifFalse: [^ self].
	(self changeRecorderFor: class)
		noteRemoveSelector: selector priorMethod: priorMethod lastMethodInfo: info
! !

!ChangeSet methodsFor: 'method changes' stamp: 'di 4/4/2000 11:14'!
removeSelectorChanges: selector class: class 
	"Remove all memory of changes associated with the argument, selector, in 
	this class."

	| chgRecord |
	(chgRecord := changeRecords at: class name ifAbsent: [^ self])
		removeSelector: selector.
	chgRecord hasNoChanges ifTrue: [changeRecords removeKey: class name]! !

!ChangeSet methodsFor: 'change logging' stamp: 'VeronicaUquillas 6/11/2010 11:37'!
renameClass: class from: oldName to: newName 
	"Include indication that a class has been renamed."

	| recorder oldMetaClassName newMetaClassName |
	(recorder := self changeRecorderFor: oldName)
		noteChangeType: #rename;
		noteNewName: newName asSymbol.
		
	"store under new name (metaclass too)"
	changeRecords at: newName put: recorder.
	changeRecords removeKey: oldName.
	self noteClassStructure: class.

	newMetaClassName := newName, ' class'.
	oldMetaClassName := oldName, ' class'.
	recorder := changeRecords at: oldMetaClassName ifAbsent: [^ nil].
	changeRecords at: newMetaClassName put: recorder.
	changeRecords removeKey: oldMetaClassName.
	recorder noteNewName: newMetaClassName! !

!ChangeSet methodsFor: 'class changes' stamp: ''!
reorganizeClass: class 
	"Include indication that a class was reorganized."

	self atClass: class add: #reorganize! !

!ChangeSet methodsFor: 'method changes' stamp: 'SqR 6/13/2000 19:16'!
selectorsInClass: aClassName
	"Used by a ChangeSorter to access the list methods."

	^ (changeRecords at: aClassName ifAbsent: [^#()]) changedSelectors! !

!ChangeSet methodsFor: 'accessing' stamp: 'tk 6/8/1999 22:25'!
structures
	^structures! !

!ChangeSet methodsFor: 'accessing' stamp: 'tk 6/8/1999 22:25'!
superclasses
	^superclasses! !

!ChangeSet methodsFor: 'class changes' stamp: 'di 5/16/2000 09:03'!
trimHistory 
	"Drop non-essential history:  methods added and then removed, as well as rename and reorganization of newly-added classes."

	changeRecords do: [:chgRecord | chgRecord trimHistory]! !

!ChangeSet methodsFor: 'initialization' stamp: 'sw 3/6/1999 09:31'!
veryDeepCopyWith: deepCopier
	"Return self; this is NOT the way to launch new change sets!! Having this method here allows Change Sorters to be in parts bins"! !

!ChangeSet methodsFor: 'initialization' stamp: 'di 3/23/2000 12:14'!
wither
	"The receiver is to be clobbered.  Clear it out.  2/7/96 sw"

	self clear.
	name := nil! !

!ChangeSet methodsFor: 'initialization' stamp: 'MarcusDenker 8/25/2010 20:55'!
zapHistory 
	"Much stronger than trimHistory, but it should still leave the changeSet in good shape.
	Must not be done on revertable changeSets
		ChangeSet allInstancesDo: [:cs | cs zapHistory]."

	changeRecords do: [:chgRecord | chgRecord zapHistory]! !

Object subclass: #ChangesLog
	instanceVariableNames: 'startupStamp'
	classVariableNames: 'DefaultInstance'
	poolDictionaries: ''
	category: 'System-Support'!

!ChangesLog commentStamp: '<historical>' prior: 0!
I represent the changes file and give access to him.  If you want to log a change, look at my #logChange: method.

I have a default instance that is hearing SystemAnnouncements and logging the change logs in there.
But you can have your own instance and log whatever you want.!

!ChangesLog class methodsFor: 'accessing' stamp: 'CamilleTeruel 7/30/2012 00:45'!
default
	^DefaultInstance ifNil: [ 
		DefaultInstance := self new.
		DefaultInstance registerToAnnouncements
	].! !

!ChangesLog class methodsFor: 'accessing' stamp: 'CamilleTeruel 7/30/2012 00:46'!
reset
	SystemAnnouncer unsubscribe: DefaultInstance.
	DefaultInstance := nil.! !

!ChangesLog methodsFor: 'private' stamp: 'GuillermoPolito 7/2/2012 12:23'!
assureStartupStampLogged
	"If there is a startup stamp not yet actually logged to disk, do it now."
	| changesFile |
	startupStamp ifNil: [^ self].
	(SourceFiles isNil or: [(changesFile := SourceFiles at: 2) == nil]) ifTrue: [^ self].
	changesFile isReadOnly ifTrue:[^self].
	changesFile setToEnd; cr; cr.
	changesFile nextChunkPut: startupStamp asString; cr.
	startupStamp := nil.
	self forceChangesToDisk.! !

!ChangesLog methodsFor: 'private' stamp: 'GuillermoPolito 7/2/2012 12:23'!
forceChangesToDisk
	"Ensure that the changes file has been fully written to disk by closing and re-opening it. This makes the system more robust in the face of a power failure or hard-reboot."

	| changesFile |
	changesFile := SourceFiles at: 2.
	(changesFile isKindOf: FileStream) ifTrue: [
		changesFile flush.
		changesFile close.
		changesFile open: changesFile name forWrite: true.
		changesFile setToEnd.
	].
! !

!ChangesLog methodsFor: 'logging' stamp: 'GuillermoPolito 7/2/2012 12:21'!
logChange: aStringOrText 
	"Write the argument, aString, onto the changes file."
	| aString changesFile |
	(SourceFiles isNil or: [(SourceFiles at: 2) == nil]) ifTrue: [^ self].
	self assureStartupStampLogged.

	aString := aStringOrText asString.
	(aString findFirst: [:char | char isSeparator not]) = 0
		ifTrue: [^ self].  "null doits confuse replay"
	(changesFile := SourceFiles at: 2).
	changesFile isReadOnly ifTrue:[^self].
	changesFile setToEnd; cr; cr.
	changesFile nextChunkPut: aString.
		"If want style changes in DoIt, use nextChunkPutWithStyle:, and allow Texts to get here"
	self forceChangesToDisk.! !

!ChangesLog methodsFor: 'event-listening' stamp: 'CamilleTeruel 7/30/2012 01:55'!
logClassRemoved: annoucement
	annoucement removedClass acceptsLoggingOfCompilation ifTrue: [
		self logChange: 'Smalltalk globals removeClassNamed: #', annoucement removedClass name	
	].
! !

!ChangesLog methodsFor: 'event-listening' stamp: 'CamilleTeruel 7/29/2012 19:48'!
logExpressionEvaluated: announcement
	self logChange: announcement expression.! !

!ChangesLog methodsFor: 'event-listening' stamp: 'CamilleTeruel 7/29/2012 19:46'!
logMethodRemoved: announcement
	announcement methodClass acceptsLoggingOfCompilation ifTrue: [
		self logChange: 
			announcement methodClass name, ' removeSelector: #', announcement selector
	]
	! !

!ChangesLog methodsFor: 'accessing' stamp: 'GuillermoPolito 7/2/2012 12:25'!
recordStartupStamp
	
	startupStamp := '----STARTUP----', Time dateAndTimeNow printString, ' as ', Smalltalk imagePath.
! !

!ChangesLog methodsFor: 'event-listening' stamp: 'CamilleTeruel 7/30/2012 02:03'!
registerToAnnouncements
	SystemAnnouncer uniqueInstance 
	
		on: ClassRemoved 
		send: #logClassRemoved:
		to: self;
		
		on: MethodRemoved
		send: #logMethodRemoved:
		to: self;
		
		on: ExpressionEvaluated 
		send: #logExpressionEvaluated:
		to: self.! !

Object subclass: #ClassChangeRecord
	instanceVariableNames: 'changeTypes priorDefinition thisName priorName methodChanges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Changes'!

!ClassChangeRecord commentStamp: '<historical>' prior: 0!
A ClassChangeRecorder keeps track of most substantive changes premissible in a project, isolated or not.

Structure:
inForce		a boolean
			Tells whether these changes are in effect.
			true for all changeSets in and above the current project.
			It should be sufficient only to record this for the changeSet
			as a whole, but this redundancy could help in error recovery.
classIsLocal	a boolean
			True if and only if this class is defined in this layer of the
			project structure.
changeTypes an identitySet
			Summarizes which changes have been made in this class.
			Values include #comment, #reorganize, #rename,
			and the four more summarized below.
thisName	a string
			Retains the class name for this layer.
priorName	a string
			Preserves the prior name.
thisComment	a text
			Retains the class comment for this layer.
priorComment	a text
			Preserves the prior comment.
thisOrganization	a classOrganizer
			Retains the class organization for this layer.
priorOrganization	a classOrganizer
			Preserves the prior organization.
thisMD	a methodDictionary
			Used to prepare changes for nearly atomic invocation
			of this layer (see below).
priorMD	a methodDictionary
			Preserves the state of an altered class as it exists in the next
			outer layer of the project structure.
methodChanges		a dictionary of classChangeRecords
			Retains all the method changes for this layer.

Four of the possible changeTypes are maintained in a mutually exclusive set, analogously to MethodChangeRecords.  Here is a simple summary of the relationship between these four changeType symbols and the recording of prior state
			|	prior == nil			|	prior not nil	
	---------	|----------------------------	|--------------------
	add		|	add					|	change
	---------	|----------------------------	|--------------------
	remove	|	addedThenRemoved	|	remove

A classChangeRecorder is notified of changes by the method
		noteMethodChange: <ClassChangeRecord>.
ClassChangeRecorders are designed to invoke a set of changes relative to the definition of a class in an prior layer.  It is important that both invocation and revocation of these changes take place in a nearly atomic fashion so that interdependent changes will be adopted as a whole, and so that only one flush of the method cache should be necessary.  A further reason for revocation to be simple is that it may be requested as an attempt to recover from an error in a project that is failing.!

!ClassChangeRecord methodsFor: 'all changes' stamp: 'MarcusDenker 8/25/2010 21:02'!
allChangeTypes
	| chgs |
	(priorName notNil and: [ changeTypes includes: #rename ])
		ifTrue: [ 
			(chgs := changeTypes copy) add: 'oldName: ' , priorName.
			^ chgs ].
	^ changeTypes! !

!ClassChangeRecord methodsFor: 'all changes' stamp: 'nice 1/5/2010 15:59'!
assimilateAllChangesIn: otherRecord

	
	otherRecord isClassRemoval ifTrue: [^ self noteChangeType: #remove].

	otherRecord allChangeTypes do:
		[:chg | self noteChangeType: chg fromClass: self realClass].

	otherRecord methodChanges associationsDo:
		[:assn | | changeType selector changeRecord | selector := assn key. changeRecord := assn value.
		changeType := changeRecord changeType.
		(changeType == #remove or: [changeType == #addedThenRemoved])
			ifTrue:
				[changeType == #addedThenRemoved
					ifTrue: [self atSelector: selector put: #add].
				self noteRemoveSelector: selector priorMethod: nil
						lastMethodInfo: changeRecord methodInfoFromRemoval]
			ifFalse: 
				[self atSelector: selector put: changeType]].
! !

!ClassChangeRecord methodsFor: 'method changes' stamp: 'di 3/28/2000 10:38'!
atSelector: selector ifAbsent: absentBlock

	^ (methodChanges at: selector ifAbsent: absentBlock)
		changeType! !

!ClassChangeRecord methodsFor: 'method changes' stamp: 'di 3/28/2000 11:01'!
atSelector: selector put: changeType

	(self findOrMakeMethodChangeAt: selector priorMethod: nil)
		noteChangeType: changeType! !

!ClassChangeRecord methodsFor: 'method changes' stamp: 'di 3/28/2000 10:07'!
changedSelectors
	"Return a set of the changed or removed selectors."

	^ methodChanges keys! !

!ClassChangeRecord methodsFor: 'definition' stamp: 'di 3/27/2000 22:06'!
checkCoherence
	"If I recreate the class then don't remove it"

	(changeTypes includes: #remove) ifTrue:
		[changeTypes remove: #remove.
		changeTypes add: #change].
	(changeTypes includes: #addedThenRemoved) ifTrue:
		[changeTypes remove: #addedThenRemoved.
		changeTypes add: #add].
! !

!ClassChangeRecord methodsFor: 'method changes' stamp: 'MarcusDenker 8/25/2010 20:54'!
findOrMakeMethodChangeAt: selector priorMethod: priorMethod 
	^methodChanges at: selector
		ifAbsentPut: [MethodChangeRecord new]! !

!ClassChangeRecord methodsFor: 'removal' stamp: 'MarcusDenker 8/25/2010 21:02'!
forgetChangesIn: otherRecord
	"See forgetAllChangesFoundIn:.  Used in culling changeSets."

	| cls otherMethodChanges |
	(cls := self realClass) isNil
		ifTrue: [ ^ self ].	"We can do better now, though..."
	otherMethodChanges := otherRecord methodChangeTypes.
	otherMethodChanges
		associationsDo: [ :assoc | 
			| selector actionToSubtract |
			selector := assoc key.
			actionToSubtract := assoc value.
			(cls includesSelector: selector)
				ifTrue: [ 
					(#(#add #change) includes: actionToSubtract)
						ifTrue: [ methodChanges removeKey: selector ifAbsent: [  ] ] ]
				ifFalse: [ 
					(#(#remove #addedThenRemoved) includes: actionToSubtract)
						ifTrue: [ methodChanges removeKey: selector ifAbsent: [  ] ] ] ].
	changeTypes isEmpty
		ifFalse: [ 
			changeTypes removeAllFoundIn: otherRecord allChangeTypes.
			(changeTypes includes: #rename)
				ifFalse: [ changeTypes removeAllSuchThat: [ :x | x beginsWith: 'oldName: ' ] ] ]! !

!ClassChangeRecord methodsFor: 'all changes' stamp: 'di 3/28/2000 10:59'!
hasNoChanges

	^ changeTypes isEmpty and: [methodChanges isEmpty]! !

!ClassChangeRecord methodsFor: 'all changes' stamp: 'di 3/24/2000 09:36'!
includesChangeType: changeType

	changeType == #new ifTrue: [^ changeTypes includes: #add].  "Backwd compat"
	^ changeTypes includes: changeType! !

!ClassChangeRecord methodsFor: 'method changes' stamp: 'di 3/29/2000 16:26'!
infoFromRemoval: selector

	^ (methodChanges at: selector ifAbsent: [^ nil])
		methodInfoFromRemoval

! !

!ClassChangeRecord methodsFor: 'initialization' stamp: 'MarcusDenker 8/25/2010 20:59'!
initFor: className revertable: isRevertable

	changeTypes := IdentitySet new.
	methodChanges := IdentityDictionary new.
	priorName := thisName := className.! !

!ClassChangeRecord methodsFor: 'removal' stamp: 'di 4/1/2000 23:05'!
isClassRemoval
	"NOTE: there are other removals with changeType #addedThenRemoved,
	but this message is used to write out removals in fileOut, and those
	cases should not be written out."

	^ (changeTypes includes: #remove) or: [changeTypes includes: #removeClass]! !

!ClassChangeRecord methodsFor: 'method changes' stamp: 'eem 6/11/2008 16:53'!
methodChangeTypes
	"Return an old-style dictionary of method change types."

	| dict |
	dict := IdentityDictionary new.
	methodChanges associationsDo:
		[:assn | | selector record |
		selector := assn key.
		record := assn value.
		dict at: selector put: record changeType].
	^ dict! !

!ClassChangeRecord methodsFor: 'method changes' stamp: 'di 4/1/2000 23:49'!
methodChanges

	^ methodChanges! !

!ClassChangeRecord methodsFor: 'all changes' stamp: 'di 3/28/2000 15:14'!
noteChangeType: changeSymbol

	^ self noteChangeType: changeSymbol fromClass: nil! !

!ClassChangeRecord methodsFor: 'all changes' stamp: 'MarcusDenker 8/25/2010 20:59'!
noteChangeType: changeSymbol fromClass: class

	(changeSymbol = #new or: [changeSymbol = #add]) ifTrue:
		[changeTypes add: #add.
		changeTypes remove: #change ifAbsent: [].
		^ self].
	changeSymbol = #change ifTrue:
		[(changeTypes includes: #add) ifTrue: [^ self].
		^ changeTypes add: changeSymbol].
	changeSymbol == #addedThenRemoved ifTrue:
		[^ self].  "An entire class was added but then removed"
	changeSymbol = #comment ifTrue:
		[^ changeTypes add: changeSymbol].
	changeSymbol = #reorganize ifTrue:
		[^ changeTypes add: changeSymbol].
	changeSymbol = #rename ifTrue:
		[^ changeTypes add: changeSymbol].
	(changeSymbol beginsWith: 'oldName: ') ifTrue:
		["Must only be used when assimilating other changeSets"
		(changeTypes includes: #add) ifTrue: [^ self].
		priorName := changeSymbol copyFrom: 'oldName: ' size + 1 to: changeSymbol size.
		^ changeTypes add: #rename].
	changeSymbol = #remove ifTrue:
		[(changeTypes includes: #add)
			ifTrue: [changeTypes add: #addedThenRemoved]
			ifFalse: [changeTypes add: #remove].
		^ changeTypes removeAllFoundIn: #(add change comment reorganize)].

	self error: 'Unrecognized changeType'! !

!ClassChangeRecord methodsFor: 'method changes' stamp: 'MarcusDenker 8/25/2010 21:02'!
noteNewMethod: newMethod selector: selector priorMethod: methodOrNil
	| methodChange |
	methodChange := self findOrMakeMethodChangeAt: selector priorMethod: methodOrNil.
	methodOrNil isNil
		ifTrue: [ methodChange noteChangeType: #add ]
		ifFalse: [ methodChange noteChangeType: #change ].
	methodChange noteNewMethod: newMethod! !

!ClassChangeRecord methodsFor: 'rename' stamp: 'di 5/8/2000 20:39'!
noteNewName: newName

	thisName := newName! !

!ClassChangeRecord methodsFor: 'definition' stamp: 'di 3/27/2000 22:08'!
notePriorDefinition: oldClass

	oldClass ifNil: [^ self].
	priorDefinition ifNil: [priorDefinition := oldClass definition]! !

!ClassChangeRecord methodsFor: 'method changes' stamp: 'di 3/23/2000 23:00'!
noteRemoveSelector: selector priorMethod: priorMethod lastMethodInfo: infoOrNil

	| methodChange |
	methodChange := self findOrMakeMethodChangeAt: selector priorMethod: priorMethod.
	methodChange changeType == #add
		ifTrue: [methodChange noteChangeType: #addedThenRemoved]
		ifFalse: [methodChange noteChangeType: #remove].

	infoOrNil ifNotNil:
		["Save the source code pointer and category so can still browse old versions"
		methodChange noteMethodInfoFromRemoval: infoOrNil]

! !

!ClassChangeRecord methodsFor: 'definition' stamp: 'di 3/28/2000 09:12'!
priorDefinition

	^ priorDefinition! !

!ClassChangeRecord methodsFor: 'rename' stamp: 'di 3/24/2000 09:38'!
priorName

	^ priorName! !

!ClassChangeRecord methodsFor: 'isolation layers' stamp: 'lr 3/14/2010 21:13'!
realClass
	"Return the actual class (or meta), as determined from my name."

	thisName ifNil: [ ^ nil ].
	(thisName endsWith: ' class')
		ifTrue: [ ^ (Smalltalk globals at: (thisName copyFrom: 1 to: thisName size - 6) asSymbol ifAbsent: [ ^ nil ]) class ]
		ifFalse: [ ^ Smalltalk globals at: thisName ifAbsent: [ ^ nil ] ]! !

!ClassChangeRecord methodsFor: 'method changes' stamp: 'sw 8/14/2002 11:11'!
removeSelector: selector
	"Remove all memory of changes associated with the argument, selector, in this class."

	selector == #Comment
		ifTrue:
			[changeTypes remove: #comment ifAbsent: []]
		ifFalse:
			[methodChanges removeKey: selector ifAbsent: []]! !

!ClassChangeRecord methodsFor: 'rename' stamp: 'tk 6/8/2001 09:11'!
thisName

	^ thisName! !

!ClassChangeRecord methodsFor: 'all changes' stamp: 'di 5/16/2000 08:43'!
trimHistory
	"Drop non-essential history."

	"Forget methods added and later removed"
	methodChanges keysAndValuesRemove:
		[:sel :chgRecord | chgRecord changeType == #addedThenRemoved].

	"Forget renaming and reorganization of newly-added classes."
	(changeTypes includes: #add) ifTrue:
		[changeTypes removeAllFoundIn: #(rename reorganize)].
! !

!ClassChangeRecord methodsFor: 'initialization' stamp: 'MarcusDenker 8/25/2010 20:59'!
zapHistory
	"Drop all recorded information not needed to simply keep track of what has been changed.
	Saves a lot of space."

	methodChanges do: [:r | r noteNewMethod: nil].  "Drop all refes to old methods"! !

Object subclass: #Clipboard
	instanceVariableNames: 'contents recent'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'System-Clipboard'!

!Clipboard commentStamp: 'AlainPlantec 1/15/2010 11:42' prior: 0!
The Clipboard class is the abstract superclass for the concrete platform specific clipboard. The legacy clipboard support using the VM supplied primitives is implemented by Clipboard/ExternalClipboard.

The Clipboard implements a basic buffering scheme for text. The currently selected text is also exported to the OS so that text can be copied from and to other applications. Commonly only a single instance is used (the default clipboard) but applications are free to use other than the default clipboard if necessary.!

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:45'!
chooseRecentClipping  "Clipboard chooseRecentClipping"
	"Choose by menu from among the recent clippings"
	^self default chooseRecentClipping! !

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:35'!
clipboardText "Clipboard clipboardText"
	^self default clipboardText.! !

!Clipboard class methodsFor: 'accessing' stamp: 'ar 1/15/2001 18:35'!
clipboardText: aText 
	^self default clipboardText: aText! !

!Clipboard class methodsFor: 'accessing' stamp: 'michael.rueger 3/2/2009 11:12'!
default
	^Default ifNil: [Default := OSPlatform current clipboardClass new].! !

!Clipboard class methodsFor: 'initialization' stamp: 'michael.rueger 3/2/2009 11:11'!
initialize
	"Clipboard initialize"

	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self.
	self startUp: true.! !

!Clipboard class methodsFor: 'initialization' stamp: 'VeronicaUquillas 6/11/2010 13:53'!
shutDown: quitting
	"Pharo is shutting down. If this platform requires specific shutdown code, this is a great place to put it."
! !

!Clipboard class methodsFor: 'initialization' stamp: 'VeronicaUquillas 6/11/2010 13:53'!
startUp: resuming
	"Pharo is starting up. If this platform requires specific intialization, this is a great place to put it."
	resuming
		ifTrue: [Default := nil]! !

!Clipboard methodsFor: 'accessing' stamp: 'StephaneDucasse 1/29/2010 21:42'!
chooseRecentClipping  
	"Choose by menu from among the recent clippings"
	"Clipboard chooseRecentClipping"
	
	recent ifNil: [^ nil].
	^ UIManager default
		chooseFrom: (recent collect: [:txt | ((txt asString contractTo: 50)
									copyReplaceAll: Character cr asString with: '\')
									copyReplaceAll: Character tab asString with: '|'])
		values: recent! !

!Clipboard methodsFor: 'accessing' stamp: 'michael.rueger 6/10/2009 13:42'!
clipboardText
	"Return the text currently in the clipboard. If the system clipboard is empty, or if it differs from the Smalltalk clipboard text, use the Smalltalk clipboard. This is done since (a) the Mac clipboard gives up on very large chunks of text and (b) since not all platforms support the notion of a clipboard."

	| string decodedString |
	string := self primitiveClipboardText.
	(string isEmpty
			or: [string = contents asString])
		ifTrue: [^ contents].
	decodedString := string convertFromWithConverter: UTF8TextConverter new.
	decodedString := decodedString replaceAll: 10 asCharacter with: 13 asCharacter.
	^ decodedString = contents asString
		ifTrue: [contents]
		ifFalse: [decodedString asText].
! !

!Clipboard methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 1/28/2011 15:19'!
clipboardText: text 

        | string |
        string := text asString.
        self noteRecentClipping: text asText.
        contents := text asText.
        string := string convertToWithConverter: UTF8TextConverter new.
        self primitiveClipboardText: string.! !

!Clipboard methodsFor: 'initialize' stamp: 'alain.plantec 5/28/2009 09:47'!
initialize
	super initialize.
	contents := '' asText.
	recent := OrderedCollection new! !

!Clipboard methodsFor: 'private' stamp: 'ar 1/15/2001 18:34'!
noteRecentClipping: text
	"Keep most recent clippings in a queue for pasteRecent (paste... command)"
	text isEmpty ifTrue: [^ self].
	text size > 50000 ifTrue: [^ self].
	(recent includes: text) ifTrue: [^ self].
	recent addFirst: text.
	[recent size > 5] whileTrue: [recent removeLast].
! !

!Clipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/25/2009 14:23'!
primitiveClipboardText
	"Get the current clipboard text. Return the empty string if the primitive fails."
	<primitive: 141>
	^ ''! !

!Clipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/25/2009 14:23'!
primitiveClipboardText: aString
	"Set the current clipboard text to the given string."

	<primitive: 141>
	"don't fail if the primitive is not implemented"! !

Clipboard subclass: #ExternalClipboard
	instanceVariableNames: 'clipboard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Clipboard'!

!ExternalClipboard commentStamp: 'michael.rueger 3/2/2009 13:25' prior: 0!
An ExternalClipboard is the abstract superclass for the platform specific clipboards based on the clipboard plugin (former ExtendedClipboardInterface originally developed for Sophie).

Instance Variables
	clipboard:		SmallInteger

clipboard
	- handle for the external clipboard. If 0 the external clipboard is invalid
!

!ExternalClipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/25/2009 14:22'!
addClipboardData: data dataFormat: aFormat
	clipboard = 0 ifTrue: 
		[Clipboard clipboardText: data asString.	
		^self].
	self primAddClipboardData: clipboard data: data dataFormat: aFormat! !

!ExternalClipboard methodsFor: 'accessing' stamp: 'michael.rueger 3/2/2009 13:42'!
clearClipboard
	clipboard = 0 ifTrue: [^self].
	^ self primClearClipboard: clipboard.! !

!ExternalClipboard methodsFor: 'accessing' stamp: 'michael.rueger 6/10/2009 13:42'!
clipboardText
	"Return the text currently in the clipboard. If the system clipboard is empty, or if it differs from the Smalltalk clipboard text, use the Smalltalk clipboard. This is done since (a) the Mac clipboard gives up on very large chunks of text and (b) since not all platforms support the notion of a clipboard."

	| decodedString bytes |
	clipboard = 0 ifTrue: 
		[^super clipboardText].
	bytes := self primReadClipboardData: clipboard format: 'public.utf8-plain-text'.
	bytes
		ifNil: [^super clipboardText].
	decodedString := bytes asString convertFromWithConverter: UTF8TextConverter new.
	decodedString := decodedString replaceAll: 10 asCharacter with: 13 asCharacter.
	^decodedString = contents asString
		ifTrue: [contents]
		ifFalse: [decodedString asText].
! !

!ExternalClipboard methodsFor: 'accessing' stamp: 'michael.rueger 3/25/2009 14:47'!
clipboardText: text 

	| string data |
	string := text asString.
	self noteRecentClipping: text asText.
	contents := text asText.
	data := (string convertToWithConverter: UTF8TextConverter new) asByteArray.
	clipboard = 0 ifTrue: 
		[^super clipboardText: text].
	self clearClipboard.
	self primAddClipboardData: clipboard data: data dataFormat: 'public.utf8-plain-text'! !

!ExternalClipboard methodsFor: 'private' stamp: 'michael.rueger 3/2/2009 13:42'!
createClipboard
	clipboard = 0 ifTrue: [^self].
	^ self primCreateClipboard.! !

!ExternalClipboard methodsFor: 'initialize' stamp: 'StephaneDucasse 8/30/2009 14:55'!
initialize
	super initialize.
	clipboard := [self createClipboard] on: Error do: [:ex | clipboard := 0]! !

!ExternalClipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/25/2009 14:25'!
primAddClipboardData: aClipboard data: data dataFormat: aFormat

	<primitive:'ioAddClipboardData' module: 'ClipboardExtendedPlugin'>
	^ self primitiveFailed! !

!ExternalClipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/2/2009 13:42'!
primClearClipboard:  aClipboard
	<primitive:'ioClearClipboard' module: 'ClipboardExtendedPlugin'>
	^ self primitiveFailed.
! !

!ExternalClipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/2/2009 13:42'!
primCreateClipboard
	<primitive:'ioCreateClipboard' module: 'ClipboardExtendedPlugin'>
	^ self primitiveFailed.
! !

!ExternalClipboard methodsFor: 'primitives' stamp: 'michael.rueger 3/25/2009 14:25'!
primGetClipboardFormat: aClipboard formatNumber: formatNumber

	<primitive:'ioGetClipboardFormat' module: 'ClipboardExtendedPlugin'>
	^ self primitiveFailed! !

!ExternalClipboard methodsFor: 'primitives' stamp: 'marcus.denker 6/11/2009 12:24'!
primReadClipboardData: aClipboard format: format

	<primitive:'ioReadClipboardData' module: 'ClipboardExtendedPlugin'>
	^ self primitiveFailed! !

ExternalClipboard subclass: #MacOSClipboard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Clipboard'!

!MacOSClipboard methodsFor: 'private' stamp: 'michael.rueger 3/2/2009 12:51'!
addUF8StringClipboardData: aString
	| ba  |

	self clearClipboard.
	ba := aString convertToWithConverter: (UTF8TextConverter new).
	self addClipboardData: ba dataFormat: 'public.utf8-plain-text'
! !

Clipboard subclass: #UnixClipboard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Clipboard'!

!UnixClipboard commentStamp: 'HenrikSperreJohansen 1/28/2011 15:15' prior: 0!
The Unix vm does not do line-ending conversion, thus this subclass which specifies we convert CR's to LF's when pasting to it!

!UnixClipboard methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 1/28/2011 15:19'!
clipboardText: text 

        | string |
        string := text asString.
        self noteRecentClipping: text asText.
        contents := text asText.
        string := string convertToWithConverter: UTF8TextConverter new.
        string := string replaceAll: 13 asCharacter with: 10 asCharacter.
        self primitiveClipboardText: string.! !

Object subclass: #CommandLineArguments
	instanceVariableNames: 'arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

CommandLineArguments class
	instanceVariableNames: 'singleton'!

!CommandLineArguments commentStamp: '<historical>' prior: 0!
The CommandLineArguments represents the arguments passed to the image.
In the following case,

	$PHARO_VM myImage.image --foo bar
	
`CommandLineArguments default` contains {'--foo'. 'bar'}.!

CommandLineArguments class
	instanceVariableNames: 'singleton'!

!CommandLineArguments class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/2/2012 11:41'!
default
	^ singleton ifNil: [singleton := self new initialize]! !

!CommandLineArguments class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/1/2012 20:00'!
withArguments: aCollection
	^ self basicNew 
		initializeWithArguments: aCollection;
		yourself! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 12:33'!
allFilesWithExtension: anExtension
	^ self arguments select: [ :arg|
		arg endsWith: anExtension ]! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 19:57'!
argumentAt: index
	^ arguments at: index ! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:42'!
arguments
	^ arguments! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/13/2012 19:47'!
commandLineArguments
	"self commandLineArguments"
	
	| documentPath args |
	
	args := OrderedCollection withAll: Smalltalk arguments.
	documentPath := Smalltalk vm documentPath.
	documentPath isEmptyOrNil
		ifFalse: [ args addFirst: documentPath ].
 	^ args.! !

!CommandLineArguments methodsFor: 'copying' stamp: 'CamilloBruni 5/1/2012 20:07'!
copySubcommand
	"return a new copy of this CommandLine without the first arguments"
	^ self class withArguments: arguments allButFirst! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/1/2012 20:50'!
hasArguments
	^ arguments size > 0! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/2/2012 12:55'!
hasFileWithExtension: aFileExtension
	"return true if the first argument has the given file extension"
	^ self arguments anySatisfy: [ :arg| 
		arg endsWith: aFileExtension].! !

!CommandLineArguments methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 12:49'!
hasOption: aString
	| option |
	option := (aString size = 1
					ifTrue: [ '-']
					ifFalse: [ '--']), aString.
	^ self arguments includes: option ! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/2/2012 14:07'!
includesSubCommand: aName
	self withFirstArgument: [ :arg|
		arg = aName ifTrue: [ ^ true ]].
	^ false! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 5/13/2012 19:47'!
initialize
	
	| documentPath  |
	
	arguments := OrderedCollection withAll: Smalltalk arguments.
	documentPath := Smalltalk vm documentPath.
	documentPath isEmptyOrNil
		ifFalse: [ arguments addFirst: documentPath ].! !

!CommandLineArguments methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/1/2012 20:01'!
initializeWithArguments: aCollection
	super initialize.
	arguments := aCollection.! !

!CommandLineArguments methodsFor: 'testing' stamp: 'CamilloBruni 4/28/2012 19:54'!
withFirstArgument: aBlock
	self arguments ifEmpty: [ ^ self ].
	^ aBlock value: self arguments first! !

Object subclass: #CommandLineHandler
	instanceVariableNames: 'commandLine stdout'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!CommandLineHandler commentStamp: '<historical>' prior: 0!
A CommandLineHandler is activated by the CommandLine. 

The responsible handler with the highest priority is selected and its instance-side method #activate is invoked.

By default the handlers are selected by their class name. In the following shell invocation the FooHandler is chosen:

	$PATH_TO_VM myImage.image FooHandler

A handler may provide a short name with the class-side #commandName method. If the FooHandler defined #commandName returning 'foo' it would be activated with the following shell invocation:
	
	$PATH_TO_VM myImage.image foo
	
For more sophisticated handler selection the CommandLineHandler should implement the #isResponsibleFor: class-side method. An instance of the current command line options is passed to this method which should then return a boolean.

Between all the responsible handlers the one with the highes #priority is chosen. To change the priority overwrite the class-side accessor.
!

CommandLineHandler subclass: #BasicCodeLoader
	instanceVariableNames: 'sourceFiles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!BasicCodeLoader commentStamp: '<historical>' prior: 0!
CodeLoader provides a way to load smalltalk source file from local.

Example: 

CodeLoader new 
	loadSourceFiles: (Array with: 
		(#file:, 
		FileDirectory default fullName, 
		'/useless.st')); 
	installSourceFiles !

!BasicCodeLoader class methodsFor: 'accessing' stamp: 'CamilloBruni 5/2/2012 11:35'!
commandName
	^ 'st'! !

!BasicCodeLoader class methodsFor: 'accessing' stamp: 'CamilloBruni 5/3/2012 14:14'!
description
	^ 'handles .st source files'! !

!BasicCodeLoader class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:31'!
isResponsibleFor: aCommandLine
	"This handler is reponsible only for .st files"
	(aCommandLine hasFileWithExtension: '.st')
		ifTrue: [ ^ true ].
	^ super isResponsibleFor: aCommandLine! !

!BasicCodeLoader methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 14:17'!
activate
	self loadSourceFiles: (self commandLine allFilesWithExtension: 'st').
	self installSourceFiles.
	(self commandLine hasOption: 'quit')
		ifTrue: [ self exitSucess ].! !

!BasicCodeLoader methodsFor: 'private' stamp: 'CamilloBruni 4/28/2012 21:48'!
handleError: error	
	
	"for syntax errors we can used the default action"	"otherwise resignal it"
	(error isKindOf: SyntaxErrorNotification)
		ifTrue: [ error defaultAction ]
		ifFalse: [ error pass ]! !

!BasicCodeLoader methodsFor: 'private' stamp: 'EstebanLorenzano 4/4/2012 10:38'!
handleError: error reference: aReference
	"Print a header before failing on an errro / syntax notification from the the script loaded by the given request"
	|stderr|
	
	"spit out a warning if in headless mode, otherwise a debugger will popup"
	Smalltalk isHeadless 
		ifTrue: [ self inform: 'Errors in script loaded from ', aReference fullName ].
																														
	(error isKindOf: SyntaxErrorNotification)
		"for syntax errors we can used the default action"
		ifTrue: [ error defaultAction ]
		"otherwise resignal it"
		ifFalse: [ error pass ]! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'CamilloBruni 6/14/2012 23:15'!
handleErrorsDuring: aBlock reference: aReference 
	
	aBlock
		on: Error, ParserNotification
		do: [ :e| 	self handleError: e reference: aReference ].
	! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'CamilloBruni 6/14/2012 23:29'!
installSourceFile: aReference
	"Install the the source file given by aFileReference"
	| codeImporter |

	" parse the code given in the source file"
	aReference readStreamDo: [ :stream |
		self 
			handleErrorsDuring: [ 
				codeImporter := CodeImporter fileStream: stream.
				codeImporter parseDeclarations ]
			reference: aReference].
	
	"evaluate the declarations in a second step to allow for properly closing the source file stream"
	self 
		handleErrorsDuring: [ codeImporter evaluate ]
		reference: aReference.! !

!BasicCodeLoader methodsFor: 'installing' stamp: 'EstebanLorenzano 4/4/2012 10:31'!
installSourceFiles
	"Install the previously loaded source files"

	sourceFiles ifNil: [ ^self ].
	[ sourceFiles do: [ :reference | self installSourceFile: reference ] ]
	ensure: [ sourceFiles := nil ].
	! !

!BasicCodeLoader methodsFor: 'loading' stamp: 'CamilloBruni 5/1/2012 20:51'!
loadSourceFiles: anArray
	"Load all the source files in the given array."

	sourceFiles := anArray collect: [ :each | 
						(FileSystem disk resolve: each) asFileReference ].
! !

BasicCodeLoader subclass: #EvaluateCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!EvaluateCommandLineHandler commentStamp: '<historical>' prior: 0!
A CommandLineHandler that reads a string from the command line and outputs the evaluated result.

This handler either evaluates the arguments passed to the image:
	$PHARO_VM my.image eval  1 + 2
	
or it can read directly from stdin:

	echo "1+2" | $PHARO_VM my.image eval !

!EvaluateCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 22:48'!
commandName
	^ 'eval'! !

!EvaluateCommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/3/2012 14:14'!
description
	^ 'directly evaluates passed in one line scripts'! !

!EvaluateCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 7/4/2012 00:20'!
isResponsibleFor: commandLineArguments
	"directly handle top-level -e and --evaluate options"
	commandLineArguments withFirstArgument: [ :arg| 
		(#('-e' '--evaluate') includes: arg)
			ifTrue: [ ^ true ]].
	
	^ commandLineArguments includesSubCommand: self commandName! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 6/23/2012 22:45'!
activate
	self evaluateStdIn.
	self evaluateArguments.
	self quit.! !

!EvaluateCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 6/23/2012 22:33'!
evaluate: aStream
		
	[ self stdout print: (Compiler evaluate: aStream); cr ]
		on: Error, ParserNotification
		do: [ :e| 	self handleError: e  ].! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 6/23/2012 22:46'!
evaluateArguments
	| args |
	args :=  self arguments joinUsing: Character space.
	args ifEmpty: [ ^ self ].
	self evaluate: args! !

!EvaluateCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 6/23/2012 22:45'!
evaluateStdIn
	| stdin |
	stdin := FileStream stdin ifNil: [ ^ self ].
	stdin atEnd  ifTrue: [ ^ self ].
	
	self flag: #todo. "the whole compilation machinery does not work with stdin directly, hence we load everything a buffer first"
	self evaluate: (String streamContents: [ :s|
		[ stdin atEnd ] whileFalse: [ 
			stdin next ifNotNilDo: [ :char|
			s nextPut: char ]]])! !

!CommandLineHandler class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/2/2012 11:25'!
activateWith: aCommandLine 
	
	^ self new 
		commandLine: (self prepareSubcommand: aCommandLine);
		activate! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2012 17:38'!
allHandlers
	^ self allSubclasses reject: [ :handler| handler isAbstract ]! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:56'!
commandName
	" Overwrite this method to provide a short name for this command handler.
	You may pass the commandName as first argument to the image to select this handler"
	^ self name! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:53'!
description
	"This method should return a short one-line description of the command"
	^ nil! !

!CommandLineHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 4/29/2012 17:39'!
isAbstract
	^ self = CommandLineHandler! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 14:06'!
isResponsibleFor: aCommandLineArguments
	^ aCommandLineArguments includesSubCommand: self commandName! !

!CommandLineHandler class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/2/2012 11:28'!
prepareSubcommand: commandLineArguments

	"strip the subcommand name from the arguments"
	commandLineArguments withFirstArgument: [ :arg|
		arg = self commandName 
			ifTrue: [ ^ commandLineArguments copySubcommand ]].
	
	"not a subcommand hence we keep the same args"
	^ commandLineArguments! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 01:51'!
priority
	^ 0! !

!CommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:30'!
selectHandlersFor: aCommandLine

	^ self allHandlers select: [ :handlerClass|
		handlerClass isResponsibleFor: aCommandLine ]! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:33'!
<< aString
	^ self stdout 
		nextPutAll: aString;
		yourself! !

!CommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 4/28/2012 19:46'!
activate
	 self subclassResponsibility! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:43'!
allHandlers 
	^ CommandLineHandler allHandlers ! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:56'!
arguments
	^ commandLine arguments ! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:28'!
commandLine
	^ commandLine! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 4/28/2012 19:28'!
commandLine: aCommandLine 
	commandLine := aCommandLine! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:42'!
commandName
	^ self class commandName! !

!CommandLineHandler methodsFor: 'accessing' stamp: 'CamilloBruni 5/1/2012 13:52'!
description
	^ self class description! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:15'!
exitFailure 
	^ Smalltalk exitFailure! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/29/2012 16:15'!
exitSucess
	^ Smalltalk exitSucess.! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 4/28/2012 19:47'!
initialize
	super initialize.
	self initializeStdout
	! !

!CommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 4/28/2012 19:47'!
initializeStdout
	"install the line end conversion and initialize the converter"
	FileStream stdout wantsLineEndConversion: true; converter.
	stdout := FileStream stdout.! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/28/2012 21:36'!
quit
	Smalltalk exitSucess.! !

!CommandLineHandler methodsFor: 'utility' stamp: 'CamilloBruni 4/28/2012 19:47'!
stdout
	^ stdout! !

CommandLineHandler subclass: #DefaultCommandLineHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-CommandLine'!

!DefaultCommandLineHandler commentStamp: '<historical>' prior: 0!
A DefaultCommandLineHandler handles default command line arguments and options.

The DefaultCommandLineHandler is activated before all other handlers. It first checks if another handler is available. If so it will activate the found handler.!

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/3/2012 14:13'!
description
	^ 'responsible for the default options and activating other commands'! !

!DefaultCommandLineHandler class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/2/2012 11:34'!
initialize 
	"hook into the startup list"
	Smalltalk addToStartUpList: self.! !

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 4/28/2012 01:27'!
isResponsibleFor: aCommandLine
	^ true! !

!DefaultCommandLineHandler class methodsFor: 'handler selection' stamp: 'CamilloBruni 5/2/2012 11:34'!
priority 
	"low priority"
	^ Float infinity negated! !

!DefaultCommandLineHandler class methodsFor: 'class initialization' stamp: 'CamilloBruni 5/1/2012 21:18'!
startUp: resuming

	"only handle when lauching a new image"
	resuming ifFalse: [ ^ self ].
	Smalltalk addDeferredStartupAction: [ self new activate ]! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/1/2012 21:23'!
activate
	self arguments ifEmpty: [ ^ self ].
	
	^ self handleArgument: self arguments first.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 21:54'!
copyright
	self stdout 
		nextPutAll: Smalltalk licenseString; cr.
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 5/2/2012 00:23'!
error
	self arguments size = 1
		ifTrue:  [
			(self arguments first beginsWith: '-')
				ifTrue:  [ self << 'Invalid option: ']
				ifFalse: [ self << 'Invalid sub command: ']]
		ifFalse: [ self << 'Invalid arguments: "'].
		
	(self << (' ' join: self arguments)) 
		nextPutAll: '"'; cr.
		
	self printHelp.
	
	Smalltalk isInteractive ifFalse: [ ^ self exitFailure ].
	Error signal: 'Invalid arguments'.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 00:24'!
handleArgument: aString
	
	"give priority to subcommands"
	self handleSubcommand == self
		ifFalse: [ ^ self ].
		
	"check for default options"
	aString = '--version'
		ifTrue: [ ^ self version ].
		
	aString = '--help'
		ifTrue: [ ^ self help ].
		
	aString = '--list'
		ifTrue: [ ^ self list ].
		
	aString = '--copyright'
		ifTrue: [ ^ self copyright ].
	
	"none of the previous options matched hence we output an error message"
	self error.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/2/2012 00:24'!
handleSubcommand
	"check if there is an active subcommand"
	
	| handlers |
	
	handlers := self selectHandlers.
	
	"ignore the default command line handler"
	handlers first = DefaultCommandLineHandler
		ifTrue: [ ^ self ].
		
	^ handlers first activateWith: commandLine.! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/29/2012 16:20'!
help
	self printHelp.
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'initialize-release' stamp: 'CamilloBruni 5/1/2012 21:24'!
initialize
	super initialize.
	commandLine := CommandLineArguments new! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/26/2012 13:09'!
list
	| handlers shortNames maxShortNameSize |
	self stdout 
		nextPutAll: 'Currently installed Command Line Handlers:'; cr.
		
	handlers := self allHandlers reject: [ :cls| cls = DefaultCommandLineHandler ].
	shortNames := handlers collect: #commandName.
	maxShortNameSize := (shortNames detectMax: [ :name|  name size ]) size.
	
	handlers do: [ :handler|
		self stdout 
			nextPutAll: '    ';
			nextPutAll: (handler commandName padRightTo: maxShortNameSize); 
			nextPutAll: '    ';
			nextPutAll: (handler description ifNil: [ '--']);cr].
	self quit.! !

!DefaultCommandLineHandler methodsFor: 'activation' stamp: 'CamilloBruni 5/1/2012 13:38'!
printHelp
	self stdout 
		nextPutAll: 'Usage:'; cr;
		nextPutAll: '    --help       print this help message'; cr;
		nextPutAll: '    --copyright  print the copyrights'; cr;
		nextPutAll: '    --version    print the version for the image and the vm'; cr;
		nextPutAll: '    --list       list a description of all active command line handlers'; cr.! !

!DefaultCommandLineHandler methodsFor: 'private' stamp: 'CamilloBruni 5/1/2012 21:22'!
selectHandlers
	| handlers |
	handlers := CommandLineHandler selectHandlersFor: commandLine.
	handlers := handlers sort: [ :a :b| a priority >= b priority ].
	^ handlers! !

!DefaultCommandLineHandler methodsFor: 'commands' stamp: 'CamilloBruni 4/28/2012 22:00'!
version
	self stdout 
		nextPutAll: 'Image: '; print: SystemVersion current; cr;
		nextPutAll: 'VM:    '; nextPutAll: Smalltalk vm version; cr.
	self quit.! !

Object subclass: #DefaultExternalDropHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!DefaultExternalDropHandler commentStamp: 'dgd 4/5/2004 19:07' prior: 0!
An alternative default handler that uses the file-list services to process files.
!

!DefaultExternalDropHandler class methodsFor: 'initialization' stamp: ''!
initialize
	"initialize the receiver"
	ExternalDropHandler defaultHandler: self new! !

!DefaultExternalDropHandler class methodsFor: 'initialization' stamp: ''!
unload
	"initialize the receiver"
	ExternalDropHandler defaultHandler: nil! !

!DefaultExternalDropHandler methodsFor: 'private' stamp: ''!
chooseServiceFrom: aCollection
	"private - choose a service from aCollection asking the user if  
	needed"
	aCollection size = 1
		ifTrue: [^ aCollection anyOne].
	^ UIManager default chooseFrom: (aCollection collect: [:each | each label]) values: aCollection.
! !

!DefaultExternalDropHandler methodsFor: 'private' stamp: ''!
servicesForFileNamed: aString 
	"private - answer a collection of file-services for the file named  
	aString"
	| allServices |
	allServices := FileServices itemsForFile: aString.
	^ allServices
		reject: [:svc | self unwantedSelectors includes: svc selector]! !

!DefaultExternalDropHandler methodsFor: 'private' stamp: ''!
unwantedSelectors
	"private - answer a collection well known unwanted selectors "
	^ #(#removeLineFeeds: #addFileToNewZip: #compressFile: #putUpdate: )! !

Object subclass: #DiffElement
	instanceVariableNames: 'string hash match'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-FilePackage'!

!DiffElement commentStamp: 'HenrikSperreJohansen 5/21/2010 01:41' prior: 0!
My instances are container objects used by TextDiffBuilder for comparison. They hold a string and the precomputed hash of the string to speed up #=. They may reference another DiffElement object which is their pair in the diff.

Instance Variables
	hash:		<Integer>
	match:		<DiffElement>
	string:		<String>

hash
	- the hash of string, stored for fast access

match
	- another DiffElement object which has the same string and turned out to be my pair in the longest common subsequence found by a TextDiffBuilder, or nil if I don't a matching DiffElement

string
	- a part of a longer text, typically a line
!

!DiffElement class methodsFor: 'instance creation' stamp: 'HenrikSperreJohansen 5/21/2010 01:49'!
string: aString

	^self new
		string: aString;
		yourself! !

!DiffElement methodsFor: 'comparing' stamp: 'HenrikSperreJohansen 5/21/2010 01:50'!
= anObject

	^anObject class == self class and: [
		anObject hash = hash and: [
			anObject string = string ] ]! !

!DiffElement methodsFor: 'testing' stamp: 'HenrikSperreJohansen 5/21/2010 01:51'!
hasMatch

	^match notNil! !

!DiffElement methodsFor: 'comparing' stamp: 'HenrikSperreJohansen 5/21/2010 01:50'!
hash

	^hash! !

!DiffElement methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 5/21/2010 01:51'!
match

	^match! !

!DiffElement methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 5/21/2010 01:51'!
match: aDiffMatch

	match := aDiffMatch
	! !

!DiffElement methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 5/21/2010 01:51'!
matches: aDiffMatch

	match := aDiffMatch.
	aDiffMatch match: self! !

!DiffElement methodsFor: 'printing' stamp: 'HenrikSperreJohansen 5/21/2010 01:51'!
printOn: aStream

	super printOn: aStream.
	aStream
		nextPut: $(;
		print: hash;
		nextPutAll: ', ';
		print: string;
		nextPutAll: ', ';
		print: (match class == self class);
		nextPut: $)! !

!DiffElement methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 5/21/2010 01:51'!
string

	^string! !

!DiffElement methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 5/21/2010 01:50'!
string: aString

	string := aString.
	string isOctetString ifTrue: [ "Make sure that #hash will return the same value if the strings are equal."
		string := string asOctetString ].
	hash := string hash! !

Object subclass: #DigitalSignatureAlgorithm
	instanceVariableNames: 'randKey randSeed'
	classVariableNames: 'HighBitOfByte SmallPrimes'
	poolDictionaries: ''
	category: 'System-Hashing-DSA'!

!DigitalSignatureAlgorithm commentStamp: '<historical>' prior: 0!
This class implements the Digital Signature Algorithm (DSA) of the U.S. government's "Digital Signature Standard" (DSS). The DSA algorithm was proposed in 1991 and became a standard in May 1994. The official description is available as a Federal Information Processing Standards Publication (FIPS PUB 186, May 19, 1994). A companion standard, the Secure Hash Standard, or SHS (FIPS PUB 180-1, April 17, 1995), describes a 160-bit message digest algorithm known as the Secure Hash Algorithm (SHA). This message digest is used to compute the document signature.

Here's how to use it:

  1. The "signer" creates a pair of keys. One of these must be kept private. The other may be freely distributed. For example, it could be built into the signature checking code of an application.

  2. When the signer wishes to sign a packet of data (a "message") , he uses the secure hash algorithm to create a 160-bit message digest (hash) which is used as the input to DSA. The result of this is a pair of large numbers called a "signature" that is attached to the original message.

  3. When someone receives a signed message purported to have come from the signer, they compute the 160-bit hash of the message and pass that, along with the message signature and the signer's public key, to the signature verification algorithm. If the signature checks, then it is virtually guaranteed that the message originated from someone who had the signer's private key. That is, the message is not a forgery and has not been modified since it was signed. For example, if the message contains a program, and the recipient trusts the signer, then the recipient can run the program with the assurance that it won't do anything harmful. (At least, not intentionally. A digital signature is no guarantee against bugs!! :->)

The signer must keep the private key secure, since anyone who has the private key can forge the signer's signature on any message they like. As long as the secret key is not stolen, cryptographers believe it to be virtually impossible either to forge a signature, to find a message that matches an existing sigature, or to discover the signer's private key by analyzing message signatures. Knowing the public key (which, for example, could be recovered from an application that had it built in), does not weaken the security at all.

An excellent reference work on digital signatures and cryptography in general is:

  Schneier, Bruce
  "Applied Cryptography: Protocols, Algorithms, and Source Code in C"
  John Wiley and Sons, 1996.

I used this book as a guide to implementing many of the numerical algorithms required by DSA.

Patents and Export Restrictions:

Many digital signature technologies are patented. DSA is also patented, but the patent is owned by the U.S. government which has made DSA available royalty-free. There is a claim that the government patent infringes on an earlier patent by Schnorr, but the government is requiring the use of DSA, so they apparently believe this claim is not strong enough to be a serious threat to their own patent.

Most cryptography technology, including digital signature technology, requires an export license for it to be distributed outside the U.S. Recent legislation may have relaxed the export license requirements, but it would be prudent to check the current regulations before exporting this code.!

!DigitalSignatureAlgorithm class methodsFor: 'examples' stamp: 'jm 12/22/1999 11:23'!
example
	"Example of signing a message and verifying its signature."
	"Note: Secure random numbers are needed for key generation and message signing, but not for signature verification. There is no need to call initRandomFromUser if you are merely checking a signature."
	"DigitalSignatureAlgorithm example"

	| msg keys sig |
	msg := 'This is a test...'.
	keys := self testKeySet.
	sig := self sign: msg privateKey: keys first.
	self inform: 'Signature created'.
	(self verify: sig isSignatureOf: msg publicKey: keys last)
		ifTrue: [self inform: 'Signature verified.']
		ifFalse: [self error: 'ERROR!! Signature verification failed'].
! !

!DigitalSignatureAlgorithm class methodsFor: 'public' stamp: 'md 3/26/2011 19:15'!
generateKeySet
 	"Generate and answer a key set for code signing. The result is a pair (<private key><public key>). Each key is an array of four large integers. The signer must be sure to record this keys set and must keep the private key secret to prevent someone from forging their signature."
 	"Note: Key generation can take some time. Open a transcript so you can see what's happening and take a coffee break!!"
 	"Note: Unguessable random numbers are needed for key generation. The user will be prompted to type a really long random string (two or three lines) to initialize the random number generator before generating a key set. A different random string should be typed for every session; it is not a password and we wish to produce different random number streams."
 	"DigitalSignatureAlgorithm generateKeySet"
 
 	| dsa |
 	dsa := self new.
 	(self confirm: 'Shall I seed the random generator from the current sound input?')
 		ifTrue: [dsa initRandomNonInteractively]
 		ifFalse: [dsa initRandomFromUser].
 	^ dsa generateKeySet
 ! !

!DigitalSignatureAlgorithm class methodsFor: 'initialization' stamp: 'NorbertHartl 6/13/2008 11:38'!
initialize
	"DigitalSignatureAlgorithm initialize"

	"SmallPrimes is a list of small primes greater than two."
	SmallPrimes := Integer primesUpTo: 2000.
	SmallPrimes := SmallPrimes copyFrom: 2 to: SmallPrimes size.

	"HighBitOfByte maps a byte to the index of its top non-zero bit."
	HighBitOfByte := (0 to: 255) collect: [:byte | byte highBit].
! !

!DigitalSignatureAlgorithm class methodsFor: 'public' stamp: 'md 3/26/2011 19:16'!
sign: aStringOrStream privateKey: privateKey 
	"Sign the given message (a stream or string) and answer a signature string."
	"Note: Unguessable random numbers are needed for message signing. The user will be prompted to type a really long random string (two or three lines) to initialize the random number generator before signing a message. A different random string should be typed for every session; it is not a password and we wish to produce different random number streams."
	| dsa hasher h sig |
	dsa := self new.
	dsa initRandomFromUser.
	hasher := SecureHashAlgorithm new.
	h := aStringOrStream class isBytes 
		ifTrue: [ hasher hashMessage: aStringOrStream ]
		ifFalse: [ hasher hashStream: aStringOrStream ].
	sig := dsa 
		computeSignatureForMessageHash: h
		privateKey: privateKey.
	^ dsa signatureToString: sig! !

!DigitalSignatureAlgorithm class methodsFor: 'public' stamp: 'stephane.ducasse 5/25/2008 15:23'!
sign: aStringOrStream privateKey: privateKey dsa: dsa 
	"Sign the given message (a stream or string) and answer a signature string."
	"Note: Unguessable random numbers are needed for message signing. The user will be prompted to type a really long random string (two or three lines) to initialize the random number generator before signing a message. A different random string should be typed for every session; it is not a password and we wish to produce different random number streams."
	| hasher h sig |
	hasher := SecureHashAlgorithm new.
	h := aStringOrStream class isBytes 
		ifTrue: [ hasher hashMessage: aStringOrStream ]
		ifFalse: [ hasher hashStream: aStringOrStream ].
	sig := dsa 
		computeSignatureForMessageHash: h
		privateKey: privateKey.
	^ dsa signatureToString: sig! !

!DigitalSignatureAlgorithm class methodsFor: 'examples' stamp: 'jb 7/1/2011 10:51'!
testExamplesFromDisk
	"verify messages from file on disk"
	"Note: Secure random numbers are needed for key generation and message signing, but not for signature verification. There is no need to call initRandomFromUser if you are merely checking a signature."
	"DigitalSignatureAlgorithm testExamplesFromDisk"

	FileStream 
		readOnlyFileNamed: 'dsa.test.out' 
		do: [ :file |
			| msg sig publicKey |
			[file atEnd] whileFalse: [
				sig := file nextChunk.
				msg := file nextChunk.
				publicKey := self class evaluatorClass evaluate: file nextChunk.
				(self verify: sig isSignatureOf: msg publicKey: publicKey) 
					ifTrue: [Transcript show: 'SUCCESS: ',msg; cr.]
					ifFalse: [self error: 'ERROR!! Signature verification failed']]]! !

!DigitalSignatureAlgorithm class methodsFor: 'examples' stamp: 'jm 12/22/1999 11:28'!
testKeySet
	"Answer a pair of keys for testing. The first key is the private key, the second one is the public key."
	"WARNING: This test key set is public should be used only for testing!! In a real application, the user would create a set of keys using generateKeySet and would keep the private key secret."

	^ #(
		(8343811888543852523216773185009428259187948644369498021763210776677854991854533186365944349987509452133156416880596803846631577352387751880552969116768071 1197175832754339660404549606408619548226315875117 1433467472198821951822151391684734233265646022897503720591270330985699984763922266163182803556189497900262038518780931942996381297743579119123094520048965 957348690772296812)
		(8343811888543852523216773185009428259187948644369498021763210776677854991854533186365944349987509452133156416880596803846631577352387751880552969116768071 1197175832754339660404549606408619548226315875117 1433467472198821951822151391684734233265646022897503720591270330985699984763922266163182803556189497900262038518780931942996381297743579119123094520048965 4645213122572190617807944614677917601101008235397095646475699959851618402406173485853587185431290863173614335452934961425661774118334228449202337038283799))
! !

!DigitalSignatureAlgorithm class methodsFor: 'testing' stamp: 'RAA 5/31/2000 08:21'!
time: aBlock as: aString count: anInteger

	^{anInteger. aString. (Time millisecondsToRun: aBlock)}! !

!DigitalSignatureAlgorithm class methodsFor: 'examples' stamp: 'md 3/26/2011 19:16'!
timeDecode: count
	"Example of signing a message and verifying its signature."
	"Note: Secure random numbers are needed for key generation and message signing, but not for signature verification. There is no need to call initRandomFromUser if you are merely checking a signature."
	"DigitalSignatureAlgorithm timeDecode: 20"

	| dsa |

	dsa := self new.
	dsa initRandomFromUser.

	#(1 10 100 1000 10000 100000) do: [ :extraLen | | s msg keys sig |
		s := String new: extraLen.
		1 to: s size do: [ :i | s at: i put: (Character value: 200 atRandom)].
		msg := 'This is a test...',s.
		keys := self testKeySet.
		sig := self sign: msg privateKey: keys first dsa: dsa.
		"self inform: 'Signature created'."
		self timeDirect: [
			count timesRepeat: [
				(self verify: sig isSignatureOf: msg publicKey: keys last)
					ifFalse: [self error: 'ERROR!! Signature verification failed'].
			].
		] as: 'verify msgLen = ',msg size printString count: count
	].
! !

!DigitalSignatureAlgorithm class methodsFor: 'testing' stamp: 'RAA 5/31/2000 08:40'!
timeDirect: aBlock as: aString count: anInteger

	Transcript show: anInteger asStringWithCommas,'  ',
		aString ,' took ',
		(Time millisecondsToRun: aBlock) asStringWithCommas,' ms'; cr
! !

!DigitalSignatureAlgorithm class methodsFor: 'public' stamp: 'md 3/26/2011 19:16'!
verify: signatureString isSignatureOf: aStringOrStream publicKey: publicKey 
	"Answer true if the given signature string signs the given message (a stream or string)."
	"Note: Random numbers are not needed for signature verification; thus, there is no need to call initRandomFromUser before verifying a signature."
	| dsa hasher h sig |
	dsa := self new.
	hasher := SecureHashAlgorithm new.
	h := aStringOrStream class isBytes 
		ifTrue: [ hasher hashMessage: aStringOrStream ]
		ifFalse: [ hasher hashStream: aStringOrStream ].
	sig := dsa stringToSignature: signatureString.
	^ dsa 
		verifySignature: sig
		ofMessageHash: h
		publicKey: publicKey! !

!DigitalSignatureAlgorithm class methodsFor: 'examples' stamp: 'md 3/26/2011 19:16'!
writeExamplesToDisk
	"Example of signing a message and verifying its signature. Used to create samples from one implementation that could later be tested with a different implementation"
	"Note: Secure random numbers are needed for key generation and message signing, but not for signature verification. There is no need to call initRandomFromUser if you are merely checking a signature."
	"DigitalSignatureAlgorithm writeExamplesToDisk"

	| file keyList dsa msgList |

	dsa := self new.
	dsa initRandomFromUser.
	self inform: 'About to generate 5 key sets. Will take a while'.
	keyList := {self testKeySet},((1 to: 5) collect: [ :ignore | self generateKeySet]).
	msgList := {'This is a test...'. 'This is the second test period.'. 'And finally, a third message'}.
	file := FileStream newFileNamed: 'dsa.test.out'.
	[
		msgList do: [ :msg |
			keyList do: [ :keys | | sig |
				sig := self sign: msg privateKey: keys first dsa: dsa.
				(self verify: sig isSignatureOf: msg publicKey: keys last) ifTrue: [
					file
						nextChunkPut: sig;
						nextChunkPut: msg;
						nextChunkPut: keys last storeString.
				] ifFalse: [
					self error: 'ERROR!! Signature verification failed'
				].
			].
		].
	] ensure: [file close]
! !

!DigitalSignatureAlgorithm methodsFor: 'public' stamp: 'nice 9/2/2010 21:48'!
computeSignatureForMessageHash: hash privateKey: privateKey
	"Answer the digital signature of the given message hash using the given private key. A signature is a pair of large integers. The private key is an array of four large integers: (p, q, g, x)."

	| p q g x r s k tmp |
	p := privateKey first.
	q := privateKey second.
	g := privateKey third.
	x := privateKey fourth.

	r := s := 0.
	[r = 0 or: [s = 0]] whileTrue: [
		k := self nextRandom160 \\ q.
		r := (g raisedTo: k modulo: p) \\ q.
		tmp := (hash + (x * r)) \\ q.
		s := ((k reciprocalModulo: q) * tmp) \\ q].

	^ Array with: r with: s
! !

!DigitalSignatureAlgorithm methodsFor: 'public' stamp: 'adrian-lienhard 5/18/2009 21:08'!
generateKeySet
	"Generate and answer a key set for DSA. The result is a pair (<private key><public key>). Each key is an array of four large integers. The private key is (p, q, g, x); the public one is (p, q, g, y). The signer must be sure to record (p, q, g, x), and must keep x secret to prevent someone from forging their signature."
	"Note: Key generation can take some time. Open a transcript so you can see what's happening and take a coffee break!!"

	| qAndPandS q p exp g h x y |
	qAndPandS := self generateQandP.
	Transcript show: 'Computing g...'.
	q := qAndPandS first.
	p := qAndPandS second.
	exp := (p - 1) / q.
	h := 2.
	[g := h raisedTo: exp modulo: p. g = 1] whileTrue: [h := h + 1].
	Transcript show: 'done.'; cr.
	Transcript show: 'Computing x and y...'.
	x := self nextRandom160.
	y := g raisedTo: x modulo: p.
	Transcript show: 'done.'; cr.
	Transcript show: 'Key generation complete!!'; cr.
	^ Array
		with: (Array with: p with: q with: g with: x)
		with: (Array with: p with: q with: g with: y).
! !

!DigitalSignatureAlgorithm methodsFor: 'private' stamp: 'raa 5/30/2000 15:47'!
generateQandP
	"Generate the two industrial-grade primes, q (160-bits) and p (512-bit) needed to build a key set. Answer the array (q, p, s), where s is the seed that from which q and p were created. This seed is normally discarded, but can be used to verify the key generation process if desired."

	| pBits halfTwoToTheP chunkCount sAndq q twoQ n c w x p s |
	pBits := 512.  "desired size of p in bits"
	halfTwoToTheP := 2 raisedTo: (pBits - 1).
	chunkCount := pBits // 160.

	Transcript show: 'Searching for primes q and p...'; cr.
	[true] whileTrue: [
		sAndq := self generateSandQ.
		Transcript show: '  Found a candidate q.'; cr.
		s := sAndq first.
		q := sAndq last.
		twoQ := q bitShift: 1.
		n := 2.
		c := 0.
		[c < 4096] whileTrue: [
			w := self generateRandomLength: pBits s: s n: n.
			x := w + halfTwoToTheP.
			p := (x - ( x \\ twoQ)) + 1.
			p highBit = pBits ifTrue: [
				Transcript show: '    Testing potential p ', (c + 1) printString, '...'; cr.
				(self isProbablyPrime: p) ifTrue: [
					Transcript show: '  Found p!!'; cr.
					^ Array with: q with: p with: s]].
			n := n + chunkCount + 1.
			c := c + 1]].
! !

!DigitalSignatureAlgorithm methodsFor: 'private' stamp: 'jm 12/13/1999 16:36'!
generateRandomLength: bitLength s: s n: n
	"Answer a random number of bitLength bits generated using the secure hash algorithm."

	| sha out count extraBits v |
	sha := SecureHashAlgorithm new.
	out := 0.
	count := (bitLength // 160).
	extraBits := bitLength - (count * 160).
	0 to: count do: [:k |
		v := sha hashInteger: (s + n + k).
		k = count ifTrue: [
			v := v - ((v >> extraBits) << extraBits)].
		out := out bitOr: (v bitShift: (160 * k))].
	^ out
! !

!DigitalSignatureAlgorithm methodsFor: 'private' stamp: 'raa 5/30/2000 15:46'!
generateSandQ
	"Generate a 160-bit random seed s and an industrial grade prime q."

	| hasher s sPlusOne u q |
	hasher := SecureHashAlgorithm new.
	[true] whileTrue: [
		s := self nextRandom160.
		sPlusOne := s + 1.
		sPlusOne highBit > 160 ifTrue: [sPlusOne := sPlusOne \\ (2 raisedTo: 160)].
		u := (hasher hashInteger: s) bitXor: (hasher hashInteger: sPlusOne).
		q := u bitOr: ((1 bitShift: 159) bitOr: 1).
		(self isProbablyPrime: q) ifTrue: [^ Array with: s with: q]].
! !

!DigitalSignatureAlgorithm methodsFor: 'initialization' stamp: 'jm 1/11/2000 00:25'!
initRandom: randomInteger
	"Initialize the the secure random number generator with the given value. The argument should be a positive integer of up to 512 bits chosen randomly to avoid someone being able to predict the sequence of random values generated."
	"Note: The random generator must be initialized before generating a key set or signature. Signature verification does not require initialization of the random generator."

	randSeed := 16rEFCDAB8998BADCFE10325476C3D2E1F067452301.  "initial seed"
	randKey := randomInteger.
	Transcript show: 'Random seed: ', randomInteger printString; cr.
! !

!DigitalSignatureAlgorithm methodsFor: 'initialization' stamp: 'nice 12/14/2010 15:10'!
initRandomFromString: aString
	"Ask the user to type a long random string and use the result to seed the secure random number generator."

	| s k srcIndex |
	s := aString.
	k := LargePositiveInteger new: (s size min: 64).
	srcIndex := 0.
	k digitLength to: 1 by: -1 do: [:i |
		k digitAt: i put: (s at: (srcIndex := srcIndex + 1)) asciiValue].
	k := k normalize + (Random new next * 16r7FFFFFFF) asInteger.  "a few additional bits randomness"
	k highBit > 512 ifTrue: [k := k bitShift: k highBit - 512].
	self initRandom: k.
! !

!DigitalSignatureAlgorithm methodsFor: 'initialization' stamp: 'nice 4/1/2011 21:19'!
initRandomFromUser
	"Ask the user to type a long random string and use the result to seed the secure random number generator."

	| s |
	s := UIManager default request: 'Enter a long random string to seed the random generator.'.
	s isNil ifTrue: [s := '']. 
	^self initRandomFromString: s! !

!DigitalSignatureAlgorithm methodsFor: 'initialization' stamp: 'simon.denier 6/11/2010 17:35'!
initRandomNonInteractively
 	[self initRandom: (SoundService default randomBitsFromSoundInput: 512)]
 		ifError: [self initRandomFromString: 
 			Time millisecondClockValue printString, 
 			Date today printString, 
 			OSPlatform platformName printString].! !

!DigitalSignatureAlgorithm methodsFor: 'large integer arithmetic' stamp: 'nice 8/28/2010 21:16'!
inverseOf: x mod: n
	"Answer the inverse of x modulus n. That is, the integer y such that (x * y) \\ n is 1. Both x and n must be positive, and it is assumed that x < n and that x and n are integers."
	"Details: Use the extended Euclidean algorithm, Schneier, p. 247."

	| v u u1 u2 u3 t1 t2 t3 tmp |
	((x <= 0) or: [n <= 0]) ifTrue: [self error: 'x and n must be greater than zero'].
	x >= n ifTrue: [self error: 'x must be < n'].

	v := x.
	u := n.
	(x even and: [n even]) ifTrue: [self error: 'no inverse'].

	u1 := 1. u2 := 0. u3 := u.
	t1 := v. t2 := u - 1. t3 := v.
	[	[u3 even ifTrue: [
			((u1 odd) or: [u2 odd]) ifTrue: [
				u1 := u1 + v.
				u2 := u2 + u].
			u1 := u1 bitShift: -1.
			u2 := u2 bitShift: -1.
			u3 := u3 bitShift: -1].
		((t3 even) or: [u3 < t3]) ifTrue: [
			tmp := u1. u1 := t1. t1 := tmp.
			tmp := u2. u2 := t2. t2 := tmp.
			tmp := u3. u3 := t3. t3 := tmp].
		u3 even and: [u3 > 0]] whileTrue: ["loop while u3 is even"].

		[((u1 < t1) or: [u2 < t2]) and: [u1 > 0]] whileTrue: [
			u1 := u1 + v.
			u2 := u2 + u].
	
		u1 := u1 - t1.
		u2 := u2 - t2.
		u3 := u3 - t3.
		t3 > 0] whileTrue: ["loop while t3 > 0"].

	[u1 >= v and: [u2 >= u]] whileTrue: [
		u1 := u1 - v.
		u2 := u2 - u].

	u3 = 1 ifFalse: [self error: 'no inverse'].
	^ u - u2
! !

!DigitalSignatureAlgorithm methodsFor: 'large integer arithmetic' stamp: 'nice 8/28/2010 21:21'!
isProbablyPrime: p
	"Answer true if p is prime with very high probability. Such a number is sometimes called an 'industrial grade prime'--a large number that is so extremely likely to be prime that it can assumed that it actually is prime for all practical purposes. This implementation uses the Rabin-Miller algorithm (Schneier, p. 159)."

	| iterations factor pMinusOne b m r a j z couldBePrime |
	iterations := 50.  "Note: The DSA spec requires >50 iterations; Schneier says 5 are enough (p. 260)"

	"quick elimination: check for p divisible by a small prime"
	SmallPrimes ifNil: [  "generate list of small primes > 2"
		SmallPrimes := Integer primesUpTo: 2000.
		SmallPrimes := SmallPrimes copyFrom: 2 to: SmallPrimes size].
	factor := SmallPrimes detect: [:f | (p \\ f) = 0] ifNone: [nil].
	factor ifNotNil: [^ p = factor].

	pMinusOne := p - 1.
	b := self logOfLargestPowerOfTwoDividing: pMinusOne.
	m := pMinusOne bitShift: b negated.
	"Assert: pMinusOne = m * (2 raisedTo: b) and m is odd"

	Transcript show: '      Prime test pass '.
	r := Random new.
	1 to: iterations do: [:i |
		Transcript show: i printString; space.
		a := (r next * 16rFFFFFF) truncated.
		j := 0.
		z := (a raisedTo: m modulo: p) normalize.
		couldBePrime := z = 1.
		[couldBePrime] whileFalse: [
			z = 1 ifTrue: [Transcript show: 'failed!!'; cr. ^ false].  "not prime"
			z = pMinusOne
				ifTrue: [couldBePrime := true]
				ifFalse: [
					(j := j + 1) < b
						ifTrue: [z := (z * z) \\ p]
						ifFalse: [Transcript show: 'failed!!'; cr. ^ false]]]].  "not prime"

	Transcript show: 'passed!!'; cr.
	^ true  "passed all tests; probably prime"
! !

!DigitalSignatureAlgorithm methodsFor: 'private' stamp: 'nice 8/28/2010 21:04'!
logOfLargestPowerOfTwoDividing: aPositiveInteger
	"Answer the base-2 log of the largest power of two that divides the given integer. For example, the largest power of two that divides 24 is 8, whose log base-2 is 3. Do this efficiently even when the given number is a large integer. Assume that the given integer is > 0."
	"DigitalSignatureAlgorithm new logOfLargestPowerOfTwoDividing: (32 * 3)"

	^aPositiveInteger lowBit - 1! !

!DigitalSignatureAlgorithm methodsFor: 'private' stamp: 'jm 12/13/1999 14:39'!
nextRandom160
	"Answer a newly generated 160-bit random number in the range [1..(2^160 - 1)]."
	"Details: Try again in the extremely unlikely chance that zero is encountered."

	| result |
	result := 0.
	[result = 0] whileTrue: [
		result := SecureHashAlgorithm new hashInteger: randKey seed: randSeed.
		randKey := randKey + result + 1].
	^ result
! !

!DigitalSignatureAlgorithm methodsFor: 'public' stamp: 'PeterHugossonMiller 9/3/2009 01:13'!
signatureToString: aSignature
	"Answer a string representation of the given signature. This string can be parsed using the stringToSignature: method."

	| s |
	s := (String new: 2000) writeStream.
	s nextPutAll: '[DSA digital signature '.
	s nextPutAll: aSignature first printStringHex.
	s space.
	s nextPutAll: aSignature second printStringHex.
	s nextPutAll: ']'.
	^ s contents
! !

!DigitalSignatureAlgorithm methodsFor: 'public' stamp: 'dc 5/30/2008 10:17'!
stringToSignature: aString 
	"Answer the signature stored in the given string. A signature string has the format:

		 '[DSA digital signature <r> <s>]'

	where <r> and <s> are large positive integers represented by strings of hexidecimal digits."
	| prefix stream r s |
	prefix := '[DSA digital signature '.
	(aString beginsWith: prefix) ifFalse: [ self error: 'bad signature prefix' ].
	stream := aString readStream.
	stream position: prefix size.
	r := Integer 
		readFrom: stream
		base: 16.
	stream next.
	s := Integer 
		readFrom: stream
		base: 16.
	^ Array 
		with: r
		with: s! !

!DigitalSignatureAlgorithm methodsFor: 'public' stamp: 'nice 9/2/2010 21:48'!
verifySignature: aSignature ofMessageHash: hash publicKey: publicKey
	"Answer true if the given signature is the authentic signature of the given message hash. That is, if the signature must have been computed using the private key set corresponding to the given public key. The public key is an array of four large integers: (p, q, g, y)."

	| p q g y r s w u1 u2 v0 v |
	p := publicKey first.
	q := publicKey second.
	g := publicKey third.
	y := publicKey fourth.
	r := aSignature first.
	s := aSignature last.
	((r > 0) and: [r < q]) ifFalse: [^ false].  "reject"
	((s > 0) and: [s < q]) ifFalse: [^ false].  "reject"

	w := s reciprocalModulo: q.
	u1 := (hash * w) \\ q.
	u2 := (r * w) \\ q.
	v0 := (g raisedTo: u1 modulo: p) * (y raisedTo: u2 modulo: p).
	v := ( v0 \\ p) \\ q.
	^ v = r
! !

Object subclass: #EventManager
	instanceVariableNames: 'actionMap'
	classVariableNames: 'ActionMaps'
	poolDictionaries: ''
	category: 'System-Object Events'!

!EventManager commentStamp: 'tlk 5/7/2006 20:01' prior: 0!
An EventManager is used to registers a 'observer' object's interest in in changes to an 'observed' object.  Then when the observered object is changed,  EventManager broadcasts the an update message to all objects with a registered interest.  Finally, the Event manager can be used to remove an object from the list of observer object.

An interested object is said to be a dependant on the target object.  Registering an interest in an event is called adding a dependant. Deregistering is called removing  a dependant.  The EventManager's action map is a WeakIdentityDictionary that maps events (selectors) to dependants (objects & selectors) in a way that ensures the mapping is to specific objects (hence identity) and in a way that allows the object to be garbage collected if not other used (hence weak.)  EventManager class has ActionMaps which has one actionMap for each object.

Classic uses of an EventManager are to implement the Observer Pattern, see ChangeNotification or the MorphicModle as examples.!

!EventManager class methodsFor: 'accessing' stamp: 'reThink 2/18/2001 14:42'!
actionMapFor: anObject

    ^self actionMaps
        at: anObject
        ifAbsent: [self createActionMap]! !

!EventManager class methodsFor: 'accessing' stamp: 'nice 4/19/2011 00:02'!
actionMaps

	^ActionMaps ifNil: [ActionMaps := WeakIdentityKeyDictionary new]! !

!EventManager class methodsFor: 'cleanup' stamp: 'StephaneDucasse 3/9/2010 22:22'!
cleanUp: aggressive
	"Dump all ActionMaps but only when we're aggressively cleaning"

	aggressive ifTrue: [ActionMaps := nil].! !

!EventManager class methodsFor: 'initialization' stamp: 'nice 1/5/2010 15:59'!
flushEvents
	"Object flushEvents"
	
	self actionMaps keysAndValuesDo:[:rcvr :evtDict| rcvr ifNotNil:[
		"make sure we don't modify evtDict while enumerating"
		evtDict keys do:[:evtName| | msgSet |
			msgSet := evtDict at: evtName ifAbsent:[nil].
			(msgSet == nil) ifTrue:[rcvr removeActionsForEvent: evtName]]]].
	EventManager actionMaps finalizeValues. ! !

!EventManager class methodsFor: 'releasing' stamp: 'reThink 2/18/2001 15:34'!
releaseActionMapFor: anObject

	self actionMaps
		removeKey: anObject
		ifAbsent: []! !

!EventManager class methodsFor: 'accessing' stamp: 'reThink 2/25/2001 08:52'!
updateableActionMapFor: anObject

    ^self actionMaps
        at: anObject
        ifAbsentPut: [self createActionMap]! !

!EventManager methodsFor: 'accessing' stamp: 'reThink 2/18/2001 15:37'!
actionMap

    ^actionMap == nil
        ifTrue: [self createActionMap]
        ifFalse: [actionMap]! !

!EventManager methodsFor: 'dependents access' stamp: 'reThink 3/3/2001 10:07'!
addDependent: anObject
	"Make the given object one of the receiver's dependents."

	self
		when: self changedEventSelector
		send: self updateEventSelector
		to: anObject.
	^anObject! !

!EventManager methodsFor: 'dependents access' stamp: 'reThink 3/3/2001 10:07'!
breakDependents
	"Remove all of the receiver's dependents."

	self removeActionsForEvent: self changedEventSelector! !

!EventManager methodsFor: 'updating' stamp: 'reThink 3/3/2001 10:20'!
changed: aParameter 
	"Receiver changed. The change is denoted by the argument aParameter. 
	Usually the argument is a Symbol that is part of the dependent's change 
	protocol. Inform all of the dependents."

	self 
		triggerEvent: self changedEventSelector
		with: aParameter! !

!EventManager methodsFor: 'accessing' stamp: 'reThink 3/3/2001 10:07'!
changedEventSelector

	^#changed:! !

!EventManager methodsFor: 'dependents access' stamp: 'reThink 3/3/2001 10:18'!
dependents

	^(self actionSequenceForEvent: self changedEventSelector) asSet
		collect:
			[:each | each receiver]! !

!EventManager methodsFor: 'copying' stamp: 'StephaneDucasse 10/2/2010 17:00'!
postCopy

	super postCopy.
	self release! !

!EventManager methodsFor: 'accessing' stamp: 'reThink 2/18/2001 15:39'!
releaseActionMap

    actionMap := nil! !

!EventManager methodsFor: 'dependents access' stamp: 'reThink 3/3/2001 10:07'!
removeDependent: anObject
	"Remove the given object as one of the receiver's dependents."

	self 
		removeActionsWithReceiver: anObject
		forEvent: self changedEventSelector.
	^ anObject! !

!EventManager methodsFor: 'accessing' stamp: 'reThink 3/3/2001 10:07'!
updateEventSelector

	^#update:! !

!EventManager methodsFor: 'accessing' stamp: 'reThink 2/18/2001 15:38'!
updateableActionMap

    actionMap == nil
        ifTrue: [actionMap := self createActionMap].
    ^actionMap! !

Object subclass: #ExternalDropHandler
	instanceVariableNames: 'action type extension'
	classVariableNames: 'DefaultHandler RegisteredHandlers'
	poolDictionaries: ''
	category: 'System-Support'!

!ExternalDropHandler class methodsFor: 'private' stamp: ''!
chooseServiceFrom: aCollection
	"private - choose a service from aCollection asking the user if  
	needed"
	aCollection size = 1 ifTrue: [^ aCollection anyOne].
	^ UIManager default chooseFrom: (aCollection collect: [:each | 
			each label]) values: aCollection.
! !

!ExternalDropHandler class methodsFor: 'private' stamp: ''!
defaultGZipHandler
	^ExternalDropHandler
		type: nil
		extension: 'gz'
		action: [:stream :pasteUp :event |
			stream viewGZipContents]! !

!ExternalDropHandler class methodsFor: 'accessing' stamp: ''!
defaultHandler
	DefaultHandler ifNil: [DefaultHandler := ExternalDropHandler type: nil extension: nil action: [:dropStream | dropStream edit]].
	^DefaultHandler! !

!ExternalDropHandler class methodsFor: 'accessing' stamp: ''!
defaultHandler: externalDropHandler
	DefaultHandler := externalDropHandler! !

!ExternalDropHandler class methodsFor: 'private' stamp: ''!
defaultImageHandler
	
	^ExternalDropHandler
		type: 'image/'
		extension: nil
		action: [:stream :pasteUp :event | | image sketch |
			stream binary.
			image := Form fromBinaryStream: ((RWBinaryOrTextStream with: stream contents) reset).
			sketch := World drawingClass withForm: image.
			pasteUp addMorph: sketch centeredNear: event position.
			image := sketch := nil]! !

!ExternalDropHandler class methodsFor: 'initialization' stamp: ''!
initialize
	"ExternalDropHandler initialize"

	self resetRegisteredHandlers.
	self
		registerHandler: self defaultImageHandler;
		registerHandler: self defaultGZipHandler.
	! !

!ExternalDropHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/4/2012 21:26'!
lookupExternalDropHandler: stream

	| types extension serviceHandler |
	types := stream mimeTypes.

	types ifNotNil: [
		self registeredHandlers do: [:handler | 
			(handler matchesTypes: types)
				ifTrue: [^handler]]].

	extension := stream name asFileReference extension.
	self registeredHandlers do: [:handler | 
		(handler matchesExtension: extension)
				ifTrue: [^handler]].
	serviceHandler := self lookupServiceBasedHandler: stream.
	^serviceHandler
		ifNil: [self defaultHandler]! !

!ExternalDropHandler class methodsFor: 'accessing' stamp: 'CamilloBruni 5/23/2012 16:55'!
lookupServiceBasedHandler: dropStream
	"the file was just droped, let's do our job"
	| fileName services theOne |
	fileName := dropStream name.

	services := (Smalltalk tools fileList itemsForFile: fileName asFileReference)
		reject: [:svc | self unwantedSelectors includes: svc selector].

	"no service, default behavior"
	services isEmpty
		ifTrue: [^nil].

	theOne := self chooseServiceFrom: services.
	^theOne
		ifNotNil: [ExternalDropHandler type: nil extension: nil action: [:stream | theOne performServiceFor: stream]]! !

!ExternalDropHandler class methodsFor: 'accessing' stamp: ''!
registerHandler: aHandler
	self registeredHandlers add: aHandler! !

!ExternalDropHandler class methodsFor: 'initialization' stamp: ''!
registerStandardExternalDropHandlers
	"ExternalDropHandler registerStandardExternalDropHandlers"

	self registeredHandlers add: (
		ExternalDropHandler
			type: 'image/'
			extension: nil
			action: [:stream :pasteUp :event |
				pasteUp addMorph: (World drawingClass withForm: (Form fromBinaryStream: stream binary)) centeredNear: event position])! !

!ExternalDropHandler class methodsFor: 'private' stamp: ''!
registeredHandlers
	RegisteredHandlers ifNil: [RegisteredHandlers := OrderedCollection new].
	^RegisteredHandlers! !

!ExternalDropHandler class methodsFor: 'private' stamp: ''!
resetRegisteredHandlers
	RegisteredHandlers := nil! !

!ExternalDropHandler class methodsFor: 'instance creation' stamp: ''!
type: aType extension: anExtension action: anAction 
	^self new type: aType extension: anExtension action: anAction ! !

!ExternalDropHandler class methodsFor: 'private' stamp: ''!
unwantedSelectors
	"private - answer a collection well known unwanted selectors "
	^ #(#removeLineFeeds: #addFileToNewZip: #compressFile: #putUpdate: )! !

!ExternalDropHandler methodsFor: 'accessing' stamp: ''!
extension
	^extension! !

!ExternalDropHandler methodsFor: 'accessing' stamp: ''!
handle: dropStream in: pasteUp dropEvent: anEvent
	
	^action cull: dropStream 
			cull: pasteUp 
			cull: anEvent
	! !

!ExternalDropHandler methodsFor: 'testing' stamp: 'CamilloBruni 5/7/2012 01:19'!
matchesExtension: aExtension
	(self extension isNil or: [aExtension isNil])
		ifTrue: [^false].
	FileSystem disk isCaseSensitive
		ifTrue: [^extension = aExtension]
		ifFalse: [^extension sameAs: aExtension]! !

!ExternalDropHandler methodsFor: 'testing' stamp: ''!
matchesTypes: types
	(self type isNil or: [types isNil])
		ifTrue: [^false].
	^types anySatisfy: [:mimeType | mimeType beginsWith: self type]! !

!ExternalDropHandler methodsFor: 'accessing' stamp: ''!
type
	^type! !

!ExternalDropHandler methodsFor: 'initialize' stamp: ''!
type: aType extension: anExtension action: anAction 
	action := anAction.
	type := aType.
	extension := anExtension! !

Object subclass: #ExternalSemaphoreTable
	instanceVariableNames: ''
	classVariableNames: 'ProtectAdd ProtectRemove'
	poolDictionaries: ''
	category: 'System-Support'!

!ExternalSemaphoreTable commentStamp: 'HenrikSperreJohansen 8/18/2011 11:25' prior: 0!
By John M McIntosh johnmci@smalltalkconsulting.com
This class was written to mange the external semaphore table. When I was writing a Socket test server I discovered various race conditions on the access to the externalSemaphore table. This new class uses class side methods to restrict access using two mutex semaphores, one for removal and one for additions to the table. It seemed cleaner to deligate the reponsibility here versus adding more code and another class variable to SystemDictionary 

Note that in Smalltalk recreateSpecialObjectsArray we still directly play with the table.

Henrik Sperre Johansen
The name is somewhat of a misnomer; the table can be used for any objects, not just semaphores.
That is its main usage though, so a split which deals with semaphores and other external objects differently 
(In the same underlying table) is not currently worth it.
Therefore, while in general not all users will care if the table is above a certain size, we still guard  against adding more objects than the limit above which external signals would be lost (on some VMs.)
!

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 8/18/2011 10:18'!
clearExternalObjects
	"Clear the array of objects that have been registered for use in non-Smalltalk code."
	"Only lock additions, removals executing in parallel would have little effect on the resulting array"

	ProtectAdd critical: [
		self unprotectedExternalObjects: Array new].
! !

!ExternalSemaphoreTable class methodsFor: 'private' stamp: 'HenrikSperreJohansen 8/18/2011 10:33'!
collectionBasedOn: externalObjects withRoomFor: anObject
	"Called if no slots to put anObject in have been found in externalObjects "
	"Return a externalObject collection which does, either:
		- Same collection with some slots freed up by finalization logic
		- A larger array, which has replaced the parameter as canonical externalObject array.
	An error must be raised if this method is incapable of fulfilling its duties"
	
	| newObjects newSize |
	"grow linearly"
	newSize :=  externalObjects size +20.
	(self freedSlotsIn: externalObjects ratherThanIncreaseSizeTo: newSize) 
		ifTrue: [newObjects := externalObjects]
		ifFalse: [
			newObjects := externalObjects species new: newSize. 
			newObjects replaceFrom: 1 to:  externalObjects size with: externalObjects startingAt: 1.	
			self unprotectedExternalObjects: newObjects.].
	^newObjects! !

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 8/18/2011 10:42'!
externalObjects
	"Not really sure why this is protected, once called you are out of protection of the locks anyways, and any use of the object is dangerous...
	Only additions can potentially change the actual array in use though, so only lock that."
	^ProtectAdd critical: [self unprotectedExternalObjects].! !

!ExternalSemaphoreTable class methodsFor: 'private' stamp: 'StephaneDucasse 11/20/2011 15:32'!
freedSlotsIn: externalObjects ratherThanIncreaseSizeTo: newSize
	"In some VM's, the external object table has a max size, which has to be increased for vm to reference them correctly."
	"In that case, try to gc to free slots first before actually increasing the max size"
	"Return whether I ended up freeing slots by GC'ing, or one should increase the size of "
	^Smalltalk vm maxExternalSemaphores
		ifNotNil: [:maxSize |
			(maxSize < newSize) and: 
				[| needToGrow |
				Smalltalk garbageCollect.
				"Do we have free slots now? If not, performing the GC didn't help and we still have to grow."
				needToGrow := externalObjects includes: nil.
				
				needToGrow 
					ifTrue: ["If we did GC, warn we had to gc so actions could be taken if appropriate."
						self traceCr: TimeStamp now printString;
							traceCr:  'WARNING:  Had to GC to make room for more external objects.';
							traceCr: 'If this happens often, it would be a good idea to either:';
							traceCr: '- Raise the maxExternalObjects size.';
							traceCr: '- Write your code to explicitly release them rather than wait for finalization.']
					ifFalse:  [Smalltalk vm maxExternalSemaphores: newSize].
				needToGrow]]
		ifNil:[false]! !

!ExternalSemaphoreTable class methodsFor: 'initialize' stamp: 'HenrikSperreJohansen 8/17/2011 14:54'!
initialize
	ProtectAdd := Semaphore forMutualExclusion.
	ProtectRemove := Semaphore forMutualExclusion! !

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 8/17/2011 14:55'!
registerExternalObject: anObject
	^ ProtectAdd critical: [self safelyRegisterExternalObject: anObject]
! !

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 8/18/2011 10:24'!
safelyRegisterExternalObject: anObject
	"Register the given object in the external objects array and return its index. If it is already there, just return its index."

	| objects firstEmptyIndex obj sz newObjects |
	objects := self unprotectedExternalObjects.

	"find the first empty slot"
	firstEmptyIndex := 
		(self slotFor: anObject in: objects) 
			ifNil: ["if object has no empty slots,  we need to get a collection which does"
				objects := self collectionBasedOn: objects withRoomFor: anObject.
				self slotFor: anObject in: objects.].
	objects at: firstEmptyIndex put: anObject.
	^ firstEmptyIndex
! !

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 7/15/2011 15:30'!
safelyUnregisterExternalObject: anObject
	"Unregister the given object in the external objects array. Do nothing if it isn't registered.
	JMM change to return if we clear the element, since it should only appear once in the array"

	| objects |
	anObject ifNil: [^ self].
	objects := self unprotectedExternalObjects.
	1 to: objects size do: [:i |
		(objects at: i) == anObject ifTrue: 
		[objects at: i put: nil.
		^self]].
! !

!ExternalSemaphoreTable class methodsFor: 'private' stamp: 'HenrikSperreJohansen 8/18/2011 10:20'!
slotFor: anObject in: aCollection
	"find the first empty slot, or nil if there is none"
	"The following was written in an atomic fashion using special methods with no suspension points, not sure if on purpose, but keeping it that way for now.
	Uses should be protected by the ProtectTable semaphore anyhow, but... it's too much work to reason 100% about it"

	| firstEmptyIndex |
	1 to: aCollection size do: [:i | | obj |
		obj := aCollection at: i.
		obj == anObject ifTrue: [^ i].  "object already there, just return its index"
		(firstEmptyIndex == nil and: [obj == nil])  ifTrue: [firstEmptyIndex := i]].
	^firstEmptyIndex! !

!ExternalSemaphoreTable class methodsFor: 'private' stamp: 'HenrikSperreJohansen 8/16/2011 14:45'!
unprotectedExternalObjects
	^Smalltalk specialObjectsArray at: 39! !

!ExternalSemaphoreTable class methodsFor: 'private' stamp: 'HenrikSperreJohansen 8/16/2011 14:45'!
unprotectedExternalObjects: aCollection
	^Smalltalk specialObjectsArray at: 39 put: aCollection! !

!ExternalSemaphoreTable class methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 8/17/2011 14:55'!
unregisterExternalObject: anObject
	ProtectRemove critical: [self safelyUnregisterExternalObject: anObject]
! !

Object subclass: #FilePackage
	instanceVariableNames: 'fullName sourceSystem classes doIts classOrder'
	classVariableNames: 'LogFileStream'
	poolDictionaries: ''
	category: 'System-FilePackage'!

!FilePackage class methodsFor: 'instance creation' stamp: ''!
fromFileNamed: aName
	^self new fromFileNamed: aName! !

!FilePackage methodsFor: 'filein/fileout' stamp: 'alain.plantec 2/6/2009 17:01'!
askForDoits
	| choice choices |
	choices := {'do not process' translated. 'at the beginning' translated. 'at the end' translated}.
	choice := nil.
	[choices includes: choice] whileFalse: [
		choice := UIManager default 
				chooseFrom: choices 
				values: choices 
				title: 'Unprocessed doIts found. When to process those?' translated.
		choice ifNil: [^0]].
	^choices indexOf: choice! !

!FilePackage methodsFor: 'accessing' stamp: ''!
classAt: className
	^self classes at: className! !

!FilePackage methodsFor: 'change record types' stamp: ''!
classComment: chgRec

	(self getClass: chgRec methodClassName) classComment: chgRec! !

!FilePackage methodsFor: 'change record types' stamp: 'al 12/2/2005 13:58'!
classDefinition: string with: chgRec
	| tokens theClass |
	
	self flag: #traits.
		
	tokens := Scanner new scanTokens: string.

	"tokens size = 11 ifFalse:[^doIts add: chgRec]."

	theClass := self getClass: (tokens at: 3).
	theClass definition: string.
	classOrder add: theClass.! !

!FilePackage methodsFor: 'accessing' stamp: ''!
classes
	^classes! !

!FilePackage methodsFor: 'change record types' stamp: ''!
doIt: chgRec
	| string |
	string := chgRec string.
	('*ubclass:*instanceVariableNames:*classVariableNames:*poolDictionaries:*category:*'
		match: string) ifTrue:[^self classDefinition: string with: chgRec].
	('* class*instanceVariableNames:*'
		match: string) ifTrue:[^self metaClassDefinition: string with: chgRec].
	('* removeSelector: *'
		match: string) ifTrue:[^self removedMethod: string with: chgRec].
	('* comment:*'
		match: string) ifTrue:[^self msgClassComment: string with: chgRec].
	('* initialize'
		match: string) ifTrue:[^self]. "Initialization is done based on class>>initialize"
	('''From *'
		match: string) ifTrue:[^self possibleSystemSource: chgRec].
	doIts add: chgRec.! !

!FilePackage methodsFor: 'change record types' stamp: 'avi 1/19/2004 23:47'!
doIts
	^ doIts! !

!FilePackage methodsFor: 'filein/fileout' stamp: 'alain.plantec 2/6/2009 17:02'!
fileIn
	| doitsMark |
	doitsMark := 1.
	doIts isEmpty ifFalse:[doitsMark := self askForDoits].
	doitsMark = 0 ifTrue: [^nil].
	doitsMark = 2 ifTrue:[self fileInDoits].
	classOrder do:[:cls|
		cls fileInDefinition.
	].
	classes do:[:cls|
		Transcript cr; show:'Filing in ', cls name.
		cls fileInMethods.
		cls hasMetaclass ifTrue:[cls metaClass fileInMethods].
	].
	doitsMark = 3 ifTrue:[self fileInDoits].! !

!FilePackage methodsFor: 'filein/fileout' stamp: ''!
fileInDoits
	doIts do:[:chgRec| chgRec fileIn].! !

!FilePackage methodsFor: 'reading' stamp: 'SeanDeNigris 6/21/2012 09:04'!
fileInFrom: aStream
	| changes |
	changes := ChangeSet scanFile: aStream from: 0 to: aStream size.
	aStream close.
	('Processing ', self packageName) 
		displayProgressFrom: 1
		to: changes size
		during:[:bar| | chgRec |
			1 to: changes size do:[:i|
				bar current: i.
				chgRec := changes at: i.
				self perform: (chgRec type copyWith: $:) asSymbol
with: chgRec.
			].
		].! !

!FilePackage methodsFor: 'filein/fileout' stamp: 'ar 7/17/2005 03:36'!
fileOut
	| fileName stream |
	fileName := UIManager default request: 'Enter the file name' initialAnswer:''.
	stream := FileStream newFileNamed: fileName.
	sourceSystem isEmpty ifFalse:[
		stream nextChunkPut: sourceSystem printString;cr ].
	self fileOutOn: stream.
	stream cr; cr.
	self classes do:[:cls|
		cls needsInitialize ifTrue:[
			stream cr; nextChunkPut: cls name,' initialize']].
	stream cr.
	stream close.

	"DeepCopier new checkVariables."
! !

!FilePackage methodsFor: 'filein/fileout' stamp: ''!
fileOutDoits: aStream
	doIts do:[:chgRec| chgRec fileOutOn: aStream].! !

!FilePackage methodsFor: 'filein/fileout' stamp: 'alain.plantec 2/6/2009 17:02'!
fileOutOn: aStream
	| doitsMark |
	doitsMark := 1.
	doIts isEmpty ifFalse:[doitsMark := self askForDoits].
	doitsMark = 0 ifTrue: [^nil].
	doitsMark = 2 ifTrue:[self fileOutDoits: aStream].
	classOrder do:[:cls|
		cls fileOutDefinitionOn: aStream.
	].
	classes do:[:cls|
		cls fileOutMethodsOn: aStream.
		cls hasMetaclass ifTrue:[cls metaClass fileOutMethodsOn: aStream].
	].
	doitsMark = 3 ifTrue:[self fileOutDoits: aStream].! !

!FilePackage methodsFor: 'initialize' stamp: 'StephaneDucasse 2/2/2010 21:52'!
fromFileNamed: aName
	
	fullName := aName.
	FileStream 
		readOnlyFileNamed: aName 
		do: [ :aStream |
			aStream setConverterForCode.
			self fileInFrom: aStream] ! !

!FilePackage methodsFor: 'accessing' stamp: 'pnm
8/23/2000 17:10'!
fullName: aString
	fullName := aString! !

!FilePackage methodsFor: 'accessing' stamp: ''!
fullPackageName
	^fullName! !

!FilePackage methodsFor: 'private' stamp: ''!
getClass: className
	| pseudoClass |
	(classes includesKey: className) ifTrue:[
		^classes at: className.
	].
	pseudoClass := PseudoClass new.
	pseudoClass name: className.
	classes at: className put: pseudoClass.
	^pseudoClass.! !

!FilePackage methodsFor: 'initialize' stamp: 'alain.plantec 5/28/2009 09:54'!
initialize
	super initialize.
	classes := Dictionary new.
	classOrder := OrderedCollection new.
	sourceSystem := ''.
	doIts := OrderedCollection new.! !

!FilePackage methodsFor: 'private' stamp: ''!
metaClassDefinition: string with: chgRec
	| tokens theClass |
	tokens := Scanner new scanTokens: string.
	theClass := self getClass: (tokens at: 1).
	theClass metaClass definition: string.
	classOrder add: theClass metaClass.! !

!FilePackage methodsFor: 'change record types' stamp: ''!
method: chgRec
	(self getClass: chgRec methodClassName) methodChange: chgRec! !

!FilePackage methodsFor: 'private' stamp: 'ar 4/10/2005 18:46'!
msgClassComment: string with: chgRec
	| tokens theClass |
	tokens := Scanner new scanTokens: string.
	(tokens size = 3 and:[(tokens at: 3) isString]) ifTrue:[
		theClass := self getClass: tokens first.
		^theClass commentString: tokens last].
	(tokens size = 4 and:[(tokens at: 3) asString = 'class' and:[(tokens at: 4) isString]]) ifTrue:[
		theClass := self getClass: tokens first.
		theClass metaClass commentString: tokens last].
! !

!FilePackage methodsFor: 'accessing' stamp: ''!
packageInfo
	^String streamContents:[:s|
		s nextPutAll:'Package: '.
		s nextPutAll: self fullPackageName; cr; cr.
		sourceSystem isEmpty ifFalse:[
			s nextPutAll: sourceSystem; cr; cr].
		doIts isEmpty ifFalse:[
			s nextPutAll:'Unresolvable doIts:'; cr; cr.
			doIts do:[:chgRec|
				s nextPut:$!!; nextPutAll: chgRec string; nextPut: $!!; cr]]].! !

!FilePackage methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2012 02:09'!
packageName
	^ self fullPackageName  asFileReference basename! !

!FilePackage methodsFor: 'private' stamp: 'ar 4/10/2005 18:46'!
possibleSystemSource: chgRec
	| tokens |
	sourceSystem isEmpty ifTrue:[
		tokens := Scanner new scanTokens: chgRec string.
		(tokens size = 1 and:[tokens first isString]) ifTrue:[
			sourceSystem := tokens first.
			^self]].
	doIts add: chgRec.! !

!FilePackage methodsFor: 'change record types' stamp: ''!
preamble: chgRec
	self doIt: chgRec! !

!FilePackage methodsFor: 'accessing' stamp: ''!
removeClass: aPseudoClass
	(self classes removeKey: aPseudoClass name).
	classOrder copy do:[:cls|
		cls name = aPseudoClass name ifTrue:[ classOrder remove: cls].
	].! !

!FilePackage methodsFor: 'private' stamp: ''!
removedMethod: string with: chgRec
	| class tokens |
	tokens := Scanner new scanTokens: string.
	(tokens size = 3 and:[(tokens at: 2) == #removeSelector: ]) ifTrue:[
		class := self getClass: (tokens at: 1).
		^class removeSelector: (tokens at: 3).
	].
	(tokens size = 4 and:[(tokens at: 2) == #class and:[(tokens at: 3) == #removeSelector:]]) ifTrue:[
		class := self getClass: (tokens at: 1).
		^class metaClass removeSelector: (tokens at: 4).
	].
	doIts add: chgRec! !

!FilePackage methodsFor: 'accessing' stamp: ''!
renameClass: aPseudoClass to: newName
	| oldName |
	oldName := aPseudoClass name.
	self classes removeKey: oldName.
	self classes at: newName put: aPseudoClass.
	aPseudoClass renameTo: newName.! !

Object subclass: #FileServices
	instanceVariableNames: ''
	classVariableNames: 'FileReaderRegistry'
	poolDictionaries: ''
	category: 'System-FileRegistry'!

!FileServices class methodsFor: 'accessing' stamp: 'ar 7/16/2005 16:59'!
allRegisteredServices
	"self allRegisteredServices"

	| col |
	col := OrderedCollection new.
	self registeredFileReaderClasses do: [:each | col addAll: (each services)].
	^ col! !

!FileServices class methodsFor: 'cleanup' stamp: 'StephaneDucasse 3/9/2010 16:31'!
cleanUp
	"Remove obsolete services"

	self removeObsolete! !

!FileServices class methodsFor: 'accessing' stamp: 'StephaneDucasse 9/9/2010 09:49'!
initialize
	"FileServices initialize"
	Smalltalk globals allClassesDo: [ :aClass |
		(aClass class includesSelector: #fileReaderServicesForFile:suffix:)
			ifTrue: [self registerFileReader: aClass]].! !

!FileServices class methodsFor: 'accessing' stamp: 'ar 7/16/2005 17:00'!
isReaderNamedRegistered: aSymbol
	"return if a given reader class has been registered. Note that this is on purpose that the argument is
	a symbol and not a class"

	 ^ (self registeredFileReaderClasses collect: [:each | each name]) includes: aSymbol
! !

!FileServices class methodsFor: 'accessing' stamp: 'ar 7/16/2005 17:00'!
itemsForDirectory: aFileDirectory
	"Answer a list of services appropriate when no file is selected."

	| services |
	services := OrderedCollection new.
	self registeredFileReaderClasses do: [:reader |
		reader ifNotNil: [services addAll: (reader fileReaderServicesForDirectory: aFileDirectory) ]].
	^ services! !

!FileServices class methodsFor: 'accessing' stamp: 'ar 7/16/2005 17:00'!
itemsForFile: fullName
	"Answer a list of services appropriate for a file of the given full name"

	| services suffix |
	suffix := self suffixOf: fullName.
	services := OrderedCollection new.
	self registeredFileReaderClasses do: [:reader |
		reader ifNotNil: [services addAll: (reader fileReaderServicesForFile: fullName suffix: suffix)]].
	^ services! !

!FileServices class methodsFor: 'accessing' stamp: 'ar 7/16/2005 17:00'!
registerFileReader: aProviderClass
	"register the given class as providing services for reading files"

	| registeredReaders |
	registeredReaders := self registeredFileReaderClasses.
	(registeredReaders includes: aProviderClass) 
			ifFalse: [ registeredReaders addLast: aProviderClass ]! !

!FileServices class methodsFor: 'accessing' stamp: 'ar 7/16/2005 16:59'!
registeredFileReaderClasses
	FileReaderRegistry ifNil: [FileReaderRegistry := OrderedCollection new].
	^ FileReaderRegistry! !

!FileServices class methodsFor: 'accessing' stamp: 'StephaneDucasse 9/9/2010 09:50'!
removeObsolete
	"FileServices removeObsolete"
	self registeredFileReaderClasses copy 
		do: [:cls| cls isObsolete ifTrue:[self unregisterFileReader: cls]]! !

!FileServices class methodsFor: 'accessing' stamp: 'CamilloBruni 5/4/2012 20:10'!
suffixOf: aName
	"Answer the file extension of the given file"
	^ aName
		ifNil: ['']
		ifNotNil: [  aName asFileReference extension asLowercase]! !

!FileServices class methodsFor: 'accessing' stamp: 'ar 7/16/2005 17:00'!
unregisterFileReader: aProviderClass
	"unregister the given class as providing services for reading files"

	self registeredFileReaderClasses remove: aProviderClass ifAbsent: [nil]! !

Object subclass: #HMAC
	instanceVariableNames: 'hash key ipad epad'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Hashing-Core'!

!HMAC commentStamp: '<historical>' prior: 0!
HMAC is a mechanism for message authentication using cryptographic hash functions. HMAC can be used with any iterative cryptographic hash function, e.g., MD5, SHA-1, in combination with a secret shared key.  The cryptographic strength of HMAC depends on the properties of the underlying hash function.

See RFC 2114.!

!HMAC class methodsFor: 'instance creation' stamp: 'len 8/15/2002 01:42'!
on: aHashFunction
	^ self new setHash: aHashFunction! !

!HMAC methodsFor: 'accessing' stamp: 'cmm 12/2/2006 14:57'!
destroy
	key destroy! !

!HMAC methodsFor: 'accessing' stamp: 'len 10/16/2002 16:43'!
digestMessage: aByteArray
	^ hash hashMessage: (key bitXor: epad), (hash hashMessage: (key bitXor: ipad), aByteArray)! !

!HMAC methodsFor: 'accessing' stamp: 'len 8/3/2002 02:06'!
digestSize
	^ hash hashSize! !

!HMAC methodsFor: 'accessing' stamp: 'StephaneDucasse 10/17/2009 17:15'!
key: aByteArray 
	key := aByteArray.
	key size > hash blockSize ifTrue: [ key := hash hashMessage: key ].
	key size < hash blockSize ifTrue: [ key := key , (ByteArray new: hash blockSize - key size) ]! !

!HMAC methodsFor: 'printing' stamp: 'len 8/3/2002 02:08'!
printOn: aStream
	aStream nextPutAll: 'HMAC-'; print: hash! !

!HMAC methodsFor: 'initialization' stamp: 'StephaneDucasse 10/17/2009 17:15'!
setHash: aHash 
	hash := aHash.
	ipad := ByteArray 
		new: aHash blockSize
		withAll: 54.
	epad := ByteArray 
		new: aHash blockSize
		withAll: 92! !

Object subclass: #HashFunction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Hashing-Core'!

!HashFunction class methodsFor: 'accessing' stamp: 'len 8/15/2002 01:43'!
blockSize
	^ self subclassResponsibility! !

!HashFunction class methodsFor: 'hashing' stamp: 'len 8/2/2002 02:20'!
hashMessage: aStringOrByteArray
	^ self new hashMessage: aStringOrByteArray! !

!HashFunction class methodsFor: 'accessing' stamp: 'len 8/9/2002 13:17'!
hashSize
	^ self subclassResponsibility! !

!HashFunction class methodsFor: 'hashing' stamp: 'len 8/2/2002 02:20'!
hashStream: aPositionableStream
	^ self new hashStream: aPositionableStream! !

!HashFunction methodsFor: 'accessing' stamp: 'len 8/15/2002 01:43'!
blockSize
	^ self class blockSize! !

!HashFunction methodsFor: 'accessing' stamp: 'cmm 2/20/2006 23:22'!
doubleHashMessage: aStringOrByteArray
	"SHA1 new doubleHashMessage: 'foo'"
	^ self doubleHashStream: aStringOrByteArray asByteArray readStream! !

!HashFunction methodsFor: 'accessing' stamp: 'cmm 2/20/2006 23:21'!
doubleHashStream: aStream
	^ self hashStream: ((self hashStream: aStream) asByteArray readStream)! !

!HashFunction methodsFor: 'accessing' stamp: 'len 8/7/2002 16:30'!
hashMessage: aStringOrByteArray
	"MD5 new hashMessage: 'foo'"
	^ self hashStream: aStringOrByteArray asByteArray readStream! !

!HashFunction methodsFor: 'accessing' stamp: 'len 8/9/2002 13:17'!
hashSize
	^ self class hashSize! !

!HashFunction methodsFor: 'accessing' stamp: 'len 8/2/2002 02:21'!
hashStream: aStream
	^ self subclassResponsibility! !

!HashFunction methodsFor: 'converting' stamp: 'len 8/3/2002 02:42'!
hmac
	^ HMAC on: self! !

HashFunction subclass: #MD5
	instanceVariableNames: 'state'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Hashing-MD5'!

!MD5 commentStamp: 'ul 3/3/2008 23:40' prior: 0!
This class implements the MD5 128-bit one-way hash function. It uses the MD5Plugin for better performance. Some methods are taken from the original version of MD5NonPrimitive.!

!MD5 class methodsFor: 'accessing' stamp: 'ul 3/3/2008 23:37'!
blockSize
	^ 64! !

!MD5 class methodsFor: 'accessing' stamp: 'ul 3/3/2008 23:37'!
hashSize
	^ 16! !

!MD5 class methodsFor: 'as yet unclassified' stamp: 'ul 3/3/2008 22:46'!
isPluginAvailable

	<primitive: 'primitivePluginAvailable' module: 'MD5Plugin'>
	^false! !

!MD5 class methodsFor: 'as yet unclassified' stamp: 'ul 3/3/2008 23:01'!
new

	self isPluginAvailable 
		ifTrue: [ ^self basicNew ]
		ifFalse: [ ^MD5NonPrimitive basicNew ]! !

!MD5 methodsFor: 'private-buffers' stamp: 'ul 3/3/2008 21:34'!
finalValue

	^state! !

!MD5 methodsFor: 'accessing' stamp: 'ul 3/3/2008 22:00'!
hashStream: aPositionableStream

	| startPosition buf bitLength |
	self initialize.

	aPositionableStream atEnd ifTrue: [
		buf := ByteArray new: 64.
		buf at: 1 put: 128.
		self processBuffer: buf.
		^self finalValue ].

	startPosition := aPositionableStream position.
	[aPositionableStream atEnd] whileFalse: [
		buf := aPositionableStream next: 64.
		(aPositionableStream atEnd not and: [buf size = 64])
			ifTrue: [self processBuffer: buf]
			ifFalse: [
				bitLength := (aPositionableStream position - startPosition) * 8.
				self processFinalBuffer: buf bitLength: bitLength]].

	^ self finalValue! !

!MD5 methodsFor: 'initialization' stamp: 'StephaneDucasse 2/3/2010 22:14'!
initialize
	"Some magic numbers to get the process started"

	state :=  #[1 35 69 103 137 171 205 239 254 220 186 152 118 84 50 16]

! !

!MD5 methodsFor: 'private-buffers' stamp: 'ul 3/3/2008 22:28'!
primProcessBuffer: aByteArray withState: s

	<primitive: 'primitiveProcessBufferWithState' module: 'MD5Plugin'>
	self primitiveFailed! !

!MD5 methodsFor: 'private-buffers' stamp: 'ul 3/3/2008 19:17'!
processBuffer: aByteArray

	self primProcessBuffer: aByteArray withState: state.
	! !

!MD5 methodsFor: 'private-buffers' stamp: 'StephaneDucasse 10/17/2009 17:15'!
processFinalBuffer: aByteArray bitLength: bitLength 
	"Pad the buffer until we have an even 64 bytes, then transform"
	| out |
	out := ByteArray new: 64.
	out 
		replaceFrom: 1
		to: aByteArray size
		with: aByteArray
		startingAt: 1.
	aByteArray size < 56 ifTrue: 
		[ out 
			at: aByteArray size + 1
			put: 128.	"trailing bit"
		self 
			storeLength: bitLength
			in: out.
		self processBuffer: out.
		^ self ].

	"not enough room for the length, so just pad this one, then..."
	aByteArray size < 64 ifTrue: 
		[ out 
			at: aByteArray size + 1
			put: 128 ].
	self processBuffer: out.

	"process one additional block of padding ending with the length"
	out := ByteArray new: 64.	"filled with zeros"
	aByteArray size = 64 ifTrue: 
		[ out 
			at: 1
			put: 128 ].
	self 
		storeLength: bitLength
		in: out.
	self processBuffer: out! !

!MD5 methodsFor: 'private-buffers' stamp: 'StephaneDucasse 10/17/2009 17:15'!
storeLength: bitLength in: aByteArray 
	"Fill in the final 8 bytes of the given ByteArray with a 64-bit
	little-endian representation of the original message length in bits."
	| n i |
	n := bitLength.
	i := aByteArray size - 8 + 1.
	[ n > 0 ] whileTrue: 
		[ aByteArray 
			at: i
			put: (n bitAnd: 255).
		n := n bitShift: -8.
		i := i + 1 ]! !

MD5 subclass: #MD5NonPrimitive
	instanceVariableNames: ''
	classVariableNames: 'ABCDTable IndexTable ShiftTable SinTable'
	poolDictionaries: ''
	category: 'System-Hashing-MD5'!

!MD5NonPrimitive commentStamp: '<historical>' prior: 0!
This class implements the MD5 128-bit one-way hash function.  It relies on
the ThirtyTwoBitRegister class supplied as part of the "Digital Signatures"
functionality included in Squeak 2.7.  As of this date (1/20/2000), the
U.S. Government has lifted many of the previous restrictions on the export
of encryption software, but you should check before exporting anything
including this code.  MD5 is commonly used for some secure Internet
protocols, including authentication in HTTP, which is why I wrote it.

Submitted by Duane Maxwell

!

!MD5NonPrimitive class methodsFor: 'class initialization' stamp: 'StephaneDucasse 10/17/2009 17:15'!
initialize
	"MD5 initialize"
	"Obscure fact: those magic hex numbers that are hard to type in correctly are
	actually the result of a simple trigonometric function and are therefore
	easier to compute than proofread.  Laziness is sometimes a virtue."
	| c |
	c := 2 raisedTo: 32.
	SinTable := Array new: 64.
	1 
		to: 64
		do: 
			[ :i | 
			SinTable 
				at: i
				put: (ThirtyTwoBitRegister new load: (c * i sin abs) truncated) ].
	ShiftTable := { 
		#(7 12 17 22 ).
		#(5 9 14 20 ).
		#(4 11 16 23 ).
		#(6 10 15 21 )
	 }.
	IndexTable := { 
		#(
			1
			2
			3
			4
			5
			6
			7
			8
			9
			10
			11
			12
			13
			14
			15
			16
		).
		#(
			2
			7
			12
			1
			6
			11
			16
			5
			10
			15
			4
			9
			14
			3
			8
			13
		).
		#(
			6
			9
			12
			15
			2
			5
			8
			11
			14
			1
			4
			7
			10
			13
			16
			3
		).
		#(
			1
			8
			15
			6
			13
			4
			11
			2
			9
			16
			7
			14
			5
			12
			3
			10
		)
	 }.
	ABCDTable := { 
		#(1 2 3 4 ).
		#(4 1 2 3 ).
		#(3 4 1 2 ).
		#(2 3 4 1 )
	 }! !

!MD5NonPrimitive methodsFor: 'private-functions' stamp: 'DSM 1/20/2000 01:47'!
fX: x Y: y Z: z
	" compute 'xy or (not x)z'"
	^ x copy bitAnd: y; bitOr: (x copy bitInvert; bitAnd: z)

	! !

!MD5NonPrimitive methodsFor: 'private-functions' stamp: 'DSM 1/20/2000 13:38'!
ffA: a B: b C: c D: d M: m S: s T: t
	"compute a = b + ((a + f(b,c,d) + m + t) <<< s)"
	^ a += (self fX: b Y: c Z: d); += m; += t; leftRotateBy: s; += b.
! !

!MD5NonPrimitive methodsFor: 'private-buffers' stamp: 'len 10/15/2002 19:58'!
finalValue
	"Concatenate the state values to produce the 128-bite result"
	^ (state at: 1) asByteArray, (state at: 2) asByteArray, (state at: 3) asByteArray, (state at: 4) asByteArray! !

!MD5NonPrimitive methodsFor: 'private-functions' stamp: 'DSM 1/20/2000 01:48'!
gX: x Y: y Z: z
	" compute 'xz or y(not z)'"
	^ x copy bitAnd: z; bitOr: (z copy bitInvert; bitAnd: y)

	! !

!MD5NonPrimitive methodsFor: 'private-functions' stamp: 'DSM 1/20/2000 13:38'!
ggA: a B: b C: c D: d M: m S: s T: t
	"compute a = b + ((a + g(b,c,d) + m + t) <<< s)"
	^ a += (self gX: b Y: c Z: d); += m; += t; leftRotateBy: s; += b.
! !

!MD5NonPrimitive methodsFor: 'private-functions' stamp: 'DSM 1/20/2000 01:48'!
hX: x Y: y Z: z
	" compute 'x xor y xor z'"
	^ x copy bitXor: y; bitXor: z

	! !

!MD5NonPrimitive methodsFor: 'private-functions' stamp: 'DSM 1/20/2000 13:38'!
hhA: a B: b C: c D: d M: m S: s T: t
	"compute a = b + ((a + h(b,c,d) + m + t) <<< s)"
	^ a += (self hX: b Y: c Z: d); += m; += t; leftRotateBy: s; += b.
! !

!MD5NonPrimitive methodsFor: 'private-functions' stamp: 'DSM 1/20/2000 01:48'!
iX: x Y: y Z: z
	" compute 'y xor (x or (not z))'"
	^ y copy bitXor: (z copy bitInvert; bitOr: x)
! !

!MD5NonPrimitive methodsFor: 'private-functions' stamp: 'DSM 1/20/2000 13:39'!
iiA: a B: b C: c D: d M: m S: s T: t
	"compute a = b + ((a + i(b,c,d) + m + t) <<< s)"
	^ a += (self iX: b Y: c Z: d); += m; += t; leftRotateBy: s; += b.
! !

!MD5NonPrimitive methodsFor: 'initialization' stamp: 'StephaneDucasse 10/17/2009 17:15'!
initialize
	"Some magic numbers to get the process started"
	state := OrderedCollection newFrom: { 
			(ThirtyTwoBitRegister new load: 1732584193).
			(ThirtyTwoBitRegister new load: 4023233417).
			(ThirtyTwoBitRegister new load: 2562383102).
			(ThirtyTwoBitRegister new load: 271733878)
		 }! !

!MD5NonPrimitive methodsFor: 'private-buffers' stamp: 'StephaneDucasse 10/17/2009 17:15'!
processBuffer: aByteArray 
	"Process a 64-byte buffer"
	| saveState data |
	saveState := state collect: [ :item | item copy ].
	data := Array new: 16.
	1 
		to: 16
		do: 
			[ :index | 
			data 
				at: index
				put: (ThirtyTwoBitRegister new 
						reverseLoadFrom: aByteArray
						at: index * 4 - 3) ].
	self rounds: data.
	1 
		to: 4
		do: [ :index | (state at: index) += (saveState at: index) ]! !

!MD5NonPrimitive methodsFor: 'private-rounds' stamp: 'StephaneDucasse 10/17/2009 17:15'!
round: data selector: selector round: round 
	"Do one round with the given function"
	| shiftIndex template abcd |
	1 
		to: 16
		do: 
			[ :i | 
			shiftIndex := (i - 1) \\ 4 + 1.
			abcd := ABCDTable at: shiftIndex.
			template := { 
				(abcd at: 1).
				(abcd at: 2).
				(abcd at: 3).
				(abcd at: 4).
				((IndexTable at: round) at: i).
				((ShiftTable at: round) at: shiftIndex).
				(SinTable at: (round - 1) * 16 + i)
			 }.
			self 
				step: data
				template: template
				selector: selector ]! !

!MD5NonPrimitive methodsFor: 'private-rounds' stamp: 'DSM 1/20/2000 17:58'!
rounds: data
	"Perform the four rounds with different functions"
	#(
	ffA:B:C:D:M:S:T:
	ggA:B:C:D:M:S:T:
	hhA:B:C:D:M:S:T:
	iiA:B:C:D:M:S:T:
	) doWithIndex: [ :selector :index |
		self round: data selector: selector round: index.]
! !

!MD5NonPrimitive methodsFor: 'private-functions' stamp: 'StephaneDucasse 10/17/2009 17:15'!
step: data template: item selector: selector 
	"Perform one step in the round"
	| args |
	args := { 
		(state at: (item at: 1)).
		(state at: (item at: 2)).
		(state at: (item at: 3)).
		(state at: (item at: 4)).
		(data at: (item at: 5)).
		(item at: 6).
		(item at: 7)
	 }.
	self 
		perform: selector
		withArguments: args! !

HashFunction subclass: #SHA1
	instanceVariableNames: 'totalA totalB totalC totalD totalE totals'
	classVariableNames: 'K1 K2 K3 K4'
	poolDictionaries: ''
	category: 'System-Hashing-SHA1'!

!SHA1 commentStamp: '<historical>' prior: 0!
This class implements the Secure Hash Algorithm (SHA) described in the U.S. government's Secure Hash Standard (SHS). This standard is described in FIPS PUB 180-1, "SECURE HASH STANDARD", April 17, 1995.

The Secure Hash Algorithm is also described on p. 442 of 'Applied Cryptography: Protocols, Algorithms, and Source Code in C' by Bruce Scheier, Wiley, 1996.

See the comment in class DigitalSignatureAlgorithm for details on its use.

Implementation notes:
The secure hash standard was created with 32-bit hardware in mind. All arithmetic in the hash computation must be done modulo 2^32. This implementation uses ThirtyTwoBitRegister objects to simulate hardware registers; this implementation is about six times faster than using LargePositiveIntegers (measured on a Macintosh G3 Powerbook). Implementing a primitive to process each 64-byte buffer would probably speed up the computation by a factor of 20 or more.
!

!SHA1 class methodsFor: 'accessing' stamp: 'rww 9/22/2006 19:25'!
blockSize
	^ 64! !

!SHA1 class methodsFor: 'accessing' stamp: 'len 8/15/2002 01:45'!
hashSize
	^ 20! !

!SHA1 class methodsFor: 'class initialization' stamp: 'StephaneDucasse 10/17/2009 17:15'!
initialize
	"SecureHashAlgorithm initialize"
	"For the curious, here's where these constants come from:
	  #(2 3 5 10) collect: [:x | ((x sqrt / 4.0) * (2.0 raisedTo: 32)) truncated hex]"
	K1 := ThirtyTwoBitRegister new load: 1518500249.
	K2 := ThirtyTwoBitRegister new load: 1859775393.
	K3 := ThirtyTwoBitRegister new load: 2400959708.
	K4 := ThirtyTwoBitRegister new load: 3395469782! !

!SHA1 methodsFor: 'private' stamp: 'jm 12/7/1999 23:25'!
constantForStep: i
	"Answer the constant for the i-th step of the block hash loop. We number our steps 1-80, versus the 0-79 of the standard."

	i <= 20 ifTrue: [^ K1].
	i <= 40 ifTrue: [^ K2].
	i <= 60 ifTrue: [^ K3].
	^ K4
! !

!SHA1 methodsFor: 'private' stamp: 'StephaneDucasse 10/17/2009 17:15'!
expandedBlock: aByteArray 
	"Convert the given 64 byte buffer into 80 32-bit registers and answer the result."
	| out src v |
	out := Array new: 80.
	src := 1.
	1 
		to: 16
		do: 
			[ :i | 
			out 
				at: i
				put: (ThirtyTwoBitRegister new 
						loadFrom: aByteArray
						at: src).
			src := src + 4 ].
	17 
		to: 80
		do: 
			[ :i | 
			v := (out at: i - 3) copy.
			v
				bitXor: (out at: i - 8);
				bitXor: (out at: i - 14);
				bitXor: (out at: i - 16);
				leftRotateBy: 1.
			out 
				at: i
				put: v ].
	^ out! !

!SHA1 methodsFor: 'private' stamp: 'StephaneDucasse 10/17/2009 17:15'!
finalHash
	"Concatenate the final totals to build the 160-bit integer result."
	"Details: If the primitives are supported, the results are in the totals array. Otherwise, they are in the instance variables totalA through totalE."
	| r |
	totals ifNil: 
		[ "compute final hash when not using primitives"
		^ (totalA asInteger bitShift: 128) + (totalB asInteger bitShift: 96) + (totalC asInteger bitShift: 64) + (totalD asInteger bitShift: 32) + totalE asInteger ].

	"compute final hash when using primitives"
	r := 0.
	1 
		to: 5
		do: [ :i | r := r bitOr: ((totals at: i) bitShift: 32 * (5 - i)) ].
	^ r! !

!SHA1 methodsFor: 'private' stamp: 'jm 12/7/1999 22:15'!
hashFunction: i of: x with: y with: z
	"Compute the hash function for the i-th step of the block hash loop. We number our steps 1-80, versus the 0-79 of the standard."
	"Details: There are four functions, one for each 20 iterations. The second and fourth are the same."

	i <= 20 ifTrue: [^ x copy bitAnd: y; bitOr: (x copy bitInvert; bitAnd: z)].
	i <= 40 ifTrue: [^ x copy bitXor: y; bitXor: z].
	i <= 60 ifTrue: [^ x copy bitAnd: y; bitOr: (x copy bitAnd: z); bitOr: (y copy bitAnd: z)].
	^ x copy bitXor: y; bitXor: z
! !

!SHA1 methodsFor: 'accessing' stamp: 'StephaneDucasse 10/17/2009 17:15'!
hashInteger: aPositiveInteger 
	"Hash the given positive integer. The integer to be hashed should have 512 or fewer bits. This entry point is used in key generation."
	| buffer dstIndex |
	self initializeTotals.

	"pad integer with zeros"
	aPositiveInteger highBit <= 512 ifFalse: [ self error: 'integer cannot exceed 512 bits' ].
	buffer := ByteArray new: 64.
	dstIndex := 0.
	aPositiveInteger digitLength 
		to: 1
		by: -1
		do: 
			[ :i | 
			buffer 
				at: (dstIndex := dstIndex + 1)
				put: (aPositiveInteger digitAt: i) ].

	"process that one block"
	self processBuffer: buffer.
	^ self finalHash! !

!SHA1 methodsFor: 'accessing' stamp: 'StephaneDucasse 10/17/2009 17:15'!
hashInteger: aPositiveInteger seed: seedInteger 
	"Hash the given positive integer. The integer to be hashed should have 512 or fewer bits. This entry point is used in the production of random numbers"
	"Initialize totalA through totalE to their seed values."
	| buffer dstIndex |
	totalA := ThirtyTwoBitRegister new load: ((seedInteger bitShift: -128) bitAnd: 4294967295).
	totalB := ThirtyTwoBitRegister new load: ((seedInteger bitShift: -96) bitAnd: 4294967295).
	totalC := ThirtyTwoBitRegister new load: ((seedInteger bitShift: -64) bitAnd: 4294967295).
	totalD := ThirtyTwoBitRegister new load: ((seedInteger bitShift: -32) bitAnd: 4294967295).
	totalE := ThirtyTwoBitRegister new load: (seedInteger bitAnd: 4294967295).
	self initializeTotalsArray.

	"pad integer with zeros"
	buffer := ByteArray new: 64.
	dstIndex := 0.
	aPositiveInteger digitLength 
		to: 1
		by: -1
		do: 
			[ :i | 
			buffer 
				at: (dstIndex := dstIndex + 1)
				put: (aPositiveInteger digitAt: i) ].

	"process that one block"
	self processBuffer: buffer.
	^ self finalHash! !

!SHA1 methodsFor: 'accessing' stamp: 'StephaneDucasse 10/17/2009 17:15'!
hashStream: aPositionableStream 
	"Hash the contents of the given stream from the current position to the end using the Secure Hash Algorithm. The SHA algorithm is defined in FIPS PUB 180-1. It is also described on p. 442 of 'Applied Cryptography: Protocols, Algorithms, and Source Code in C' by Bruce Scheier, Wiley, 1996."
	"SecureHashAlgorithm new hashStream: (ReadStream on: 'foo')"
	| startPosition buf bitLength |
	self initializeTotals.
	aPositionableStream atEnd ifTrue: [ self error: 'empty stream' ].
	startPosition := aPositionableStream position.
	[ aPositionableStream atEnd ] whileFalse: 
		[ buf := aPositionableStream next: 64.
		(aPositionableStream atEnd not and: [ buf size = 64 ]) 
			ifTrue: [ self processBuffer: buf ]
			ifFalse: 
				[ bitLength := (aPositionableStream position - startPosition) * 8.
				self 
					processFinalBuffer: buf
					bitLength: bitLength ] ].
	^ self finalHash asByteArrayOfSize: 20! !

!SHA1 methodsFor: 'private' stamp: 'StephaneDucasse 10/17/2009 17:15'!
initializeTotals
	"Initialize totalA through totalE to their seed values."
	"total registers for use when primitives are absent"
	totalA := ThirtyTwoBitRegister new load: 1732584193.
	totalB := ThirtyTwoBitRegister new load: 4023233417.
	totalC := ThirtyTwoBitRegister new load: 2562383102.
	totalD := ThirtyTwoBitRegister new load: 271733878.
	totalE := ThirtyTwoBitRegister new load: 3285377520.
	self initializeTotalsArray! !

!SHA1 methodsFor: 'private' stamp: 'StephaneDucasse 10/17/2009 17:15'!
initializeTotalsArray
	"Initialize the totals array from the registers for use with the primitives."
	totals := Bitmap new: 5.
	totals 
		at: 1
		put: totalA asInteger.
	totals 
		at: 2
		put: totalB asInteger.
	totals 
		at: 3
		put: totalC asInteger.
	totals 
		at: 4
		put: totalD asInteger.
	totals 
		at: 5
		put: totalE asInteger! !

!SHA1 methodsFor: 'primitives' stamp: 'jm 12/21/1999 20:11'!
primExpandBlock: aByteArray into: wordBitmap
	"Expand the given 64-byte buffer into the given Bitmap of length 80."

	<primitive: 'primitiveExpandBlock' module: 'DSAPrims'>
	^ self primitiveFailed
! !

!SHA1 methodsFor: 'primitives' stamp: 'jm 12/21/1999 22:58'!
primHasSecureHashPrimitive
	"Answer true if this platform has primitive support for the Secure Hash Algorithm."

	<primitive: 'primitiveHasSecureHashPrimitive' module: 'DSAPrims'>
	^ false
! !

!SHA1 methodsFor: 'primitives' stamp: 'jm 12/21/1999 20:13'!
primHashBlock: blockBitmap using: workingTotalsBitmap
	"Hash the given block (a Bitmap) of 80 32-bit words, using the given workingTotals."

	<primitive: 'primitiveHashBlock' module: 'DSAPrims'>
	^ self primitiveFailed
! !

!SHA1 methodsFor: 'private' stamp: 'StephaneDucasse 10/17/2009 17:15'!
processBuffer: aByteArray 
	"Process given 64-byte buffer, accumulating the results in totalA through totalE."
	| a b c d e w tmp |
	self primHasSecureHashPrimitive 
		ifTrue: [ ^ self processBufferUsingPrimitives: aByteArray ]
		ifFalse: [ totals := nil ].

	"initialize registers a through e from the current totals"
	a := totalA copy.
	b := totalB copy.
	c := totalC copy.
	d := totalD copy.
	e := totalE copy.

	"expand and process the buffer"
	w := self expandedBlock: aByteArray.
	1 
		to: 80
		do: 
			[ :i | 
			tmp := (a copy leftRotateBy: 5)
				+= (self 
						hashFunction: i
						of: b
						with: c
						with: d);
				+= e;
				+= (w at: i);
				+= (self constantForStep: i).
			e := d.
			d := c.
			c := b copy leftRotateBy: 30.
			b := a.
			a := tmp ].

	"add a through e into total accumulators"
	totalA += a.
	totalB += b.
	totalC += c.
	totalD += d.
	totalE += e! !

!SHA1 methodsFor: 'private' stamp: 'StephaneDucasse 10/17/2009 17:15'!
processBufferUsingPrimitives: aByteArray 
	"Process given 64-byte buffer using the primitives, accumulating the results in totals."
	"expand and process the buffer"
	| w |
	w := Bitmap new: 80.
	self 
		primExpandBlock: aByteArray
		into: w.
	self 
		primHashBlock: w
		using: totals! !

!SHA1 methodsFor: 'private' stamp: 'StephaneDucasse 10/17/2009 17:15'!
processFinalBuffer: buffer bitLength: bitLength 
	"Process given buffer, whose length may be <= 64 bytes, accumulating the results in totalA through totalE. Also process the final padding bits and length."
	| out |
	out := ByteArray new: 64.
	out 
		replaceFrom: 1
		to: buffer size
		with: buffer
		startingAt: 1.
	buffer size < 56 ifTrue: 
		[ "padding and length fit in last data block"
		out 
			at: buffer size + 1
			put: 128.	"trailing one bit"
		self 
			storeLength: bitLength
			in: out.	"end with length"
		self processBuffer: out.
		^ self ].

	"process the final data block"
	buffer size < 64 ifTrue: 
		[ out 
			at: buffer size + 1
			put: 128 ].	"trailing one bit"
	self processBuffer: out.

	"process one additional block of padding ending with the length"
	out := ByteArray new: 64.	"filled with zeros"
	buffer size = 64 ifTrue: 
		[ "add trailing one bit that didn't fit in final data block"
		out 
			at: 1
			put: 128 ].
	self 
		storeLength: bitLength
		in: out.
	self processBuffer: out! !

!SHA1 methodsFor: 'private' stamp: 'StephaneDucasse 10/17/2009 17:15'!
storeLength: bitLength in: aByteArray 
	"Fill in the final 8 bytes of the given ByteArray with a 64-bit big-endian representation of the original message length in bits."
	| n i |
	n := bitLength.
	i := aByteArray size.
	[ n > 0 ] whileTrue: 
		[ aByteArray 
			at: i
			put: (n bitAnd: 255).
		n := n bitShift: -8.
		i := i - 1 ]! !

Object subclass: #HistoryIterator
	instanceVariableNames: 'index plugged recorder maxSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-History'!

!HistoryIterator commentStamp: 'AlainPlantec 12/14/2010 23:40' prior: 0!
A HistoryIterator holds an UndoRedoGroup in order to store an history of commands with the possibility of undoing and redoing. The iterator cursor is represented by the index inst var. Index always contains the position of the command that can be currently undone. So, undo decrease index and redo increase index. When a new record is stored, then, index contains the newly added record position.
See HistoryIteratorTest for examples.

Instance Variables
	index:		<Integer>
	maxSize:		<Integer>
	plugged:		<Boolean>
	recorder:		<UndoRedoGroup>

index
	- the iterator cursor

maxSize
	- the maximum number of records that can be added in the root group.

plugged
	- if false, then adding of command is not allowed. Useful to prevent bad history recording recurssions (record while undoing or redoing).

recorder
	- The root of the history tree which records undo/redo commands
!

!HistoryIterator methodsFor: 'adding' stamp: 'AlainPlantec 12/14/2010 18:52'!
addItem: anHistoryItem
	| result |
	self isPlugged ifFalse: [^ false].
	self recorder removeLast: (self size - self index).
	result := self recorder addItem: anHistoryItem.
	self updateIndex.
	^ result! !

!HistoryIterator methodsFor: 'adding' stamp: 'AlainPlantec 12/14/2010 12:54'!
addRecord: anItem
	^ self addItem: anItem
! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 19:03'!
at: anInteger
	^ self recorder at: anInteger! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 19:59'!
at: anInteger put: anItem
	self recorder at: anInteger put: anItem! !

!HistoryIterator methodsFor: 'grouping' stamp: 'AlainPlantec 12/14/2010 14:33'!
closeGroup
	self recorder closeGroup
! !

!HistoryIterator methodsFor: 'compatibility' stamp: 'AlainPlantec 12/14/2010 12:54'!
closeRecordGroup
	^ self closeGroup! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 13:00'!
current
	^ (self index <= self size and: [self index > 0])
		ifTrue: [self recorder at: self index]! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 18:49'!
defaultMaximumSize
	"unlimited by default"
	^ 999999999! !

!HistoryIterator methodsFor: 'undo-undo' stamp: 'AlainPlantec 12/14/2010 18:31'!
do
	^ self redo! !

!HistoryIterator methodsFor: 'undo-undo' stamp: 'AlainPlantec 12/14/2010 22:27'!
doAndAddRecord: anUndoRedoRecord
	| result |
	result := anUndoRedoRecord do.
	self addItem: anUndoRedoRecord.
	^ result! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 19:08'!
first
	^ self recorder first! !

!HistoryIterator methodsFor: 'grouping' stamp: 'MarcusDenker 5/31/2011 15:33'!
groupFrom: firstIdx to: secondIdx
	| group |
	group := UndoRedoGroup new.
	firstIdx to: (secondIdx min: index) do: [:i | group addItem: (self at:i)].
	group close.
	group 
		isEmpty ifTrue: [^ self].
	firstIdx + 1 to: (secondIdx min: index) do: [:i | self removeAt: i].
	self at: firstIdx put: group.
	index := index - group size + 1.! !

!HistoryIterator methodsFor: 'testing' stamp: 'AlainPlantec 12/14/2010 12:58'!
hasNext
	"is there an item after current index"
	^ self recorder size - self index > 0! !

!HistoryIterator methodsFor: 'testing' stamp: 'AlainPlantec 12/14/2010 12:54'!
hasPrevious
	"is there an item before current index"
	^ self index > 0! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 12:54'!
index
	^ index ifNil: [index := self size]! !

!HistoryIterator methodsFor: 'testing' stamp: 'AlainPlantec 12/14/2010 12:54'!
isPlugged
	^ plugged ifNil: [plugged := true]! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 19:06'!
last
	^ self recorder last! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 18:46'!
maxSize
	^ maxSize ifNil: [maxSize := self defaultMaximumSize]! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 18:54'!
maxSize: anInteger
	maxSize := anInteger! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 12:54'!
next
	"next item in history"
	^ self hasNext
		ifTrue: [ index := self index + 1.
			self current]! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 13:01'!
nextList
	"return 'next' items sublist "
	^ self recorder copyFrom: self index + 1 to: self size! !

!HistoryIterator methodsFor: 'grouping' stamp: 'AlainPlantec 12/14/2010 18:52'!
openGroup
	self recorder openGroup.
	self updateIndex.
! !

!HistoryIterator methodsFor: 'compatibility' stamp: 'AlainPlantec 12/14/2010 12:54'!
openRecordGroup
	^ self openGroup! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 12:54'!
previous
	"previous item in history"
	^ self hasPrevious
		ifTrue: [index := self index - 1.
			self current]! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 13:01'!
previousList
	"return 'prev' items sublist"
	^ self recorder copyFrom: 1 to: self index - 1! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 14:37'!
recorder
	^ recorder ifNil: [recorder := UndoRedoGroup new]! !

!HistoryIterator methodsFor: 'undo-undo' stamp: 'AlainPlantec 12/14/2010 13:03'!
redo
	self unplugWhile: [
		self hasNext ifFalse: [^false].
		self next redo].
	^ true! !

!HistoryIterator methodsFor: 'undo-undo' stamp: 'AlainPlantec 12/14/2010 22:27'!
redo: doMessageSend undo: undoMessageSend
	self addItem: (UndoRedoRecord redo: doMessageSend undo: undoMessageSend)! !

!HistoryIterator methodsFor: 'undo-undo' stamp: 'AlainPlantec 12/14/2010 22:28'!
redoArray: doArray undoArray: undoArray
	self addItem: (UndoRedoRecord redoArray: doArray undoArray: undoArray)! !

!HistoryIterator methodsFor: 'removing' stamp: 'AlainPlantec 12/14/2010 19:58'!
removeAt: anIndex
	self recorder removeAt: anIndex! !

!HistoryIterator methodsFor: 'removing' stamp: 'AlainPlantec 12/14/2010 18:51'!
removeFirst
	self recorder removeFirst! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 13:02'!
reset
	self recorder reset.
	index := nil.! !

!HistoryIterator methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 13:00'!
size
	^ self recorder size! !

!HistoryIterator methodsFor: 'undo-undo' stamp: 'AlainPlantec 12/14/2010 13:03'!
undo
	self unplugWhile: [
		self current ifNotNil: [:curr | curr undo].
		self previous].
	^ true! !

!HistoryIterator methodsFor: 'undo-undo' stamp: 'AlainPlantec 12/14/2010 12:54'!
unplugWhile: aBlock
	| wasPlugged |
	wasPlugged := self isPlugged.
	plugged := false.
	aBlock ensure: [ plugged := wasPlugged ]! !

!HistoryIterator methodsFor: 'adding' stamp: 'AlainPlantec 12/14/2010 18:51'!
updateIndex
	self size <= self maxSize
		ifFalse: [self removeFirst].
	index := self size.
! !

Object subclass: #HistoryLeaf
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-History'!

!HistoryLeaf commentStamp: 'AlainPlantec 12/14/2010 23:06' prior: 0!
This class represents the abstract leaf structure of items stored in History tree. Composite nodes of such trees are made of HistoryNode instances. For a particular use, this class need to be subclassed. See for example UndoRedoRecord!

!HistoryLeaf methodsFor: 'adding' stamp: 'AlainPlantec 12/13/2010 13:07'!
addItem: anHistoryItem 
	^ false! !

!HistoryLeaf methodsFor: 'testing' stamp: 'AlainPlantec 12/13/2010 13:04'!
isComposite
	^ false! !

!HistoryLeaf methodsFor: 'testing' stamp: 'AlainPlantec 12/13/2010 11:54'!
opened
	^ false! !

HistoryLeaf subclass: #HistoryNode
	instanceVariableNames: 'history opened'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-History'!

!HistoryNode commentStamp: 'AlainPlantec 12/15/2010 00:04' prior: 0!
A HistoryNode is composite node of an history tree. It is made to contain some other HistoryNode or HistoryLeaf instances.
A subnode is added with #addItem:
In order to add and feed a new subtree, one can use openGroup which add a new HistoryNode. When openGroup is sent to an HistoryNode named H, then a new group G is added and all subsequent sent of #addItem: or of #openGroup to H will update the new node G until G is closed by a closeGroup. 

As examples:
---------------
H := HistoryNode new.
H addItem: (i1 := HistoryLeaf new).
---------------
gives:
H
	i1

---------------
H := HistoryNode new.
H openGroup. "add a new group named g1"
H addItem: (i1 := HistoryLeaf new).
H addItem: (i2 := HistoryLeaf new).
--------------
gives:
H
	g1
		i1
		i2

--------------
H := HistoryNode new.
H openGroup. "add a new group named g1"
H openGroup. "add a new group named g2"
H addItem: (i1 := HistoryLeaf new).
H addItem: (i2 := HistoryLeaf new).
--------------
gives:
H
	g1
		g2
			i1
			i2
			
--------------
H := HistoryNode new.
H openGroup. "add a new group named g1"
H openGroup. "add a new group named g2"
H addItem: (i1 := HistoryLeaf new).
H closeGroup. "close g2"
H addItem: (i2 := HistoryLeaf new).
H closeGroup. "close g1"
H addItem: (i3 := HistoryLeaf new).
--------------
gives:
H
	g1
		g2
			i1
		i2
	i3
	
Also se HistoryNodeTest.


Instance Variables
	history:		<OrderedCollection>
	opened:		<Boolean>

history
	- The list of subnodes (HistoryLeaf or HistoryNode instances)

opened
	- true if the node is opened for adding
!

!HistoryNode methodsFor: 'adding' stamp: 'AlainPlantec 12/14/2010 18:49'!
addItem: anHistoryItem 
	self current notNil
		ifTrue: [(self current addItem: anHistoryItem)
				ifTrue: [^ true]].
	self opened
		ifTrue: [self history add: anHistoryItem.
			^ true].
	^ false! !

!HistoryNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/12/2010 19:01'!
at: aPosition
	^ self history at: aPosition! !

!HistoryNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 19:42'!
at: aPosition put: anItem
	self history at: aPosition put: anItem! !

!HistoryNode methodsFor: 'opening-closing' stamp: 'AlainPlantec 12/13/2010 11:53'!
close
	opened := false! !

!HistoryNode methodsFor: 'opening-closing' stamp: 'AlainPlantec 1/5/2011 09:36'!
closeGroup
	(self current notNil
			and: [self current isComposite
				and: [self current opened]])
		ifTrue: [self current closeGroup]
		ifFalse: [self close]! !

!HistoryNode methodsFor: 'testing' stamp: 'AlainPlantec 12/14/2010 19:04'!
closed
	^ self opened not! !

!HistoryNode methodsFor: 'copying' stamp: 'AlainPlantec 12/14/2010 13:06'!
copyFrom: start to: stop
	^ self history copyFrom: start to: stop! !

!HistoryNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/13/2010 15:05'!
current
	^ self history isEmpty
		ifFalse: [self history last] ! !

!HistoryNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 19:08'!
first
	^ self history first! !

!HistoryNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/13/2010 10:39'!
groupClass
	^ self class! !

!HistoryNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/12/2010 18:37'!
history
	^ history ifNil: [history := OrderedCollection new] ! !

!HistoryNode methodsFor: 'testing' stamp: 'AlainPlantec 12/13/2010 13:04'!
isComposite
	^ true! !

!HistoryNode methodsFor: 'testing' stamp: 'AlainPlantec 12/13/2010 15:59'!
isEmpty
	^ self history isEmpty! !

!HistoryNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/14/2010 19:08'!
last
	^ self history last! !

!HistoryNode methodsFor: 'opening-closing' stamp: 'AlainPlantec 12/13/2010 11:53'!
open
	opened := true! !

!HistoryNode methodsFor: 'opening-closing' stamp: 'AlainPlantec 12/13/2010 13:07'!
openGroup
	^ self addItem: self groupClass new! !

!HistoryNode methodsFor: 'testing' stamp: 'AlainPlantec 12/13/2010 11:54'!
opened
	^ opened ifNil: [opened := true]! !

!HistoryNode methodsFor: 'removing' stamp: 'AlainPlantec 12/14/2010 19:57'!
removeAt: anIndex
	self history removeAt: anIndex! !

!HistoryNode methodsFor: 'private ' stamp: 'AlainPlantec 12/12/2010 19:06'!
removeFirst
	self history removeFirst! !

!HistoryNode methodsFor: 'removing' stamp: 'AlainPlantec 12/14/2010 12:56'!
removeLast
	self history removeLast! !

!HistoryNode methodsFor: 'removing' stamp: 'AlainPlantec 12/14/2010 12:59'!
removeLast: count
	self history removeLast: count! !

!HistoryNode methodsFor: 'opening-closing' stamp: 'AlainPlantec 12/13/2010 22:00'!
reset
	history := nil.
	opened := true! !

!HistoryNode methodsFor: 'accessing' stamp: 'AlainPlantec 12/12/2010 19:03'!
size
	^ self history size! !

HistoryNode subclass: #UndoRedoGroup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-History'!

!UndoRedoGroup commentStamp: 'AlainPlantec 12/14/2010 23:30' prior: 0!
An UndoRedoGroup is a composite node of an history tree. It is made to allow redoing or undoing of several commands in one shot. See for example its use to record all text replacement when a "replace all" is performed from a FindReplaceService (see TextEditor>>replaceAll: with: startingAt:)

Instance Variables
!

!UndoRedoGroup methodsFor: 'undo-undo' stamp: 'AlainPlantec 12/14/2010 18:31'!
do
	self redo! !

!UndoRedoGroup methodsFor: 'undo-undo' stamp: 'AlainPlantec 12/14/2010 13:18'!
redo
	self history do: [:i | i redo]! !

!UndoRedoGroup methodsFor: 'undo-undo' stamp: 'AlainPlantec 12/14/2010 19:29'!
undo
	self history reverseDo: [:i | i undo]! !

HistoryLeaf subclass: #UndoRedoRecord
	instanceVariableNames: 'redoMessage undoMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-History'!

!UndoRedoRecord commentStamp: 'AlainPlantec 12/14/2010 23:26' prior: 0!
An UndoRedoRecord is a leaf of an history tree. It  holds undo and redo commands. Such a command is represented by a MessageSend. 

Instance Variables
	redoMessage:		<MessageSend>
	undoMessage:		<MessageSend>

redoMessage
	- The command which is performed for redoing

undoMessage
	- The command which is performed for undoing
!

!UndoRedoRecord class methodsFor: 'compatibility' stamp: 'AlainPlantec 12/14/2010 08:15'!
do: doMessageSend undo: undoMessageSend
	^ self redo: doMessageSend undo: undoMessageSend! !

!UndoRedoRecord class methodsFor: 'compatibility' stamp: 'AlainPlantec 12/14/2010 08:15'!
doArray: doArray undoArray: undoArray
	^ self redoArray: doArray undoArray: undoArray! !

!UndoRedoRecord class methodsFor: 'instance creation' stamp: 'AlainPlantec 12/13/2010 22:43'!
redo: doMessageSend undo: undoMessageSend

	| anUndoRecord |
	anUndoRecord := self new.
	anUndoRecord redoMessage: doMessageSend.
	anUndoRecord undoMessage: undoMessageSend.
	^anUndoRecord.! !

!UndoRedoRecord class methodsFor: 'instance creation' stamp: 'AlainPlantec 12/13/2010 22:43'!
redoArray: doArray undoArray: undoArray
	" doArray and undoArray are constructed as follow: #(receiver selector arguments) "
	| anUndoRecord |
	anUndoRecord := self new.
	anUndoRecord redoMessage: (MessageSend 
		receiver: 	(doArray first) 
		selector: 	(doArray second) 
		arguments: 	(doArray third)).
	undoArray 
		ifNotNil: [ 
			anUndoRecord undoMessage: (MessageSend 
				receiver: 	(undoArray first) 
				selector: 	(undoArray second) 
				arguments: 	(undoArray third))].
	^anUndoRecord.! !

!UndoRedoRecord methodsFor: 'compatibility' stamp: 'AlainPlantec 12/14/2010 08:16'!
do
	^self redo! !

!UndoRedoRecord methodsFor: 'compatibility' stamp: 'AlainPlantec 12/14/2010 08:16'!
doMessage

	^ self redoMessage! !

!UndoRedoRecord methodsFor: 'compatibility' stamp: 'AlainPlantec 12/14/2010 08:17'!
doMessage: aMessageSend
	^ self redoMessage: aMessageSend
! !

!UndoRedoRecord methodsFor: 'redo-undo' stamp: 'AlainPlantec 12/14/2010 13:12'!
redo
	^self redoMessage value.! !

!UndoRedoRecord methodsFor: 'accessing' stamp: 'AlainPlantec 12/13/2010 22:42'!
redoMessage

	^ redoMessage! !

!UndoRedoRecord methodsFor: 'accessing' stamp: 'AlainPlantec 12/13/2010 22:42'!
redoMessage: aMessageSend

	redoMessage := aMessageSend! !

!UndoRedoRecord methodsFor: 'redo-undo' stamp: 'AlainPlantec 12/14/2010 13:12'!
undo
	^self undoMessage
		ifNotNil: [self undoMessage value].! !

!UndoRedoRecord methodsFor: 'accessing' stamp: 'AlainPlantec 12/13/2010 22:22'!
undoMessage

	^ undoMessage! !

!UndoRedoRecord methodsFor: 'accessing' stamp: 'AlainPlantec 12/13/2010 22:22'!
undoMessage: aMessageSend

	undoMessage := aMessageSend! !

Object subclass: #ISOLanguageDefinition
	instanceVariableNames: 'iso3 iso2 iso3Alternate language'
	classVariableNames: 'ISO2Countries ISO2Table ISO3Countries ISO3Table'
	poolDictionaries: ''
	category: 'System-Localization'!

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 9/1/2005 14:06'!
buildIso3166CodesTables
	"ISOLanguageDefinition buildIso3166CodesTables"
	| rawdata stream country isoa2 isoa3 unNumeric macName macCode windowsName windowsCode empty table |
	rawdata := self iso3166Codes.
	table := OrderedCollection new: 200. 
	stream := rawdata readStream.
	empty := 160 asCharacter asString.
	[stream atEnd] whileFalse: 
		[country := stream nextLine.
		isoa2 := stream nextLine.
		isoa3 := stream nextLine.
		unNumeric := stream nextLine.
		windowsName := stream nextLine.
		windowsName = empty ifTrue: [windowsName := nil].
		windowsCode := stream nextLine. 
		windowsCode = empty ifTrue: [windowsCode := nil].
		macName := stream nextLine.
		macName = empty ifTrue: [macName := nil].
		macCode := stream nextLine.
		macCode = empty ifTrue: [macCode := nil].
		table add: { country.  isoa2. isoa3.  unNumeric. windowsName.  windowsCode.  macName. macCode. }].
	^table! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 9/1/2005 14:14'!
extraCountryDefinitions
	^{
	{'Kids'. 'KIDS'. 'KIDS'.}.
	}! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 7/15/2004 18:14'!
extraISO3Definitions

	^self readISOLanguagesFrom: 'jpk		Japanese (Kids)
' readStream! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 7/15/2004 18:13'!
initISO3LanguageTable
	"ISOLanguageDefinition initIso3LanguageTable"

	| table |
	table := ISOLanguageDefinition readISOLanguagesFrom: ISOLanguageDefinition isoLanguages readStream.
	table addAll: self extraISO3Definitions.
	^table! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 9/1/2005 14:12'!
initISOCountries
	"ISOLanguageDefinition initISOCountries"
	| iso3166Table |
	iso3166Table := ISOLanguageDefinition buildIso3166CodesTables.
	ISO2Countries := Dictionary new.
	ISO3Countries := Dictionary new.
	iso3166Table do: [:entry | 
		ISO2Countries at: (entry at: 2) put: (entry at: 1).
		ISO3Countries at: (entry at: 3) put: (entry at: 1)].
	self extraCountryDefinitions do: [:entry | 
		ISO2Countries at: (entry at: 2) put: (entry at: 1).
		ISO3Countries at: (entry at: 3) put: (entry at: 1)]! !

!ISOLanguageDefinition class methodsFor: 'initialization' stamp: 'mir 7/1/2004 18:19'!
initialize
	"ISOLanguageDefinition initialize"

	ISO3Table := nil.
	ISO2Table := nil! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 9/1/2005 14:18'!
iso2Countries
	"ISOLanguageDefinition iso2Countries"
	"ISO2Countries := nil. ISO3Countries := nil"

	ISO2Countries ifNil: [self initISOCountries].
	^ISO2Countries! !

!ISOLanguageDefinition class methodsFor: 'accessing' stamp: 'mir 7/1/2004 18:06'!
iso2LanguageDefinition: aString
	^self iso2LanguageTable at: aString! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 7/1/2004 18:14'!
iso2LanguageTable
	"ISOLanguageDefinition iso2LanguageTable"

	ISO2Table ifNotNil: [^ISO2Table].
	ISO2Table := Dictionary new: self iso3LanguageTable basicSize.
	self iso3LanguageTable do: [:entry |
		ISO2Table at: entry iso2 put: entry].
	^ISO2Table! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 9/1/2005 13:57'!
iso3166Codes
"http://www.unicode.org/onlinedat/countries.html"

^'LAND ISLANDS
AX






AFGHANISTAN
AF
AFG
004




ALBANIA
AL
ALB
008
CTRY_ALBANIA
355


ALGERIA
DZ
DZA
012
CTRY_ALGERIA
213
verArabic
16
AMERICAN SAMOA
AS
ASM
016




ANDORRA
AD
AND
020




ANGOLA
AO
AGO
024




ANGUILLA
AI
AIA
660




ANTARCTICA
AQ
ATA
010




ANTIGUA AND BARBUDA
AG
ATG
028




ARGENTINA
AR
ARG
032
CTRY_ARGENTINA
54


ARMENIA
AM
ARM
051
CTRY_ARMENIA
374
verArmenian
84
ARUBA
AW
ABW
533




AUSTRALIA
AU
AUS
036
CTRY_AUSTRALIA
61
verAustralia
15
AUSTRIA
AT
AUT
040
CTRY_AUSTRIA
43
verAustria
92
AZERBAIJAN
AZ
AZE
031
CTRY_AZERBAIJAN
994


BAHAMAS
BS
BHS
044




BAHRAIN
BH
BHR
048
CTRY_BAHRAIN
973


BANGLADESH
BD
BGD
050


verBengali
60
BARBADOS
BB
BRB
052




BELARUS
BY
BLR
112
CTRY_BELARUS
375


BELGIUM
BE
BEL
056
CTRY_BELGIUM
32
verFrBelgium, verFlemish
98
BELIZE
BZ
BLZ
084
CTRY_BELIZE
501


BENIN
BJ
BEN
204




BERMUDA
BM
BMU
060




BHUTAN
BT
BTN
064


verBhutan
83
BOLIVIA
BO
BOL
068
CTRY_BOLIVIA
591


BOSNIA AND HERZEGOVINA
BA
BIH
070




BOTSWANA
BW
BWA
072




BOUVET ISLAND
BV
BVT
074




BRAZIL
BR
BRA
076
CTRY_BRAZIL
55
verBrazil
71
BRITISH INDIAN OCEAN TERRITORY
IO
IOT
086




BRUNEI DARUSSALAM
BN
BRN
096
CTRY_BRUNEI_DARUSSALAM
673


BULGARIA
BG
BGR
100
CTRY_BULGARIA
359
verBulgaria
72
BURKINA FASO
BF
BFA
854




BURUNDI
BI
BDI
108




CAMBODIA
KH
KHM
116




CAMEROON
CM
CMR
120




CANADA
CA
CAN
124
CTRY_CANADA
2
verFrCanada, verEndCanada
82
CAPE VERDE
CV
CPV
132




CAYMAN ISLANDS
KY
CYM
136




CENTRAL AFRICAN REPUBLIC
CF
CAF
140




CHAD
TD
TCD
148




CHILE
CL
CHL
152
CTRY_CHILE
56


CHINA
CN
CHN
156
CTRY_PRCHINA
86
verChina
52
CHRISTMAS ISLAND
CX
CXR
162




COCOS (KEELING) ISLANDS
CC
CCK
166




COLOMBIA
CO
COL
170
CTRY_COLOMBIA
57


COMOROS
KM
COM
174




CONGO
CG
COG
178




CONGO, THE DEMOCRATIC REPUBLIC OF THE
CD






COOK ISLANDS
CK
COK
184




COSTA RICA
CR
CRI
188
CTRY_COSTA_RICA
506


COTE D''IVOIRE
CI
CIV
384




CROATIA (local name: Hrvatska)
HR
HRV
191
CTRY_CROATIA
385
verCroatia, verYugoCroatian
68 (c), 25 (y)
CUBA
CU
CUB
192




CYPRUS
CY
CYP
196


verCyprus
23
CZECH REPUBLIC
CZ
CZE
203
CTRY_CZECH
420
verCzech
56
DENMARK
DK
DNK
208
CTRY_DENMARK
45
verDenmark(da), verFaeroeIsl(fo)
9(da), 47(fo)
DJIBOUTI
DJ
DJI
262




DOMINICA
DM
DMA
212




DOMINICAN REPUBLIC
DO
DOM
214
CTRY_DOMINICAN_REPUBLIC
1


EAST TIMOR
TL
TLS
626




ECUADOR
EC
ECU
218
CTRY_ECUADOR
593


EGYPT
EG
EGY
818
CTRY_EGYPT
20
verArabic
16
EL SALVADOR
SV
SLV
222
CTRY_EL_SALVADOR
503


EQUATORIAL GUINEA
GQ
GNQ
226




ERITREA
ER
ERI
232




ESTONIA
EE
EST
233
CTRY_ESTONIA
372
verEstonia
44
ETHIOPIA
ET
ETH
210




FALKLAND ISLANDS (MALVINAS)
FK
FLK
238




FAROE ISLANDS
FO
FRO
234
CTRY_FAEROE_ISLANDS
298


FIJI
FJ
FJI
242




FINLAND
FI
FIN
246
CTRY_FINLAND
358
verFinland
17
FRANCE
FR
FRA
250
CTRY_FRANCE
33
verFrance
1
FRANCE, METROPOLITAN
FX
FXX
249




FRENCH GUIANA
GF
GUF
254




FRENCH POLYNESIA
PF
PYF
258




FRENCH SOUTHERN TERRITORIES
TF
ATF
260




GABON
GA
GAB
266




GAMBIA
GM
GMB
270




GEORGIA
GE
GEO
268
CTRY_GEORGIA
995
verGeorgian
85
GERMANY
DE
DEU
276
CTRY_GERMANY
49
verGermany
3
GHANA
GH
GHA
288




GIBRALTAR
GI
GIB
292




GREECE
GR
GRC
300
CTRY_GREECE
30
verGreece, verGreecePoly
20, 40
GREENLAND
GL
GRL
304


verGreenland
107
GRENADA
GD
GRD
308




GUADELOUPE
GP
GLP
312




GUAM
GU
GUM
316




GUATEMALA
GT
GTM
320
CTRY_GUATEMALA
502


GUINEA
GN
GIN
324




GUINEA-BISSAU
GW
GNB
624




GUYANA
GY
GUY
328




HAITI
HT
HTI
332




HEARD ISLAND & MCDONALD ISLANDS
HM
HMD
334




HONDURAS
HN
HND
340
CTRY_HONDURAS
504


HONG KONG
HK
HKG
344
CTRY_HONG_KONG
852


HUNGARY
HU
HUN
348
CTRY_HUNGARY
36
verHungary
43
ICELAND
IS
ISL
352
CTRY_ICELAND
354
verIceland
21
INDIA
IN
IND
356
CTRY_INDIA
91
verIndiaHindi(hi)
33
INDONESIA
ID
IDN
360
CTRY_INDONESIA
62


IRAN, ISLAMIC REPUBLIC OF
IR
IRN
364
CTRY_IRAN
981
verIran
48
IRAQ
IQ
IRQ
368
CTRY_IRAQ
964
verArabic
16
IRELAND
IE
IRL
372
CTRY_IRELAND
353
verIreland
50
ISRAEL
IL
ISR
376
CTRY_ISRAEL
972
verIsrael
13
ITALY
IT
ITA
380
CTRY_ITALY
39
verItaly
4
JAMAICA
JM
JAM
388
CTRY_JAMAICA
1


JAPAN
JP
JPN
392
CTRY_JAPAN
81
verJapan
14
JORDAN
JO
JOR
400
CTRY_JORDAN
962


KAZAKHSTAN
KZ
KAZ
398
CTRY_KAZAKSTAN
7


KENYA
KE
KEN
404
CTRY_KENYA
254


KIRIBATI
KI
KIR
296




KOREA, DEMOCRATIC PEOPLE''S REPUBLIC OF
KP
PRK
408


verKorea
51
KOREA, REPUBLIC OF
KR
KOR
410
CTRY_SOUTH_KOREA
82
verKorea

KUWAIT
KW
KWT
414
CTRY_KUWAIT
965


KYRGYZSTAN
KG
KGZ
417
CTRY_KYRGYZSTAN
996


LAO PEOPLE''S DEMOCRATIC REPUBLIC
LA
LAO
418




LATVIA
LV
LVA
428
CTRY_LATVIA
371
verLatvia
45
LEBANON
LB
LBN
422
CTRY_LEBANON
961


LESOTHO
LS
LSO
426




LIBERIA
LR
LBR
430




LIBYAN ARAB JAMAHIRIYA
LY
LBY
434
CTRY_LIBYA
218
verArabic
16
LIECHTENSTEIN
LI
LIE
438
CTRY_LIECHTENSTEIN
41


LITHUANIA
LT
LTU
440
CTRY_LITHUANIA
370
verLithuania
41
LUXEMBOURG
LU
LUX
442
CTRY_LUXEMBOURG
352
verFrBelgiumLux
6
MACAU
MO
MAC
446
CTRY_MACAU
853


MACEDONIA, THE FORMER YUGOSLAV REPUBLIC OF
MK
MKD
807
CTRY_MACEDONIA
389
verMacedonian

MADAGASCAR
MG
MDG
450




MALAWI
MW
MWI
454




MALAYSIA
MY
MYS
458
CTRY_MALAYSIA
60


MALDIVES
MV
MDV
462
CTRY_MALDIVES
960


MALI
ML
MLI
466




MALTA
MT
MLT
470


verMalta
22
MARSHALL ISLANDS
MH
MHL
584




MARTINIQUE
MQ
MTQ
474




MAURITANIA
MR
MRT
478




MAURITIUS
MU
MUS
480




MAYOTTE
YT
MYT
175




MEXICO
MX
MEX
484
CTRY_MEXICO
52


MICRONESIA, FEDERATED STATES OF
FM
FSM
583




MOLDOVA, REPUBLIC OF
MD
MDA
498




MONACO
MC
MCO
492
CTRY_MONACO
33


MONGOLIA
MN
MNG
496
CTRY_MONGOLIA
976


MONTSERRAT
MS
MSR
500




MOROCCO
MA
MAR
504
CTRY_MOROCCO
212
verArabic
16
MOZAMBIQUE
MZ
MOZ
508




MYANMAR
MM
MMR
104




NAMIBIA
NA
NAM
516




NAURU
NR
NRU
520




NEPAL
NP
NPL
524


verNepal
106
NETHERLANDS
NL
NLD
528
CTRY_NETHERLANDS
31
verNetherlands
5
NETHERLANDS ANTILLES
AN
ANT
530




NEW CALEDONIA
NC
NCL
540




NEW ZEALAND
NZ
NZL
554
CTRY_NEW_ZEALAND
64


NICARAGUA
NI
NIC
558
CTRY_NICARAGUA
505


NIGER
NE
NER
562




NIGERIA
NG
NGA
566




NIUE
NU
NIU
570




NORFOLK ISLAND
NF
NFK
574




NORTHERN MARIANA ISLANDS
MP
MNP
580




NORWAY
NO
NOR
578
CTRY_NORWAY
47
verNorway
12
OMAN
OM
OMN
512
CTRY_OMAN
968


PAKISTAN
PK
PAK
586
CTRY_PAKISTAN
92
verPakistanUrdu, verPunjabi
34 (U), 95 (P)
PALAU
PW
PLW
585




PANAMA
PA
PAN
591
CTRY_PANAMA
507


PALESTINIAN TERRITORY, OCCUPIED
PS






PAPUA NEW GUINEA
PG
PNG
598




PARAGUAY
PY
PRY
600
CTRY_PARAGUAY
595


PERU
PE
PER
604
CTRY_PERU
51


PHILIPPINES
PH
PHL
608
CTRY_PHILIPPINES
63


PITCAIRN
PN
PCN
612




POLAND
PL
POL
616
CTRY_POLAND
48
verPoland
42
PORTUGAL
PT
PRT
620
CTRY_PORTUGAL
351
verPortugal
10
PUERTO RICO
PR
PRI
630
CTRY_PUERTO_RICO
1


QATAR
QA
QAT
634
CTRY_QATAR
974


REUNION
RE
REU
638




ROMANIA
RO
ROU*
642
CTRY_ROMANIA
40
verRomania
39
RUSSIAN FEDERATION
RU
RUS
643
CTRY_RUSSIA
7
verRussia
49
RWANDA
RW
RWA
646




SAINT KITTS AND NEVIS
KN
KNA
659




SAINT LUCIA
LC
LCA
662




SAINT VINCENT AND THE GRENADINES
VC
VCT
670




SAMOA
WS
WSM
882




SAN MARINO
SM
SMR
674




SAO TOME AND PRINCIPE
ST
STP
678




SAUDI ARABIA
SA
SAU
682
CTRY_SAUDI_ARABIA
966
verArabic
16
SENEGAL
SN
SEN
686




SERBIA AND MONTENEGRO
CS


CTRY_SERBIA
381


SEYCHELLES
SC
SYC
690




SIERRA LEONE
SL
SLE
694




SINGAPORE
SG
SGP
702
CTRY_SINGAPORE
65
verSingapore
100
SLOVAKIA (Slovak Republic)
SK
SVK
703
CTRY_SLOVAK
421
verSlovak
57
SLOVENIA
SI
SVN
705
CTRY_SLOVENIA
386
verSlovenian
66
SOLOMON ISLANDS
SB
SLB
90




SOMALIA
SO
SOM
706




SOUTH AFRICA
ZA
ZAF
710
CTRY_SOUTH_AFRICA
27


SOUTH GEORGIA AND THE SOUTH SANDWICH ISLANDS
GS






SPAIN
ES
ESP
724
CTRY_SPAIN
34
verSpain
8
SRI LANKA
LK
LKA
144




SAINT HELENA
SH
SHN
654




SAINT PIERRE AND MIQUELON
PM
SPM
666




SUDAN
SD
SDN
736




SURINAME
SR
SUR
740




SVALBARD AND JAN MAYEN ISLANDS
SJ
SJM
744




SWAZILAND
SZ
SWZ
748




SWEDEN
SE
SWE
752
CTRY_SWEDEN
46
verSweden
7
SWITZERLAND
CH
CHE
756
CTRY_SWITZERLAND
41
verFrSwiss(fr), verGrSwiss(de)
18(fr), 19(de)
SYRIAN ARAB REPUBLIC
SY
SYR
760
CTRY_SYRIA
963


TAIWAN, PROVINCE OF CHINA
TW
TWN
158
CTRY_TAIWAN
886
verTaiwan
53
TAJIKISTAN
TJ
TJK
762




TANZANIA, UNITED REPUBLIC OF
TZ
TZA
834




TATARSTAN



CTRY_TATARSTAN
7


THAILAND
TH
THA
764
CTRY_THAILAND
66
verThailand
54
TIMOR-LESTE
TL






TOGO
TG
TGO
768




TOKELAU
TK
TKL
772




TONGA
TO
TON
776


verTonga
88
TRINIDAD AND TOBAGO
TT
TTO
780
CTRY_TRINIDAD_Y_TOBAGO
1


TUNISIA
TN
TUN
788
CTRY_TUNISIA
216
verArabic
16
TURKEY
TR
TUR
792
CTRY_TURKEY
90
verTurkey
24
TURKMENISTAN
TM
TKM
795




TURKS AND CAICOS ISLANDS
TC
TCA
796




TUVALU
TV
TUV
798




UGANDA
UG
UGA
800




UKRAINE
UA
UKR
804
CTRY_UKRAINE
380
verUkraine
62
UNITED ARAB EMIRATES
AE
ARE
784
CTRY_UAE
971


UNITED KINGDOM
GB
GBR
826
CTRY_UNITED_KINGDOM
44
verBritain
2
UNITED STATES
US
USA
840
CTRY_UNITED_STATES
1
verUS
0
UNITED STATES MINOR OUTLYING ISLANDS
UM
UMI
581




URUGUAY
UY
URY
858
CTRY_URUGUAY
598


UZBEKISTAN
UZ
UZB
860
CTRY_UZBEKISTAN
7


VANUATU
VU
VUT
548




VATICAN CITY STATE (HOLY SEE)
VA
VAT
336




VENEZUELA
VE
VEN
862
CTRY_VENEZUELA
58


VIET NAM
VN
VNM
704
CTRY_VIET_NAM
84
verVietnam

VIRGIN ISLANDS (BRITISH)
VG
VGB
92




VIRGIN ISLANDS (U.S.)
VI
VIR
850




WALLIS AND FUTUNA ISLANDS
WF
WLF
876




WESTERN SAHARA
EH
ESH
732




YEMEN
YE
YEM
887
CTRY_YEMEN
967


YUGOSLAVIA
YU
YUG
891




ZAIRE
ZR
ZAR
180




ZAMBIA
ZM
ZMB
894




ZIMBABWE
ZW
ZWE
716
CTRY_ZIMBABWE
263


'! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 9/1/2005 14:18'!
iso3Countries
	"ISOLanguageDefinition iso3Countries"
	"ISO2Countries := nil. ISO3Countries := nil"

	ISO3Countries ifNil: [self initISOCountries].
	^ISO3Countries! !

!ISOLanguageDefinition class methodsFor: 'accessing' stamp: 'mir 7/1/2004 18:06'!
iso3LanguageDefinition: aString
	^self iso3LanguageTable at: aString! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 7/21/2004 13:10'!
iso3LanguageTable
	"ISOLanguageDefinition iso3LanguageTable"

	^ISO3Table ifNil: [ISO3Table := self initISO3LanguageTable]! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'yo 12/3/2004 17:46'!
isoLanguages
	"ISO 639: 3-letter codes"
	^'abk	ab	Abkhazian
ace		Achinese
ach		Acoli
ada		Adangme
aar	aa	Afar
afh		Afrihili
afr	af	Afrikaans
afa		Afro-Asiatic (Other)
aka		Akan
akk		Akkadian
alb/sqi	sq	Albanian
ale		Aleut
alg		Algonquian languages
tut		Altaic (Other)
amh	am	Amharic
apa		Apache languages
ara	ar	Arabic
arc		Aramaic
arp		Arapaho
arn		Araucanian
arw		Arawak
arm/hye	hy	Armenian
art		Artificial (Other)
asm	as	Assamese
ath		Athapascan languages
map		Austronesian (Other)
ava		Avaric
ave		Avestan
awa		Awadhi
aym	ay	Aymara
aze	az	Azerbaijani
nah		Aztec
ban		Balinese
bat		Baltic (Other)
bal		Baluchi
bam		Bambara
bai		Bamileke languages
bad		Banda
bnt		Bantu (Other)
bas		Basa
bak	ba	Bashkir
baq/eus	eu	Basque
bej		Beja
bem		Bemba
ben	bn	Bengali
ber		Berber (Other)
bho		Bhojpuri
bih	bh	Bihari
bik		Bikol
bin		Bini
bis	bi	Bislama
bra		Braj
bre	be	Breton
bug		Buginese
bul	bg	Bulgarian
bua		Buriat
bur/mya	my	Burmese
bel	be	Byelorussian
cad		Caddo
car		Carib
cat	ca	Catalan
cau		Caucasian (Other)
ceb		Cebuano
cel		Celtic (Other)
cai		Central American Indian (Other)
chg		Chagatai
cha		Chamorro
che		Chechen
chr		Cherokee
chy		Cheyenne
chb		Chibcha
chi/zho	zh	Chinese
chn		Chinook jargon
cho		Choctaw
chu		Church Slavic
chv		Chuvash
cop		Coptic
cor		Cornish
cos	co	Corsican
cre		Cree
mus		Creek
crp		Creoles and Pidgins (Other)
cpe		Creoles and Pidgins, English-based (Other)
cpf		Creoles and Pidgins, French-based (Other)
cpp		Creoles and Pidgins, Portuguese-based (Other)
cus		Cushitic (Other)
	hr	Croatian
ces/cze	cs	Czech
dak		Dakota
dan	da	Danish
del		Delaware
din		Dinka
div		Divehi
doi		Dogri
dra		Dravidian (Other)
dua		Duala
dut/nla	nl	Dutch
dum		Dutch, Middle (ca. 1050-1350)
dyu		Dyula
dzo	dz	Dzongkha
efi		Efik
egy		Egyptian (Ancient)
eka		Ekajuk
elx		Elamite
eng	en	English
enm		English, Middle (ca. 1100-1500)
ang		English, Old (ca. 450-1100)
esk		Eskimo (Other)
epo	eo	Esperanto
est	et	Estonian
ewe		Ewe
ewo		Ewondo
fan		Fang
fat		Fanti
fao	fo	Faroese
fij	fj	Fijian
fin	fi	Finnish
fiu		Finno-Ugrian (Other)
fon		Fon
fra/fre	fr	French
frm		French, Middle (ca. 1400-1600)
fro		French, Old (842- ca. 1400)
fry	fy	Frisian
ful		Fulah
gaa		Ga
gae/gdh		Gaelic (Scots)
glg	gl	Gallegan
lug		Ganda
gay		Gayo
gez		Geez
geo/kat	ka	Georgian
deu/ger	de	German
gmh		German, Middle High (ca. 1050-1500)
goh		German, Old High (ca. 750-1050)
gem		Germanic (Other)
gil		Gilbertese
gon		Gondi
got		Gothic
grb		Grebo
grc		Greek, Ancient (to 1453)
ell/gre	el	Greek, Modern (1453-)
kal	kl	Greenlandic
grn	gn	Guarani
guj	gu	Gujarati
hai		Haida
hau	ha	Hausa
haw		Hawaiian
heb	he	Hebrew
her		Herero
hil		Hiligaynon
him		Himachali
hin	hi	Hindi
hmo		Hiri Motu
hun	hu	Hungarian
hup		Hupa
iba		Iban
ice/isl	is	Icelandic
ibo		Igbo
ijo		Ijo
ilo		Iloko
inc		Indic (Other)
ine		Indo-European (Other)
ind	id	Indonesian
ina	ia	Interlingua (International Auxiliary language Association)
ine		 Interlingue
iku	iu	Inuktitut
ipk	ik	Inupiak
ira		Iranian (Other)
gai/iri	ga	Irish
sga		Irish, Old (to 900)
mga		Irish, Middle (900 - 1200)
iro		Iroquoian languages
ita	it	Italian
jpn	ja	Japanese
jav/jaw	jv/jw Javanese
jrb		Judeo-Arabic
jpr		Judeo-Persian
kab		Kabyle
kac		Kachin
kam		Kamba
kan	kn	Kannada
kau		Kanuri
kaa		Kara-Kalpak
kar		Karen
kas	ks	Kashmiri
kaw		Kawi
kaz	kk	Kazakh
kha		Khasi
khm	km	Khmer
khi		Khoisan (Other)
kho		Khotanese
kik		Kikuyu
kin	rw	Kinyarwanda
kir	ky	Kirghiz
kom		Komi
kon		Kongo
kok		Konkani
kor	ko	Korean
kpe		Kpelle
kro		Kru
kua		Kuanyama
kum		Kumyk
kur	ku	Kurdish
kru		Kurukh
kus		Kusaie
kut		Kutenai
lad		Ladino
lah		Lahnda
lam		Lamba
oci	oc	Langue d''Oc (post 1500)
lao	lo	Lao
lat	la	Latin
lav	lv	Latvian
ltz		Letzeburgesch
lez		Lezghian
lin	ln	Lingala
lit	lt	Lithuanian
loz		Lozi
lub		Luba-Katanga
lui		Luiseno
lun		Lunda
luo		Luo (Kenya and Tanzania)
mac/mak	mk	Macedonian
mad		Madurese
mag		Magahi
mai		Maithili
mak		Makasar
mlg	mg	Malagasy
may/msa	ms	Malay
mal		Malayalam
mlt	ml	Maltese
man		Mandingo
mni		Manipuri
mno		Manobo languages
max		Manx
mao/mri	mi	Maori
mar	mr	Marathi
chm		Mari
mah		Marshall
mwr		Marwari
mas		Masai
myn		Mayan languages
men		Mende
mic		Micmac
min		Minangkabau
mis		Miscellaneous (Other)
moh		Mohawk
mol	mo	Moldavian
mkh		Mon-Kmer (Other)
lol		Mongo
mon	mn	Mongolian
mos		Mossi
mul		Multiple languages
mun		Munda languages
nau	na	Nauru
nav		Navajo
nde		Ndebele, North
nbl		Ndebele, South
ndo		Ndongo
nep	ne	Nepali
new		Newari
nic		Niger-Kordofanian (Other)
ssa		Nilo-Saharan (Other)
niu		Niuean
non		Norse, Old
nai		North American Indian (Other)
nor	no	Norwegian
nno		Norwegian (Nynorsk)
nub		Nubian languages
nym		Nyamwezi
nya		Nyanja
nyn		Nyankole
nyo		Nyoro
nzi		Nzima
oji		Ojibwa
ori	or	Oriya
orm	om	Oromo
osa		Osage
oss		Ossetic
oto		Otomian languages
pal		Pahlavi
pau		Palauan
pli		Pali
pam		Pampanga
pag		Pangasinan
pan	pa	Panjabi
pap		Papiamento
paa		Papuan-Australian (Other)
fas/per	fa	Persian
peo		Persian, Old (ca 600 - 400 B.C.)
phn		Phoenician
pol	pl	Polish
pon		Ponape
por	pt	Portuguese
pra		Prakrit languages
pro		Provencal, Old (to 1500)
pus	ps	Pushto
que	qu	Quechua
roh	rm	Rhaeto-Romance
raj		Rajasthani
rar		Rarotongan
roa		Romance (Other)
ron/rum	ro	Romanian
rom		Romany
run	rn	Rundi
rus	ru	Russian
sal		Salishan languages
sam		Samaritan Aramaic
smi		Sami languages
smo	sm	Samoan
sad		Sandawe
sag	sg	Sango
san	sa	Sanskrit
srd		Sardinian
sco		Scots
sel		Selkup
sem		Semitic (Other)
	sr	Serbian
scr	sh	Serbo-Croatian
srr		Serer
shn		Shan
sna	sn	Shona
sid		Sidamo
bla		Siksika
snd	sd	Sindhi
sin	si	Singhalese
sit		Sino-Tibetan (Other)
sio		Siouan languages
sla		Slavic (Other)
ssw	ss	Siswant
slk/slo	sk	Slovak
slv	sl	Slovenian
sog		Sogdian
som	so	Somali
son		Songhai
wen		Sorbian languages
nso		Sotho, Northern
sot	st	Sotho, Southern
sai		South American Indian (Other)
esl/spa	es	Spanish
suk		Sukuma
sux		Sumerian
sun	su	Sudanese
sus		Susu
swa	sw	Swahili
ssw		Swazi
sve/swe	sv	Swedish
syr		Syriac
tgl	tl	Tagalog
tah		Tahitian
tgk	tg	Tajik
tmh		Tamashek
tam	ta	Tamil
tat	tt	Tatar
tel	te	Telugu
ter		Tereno
tha	th	Thai
bod/tib	bo	Tibetan
tig		Tigre
tir	ti	Tigrinya
tem		Timne
tiv		Tivi
tli		Tlingit
tog	to	Tonga (Nyasa)
ton		Tonga (Tonga Islands)
tru		Truk
tsi		Tsimshian
tso	ts	Tsonga
tsn	tn	Tswana
tum		Tumbuka
tur	tr	Turkish
ota		Turkish, Ottoman (1500 - 1928)
tuk	tk	Turkmen
tyv		Tuvinian
twi	tw	Twi
uga		Ugaritic
uig	ug	Uighur
ukr	uk	Ukrainian
umb		Umbundu
und		Undetermined
urd	ur	Urdu
uzb	uz	Uzbek
vai		Vai
ven		Venda
vie	vi	Vietnamese
vol	vo	Volapk
vot		Votic
wak		Wakashan languages
wal		Walamo
war		Waray
was		Washo
cym/wel	cy	Welsh
wol	wo	Wolof
xho	xh	Xhosa
sah		Yakut
yao		Yao
yap		Yap
yid	yi	Yiddish
yor	yo	Yoruba
zap		Zapotec
zen		Zenaga
zha	za	Zhuang
zul	zu	Zulu
zun		Zuni'! !

!ISOLanguageDefinition class methodsFor: 'private' stamp: 'mir 7/1/2004 18:07'!
readISOLanguagesFrom: stream
	"ISOLanguageDefinition readISOLanguagesFrom: ISOLanguageDefinition isoLanguages readStream "
	| languages language code3 index line |
	languages := Dictionary new.
	[stream atEnd
		or: [(line := stream nextLine readStream) atEnd]]
		whileFalse: [
			language := ISOLanguageDefinition new.
			code3 := line upTo: Character tab.
			(index := code3 indexOf: $/) > 0
				ifTrue: [
					language iso3: (code3 copyFrom: 1 to: index-1).
					language iso3Alternate: (code3 copyFrom: index+1 to: code3 size)]
				ifFalse: [language iso3: code3].
			language
				iso2: (line upTo: Character tab);
				language: line upToEnd.
			languages at: language iso3 put: language].
	^languages! !

!ISOLanguageDefinition methodsFor: 'accessing' stamp: 'mir 7/1/2004 18:20'!
iso2
	^iso2 ifNil: [self iso3]! !

!ISOLanguageDefinition methodsFor: 'initialize' stamp: 'mir 6/30/2004 15:54'!
iso2: aString
	iso2 := aString ifEmpty: [nil] ifNotEmpty: [aString]! !

!ISOLanguageDefinition methodsFor: 'accessing' stamp: 'mir 7/1/2004 18:21'!
iso3
	^iso3 ifNil: ['']! !

!ISOLanguageDefinition methodsFor: 'initialize' stamp: 'mir 6/30/2004 15:54'!
iso3: aString
	iso3 := aString ifEmpty: [nil] ifNotEmpty: [aString]! !

!ISOLanguageDefinition methodsFor: 'accessing' stamp: 'mir 6/30/2004 15:47'!
iso3Alternate
	^iso3Alternate ifNil: ['']! !

!ISOLanguageDefinition methodsFor: 'initialize' stamp: 'mir 6/30/2004 15:54'!
iso3Alternate: aString
	iso3Alternate := aString ifEmpty: [nil] ifNotEmpty: [aString]! !

!ISOLanguageDefinition methodsFor: 'accessing' stamp: 'mir 8/15/2003 13:13'!
language
	^language! !

!ISOLanguageDefinition methodsFor: 'initialize' stamp: 'mir 8/15/2003 13:40'!
language: aString
	language := aString! !

Object subclass: #LRUCache
	instanceVariableNames: 'size factory calls hits values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!LRUCache commentStamp: '<historical>' prior: 0!
I'm a cache of values, given a key I return a Value from the cache or from the factory!

!LRUCache class methodsFor: 'instance creation' stamp: ''!
size: aNumber factory: aBlock 
	"answer an instance of the receiver"
	^ self new initializeSize: aNumber factory: aBlock! !

!LRUCache class methodsFor: 'testing' stamp: ''!
test
	" 
	LRUCache test 
	"
	| c |
	c := LRUCache
				size: 5
				factory: [:key | key * 2].
	c at: 1.
	c at: 2.
	c at: 3.
	c at: 4.
	c at: 1.
	c at: 5.
	c at: 6.
	c at: 7.
	c at: 8.
	c at: 1.
	^ c! !

!LRUCache class methodsFor: 'testing' stamp: ''!
test2
	" 
	LRUCache test2.  
	Time millisecondsToRun:[LRUCache test2]. 
	MessageTally spyOn:[LRUCache test2].  
	"
	| c |
	c := LRUCache
				size: 600
				factory: [:key | key * 2].
	1
		to: 6000
		do: [:each | c at: each].
	^ c! !

!LRUCache methodsFor: 'accessing' stamp: ''!
at: aKey 
	"answer the object for aKey, if not present in the cache creates it.
	Clone the factory block before calling in case of multiple processes!!"
	
	| element keyHash |
	calls := calls + 1.
	keyHash := aKey hash.
	1
		to: size
		do: [:index | 
			element := values at: index.
			(keyHash
						= (element at: 2)
					and: [aKey
							= (element at: 1)])
				ifTrue: ["Found!!"
					hits := hits + 1.
					values
						replaceFrom: 2
						to: index
						with: (values first: index - 1).
					values at: 1 put: element.
					^ element at: 3]].
	"Not found!!"
	element := {aKey. keyHash. factory shallowCopy value: aKey}.
	values
		replaceFrom: 2
		to: size
		with: values allButLast.
	values at: 1 put: element.
	^ element at: 3! !

!LRUCache methodsFor: 'initialization' stamp: ''!
initializeSize: aNumber factory: aBlock 
	"initialize the receiver's size and factory"
	size := aNumber.
	values := Array new: aNumber withAll: {nil. nil. nil}.
	factory := aBlock.
	calls := 0.
	hits := 0! !

!LRUCache methodsFor: 'printing' stamp: ''!
printOn: aStream 
	"Append to the argument, aStream, a sequence of characters 
	that identifies the receiver."
	aStream nextPutAll: self class name;
		 nextPutAll: ' size:';
		 nextPutAll: size asString;
		 nextPutAll: ', calls:';
		 nextPutAll: calls asString;
		 nextPutAll: ', hits:';
		 nextPutAll: hits asString;
		 nextPutAll: ', ratio:'.
	calls = 0
		ifTrue: [aStream nextPutAll: 0 asFloat asString]
		ifFalse: [aStream nextPutAll: (hits / calls) asFloat asString]! !

Object subclass: #Locale
	instanceVariableNames: 'id shortDate longDate time decimalSymbol digitGrouping currencySymbol currencyNotation measurement offsetLocalToUTC offsetVMToUTC dstActive'
	classVariableNames: 'Activated Current CurrentPlatform KnownLocales LanguageSymbols LocaleChangeListeners PlatformEncodings'
	poolDictionaries: ''
	category: 'System-Localization'!

!Locale commentStamp: '<historical>' prior: 0!
Main comment stating the purpose of this class and relevant relationship to other classes.



	http://www.w3.org/WAI/ER/IG/ert/iso639.htm
	http://www.oasis-open.org/cover/iso639a.html
	See also
	http://oss.software.ibm.com/cvs/icu/~checkout~/icuhtml/design/language_code_issues.html
	http://www.w3.org/Protocols/rfc2616/rfc2616-sec3.html#sec3.10
	
ISO 3166
http://mitglied.lycos.de/buran/knowhow/codes/locales/
!

!Locale class methodsFor: 'settings' stamp: 'AlainPlantec 12/11/2009 10:59'!
activated
	^ Activated ifNil: [Activated := false]! !

!Locale class methodsFor: 'settings' stamp: 'AlainPlantec 12/11/2009 10:59'!
activated: aBoolean
	Activated := aBoolean! !

!Locale class methodsFor: 'notification' stamp: 'mir 6/30/2004 16:15'!
addLocalChangedListener: anObjectOrClass
	self localeChangedListeners add: anObjectOrClass! !

!Locale class methodsFor: 'accessing' stamp: 'mir 8/31/2005 17:36'!
current
	"Current := nil"
	Current ifNil: [
		Current := self determineCurrentLocale.
		"Transcript show: 'Current locale: ' , Current localeID asString; cr"].
	^Current! !

!Locale class methodsFor: 'accessing' stamp: 'yo 7/28/2004 20:32'!
currentPlatform
	"CurrentPlatform := nil"
	CurrentPlatform ifNil: [CurrentPlatform := self determineCurrentLocale].
	^CurrentPlatform! !

!Locale class methodsFor: 'accessing' stamp: 'yo 7/28/2004 20:39'!
currentPlatform: locale
	CurrentPlatform := locale.
	LanguageEnvironment startUp.
! !

!Locale class methodsFor: 'accessing' stamp: 'tak 10/18/2005 22:33'!
currentPlatform: locale during: aBlock 
	"Alter current language platform during a block"
	| backupPlatform |
	backupPlatform := self currentPlatform.
	[self currentPlatform: locale.
	aBlock value]
		ensure: [self currentPlatform: backupPlatform]! !

!Locale class methodsFor: 'platform specific' stamp: 'MarcusDenker 7/13/2012 14:30'!
defaultEncodingName: languageSymbol 
	| encodings platformName osVersion |
	platformName := OSPlatform platformName.
	osVersion := OSPlatform version.
	encodings := self platformEncodings at: languageSymbol
				ifAbsent: [self platformEncodings at: #default].
	encodings at: platformName ifPresent: [:encoding | ^encoding].
	encodings at: platformName , ' ' , osVersion
		ifPresent: [:encoding | ^encoding].
	^encodings at: #default! !

!Locale class methodsFor: 'private' stamp: 'mir 7/28/2005 00:24'!
determineCurrentLocale
	"For now just return the default locale.
	A smarter way would be to determine the current platforms default locale."
	"Locale determineCurrentLocale"

	^self new determineLocale! !

!Locale class methodsFor: 'private' stamp: 'mir 7/15/2004 19:44'!
initKnownLocales
	| locales |
	locales := Dictionary new.

	"Init the locales for which we have translations"
	NaturalLanguageTranslator availableLanguageLocaleIDs do: [:id |
		locales at: id put: (Locale new localeID: id)].
	^locales! !

!Locale class methodsFor: 'initialization' stamp: 'AlainPlantec 1/5/2010 12:24'!
initialize
	"Locale initialize"
	Smalltalk addToStartUpList: Locale.
! !

!Locale class methodsFor: 'initialization' stamp: 'MarcusDenker 1/24/2010 11:07'!
initializePlatformEncodings
	"Locale initializePlatformEncodings"

	| platform |
	PlatformEncodings ifNil: [ PlatformEncodings := Dictionary new ].

	platform := PlatformEncodings at: 'default' ifAbsentPut: Dictionary new.
	platform
		at: 'default' put: 'iso8859-1';
		at: 'Win32 CE' put: 'utf-8'.

	platform := PlatformEncodings at: 'ja' ifAbsentPut: Dictionary new.
	platform
		at: 'default' put: 'shift-jis';
		at: 'unix' put: 'euc-jp';
		at: 'Win32 CE' put: 'utf-8'.
		
	platform := PlatformEncodings at: 'ko' ifAbsentPut: Dictionary new.
	platform
		at: 'default' put: 'euc-kr';
		at: 'Win32 CE' put: 'utf-8'.

	platform := PlatformEncodings at: 'zh' ifAbsentPut: Dictionary new.
	platform
		at: 'default' put: 'gb2312';
		at: 'unix' put: 'euc-cn';
		at: 'Win32 CE' put: 'utf-8'.
! !

!Locale class methodsFor: 'accessing' stamp: 'mir 7/15/2004 14:20'!
isoLanguage: isoLanguage
	^self isoLanguage: isoLanguage isoCountry: nil! !

!Locale class methodsFor: 'accessing' stamp: 'mir 7/15/2004 14:31'!
isoLanguage: isoLanguage isoCountry: isoCountry
	^self localeID: (LocaleID  isoLanguage: isoLanguage isoCountry: isoCountry)! !

!Locale class methodsFor: 'accessing' stamp: 'mir 7/15/2004 12:42'!
isoLocale: aString
	! !

!Locale class methodsFor: 'private' stamp: 'mir 7/15/2004 16:44'!
knownLocales
	"KnownLocales := nil"
	^KnownLocales ifNil: [KnownLocales := self initKnownLocales]! !

!Locale class methodsFor: 'accessing' stamp: 'mir 7/13/2004 00:24'!
languageSymbol: languageSymbol
	"Locale languageSymbol: #Deutsch"

	^self isoLanguage: (LanguageSymbols at: languageSymbol)! !

!Locale class methodsFor: 'system startup' stamp: 'CamilloBruni 7/17/2012 16:11'!
localTimeZone
	| offset abbreviation |
	offset := Duration minutes: self current primTimezone.
	abbreviation := String streamContents: [ :s |
		s nextPutAll: 'LT'; print: offset hours; nextPut: $:.
		s nextPutAll: (offset minutes printPaddedWith: $0 to: 2) ].
	^ TimeZone
		offset:  offset
		name: 'Local Time'
		abbreviation: abbreviation! !

!Locale class methodsFor: 'notification' stamp: 'MarcusDenker 3/24/2011 16:36'!
localeChanged! !

!Locale class methodsFor: 'notification' stamp: 'mir 6/30/2004 16:15'!
localeChangedListeners
	^LocaleChangeListeners ifNil: [LocaleChangeListeners := OrderedCollection new]! !

!Locale class methodsFor: 'accessing' stamp: 'mir 7/15/2004 14:30'!
localeID: id
	^self knownLocales at: id ifAbsentPut: [Locale new localeID: id]! !

!Locale class methodsFor: 'initialization' stamp: 'nk 8/29/2004 13:20'!
platformEncodings
	PlatformEncodings isEmptyOrNil ifTrue: [ self initializePlatformEncodings ].
	^PlatformEncodings
! !

!Locale class methodsFor: 'system startup' stamp: 'CamilloBruni 7/17/2012 15:55'!
startUp: resuming
	| newID |
	resuming ifFalse: [^self].
	DateAndTime localTimeZone: self localTimeZone.
	(self activated)
		ifTrue: [
			newID := self current determineLocaleID.
			newID ~= LocaleID current
				ifTrue: [self switchToID: newID]]! !

!Locale class methodsFor: 'accessing' stamp: 'HilaireFernandes 5/6/2010 21:31'!
stringForLanguageNameIs: localeID 
	"Answer a string for a menu determining whether the given  
	symbol is the project's natural language"
	^ (self current localeID = localeID
		ifTrue: ['<yes>']
		ifFalse: ['<no>'])
		, localeID displayLanguage! !

!Locale class methodsFor: 'accessing' stamp: 'tak 8/4/2005 16:30'!
switchTo: locale 
	"Locale switchTo: Locale isoLanguage: 'de'"
	Current localeID = locale localeID
		ifFalse: [Current := locale.
			CurrentPlatform := locale.
			self localeChanged]! !

!Locale class methodsFor: 'accessing' stamp: 'mir 7/15/2004 19:07'!
switchToID: localeID
	"Locale switchToID: (LocaleID isoLanguage: 'de') "

	self switchTo: (Locale localeID: localeID)! !

!Locale methodsFor: 'accessing' stamp: 'mir 8/31/2005 17:03'!
determineLocale
	self localeID: self determineLocaleID! !

!Locale methodsFor: 'accessing' stamp: 'mir 8/31/2005 16:32'!
determineLocaleID
	"Locale current determineLocaleID"
	| langCode isoLang countryCode isoCountry |
	langCode := self fetchISO2Language.
	isoLang := langCode
		ifNil: [^self localeID]
		ifNotNil: [langCode].
	countryCode := self primCountry.
	isoCountry := countryCode
		ifNil: [^LocaleID isoLanguage: isoLang]
		ifNotNil: [countryCode].
	^LocaleID isoLanguage: isoLang isoCountry: isoCountry! !

!Locale methodsFor: 'private' stamp: 'StephaneDucasse 5/28/2011 13:45'!
fetchISO2Language
	"Locale current fetchISO2Language"
	| lang isoLang |
	lang := self primLanguage.
	lang ifNil: [^nil].
	lang := lang copyUpTo: 0 asCharacter.
	lang size = 2
		ifTrue: [^lang].
	isoLang := ISOLanguageDefinition iso3LanguageDefinition: lang.
	^isoLang
		ifNil: [nil]
		ifNotNil: [isoLang iso2]! !

!Locale methodsFor: 'accessing' stamp: 'nk 8/31/2004 09:39'!
isoCountry
	^self localeID isoCountry! !

!Locale methodsFor: 'accessing' stamp: 'nk 8/31/2004 09:39'!
isoLanguage
	^self localeID isoLanguage! !

!Locale methodsFor: 'accessing' stamp: 'mir 7/15/2004 12:41'!
isoLocale
	"<language>-<country>"
	^self isoCountry
		ifNil: [self isoLanguage]
		ifNotNil: [self isoLanguage , '-' , self isoCountry]! !

!Locale methodsFor: 'accessing' stamp: 'mir 7/15/2004 15:52'!
languageEnvironment
	^LanguageEnvironment localeID: self localeID! !

!Locale methodsFor: 'accessing' stamp: 'mir 7/15/2004 14:55'!
localeID
	^id! !

!Locale methodsFor: 'accessing' stamp: 'mir 7/15/2004 14:55'!
localeID: anID
	id := anID! !

!Locale methodsFor: 'system primitives' stamp: 'jannik.laval 2/4/2010 15:09'!
primCountry
	"Returns string with country tag according to ISO 639"
	<primitive: 'primitiveCountry' module: 'LocalePlugin'>
	^'FR'! !

!Locale methodsFor: 'system primitives' stamp: 'tpr 6/1/2005 18:45'!
primCurrencyNotation
	"Returns boolean if symbol is pre- (true) or post-fix (false)"
	<primitive: 'primitiveCurrencyNotation' module: 'LocalePlugin'>
	^true! !

!Locale methodsFor: 'system primitives' stamp: 'tpr 6/1/2005 18:47'!
primCurrencySymbol
	"Returns string with currency symbol"
	<primitive: 'primitiveCurrencySymbol' module:'LocalePlugin'>
	^'$'! !

!Locale methodsFor: 'system primitives' stamp: 'tpr 6/1/2005 18:48'!
primDST
	"Returns boolean if DST  (daylight saving time) is active or not"
	<primitive:'primitiveDaylightSavings' module: 'LocalePlugin'>
	^false! !

!Locale methodsFor: 'system primitives' stamp: 'tpr 6/2/2005 13:42'!
primDecimalSymbol
	"Returns string with e.g. '.' or ','"
	<primitive:'primitiveDecimalSymbol' module: 'LocalePlugin'>
	^'.'! !

!Locale methodsFor: 'system primitives' stamp: 'tpr 6/2/2005 13:42'!
primDigitGrouping
	"Returns string with e.g. '.' or ',' (thousands etc)"
	<primitive:'primitiveDigitGroupingSymbol' module: 'LocalePlugin'>
	^','! !

!Locale methodsFor: 'system primitives' stamp: 'jannik.laval 2/4/2010 15:07'!
primLanguage
	"returns string with language tag according to ISO 639"
	<primitive:'primitiveLanguage' module: 'LocalePlugin'>
	^'en'! !

!Locale methodsFor: 'system primitives' stamp: 'jannik.laval 2/4/2010 15:09'!
primLongDateFormat
	"Returns the long date format
	d day, m month, y year,
	double symbol is null padded, single not padded (m=6, mm=06)
	dddd weekday
	mmmm month name"
	<primitive:'primitiveLongDateFormat' module: 'LocalePlugin'>
	^'dddd, mmmm d, yyyy'
! !

!Locale methodsFor: 'system primitives' stamp: 'tpr 6/2/2005 13:43'!
primMeasurement
	"Returns boolean denoting metric(true) or imperial(false)."
	<primitive:'primitiveMeasurementMetric' module: 'LocalePlugin'>
	^true
! !

!Locale methodsFor: 'system primitives' stamp: 'jannik.laval 2/4/2010 15:11'!
primShortDateFormat
	"Returns the short date format
	d day, m month, y year,
	double symbol is null padded, single not padded (m=6, mm=06)
	dddd weekday
	mmmm month name"
	<primitive:'primitiveShortDateFormat' module: 'LocalePlugin'>
	^'m/d/yy'
! !

!Locale methodsFor: 'system primitives' stamp: 'jannik.laval 2/4/2010 15:11'!
primTimeFormat
	"Returns string time format
	Format is made up of 
	h hour (h 12, H 24), m minute, s seconds, x (am/pm String)
	double symbol is null padded, single not padded (h=6, hh=06)"
	<primitive:'primitiveTimeFormat' module: 'LocalePlugin'>
	^'h:mmx'
! !

!Locale methodsFor: 'system primitives' stamp: 'jannik.laval 2/4/2010 15:08'!
primTimezone
	"The offset from UTC in minutes, with positive offsets being towards the east.
	(San Francisco is in UTC -07*60 and Paris is in UTC +02*60 (daylight savings is not in effect)."
	<primitive:'primitiveTimezoneOffset' module: 'LocalePlugin'>
	^0! !

!Locale methodsFor: 'system primitives' stamp: 'tpr 6/2/2005 13:44'!
primVMOffsetToUTC
	"Returns the offset in minutes between the VM and UTC.
	If the VM does not support UTC times, this is 0.
	Also gives us backward compatibility with old VMs as the primitive will fail and we then can return 0."
	<primitive:'primitiveVMOffsetToUTC' module: 'LocalePlugin'>
	^0! !

!Locale methodsFor: 'accessing' stamp: 'tak 8/4/2005 15:18'!
printOn: aStream 
	super printOn: aStream.
	aStream nextPutAll: '(' , id printString , ')'! !

Object subclass: #LocaleID
	instanceVariableNames: 'isoLanguage isoCountry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Localization'!

!LocaleID class methodsFor: 'accessing' stamp: 'mir 7/15/2004 15:09'!
current
	^Locale current localeID! !

!LocaleID class methodsFor: 'instance creation' stamp: 'mir 7/15/2004 14:35'!
isoLanguage: langString
	^self isoLanguage: langString isoCountry: nil! !

!LocaleID class methodsFor: 'instance creation' stamp: 'mir 7/15/2004 12:46'!
isoLanguage: langString isoCountry: countryStringOrNil
	^self new isoLanguage: langString isoCountry: countryStringOrNil! !

!LocaleID class methodsFor: 'instance creation' stamp: 'mir 7/21/2004 13:59'!
isoString: isoString
	"Parse the isoString (<language>-<country>) into its components and return the matching LocaleID"
	"LocaleID isoString: 'en' "
	"LocaleID isoString: 'en-us' "

	| parts language country |
	parts := isoString findTokens: #($- ).
	language := parts first.
	parts size > 1
		ifTrue: [country := parts second].
	^self isoLanguage: language isoCountry: country! !

!LocaleID class methodsFor: 'instance creation' stamp: 'HilaireFernandes 4/30/2010 18:00'!
posixName: aString 
	^ self
		isoString: (aString copyReplaceAll: '_' with: '-')! !

!LocaleID class methodsFor: 'accessing' stamp: 'bf 9/26/2007 16:24'!
previous
	^Locale previous localeID! !

!LocaleID methodsFor: 'comparing' stamp: 'mir 7/15/2004 14:23'!
= anotherObject
	self class == anotherObject class
		ifFalse: [^false].
	^self isoLanguage = anotherObject isoLanguage
		and: [self isoCountry = anotherObject isoCountry]! !

!LocaleID methodsFor: 'accessing' stamp: 'mir 9/1/2005 14:17'!
displayCountry
	^(ISOLanguageDefinition iso2Countries at: self isoCountry asUppercase ifAbsent: [ self isoCountry ]) ! !

!LocaleID methodsFor: 'accessing' stamp: 'tak 3/23/2006 12:26'!
displayLanguage
	| language |
	language := (ISOLanguageDefinition iso2LanguageTable
				at: self isoLanguage
				ifAbsent: [^ self isoLanguage]) language.
	^ self isoCountry
		ifNil: [language]
		ifNotNil: [language , ' (' , self displayCountry , ')']! !

!LocaleID methodsFor: 'testing' stamp: 'mir 7/15/2004 14:34'!
hasParent
	^self isoCountry notNil! !

!LocaleID methodsFor: 'comparing' stamp: 'mir 7/15/2004 14:23'!
hash
	^self isoLanguage hash bitXor: self isoCountry hash! !

!LocaleID methodsFor: 'accessing' stamp: 'mir 7/15/2004 12:43'!
isoCountry
	^isoCountry! !

!LocaleID methodsFor: 'accessing' stamp: 'mir 7/15/2004 12:43'!
isoLanguage
	^isoLanguage! !

!LocaleID methodsFor: 'initialize' stamp: 'mir 7/15/2004 12:44'!
isoLanguage: langString isoCountry: countryStringOrNil
	isoLanguage := langString.
	isoCountry := countryStringOrNil! !

!LocaleID methodsFor: 'accessing' stamp: 'mir 7/21/2004 19:17'!
isoString
	^self asString! !

!LocaleID methodsFor: 'accessing' stamp: 'mir 7/15/2004 14:34'!
parent
	^self class isoLanguage: self isoLanguage! !

!LocaleID methodsFor: 'printing' stamp: 'tak 8/6/2007 12:16'!
posixName
	"(LocaleID isoString: 'es-MX') posixName"
	"(LocaleID isoString: 'es') posixName"
	"language[_territory]"
	^ self isoCountry
		ifNil: [self isoLanguage]
		ifNotNil: [self isoLanguage , '_' , self isoCountry]! !

!LocaleID methodsFor: 'printing' stamp: 'mir 7/15/2004 12:45'!
printOn: stream
	"<language>-<country>"
	stream nextPutAll: self isoLanguage.
	self isoCountry
		ifNotNil: [stream nextPut: $-; nextPutAll: self isoCountry]! !

!LocaleID methodsFor: 'printing' stamp: 'tak 11/15/2004 12:45'!
storeOn: aStream 
	aStream nextPut: $(.
	aStream nextPutAll: self class name.
	aStream nextPutAll: ' isoString: '.
	aStream nextPutAll: '''' , self printString , ''''.
	aStream nextPut: $).
! !

!LocaleID methodsFor: 'accessing' stamp: 'HilaireFernandes 5/6/2010 21:30'!
translator
	^ NaturalLanguageTranslator localeID: self ! !

Object subclass: #MczInstaller
	instanceVariableNames: 'stream zip'
	classVariableNames: 'Versions'
	poolDictionaries: ''
	category: 'System-Installers'!

!MczInstaller class methodsFor: 'versioninfo' stamp: 'avi 1/19/2004 13:13'!
clearVersionInfo
	Versions := Dictionary new! !

!MczInstaller class methodsFor: 'services' stamp: 'cwp 8/7/2003 18:49'!
extension
	^ 'mcz'! !

!MczInstaller class methodsFor: 'services' stamp: 'nk 6/8/2004 17:29'!
fileReaderServicesForFile: fileName suffix: suffix
	^({ self extension. '*' } includes: suffix)
		ifTrue: [ self services ]
		ifFalse: [#()].
! !

!MczInstaller class methodsFor: 'services' stamp: 'avi 3/7/2004 14:51'!
initialize
	self clearVersionInfo.
	self registerForFileList.! !

!MczInstaller class methodsFor: 'installing' stamp: 'cwp 8/7/2003 18:13'!
installFileNamed: aFileName
	self installStream: (FileStream readOnlyFileNamed: aFileName)! !

!MczInstaller class methodsFor: 'installing' stamp: 'cwp 8/7/2003 17:56'!
installStream: aStream
	(self on: aStream) install! !

!MczInstaller class methodsFor: 'services' stamp: 'cwp 8/7/2003 18:54'!
loadVersionFile: fileName
	self installFileNamed: fileName
! !

!MczInstaller class methodsFor: 'instance creation' stamp: 'cwp 8/7/2003 17:56'!
on: aStream
	^ self new stream: aStream! !

!MczInstaller class methodsFor: 'services' stamp: 'IgorStasenko 4/15/2011 17:17'!
registerForFileList
	Smalltalk globals at: #MCReader ifAbsent: [ Smalltalk tools fileList registerFileReader: self ]! !

!MczInstaller class methodsFor: 'services' stamp: 'tbn 8/11/2010 10:20'!
serviceLoadVersion
	^ SimpleServiceEntry
		provider: self
		label: 'Load'
		selector: #loadVersionFile:
		description: 'Load a package version'! !

!MczInstaller class methodsFor: 'services' stamp: 'ab 8/8/2003 18:01'!
services
	^ Array with: self serviceLoadVersion! !

!MczInstaller class methodsFor: 'versioninfo' stamp: 'cwp 8/11/2003 23:49'!
storeVersionInfo: aVersion
	Versions 
		at: aVersion package name
		put: aVersion info asDictionary! !

!MczInstaller class methodsFor: 'shrinking' stamp: 'EstebanLorenzano 7/27/2012 18:21'!
unloadMonticello
	"self unloadMonticello"

	self breakDependents.
	Smalltalk globals
		at: #MCWorkingCopy
		ifPresent: [ :wc | 
			wc allInstances
				do: [ :ea | 
					Versions at: ea package name put: ea currentVersionInfo asDictionary.
					ea breakDependents.
					SystemAnnouncer uniqueInstance unsubscribe: ea.
					]
				displayingProgress: 'Saving version info...' ].	"keep things simple and don't unload any class extensions"
	(Class superclassOrder: (PackageInfo named: 'Monticello') classes) reverseDo: [ :ea | ea removeFromSystem ].
	self registerForFileList! !

!MczInstaller class methodsFor: 'versioninfo' stamp: 'avi 2/17/2004 02:49'!
versionInfo
	^ Versions! !

!MczInstaller methodsFor: 'utilities' stamp: 'stephane.ducasse 3/31/2009 21:10'!
associate: tokens
	| result |
	result := Dictionary new.
	tokens pairsDo: [:key :value | 
					| tmp |
					tmp := value.
					value isString ifFalse: [tmp := value collect: [:ea | self associate: ea]].
					value = 'nil' ifTrue: [tmp := ''].
					result at: key put: tmp].
	^ result! !

!MczInstaller methodsFor: 'utilities' stamp: 'avi 2/17/2004 02:53'!
checkDependencies
	| dependencies unmet |
	dependencies := (zip membersMatching: 'dependencies/*') 
			collect: [:member | self extractInfoFrom: (self parseMember: member)].
	unmet := dependencies reject: [:dep |
		self versions: Versions anySatisfy: (dep at: #id)].
	^ unmet isEmpty or: [
		self confirm: (String streamContents: [:s|
			s nextPutAll: 'The following dependencies seem to be missing:'; cr.
			unmet do: [:each | s nextPutAll: (each at: #name); cr].
			s nextPutAll: 'Do you still want to install this package?'])]! !

!MczInstaller methodsFor: 'utilities' stamp: 'avi 2/17/2004 03:26'!
extractInfoFrom: dict
	dict at: #id put: (UUID fromString: (dict at: #id)).
	dict at: #date ifPresent: [:d | d isEmpty ifFalse: [dict at: #date put: (Date fromString: d)]].
	dict at: #time ifPresent: [:t | t isEmpty ifFalse: [dict at: #time put: (Time readFrom: t readStream)]].
	dict at: #ancestors ifPresent: [:a | dict at: #ancestors put: (a collect: [:ea | self extractInfoFrom: ea])].
	^ dict! !

!MczInstaller methodsFor: 'accessing' stamp: 'cwp 8/13/2003 01:58'!
extractPackageName
	^ (self parseMember: 'package') at: #name.
	! !

!MczInstaller methodsFor: 'accessing' stamp: 'cwp 8/13/2003 02:17'!
extractVersionInfo
	^ self extractInfoFrom: (self parseMember: 'version')! !

!MczInstaller methodsFor: 'installation' stamp: 'avi 2/17/2004 02:56'!
install
	| sources |
	zip := ZipArchive new.
	zip readFrom: stream.
	self checkDependencies ifFalse: [^false].
	self recordVersionInfo.
	sources := (zip membersMatching: 'snapshot/*') 
				asSortedCollection: [:a :b | a fileName < b fileName].
	sources do: [:src | self installMember: src].! !

!MczInstaller methodsFor: 'installation' stamp: 'GuillermoPolito 5/5/2012 23:10'!
installMember: member
	 
	self useNewChangeSetDuring:
		[ | str |str := member contentStream text.
		str setConverterForCode.
		CodeImporter evaluateReadStream: str readStream.
		]! !

!MczInstaller methodsFor: 'utilities' stamp: 'cwp 8/13/2003 01:58'!
parseMember: fileName
	| tokens |
	tokens := (self scanner scanTokens: (zip contentsOf: fileName)) first.
	^ self associate: tokens! !

!MczInstaller methodsFor: 'accessing' stamp: 'cwp 8/7/2003 19:18'!
recordVersionInfo
	Versions 
		at: self extractPackageName 
		put: self extractVersionInfo! !

!MczInstaller methodsFor: 'accessing' stamp: 'cwp 8/13/2003 02:04'!
scanner
	^ Scanner new! !

!MczInstaller methodsFor: 'accessing' stamp: 'avi 2/17/2004 02:55'!
stream: aStream
	stream := aStream! !

!MczInstaller methodsFor: 'utilities' stamp: 'bf 2/9/2004 13:56'!
useNewChangeSetDuring: aBlock
	| changeHolder oldChanges newChanges |
	changeHolder := (ChangeSet respondsTo: #newChanges:)
						ifTrue: [ChangeSet]
						ifFalse: [Smalltalk].
	oldChanges := (ChangeSet respondsTo: #current)
						ifTrue: [ChangeSet current]
						ifFalse: [Smalltalk changes].

	newChanges := ChangeSet new name: (ChangeSet uniqueNameLike: self extractPackageName).
	changeHolder newChanges: newChanges.
	[aBlock value] ensure: [changeHolder newChanges: oldChanges].! !

!MczInstaller methodsFor: 'utilities' stamp: 'bf 2/9/2004 15:00'!
versions: aVersionList anySatisfy: aDependencyID
	^ aVersionList anySatisfy: [:version | 
			aDependencyID = (version at: #id)
				or: [self versions: (version at: #ancestors) anySatisfy: aDependencyID]]! !

Object subclass: #MethodChangeRecord
	instanceVariableNames: 'changeType currentMethod infoFromRemoval'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Changes'!

!MethodChangeRecord commentStamp: '<historical>' prior: 0!
MethodChangeRecords are used to record method changes.  Here is a simple summary of the relationship between the changeType symbol and the recording of prior state

			|	prior == nil			|	prior not nil	
	---------	|----------------------------	|--------------------
	add		|	add					|	change
	---------	|----------------------------	|--------------------
	remove	|	addedThenRemoved	|	remove

Structure:
changeType			symbol -- as summarized above
currentMethod	method
				This is the current version of the method.
				It can be used to assert this change upon entry to a layer. 
infoFromRemoval -- an array of size 2.
				The first element is the source index of the last version of the method.
				The second element is the category in which it was defined, so it
				can be put back there if re-accepted from a version browser.

Note that the above states each have an associated revoke action:
	add --> remove
	change --> change back
	remove --> add back
	addedThenRemoved --> no change
However all of these are accomplished trivially by restoring the original method dictionary.!

!MethodChangeRecord methodsFor: 'as yet unclassified' stamp: 'di 4/1/2000 12:02'!
changeType

	^ changeType! !

!MethodChangeRecord methodsFor: 'as yet unclassified' stamp: 'di 4/1/2000 12:02'!
methodInfoFromRemoval
	"Return an array with the source index of the last version of the method,
	and the category in which it was defined (so it can be put back there if
	re-accepted from a version browser)."

	(changeType == #remove or: [changeType == #addedThenRemoved])
		ifTrue: [^ infoFromRemoval]
		ifFalse: [^ nil]! !

!MethodChangeRecord methodsFor: 'as yet unclassified' stamp: 'di 4/4/2000 11:05'!
noteChangeType: newChangeType

	(changeType == #addedThenRemoved and: [newChangeType == #change])
		ifTrue: [changeType := #add]
		ifFalse: [changeType := newChangeType]! !

!MethodChangeRecord methodsFor: 'as yet unclassified' stamp: 'di 4/1/2000 11:05'!
noteMethodInfoFromRemoval: info
	"Store an array with the source index of the last version of the method,
	and the category in which it was defined (so it can be put back there if
	re-accepted from a version browser)."

	infoFromRemoval := info! !

!MethodChangeRecord methodsFor: 'as yet unclassified' stamp: 'ar 6/3/2005 17:01'!
noteNewMethod: newMethod
	"NEVER do this. It is evil."
	currentMethod := nil.! !

!MethodChangeRecord methodsFor: 'printing' stamp: 'di 4/1/2000 12:02'!
printOn: strm

	super printOn: strm.
	strm nextPutAll: ' ('; print: changeType; nextPutAll: ')'! !

Object subclass: #Model
	instanceVariableNames: 'dependents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!Model commentStamp: '<historical>' prior: 0!
Provides a superclass for classes that function as models.  The only behavior provided is fast dependents maintenance, which bypasses the generic DependentsFields mechanism.  1/23/96 sw!

!Model methodsFor: 'dependents' stamp: 'sma 2/29/2000 19:54'!
myDependents
	^ dependents! !

!Model methodsFor: 'dependents' stamp: 'sma 2/29/2000 19:54'!
myDependents: aCollectionOrNil
	dependents := aCollectionOrNil! !

!Model methodsFor: 'copying' stamp: ''!
veryDeepFixupWith: deepCopier 
	"See if the dependents are being copied also.  If so, point at the new copies.  (The dependent has self as its model.)
	Dependents handled in class Object, when the model is not a Model, are fixed up in Object veryDeepCopy."

	| originalDependents refs |
	super veryDeepFixupWith: deepCopier.
	originalDependents := dependents.
	originalDependents ifNil: [
		^self.
		].
	dependents := nil.
	refs := deepCopier references.
	originalDependents
		do: [:originalDependent | | newDependent | 
			newDependent := refs
						at: originalDependent
						ifAbsent: [].
			newDependent
				ifNotNil: [self addDependent: newDependent]]! !

Model subclass: #SimpleServiceEntry
	instanceVariableNames: 'provider label selector usingLineAfter stateSelector description argumentGetter buttonLabel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-FileRegistry'!

!SimpleServiceEntry commentStamp: '<historical>' prior: 0!
I represent a service

provider : the service provider
label : to be display in a menu
selector : to do the service
useLineAfter
stateSelector : a secondary selector (to be able to query state of the provider for example)
description : a description for balloon for example
argumentGetter : a selector to get additional arguments with (if selector requres them)
buttonLabel : a short label

The entire client interface (provided by FileList and other users of the registry)
is this (browse #getArgumentsFrom: and the 
senders of #argumentGetter:):

fullName (returns a String with the full filename)
dirAndFileName (returns {directory. fileName})
readOnlyStream (returns an open read-only stream)
!

SimpleServiceEntry subclass: #FileModifyingSimpleServiceEntry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-FileRegistry'!

!FileModifyingSimpleServiceEntry commentStamp: 'nk 11/26/2002 12:03' prior: 0!
I represent a service that may change the contents of a directory.
Such changes include:
* file creation
* file deletion
* file modification!

!FileModifyingSimpleServiceEntry methodsFor: 'as yet unclassified' stamp: 'nk 11/26/2002 12:08'!
performServiceFor: anObject
	| retval |
	retval := super performServiceFor: anObject.
	self changed: #fileListChanged.
	^retval	"is this used anywhere?"! !

!SimpleServiceEntry class methodsFor: 'instance creation' stamp: 'yo 8/22/2008 20:58'!
provider: anObject label: aString selector: aSymbol

	^self provider: anObject label: aString selector: aSymbol description: ''.
! !

!SimpleServiceEntry class methodsFor: 'instance creation' stamp: 'sd 1/31/2002 22:05'!
provider: anObject label: aString selector: aSymbol description: anotherString

	^self new provider: anObject label: aString selector: aSymbol description: anotherString! !

!SimpleServiceEntry class methodsFor: 'instance creation' stamp: 'sw 2/17/2002 02:48'!
provider: anObject label: aString selector: aSymbol description: anotherString buttonLabel: aLabel
	"Answer a new service object with the given initializations.  This variant allows a custom button label to be provided, in order to preserve precious horizontal real-estate in the button pane, while still allowing more descriptive wordings in the popup menu"

	^ self new provider: anObject label: aString selector: aSymbol description: anotherString; buttonLabel: aLabel; yourself! !

!SimpleServiceEntry methodsFor: 'services menu' stamp: 'StephaneDucasse 3/5/2010 14:41'!
addServiceFor: served toMenu: aMenu
	aMenu add: self label 
		target: self 
		selector: #performServiceFor: "self requestSelector "
		argument: served.
	self usingLineAfter ifTrue: [ aMenu addLine ].! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'RAA 2/2/2002 08:14'!
argumentGetter: aBlock

	argumentGetter := aBlock! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'sw 2/15/2002 17:53'!
buttonLabel
	"Answer the label to be emblazoned on a button representing the service in a file list, for example"

	^ buttonLabel ifNil: [label]! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'dgd 9/1/2003 12:16'!
buttonLabel: aString 
	"Set the receiver's buttonLabel, to be used on a button in a 
	tool-pane; this is split out so that a more abbreviated wording 
	can be deployed if desired"
	buttonLabel := aString translated! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'sd 1/31/2002 21:03'!
description
	"may be used for balloon or other"

	^ description
! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'sd 1/31/2002 21:03'!
description: aString
	"may be used for balloon or other"

	description := aString
! !

!SimpleServiceEntry methodsFor: 'services menu' stamp: 'StephaneDucasse 3/5/2010 14:41'!
doNotUseLineAfter

	self usingLineAfter: false! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'sd 1/31/2002 21:10'!
extraSelector
	"normally should not be used directly"

	^stateSelector! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'sd 1/31/2002 21:11'!
extraSelector: aSymbol

	stateSelector := aSymbol! !

!SimpleServiceEntry methodsFor: 'performing service' stamp: 'RAA 2/2/2002 08:31'!
getArgumentsFrom: aProvider

	argumentGetter ifNil: [^aProvider fullName].
	^argumentGetter value: aProvider! !

!SimpleServiceEntry methodsFor: 'initialize-release' stamp: 'StephaneDucasse 3/5/2010 14:36'!
initialize
	
	super initialize.
	self useLineAfter! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'tak 9/25/2008 16:09'!
label
	^label! !

!SimpleServiceEntry methodsFor: 'extra' stamp: 'nk 6/8/2004 16:42'!
performExtraFor: anObject
	"carry out the extra service I provide"
	"the stateSelector can be used to ask state of the provider to be reflected in button or other"

	^stateSelector numArgs = 0
		ifTrue: [provider perform: stateSelector]
		ifFalse: [provider perform: stateSelector with: (self getArgumentsFrom: anObject) ]
! !

!SimpleServiceEntry methodsFor: 'performing service' stamp: 'nk 8/31/2004 19:30'!
performServiceFor: anObject
	"carry out the service I provide"

	^selector numArgs = 0
		ifTrue: [provider perform: selector]
		ifFalse: [
			selector numArgs = 1
				ifTrue: [ provider perform: selector with: (self getArgumentsFrom: anObject) ]
				ifFalse: [ provider perform: selector withArguments: (self getArgumentsFrom: anObject) ]]! !

!SimpleServiceEntry methodsFor: 'printing' stamp: 'nk 10/14/2003 10:04'!
printOn: aStream

	aStream nextPutAll: self class name; nextPutAll: ': ('.
	self provider notNil
		ifTrue: [aStream nextPutAll: provider printString].
	aStream nextPutAll: ' --- '. 
	self selector notNil
		ifTrue: [aStream nextPutAll: selector asString].
	aStream nextPut: $)

! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'sd 1/31/2002 21:38'!
provider

	^ provider! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'dgd 9/1/2003 12:12'!
provider: anObject label: aString selector: aSymbol 
	"basic initialization message"
	provider := anObject.
	label := aString translated.
	selector := aSymbol.
	stateSelector := #none.
	description := ''! !

!SimpleServiceEntry methodsFor: 'performing service' stamp: 'dgd 9/1/2003 12:13'!
provider: anObject label: aString selector: aSymbol description: anotherString 
	"basic initialization message"
	self
		provider: anObject
		label: aString
		selector: aSymbol.
	stateSelector := #none.
	description := anotherString translated! !

!SimpleServiceEntry methodsFor: 'extra' stamp: 'sd 1/31/2002 21:08'!
requestExtraSelector
	"send me this message to ask me to perform secondary service"

	^#performExtraFor:
! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'sd 1/31/2002 21:09'!
selector
	"normally should not be used directly"

	^selector! !

!SimpleServiceEntry methodsFor: 'services menu' stamp: 'StephaneDucasse 3/5/2010 14:42'!
useLineAfter

	self usingLineAfter: true! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'StephaneDucasse 3/5/2010 14:40'!
usingLineAfter

	^ usingLineAfter! !

!SimpleServiceEntry methodsFor: 'accessing' stamp: 'StephaneDucasse 3/5/2010 14:41'!
usingLineAfter: aBoolean

	usingLineAfter := aBoolean! !

Object subclass: #NaturalLanguageFormTranslator
	instanceVariableNames: 'id generics'
	classVariableNames: 'CachedTranslations'
	poolDictionaries: ''
	category: 'System-Localization'!

!NaturalLanguageFormTranslator commentStamp: 'LaurentLaffont 3/4/2011 22:44' prior: 0!
Provides support for looking up Forms by name for presentation in the UI.
Different forms can be registered for a name for different locales allowing
images presented in the UI to be localised.

Typically used where images contain language dependent text.

E.g.

buttonForm := (NaturalLanguageFormTranslator localeID: Locale current localeID)
  translate: 'submit button'


Form translations are added like so:

(NaturalLanguageFormTranslator isoLanguage: 'en' isoCountry: 'gb')
  name: 'submit button' form: aForm!

!NaturalLanguageFormTranslator class methodsFor: 'accessing' stamp: 'yo 1/13/2005 11:13'!
cachedTranslations
	"CachedTranslations := nil" 
	^CachedTranslations ifNil: [CachedTranslations := Dictionary new]! !

!NaturalLanguageFormTranslator class methodsFor: 'cleanup' stamp: 'StephaneDucasse 3/9/2010 22:11'!
cleanUp
	"Flush caches"

	CachedTranslations := nil! !

!NaturalLanguageFormTranslator class methodsFor: 'accessing' stamp: 'yo 1/13/2005 11:13'!
isoLanguage: isoLanguage
	"Return the generic language translator as there is no information about the country code"

	^self isoLanguage: isoLanguage isoCountry: nil! !

!NaturalLanguageFormTranslator class methodsFor: 'accessing' stamp: 'yo 1/13/2005 11:13'!
isoLanguage: isoLanguage isoCountry: isoCountry
	^self localeID: (LocaleID  isoLanguage: isoLanguage isoCountry: isoCountry)! !

!NaturalLanguageFormTranslator class methodsFor: 'accessing' stamp: 'yo 1/13/2005 11:13'!
localeID: localeID 
	^ self cachedTranslations
		at: localeID
		ifAbsentPut: [self new localeID: localeID]! !

!NaturalLanguageFormTranslator methodsFor: 'accessing' stamp: 'yo 1/13/2005 11:15'!
generics
	^generics ifNil: [generics := Dictionary new]! !

!NaturalLanguageFormTranslator methodsFor: 'accessing' stamp: 'yo 1/13/2005 11:27'!
localeID
	^id! !

!NaturalLanguageFormTranslator methodsFor: 'accessing' stamp: 'yo 1/13/2005 11:26'!
localeID: anID
	id := anID! !

!NaturalLanguageFormTranslator methodsFor: 'accessing' stamp: 'yo 1/13/2005 11:17'!
name: formName form: translatedForm 
	self generics at: formName put: translatedForm.
! !

!NaturalLanguageFormTranslator methodsFor: 'utilities' stamp: 'yo 1/13/2005 11:35'!
translate: aString

	^ (self generics
		at: aString ifAbsent: [nil]) deepCopy.

	"Do you like to write 'form ifNotNil: [form deepCopy]'?"
! !

Object subclass: #NaturalLanguageTranslator
	instanceVariableNames: ''
	classVariableNames: 'AllKnownPhrases'
	poolDictionaries: ''
	category: 'System-Localization'!

!NaturalLanguageTranslator commentStamp: 'HilaireFernandes 5/13/2010 11:48' prior: 0!
A NaturalLanguageTranslator is a dummy translator.
The localization framework is found in the gettext package.
!

!NaturalLanguageTranslator class methodsFor: 'cleanup' stamp: 'MarcusDenker 4/22/2011 10:34'!
cleanUp
	AllKnownPhrases := nil! !

!NaturalLanguageTranslator class methodsFor: 'translate' stamp: 'HilaireFernandes 5/13/2010 11:43'!
translate: aString
	^ aString! !

!NaturalLanguageTranslator class methodsFor: 'translate' stamp: 'HilaireFernandes 5/13/2010 11:43'!
translate: aString toLocale: localeID
	^ aString! !

Object subclass: #OSPlatform
	instanceVariableNames: ''
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'System-Platforms'!

!OSPlatform commentStamp: 'michael.rueger 2/25/2009 18:29' prior: 0!
An OSPlatform is an abstract representation of a 'OS platform'.
Platforms can be hierarchical, e.g., a "general" platform as superclass and more specific platforms as subclasses as long as the subclasses provide sufficient means to identify themselves.
The original implementation was for Tweak.

Current		holds the current OSPlatform subclass

Architectural considerations:
most platform specific methods that need to be added to the platform class should be in the form of extensions rather then adding them directly to this package. Otherwise the platform class will degenerate very quickly into a dependence hub for all kinds of sub systems.!

OSPlatform subclass: #MacOSXPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Platforms'!

!MacOSXPlatform class methodsFor: 'private' stamp: 'MarcusDenker 7/13/2012 14:30'!
isActivePlatform
	^OSPlatform isMacOS and: [OSPlatform version asNumber >= 1000]! !

!MacOSXPlatform methodsFor: '*System-Clipboard' stamp: 'michael.rueger 3/2/2009 10:56'!
clipboardClass
	^MacOSClipboard! !

!MacOSXPlatform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 18:18'!
platformFamily
	^#MacOSX! !

!OSPlatform class methodsFor: 'accessing' stamp: 'ar 12/11/2004 23:04'!
current
	"Answer the current platform"
	^Current! !

!OSPlatform class methodsFor: 'private' stamp: 'michael.rueger 2/25/2009 18:20'!
determineActivePlatformStartingAt: parentClass
	"Determine the current platform starting at parentClass.
	This is a potentially recursive process as we want to determine the most specific (in terms of subclasses) platform available."
	"OSPlatform determineActivePlatformStartingAt: OSPlatform"

	| platformClass |
	parentClass subclasses isEmpty
		ifTrue: [^parentClass].

	"Look for the matching platform class"
	platformClass := parentClass allSubclasses detect:[:any| any isActivePlatform] ifNone:[nil].

	"Check if there is a more specific subclass"
	^self determineActivePlatformStartingAt: platformClass! !

!OSPlatform class methodsFor: 'class initialization' stamp: 'michael.rueger 3/2/2009 11:16'!
initialize
	"Initialize the receiver"
	"OSPlatform initialize"

	Smalltalk removeFromStartUpList: self.
	Smalltalk addToStartUpList: self after: Delay.
	Smalltalk removeFromShutDownList: self.
	Smalltalk addToShutDownList: self after: DisplayScreen.
	self startUp: true.! !

!OSPlatform class methodsFor: 'private' stamp: 'ar 12/11/2004 22:22'!
isActivePlatform
	"Answer whether the receiver is the active platform"
	^false! !

!OSPlatform class methodsFor: 'testing' stamp: 'dfsdfsd 6/11/2010 15:44'!
isMacOS

	^ self platformName = 'Mac OS'! !

!OSPlatform class methodsFor: 'testing' stamp: 'MarcusDenker 7/13/2012 14:30'!
isMacOSX
	^ self isMacOS
		and: ['10*' match: self version]! !

!OSPlatform class methodsFor: 'testing' stamp: 'dfsdfsd 6/11/2010 15:45'!
isUnix

	^ self platformName = 'unix'! !

!OSPlatform class methodsFor: 'testing' stamp: 'dfsdfsd 6/11/2010 15:45'!
isWin32

	^ self platformName = 'Win32' ! !

!OSPlatform class methodsFor: 'testing' stamp: 'dfsdfsd 6/11/2010 15:46'!
isX11

	^ self isUnix and: [self windowSystemName = 'X11']! !

!OSPlatform class methodsFor: 'system attributes' stamp: ''!
platformName
	"Return the name of the platform we're running on"

	^ Smalltalk vm getSystemAttribute: 1001! !

!OSPlatform class methodsFor: 'system attributes' stamp: ''!
platformSubtype
	"Return the subType of the platform we're running on"

	^ self subtype! !

!OSPlatform class methodsFor: 'class initialization' stamp: 'ar 12/11/2004 22:49'!
shutDown: quitting
	"The system is going down"
	Current ifNotNil:[Current shutDown: quitting].
! !

!OSPlatform class methodsFor: 'class initialization' stamp: 'michael.rueger 2/25/2009 18:20'!
startUp: resuming
	"Determine the current platform.
	Use the most specific (in terms of subclasses) platform available."

	| platformClass |
	"Look for the matching platform class"
	platformClass := self determineActivePlatformStartingAt: self.
	platformClass
		ifNil: [^self].
	Current := platformClass new.
	Current startUp: resuming! !

!OSPlatform class methodsFor: 'system attributes' stamp: ''!
subtype
	"Return the subType of the platform we're running on"
	"Smalltalk platformSubtype"

	^ (Smalltalk vm getSystemAttribute: 1003)! !

!OSPlatform class methodsFor: 'system attributes' stamp: 'CamilloBruni 5/13/2012 19:37'!
version
	"Return the version number string of the platform we're running on"
	"OSPlatform osVersion"

	^(Smalltalk vm getSystemAttribute: 1002) asString! !

!OSPlatform class methodsFor: 'accessing' stamp: 'michael.rueger 2/27/2009 17:25'!
virtualKey: virtualKeyCode
	"Subclass responsibility to override if necessary"
	^nil! !

!OSPlatform class methodsFor: 'system attributes' stamp: 'StephaneDucasse 8/18/2011 11:34'!
vmVersion
	"Return the version number string of the Virtual machine built"
	"OSPlatform vmVersion"

	^Smalltalk vm version! !

!OSPlatform class methodsFor: 'system attributes' stamp: 'StephaneDucasse 8/18/2011 11:34'!
windowSystemName			
	"Return the name of the window system currently being used for display."
	"OSPlatform windowSystemName"

	^ Smalltalk vm getSystemAttribute: 1005! !

!OSPlatform methodsFor: '*System-Clipboard' stamp: 'AlainPlantec 3/2/2010 14:41'!
clipboardClass
	^Clipboard! !

!OSPlatform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 18:18'!
platformFamily
	"Returns a symbol specific to the platform family (MacOSX, Windows, Unix, RiscOS).
	This may need to be extended if there are new platforms added or significant differences within a platform family arise (as was the case between MacOS 9 and X)."

	self subclassResponsibility! !

!OSPlatform methodsFor: 'initialize' stamp: 'VeronicaUquillas 6/11/2010 14:40'!
shutDown: quitting
	"Pharo is shutting down. If this platform requires specific shutdown code, this is a great place to put it."
! !

!OSPlatform methodsFor: 'initialize' stamp: 'VeronicaUquillas 6/11/2010 14:40'!
startUp: resuming
	"Pharo is starting up. If this platform requires specific intialization, this is a great place to put it."
! !

!OSPlatform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 22:19'!
virtualKey: virtualKeyCode
	^self class virtualKey: virtualKeyCode! !

OSPlatform subclass: #UnixPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Platforms'!

!UnixPlatform class methodsFor: 'private' stamp: 'simon.denier 6/11/2010 17:43'!
isActivePlatform
	^OSPlatform isUnix! !

!UnixPlatform methodsFor: '*System-Clipboard' stamp: 'HenrikSperreJohansen 1/28/2011 15:21'!
clipboardClass
	^UnixClipboard! !

!UnixPlatform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 18:18'!
platformFamily
	^#Unix! !

OSPlatform subclass: #Win32Platform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Platforms'!

!Win32Platform class methodsFor: 'private' stamp: 'simon.denier 6/11/2010 17:43'!
isActivePlatform
	"Answer whether the receiver is the active platform"
	^OSPlatform isWin32! !

!Win32Platform class methodsFor: 'accessing' stamp: 'michael.rueger 3/2/2009 11:20'!
virtualKey: virtualKeyCode
	"Win32Platform virtualKey: $C charCode"

	(virtualKeyCode <=  90 "$Z charCode"
		and: [virtualKeyCode >=  65 "$A "])
		ifFalse: [^nil].
	"#($a $b $c $d $e $f $g $h $i $j $k $l $m $n $o $p $q $r $s $t $u $v $w $x $y $z)"
	
	^(#($a nil $c $d nil $f $g nil nil nil nil $l $m $n nil $p nil nil $s nil nil $v nil $x nil $z)
		at: virtualKeyCode-64) ifNotNil: [:char | char charCode]! !

!Win32Platform methodsFor: 'accessing' stamp: 'michael.rueger 2/25/2009 18:18'!
platformFamily
	^#Windows! !

!Object class methodsFor: '*System-Support' stamp: ''!
registerToolsOn: aToolRegistry

	" Override to register any tools for Smalltalk tools registry. " ! !

!Object methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 3/24/2009 23:21'!
settingFixedDomainValueNodeFrom: aSettingNode 
	^ aSettingNode fixedDomainValueNodeForObject: self! !

!Object methodsFor: '*System-Settings-Browser' stamp: 'alain.plantec 3/19/2009 09:50'!
settingStoreOn: aStream
	^ self storeOn: aStream! !

!Object methodsFor: '*System-Support' stamp: 'MarcusDenker 7/12/2012 17:58'!
systemNavigation

	^ SystemNavigation new! !

Object subclass: #PragmaCollector
	instanceVariableNames: 'announcer collected filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-PragmaCollector'!

!PragmaCollector commentStamp: 'AlainPlantec 11/28/2009 01:09' prior: 0!
A PragmaCollector is used in order to collect some Pragma instances. A PragmaCollector makes use of SystemChangeNotifier event notifications in order to maintain its contents up-to-date according to its filter: when a method is added, removed or updated, if the method is defined with a pragma which is acceptable according to its filter, then the collector contents is updated. A PragmaCollector makes use of an announcer in order to notify all registered listeners when a pragma is added, removed or updated. A PragmaAnnouncement is announced when a Pragma is added, removed or updated. Corresponding announcement classes are, respectiveley, PragmaAdded, PragmaRemoved and PragmaUpdated. 

Explore the result of the expression below. In the collected instance variable should be stored all pragmas of the system:
---------------------------
(PragmaCollector filter: [:pragma | true]) reset
---------------------------

In the following example, collected pragma are thoses with the 'primitive:' keyword (<primitive:>)
---------------------------
(PragmaCollector filter: [:prg | prg keyword = 'primitive:']) reset
---------------------------

Instance Variables	
	announcer:		<Announcer>	
	collected:		<Collection>
	filter:			<Block or MessageSend>
				
announcer
	the announcer which is used to announce the adding, the removing or the updating of a method with an acceptable pragma declaration

collected		
	the current collection of Pragma
	
filter
	a block or a message send which is used in order to filter the pragma. This is a one argument valuable. When evaluated, the candidate pragam is passed as argument and the result must be a boolean. 
	!

!PragmaCollector class methodsFor: 'utilities' stamp: 'MarcusDenker 7/12/2012 17:59'!
allSystemPragmas
	^ (Array
		streamContents: [:stream | SystemNavigation new
				allBehaviorsDo: [:behavior | Pragma
						withPragmasIn: behavior
						do: [:pragma | stream nextPut: pragma]]]) ! !

!PragmaCollector class methodsFor: 'instance creation' stamp: 'alain.plantec 10/19/2009 16:16'!
filter: aOneArgValuable
	^ self new filter: aOneArgValuable! !

!PragmaCollector methodsFor: 'updating' stamp: 'alain.plantec 10/19/2009 10:54'!
addPragma: aPragma
	"if aPragma is to be kept, then add it and announce its adding"
	(self keepPragma: aPragma)
		ifTrue: [self collected add: aPragma.
			self announce: (PragmaAdded pragma: aPragma)]! !

!PragmaCollector methodsFor: 'system changes' stamp: 'alain.plantec 10/19/2009 10:55'!
addedEventOccurs: anEvent
	"method adding event occured: if the 
	concerned method contains a pragma then 
	try to update myself with it"
	Pragma
		withPragmasIn: anEvent itemClass
		do: [:pragma | pragma selector = anEvent itemSelector
				ifTrue: [self addPragma: pragma]] ! !

!PragmaCollector methodsFor: 'subscription' stamp: 'alain.plantec 9/18/2009 15:26'!
announce: anAnnouncement
	"see Announcements packages"
	self announcer ifNotNil: [announcer announce: anAnnouncement]! !

!PragmaCollector methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 15:26'!
announcer
	^ announcer! !

!PragmaCollector methodsFor: 'system changes' stamp: 'StephaneDucasse 6/17/2012 17:40'!
classRemovedEventOccurs: anEvent
	"a class has been removed: first see if the class is not my class
	because then I must be unplugged from system event notifications"
	anEvent removedClass = self class
		ifTrue: [^ self noMoreNotifications].
	"remove all handler which are from the removed class"
	(self pragmasOfClass: anEvent removedClass class)
		do: [:handler | self removePragma: handler] ! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
collect: aBlock
	^ self collected collect: aBlock
	! !

!PragmaCollector methodsFor: 'accessing' stamp: 'alain.plantec 10/20/2009 13:15'!
collected
	^ collected ifNil: [collected := OrderedCollection new] ! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
detect: aBlock
	^ self collected detect: aBlock ifNone:[]
	! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
do: aBlock
	self collected do: aBlock
	! !

!PragmaCollector methodsFor: 'accessing' stamp: 'alain.plantec 10/19/2009 10:59'!
filter
	^ filter ifNil: [filter := [:prg | true]]! !

!PragmaCollector methodsFor: 'accessing' stamp: 'alain.plantec 10/19/2009 11:00'!
filter: aOneArgValuable
	filter := aOneArgValuable! !

!PragmaCollector methodsFor: 'testing' stamp: 'alain.plantec 9/18/2009 15:26'!
ifNotEmpty: aBlock
	self collected ifNotEmpty: aBlock! !

!PragmaCollector methodsFor: 'initializing' stamp: 'alain.plantec 9/18/2009 15:26'!
initialize
	super initialize.
	announcer := Announcer new.
	self installSystemNotifications! !

!PragmaCollector methodsFor: 'system changes' stamp: 'GuillermoPolito 6/16/2012 17:03'!
installSystemNotifications
	"Allows myself to be kept up-to-date regarding system changes"
	SystemAnnouncer current weak
			on: ClassRemoved send: #classRemovedEventOccurs: to: self;
			on: MethodRemoved send: #removedEventOccurs: to: self;
			on: MethodAdded send: #addedEventOccurs: to: self;
			on: MethodModified send: #modifiedEventOccurs: to: self! !

!PragmaCollector methodsFor: 'testing' stamp: 'alain.plantec 9/18/2009 15:26'!
isEmpty
	^ self collected isEmpty! !

!PragmaCollector methodsFor: 'testing' stamp: 'alain.plantec 9/18/2009 15:26'!
isNotEmpty
	^ self collected isNotEmpty! !

!PragmaCollector methodsFor: 'updating' stamp: 'alain.plantec 10/19/2009 10:58'!
keepPragma: aPragma
	^ self filter value: aPragma
! !

!PragmaCollector methodsFor: 'system changes' stamp: 'alain.plantec 10/19/2009 10:55'!
modifiedEventOccurs: anEvent 
	"
	a method has been updated: try to update an handler:
	1 - I already have one for the method, then
	do nothing, 
	2 - I do not have one but method has an acceptable
	pragma in it, then I try to add a new handler
	3 - I have one but changed method has no more
	acceptable pragma in it, then the handler is removed.
	"
	Pragma
		withPragmasIn: anEvent itemClass
		do: [:pragma | pragma selector = anEvent itemSelector
				ifTrue: [(self 
								detect: [:oldprag | oldprag selector = pragma selector
										and: [oldprag methodClass = anEvent itemClass]])
						ifNotNil: [:oldprag | ^ self updatePragma: oldprag].
					^ self addPragma: pragma]].
	"No pragma but an handler for the method"
	(self pragmaWithSelector: anEvent itemSelector inClass: anEvent itemClass)
		ifNotNil: [:found | self removePragma: found]! !

!PragmaCollector methodsFor: 'system changes' stamp: 'alain.plantec 9/18/2009 15:26'!
noMoreAnnounceWhile: aBlock
	"unplug the announcer during aBlock"
	| oldAnnouncer |
	[oldAnnouncer := announcer.
	announcer := nil.
	aBlock value]
		ensure: [announcer := oldAnnouncer]! !

!PragmaCollector methodsFor: 'system changes' stamp: 'GuillermoPolito 6/16/2012 17:02'!
noMoreNotifications
	"Do not receiver any system change notification anymore"
	SystemAnnouncer current
		unsubscribe: self.! !

!PragmaCollector methodsFor: 'system changes' stamp: 'alain.plantec 9/18/2009 15:26'!
noMoreNotificationsWhile: aBlock
	"don not receive any system change notification during aBloc"
	self noMoreNotifications.
	[ aBlock value ] ensure: [ self installSystemNotifications ]! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
noneSatisfy: aBlock
	^ self collected noneSatisfy: aBlock! !

!PragmaCollector methodsFor: 'querying' stamp: 'alain.plantec 9/18/2009 15:44'!
pragmaWithSelector: aSelector inClass: aClass 
	"return the handler corresponding to a pragma method which selector is aSelector in class aClass"
	^ (self detect: [:prag | prag methodClass = aClass and: [prag  selector = aSelector]])
	! !

!PragmaCollector methodsFor: 'querying' stamp: 'alain.plantec 9/18/2009 15:30'!
pragmasOfClass: aClass 
	"return all handlers of class aClass"
	^ self	select: [:prag | prag methodClass = aClass ]! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
reject: aBlock
	^ self collected reject: aBlock ! !

!PragmaCollector methodsFor: 'dependents access' stamp: 'alain.plantec 9/18/2009 22:09'!
release
	self noMoreNotifications.
	announcer := nil.
	collected := nil.
	super release! !

!PragmaCollector methodsFor: 'updating' stamp: 'alain.plantec 10/19/2009 10:48'!
removePragma: aPragma 
	"remove an handler an announce it"
	self collected remove: aPragma ifAbsent: [].
	self announce: (PragmaRemoved pragma: aPragma)! !

!PragmaCollector methodsFor: 'system changes' stamp: 'alain.plantec 9/18/2009 15:40'!
removedEventOccurs: anEvent
	"a method has been removed, remove any corresponding handler if found"
	(self
				detect: [:prag | prag selector = anEvent itemSelector
						and: [prag methodClass = anEvent itemClass ]])
		ifNotNil: [:found | self removePragma: found]! !

!PragmaCollector methodsFor: 'initializing' stamp: 'alain.plantec 10/19/2009 11:46'!
reset
	"reinitialize current system settings"
	self
		noMoreAnnounceWhile: [self collected copy
				do: [:pragma | self removePragma: pragma].
			self class allSystemPragmas
				do: [:pragma | self addPragma: pragma]].
	self announce: (PragmaCollectorReset collector: self)! !

!PragmaCollector methodsFor: 'enumerating' stamp: 'alain.plantec 9/18/2009 15:26'!
select: aBlock
	^ self collected select: aBlock
	! !

!PragmaCollector methodsFor: 'subscription' stamp: 'alain.plantec 9/18/2009 15:26'!
subscribe: anAnnouncement send: aSelector to: anObject
	"see Announcements packages"
	self announcer subscribe: anAnnouncement send: aSelector to: anObject

! !

!PragmaCollector methodsFor: 'subscription' stamp: 'alain.plantec 9/18/2009 15:26'!
unsubscribe: anObject
	"see Announcements packages"
	self announcer unsubscribe: anObject

! !

!PragmaCollector methodsFor: 'updating' stamp: 'alain.plantec 10/19/2009 10:49'!
updatePragma: aPragma
	"only announce that the pragma has been updated"
	self announcer announce: (PragmaUpdated pragma: aPragma)! !

!PragmaCollector methodsFor: 'subscription' stamp: 'alain.plantec 9/18/2009 15:26'!
whenChangedSend: aSelector to: anObject
	"record a change listener"
	self subscribe: PragmaAnnouncement send: aSelector to: anObject
! !

!PragmaCollector methodsFor: 'subscription' stamp: 'alain.plantec 10/19/2009 10:42'!
whenResetSend: aSelector to: anObject
	"record a change listener"
	self subscribe: PragmaCollectorReset send: aSelector to: anObject
! !

Object subclass: #PragmaSetting
	instanceVariableNames: 'precondition target targetSelector name label description order icon ordering dialog allowedInStyle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!PragmaSetting commentStamp: 'AlainPlantec 1/3/2011 10:54' prior: 0!
A PragmaSetting is a holder for a setting. A setting is declared in a class method which contains a pragma (as examples, <menu> or <setting>). 
Settings can be collected on the fly by a PragmaCollector and browsed by a SettingBrowser. Settings are organized in trees.

Instance Variables
	allowedInStyle:		<Object>
	description:		<Object>
	dialog:		<Object>
	icon:		<Object>
	label:		<Object>
	name:		<Object>
	order:		<Object>
	ordering:		<Object>
	precondition:		<Object>
	target:		<Object>
	targetSelector:		<Object>

allowedInStyle
	- xxxxx

description
	- xxxxx

dialog
	- xxxxx

icon
	- xxxxx

label
	- xxxxx

name
	- xxxxx

order
	- xxxxx

ordering
	- xxxxx

precondition
	- xxxxx

target
	- xxxxx

targetSelector
	- xxxxx
!

!PragmaSetting methodsFor: 'comparing' stamp: 'alain.plantec 4/20/2009 15:18'!
= other 
	^ self species = other species
		and: [self name = other name]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 13:58'!
allowedInStyle
	^ allowedInStyle ifNil: [allowedInStyle := self hasValue]! !

!PragmaSetting methodsFor: 'user interface' stamp: 'alain.plantec 3/11/2009 15:38'!
asString
	^ self name! !

!PragmaSetting methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:13'!
chooseFileDirectory
	| result |
	result := self theme chooseDirectoryIn: World title: 'Choose a file' path: nil.
	result ifNotNil: [ self realValue: result fullName ].! !

!PragmaSetting methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:13'!
chooseFilePath
	| result |
	result := self theme fileOpenIn: World title: 'Choose a file' extensions: nil path: nil preview: true.
	result 
		ifNotNil: [
			self realValue: result name. 
			result close]! !

!PragmaSetting methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:13'!
defaultIcon
	^ self theme smallConfigurationIcon! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/13/2009 15:52'!
description
	"Answer the value of description"

	^ description ifNil: [description := '']! !

!PragmaSetting methodsFor: 'accessing' stamp: 'alain.plantec 3/25/2009 19:36'!
description: aText
	"Set the value of description"

	description := aText! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/30/2010 08:07'!
dialog
	^ dialog! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/30/2010 07:59'!
dialog: aValuable
	dialog := aValuable! !

!PragmaSetting methodsFor: 'user interface' stamp: 'alain.plantec 3/12/2009 12:58'!
enabled
	^ true! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:12'!
fontButtonLabel
	| emphases |
	emphases := IdentityDictionary new at: nil put: 'Regular' translated;
				 at: 0 put: 'Regular' translated;
				 at: 1 put: 'Bold' translated;
				 at: 2 put: 'Italic' translated;
				 at: 4 put: 'Underlined' translated;
				 at: 8 put: 'Narrow' translated; 
				 at: 16 put: 'StruckOut' translated;
				 yourself.
	^ StringMorph contents: self realValue familyName , ' '
			, (emphases
					at: self realValue emphasis
					ifAbsent: ['']) , ' ' , self realValue pointSize asString font: self realValue! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:12'!
getFont
	^ FontChooser
		openWithWindowTitle: 'Font for ' , self name
		for: self
		setSelector: #realValue:
		getSelector: #realValue ! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 14:02'!
hasDefault
	^ false! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 14:03'!
hasEditableList
	^ false! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 14:03'!
hasValue
	^ false! !

!PragmaSetting methodsFor: 'comparing' stamp: 'alain.plantec 4/20/2009 15:19'!
hash
	^ self species hash bitXor: self name hash ! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 08:27'!
icon
	^ icon ifNil: [icon := self defaultIcon]! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 11/23/2009 22:18'!
icon: aForm 
	icon := aForm! !

!PragmaSetting methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:13'!
inputMorphFor: aContainer 
	^ self inputWidget
		ifNotNil: [:iw | 
			( self theme newRowIn: World for: {iw})
				 clipSubmorphs: true;
				 hResizing: #shrinkWrap;
				 cellInset: 0;
				 yourself]! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/30/2010 08:07'!
inputWidget
	"return the default widget for the input a the setting"
	^ self dialog
		ifNotNil: [:d | d numArgs = 0
				ifTrue: [d value]
				ifFalse: [d value: self]] ! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 14:11'!
label
	^ label ifNil: [self name] ifNotNil: [label value]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 14:11'!
label: aStringOrBlockOrMessageSend
	label := aStringOrBlockOrMessageSend! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 2/11/2011 14:09'!
labelMorphFor: aContainer
	^ StringMorph contents: (aContainer model viewedLabelOfSetting: self)! !

!PragmaSetting methodsFor: 'accessing' stamp: 'alain.plantec 10/18/2009 12:40'!
name
	^ name ifNil: [super name] ifNotNil: [name value]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'alain.plantec 10/18/2009 12:39'!
name: aNameOrBlockOrMessageSend
	name := aNameOrBlockOrMessageSend! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 08:18'!
noOrdering
	ordering := false! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:04'!
notInStyle
	allowedInStyle := false! !

!PragmaSetting methodsFor: 'accessing' stamp: 'alain.plantec 4/2/2009 11:59'!
order
	^ order! !

!PragmaSetting methodsFor: 'accessing' stamp: 'alain.plantec 4/2/2009 10:53'!
order: aNumber
	order := aNumber! !

!PragmaSetting methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 08:18'!
ordering
	^ ordering ifNil: [ordering := true]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 08:42'!
precondition
	^ precondition ifNil: [precondition := [true]].! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 08:43'!
precondition: aValuable
	precondition := aValuable! !

!PragmaSetting methodsFor: 'accessing' stamp: 'lr 3/14/2010 21:13'!
realTarget
	^ target isSymbol
		ifTrue: [ Smalltalk globals at: target ifAbsent: [  ] ]
		ifFalse: [ target ]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:47'!
settingReceiver
	^ self targetSelector ifNil: [self realTarget] ifNotNil: [self realTarget perform: self targetSelector]! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 12/2/2009 16:40'!
target
	^ target! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/26/2009 21:15'!
target: anObject
	target := anObject! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/26/2009 21:15'!
targetSelector
	^ targetSelector! !

!PragmaSetting methodsFor: 'accessing' stamp: 'AlainPlantec 11/26/2009 21:16'!
targetSelector: aSelector
	targetSelector := aSelector! !

!PragmaSetting methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:13'!
theme
	^ UITheme current ! !

PragmaSetting subclass: #SettingDeclaration
	instanceVariableNames: 'getSelector setSelector type default ghostHelp'
	classVariableNames: 'ValueListCache'
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!SettingDeclaration commentStamp: 'AlainPlantec 1/3/2011 10:52' prior: 0!
A SettingDeclaration is an handler for a writtable setting value. 
Its main responsibility is allow real value changing and to hold descriptive data about the setting. When the real value is changed, all declared listeners are notified. The standard event mechanism is used for that purpose. See #value: method which is call in order to change a setting value. Whatever object can declare its interest in the real value changes by sending #whenChangedSend:to: to a setting. See notification protocol.

For convenience, my olds values are registered in a list. This list is used by the SystemSettingBrowser in order to easily retrieve previous values.

The type instance variable is a Symbol. It is used by the SystemSettingBrowser in order to build the input widget of a setting.
It can be the name of the class or the name of a superclass of the value. In that case, the input widget is built by #settingInputWidgetForNode: implemented by the class which name is the value of the type instance variable. As an example, the input widget for a Boolean is built by Boolean class>>settingInputWidgetForNode:.

Type value is not limited to be a class name. It can also be whatever symbol. In that case, the selector corresponding to the message to send to the setting in order to build the inputWidget is built by SettingDeclaration>>localInputWidgetSelector. If the type instance variable is not set, then it is dynamically set to the class name of the value. 

See SettingManager comment for more explanations.

Instance Variables
	default:		<Object>
	getSelector:		<Object>
	ghostHelp:		<Object>
	setSelector:		<Object>
	type:		<Object>

default
	- xxxxx

getSelector
	- xxxxx

ghostHelp
	- xxxxx

setSelector
	- xxxxx

type
	- xxxxx
!

SettingDeclaration subclass: #MultipleSettingDeclaration
	instanceVariableNames: 'domainValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!MultipleSettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 9/3/2010 15:51'!
domainValues
	^ domainValues ifNil: [domainValues := OrderedCollection new]! !

!MultipleSettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 11/25/2009 22:09'!
domainValues: aCollection 
	domainValues := aCollection asArray collect: [ :v | v settingFixedDomainValueNodeFrom: self]! !

!MultipleSettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 22:09'!
domainValuesLabels
	^ self domainValues
		collect: [:f | f name]! !

!MultipleSettingDeclaration methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 9/3/2010 17:04'!
fixedDomainValueNodeForAssociation: anAssociation 
	| s |
	^ (s := (SettingDeclaration new name: anAssociation key))
		target: s;
		selector: #default;
		default: anAssociation value! !

!MultipleSettingDeclaration methodsFor: 'as yet unclassified' stamp: 'AlainPlantec 9/3/2010 17:04'!
fixedDomainValueNodeForObject: anObject 
	| s |
	^ (s := (SettingDeclaration new name: anObject asString))
		target: s;
		selector: #default;
		default: anObject! !

!MultipleSettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 22:09'!
hasEditableList
	^ false! !

MultipleSettingDeclaration subclass: #PickOneSettingDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!PickOneSettingDeclaration commentStamp: 'AlainPlantec 1/3/2011 10:53' prior: 0!
A CheckListSettingDeclaration is a setting for which the value domain is a list. Each element of the list is an instance of FixedSettingValue. If domainValues is set, then the list of valid values is constant (initialized at declaration time). Instead, if getter is set, then the setting list is always dynamically computed. See SettingManager comment for more explanations. 

Instance Variables
!

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 11/30/2009 14:23'!
chooseValue
	| chosen |
	chosen := UIManager default
				chooseFrom: self domainValuesLabels
				values: self domainValues
				title: self label translated, ' choices' translated.
	chosen
		ifNotNil: [self realValue: chosen realValue]! !

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 10/17/2009 23:47'!
content
	^ (self domainValues
		detect: [:v | v realValue = self realValue]
		ifNone: [^ 'nil']) name! !

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 9/3/2010 11:19'!
defaultValue
	^ self default value ifNil: [self domainValues first realValue]
! !

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 10/17/2009 23:44'!
index
	self realValue ifNil: [self realValue: self default].
	^ self domainValues indexOf: (self domainValues detect: [:dv | dv realValue = self realValue] ifNone: [^ 0])! !

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 10/17/2009 23:39'!
index: anInteger
	self realValue: (self domainValues at: anInteger) realValue.
! !

!PickOneSettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:11'!
inputWidget
	| widget row |
	row := self theme newRowIn: World for: {
				widget := (self theme 
						newDropListIn: World
						for: self
						list: #domainValuesLabels
						getSelected: #index
						setSelected: #index:
						getEnabled: #enabled
						useIndex: true
						help: self description)
						extent: 1 @ 30}.
	widget hResizing: #rigid.
	widget width: (self maxNameWidthForFont: widget font) + 50.
	^ row! !

SettingDeclaration subclass: #RangeSettingDeclaration
	instanceVariableNames: 'range'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!RangeSettingDeclaration commentStamp: 'alain.plantec 4/7/2009 23:03' prior: 0!
A RangeSettingValue is a setting for which the value domain is an interval.  See SettingManager comment for more explanations.

Instance Variables
	range:		<Interval>

range
	- The interval which define the value domain
!

!RangeSettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 16:57'!
hasEditableList
	^ false! !

!RangeSettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:14'!
inputWidget
	^ (self theme newRowIn: World for: {(self theme 
							newTextEntryIn: World 
							for: self
							get: #realValue
							set: #realValue:
							class: (Smalltalk globals at: self type)
							getEnabled: #enabled
							help: nil) hResizing: #rigid;
							width: 80;
							wantsFrameAdornments: true;
							yourself. 
						(PluggableSliderMorph new max: self range last;
							 min: self range first;
							 quantum: self range increment;
							on: self
							getValue: #realValue
							setValue: #realValue:) hResizing: #spaceFill;
							 vResizing: #spaceFIll;
							 setBalloonText: self description;
							 minWidth: 300; height: 20;
							 yourself}) 
				cellInset: 10;
				yourself! !

!RangeSettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 3/12/2009 16:19'!
range
	^ range ! !

!RangeSettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 3/3/2009 09:38'!
range: anInterval
	range := anInterval! !

!RangeSettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:48'!
realValue: aValue
	aValue
		ifNil: [self settingReceiver perform: self setSelector with: nil]
		ifNotNil: [self settingReceiver perform: self setSelector with: ((aValue max: self range first) min: self range last)].
	self triggerEvent: self privateChangeEvent with: self
	! !

!SettingDeclaration class methodsFor: 'icon accessing' stamp: 'alain.plantec 4/9/2009 00:15'!
nullifyForm
	^ (Form
	extent: 16@16
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 463732736 4285815129 4285815129 4285815129 4285815129 4285815129 2292449280 463732736 0 0 0 0 0 0 0 1688469504 4285815129 4278190080 4278190080 4278190080 4278190080 4278190080 4280091395 4285815129 1688469504 0 0 0 0 0 1688469504 4285815129 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4285815129 1688469504 0 0 0 463732736 4285815129 4278190080 4294967295 4294967295 4278190080 4278190080 4278190080 4278190080 4294967295 4294967295 4278190080 4285815129 463732736 0 0 4285815129 4278190080 4278190080 4294967295 4294967295 4294967295 4278190080 4278190080 4294967295 4294967295 4294967295 4278190080 4285815129 4285815129 0 0 4285815129 4278190080 4278190080 4278190080 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4278190080 4278190080 4278190080 4285815129 0 0 4285815129 4278190080 4278190080 4278190080 4278190080 4294967295 4294967295 4294967295 4294967295 4278190080 4278190080 4278190080 4278190080 4285815129 0 0 4285815129 4278190080 4278190080 4278190080 4278190080 4294967295 4294967295 4294967295 4294967295 4278190080 4278190080 4278190080 4278190080 4285815129 0 0 4285815129 4278190080 4278190080 4278190080 4294967295 4294967295 4294967295 4294967295 4294967295 4294967295 4278190080 4278190080 4278190080 4285815129 0 0 2292449280 4281927174 4280025345 4294967295 4294967295 4294967295 4278190080 4278190080 4294967295 4294967295 4294967295 4278190080 4283696903 4285815129 0 0 463732736 4285815129 4278190080 4294967295 4294967295 4278190080 4278190080 4278190080 4278190080 4294967295 4294967295 4278190080 4285815129 463732736 0 0 0 1688469504 4285815129 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4278190080 4285815129 1688469504 0 0 0 0 0 1688469504 4285815129 4285815129 4278190080 4278190080 4278190080 4278190080 4283696646 4285815129 1688469504 0 0 0 0 0 0 0 463732736 2292449280 4285815129 4285815129 4285815129 4285815129 2292449280 463732736 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)! !

!SettingDeclaration class methodsFor: 'icon accessing' stamp: 'alain.plantec 4/8/2009 17:40'!
smallCautionForm
	^ (Form
	extent: 16@16
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 463732736 2292449280 3517186048 4154720256 4154720256 3517186048 2292449280 463732736 0 0 0 0 0 0 0 1688469504 4137943040 4292614939 4293600292 4293534756 4293534499 4293402913 4292483352 4137943040 1688469504 0 0 0 0 0 1688469504 4290185224 4293600292 4292284175 4291428866 4291166208 4291100672 4291297794 4292087053 4293271070 4290184967 1688469504 0 0 0 463732736 4137943040 4293534756 4291823367 4291231744 4291166208 4294967295 4294967295 4290969600 4290969600 4291429637 4293073948 4137943040 463732736 0 0 2292449280 4292614682 4292284175 4291231744 4291035136 4291035136 4294967295 4294967295 4291035136 4291035136 4290838528 4291824395 4292351253 2292449280 0 0 3517186048 4293468963 4291363330 4291166208 4291035136 4291035136 4294967295 4294967295 4291035136 4291035136 4290707456 4290838785 4292942105 3517186048 0 0 4154720256 4293534242 4291166208 4291100672 4291035136 4291035136 4294967295 4294967295 4291035136 4290707456 4290641920 4290641920 4292942104 4154720256 0 0 4154720256 4293468449 4291035136 4291035136 4291035136 4291035136 4291100672 4291100672 4291035136 4290641920 4290576384 4290510848 4292876056 4154720256 0 0 3517186048 4293271328 4291166722 4290969600 4291035136 4291035136 4291035136 4291035136 4291035136 4291035136 4290510848 4290576641 4292744726 3517186048 0 0 2292449280 4292417303 4291955981 4290838528 4291035136 4291035136 4294967295 4294967295 4291035136 4291035136 4290445312 4291430665 4292088338 2292449280 0 0 463732736 4137943040 4293139484 4291298565 4290707456 4290641920 4294967295 4294967295 4290510848 4290379776 4290905092 4292678933 4137943040 463732736 0 0 0 1688469504 4290184967 4293007898 4291693323 4290707713 4290510848 4290445312 4290511105 4291365129 4292678932 4290053638 1688469504 0 0 0 0 0 1688469504 4137943040 4292219924 4292876311 4292876055 4292810518 4292744726 4292088081 4137943040 1688469504 0 0 0 0 0 0 0 463732736 2292449280 3517186048 4154720256 4154720256 3517186048 2292449280 463732736 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)! !

!SettingDeclaration class methodsFor: 'cache accessing' stamp: 'alain.plantec 9/18/2009 09:57'!
valueListCache
	^ ValueListCache ifNil: [ValueListCache := WeakKeyDictionary new]! !

!SettingDeclaration class methodsFor: 'cache accessing' stamp: 'AlainPlantec 1/29/2010 10:50'!
valueListFor: aSettingValue
	| targetEntry |
	targetEntry := self valueListCache at: aSettingValue settingReceiver ifAbsentPut: [IdentityDictionary new].
	^ targetEntry at: aSettingValue getSelector ifAbsentPut: [OrderedCollection new].! !

!SettingDeclaration methodsFor: 'comparing' stamp: 'AlainPlantec 12/2/2009 16:44'!
= other
	self class = other class
		ifFalse: [^ false].
	self target == other target ifFalse: [^ false].
	self targetSelector = other targetSelector ifFalse: [^ false].
	self getSelector = other getSelector ifFalse: [^ false].
	self setSelector = other setSelector ifFalse: [^ false].
	^ true
	
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/9/2009 09:41'!
addToList: anItem
	(self list includes: anItem)
		ifFalse: [self list size > self maxKept
				ifTrue: [self list removeFirst].
			self list add: anItem.
			self changed: #list]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 3/11/2009 19:40'!
default
	^ default ! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 09:13'!
default: anObject 
	"initialize the default value"
	default := anObject.
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:17'!
defaultIcon
	^ (self type = #LogicalFont or: [self realValue isKindOf: AbstractFont])
		ifTrue: [ self theme   smallFontsIcon]
		ifFalse: [ self theme   smallPaintIcon]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 3/4/2009 11:48'!
defaultValue
	^ default value! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/19/2009 09:18'!
emptyList
	self list removeAll.
	self value ifNotNil: [self addToList: self realValue].! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 22:35'!
getSelector
	^ getSelector ifNil: [self name]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 23:07'!
getSelector: aSymbol
	getSelector := aSymbol! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 23:52'!
ghostHelp
	^ ghostHelp ! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 23:52'!
ghostHelp: aString
	ghostHelp := aString! !

!SettingDeclaration methodsFor: 'testing' stamp: 'alain.plantec 4/8/2009 16:59'!
hasDefault
	^ true! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 3/7/2010 14:34'!
hasEditableList
	^ (self realValue isKindOf: Boolean) not! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 16:50'!
hasPrivateListener
	^ self hasActionForEvent: self privateChangeEvent! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 11/25/2009 08:22'!
hasValue
	^ true! !

!SettingDeclaration methodsFor: 'comparing' stamp: 'AlainPlantec 12/13/2009 08:55'!
hash
	"hash is re-implemented because #= is re-implemented"
	^ self target hash
		bitXor: (self targetSelector hash
				bitXor: (self getSelector hash bitXor: self setSelector hash)) ! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/18/2009 23:20'!
index
	^ self list
		indexOf: self realValue
		ifAbsent: [self realValue
				ifNil: [0]
				ifNotNil: [self addToList: self realValue.
					self list size]]! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/18/2009 09:44'!
index: anInteger 
	self realValue: (self list at: anInteger)! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 5/20/2009 12:24'!
inputMorphFor: aContainer
	^ (super inputMorphFor: aContainer)
		ifNotNil: [:im |			
				self whenPrivateChangedSend: #settingValueChanged: to: aContainer model.
				self whenPrivateChangedSend: #innerWidgetChanged to: aContainer.
				im]! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:17'!
inputWidget
	"return the default widget for the input a the setting"
	| inputWidget xtraDialogWidget | 
	xtraDialogWidget := super inputWidget.
	inputWidget :=  self localInputWidgetSelector ifNotNil: [:localSel | self perform: localSel].
	(inputWidget isNil and: [self typeClass notNil])
		ifTrue: [inputWidget := (self inputWidgetSelectorInClass: self typeClass)
			ifNotNil: [:selector | self typeClass perform: selector with: self]].
	inputWidget
		ifNotNil: [(inputWidget respondsTo: #model:) 
			ifTrue: [inputWidget model: self]]. 
	xtraDialogWidget 
		ifNotNil: [inputWidget := inputWidget ifNil: [xtraDialogWidget] 
			ifNotNil: [ self theme  newRowIn: World for: {inputWidget. xtraDialogWidget}]].
	^ inputWidget ! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:17'!
inputWidgetForBoolean
	^  self theme  
			newCheckboxIn: World
			for: self
			getSelected: #realValue
			setSelected: #realValue:
			getEnabled: #enabled
			label: '' 
			help: nil! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:16'!
inputWidgetForColor
	^ (  
	 self theme 
		newColorChooserIn: World
		for: self
		getColor: #realValue
		setColor: #realValue:
		getEnabled: #enabled 
		help: 'Choose a color') minWidth: 150; width: 150; yourself! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'SeanDeNigris 7/17/2012 18:37'!
inputWidgetForFileOrDirectoryWithAction: aSymbol

	| list button |
	list := EditableDropListMorph
				on: self
				list: #list
				selected: #index
				changeSelected: #index:
				useIndex: true 
				addToList: #addToList:
				class: String
				getEnabled: #enabled
				default: self defaultValue.
	list
		ghostText: self ghostHelp;
		hResizing: #rigid;
		width: 450;
		wantsFrameAdornments: true.
		
	button :=  self theme  
						newButtonIn: World
						for: self
						getState: nil
						action: aSymbol
						arguments: nil
						getEnabled: nil
						label:  (AlphaImageMorph new image:  self theme  smallOpenIcon)
						help: 'Open directory chooser dialog' translated.
		
	^  self theme newRowIn: World for: { list. button }.! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:17'!
inputWidgetForFont
	^ (  self theme 
		newButtonIn: World
		for: self
		getState: nil 
		action: #getFont
		arguments: {}
		getEnabled: #enabled
		getLabel: #fontButtonLabel
		help: 'Choose a font ...' translated) cornerStyle: #square
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 9/17/2011 15:39'!
inputWidgetForLabel
	| widget |
	widget := self theme newLabelIn: World for: self label: '' getEnabled: nil.
	widget getTextSelector: #preferenceValue.
	widget width: 450.
	^ widget! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:20'!
inputWidgetForNumber
	^ self inputWidgetForString width: 200.
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'FernandoOlivero 4/12/2011 10:16'!
inputWidgetForPassword
	| widget |
	widget :=  self theme  
				newTextEntryIn: World
				for: self
				get: #realValue
				set: #realValue:
				class: String
				getEnabled: #enabled
				help: nil.
	widget
		wantsFrameAdornments: true;
		hResizing: #rigid;
		width: 450; 
		font: (StrikeFont passwordFontSize:  self theme  textFont pointSize).
	^ widget! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:20'!
inputWidgetForPoint
	^ self inputWidgetForString width: 200.
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 4/15/2012 13:24'!
inputWidgetForString
	| widget |
	widget := EditableDropListMorph
		on: self
		list: #list
		selected: #index
		changeSelected: #index:
		useIndex: true
		addToList: #addToList:
		class: (Smalltalk globals at: self type)
		getEnabled: #enabled
		default: self defaultValue.
	widget ghostText: self ghostHelp.
	widget contentMorph wantsFrameAdornments: true.
	widget hResizing: #rigid.
	widget width: 450.
	^ widget! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:21'!
inputWidgetSelectorInClass: aClass 
	| selector |
	^ (aClass respondsTo: (selector := #settingInputWidgetForNode:))
		ifTrue: [selector] ! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/18/2009 23:10'!
list
	^ self class valueListFor: self! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/17/2009 21:20'!
listenersForEvent: anEvent
	| listeners actionSequence |	
	listeners := Set new.	
	actionSequence := self actionMap at: anEvent ifAbsent: [].	
	actionSequence ifNotNil: [
		actionSequence isCollection	 
			ifTrue: [actionSequence do: [:oneAction | 
					oneAction receiver ifNotNil: [:r | listeners add: r]]]
			ifFalse: [listeners add: actionSequence receiver]].
	^ listeners! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 1/29/2010 14:18'!
localInputWidgetSelector
	| imSel |
	^ (self respondsTo: (imSel := ('inputWidgetFor' , self type) asSymbol)) ifTrue: [imSel]! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/6/2009 21:54'!
maxKept
	"maximum number of old values which are kept in
	list instance variable"
	^ 5! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 11/25/2009 21:35'!
maxNameWidthForFont: aFont 
	| max |
	max := 0.
	self domainValuesLabels
		do: [:v | max := max max: (aFont widthOfStringOrText: v)].
	^ max ! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 9/14/2011 16:37'!
preferenceValue
	^ self realValue! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 4/8/2009 15:59'!
privateChangeEvent
	^ #privateSettingValueChanged! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 16:10'!
privateForget: aReceiver 
	"do not notify aReceiver anymore"
	self removeActionsWithReceiver: aReceiver forEvent: self privateChangeEvent! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 16:50'!
privateForget: aReceiver selector: aSelector 
	"Private"
	self
		removeActionsSatisfying: [:action | action receiver == aReceiver
				and: [action selector = aSelector]]
		forEvent: self privateChangeEvent! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 16:35'!
privateListeners
	^ self listenersForEvent: self privateChangeEvent! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:49'!
realValue
	"Answer the value of realValue"

	^ (self settingReceiver perform: self getSelector) 
		ifNil: [self setToDefault. 
			self settingReceiver perform: self getSelector]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:49'!
realValue: anObject
	"Set the value of realValue"
	self settingReceiver perform: self setSelector with: anObject.
	self triggerEvent: self privateChangeEvent with: self.! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 13:49'!
selector: aSymbol
	self getSelector: aSymbol.
	self setSelector: (aSymbol, ':') asSymbol.! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'AlainPlantec 11/23/2009 22:36'!
setSelector
	^ setSelector ifNil: [self getSelector ifNotNil: [:gs | (gs, ':') asSymbol]]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 23:07'!
setSelector: aSymbol
	setSelector := aSymbol! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 9/18/2009 09:23'!
setToDefault
	self realValue: self defaultValue.
! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'AlainPlantec 9/14/2011 16:47'!
settingStoreOn: aStream
	aStream tab;
		nextPutAll: (self target isSymbol
				ifTrue: [self target]
				ifFalse: [self target name]);
		nextPutAll: (self targetSelector
				ifNil: ['']
				ifNotNil: [' ' , self targetSelector]);
		 nextPutAll: ' ' , self setSelector , ' '.
	self preferenceValue settingStoreOn: aStream.
	aStream nextPut: $.;
		 cr! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 9/18/2009 09:24'!
type
	"Answer the value of type"

	^ type ifNil: [type := self realValue class name]! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'alain.plantec 3/11/2009 18:22'!
type: anObject
	"Set the value of type"

	type := anObject! !

!SettingDeclaration methodsFor: 'accessing' stamp: 'lr 3/14/2010 21:13'!
typeClass
	^ Smalltalk globals at: self type ifAbsent: [  ]! !

!SettingDeclaration methodsFor: 'user interface' stamp: 'alain.plantec 4/8/2009 15:59'!
whenPrivateChangedSend: zeroOrOneArgSelector to: aSubscriber 
	"private, use #whenChangedSend:to: instead"
	self
		when: self privateChangeEvent
		send: zeroOrOneArgSelector
		to: aSubscriber! !

PragmaSetting subclass: #SystemSettingLauncher
	instanceVariableNames: 'script'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Core'!

!SystemSettingLauncher methodsFor: 'accessing' stamp: 'FernandoOlivero 4/12/2011 10:32'!
defaultIcon
	^ self theme  smallDoItIcon! !

!SystemSettingLauncher methodsFor: 'accessing' stamp: 'FernandoOlivero 4/12/2011 10:45'!
inputWidget
	^ (self theme  
				newButtonIn: World 
				for: self
				getState: nil
				action: #launch
				arguments: {}
				getEnabled: nil
				getLabel: nil
				help: self description)
		 		label: ( self theme  newRowIn: World for: {AlphaImageMorph new image: self theme smallRedoIcon. 
				self theme buttonLabelForText: 'Launch' translated});
		 		yourself.! !

!SystemSettingLauncher methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 10:51'!
launch
	self script 
			ifNotNil: [:s | s isSymbol 
				ifTrue: [s numArgs = 0 
					ifTrue: [self settingReceiver perform: s]
					ifFalse: [self settingReceiver perform: s with: self]]
				ifFalse: [s numArgs = 0 
					ifTrue: [s value]
					ifFalse: [s value: self]]]! !

!SystemSettingLauncher methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 13:22'!
script
	^ script ifNil: [script := []]! !

!SystemSettingLauncher methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 14:19'!
script: aValuableOrSymbol
	script := aValuableOrSymbol! !

Object subclass: #PseudoClass
	instanceVariableNames: 'name definition organization source metaClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-FilePackage'!

!PseudoClass commentStamp: '<historical>' prior: 0!
I provide an inert model of a Class, used by FileContentsBrowser to manipulate filedout code. Instead of a method dictionary or selectors onto CompiledMethods, I have a dictionary ("source") of selectors onto ChangeRecords, which were, in the case of FileContentsBrowser, parsed from a source or change set file.!

!PseudoClass methodsFor: 'methods' stamp: 'sma 6/1/2000 14:54'!
addMethodChange: aChangeRecord
	| selector |
	selector := self parserClass new parseSelector: aChangeRecord string.
	self organization classify: selector under: aChangeRecord category.
	self sourceCodeAt: selector put: aChangeRecord! !

!PseudoClass methodsFor: 'accessing' stamp: 'nk 4/29/2004 06:59'!
allCallsOn
	^ (self realClass ifNil: [ ^#() ]) allCallsOn! !

!PseudoClass methodsFor: 'accessing' stamp: 'StephaneDucasse 4/30/2011 21:38'!
allCallsOnIn: aSystemNavigation
	^ (self realClass ifNil: [ ^#() ]) allCallsOn! !

!PseudoClass methodsFor: 'accessing' stamp: 'sma 6/16/1999 22:59'!
allInstVarNames
	^#()! !

!PseudoClass methodsFor: 'private' stamp: 'nk 2/18/2004 18:33'!
allSubclassesWithLevelDo: classAndLevelBlock startingLevel: level
	^ (self realClass ifNil: [ ^self ])  allSubclassesWithLevelDo: classAndLevelBlock startingLevel: level! !

!PseudoClass methodsFor: 'accessing' stamp: 'nk 2/18/2004 18:32'!
allSuperclasses
	^ (self realClass ifNil: [ ^#() ]) allSuperclasses! !

!PseudoClass methodsFor: 'testing method dictionary' stamp: 'marcus.denker 11/10/2008 10:04'!
bindingOf: varName
	self exists ifTrue:[
		(self realClass bindingOf: varName) ifNotNil:[:binding| ^binding].
	].
	^Smalltalk bindingOf: varName asSymbol! !

!PseudoClass methodsFor: 'class' stamp: ''!
classComment: aChangeRecord
	self organization classComment: aChangeRecord! !

!PseudoClass methodsFor: 'errors' stamp: ''!
classNotDefined
	^self inform: self name,' is not defined in the system.
You have to define this class first.'.! !

!PseudoClass methodsFor: 'class' stamp: 'di 1/13/1999 12:00'!
classPool 
	self exists ifFalse: [^ nil].
	^ self realClass classPool! !

!PseudoClass methodsFor: 'class' stamp: 'BJP 4/23/2001 13:50'!
comment
	| rStr |
	rStr := self organization commentRemoteStr.
	^rStr isNil
		ifTrue:[self name,' has not been commented in this file']
		ifFalse:[rStr string]! !

!PseudoClass methodsFor: 'class' stamp: ''!
comment: aString
	self commentString: aString.! !

!PseudoClass methodsFor: 'class' stamp: 'ar 2/5/2004 15:18'!
commentString
	^self comment asString! !

!PseudoClass methodsFor: 'class' stamp: ''!
commentString: aString
	self classComment: aString asText. "Just wrap it"! !

!PseudoClass methodsFor: 'accessing' stamp: 'jb 7/1/2011 10:54'!
compilerClass
	^ (Smalltalk globals at: name ifAbsent: [ ^ self class compilerClass ]) compilerClass! !

!PseudoClass methodsFor: 'private' stamp: ''!
confirmRemovalOf: aString
	^self confirm:'Remove ',aString,' ?'! !

!PseudoClass methodsFor: 'class' stamp: 'LC 10/8/2001 04:46'!
definition
	| link linkText defText |
	^definition ifNil:
		[defText := Text fromString: 'There is no class definition for '.
		link := TextLink new.
		linkText := link analyze: self name with: 'Definition'.
		linkText := Text string: (linkText ifNil: ['']) attribute: link.
		defText append: linkText; append: ' in this file'].! !

!PseudoClass methodsFor: 'class' stamp: ''!
definition: aString
	definition := aString! !

!PseudoClass methodsFor: 'private' stamp: 'jb 7/1/2011 10:54'!
evaluate: aString
	^self class evaluatorClass evaluate: aString for: nil logged: true! !

!PseudoClass methodsFor: 'testing' stamp: 'lr 3/14/2010 21:13'!
exists
	^ (Smalltalk globals at: self name asSymbol ifAbsent: [ ^ false ]) isKindOf: Behavior! !

!PseudoClass methodsFor: 'filein/fileout' stamp: ''!
fileIn
	"FileIn the receiver"
	self hasDefinition ifTrue:[self fileInDefinition].
	self fileInMethods: self selectors.
	metaClass ifNotNil:[metaClass fileIn].
	self needsInitialize ifTrue:[
		self evaluate: self name,' initialize'.
	].! !

!PseudoClass methodsFor: 'filein/fileout' stamp: ''!
fileInCategory: aCategory
	^self fileInMethods: (self organization listAtCategoryNamed: aCategory)! !

!PseudoClass methodsFor: 'filein/fileout' stamp: 'ar 7/16/2005 14:06'!
fileInDefinition
	self hasDefinition ifFalse:[^self].
	(self makeSureSuperClassExists: (definition copyUpTo: Character space)) ifFalse:[^self].
	self hasDefinition ifTrue:[
		Transcript cr; show:'Defining ', self name.
		self evaluate: self definition].
	self exists ifFalse:[^self].
	self hasComment ifTrue:[self realClass classComment: self comment].! !

!PseudoClass methodsFor: 'filein/fileout' stamp: ''!
fileInMethod: selector
	^self fileInMethods: (Array with: selector)! !

!PseudoClass methodsFor: 'filein/fileout' stamp: ''!
fileInMethods
	^self fileInMethods: self selectors! !

!PseudoClass methodsFor: 'filein/fileout' stamp: 'nice 1/5/2010 15:59'!
fileInMethods: aCollection
	"FileIn all methods with selectors taken from aCollection"
	| theClass |
	self exists ifFalse:[^self classNotDefined].
	theClass := self realClass.
	aCollection do:[:sel| | cat |
		cat := self organization categoryOfElement: sel.
		cat = self removedCategoryName ifFalse:[
			theClass 
				compile: (self sourceCodeAt: sel) 
				classified: cat
				withStamp: (self stampAt: sel)
				notifying: nil.
		].
	].! !

!PseudoClass methodsFor: 'filein/fileout' stamp: 'PeterHugossonMiller 9/3/2009 10:52'!
fileOut
	| internalStream |
	internalStream := (String new: 1000) writeStream.
	self fileOutOn: internalStream.
	self needsInitialize ifTrue:[
		internalStream cr; nextChunkPut: self name,' initialize'.
	].

	FileStream writeSourceCodeFrom: internalStream baseName: self name isSt: true! !

!PseudoClass methodsFor: 'filein/fileout' stamp: 'PeterHugossonMiller 9/3/2009 10:52'!
fileOutCategory: categoryName

	| internalStream |
	internalStream := (String new: 1000) writeStream.
	self fileOutMethods: (self organization listAtCategoryNamed: categoryName)
			on: internalStream.
	FileStream writeSourceCodeFrom: internalStream baseName: (self name, '-', categoryName) isSt: true! !

!PseudoClass methodsFor: 'fileIn/fileOut' stamp: 'mir 9/25/2008 15:04'!
fileOutDefinitionOn: aStream
	self hasDefinition ifFalse:[^self].
	aStream nextChunkPut: self definition; cr.
	self hasComment ifTrue: [
		aStream cr.
		self organization commentRemoteStr fileOutOn: aStream]! !

!PseudoClass methodsFor: 'filein/fileout' stamp: 'PeterHugossonMiller 9/3/2009 10:52'!
fileOutMethod: selector
	| internalStream |

	internalStream := (String new: 1000) writeStream.

	self fileOutMethods: (Array with: selector) on: internalStream.

	FileStream writeSourceCodeFrom: internalStream baseName: (self name , '-' , (selector copyReplaceAll: ':' with: '')) isSt: true! !

!PseudoClass methodsFor: 'filein/fileout' stamp: 'ar 2/7/2004 01:04'!
fileOutMethods: aCollection on: aStream
	"FileOut all methods with selectors taken from aCollection"
	| cat categories |
	categories := Dictionary new.
	aCollection do:[:sel|
		cat := self organization categoryOfElement: sel.
		cat = self removedCategoryName ifFalse:[
			(categories includesKey: cat) 
				ifFalse:[categories at: cat put: Set new].
			(categories at: cat) add: sel].
	].
	categories associationsDo:[:assoc|
		cat := assoc key.
		assoc value do:[:sel|
			aStream cr.
			(self sourceCode at: sel) fileOutOn: aStream.
		].
	].! !

!PseudoClass methodsFor: 'filein/fileout' stamp: ''!
fileOutMethodsOn: aStream
	^self fileOutMethods: self selectors on: aStream.! !

!PseudoClass methodsFor: 'filein/fileout' stamp: ''!
fileOutOn: aStream
	"FileOut the receiver"
	self fileOutDefinitionOn: aStream.
	metaClass ifNotNil:[metaClass fileOutDefinitionOn: aStream].
	self fileOutMethods: self selectors on: aStream.
	metaClass ifNotNil:[metaClass fileOutMethods: metaClass selectors on: aStream].! !

!PseudoClass methodsFor: 'accessing' stamp: ''!
fullName
	^self name! !

!PseudoClass methodsFor: 'testing' stamp: 'marcus.denker 7/29/2009 15:27'!
hasChanges

	self sourceCode isEmpty ifFalse:[^true].
	self organization hasComment ifTrue:[^true].
	definition isNil ifFalse:[^true].
	metaClass isNil ifFalse:[^metaClass hasChanges].
	^false! !

!PseudoClass methodsFor: 'testing' stamp: ''!
hasComment
	^self organization commentRemoteStr notNil! !

!PseudoClass methodsFor: 'testing' stamp: ''!
hasDefinition
	^definition notNil! !

!PseudoClass methodsFor: 'testing' stamp: ''!
hasMetaclass
	^metaClass notNil! !

!PseudoClass methodsFor: 'methods' stamp: 'StephaneDucasse 2/20/2010 21:53'!
includesSelector: aSymbol 
	"Answer whether the message whose selector is the argument is in the 
	method dictionary of the receiver's class."

	^ self exists
		ifTrue: [self realClass includesSelector: aSymbol]
		ifFalse: [false]
		
	"Note that this method is only used to check whether the class should initialized so returning false is better than raising an error"! !

!PseudoClass methodsFor: 'accessing' stamp: 'nk 3/9/2004 10:24'!
instVarNames
	^ #()! !

!PseudoClass methodsFor: 'override' stamp: 'nk 2/18/2004 18:30'!
isMeta
	^false! !

!PseudoClass methodsFor: 'printing' stamp: 'sma 6/17/1999 00:00'!
literalScannedAs: scannedLiteral notifying: requestor 
	^ scannedLiteral! !

!PseudoClass methodsFor: 'private' stamp: 'lr 3/14/2010 21:13'!
makeSureClassExists: aString
	| theClass |
	theClass := Smalltalk globals at: aString asSymbol ifAbsent: [ nil ].
	theClass ifNotNil: [ ^ true ].
	^ self confirm: aString , ' does not exist in the system. Use nil instead?'! !

!PseudoClass methodsFor: 'private' stamp: 'lr 3/14/2010 21:13'!
makeSureSuperClassExists: aString
	| theClass |
	theClass := Smalltalk globals at: aString asSymbol ifAbsent: [ nil ].
	theClass ifNotNil: [ ^ true ].
	^ self confirm: 'The super class ' , aString , ' does not exist in the system. Use nil instead?'! !

!PseudoClass methodsFor: 'class' stamp: ''!
metaClass
	^metaClass ifNil:[metaClass := PseudoMetaclass new name: (self name)].! !

!PseudoClass methodsFor: 'methods' stamp: ''!
methodChange: aChangeRecord
	aChangeRecord isMetaClassChange ifTrue:[
		^self metaClass addMethodChange: aChangeRecord.
	] ifFalse:[
		^self addMethodChange: aChangeRecord.
	].
! !

!PseudoClass methodsFor: 'accessing' stamp: ''!
name
	^name! !

!PseudoClass methodsFor: 'accessing' stamp: ''!
name: anObject
	name := anObject! !

!PseudoClass methodsFor: 'testing' stamp: 'lr 3/14/2010 21:13'!
nameExists
	^ Smalltalk globals includesKey: self name asSymbol! !

!PseudoClass methodsFor: 'testing' stamp: 'StephaneDucasse 2/20/2010 21:50'!
needsInitialize
	^self hasMetaclass 
		and: [self metaClass includesSelector: #initialize]! !

!PseudoClass methodsFor: 'accessing' stamp: 'NS 4/6/2004 15:46'!
organization
	organization ifNil: [organization := PseudoClassOrganizer defaultList: SortedCollection new].

	"Making sure that subject is set correctly. It should not be necessary."
	organization setSubject: self.
	^ organization! !

!PseudoClass methodsFor: 'private' stamp: 'jb 7/1/2011 10:54'!
parserClass

	^ self class evaluatorClass parserClass! !

!PseudoClass methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:00'!
prettyPrinterClass
	^self class prettyPrinterClass! !

!PseudoClass methodsFor: 'printing' stamp: 'ar 2/5/2004 16:04'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut:$(; print: name; nextPut:$)! !

!PseudoClass methodsFor: 'accessing' stamp: 'lr 3/14/2010 21:13'!
realClass
	^ Smalltalk globals at: self name asSymbol ifAbsent: [  ]! !

!PseudoClass methodsFor: 'removing' stamp: ''!
removeAllUnmodified
	| stClass |
	self exists ifFalse:[^self].
	self removeUnmodifiedMethods: self selectors.
	stClass := self realClass.
	(self hasDefinition and:[stClass definition = self definition]) ifTrue:[definition := nil].
	(self hasComment and:[stClass comment asString = self commentString]) ifTrue:[ self classComment: nil].
	metaClass isNil ifFalse:[metaClass removeAllUnmodified].! !

!PseudoClass methodsFor: 'categories' stamp: ''!
removeCategory: selector
	(self organization listAtCategoryNamed: selector) do:[:sel|
		self organization removeElement: sel.
		self sourceCode removeKey: sel.
	].
	self organization removeCategory: selector.! !

!PseudoClass methodsFor: 'methods' stamp: ''!
removeMethod: selector
	self organization removeElement: selector.
	self sourceCode removeKey: selector.
! !

!PseudoClass methodsFor: 'methods' stamp: ''!
removeSelector: aSelector
	| catName |
	catName := self removedCategoryName.
	self organization addCategory: catName before: self organization categories first.
	self organization classify: aSelector under: catName.
	self sourceCodeAt: aSelector put:'methodWasRemoved' asText.! !

!PseudoClass methodsFor: 'removing' stamp: ''!
removeUnmodifiedMethods: aCollection
	| stClass |
	self exists ifFalse:[^self].
	stClass := self realClass.
	aCollection do:[:sel|
		(self sourceCodeAt: sel) = (stClass sourceCodeAt: sel ifAbsent:['']) asString ifTrue:[
			self removeMethod: sel.
		].
	].
	self organization removeEmptyCategories.! !

!PseudoClass methodsFor: 'categories' stamp: ''!
removedCategoryName
	^'*** removed methods ***' asSymbol! !

!PseudoClass methodsFor: 'class' stamp: 'nk 2/18/2004 18:30'!
renameTo: aString

	self hasDefinition ifTrue:[
		self isMeta ifTrue:[
			self definition: (self definition
				copyReplaceAll: name,' class'
				with: aString, ' class').
		] ifFalse:[
			self definition: (self definition 
					copyReplaceAll:'ubclass: #',name
					with:'ubclass: #', aString)]].
	name := aString.
	metaClass ifNotNil:[metaClass renameTo: aString].! !

!PseudoClass methodsFor: 'methods' stamp: ''!
selectors
	^self sourceCode keys! !

!PseudoClass methodsFor: 'class' stamp: 'di 1/13/1999 12:00'!
sharedPools 
	self exists ifFalse: [^ nil].
	^ self realClass sharedPools! !

!PseudoClass methodsFor: 'methods' stamp: ''!
sourceCode
	^source ifNil:[source := Dictionary new]! !

!PseudoClass methodsFor: 'methods' stamp: ''!
sourceCodeAt: sel
	^(self sourceCode at: sel) string! !

!PseudoClass methodsFor: 'methods' stamp: ''!
sourceCodeAt: sel put: object
	self sourceCode at: sel put: object! !

!PseudoClass methodsFor: 'methods' stamp: ''!
sourceCodeTemplate
	^''! !

!PseudoClass methodsFor: 'methods' stamp: 'sw 6/10/2003 17:31'!
stampAt: selector
	"Answer the authoring time-stamp of the change"

	| code |
	^ ((code := self sourceCode at: selector) isKindOf: ChangeRecord)
		ifTrue:
			[code stamp]
		ifFalse:
			[code string]! !

!PseudoClass methodsFor: 'accessing' stamp: 'NorbertHartl 6/20/2008 21:25'!
theMetaClass
	^ self metaClass! !

!PseudoClass methodsFor: 'accessing' stamp: 'wod 5/19/1998 17:42'!
theNonMetaClass
	"Sent to a class or metaclass, always return the class"

	^self! !

!PseudoClass methodsFor: 'categories' stamp: ''!
whichCategoryIncludesSelector: aSelector 
	"Answer the category of the argument, aSelector, in the organization of 
	the receiver, or answer nil if the receiver does not inlcude this selector."

	^ self organization categoryOfElement: aSelector! !

PseudoClass subclass: #PseudoMetaclass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-FilePackage'!

!PseudoMetaclass methodsFor: 'accessing' stamp: ''!
fullName
	^self name,' class'! !

!PseudoMetaclass methodsFor: 'testing' stamp: 'nk 2/18/2004 18:30'!
isMeta
	^true! !

!PseudoMetaclass methodsFor: 'accessing' stamp: ''!
realClass
	^super realClass class! !

!PseudoMetaclass methodsFor: 'accessing' stamp: 'FBS 3/4/2004 14:17'!
theNonMetaClass
	"Sent to a class or metaclass, always return the class"

	^self realClass theNonMetaClass! !

Object subclass: #SecureHashAlgorithm
	instanceVariableNames: 'totalA totalB totalC totalD totalE totals'
	classVariableNames: 'K1 K2 K3 K4'
	poolDictionaries: ''
	category: 'System-Hashing-SHA1'!

!SecureHashAlgorithm commentStamp: '<historical>' prior: 0!
This class implements the Secure Hash Algorithm (SHA) described in the U.S. government's Secure Hash Standard (SHS). This standard is described in FIPS PUB 180-1, "SECURE HASH STANDARD", April 17, 1995.

The Secure Hash Algorithm is also described on p. 442 of 'Applied Cryptography: Protocols, Algorithms, and Source Code in C' by Bruce Scheier, Wiley, 1996.

See the comment in class DigitalSignatureAlgorithm for details on its use.

Implementation notes:
The secure hash standard was created with 32-bit hardware in mind. All arithmetic in the hash computation must be done modulo 2^32. This implementation uses ThirtyTwoBitRegister objects to simulate hardware registers; this implementation is about six times faster than using LargePositiveIntegers (measured on a Macintosh G3 Powerbook). Implementing a primitive to process each 64-byte buffer would probably speed up the computation by a factor of 20 or more.
!

!SecureHashAlgorithm class methodsFor: 'initialization' stamp: 'jm 12/7/1999 23:25'!
initialize
	"SecureHashAlgorithm initialize"
	"For the curious, here's where these constants come from:
	  #(2 3 5 10) collect: [:x | ((x sqrt / 4.0) * (2.0 raisedTo: 32)) truncated hex]"

	K1 := ThirtyTwoBitRegister new load: 16r5A827999.
	K2 := ThirtyTwoBitRegister new load: 16r6ED9EBA1.
	K3 := ThirtyTwoBitRegister new load: 16r8F1BBCDC.
	K4 := ThirtyTwoBitRegister new load: 16rCA62C1D6.
! !

!SecureHashAlgorithm methodsFor: 'private' stamp: 'jm 12/7/1999 23:25'!
constantForStep: i
	"Answer the constant for the i-th step of the block hash loop. We number our steps 1-80, versus the 0-79 of the standard."

	i <= 20 ifTrue: [^ K1].
	i <= 40 ifTrue: [^ K2].
	i <= 60 ifTrue: [^ K3].
	^ K4
! !

!SecureHashAlgorithm methodsFor: 'private' stamp: 'jm 12/21/1999 20:06'!
expandedBlock: aByteArray
	"Convert the given 64 byte buffer into 80 32-bit registers and answer the result." 
	| out src v |
	out := Array new: 80.
	src := 1.
	1 to: 16 do: [:i |
		out at: i put: (ThirtyTwoBitRegister new loadFrom: aByteArray at: src).
		src := src + 4].

	17 to: 80 do: [:i |
		v := (out at: i - 3) copy.
		v	bitXor: (out at: i - 8);
			bitXor: (out at: i - 14);
			bitXor: (out at: i - 16);
			leftRotateBy: 1.
		out at: i put: v].
	^ out
! !

!SecureHashAlgorithm methodsFor: 'private' stamp: 'jm 12/21/1999 20:02'!
finalHash
	"Concatenate the final totals to build the 160-bit integer result."
	"Details: If the primitives are supported, the results are in the totals array. Otherwise, they are in the instance variables totalA through totalE."

	| r |
	totals ifNil: [  "compute final hash when not using primitives"
		^ (totalA asInteger bitShift: 128) +
		  (totalB asInteger bitShift:  96) +
		  (totalC asInteger bitShift:  64) +
		  (totalD asInteger bitShift:  32) +
		  (totalE asInteger)].

	"compute final hash when using primitives"
	r := 0.
	1 to: 5 do: [:i |
		r := r bitOr: ((totals at: i) bitShift: (32 * (5 - i)))].
	^ r
! !

!SecureHashAlgorithm methodsFor: 'private' stamp: 'jm 12/7/1999 22:15'!
hashFunction: i of: x with: y with: z
	"Compute the hash function for the i-th step of the block hash loop. We number our steps 1-80, versus the 0-79 of the standard."
	"Details: There are four functions, one for each 20 iterations. The second and fourth are the same."

	i <= 20 ifTrue: [^ x copy bitAnd: y; bitOr: (x copy bitInvert; bitAnd: z)].
	i <= 40 ifTrue: [^ x copy bitXor: y; bitXor: z].
	i <= 60 ifTrue: [^ x copy bitAnd: y; bitOr: (x copy bitAnd: z); bitOr: (y copy bitAnd: z)].
	^ x copy bitXor: y; bitXor: z
! !

!SecureHashAlgorithm methodsFor: 'public' stamp: 'jm 12/14/1999 11:56'!
hashInteger: aPositiveInteger
	"Hash the given positive integer. The integer to be hashed should have 512 or fewer bits. This entry point is used in key generation."

	| buffer dstIndex |
	self initializeTotals.

	"pad integer with zeros"
	aPositiveInteger highBit <= 512
		ifFalse: [self error: 'integer cannot exceed 512 bits'].
	buffer := ByteArray new: 64.
	dstIndex := 0.
	aPositiveInteger digitLength to: 1 by: -1 do: [:i |
		buffer at: (dstIndex := dstIndex + 1) put: (aPositiveInteger digitAt: i)].

	"process that one block"
	self processBuffer: buffer.

	^ self finalHash
! !

!SecureHashAlgorithm methodsFor: 'public' stamp: 'md 11/14/2003 17:17'!
hashInteger: aPositiveInteger seed: seedInteger
	"Hash the given positive integer. The integer to be hashed should have 512 or fewer bits. This entry point is used in the production of random numbers"

	| buffer dstIndex |
	"Initialize totalA through totalE to their seed values."
	totalA := ThirtyTwoBitRegister new
		load: ((seedInteger bitShift: -128) bitAnd: 16rFFFFFFFF).
	totalB := ThirtyTwoBitRegister new
		load: ((seedInteger bitShift: -96) bitAnd: 16rFFFFFFFF).
	totalC := ThirtyTwoBitRegister new
		load: ((seedInteger bitShift: -64) bitAnd: 16rFFFFFFFF).
	totalD := ThirtyTwoBitRegister new
		load: ((seedInteger bitShift: -32) bitAnd: 16rFFFFFFFF).
	totalE := ThirtyTwoBitRegister new
		load: (seedInteger bitAnd: 16rFFFFFFFF).
	self initializeTotalsArray.

	"pad integer with zeros"
	buffer := ByteArray new: 64.
	dstIndex := 0.
	aPositiveInteger digitLength to: 1 by: -1 do: [:i |
		buffer at: (dstIndex := dstIndex + 1) put: (aPositiveInteger digitAt: i)].

	"process that one block"
	self processBuffer: buffer.

	^ self finalHash
! !

!SecureHashAlgorithm methodsFor: 'public' stamp: 'dc 5/30/2008 10:17'!
hashMessage: aStringOrByteArray 
	"Hash the given message using the Secure Hash Algorithm."
	^ self hashStream: aStringOrByteArray asByteArray readStream! !

!SecureHashAlgorithm methodsFor: 'public' stamp: 'StephaneDucasse 2/28/2010 11:07'!
hashStream: aPositionableStream
	"Hash the contents of the given stream from the current position to the end using the Secure Hash Algorithm. The SHA algorithm is defined in FIPS PUB 180-1. It is also described on p. 442 of 'Applied Cryptography: Protocols, Algorithms, and Source Code in C' by Bruce Scheier, Wiley, 1996."
	"SecureHashAlgorithm new hashStream: (ReadStream on: 'foo')"
	"(SecureHashAlgorithm new hashMessage: '') radix: 16
	=> 'DA39A3EE5E6B4B0D3255BFEF95601890AFD80709'"
	
	| startPosition buf bitLength |
	self initializeTotals.
  	
	aPositionableStream atEnd ifTrue: [self processFinalBuffer: #() bitLength: 0].

	startPosition := aPositionableStream position.
	[aPositionableStream atEnd] whileFalse: 
		[ buf := aPositionableStream next: 64.
		(aPositionableStream atEnd not and: [buf size = 64])
			ifTrue: [self processBuffer: buf]
			ifFalse: [ bitLength := (aPositionableStream position - startPosition) * 8.
					self processFinalBuffer: buf bitLength: bitLength]].
	^ self finalHash
! !

!SecureHashAlgorithm methodsFor: 'private' stamp: 'jm 12/21/1999 19:38'!
initializeTotals
	"Initialize totalA through totalE to their seed values."

	"total registers for use when primitives are absent"
	totalA := ThirtyTwoBitRegister new load: 16r67452301.
	totalB := ThirtyTwoBitRegister new load: 16rEFCDAB89.
	totalC := ThirtyTwoBitRegister new load: 16r98BADCFE.
	totalD := ThirtyTwoBitRegister new load: 16r10325476.
	totalE := ThirtyTwoBitRegister new load: 16rC3D2E1F0.
	self initializeTotalsArray.
! !

!SecureHashAlgorithm methodsFor: 'private' stamp: 'jm 12/21/1999 19:38'!
initializeTotalsArray
	"Initialize the totals array from the registers for use with the primitives."

	totals := Bitmap new: 5.
	totals at: 1 put: totalA asInteger.
	totals at: 2 put: totalB asInteger.
	totals at: 3 put: totalC asInteger.
	totals at: 4 put: totalD asInteger.
	totals at: 5 put: totalE asInteger.
! !

!SecureHashAlgorithm methodsFor: 'primitives' stamp: 'jm 12/21/1999 20:11'!
primExpandBlock: aByteArray into: wordBitmap
	"Expand the given 64-byte buffer into the given Bitmap of length 80."

	<primitive: 'primitiveExpandBlock' module: 'DSAPrims'>
	^ self primitiveFailed
! !

!SecureHashAlgorithm methodsFor: 'primitives' stamp: 'jm 12/21/1999 22:58'!
primHasSecureHashPrimitive
	"Answer true if this platform has primitive support for the Secure Hash Algorithm."

	<primitive: 'primitiveHasSecureHashPrimitive' module: 'DSAPrims'>
	^ false
! !

!SecureHashAlgorithm methodsFor: 'primitives' stamp: 'jm 12/21/1999 20:13'!
primHashBlock: blockBitmap using: workingTotalsBitmap
	"Hash the given block (a Bitmap) of 80 32-bit words, using the given workingTotals."

	<primitive: 'primitiveHashBlock' module: 'DSAPrims'>
	^ self primitiveFailed
! !

!SecureHashAlgorithm methodsFor: 'private' stamp: 'jm 12/21/1999 19:43'!
processBuffer: aByteArray
	"Process given 64-byte buffer, accumulating the results in totalA through totalE."

	| a b c d e w tmp |
	self primHasSecureHashPrimitive
		ifTrue: [^ self processBufferUsingPrimitives: aByteArray]
		ifFalse: [totals := nil].

	"initialize registers a through e from the current totals" 
	a := totalA copy.
	b := totalB copy.
	c := totalC copy.
	d := totalD copy.
	e := totalE copy.

	"expand and process the buffer"
	w := self expandedBlock: aByteArray.
	1 to: 80 do: [:i |
		tmp := (a copy leftRotateBy: 5)
			+= (self hashFunction: i of: b with: c with: d);
			+= e;
			+= (w at: i);
			+= (self constantForStep: i).
		e := d.
		d := c.
		c := b copy leftRotateBy: 30.
		b := a.
		a := tmp].

	"add a through e into total accumulators"
	totalA += a.
	totalB += b.
	totalC += c.
	totalD += d.
	totalE += e.
! !

!SecureHashAlgorithm methodsFor: 'private' stamp: 'jm 12/21/1999 23:32'!
processBufferUsingPrimitives: aByteArray
	"Process given 64-byte buffer using the primitives, accumulating the results in totals."

	| w |
	"expand and process the buffer"
	w := Bitmap new: 80.
	self primExpandBlock: aByteArray into: w.
	self primHashBlock: w using: totals.
! !

!SecureHashAlgorithm methodsFor: 'private' stamp: 'jm 12/14/1999 11:40'!
processFinalBuffer: buffer bitLength: bitLength
	"Process given buffer, whose length may be <= 64 bytes, accumulating the results in totalA through totalE. Also process the final padding bits and length."

	| out |
	out := ByteArray new: 64.
	out replaceFrom: 1 to: buffer size with: buffer startingAt: 1.
	buffer size < 56 ifTrue: [  "padding and length fit in last data block"
		out at: buffer size + 1 put: 128.  "trailing one bit"
		self storeLength: bitLength in: out.  "end with length"
		self processBuffer: out.
		^ self].

	"process the final data block"
	buffer size < 64 ifTrue: [
		out at: buffer size + 1 put: 128].  "trailing one bit"
	self processBuffer: out.

	"process one additional block of padding ending with the length"
	out := ByteArray new: 64.  "filled with zeros"
	buffer size = 64 ifTrue: [
		"add trailing one bit that didn't fit in final data block"
		out at: 1 put: 128].
	self storeLength: bitLength in: out.
	self processBuffer: out.
! !

!SecureHashAlgorithm methodsFor: 'private' stamp: 'jm 12/14/1999 11:10'!
storeLength: bitLength in: aByteArray
	"Fill in the final 8 bytes of the given ByteArray with a 64-bit big-endian representation of the original message length in bits."

	| n i |
	n := bitLength.
	i := aByteArray size.
	[n > 0] whileTrue: [
		aByteArray at: i put: (n bitAnd: 16rFF).
		n := n bitShift: -8.
		i := i - 1].
! !

Object subclass: #SerialPort
	instanceVariableNames: 'port baudRate stopBitsType parityType dataBits outputFlowControlType inputFlowControlType xOnByte xOffByte'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Serial Port'!

!SerialPort commentStamp: '<historical>' prior: 0!
This class supports a simple interface to the serial ports of the underlying platform, if it supports serial ports. The mapping of port numbers to hardware ports is platform specific, but typically follows platform ordering conventions. For example, on the Macintosh, port 0 is the modem port and port 1 is the printer port, since in the programmers documentation these ports are referred to as ports A and B.
!

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:19'!
baudRate

	^ baudRate
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:29'!
baudRate: anInteger
	"Set the baud rate for this serial port."

	baudRate := anInteger.
! !

!SerialPort methodsFor: 'open/close' stamp: 'StephaneDucasse 2/12/2011 16:45'!
close
	"Close the serial port. Do nothing if the port is not open."
	
	port ifNotNil: [port isString 
					ifTrue: [self primClosePortByName: port]
 					ifFalse: [self primClosePort: port]].
	port := nil! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:19'!
dataBits

	^ dataBits
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:29'!
dataBits: anInteger
	"Set the number of data bits for this serial port to 5, 6, 7, or 8."

	dataBits := anInteger.
! !

!SerialPort methodsFor: 'initialize' stamp: 'alain.plantec 5/28/2009 10:21'!
initialize
	"Default port settings."

	super initialize.
	port := nil.					"set when opened"
	baudRate := 9600.			"9600 baud"
	stopBitsType := 1.				"one stop bit"
	parityType := 0.				"no parity"
	dataBits := 8.					"8 bits"
	outputFlowControlType := 0.	"none"
	inputFlowControlType := 0.	"none"
	xOnByte := 19.				"ctrl-S"
	xOffByte := 24.				"ctrl-X"
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:21'!
inputFlowControlType

	^ inputFlowControlType
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:38'!
inputFlowControlType: anInteger
	"Set the type of input flow control, where:
		0 - none
		1 - XOn/XOff
		2 - hardware handshaking"

	inputFlowControlType := anInteger.
! !

!SerialPort methodsFor: 'input/output' stamp: 'StephaneDucasse 2/12/2011 16:43'!
nextPutAll: aStringOrByteArray 
	"Send the given bytes out this serial port. The port must be
	open. "
	^ port isString
		ifTrue: [self
					primWritePortByName: port
					from: aStringOrByteArray
					startingAt: 1
					count: aStringOrByteArray size]
		ifFalse: [self
					primWritePort: port
					from: aStringOrByteArray
					startingAt: 1
					count: aStringOrByteArray size]
! !

!SerialPort methodsFor: 'open/close' stamp: 'BillSchwab 7/16/2011 17:42'!
openPort: portId 
	"Open the given serial port, using the settings specified by my
	instance variables."

		| result |	

	self close.
	
	"8-10 - wks - this has to tell us if it fails: step 1, save the primitive result"
	result := portId isString
		ifTrue: [self
				primOpenPortByName: portId
				baudRate: baudRate
				stopBitsType: stopBitsType
				parityType: parityType
				dataBits: dataBits
				inFlowControlType: inputFlowControlType
				outFlowControlType: outputFlowControlType
				xOnByte: xOnByte
				xOffByte: xOffByte]
		ifFalse: [self
				primOpenPort: portId
				baudRate: baudRate
				stopBitsType: stopBitsType
				parityType: parityType
				dataBits: dataBits
				inFlowControlType: inputFlowControlType
				outFlowControlType: outputFlowControlType
				xOnByte: xOnByte
				xOffByte: xOffByte].

	"wks - raise an error if there was a failure."
	result isNil ifTrue:[ self error:'Cannot open ', portId printString. ].

	port := portId
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:19'!
outputFlowControlType

	^ outputFlowControlType
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:38'!
outputFlowControlType: anInteger
	"Set the type of output flow control, where:
		0 - none
		1 - XOn/XOff
		2 - hardware handshaking"

	outputFlowControlType := anInteger.
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:19'!
parityType

	^ parityType
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:29'!
parityType: anInteger
	"Set the parity type for this serial port, where:
		0 - no parity
		1 - odd parity
		2 - even parity"

	parityType := anInteger.
! !

!SerialPort methodsFor: 'primitives' stamp: 'ar 2/2/2001 15:09'!
primClosePort: portNumber

	<primitive: 'primitiveSerialPortClose' module: 'SerialPlugin'>
	^ nil  "(DNS)"
	"self primitiveFailed."
! !

!SerialPort methodsFor: 'primitives' stamp: 'Sk 1/4/2009 18:24'!
primClosePortByName: portName
	<primitive: 'primitiveSerialPortCloseByName' module:'SerialPlugin'>
	^ nil"(DNS)"
	"self primitiveFailed."! !

!SerialPort methodsFor: 'primitives' stamp: 'ar 2/2/2001 15:09'!
primOpenPort: portNumber baudRate: baud stopBitsType: stop
	parityType: parity dataBits: numDataBits
	inFlowControlType: inFlowCtrl outFlowControlType: outFlowCtrl
	xOnByte: xOn xOffByte: xOff

	<primitive: 'primitiveSerialPortOpen' module: 'SerialPlugin'>
	^ nil  "(DNS)"
! !

!SerialPort methodsFor: 'primitives' stamp: 'Sk 1/4/2009 18:24'!
primOpenPortByName: portName baudRate: baud stopBitsType: stop parityType: parity dataBits: numDataBits inFlowControlType: inFlowCtrl outFlowControlType: outFlowCtrl xOnByte: xOn xOffByte: xOff 
	<primitive: 'primitiveSerialPortOpenByName' module:'SerialPlugin'>
	^ nil"(DNS)"! !

!SerialPort methodsFor: 'primitives' stamp: 'ar 2/2/2001 15:09'!
primReadPort: portNumber into: byteArray startingAt: startIndex count: count

	<primitive: 'primitiveSerialPortRead' module: 'SerialPlugin'>
	self primitiveFailed.
! !

!SerialPort methodsFor: 'primitives' stamp: 'Sk 1/4/2009 18:24'!
primReadPortByName: portName into: byteArray startingAt: startIndex count: count 
	<primitive: 'primitiveSerialPortReadByName' module:'SerialPlugin'>
	self primitiveFailed! !

!SerialPort methodsFor: 'primitives' stamp: 'ar 2/2/2001 15:09'!
primWritePort: portNumber from: byteArray startingAt: startIndex count: count

	<primitive: 'primitiveSerialPortWrite' module: 'SerialPlugin'>
	self primitiveFailed.
! !

!SerialPort methodsFor: 'primitives' stamp: 'Sk 1/4/2009 18:25'!
primWritePortByName: portName from: byteArray startingAt: startIndex count: count 
	<primitive: 'primitiveSerialPortWriteByName' module:'SerialPlugin'>
	self primitiveFailed! !

!SerialPort methodsFor: 'printing' stamp: 'jm 5/1/1998 18:02'!
printOn: aStream

	aStream
		nextPutAll: 'SerialPort(';
		nextPutAll:
			(port ifNil: ['closed'] ifNotNil: ['#', port printString]);
		nextPutAll: ', ';
		print: baudRate; nextPutAll: ' baud, ';
		print: dataBits; nextPutAll: ' bits, ';
		nextPutAll: (#('1.5' '1' '2') at: stopBitsType + 1); nextPutAll: ' stopbits, ';
		nextPutAll: (#('no' 'odd' 'even') at: parityType + 1); nextPutAll: ' parity)'.
! !

!SerialPort methodsFor: 'input/output' stamp: 'StephaneDucasse 2/12/2011 16:43'!
readByteArray
	"Answer a ByteArray read from this serial port. Answer an empty ByteArray if no data is available. The port must be open. "
	
	| buf count |
	buf := ByteArray new: 1000.
	count := port isString
				ifTrue: [self
						primReadPortByName: port
						into: buf
						startingAt: 1
						count: buf size]
				ifFalse: [self
						primReadPort: port
						into: buf
						startingAt: 1
						count: buf size].
	^ buf copyFrom: 1 to: count! !

!SerialPort methodsFor: 'input/output' stamp: 'StephaneDucasse 2/12/2011 16:44'!
readInto: aStringOrByteArray startingAt: index 
	"Read data into the given String or ByteArray object starting at the given index, and answer the number of bytes read. Does not go past the end of the given String or ByteArray."
	
	^ port isString 
		ifTrue: [self primReadPortByName: port
					into: aStringOrByteArray
					startingAt: index
					count: aStringOrByteArray size - index + 1]
		ifFalse: [self primReadPort: port
					into: aStringOrByteArray
					startingAt: index
					count: aStringOrByteArray size - index + 1].
		! !

!SerialPort methodsFor: 'input/output' stamp: 'StephaneDucasse 2/12/2011 16:44'!
readString
	"Answer a String read from this serial port. Answer the empty
	String if no data is available. The port must be open."
	| buf count |
	buf := String new: 1000.
	count := port isString 
				ifTrue: [ self
							primReadPortByName: port
							into: buf
							startingAt: 1
							count: buf size.]
				ifFalse: [ self
						 	primReadPort: port
							into: buf
							startingAt: 1
							count: buf size.].
	^ buf copyFrom: 1 to: count! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:19'!
stopBitsType

	^ stopBitsType
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 18:02'!
stopBitsType: anInteger
	"Set the stop bits type for this serial port, where:
		0 - 1.5 stop bits
		1 - one stop bit
		2 - two stop bits"

	stopBitsType := anInteger.
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:20'!
xOffByte

	^ xOffByte
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:28'!
xOffByte: anInteger
	"Set the value of the XOff byte to be used if XOn/XOff flow control is enabled."

	xOffByte := anInteger.
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:20'!
xOnByte

	^ xOnByte
! !

!SerialPort methodsFor: 'accessing' stamp: 'jm 5/1/1998 17:28'!
xOnByte: anInteger
	"Set the value of the XOn byte to be used if XOn/XOff flow control is enabled."

	xOnByte := anInteger.
! !

Object subclass: #SettingFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Filter'!

!SettingFilter commentStamp: 'alain.plantec 4/8/2009 12:51' prior: 0!
A SettingFilter is a filter wich is used by a SettingBrowser in order to select which nodes of the setting trees are to be shown. A SettingFilter subclass must redefine the #keepHandler: method which return true if the argument handler is to be kept.

Instance Variables
!

!SettingFilter methodsFor: 'filtering' stamp: 'alain.plantec 9/18/2009 23:46'!
keepSettingTreeNode: aSettingNode 
	^ false! !

SettingFilter subclass: #SettingPackageFilter
	instanceVariableNames: 'packages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Filter'!

!SettingPackageFilter commentStamp: 'AlainPlantec 12/31/2010 11:38' prior: 0!
A SettingPackageFilter is a SettingFilter used to select settings from a given list of packages: a setting is kept only if the package in which it is defined is present in the given package list

Instance Variables
	packages:		<Collection>

packages
	- The collection of PackageInfo the filter is currently using!

!SettingPackageFilter methodsFor: 'filtering' stamp: 'AlainPlantec 11/27/2009 10:28'!
keepSettingTreeNode: aSettingNode 
	| rp |
	^ (rp := aSettingNode receiverPackage) notNil and: [self packages includes: rp]! !

!SettingPackageFilter methodsFor: 'accessing' stamp: 'alain.plantec 3/16/2009 23:00'!
packages
	"Answer the value of packages"

	^ packages! !

!SettingPackageFilter methodsFor: 'accessing' stamp: 'alain.plantec 4/8/2009 12:55'!
packages: aCollectionOfPackageInfo
	"Set the value of packages"

	packages := aCollectionOfPackageInfo
! !

SettingFilter subclass: #SettingRegexpFilter
	instanceVariableNames: 'regexp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Filter'!

!SettingRegexpFilter commentStamp: 'alain.plantec 4/8/2009 13:02' prior: 0!
A SettingRegexpFilter is SettingFilter which makes use of a regular expression in order to keep or not a SettingHandler.

Instance Variables
	regexp:		<String>

regexp
	- a regular expression string
!

!SettingRegexpFilter methodsFor: 'filtering' stamp: 'AlainPlantec 1/1/2012 22:39'!
keepSettingDeclaration: aSettingDeclaration
	^ ({aSettingDeclaration name. aSettingDeclaration label. aSettingDeclaration description}
				collect: [:e | e ifNil: ['']])
				anySatisfy: [:token | token  matchesRegexIgnoringCase: self regexp]
! !

!SettingRegexpFilter methodsFor: 'filtering' stamp: 'alain.plantec 9/18/2009 23:46'!
keepSettingTreeNode: aSettingNode 
	^ (({aSettingNode methodClass name. aSettingNode selector}
			collect: [:e | e ifNil: ['']])
			anySatisfy: [:token | token substrings anySatisfy: [:sub | sub matchesRegexIgnoringCase: self regexp]])
		or: [self keepSettingDeclaration: aSettingNode settingDeclaration] ! !

!SettingRegexpFilter methodsFor: 'accessing' stamp: 'alain.plantec 4/7/2009 23:52'!
regexp
	"Answer the value of regexp"

	^ regexp! !

!SettingRegexpFilter methodsFor: 'accessing' stamp: 'alain.plantec 4/7/2009 23:52'!
regexp: anObject
	"Set the value of regexp"

	regexp := anObject! !

SettingFilter subclass: #SettingTokenFilter
	instanceVariableNames: 'token'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Filter'!

!SettingTokenFilter commentStamp: 'alain.plantec 4/8/2009 13:03' prior: 0!
A SettingTokenFilter is SettingFilter which makes use of a token (substring) in order to keep or not a SettingHandler.


Instance Variables
	token:		<String>

!

!SettingTokenFilter methodsFor: 'filtering' stamp: 'AlainPlantec 10/30/2011 13:03'!
keepSettingDeclaration: aSettingDeclaration
	| selector |
	selector := (aSettingDeclaration respondsTo: #getSelector) 
		ifTrue: [aSettingDeclaration getSelector] ifFalse: [''].
	^ (({aSettingDeclaration name.  
		aSettingDeclaration label. 
		selector asUppercase. 
		aSettingDeclaration description}
				collect: [:e | (e ifNil: ['']) asUppercase])
				includesSubstringAnywhere: self token asUppercase)
! !

!SettingTokenFilter methodsFor: 'filtering' stamp: 'AlainPlantec 9/24/2011 22:18'!
keepSettingTreeNode: aSettingNode 
	^ (({aSettingNode methodClass name asUppercase. aSettingNode selector asUppercase}
			collect: [:e | e ifNil: ['']])
			includesSubstringAnywhere: self token asUppercase)
		or: [self keepSettingDeclaration: aSettingNode settingDeclaration]! !

!SettingTokenFilter methodsFor: 'accessing' stamp: 'alain.plantec 3/16/2009 22:51'!
token
	"Answer the value of token"

	^ token! !

!SettingTokenFilter methodsFor: 'accessing' stamp: 'alain.plantec 3/16/2009 22:51'!
token: aString
	"Set the value of token"

	token := aString! !

Object subclass: #SettingNodeBuilder
	instanceVariableNames: 'node builder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Browser'!

!SettingNodeBuilder methodsFor: 'private tree building' stamp: 'AlainPlantec 11/24/2009 13:34'!
asParentWhile: aBlock
	builder parent: self while: aBlock
! !

!SettingNodeBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 13:33'!
builder: aTreeBuilder
	builder := aTreeBuilder! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:50'!
default: anObject
	node item  default: anObject
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
description: aText
	node item description: aText! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
dialog: aSymbol
	node item dialog: aSymbol
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
domainValues: aCollection 
	node item domainValues: aCollection 
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
getSelector: aSymbol
	node item getSelector: aSymbol
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
ghostHelp: aString
	node item ghostHelp: aString
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
icon: aForm
	node item icon: aForm! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
label: aStringOrBlockOrMessageSend
	node item label: aStringOrBlockOrMessageSend.
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 14:02'!
name
	^ node item name
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
name: aStringOrBlockOrMessageSend
	node item name: aStringOrBlockOrMessageSend.
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/27/2009 23:26'!
noOrdering
	node item noOrdering
! !

!SettingNodeBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 12:43'!
node
	^ node! !

!SettingNodeBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 12:43'!
node: aSettingTreeNode
	node := aSettingTreeNode! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 12/3/2009 16:21'!
notInStyle
	node item notInStyle! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
order: aNumber
	node item order: aNumber
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:51'!
parent: aSymbol
	node parentName: aSymbol! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
parentName
	^ node parentName
	! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
parentName: aSymbol
	node parentName: aSymbol.
	! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 12/5/2009 08:44'!
precondition: aValuable
	node item precondition: aValuable
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
range: anInterval
	node item range: anInterval
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 12/5/2009 13:52'!
script: aSymbol
	node item script: aSymbol
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
selector: aSymbol
	node item selector: aSymbol
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
setSelector: aSymbol
	node item setSelector: aSymbol
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
target: anObject
	node item target: anObject
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/26/2009 21:39'!
targetSelector: aSelector
	node item targetSelector: aSelector
! !

!SettingNodeBuilder methodsFor: 'public declaration accessing' stamp: 'AlainPlantec 11/24/2009 13:52'!
type: anObject
	node item type: anObject
! !

!SettingNodeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 11/24/2009 12:40'!
with: aBlock
	self asParentWhile: aBlock! !

Object subclass: #SettingTree
	instanceVariableNames: 'collector nodeList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Browser'!

!SettingTree commentStamp: 'AlainPlantec 1/2/2011 22:56' prior: 0!
A SettingTree is responsible for the building of system setting trees from a collection of pragma.
Built tree nodes are instances of SettingNode. Built trees are suitable for tree browsing with the help of a MorphTreeMorph (see SettingBrowser>>treeMorphIn: and SettingBrowser>>roots to know how a SettingBrowser is initializing its MorphTreeMorph with the help of a SettingTree). 

Below, two examples of tree building with a SettingCollector are shown.
---------------
(SettingTree acceptableKeywords: #(#'systemsettings')) settingTreeRoots. "get system setting trees"
---------------

Instance Variables
	collector		<PragmaCollector>
	nodeList		<A list of SettingNode>
!

!SettingTree class methodsFor: 'instance creation' stamp: 'AlainPlantec 1/2/2011 22:55'!
acceptableKeywords: aCollectionOfStrings
	^ self new acceptableKeywords: aCollectionOfStrings 
! !

!SettingTree methodsFor: 'accessing' stamp: 'asasa  4/28/2010 18:33'!
acceptableKeywords: keywords
	self collector filter: [:prg | prg methodClass isMeta and: [keywords includes: prg keyword]].
	nodeList := nil.
	self collector reset.
! !

!SettingTree methodsFor: 'checking' stamp: 'FernandoOlivero 4/12/2011 10:17'!
checkForUnknownParent
	self unclassified ifNotEmpty: [:nodesWithUnknownParent | | builder g |
		builder := SettingTreeBuilder new.
		g := (builder group: #nodesWithUnknownParent) node.
		self nodeList addFirst: g.
		g model: self.
		g item label: '*** Unclassified ***'.
		g item icon:  self theme  smallDebugIcon.
		g item order: 0.0.
		nodesWithUnknownParent do: [:node | node parentName: #nodesWithUnknownParent]]! !

!SettingTree methodsFor: 'checking' stamp: 'AlainPlantec 12/7/2009 08:56'!
checkForUnknownTarget
	"check only root because other nodes with unknown target inherits it from its parent"
	self settingTreeRoots do: [:node | node item target ifNil: [node item target: (node pragma ifNotNil: [:prg | prg methodClass theNonMetaClass])]].
	self settingTreeRoots do: [:node | node checkForUnknownTarget].
! !

!SettingTree methodsFor: 'querying' stamp: 'MarcusDenker 5/31/2011 15:34'!
childrenOf: aNode 
	| children  idx order |
	"Get aNode children. Assign children order if one child has an order which is not nil"
	children := self nodeList select: [:p | p parentName = aNode settingDeclaration name].
	(children detect: [:n | n order notNil] ifNone:[])  ifNotNil: [:firstWithOrder |
		idx := children indexOf: firstWithOrder.
		order := firstWithOrder order.
		idx > 1
			ifTrue: [(idx - 1 to: 1) 
				do: [:pos | 
					(children at: pos) order: order - 1. 
					order := order - 1]].
		order := firstWithOrder order.
		idx + 1 to: children size
			do: [:pos | 
				(children at: pos) order 
					ifNil:  [(children at: pos) order: order + 1] 
					ifNotNil: [order := (children at: pos) order].
				order := order + 1]].
	^ children! !

!SettingTree methodsFor: 'accessing' stamp: 'alain.plantec 10/19/2009 11:28'!
collector
	^ collector ifNil: [collector := PragmaCollector new]! !

!SettingTree methodsFor: 'querying' stamp: 'alain.plantec 9/19/2009 09:32'!
deeplyDetect: aBlock 
	self settingTreeRoots
		do: [:sub | (sub deeplyDetect: aBlock)
				ifNotNil: [:found | ^ found]].
	^ nil ! !

!SettingTree methodsFor: 'querying' stamp: 'alain.plantec 4/3/2009 07:53'!
deeplySelect: aBlock
	^ self deeplySelect: aBlock in: OrderedCollection new
! !

!SettingTree methodsFor: 'querying' stamp: 'alain.plantec 4/13/2009 08:04'!
deeplySelect: aBlock in: aCollection 
	self settingTreeRoots
		do: [:aRoot | aRoot deeplySelect: aBlock in: aCollection]. 
	^ aCollection! !

!SettingTree methodsFor: 'private tree building' stamp: 'AlainPlantec 11/27/2009 22:05'!
emptyNodeNamed: aSymbol
	(self nodeNamed: aSymbol)
! !

!SettingTree methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 13:52'!
itemSortBlock
	^ [:a :b | 
		((a order notNil and: [b order notNil]) and: [a order ~= b order])
			ifTrue: [a order < b order]
			ifFalse: [((a order isNil and: [b order isNil]) or: [a order = b order])
				ifTrue: [a label < b label]
				ifFalse: [a order ifNil: [false] ifNotNil: [true]]]]! !

!SettingTree methodsFor: 'accessing' stamp: 'AlainPlantec 12/5/2009 08:46'!
nodeList
	| builder |
	^ nodeList 
		ifNil: [
			builder := SettingTreeBuilder new.
			self pragmasDo: [:p | builder buildPragma: p].
			nodeList := builder nodeList.
			nodeList do: [:n | n model: self].
			self checkForUnknownParent.
			self checkForUnknownTarget.
			self nodeList]! !

!SettingTree methodsFor: 'private tree building' stamp: 'AlainPlantec 11/24/2009 09:35'!
nodeNamed: aSymbol
	^ self nodeNamed: aSymbol ifAbsent: []
! !

!SettingTree methodsFor: 'private tree building' stamp: 'AlainPlantec 11/24/2009 09:34'!
nodeNamed: aSymbol ifAbsent: aBlock
	^ self nodeList detect: [:d | d settingDeclaration name = aSymbol] ifNone: aBlock! !

!SettingTree methodsFor: 'querying' stamp: 'alain.plantec 10/19/2009 13:56'!
pragmasDo: aBlock
	^  self collector do: aBlock! !

!SettingTree methodsFor: 'querying' stamp: 'AlainPlantec 12/5/2009 08:51'!
retainedNodesFromList: aListOfNodes
	| retained |
	retained := OrderedCollection new.
	aListOfNodes do: [:n | [n item precondition value ifTrue: [retained add: n]] on: Error do: [retained add: n]].
	^ retained! !

!SettingTree methodsFor: 'querying' stamp: 'AlainPlantec 12/5/2009 08:51'!
settingTreeRoots
	^ self retainedNodesFromList: ((self nodeList select: [:n | n parentName isNil]) asArray sort: self sortBlock) asOrderedCollection.
! !

!SettingTree methodsFor: 'accessing' stamp: 'AlainPlantec 1/29/2010 13:52'!
sortBlock
	^ [:a :b | self itemSortBlock value: a settingDeclaration value: b settingDeclaration]! !

!SettingTree methodsFor: 'theme' stamp: 'BenjaminVanRyseghem 5/16/2011 16:12'!
theme

	^ UITheme current! !

!SettingTree methodsFor: 'querying' stamp: 'AlainPlantec 11/24/2009 12:01'!
unclassified
	^ self nodeList select: [:node | node parentName notNil and: [node parentNode isNil]].! !

!SettingTree methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 11:51'!
updateList
	nodeList := nil.
! !

!SettingTree methodsFor: 'updating' stamp: 'alain.plantec 10/19/2009 11:37'!
whenChangedSend: aSelector to: aTarget
	self collector whenChangedSend: aSelector to: aTarget.
	self collector whenResetSend: aSelector to: aTarget.! !

Object subclass: #SettingTreeBuilder
	instanceVariableNames: 'nodeList currentParent currentPragma'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Settings-Browser'!

!SettingTreeBuilder class methodsFor: 'instance creation' stamp: 'AlainPlantec 11/24/2009 11:53'!
acceptableKeywords: aCollectionOfRegExps 
	^ self new acceptableKeywords: aCollectionOfRegExps 
! !

!SettingTreeBuilder methodsFor: 'accessing' stamp: 'asasa  4/28/2010 18:21'!
buildPragma: aPragma
	currentPragma := aPragma.
	self perform: aPragma keyword withArguments: aPragma arguments.
	^ nodeList! !

!SettingTreeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 1/29/2010 14:07'!
group: aSymbol
	^ self nodeClass: PragmaSetting name: aSymbol
! !

!SettingTreeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 12/5/2009 13:50'!
launcher: aSymbol
	^ self nodeClass: SystemSettingLauncher name: aSymbol
! !

!SettingTreeBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 1/30/2010 11:53'!
nodeClass
	^ SettingNode! !

!SettingTreeBuilder methodsFor: 'private tree building' stamp: 'AlainPlantec 1/30/2010 11:54'!
nodeClass: aClass name: aSymbol
	| node |
	node := self nodeClass with: aClass new.
	node item name: aSymbol.
	node pragma: currentPragma.
	node parentName: (currentParent ifNotNil: [currentParent name]).
	self nodeList add: node.
	^ (SettingNodeBuilder new) node: node; builder: self; yourself! !

!SettingTreeBuilder methodsFor: 'accessing' stamp: 'AlainPlantec 11/24/2009 12:05'!
nodeList
	^ nodeList ifNil: [nodeList := OrderedCollection new]
! !

!SettingTreeBuilder methodsFor: 'private tree building' stamp: 'AlainPlantec 11/24/2009 13:35'!
parent: aNode while: aBlock
	| oldParent |
	oldParent := currentParent.
	currentParent := aNode.
	aBlock value.
	currentParent := oldParent! !

!SettingTreeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 12/2/2009 11:47'!
pickOne: aSymbol
	^ self nodeClass: PickOneSettingDeclaration name: aSymbol
! !

!SettingTreeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 11/24/2009 12:15'!
range: aSymbol
	^ self nodeClass: RangeSettingDeclaration name: aSymbol
! !

!SettingTreeBuilder methodsFor: 'public tree building' stamp: 'AlainPlantec 11/24/2009 12:15'!
setting: aSymbol
	^ self nodeClass: SettingDeclaration name: aSymbol
! !

!SettingTreeBuilder methodsFor: 'pragmas' stamp: 'AlainPlantec 4/29/2010 17:44'!
systemsettings
	<settingPragmaProcessor>
	"Process a <systemsettings> pragma"
	currentPragma methodClass theNonMetaClass perform: currentPragma selector with: self.! !

Object subclass: #SmalltalkImage
	instanceVariableNames: 'globals deferredStartupActions specialObjectsArray'
	classVariableNames: 'CheckChangesFileAvailability CheckSourcesFileAvailability EndianCache LastImagePath LastQuitLogPosition LowSpaceProcess LowSpaceSemaphore MemoryHogs ShutDownList SourceFileVersionString SpecialSelectors StartUpList Tools'
	poolDictionaries: ''
	category: 'System-Support'!

!SmalltalkImage commentStamp: 'StephaneDucasse 6/7/2011 15:48' prior: 0!
My singleton is a central entry point to the system.

It has many roles and responsibilities (to be documented)
	- startup/shutdown image
	- source management
	- namespace access
	- tool access
	- platform access
	- VM information and parameters	
	
Startup
-----------
At startup and shutdown the image execute the methods startUp: and shutdown: of registered classes (registered using addToStartUpList:, addToShutDownList:, ... methods and friends).
	
Startup phases
During the first stage of start up the UI manager, the default uimanager is switched to a specific non interactive ui manager (StartupUIManager). Note that this specific non interactive UIManager kills the system on any attempt to open windows and interaction. So be warned, don't use interaction in the first phase. 
Then all registered classes execute their start up procedures (which should not imply interactive behavior). 
After startup list is finished, any deferred startup actions are executed, which you can add using the method
addDeferredStartupAction: method.  





!

!SmalltalkImage class methodsFor: 'settings' stamp: ''!
checkChangesFileAvailability
	^ CheckChangesFileAvailability ifNil: [CheckChangesFileAvailability := true]! !

!SmalltalkImage class methodsFor: 'settings' stamp: ''!
checkChangesFileAvailability: aBoolean
	CheckChangesFileAvailability := aBoolean! !

!SmalltalkImage class methodsFor: 'settings' stamp: ''!
checkSourcesFileAvailability
	^ CheckSourcesFileAvailability ifNil: [CheckSourcesFileAvailability := true]! !

!SmalltalkImage class methodsFor: 'settings' stamp: ''!
checkSourcesFileAvailability: aBoolean
	CheckSourcesFileAvailability := aBoolean! !

!SmalltalkImage class methodsFor: 'cleanup' stamp: 'AlainPlantec 12/1/2010 09:53'!
cleanUp
	"Flush caches"
	
	"should probably distribute cleaning to systemDictionary"
	
	Smalltalk globals flushClassNameCache.
	Undeclared removeUnreferencedKeys! !

!SmalltalkImage class methodsFor: 'instance creation' stamp: ''!
current
	^Smalltalk! !

!SmalltalkImage class methodsFor: 'initialization' stamp: 'MarcusDenker 7/12/2012 17:59'!
howToGetClassesNotRegisteredInStartups

	| classes startClassNames stopClassNames |
	classes := ((SystemNavigation new allCallsOn:
#'addToStartUpList:') collect: [ :e | e methodClass ]),
((SystemNavigation new allCallsOn: #'addToStartUpList:after:')
collect: [ :e | e methodClass ]).
startClassNames := classes collect: [ :n | n instanceSide name ].
(Smalltalk class classVarNamed: 'StartUpList') do: [ :s |
	startClassNames remove: s ifAbsent: [] ].


classes := ((SystemNavigation ne allCallsOn:
#'addToShutDownList:') collect: [ :e | e methodClass ]),
((SystemNavigation new allCallsOn: #'addToShutDownList:after:')
collect: [ :e | e methodClass ]).
stopClassNames := classes collect: [ :n | n instanceSide name ].
(Smalltalk class classVarNamed: 'ShutDownList') do: [ :s |
	stopClassNames remove: s ifAbsent: [] ].

((startClassNames, stopClassNames) asSet asSortedCollection remove: #InputEventSensor ; yourself)
	"do: [:each | (Smalltalk at: each) initialize] does not work because of dependencies"! !

!SmalltalkImage class methodsFor: 'initialization' stamp: ''!
initialize
	"SmalltalkImage initialize"

	self initializeStartUpList.
	self initializeShutDownList.
	Smalltalk addToStartUpList: SmalltalkImage! !

!SmalltalkImage class methodsFor: 'initialization' stamp: 'CamilloBruni 5/24/2012 15:03'!
initializeShutDownList
	"SmalltalkImage initializeShutDownList"

	| oldList |
	oldList := ShutDownList.
	ShutDownList := OrderedCollection new.	"These get processed from the bottom up..."
	#(#Delay #DisplayScreen #InputEventFetcher #Form #PasteUpMorph #StrikeFont #Color #SoundPlayer #HttpUrl #Password)
		do: [ :clsName | Smalltalk globals at: clsName ifPresent: [ :cls | Smalltalk addToShutDownList: cls ] ].
	oldList
		ifNotNil: [ 
			oldList
				reverseDo: [ :className | Smalltalk globals at: className ifPresent: [ :theClass | Smalltalk addToShutDownList: theClass ] ] ]! !

!SmalltalkImage class methodsFor: 'initialization' stamp: 'CamilloBruni 5/24/2012 15:03'!
initializeStartUpList
	"SmalltalkImage initializeStartUpList"

	| oldList |
	oldList := StartUpList.
	StartUpList := OrderedCollection new.	"These get processed from the top down..."
	#(#Delay #DisplayScreen #Cursor #InputEventFetcher #ProcessorScheduler #LanguageEnvironment #NaturalLanguageTranslator #ShortIntegerArray #ShortRunArray)
		do: [ :clsName | Smalltalk globals at: clsName ifPresent: [ :cls | Smalltalk addToStartUpList: cls ] ].
	oldList
		ifNotNil: [ 
			oldList
				do: [ :className | Smalltalk globals at: className ifPresent: [ :theClass | Smalltalk addToStartUpList: theClass ] ] ].
	#(#PasteUpMorph) do: [ :clsName | Smalltalk globals at: clsName ifPresent: [ :cls | Smalltalk addToStartUpList: cls ] ]! !

!SmalltalkImage class methodsFor: 'instance creation' stamp: ''!
new

	self error: 'Use current'.! !

!SmalltalkImage class methodsFor: 'initialization' stamp: 'EstebanLorenzano 7/27/2012 14:12'!
startUp
	SystemAnnouncer uniqueInstance
		unsubscribe: ChangesLog default;
		on: SystemAnnouncement send: #logAnnouncement: to: ChangesLog default.
	Smalltalk openSourceFiles! !

!SmalltalkImage class methodsFor: 'initialize-release' stamp: 'IgorStasenko 8/17/2011 15:04'!
wordSize
	^ Smalltalk vm wordSize! !

!SmalltalkImage methodsFor: 'startup list' stamp: 'MarcusDenker 4/16/2011 23:09'!
add: aClass toList: startUpOrShutDownList after: predecessor
	"Add the name of aClass to the startUp or shutDown list.
	Add it after the name of predecessor"

	(Smalltalk globals includes: aClass) 
		ifFalse: [self error: aClass name , ' cannot be found in Smalltalk dictionary.'].

	"Add after predecessor, moving it if already there."
	(Smalltalk globals includes: predecessor)  
		ifFalse: [self error: predecessor name , ' cannot be found in Smalltalk dictionary.'].
	(startUpOrShutDownList includes: predecessor name) 
		ifFalse: [self error: predecessor name , ' cannot be found in the list.'].
	startUpOrShutDownList remove: aClass name ifAbsent:[].
	startUpOrShutDownList add: aClass name after: predecessor name! !

!SmalltalkImage methodsFor: 'startup list' stamp: 'MarcusDenker 4/16/2011 23:09'!
add: aClass toList: startUpOrShutDownList before: successor
	"Add the name of aClass to the startUp or shutDown list.
	Add it before the name of successor"

	(Smalltalk globals includes: aClass) 
		ifFalse: [self error: aClass name , ' cannot be found in Smalltalk dictionary.'].
		
	"Add before successor, moving it if already there."
	(Smalltalk globals includes: successor)  
		ifFalse: [self error: successor name , ' cannot be found in Smalltalk dictionary.'].
	(startUpOrShutDownList includes: successor name) 
		ifFalse: [self error: successor name , ' cannot be found in the list.'].
	startUpOrShutDownList remove: aClass name ifAbsent: [].
	startUpOrShutDownList add: aClass name before: successor name.! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'StephaneDucasse 6/7/2011 15:22'!
addDeferredStartupAction: aBlock
	"Add the monadic or nulladic block to the list of actions that we will be performed immediately after the startup is totally executed."
	
	deferredStartupActions 
		ifNil: [deferredStartupActions := OrderedCollection new].
	deferredStartupActions add: aBlock.
	! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'CamilloBruni 5/13/2012 19:28'!
addSnapshotRecord: save andQuit: quit
	"Originally this code were in #snapshot:andQuit: .
	Just moved it to separate method to make caller more readable
	and don't look like swiss army knife which tries to do everything by own "
	
	| msg |
	(SourceFiles at: 2)
		ifNotNil: [ 
			msg := String
				streamContents: [ :s | 
					s
						nextPutAll: '----';
						nextPutAll:
								(save
										ifTrue: [ 
											quit
												ifTrue: [ 'QUIT' ]
												ifFalse: [ 'SNAPSHOT' ] ]
										ifFalse: [ 
											quit
												ifTrue: [ 'QUIT/NOSAVE' ]
												ifFalse: [ 'NOP' ] ]);
						nextPutAll: '----';
						print: Date dateAndTimeNow;
						space;
						nextPutAll: (self imageFile basename);
						nextPutAll: ' priorSource: ';
						print: LastQuitLogPosition ].
			self assureStartupStampLogged.
			save
				ifTrue: [ 
					LastQuitLogPosition := (SourceFiles at: 2)
						setToEnd;
						position ].
			self logChange: msg.
			self traceCr: msg ].
! !

!SmalltalkImage methodsFor: 'startup list' stamp: 'MarcusDenker 4/16/2011 23:07'!
addToShutDownList: aClass
	"This will add a ref to this class at the BEGINNING of the shutDown list."
	"No-op if already in the list."
	
	(ShutDownList includes: aClass name) ifFalse: [ShutDownList addFirst: aClass name]! !

!SmalltalkImage methodsFor: 'startup list' stamp: ''!
addToShutDownList: aClass after: predecessor

	self add: aClass toList: ShutDownList after: predecessor! !

!SmalltalkImage methodsFor: 'startup list' stamp: 'MarcusDenker 4/16/2011 22:36'!
addToShutDownList: aClass before: predecessor

	self add: aClass toList: ShutDownList before: predecessor! !

!SmalltalkImage methodsFor: 'startup list' stamp: 'MarcusDenker 4/16/2011 23:08'!
addToStartUpList: aClass
	"This will add a ref to this class at the END of the startUp list."
	"No-op if already in the list."

	(StartUpList includes: aClass name) ifFalse: [StartUpList addLast: aClass name]! !

!SmalltalkImage methodsFor: 'startup list' stamp: ''!
addToStartUpList: aClass after: predecessor

	self add: aClass toList: StartUpList after: predecessor! !

!SmalltalkImage methodsFor: 'startup list' stamp: 'MarcusDenker 4/16/2011 22:36'!
addToStartUpList: aClass before: predecessor

	self add: aClass toList: StartUpList before: predecessor! !

!SmalltalkImage methodsFor: 'to clean later' stamp: 'MarcusDenker 4/21/2011 14:45'!
allClasses  
	"Return all the class defines in the SmalltalkImage"
	"Smalltalk allClasses"
	"May be we could discard this one in the future"
	^ globals allClasses! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: 'MarcusDenker 4/21/2011 14:45'!
allClassesAndTraits
	"Return all the classes and traits defined in the Smalltalk SmalltalkImage"

	^ globals allClassesAndTraits! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: 'MarcusDenker 4/21/2011 21:33'!
allClassesAndTraitsDo: aBlock
	^globals allClassesAndTraitsDo: aBlock! !

!SmalltalkImage methodsFor: 'to clean later' stamp: 'MarcusDenker 4/21/2011 14:45'!
allClassesDo: aBlock
	"Evaluate the argument, aBlock, for each class in the system."

	globals allClassesDo: aBlock! !

!SmalltalkImage methodsFor: 'to clean later' stamp: 'MarcusDenker 4/21/2011 21:39'!
allTraits
	"Return all traits defined in the Smalltalk SmalltalkImage"

	^globals allTraits! !

!SmalltalkImage methodsFor: 'to clean later' stamp: 'IgorStasenko 8/17/2011 16:59'!
argumentAt: i
	"Answer the i-th argument of the command line, or nil if not so many argument."
	
	^self vm getSystemAttribute: 2 + i! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'CamilloBruni 8/15/2011 23:46'!
arguments
	"Answer an array with all the command line arguments.
	This does not include imagePath, documentPath nor any option."
	
	"Smalltalk commandLine arguments"
	
	^Array streamContents: [ :str || arg i |
		i := 1.
		[i > 998 or: [(arg := self argumentAt: i) == nil]]
			whileFalse:
				[str nextPut: arg.
				i := i + 1]].! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
associationAt: key ifAbsent: aBlock
	"Obsoleted."

	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals associationAt: key ifAbsent: aBlock! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
associationOrUndeclaredAt: key 
	"return an association or install in undeclared.  Used for mating up ImageSegments."

	^ globals associationOrUndeclaredAt: key! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: 'GuillermoPolito 7/2/2012 12:30'!
assureStartupStampLogged
	"If there is a startup stamp not yet actually logged to disk, do it now."
	ChangesLog default assureStartupStampLogged! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
at: aKey
	"Answer the global associated with the key."

	^globals at: aKey! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
at: key ifAbsent: aBlock
	"Answer the global associated with the key or, if key isn't found,
	answer the result of evaluating aBlock."

	^globals at: key ifAbsent: aBlock! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
at: aKey ifAbsentPut: aBlock 
	"Return the global at the given key. 
	If key is not included in the receiver store the result 
	of evaluating aBlock as new value."

	^globals at: aKey ifAbsentPut: aBlock! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
at: key ifPresent: aBlock
	"Lookup the given key in the globals. If it is present, answer the value of evaluating the given block with the value associated with the key. Otherwise, answer nil."

	^globals at: key ifPresent: aBlock! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
at: aKey put: anObject 
	"Set the global at key to be anObject.  If key is not found, create a
	new entry for key and set is value to anObject. Answer anObject."

	^globals at: aKey put: anObject! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
bindingOf: varName
	"Answer the binding of some variable resolved in the scope of the receiver"

	^globals bindingOf: varName! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
bytesLeft
	"Answer the number of bytes of space available. Does a full garbage collection."

	^ self garbageCollect
! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
bytesLeft: aBool
	"Return the amount of available space. If aBool is true, include possibly available swap space. If aBool is false, include possibly available physical memory. For a report on the largest free block currently availabe within Pharo memory but not counting extra memory use #primBytesLeft."
	<primitive: 112>
	^self primBytesLeft! !

!SmalltalkImage methodsFor: 'endian' stamp: ''!
calcEndianness
	| bytes word blt |
	"What endian-ness is the current hardware?  The String '1234' will be stored into a machine word.  On BigEndian machines (the Mac), $1 will be the high byte if the word.  On LittleEndian machines (the PC), $4 will be the high byte."
	"SmalltalkImage current endianness"

	bytes := #[0 0 0 0].  "(1 2 3 4) or (4 3 2 1)"
	word := WordArray with: 16r01020304.
	blt := (BitBlt toForm: (Form new hackBits: bytes)) 
				sourceForm: (Form new hackBits: word).
	blt combinationRule: Form over.  "store"
	blt sourceY: 0; destY: 0; height: 1; width: 4.
	blt sourceX: 0; destX: 0.
	blt copyBits.  "paste the word into the bytes"
	bytes first = 1 ifTrue: [^ #big].
	bytes first = 4 ifTrue: [^ #little].
	self error: 'Ted is confused'.! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/13/2012 19:31'!
changeImagePathTo: aString
	
	self imagePath: aString asSqueakPathName.
	LastImagePath := self imagePath! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 7/10/2012 21:49'!
changesFile
	"Answer the file containing the current changes."

	^ FileLocator changes
! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 01:53'!
changesName
	"Answer the name for the changes file corresponding to the image file name."
	"Smalltalk changesName"

	^ self changesFile fullName ! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 01:47'!
changesSuffix
	^ 'changes'! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: 'CamilloBruni 5/13/2012 19:29'!
checkAndOpenSourcesAndChanges
	"Open the changes and sources files and install them in SourceFiles. 
	Inform the user of problems regarding write permissions or CR/CRLF mixups."
	"Note: SourcesName and imageName are full paths; 
	changesName is a  local name."
	| sources changes msg wmsg |
	msg := 'Pharo cannot locate &fileRef.

Please check that the file is named properly and is in the
same directory as this image.'.
	wmsg := 'Pharo cannot write to &fileRef.

Please check that you have write permission for this file.

You won''t be able to save this image correctly until you fix this.'.

	sources := self openSources: self sourcesName forImage: LastImagePath.
	changes := self openChanges: self changesName forImage: LastImagePath.

	((sources == nil or: [sources atEnd])
			and: [self checkSourcesFileAvailability])
		ifTrue: [OSPlatform isMacOS
				ifTrue: [msg := msg , '
Make sure the sources file is not an Alias.'].
self inform: (msg copyReplaceAll: '&fileRef' with: 'the sources file named ' , self sourcesName)].

	(changes == nil
			and: [self checkChangesFileAvailability])
		ifTrue: [self inform: (msg copyReplaceAll: '&fileRef' with: 'the changes file named ' , self changesName)].

	(self checkChangesFileAvailability and: [changes notNil])
		ifTrue: [changes isReadOnly
				ifTrue: [self inform: (wmsg copyReplaceAll: '&fileRef' with: 'the changes file named ' , self changesName)].

			((changes next: 200)
					includesSubString: String crlf)
				ifTrue: [self inform: 'The changes file named ' , self changesName , '
has been injured by an unpacking utility.  Crs were changed to CrLfs.
Please set the preferences in your decompressing program to 
"do not convert text files" and unpack the system again.']].

	SourceFiles := Array with: sources with: changes! !

!SmalltalkImage methodsFor: 'settings' stamp: ''!
checkChangesFileAvailability
	^ self class checkChangesFileAvailability! !

!SmalltalkImage methodsFor: 'settings' stamp: ''!
checkSourcesFileAvailability
	^ self class checkSourcesFileAvailability! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: 'MarcusDenker 4/21/2011 21:34'!
classNamed: className 
	^globals classOrTraitNamed: className.! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: ''!
classNames
	^globals classNames! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: 'MarcusDenker 4/21/2011 21:34'!
classOrTraitNamed: aString
	"aString is either a class or trait name or a class or trait name followed by ' class' or 'classTrait' respectively. Answer the class or metaclass it names."

	^ globals classOrTraitNamed: aString! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: ''!
cleanOutUndeclared 
	Undeclared removeUnreferencedKeys! !

!SmalltalkImage methodsFor: 'cleanup' stamp: ''!
cleanUp	
	"Smalltalk cleanUp"
	"Gently clean up"
	
	^self cleanUp: false! !

!SmalltalkImage methodsFor: 'cleanup' stamp: ''!
cleanUp: aggressive
	"Clean up. When aggressive is true, this will destroy change sets, etc."
	"Smalltalk cleanUp: false"
	"Smalltalk cleanUp: true"

	^self cleanUp: aggressive except: #()! !

!SmalltalkImage methodsFor: 'cleanup' stamp: ''!
cleanUp: aggressive except: exclusions
	"Clean up. When aggressive is true, this will destroy resources, change sets, etc.
	Leave out any classes specifically listed in exclusions."

	"Smalltalk cleanUp: true except: {Point . ChangeSet}"

	^self cleanUp: aggressive except: exclusions confirming: aggressive! !

!SmalltalkImage methodsFor: 'cleanup' stamp: ''!
cleanUp: aggressive except: exclusions confirming: aBool
	"Clean up. When aggressive is true, this will destroy resources, change sets, etc.
	Leave out any classes specifically listed in exclusions."

	"Smalltalk cleanUp: true except: {Point . ChangeSet} 
		- will ask for confirmation"
	"SmalltalkImage current cleanUp: true except: {} confirming: false 
		- will not ask for confirmation"

	| classes |
	aBool ifTrue:[
		"Give the user a chance to bail"
		(self confirm: 'Cleanup will destroy projects, change sets and more.
Are you sure you want to proceed?') ifFalse: [^self]].

	"Find all classes implementing #cleanUp or cleanUp:"
	classes := Smalltalk allClasses select: [:aClass| 
		(aClass class includesSelector: #cleanUp) 
			or: [aClass class includesSelector: #cleanUp:]].

	"Leave out the classes in the exclusion set"
	classes := classes reject: [:aClass| exclusions includes: aClass].

	"Arrange classes in superclass order, superclasses before subclasses.
	This will ensure that specific cleanup (like MethodDictionary compaction)
	will run after generic superclass cleanup (HashedCollection rehashing).
	Otherwise generic superclass cleanup might undo specific one (in this
	case rehashing will undo a good bit of MD compaction)."
	classes := Class superclassOrder: classes.

	"Run the cleanup code"
	classes 
		do:[:aClass| aClass cleanUp: aggressive]
		displayingProgress: [:aClass| 'Cleaning up in ', aClass name].! !

!SmalltalkImage methodsFor: 'special objects' stamp: ''!
clearExternalObjects
	"Clear the array of objects that have been registered for use in non-Smalltalk code."
	"Smalltalk clearExternalObjects"

	ExternalSemaphoreTable clearExternalObjects
! !

!SmalltalkImage methodsFor: 'miscellaneous' stamp: 'IgorStasenko 1/21/2011 18:57'!
closeLog: logger
	"This is a _private_ method,
	Because it really belongs to logging facility,
	we should delegate to it at some point "
	
	logger ifNotNil: [ logger close ]
! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: ''!
closeSourceFiles
	"Shut down the source files if appropriate.  1/29/96 sw: changed so that the closing and nilification only take place if the entry was a FileStream, thus allowing stringified sources to remain in the saved image file"

	1 to: 2 do: [:i |
		((SourceFiles at: i) isKindOf: FileStream)
			ifTrue:
				[(SourceFiles at: i) close.
				SourceFiles at: i put: nil]]! !

!SmalltalkImage methodsFor: 'system attribute' stamp: ''!
commandLine
	"Answer the object to query about command line."
	
	^self! !

!SmalltalkImage methodsFor: 'special objects' stamp: ''!
compactClassesArray
	"Smalltalk compactClassesArray"
	"Return the array of 31 classes whose instances may be
	represented compactly"
	^ self specialObjectsArray at: 29! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: 'SeanDeNigris 7/12/2012 08:44'!
condenseChanges
	"Move all the changes onto a compacted sources file."
	"Smalltalk condenseChanges"
	| f file oldChanges |
	f := FileStream fileNamed: 'ST80.temp'.
	f header; timeStamp.
	'Condensing Changes File...'
		displayProgressFrom: 0
		to: self classNames size + self traitNames size
		during: [:bar | | count | 
			count := 0.
			self
				allClassesAndTraitsDo: [:classOrTrait | 
					bar current: (count := count + 1).
					classOrTrait moveChangesTo: f.
					classOrTrait putClassCommentToCondensedChangesFile: f.
					classOrTrait classSide moveChangesTo: f]].
	self lastQuitLogPosition: f position.
	f trailer; close.
	CompiledMethod cleanUp.
	
	oldChanges := SourceFiles at: 2.
	oldChanges close.
	
	(oldChanges name , '.old') asFileReference delete.
	(file := oldChanges name asFileReference) renameTo: file basename, '.old'.
	f name asFileReference renameTo: file basename.
	
	self setMacFileInfoOn: oldChanges name.
	SourceFiles
		at: 2
		put: (FileStream oldFileNamed: oldChanges name)! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: 'MarcusDenker 7/10/2012 10:38'!
condenseSources	
	"Move all the changes onto a compacted sources file."
	"Smalltalk condenseSources"

	| f dir newVersionString |
	self fixUpProblemsWithAllCategory.
	"The above removes any concrete, spurious '-- all --' categories, which mess up the process."
	dir := FileSystem workingDirectory.
	newVersionString := UIManager default request: 'Please designate the version
for the new source code file...' initialAnswer: Smalltalk  sourceFileVersionString.
	newVersionString ifNil: [^ self].
	newVersionString = self sourceFileVersionString ifTrue:
		[^ self error: 'The new source file must not be the same as the old.'].
	Smalltalk sourceFileVersionString: newVersionString.

	"Write all sources with fileIndex 1"
	f := FileStream newFileNamed: Smalltalk sourcesName.
	f header; timeStamp.
'Condensing Sources File...'
	displayProgressFrom: 0 to: self classNames size + self traitNames size
	during:
		[:bar | | count | count := 0.
		Smalltalk allClassesAndTraitsDo:
			[:classOrTrait | bar current: (count := count + 1).
			classOrTrait fileOutOn: f moveSource: true toFile: 1]].
	f trailer; close.
	
	CompiledMethod cleanUp.
	
	"Make a new empty changes file"
	self closeSourceFiles.
	Smalltalk changesFile renameTo:  Smalltalk changesFile basename, '.old'.
	
	(FileStream newFileNamed: Smalltalk changesName)
		header; timeStamp; close.
	self lastQuitLogPosition: 0.

	self setMacFileInfoOn: Smalltalk image changesName.
	self setMacFileInfoOn: Smalltalk image sourcesName.
	self openSourceFiles.
	self inform: 'Source files have been rewritten!!
Check that all is well,
and then save/quit.'! !

!SmalltalkImage methodsFor: 'PharoCore' stamp: 'PavelKrivanek 2/4/2012 19:56'!
corePackageNames

  ^ #('Announcements-Help' 'Announcements-Tests-Core' 'Announcements-View' 'AST-Core' 'AST-Semantic' 'AST-Tests-Core' 'AST-Tests-Semantic' 'Balloon' 'BalloonTests' 'CollectionsTests' 'Compatibility' 'CompilerTests' 'CompressionTests' 'Deprecated14' 'DialogItemsChooser' 'EmergencyEvaluator' 'FS-AnsiStreams' 'FS-Core' 'FS-Disk' 'FS-Memory' 'FS-Tests-AnsiStreams' 'FS-Tests-Core' 'FS-Tests-Disk' 'FS-Tests-Memory' 'FS-Zip' 'Finder' 'FixUnderscores' 'FreeType' 'FreeTypeTests' 'Gofer-Core' 'Gofer-Tests' 'Graphics-External' 'Graphics-Files' 'Graphics-Fonts' 'Graphics-Resources' 'Graphics-Support' 'Graphics-Tests' 'Graphics-Text' 'HelpSystem-Core' 'HelpSystem-Tests' 'HostMenus' 'KernelTests' 'MenuRegistration' 'Monticello' 'MonticelloConfigurations' 'MonticelloGUI' 'MonticelloMocks' 'Morphic' 'MorphicTests' 'Multilingual-Display' 'Multilingual-Scanning' 'Multilingual-Tests' 'Network-Kernel' 'Network-MIME' 'Network-MailSending' 'Network-Protocols' 'Network-RFC822' 'Network-RemoteDirectory' 'Network-URI' 'Network-UUID' 'Network-Url' 'NetworkTests' 'PackageInfo' 'Polymorph-EventEnhancements' 'Polymorph-Geometry' 'Polymorph-TaskbarIcons' 'Polymorph-Tools-Diff' 'Polymorph-Widgets' 'RecentSubmissions'  'Refactoring-Changes' 'Refactoring-Core' 'Refactoring-Critics' 'Refactoring-Environment' 'Refactoring-Extensions-Environment' 'Refactoring-Tests-Changes' 'Refactoring-Tests-Core' 'Refactoring-Tests-Critics' 'Refactoring-Tests-Environment' 'Regex-Core' 'Regex-Help' 'Regex-Tests-Core' 'Ring-Core-Containers' 'Ring-Core-Kernel' 'Ring-Tests-Containers' 'Ring-Tests-Kernel' 'SUnit-Core' 'SUnit-Help' 'SUnit-Tests' 'SUnit-UI' 'ScriptLoader14' 'Settings-Compiler' 'Settings-Display' 'Settings-FreeType' 'Settings-Graphics' 'Settings-Kernel' 'Settings-Monticello' 'Settings-Network' 'Settings-Polymorph' 'Settings-System' 'Settings-Tools' 'Shout' 'ShoutTests' 'StratifiedProxy' 'SystemAnnouncements' 'System-Applications' 'System-Download' 'System-FilePackage' 'System-Hashing' 'System-History' 'System-Installers' 'System-Serial Port' 'System-Settings' 'System-Sound' 'System-Text' 'System-Tools' 'Tests' 'Tools' 'ToolsTest' 'Transcript' 'TrueType' 'UI-Basic' 'Zinc-HTTP' 'Zinc-Patch-HTTPSocket' 'Zinc-Tests' ) ! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
createStackOverflow
	"For testing the low space handler..."
	"Smalltalk installLowSpaceWatcher; createStackOverflow"

	self createStackOverflow.  "infinite recursion"! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: ''!
datedVersion
	"Answer the version of this release."

	^SystemVersion current datedVersion! !

!SmalltalkImage methodsFor: 'modules' stamp: 'IgorStasenko 3/22/2011 14:07'!
disableModuleLoading
	"Primitive. Disable a new module loading mechanism for the rest of current session.
	This operation is not reversable.
	Any subsequent attempts to load either external or internal module(s) will fail"

	<primitive: 'primitiveDisableModuleLoading' module: ''>
	! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
do: aBlock
	"Obsoleted."
	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals do: aBlock
	
! !

!SmalltalkImage methodsFor: 'endian' stamp: ''!
endianness

	EndianCache ifNil: [EndianCache := self calcEndianness].
	^ EndianCache.
! !

!SmalltalkImage methodsFor: 'accessing' stamp: ''!
environment
	"For conversion from Smalltalk to SystemDictionary"

	^globals! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: 'EstebanLorenzano 7/27/2012 14:02'!
event: anEvent
	"Hook for SystemChangeNotifier"
	ChangesLog default logEvent: anEvent.! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'CamilloBruni 2/13/2012 23:35'!
executeDeferredStartupActions: resuming
	"Execute the deferred actions which where added during startup, resuming is true if the iage was started fresh, false if we just saved an image"

	| errors |
	
	deferredStartupActions ifNil: [^ #()].
	
	errors := OrderedCollection new.	
		
	deferredStartupActions do: [:each |
		self 
			logStartUpErrorDuring: [ each cull: resuming ] 
			into: errors
			tryDebugger: self isInteractive].
		
	^ errors! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'CamilloBruni 2/21/2012 01:05'!
exit: exitStatus
	"Primitive. Exit to another operating system on the host machine, if one
	exists. All state changes in the object space since the last snapshot are lost.
	Essential. See Object documentation whatIsAPrimitive.
	
	Possible values for exitStatus:
	0:   success
	> 1: error"

	<primitive: 113>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'CamilloBruni 2/21/2012 01:12'!
exitFailure
	"Quit the VM with a failing signal.
	Will lose all current changes."
	self exit: 1! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'CamilloBruni 2/21/2012 01:12'!
exitSucess
	"Quit the VM with a success signal.
	Will lose all current changes."
	self exit: 0! !

!SmalltalkImage methodsFor: 'miscellaneous' stamp: ''!
exitToDebugger
	"Primitive. Enter the machine language debugger, if one exists. Essential.
	See Object documentation whatIsAPrimitive."

	<primitive: 114>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'special objects' stamp: ''!
externalObjects
	"Return an array of objects that have been registered for use in non-Smalltalk code. Smalltalk objects should be referrenced by external code only via indirection through this array, thus allowing the objects to move during compaction. This array can be cleared when the VM re-starts, since variables in external code do not survive snapshots. Note that external code should not attempt to access a Smalltalk object, even via this mechanism, while garbage collection is in progress."
	"Smalltalk externalObjects"

	^ ExternalSemaphoreTable externalObjects
! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'IgorStasenko 8/17/2011 17:00'!
extractMinusParameters
	"Returns a dictionary matching parameters beginning by a minus character and the list of the next parameters.
	Here is an example: 
		I suppose I run pharo with: `pharo myImage.image -minus1 'Hello' -minus2 -minus3 'Hello' 'World'`.
		This message will return a citionary with: 
			minus1 --> #(Hello)
			minus2 --> #()
			minus3 --> #('Hello' 'World')"
			
	| args dict str previous previousWithMinus index |
	
	dict := Dictionary new.
	args := OrderedCollection new.
	previous := previousWithMinus := ''.

	str := self vm getSystemAttribute: (index := 2).
	[ str isNil ] whileFalse: [ args add: str. str := self vm getSystemAttribute: (index := index + 1) ].

	args ifEmpty: [ ^ dict ].

	previous := previousWithMinus := args detect: [ :each | each first = $- ].
	[ (args first = previous) & (args isEmpty not) ] whileFalse: [ args removeFirst ].
	
	args do: [ :each |
		(each first = $-)
			ifTrue: [ 
				(previous first = $-) ifTrue: [ dict at: previous allButFirst put: #() ].
				previousWithMinus := each ]
			ifFalse: [ dict at: previousWithMinus allButFirst put: ((dict at: previousWithMinus allButFirst ifAbsent: [ #() ]), (Array with: each)) ].
				
		previous := each.
	].

	(previous first = $-) ifTrue: [ dict at: previous allButFirst put: #() ].

	^ dict! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'IgorStasenko 8/17/2011 17:00'!
extractParameters

	| pName value index dict |
	dict := Dictionary new.
	index := 3. "Muss bei 3 starten, da 2 documentName ist"
	[pName := self vm getSystemAttribute: index.
	pName isEmptyOrNil] whileFalse:[
		index := index + 1.
		value := self vm getSystemAttribute: index.
		value ifNil: [value := ''].
 		dict at: pName asUppercase put: value.
		index := index + 1].
	^dict! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 02:57'!
fileForChangesNamed: aName
	^ (self imageDirectory resolve: aName) , self changesSuffix.! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 02:57'!
fileForImageNamed: aString
	^ (self imageDirectory resolve: aString), self imageSuffix! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: 'MarcusDenker 7/12/2012 18:04'!
fixObsoleteReferences
	"SmalltalkImage current fixObsoleteReferences.
	SystemNavigation new obsoleteBehaviors size > 0
		ifTrue: [ SystemNavigation new obsoleteBehaviors inspect.
			self error:'Still have obsolete behaviors. See inspector']"

	| informee obsoleteBindings obsName realName realClass |
	Smalltalk
		garbageCollect;
		garbageCollect.
	CompiledMethod allInstances
		do: [ :method | 
			obsoleteBindings := method literals
				select: [ :literal | literal isVariableBinding 
									and: [ literal value isBehavior  and: [ literal value isObsolete ]]].
			obsoleteBindings
				do: [ :binding | 
					obsName := binding value name.
					self trace: 'Binding: ';
						traceCr: obsName.
					realName := obsName copyReplaceAll: 'AnObsolete' with: ''.
					realClass := Smalltalk globals at: realName asSymbol ifAbsent: [ UndefinedObject ].
					binding key: binding key value: realClass ] ].
	Behavior flushObsoleteSubclasses.
	Smalltalk
		garbageCollect;
		garbageCollect! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: 'StephaneDucasse 11/7/2011 22:43'!
fixUpProblemsWithAllCategory
	"Moves all methods that are in formally classified a category named '-- all --' into the default 'as yet unclassified' category"

	"Smalltalk fixUpProblemsWithAllCategory"

	| methodCount classCount |
	self flag: #ShouldBeMovedInClassOrganization.
	methodCount := 0.
	classCount := 0.
	self systemNavigation allBehaviorsDo:
		[:aClass | | org any | org := aClass organization.
			(org categories includes: #'-- all --') ifTrue:
				[any := false.
				aClass selectorsDo:
					[:aSelector | | aCategory |
						aCategory := org categoryOfElement: aSelector.
						aCategory = #'-- all --' ifTrue:
							[org classify: aSelector under: ClassOrganizer default suppressIfDefault: false.
							self crTrace: aClass name, ' >> ', aSelector.
							methodCount := methodCount + 1.
							any := true]].
			any ifTrue: [classCount := classCount + 1].
			org removeEmptyCategories]].
	self crTrace: methodCount printString, ' methods in ', classCount printString, ' classes moved from "-- all --" to "as yet unclassified"'
! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
flushClassNameCache
	"Smalltalk flushClassNameCache"
	"Force recomputation of the cached list of class names."

	globals flushClassNameCache.! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: 'GuillermoPolito 7/2/2012 13:08'!
forceChangesToDisk
	"Ensure that the changes file has been fully written to disk by closing and re-opening it. This makes the system more robust in the face of a power failure or hard-reboot."

	| changesFile |
	changesFile := SourceFiles at: 2.
	(changesFile isKindOf: FileStream) ifTrue: [
		changesFile flush.
		changesFile close.
		changesFile open: changesFile name forWrite: true.
		changesFile setToEnd.
	].
! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
forceTenure
	"Primitive. Tell the GC logic to force a tenure on the next increment GC."
	<primitive: 'primitiveForceTenure'>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'modules' stamp: ''!
forgetModule: aString
	"Primitive. If the module named aString is loaded, unloaded. If not, and it is marked an unloadable, unmark it so the VM will try to load it again next time. See comment for #unloadModule:."
	<primitive: 571>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 01:50'!
fullNameForChangesNamed: aName
	^ (self fileForChangesNamed: aName) fullName! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 01:40'!
fullNameForImageNamed: aString
	^ (self fileForImageNamed: aString) fullName! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
garbageCollect
	"Primitive. Reclaims all garbage and answers the number of bytes of available space."
	Object flushDependents.
	Object flushEvents.
	^self primitiveGarbageCollect! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
garbageCollectMost
	"Primitive. Reclaims recently created garbage (which is usually most of it) fairly quickly and answers the number of bytes of available space."

	<primitive: 131>
	^ self primBytesLeft! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'StephaneDucasse 6/11/2012 18:12'!
getFileNameFromUser

	| newName |
	newName := UIManager default
		request: 'New File Name?' translated
		initialAnswer: (self imageFile basename copyUpToLast:  Path extensionDelimiter).
		
	newName isEmptyOrNil 
		ifTrue: [^nil].
	
	((self fileForImageNamed: newName) exists 
	or: [ (self fileForChangesNamed: newName) exists ]) 
		ifTrue: [ 
			(self confirm: ('{1} already exists. Overwrite?' translated format: {newName})) 
				ifFalse: [ ^ nil ]].
	^newName
! !

!SmalltalkImage methodsFor: 'accessing' stamp: ''!
globals
	"Answer the global SystemDictionary"
	^globals! !

!SmalltalkImage methodsFor: 'private' stamp: ''!
globals: aSystemDictionary
	"Sets the system-wide globals"
	
	globals ifNotNil: [self error: 'Cannot overwrite existing globals'].
	globals := aSystemDictionary! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'CamilloBruni 2/21/2012 01:11'!
handleStartupErrors: startupErrors
	"Handle the errors produced during startup.
	
	Resume to open a non-interactive debugger on the recorded errors"
	self logError: 'Got startup errors: ' inContext: nil.
	
	startupErrors do: [ :error|
		self logError: '    ', error description inContext: nil ].
	
	self isInteractive 
		ifFalse: [ self exitFailure ]
		ifTrue: [  |answer|
			answer := (UIManager default 
				confirm: 'Got startup errors, proceed to open debuggers'
				trueChoice: 'Debug'
				falseChoice: 'Cancel').
			answer == true
				ifTrue: [ startupErrors do: #debug ]]! !

!SmalltalkImage methodsFor: 'to clean later' stamp: 'MarcusDenker 5/3/2011 19:30'!
hasBindingThatBeginsWith: aString
	^globals hasBindingThatBeginsWith: aString! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: ''!
hasClassNamed: aString
	"Answer whether there is a class of the given name, but don't intern aString if it's not alrady interned."

	^ globals hasClassNamed: aString! !

!SmalltalkImage methodsFor: 'special objects' stamp: ''!
hasSpecialSelector: aLiteral ifTrueSetByte: aBlock
	"Some basic messages such as #+ have their own bytecode for speed.
	To gain some space, the literal array of compiled methods does not contain such message selectors.
	This implies that the cross referencer and other tools have to check if a selector is of the optimized one 
	or not by calling this method. This is an optimization when machines were slow and small and it 
	may be removed in the future to gain simplicity."
	
	1 to: self specialSelectorSize do:
		[:index | 
		(self specialSelectorAt: index) == aLiteral
			ifTrue: [aBlock value: index + 16rAF. ^true]].
	^false! !

!SmalltalkImage methodsFor: 'image' stamp: ''!
image
	"Answer the object to query about the current object memory and execution environment."
	
	^self! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 7/10/2012 21:50'!
imageDirectory
	"Answer the directory containing the current image."

	^ FileLocator imageDirectory
! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 7/10/2012 21:49'!
imageFile
	"Answer the file for the current image."
	"SmalltalkImage current imageName"

	^ FileLocator image! !

!SmalltalkImage methodsFor: 'image' stamp: ''!
imageFormatVersion
	"Answer an integer identifying the type of image. The image version number may
	identify the format of the image (e.g. 32 or 64-bit word size) or specific requirements
	of the image (e.g. block closure support required). This invokes an optional primitive
	that may not be available on all virtual machines."

	"Smalltalk image imageFormatVersion"

	<primitive: 'primitiveImageFormatVersion'>
	self notify: 'This virtual machine does not support the optional primitive #primitiveImageFormatVersion' translated.
	^''! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: ''!
imageName
	"Answer the full path name for the current image."
	"SmalltalkImage current imageName"

	| str |
	str := self primImageName.
	^ (FilePath pathName: str isEncoded: true) asSqueakPathName.
! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: ''!
imageName: newName
	"Set the the full path name for the current image.  All further snapshots will use this."

	| encoded |
	encoded := (FilePath pathName: newName isEncoded: false) asVmPathName.
	self primImageName: encoded.
! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/13/2012 19:24'!
imagePath
	"Answer the full path name for the current image."
	"SmalltalkImage current imageName"

	| str |
	str := self primImagePath.
	^ (FilePath pathName: str isEncoded: true) asSqueakPathName.
! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/13/2012 19:24'!
imagePath: newPath
	"Set the the full path name for the current image.  All further snapshots will use this."

	| encoded |
	encoded := (FilePath pathName: newPath isEncoded: false) asVmPathName.
	self primImagePath: encoded.
! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 01:39'!
imageSuffix
	^ 'image'! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
includes: element
	"Obsoleted."

	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals includes: element! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
includesKey: key
	"Answer whether the receiver has a key equal to the argument, key."

	^globals includesKey: key! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: 'BernardoContreras 10/27/2011 19:18'!
informSpaceLeftAfterGarbageCollection
	"Do a garbage collection, and report results to the user."

	"SmalltalkImage current informSpaceLeftAfterGarbageCollection"

	self inform: self spaceLeftAfterGarbageCollection! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
installLowSpaceWatcher
	"Start a process to watch for low-space conditions."
	"Smalltalk installLowSpaceWatcher"

	self primSignalAtBytesLeft: 0.  "disable low-space interrupts"
	LowSpaceProcess == nil ifFalse: [LowSpaceProcess terminate].
	LowSpaceProcess := [self lowSpaceWatcher] newProcess.
	LowSpaceProcess priority: Processor lowIOPriority.
	LowSpaceProcess resume.

! !

!SmalltalkImage methodsFor: 'endian' stamp: ''!
isBigEndian
	^self endianness == #big! !

!SmalltalkImage methodsFor: 'testing' stamp: 'CamilloBruni 2/9/2012 00:31'!
isHeadless
	"Check if vm were run with headless parameter.
	Different VMs for different platform have different multiple way(s) to indicate that"
	
	-1000 to: 1000 do: [ :n | 
		(#('display=none' '-headless' '-vm-display-null' '-nodisplay') includes: (self vm getSystemAttribute: n)) ifTrue: [ ^ true ] ].
	^ false
	
	
! !

!SmalltalkImage methodsFor: 'testing' stamp: 'CamilloBruni 2/13/2012 18:55'!
isInteractive
	"Check if vm were run with headless parameter.
	Different VMs for different platform have different multiple way(s) to indicate that"
	
	"non-headless mode is always interactive"
	self isHeadless ifFalse: [ ^ true ].
	
	-1000 to: 1000 do: [ :n | 
		(#('-interactive') includes: (self vm getSystemAttribute: n)) 
			ifTrue: [ ^ true ]].
	
	^ false! !

!SmalltalkImage methodsFor: 'endian' stamp: ''!
isLittleEndian
	^self endianness == #little! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
isRoot: oop
	"Primitive. Answer whether the object is currently a root for youngSpace."
	<primitive: 'primitiveIsRoot'>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'printing' stamp: ''!
isSelfEvaluating
	self == Smalltalk ifTrue: [^true].
	^super isSelfEvaluating! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
isYoung: oop
	"Primitive. Answer whether the object currently resides in youngSpace."
	<primitive: 'primitiveIsYoung'>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'PharoKernel' stamp: 'PavelKrivanek 6/21/2012 11:53'!
kernelPackageNames

  ^ #('Announcements-Core' 'Announcements-Core' 'Collections-Abstract' 'Collections-Arithmetic' 'Collections-Arrayed'
 'Collections-Atomic' 'Collections-Native' 'Collections-Sequenceable' 'Collections-Stack' 'Collections-Streams' 
'Collections-Strings' 'Collections-Support' 'Collections-Text' 'Collections-Traits' 'Collections-Unordered' 
'Collections-Weak' 'Compiler' 'Compression' 'Files' 'Graphics-Display Objects' 'Graphics-Primitives' 
'Graphics-Transformations' 'Kernel' 'Multilingual-Encodings' 'Multilingual-ImmPlugin' 'Multilingual-Languages' 
'Multilingual-TextConversion'  'System-Change Notification' 'System-Changes' 'System-CommandLine' 'System-Clipboard' 
'System-FileRegistry' 'System-Finalization' 'System-Localization' 'System-Object Events' 'System-Platforms' 
'System-Support' 'Text-Core' 'Traits' 'UIManager' 'Transcript' 'FileSystem-AnsiStreams' 'FileSystem-Core' 
'FileSystem-Disk' 'FileSystem-Memory' 'Fuel' 'FuelMetalevel' 'FuelPackageLoader' 'CodeImport' 'FuelFileSystem' 
'FuelCommandLineHandler' 'Regex-Core' 'SystemProgress' 'System-Announcements')! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
keyAtIdentityValue: anObject ifAbsent: aBlock
	"Obsoleted."

	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals keyAtIdentityValue: anObject ifAbsent: aBlock! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
keyAtValue: anObject
	"Obsoleted."
	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals keyAtValue: anObject! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
keys
	"Obsoleted."

	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals keys! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
keysAndValuesDo: aBlock
	"Obsoleted."
	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals keysAndValuesDo: aBlock
! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: ''!
lastQuitLogPosition
	^ LastQuitLogPosition! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: ''!
lastQuitLogPosition: aNumber
	"should be only use to ensure the transition from SystemDictionary to SmalltalkImage, then  	be removed"
		
	LastQuitLogPosition := aNumber! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: ''!
lastUpdateString
	"SmalltalkImage current lastUpdateString"
	^'Latest update: #' translated, SystemVersion current highestUpdate printString! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: 'MarcusDenker 5/10/2012 12:57'!
licenseString
	^ 'LICENSE
	
Licensed under the MIT License with parts under the Apache License.

Copyright (c) Pharo Project, and Contributors Copyright (c) 1996-2008 Viewpoints
Research Institute, and Contributors Copyright (c) 1996 Apple Computer, Inc.

Permission is hereby granted, free of charge, to any person obtaining a copy of this
software and associated documentation files (the "Software"), to deal in the Software
without restriction, including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons
to whom the Software is furnished to do so, subject to the following conditions: The
above copyright notice and this permission notice shall be included in all copies or
substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

You may obtain a copy of the Apache License at
http://www.apache.org/licenses/LICENSE-2.0

Pharo uses icons by Mark James (http://www.famfamfam.com) under 
Creative Commons Attribution 2.5 License.
'! !

!SmalltalkImage methodsFor: 'modules' stamp: ''!
listBuiltinModule: index
	"Return the name of the n-th builtin module.
	This list is not sorted!!"
	<primitive: 572>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'modules' stamp: ''!
listBuiltinModules
	"SmalltalkImage current listBuiltinModules"
	"Return a list of all builtin modules (e.g., plugins). Builtin plugins are those that are 	compiled with the VM directly, as opposed to plugins residing in an external shared library. 	The list will include all builtin plugins regardless of whether they are currently loaded 
	or not. Note that the list returned is not sorted!!"

	| modules index name |
	modules := Array new writeStream.
	index := 1.
	[true] whileTrue:[
		name := self listBuiltinModule: index.
		name ifNil:[^modules contents].
		modules nextPut: name.
		index := index + 1.
	].! !

!SmalltalkImage methodsFor: 'modules' stamp: ''!
listLoadedModule: index
	"Return the name of the n-th loaded module.
	This list is not sorted!!"
	<primitive: 573>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'modules' stamp: ''!
listLoadedModules
	"SmalltalkImage current listLoadedModules"
	"Return a list of all currently loaded modules (e.g., plugins). Loaded modules are those that currently in use (e.g., active). The list returned will contain all currently active modules regardless of whether they're builtin (that is compiled with the VM) or external (e.g., residing in some external shared library). Note that the returned list is not sorted!!"
	| modules index name |
	modules := Array new writeStream.
	index := 1.
	[true] whileTrue:[
		name := self listLoadedModule: index.
		name ifNil:[^modules contents].
		modules nextPut: name.
		index := index + 1.
	].! !

!SmalltalkImage methodsFor: 'modules' stamp: 'IgorStasenko 3/22/2011 14:05'!
loadModule: aModuleName
	"Primitive. Attempt to load a module of the given name.
	Fail if module cannot be found, or cannot be loaded,
	or failed to initialize"

	<primitive: 'primLoadModule'  module: '' error: errorCode >
	^ self primitiveFailed! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: 'GuillermoPolito 7/2/2012 12:29'!
logChange: aStringOrText 
	"Write the argument, aString, onto the changes file."
	ChangesLog default logChange: aStringOrText! !

!SmalltalkImage methodsFor: 'miscellaneous' stamp: 'IgorStasenko 1/21/2011 19:12'!
logDuring: aMonadicBlock
	" for safe use, if stream is a file, it needs to be closed after use " 
	| logStream |	

	[
		logStream := self openLog.
		aMonadicBlock value: logStream.
	] ensure: [ self closeLog: logStream ].

! !

!SmalltalkImage methodsFor: 'miscellaneous' stamp: 'CamilloBruni 2/13/2012 22:29'!
logError: errMsg inContext: aContext

	" we should think about integrating a toothpick here someday"
	self logStdErrorDuring: [ :stderr|
		"install the line end conversion and force initialize the converter"
		stderr
			nextPutAll: errMsg; cr;
			"reset the color"
			nextPut: Character escape; nextPutAll: '[0m'.
		aContext shortDebugStackOn: stderr.].
				
	self logDuring: [:logger | 
		logger	
			nextPutAll: 'THERE_BE_DRAGONS_HERE'; cr; 
		  	nextPutAll: errMsg; cr.
			"just print the error message if no context is given"
			aContext ifNotNil: [
				aContext errorReportOn: logger ].
		" write some type of separator"
		logger nextPutAll: (String new: 79 withAll: $- ); cr; cr]! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'CamilloBruni 2/13/2012 23:45'!
logStartUpErrorDuring: aBlock into: aCollection tryDebugger: tryDebugger
	"handle errors thrown by the given block and log a freezed version of them in aCollection"
	aBlock on: Error do: [ :err|
		self logStartupError: err into: aCollection tryDebugger: tryDebugger]! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'CamilloBruni 2/13/2012 23:47'!
logStartupError: anError into: aCollection tryDebugger: tryDebugger
	"try to debug the given error or add a freezed version to aCollection"
	tryDebugger ifTrue: [ 
		[ ^ anError debug ] 
			on: Error do: [ 
				"an error occured during error handling... treat the error as a normal startup error " ]].

	self 
		logError: '==== Startup Error: ', anError description 
		inContext: anError signalerContext.
	"freeze the error"
	aCollection add: anError freeze.! !

!SmalltalkImage methodsFor: 'miscellaneous' stamp: 'CamilloBruni 2/13/2012 19:21'!
logStdErrorDuring: aBlock
	| stderr |
	[
		"install the line end conversion and force initialize the converter"
		stderr := FileStream  stderr
					wantsLineEndConversion: true;
					converter;
					yourself.
		
		"log in red"
		stderr nextPut: Character escape; nextPutAll: '[31m'.
		"rund the loggin block"
		aBlock value: stderr.
		"reset the coloring"
		stderr nextPut: Character escape; nextPutAll: '[0m'.
	] on: Error do: [ :e| "we don't care if the logging to stdout fails..." ].! !

!SmalltalkImage methodsFor: 'memory space' stamp: 'StephaneDucasse 1/30/2012 12:14'!
lowSpaceThreshold 
	"Return the low space threshold. When the amount of free memory (after garbage collection) falls below this limit, the system is in serious danger of completely exhausting memory and crashing. This limit should be made high enough to allow the user open a debugger to diagnose a problem or to save the image."

	^ 400000  "Enough for JIT compiler"! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
lowSpaceWatcher
	"Wait until the low space semaphore is signalled, then take appropriate
	actions. "
	| free preemptedProcess |
	self garbageCollectMost <= self lowSpaceThreshold
		ifTrue: [self garbageCollect <= self lowSpaceThreshold
				ifTrue: ["free space must be above threshold before
					starting low space watcher"
					^ Beeper beep]].
	self specialObjectsArray at: 23 put: nil.
	"process causing low space will be saved here"
	LowSpaceSemaphore := Semaphore new.
	self primLowSpaceSemaphore: LowSpaceSemaphore.
	self primSignalAtBytesLeft: self lowSpaceThreshold.
	"enable low space interrupts"
	LowSpaceSemaphore wait.
	"wait for a low space condition..."
	self primSignalAtBytesLeft: 0.
	"disable low space interrupts"
	self primLowSpaceSemaphore: nil.
	LowSpaceProcess := nil.
	"The process that was active at the time of the low space interrupt."
	preemptedProcess := Smalltalk specialObjectsArray at: 23.
	Smalltalk specialObjectsArray at: 23 put: nil.
	"Note: user now unprotected until the low space watcher is re-installed"
	self memoryHogs isEmpty
		ifFalse: [free := self bytesLeft.
			self memoryHogs
				do: [:hog | hog freeSomeSpace].
			self bytesLeft > free
				ifTrue: [^ self installLowSpaceWatcher]].
	UIManager default lowSpaceWatcherDefaultAction: preemptedProcess! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
lowSpaceWatcherProcess
	^LowSpaceProcess! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'IgorStasenko 8/17/2011 17:04'!
maxFilenameLength

	^self vm maxFilenameLength! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
memoryHogs
	"Answer the list of objects to notify with #freeSomeSpace if memory gets full."

	^ MemoryHogs ifNil: [MemoryHogs := OrderedCollection new]! !

!SmalltalkImage methodsFor: 'miscellaneous' stamp: 'MarcusDenker 4/26/2012 15:36'!
metacelloPlatformAttributes
    "Returns the tags for the conditional platform loading in Metacello. Pay attention the order is important: from most  to least general." 
    "For release integrators, we should not have #'pharo1.3x' **and** #'pharo1.4.x'"
   ^ #(#squeakCommon #pharo #'pharo2.x' #'pharo2.0.x')! !

!SmalltalkImage methodsFor: 'special objects' stamp: 'GuillermoPolito 4/11/2012 15:40'!
newCompactClassesArray
	"Smalltalk compactClassesArray"
	"Return the array of 31 classes whose instances may be
	represented compactly"
	^ Array new: 31 streamContents: [ :str |
		(self compactClassesArray) do: [ :c |
				(self globals includesKey: c name)
					ifTrue: [ str nextPut: (self globals at: c name) ]
					ifFalse: [ str nextPut: nil ]
			]
		]! !

!SmalltalkImage methodsFor: 'special objects' stamp: 'GuillermoPolito 5/8/2012 14:57'!
newSpecialObjectsArray
	"Smalltalk recreateSpecialObjectsArray"
	
	"To external package developers:
	**** DO NOT OVERRIDE THIS METHOD.  *****
	If you are writing a plugin and need additional special object(s) for your own use, 
	use addGCRoot() function and use own, separate special objects registry "
	
	"The Special Objects Array is an array of objects used by the Squeak virtual machine.
	 Its contents are critical and accesses to it by the VM are unchecked, so don't even
	 think of playing here unless you know what you are doing."
	
	| newArray |
	newArray := Array new: 56.
	"Nil false and true get used throughout the interpreter"
	newArray at: 1 put: nil.
	newArray at: 2 put: false.
	newArray at: 3 put: true.
	"This association holds the active process (a ProcessScheduler)"
	newArray at: 4 put: (self globals associationAt: #Processor).
	"Numerous classes below used for type checking and instantiation"
	newArray at: 5 put: Bitmap.
	newArray at: 6 put: SmallInteger.
	newArray at: 7 put: ByteString.
	newArray at: 8 put: Array.
	newArray at: 9 put: Smalltalk.
	newArray at: 10 put: Float.
	newArray at: 11 put: MethodContext.
	newArray at: 12 put: nil. "Was BlockContext"
	newArray at: 13 put: Point.
	newArray at: 14 put: LargePositiveInteger.
	newArray at: 15 put: Display.
	newArray at: 16 put: Message.
	newArray at: 17 put: CompiledMethod.
	newArray at: 18 put: (self specialObjectsArray ifNil: [ Semaphore new ] ifNotNil: [ self specialObjectsArray at: 18 ]). "(low space Semaphore)"
	newArray at: 19 put: Semaphore.
	newArray at: 20 put: Character.
	newArray at: 21 put: #doesNotUnderstand:.
	newArray at: 22 put: #cannotReturn:.
	newArray at: 23 put: nil. "This is the process signalling low space."
	"An array of the 32 selectors that are compiled as special bytecodes,
	 paired alternately with the number of arguments each takes."
	newArray at: 24 put: #(	#+ 1 #- 1 #< 1 #> 1 #<= 1 #>= 1 #= 1 #~= 1
							#* 1 #/ 1 #\\ 1 #@ 1 #bitShift: 1 #// 1 #bitAnd: 1 #bitOr: 1
							#at: 1 #at:put: 2 #size 0 #next 0 #nextPut: 1 #atEnd 0 #== 1 nil 0
							nil 1 #value 0 #value: 1 #do: 1 #new 0 #new: 1 #x 0 #y 0 ).
	"An array of the 255 Characters in ascii order.
	 Cog inlines table into machine code at: prim so do not regenerate it."
	newArray at: 25 put: (self primitiveGetSpecialObjectsArray at: 25).
	newArray at: 26 put: #mustBeBoolean.
	newArray at: 27 put: ByteArray.
	newArray at: 28 put: Process.
	"An array of up to 31 classes whose instances will have compact headers"
	newArray at: 29 put: self newCompactClassesArray.
	newArray at: 30 put: (self specialObjectsArray ifNil: [ Semaphore new ] ifNotNil: [ self specialObjectsArray at: 30 ]). 	"(delay Semaphore)"
	newArray at: 31 put: (self specialObjectsArray ifNil: [ Semaphore new ] ifNotNil: [ self specialObjectsArray at: 31 ]). 	"(user interrupt Semaphore)"
	"Prototype instances that can be copied for fast initialization"
	newArray at: 32 put: (Float new: 2).
	newArray at: 33 put: (LargePositiveInteger new: 4).
	newArray at: 34 put: 0@0.
	newArray at: 35 put: #cannotInterpret:.
	"Note: This must be fixed once we start using context prototypes (yeah, right)"
	"(MethodContext new: CompiledMethod fullFrameSize)."
	newArray at: 36 put: (self primitiveGetSpecialObjectsArray at: 36). "Is the prototype MethodContext (unused by the VM)"
	newArray at: 37 put: BlockClosure.
	"(BlockContext new: CompiledMethod fullFrameSize)."
	newArray at: 38 put: (self primitiveGetSpecialObjectsArray at: 38). "Is the prototype BlockContext (unused by the VM)"
	"array of objects referred to by external code"
	newArray at: 39 put: (self primitiveGetSpecialObjectsArray at: 39).	"preserve external semaphores"
	newArray at: 40 put: nil. "Reserved for Mutex in Cog VMs"
	newArray at: 41 put: nil. "Reserved for a LinkedList instance for overlapped calls in CogMT"
	"finalization Semaphore"
	newArray at: 42 put: (self specialObjectsArray ifNil: [ Semaphore new ] ifNotNil: [ (self specialObjectsArray at: 42) ifNil: [ Semaphore new ] ]).
	newArray at: 43 put: LargeNegativeInteger.
	"External objects for callout.
	 Note: Written so that one can actually completely remove the FFI."
	newArray at: 44 put: (self at: #ExternalAddress ifAbsent: []).
	newArray at: 45 put: (self at: #ExternalStructure ifAbsent: []).
	newArray at: 46 put: (self at: #ExternalData ifAbsent: []).
	newArray at: 47 put: (self at: #ExternalFunction ifAbsent: []).
	newArray at: 48 put: (self at: #ExternalLibrary ifAbsent: []).
	newArray at: 49 put: #aboutToReturn:through:.
	newArray at: 50 put: #run:with:in:.
	"51 reserved for immutability message"
	"newArray at: 51 put: #attemptToAssign:withIndex:."
	newArray at: 52 put: #(nil "nil => generic error" #'bad receiver'
							#'bad argument' #'bad index'
							#'bad number of arguments'
							#'inappropriate operation'  #'unsupported operation'
							#'no modification' #'insufficient object memory'
							#'insufficient C memory' #'not found' #'bad method'
							#'internal error in named primitive machinery'
							#'object may move').
	"53 to 55 are for Alien"
	newArray at: 53 put: (self at: #Alien ifAbsent: []).
	newArray at: 54 put: #invokeCallback:stack:registers:jmpbuf:.
	newArray at: 55 put: (self at: #UnsafeAlien ifAbsent: []).

	"Weak reference finalization"
	newArray at: 56 put: (self at: #WeakFinalizationList ifAbsent: []).

	^newArray! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
okayToProceedEvenIfSpaceIsLow
	"Return true if either there is enough memory to do so safely or if the user gives permission after being given fair warning."

	self garbageCollectMost > self lowSpaceThreshold ifTrue: [^ true].  "quick"
	self garbageCollect > self lowSpaceThreshold ifTrue: [^ true].  "work harder"

	^ self confirm:
'WARNING: There is not enough space to start the low space watcher.
If you proceed, you will not be warned again, and the system may
run out of memory and crash. If you do proceed, you can start the
low space notifier when more space becomes available simply by
opening and then closing a debugger (e.g., by hitting Cmd-period.)
Do you want to proceed?'
! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: 'SeanDeNigris 7/12/2012 08:44'!
openChanges: changesPath forImage: imageName
"find the changes file by looking in
a) the directory derived from the image name
b) the DefaultDirectory (which will normally be the directory derived from the image name or the SecurityManager's choice)
If an old file is not found in either place, check for a read-only file in the same places. If that fails, return nil"
	| changes fd changesFile changesName |
	
	changesFile := changesPath asFileReference.
	changesName := changesFile basename.
	
	"look for the changes file or an alias to it in the image directory"
	changesFile exists 
		ifTrue: [ changesFile writeStream ifNotNilDo: [ :stream| ^ stream ]].

	"look for the changes in the default directory"
	(changesFile := FileSystem workingDirectory / changesName) exists 
		ifTrue: [ changesFile writeStream ifNotNilDo: [ :stream| ^ stream ]].

	"look for read-only changes in the image directory"
	(changesFile := changesPath asFileReference) exists 
		ifTrue: [ changesFile readStream ifNotNilDo: [ :stream| ^ stream ]].

	"look for read-only changes in the default directory"
	(changesFile := FileSystem workingDirectory / changesName) exists 
		ifTrue: [ changesFile readStream ifNotNilDo: [ :stream| ^ stream ]].
		
	"failed to open a changes file"
	^ nil
! !

!SmalltalkImage methodsFor: 'miscellaneous' stamp: 'IgorStasenko 4/26/2011 16:58'!
openLog
	"This is a _private_ method,
	Because it really belongs to logging facility,
	we should delegate to it at some point "
	
	^ ( FileStream fileNamed: Smalltalk tools debugger logFileName ) 
		wantsLineEndConversion: true;
		setToEnd;
		yourself! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: 'CamilloBruni 5/13/2012 19:29'!
openSourceFiles

	self imagePath = LastImagePath 
		ifFalse: ["Reset the author full name to blank when the image gets moved"
			LastImagePath := self imagePath.
			Author fullName: ''].
	self checkAndOpenSourcesAndChanges.
	SourceFileArray install! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: 'SeanDeNigris 7/12/2012 08:44'!
openSources: fullSourcesName forImage: imagePath 
"We first do a check to see if a compressed version ofthe sources file is present.
Open the .sources file read-only after searching in:
a) the directory where the VM lives
b) the directory where the image came from
c) the DefaultDirectory (which is likely the same as b unless the SecurityManager has changed it).
"
	| sources fd sourcesFile sourcesName |

	sourcesFile := fullSourcesName asFileReference.
	sourcesName := sourcesFile basename.
	
	"look for the sources file or an alias to it in the VM's directory"
	(sourcesFile := self vmDirectory / sourcesName) exists 
		ifTrue: [ sourcesFile readStream ifNotNilDo: [ :stream| ^ stream ]].
		
	"look for the sources file or an alias to it in the image directory"
	(sourcesFile := imagePath asFileReference parent / sourcesName) exists
		ifTrue: [ ^ sourcesFile readStream ifNotNilDo: [ :stream| ^ stream ]].
		
	"look for the sources in the current directory"
	(sourcesFile := sourcesName asFileReference) exists
		ifTrue: [ ^ sourcesFile readStream ifNotNilDo: [ :stream| ^ stream ]].
		
	^ nil
! !

!SmalltalkImage methodsFor: 'system attribute' stamp: 'IgorStasenko 8/17/2011 17:04'!
optionAt: i
	"Answer the i-th option of the command line, or nil if not so many options."
	
	^self  vm optionAt: i! !

!SmalltalkImage methodsFor: 'system attribute' stamp: ''!
options
	"Answer an array with all the command line options."
	
	"Smalltalk commandLine options"
	
	^Array streamContents: [:str |
		| arg i |
		i := 1.
		[i > 1000 or: [(arg := self optionAt: i) == nil]]
			whileFalse:
				[str nextPut: arg.
				i := i + 1]].! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: ''!
organization
	"Return the organizer for the receiver"
	^globals organization! !

!SmalltalkImage methodsFor: 'os' stamp: ''!
os
	"Answer the object to query about os."
	
	^ OSPlatform! !

!SmalltalkImage methodsFor: 'os' stamp: ''!
platform
	"Answer the object to query about os."
	
	^ OSPlatform! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
primBytesLeft
	"Primitive. Answer the number of bytes available for new object data.
	Not accurate unless preceded by
		Smalltalk garbageCollectMost (for reasonable accuracy), or
		Smalltalk garbageCollect (for real accuracy).
	See Object documentation whatIsAPrimitive."

	<primitive: 112>
	^ 0! !

!SmalltalkImage methodsFor: 'image, changes name' stamp: ''!
primImageName
	"Answer the full path name for the current image."
	"Smalltalk imageName"

	<primitive: 121>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'image, changes name' stamp: ''!
primImageName: newName
	"Set the the full path name for the current image.  All further snapshots will use this."

	<primitive: 121>
	^ self primitiveFailed! !

!SmalltalkImage methodsFor: 'image, changes name' stamp: 'CamilloBruni 5/13/2012 19:21'!
primImagePath
	"Answer the full path name for the current image."
	"Smalltalk imageName"

	<primitive: 121>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'image, changes name' stamp: 'CamilloBruni 5/13/2012 19:22'!
primImagePath: newName
	"Set the the full path name for the current image.  All further snapshots will use this."

	<primitive: 121>
	^ self primitiveFailed! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
primLowSpaceSemaphore: aSemaphore
	"Primitive. Register the given Semaphore to be signalled when the
	number of free bytes drops below some threshold. Disable low-space
	interrupts if the argument is nil."

	<primitive: 124>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
primSignalAtBytesLeft: numBytes
	"Tell the interpreter the low-space threshold in bytes. When the free
	space falls below this threshold, the interpreter will signal the low-space
	semaphore, if one has been registered.  Disable low-space interrupts if the
	argument is zero.  Fail if numBytes is not an Integer."

	<primitive: 125>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
primitiveGarbageCollect
	"Primitive. Reclaims all garbage and answers the number of bytes of available space."

	<primitive: 130>
	^ self primBytesLeft! !

!SmalltalkImage methodsFor: 'special objects' stamp: 'GuillermoPolito 5/3/2012 11:12'!
primitiveGetSpecialObjectsArray
	<primitive: 129>
	^ self primitiveFailed! !

!SmalltalkImage methodsFor: 'printing' stamp: ''!
printElementsOn: aStream
	aStream nextPutAll:'(lots of globals)'! !

!SmalltalkImage methodsFor: 'printing' stamp: ''!
printOn: aStream
	self == Smalltalk ifFalse: [^super printOn: aStream].
	aStream nextPutAll: 'Smalltalk'! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'PavelKrivanek 6/14/2011 16:08'!
privCleanChangeSets

	(ChangeSet allChangeSets reject: [:cs | cs = ChangeSet current]) 
		do: [:cs| ChangeSet removeChangeSet: cs ].

	ChangeSet current clear.
	ChangeSet current name: 'Unnamed1'.! !

!SmalltalkImage methodsFor: 'PharoCore' stamp: 'PavelKrivanek 8/15/2011 19:03'!
privCleanCore

	ScriptLoader new cleanUpForProduction.! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'PavelKrivanek 6/14/2011 16:02'!
privCleanGlobals

	#(ActiveEvent World ActiveWorld ActiveHand) do: [:sym |
		Smalltalk at: sym put: nil.
		 Smalltalk removeKey: sym ].! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'CamilleTeruel 7/29/2012 18:48'!
privCleanMonticello

	(Smalltalk classNamed: #MCFileBasedRepository) flushAllCaches.
	(Smalltalk classNamed: #MCDefinition) clearInstances.

	(Smalltalk classNamed: #MCWorkingCopy) allInstancesDo:[:wc| 
		SystemAnnouncer uniqueInstance unsubscribe: wc. 
		wc unregister].
	
	SystemAnnouncer uniqueInstance unsubscribe: (Smalltalk classNamed: #MCPackageManager).
	SystemAnnouncer uniqueInstance unsubscribe: (Smalltalk classNamed: #MCWorkingCopy).

! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'MarcusDenker 2/24/2012 13:34'!
privCleanProcesses

	UIManager default terminateUIProcess.

	(Process allInstances select: [:p | 
		( {	
			InputEventFetcher default instVarNamed: #fetcherProcess.
			Processor activeProcess.
			WeakArray runningFinalizationProcess.
			Processor backgroundProcess
		} includes: p ) not ])
	do: [:p | p suspend. p terminate ].	

	Delay startTimerEventLoop.! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'PavelKrivanek 6/14/2011 16:05'!
privCleanShutDownList

	#( #FreeTypeSettings #FreeTypeCache #FT2Handle #LogicalFont 
		#MCMethodDefinition #InternetConfiguration #TTCFont 
		#CPUWatcher #ProcessBrowser   #SecurityManager 
 		#CommandHistory   #Password #HttpUrl 
		#StrikeFont #PasteUpMorph #Form #ZnServer 
	) do: [:className |
		Smalltalk removeFromShutDownList: (Smalltalk classNamed: className)].! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'PavelKrivanek 6/14/2011 16:04'!
privCleanStartUpList

	#( #NaturalLanguageTranslator 
	 	#ShortIntegerArray #ShortRunArray #PasteUpMorph #SecurityManager 
		 #FreeTypeFontProvider  #UUIDGenerator #ProcessBrowser 
		 #CPUWatcher #DateAndTime #HostSystemMenus #MenuIcons
		 #InternetConfiguration  #CommandHistory #HostWindowProxy #FreeTypeSettings 
		 #UITheme #CodeLoader #ZnServer #MCWorkingCopy #RecentMessageList 
		 #EditorFindReplaceDialogWindow #CodeUpdate) 
	do: [:className |
		Smalltalk removeFromStartUpList: (Smalltalk classNamed: className)].! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'AlainPlantec 9/13/2011 21:31'!
privCleanTextConstants

	TextConstants TextSharedInformation removeKey: #DefaultTextStyle.
	TextConstants TextSharedInformation removeKey: #DefaultMultiStyle.
	TextConstants TextSharedInformation removeKey: #DefaultFixedTextStyle.
	TextConstants TextSharedInformation removeKey: StrikeFont defaultFontKey.

	Smalltalk allClasses select: [:c | c sharedPools includes: TextConstants] thenDo: [:c | c compileAll ].
! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'PavelKrivanek 7/17/2012 08:39'!
privCleanUnloadMethods

  " remove class unload methods to avoid their calling during removing of this classes "

  #(#Browser #ChangeList #ChangeSorter #DummySoundSystem #FT2Handle 
  #FileContentsBrowser #GZipReadStream #GZipWriteStream #MCVersionReader 
  #MailComposition #Morph #FreeTypeFontProvider
  #DefaultExternalDropHandler #FileStream #Form) 
    do: [:className | (Smalltalk at: className) class removeSelector: #unload. ].! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'PavelKrivanek 6/14/2011 16:17'!
privRecompileAll

	Smalltalk allClassesAndTraits do: [:classOrTrait | classOrTrait compileAll ].
! !

!SmalltalkImage methodsFor: 'private' stamp: 'PavelKrivanek 11/25/2011 14:33'!
privSaveImageWithCleanup

  " Do not use this method for common image saving. This method is designated for saving the image during the shrinking process in headless images."

  | report |

  (Delay forMilliseconds: 1000) wait.

  Processor yield.

 13 "a magic number (Camillo) "timesRepeat: [ Smalltalk garbageCollect ].
 
  [
    self snapshot: true andQuit: true.
  ] ifError: [:e |
    [ report := FileStream forceNewFileNamed: 'PharoDebug.log'.
    report nextPutAll: e asString; cr.
    ] ensure: [ report ifNotNil: [report close]].
  ]! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'PavelKrivanek 6/14/2011 15:58'!
privShrinkUnicodeTables

	#(GB2312Table JISX0208Table KSX1001Table) do: [:table| UCSTable classPool at: table put: nil ].

	Unicode classPool at: #DecimalProperty put: #(-1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 0 1 2 3 4 5 6 7 8 9 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1).

	Unicode classPool at: #GeneralCategory put: #(1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 29 21 21 21 23 21 21 21 22 18 21 25 21 17 21 21 13 13 13 13 13 13 13 13 13 13 21 21 25 25 25 21 21 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 22 21 18 24 16 24 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 22 25 18 25 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 29 21 23 23 23 23 26 26 24 26 5 20 25 17 26 24 26 25 15 15 24 5 26 21 24 15 5 19 15 15 15 21 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 9 25 9 9 9 9 9 9 9 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 5 25 5 5 5 5 5 5 5).
! !

!SmalltalkImage methodsFor: 'PharoKernel-private' stamp: 'CamilleTeruel 7/29/2012 18:34'!
privShrinkingProcess

  [
    | packages |
        
    10 timesRepeat: [
      (Delay forMilliseconds: 100) wait.
      Processor yield ].

    Author fullName: 'Mr.Shrinker'.

    self privCleanMonticello.

    UIManager default: DummyUIManager new.
    UIManager default progressBarEnabled: false.

    self privCleanProcesses.
    self privCleanGlobals.

    5 timesRepeat: [Smalltalk garbageCollect].

    MCDataStream initialize.

    Beeper instVarNamed: #default put: nil.

    Smalltalk cleanOutUndeclared.

    5 timesRepeat: [Smalltalk garbageCollect].

    Author reset.
  
    KMPragmaKeymapBuilder release.

    self privShrinkUnicodeTables.

    Display newDepth: 1.

    self privCleanStartUpList.
    self privCleanShutDownList.

"    CommandLine removeRegistrationNamed: #BasicCodeLoader.
    SimpleCodeLoader register.
"
    "move class Model to different category"
    Object subclass: #Model instanceVariableNames: 'dependents' classVariableNames: '' poolDictionaries: '' category: 'Kernel-Objects'.

    self privCleanTextConstants.

    SoundService default: nil.
    SoundService unregister: DummySoundSystem.

    ThreadSafeTranscript install.

    self privCleanUnloadMethods.

    SystemNavigation new removeAllButPackages: self kernelPackageNames.

    Smalltalk flushClassNameCache.
    3 timesRepeat: [
      Smalltalk garbageCollect.
        Symbol compactSymbolTable.].

    Undeclared removeUnreferencedKeys.
    self privRecompileAll.
    
    self privCleanChangeSets.
    
    FileServices removeObsolete.
    DebuggerMethodMap voidMapCache.

    Object flushEvents.

    MCDataStream initialize.

    Locale classPool at: #LocaleChangeListeners put: nil.
    SmalltalkImage classPool at: #Tools put: nil.

    Behavior flushObsoleteSubclasses.
    Smalltalk flushClassNameCache.
    Smalltalk organization removeEmptyCategories.

    WeakArray restartFinalizationProcess.

    "UIManager default progressBarEnabled: true."
    "Smalltalk condenseChanges."

    Smalltalk saveImageInNewContext.

  ] ifError: [:e |
    | rep |
    rep := FileStream forceNewFileNamed: 'PharoDebug.log'.
    rep nextPutAll: 'PharoKernel shrinking report'; cr.
    rep nextPutAll: 'Error:'; cr.
    rep nextPutAll: e asString; cr.
    rep nextPutAll: thisContext stack size asString.
    thisContext stack copy withIndexDo: [:stck :i |
    [ rep nextPutAll: i asString; space; nextPutAll: stck asString; cr] ifError: []].
    rep close. 
    Smalltalk exitFailure ] 



! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'Camillobruni 8/11/2011 22:56'!
processShutDownList: quitting
	"Send #shutDown to each class that needs to wrap up before a snapshot.
	Also void the endianness chace; this can't safely be done on start-up because
	Smalltalk is too late in the start-up sequence."

	EndianCache := nil.
	^ self send: #shutDown: toClassesNamedIn: ShutDownList with: quitting.
! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'Camillobruni 8/11/2011 22:54'!
processStartUpList: resuming
	"Send #startUp to each class that needs to run initialization after a snapshot."

	^ self send: #startUp: toClassesNamedIn: StartUpList with: resuming.
! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: ''!
quitPrimitive
	"Primitive. Exit to another operating system on the host machine, if one
	exists. All state changes in the object space since the last snapshot are lost.
	Essential. See Object documentation whatIsAPrimitive."

	<primitive: 113>
	self primitiveFailed! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'GuillermoPolito 7/2/2012 12:29'!
recordStartupStamp
	
	ChangesLog default recordStartupStamp! !

!SmalltalkImage methodsFor: 'special objects' stamp: 'GuillermoPolito 4/4/2012 18:01'!
recreateSpecialObjectsArray
	"Smalltalk recreateSpecialObjectsArray"
	
	"To external package developers:
	**** DO NOT OVERRIDE THIS METHOD.  *****
	If you are writing a plugin and need additional special object(s) for your own use, 
	use addGCRoot() function and use own, separate special objects registry "
	
	"The Special Objects Array is an array of objects used by the Squeak virtual machine.
	 Its contents are critical and accesses to it by the VM are unchecked, so don't even
	 think of playing here unless you know what you are doing."
	
	"Replace the interpreter's reference in one atomic operation"
	self specialObjectsArray becomeForward: self newSpecialObjectsArray.
	! !

!SmalltalkImage methodsFor: 'special objects' stamp: ''!
registerExternalObject: anObject
	"Register the given object in the external objects array and return its index. If it is already there, just return its index."

	^ExternalSemaphoreTable registerExternalObject: anObject! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: 'SeanDeNigris 6/21/2012 08:40'!
removeAllLineFeeds
	"Smalltalk removeAllLineFeeds"
	"Scan all methods for source code with lineFeeds.
	Replaces all occurrences of <CR><LF> or <LF> by <CR>.
	When done, offers to display an Inspector containing the message
	names grouped by author initials.
	In this dictionary, the key 'OK' contains the methods that had literals that contained <LF> characters."
	| totalStripped totalOK authors |
	'Scanning sources for LineFeeds.
This will take a few minutes...'
		displayProgressFrom: 0
		to: CompiledMethod instanceCount
		during: [:bar | | n | 
			n := 0.
			authors := self
						removeAllLineFeedsQuietlyCalling: [(n := n + 1) \\ 100 = 0
								ifTrue: [bar current: n]]].
	totalStripped := authors
				inject: 1
				into: [:sum :set | sum + set size].
	totalOK := (authors at: 'OK') size.
	totalStripped := totalStripped - totalOK.
	self crTrace: totalStripped printString , ' methods stripped of LFs.'.
	self crTrace: totalOK printString , ' methods still correctly contain LFs.'.
	(self confirm: 'Do you want to see the affected methods?')
		ifTrue: [ authors inspect ]! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: ''!
removeAllLineFeedsQuietly
	"Smalltalk removeAllLineFeedsQuietly"
	"Scan all methods for source code with lineFeeds.
	Replaces all occurrences of <CR><LF> or <LF> by <CR>.
	Answer a Dictionary keyed by author name containing sets of affected method names,
	as well as (at the key 'OK') a list of methods that still contain LF characters inside literal strings or characters."
	^self removeAllLineFeedsQuietlyCalling: [].! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: ''!
removeAllLineFeedsQuietlyCalling: aBlock
	"Smalltalk removeAllLineFeedsQuietly"
	"Scan all methods for source code with lineFeeds.
	Replaces all occurrences of <CR><LF> or <LF> by <CR>.
	Answer a Dictionary keyed by author name containing sets of affected method names,
	as well as (at the key 'OK') a list of methods that still contain LF characters inside literal strings or characters.
	Evaluate aBlock for each method so that status can be updated."
	| authors |
	authors := Dictionary new.
	authors at: 'OK' put: Set new.
	self systemNavigation
		allBehaviorsDo: [:cls | cls selectorsAndMethodsDo: [:selector :method |
					| oldStamp newCodeString oldCodeString oldCategory nameString | 
					aBlock cull: cls cull: selector.
					oldCodeString := cls sourceCodeAt: selector.
					(oldCodeString includes: Character lf)
						ifTrue: [
							newCodeString := oldCodeString withSqueakLineEndings.
							nameString := cls name , '>>' , selector.
							((cls compiledMethodAt: selector) hasLiteralSuchThat: [ :lit | lit asString includes: Character lf ])
								ifTrue: [(authors at: 'OK')
										add: nameString]
								ifFalse: [oldStamp := (method timeStamp)
												copy replaceAll: Character cr
												with: Character space.
									(authors
										at: (oldStamp copyFrom: 1 to: (oldStamp findFirst: [ :c | c isAlphaNumeric not ]))
										ifAbsentPut: [Set new])
										add: nameString.
									oldCategory := cls whichCategoryIncludesSelector: selector.
									cls
										compile: newCodeString
										classified: oldCategory
										withStamp: oldStamp
										notifying: nil ]]]].
	^ authors! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: 'MarcusDenker 4/21/2011 21:34'!
removeClassNamed: aName
	"Invoked from fileouts:  if there is currently a class in the system named aName, then remove it.  If anything untoward happens, report it in the Transcript.  "

	globals removeClassNamed: aName! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: ''!
removeEmptyMessageCategories
	"Smalltalk removeEmptyMessageCategories"
	self garbageCollect.
	(ClassOrganizer allInstances copyWith: SystemOrganization)
		do: [:org | org removeEmptyCategories]! !

!SmalltalkImage methodsFor: 'startup list' stamp: ''!
removeFromShutDownList: aClass

	ShutDownList remove: aClass name ifAbsent: []! !

!SmalltalkImage methodsFor: 'startup list' stamp: ''!
removeFromStartUpList: aClass

	StartUpList remove: aClass name ifAbsent: []! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
removeKey: key
	"Obsoleted."

	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals removeKey: key! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
removeKey: key ifAbsent: aBlock
	"Obsoleted."

	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals removeKey: key ifAbsent: aBlock! !

!SmalltalkImage methodsFor: 'shrinking' stamp: 'MarcusDenker 4/27/2011 21:08'!
removeSelector: descriptor 
	"Safely remove a selector from a class (or metaclass). If the
	class or the method doesn't exist anymore, never mind and
	answer nil.
	This method should be used instead of 'Class removeSelector:
	#method' to omit global class references."
	| class sel |
	class := self
				at: descriptor first
				ifAbsent: [^ nil].
	(descriptor size > 2 and: [descriptor second == #class])
		ifTrue: 
			[class := class class.
			sel := descriptor third]
		ifFalse: 
			[sel := descriptor second].
	^ class removeSelector: sel! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: ''!
renameClass: aClass as: newName 
	"Rename the class, aClass, to have the title newName."

	^globals renameClass: aClass as: newName! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: ''!
renameClass: aClass from: oldName
	"Rename the class, aClass, to have the title newName."

	^globals renameClass: aClass from: oldName! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: 'StephaneDucasse 11/7/2011 22:44'!
renameClassNamed: oldName as: newName
	"Invoked from fileouts:  if there is currently a class in the system named oldName, then rename it to newName.  If anything untoward happens, report it in the Transcript.  "

	| oldClass |
	(oldClass := self at: oldName asSymbol ifAbsent: [nil]) == nil
		ifTrue:
			[self crTrace: 'Class-rename for ', oldName, ' ignored because ', oldName, ' does not exist.'.
			^ self].

	oldClass rename: newName! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: ''!
renamedClass: aClass from: oldName to: newName
	"Inform Smalltalk that aClass has been renamed"
	
	(Array with: StartUpList with: ShutDownList) 
		do: [ :list | | index |  
			index := list indexOf: oldName ifAbsent: [0].
			index > 0 ifTrue: [list at: index put: newName]].
! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
rootTable
	"Primitive. Answer a snapshot of the VMs root table. 
	Keep in mind that the primitive may itself cause GC."
	<primitive: 'primitiveRootTable'>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
rootTableAt: index
	"Primitive. Answer the nth element of the VMs root table"
	<primitive: 'primitiveRootTableAt'>
	^nil! !

!SmalltalkImage methodsFor: 'saving' stamp: ''!
saveAs

	"Put up the 'saveAs' prompt, obtain a name, and save the image  under that new name."

	self saveAs: self getFileNameFromUser! !

!SmalltalkImage methodsFor: 'saving' stamp: ''!
saveAs: newName
	newName ifNil: [ ^ self ].
	(SourceFiles at: 2) ifNotNil:
		[self closeSourceFiles; "so copying the changes file will always work"
			 saveChangesInFileNamed: (self fullNameForChangesNamed: newName)].
	^self saveImageInFileNamed: (self fullNameForImageNamed: newName)! !

!SmalltalkImage methodsFor: 'saving' stamp: 'CamilloBruni 5/7/2012 03:02'!
saveAsNewVersion
	"Save the image/changes using the next available version number."
	"SmalltalkImage current saveAsNewVersion"
	
	| newImageFile newChangesFile |
	newImageFile := self imageFile nextVersion.
	newChangesFile := newImageFile withExtension: self changesSuffix.

	"Check to see if there is a .changes file that would cause a problem if we saved a new .image file with the new version number"
	newChangesFile exists
		ifTrue:
			[^ self inform:
'There is already .changes file of the desired name,
', newChangesFile fullName, '
curiously already present, even though there is
no corresponding .image file.   Please remedy
manually and then repeat your request.'].

	"strip awaay the image extension"
	^self saveAs: (newImageFile fullName copyUpToLast: $.)


! !

!SmalltalkImage methodsFor: 'saving' stamp: 'CamilloBruni 5/9/2012 00:14'!
saveChangesInFileNamed: aString
	| changesFile |
	
	changesFile := aString asFileReference.
	changesFile parent ensureDirectory.
	
	Smalltalk changesFile copyTo: changesFile.
		
	self setMacFileInfoOn:  changesFile fullName.! !

!SmalltalkImage methodsFor: 'saving' stamp: 'CamilloBruni 5/13/2012 19:32'!
saveImageInFileNamed: aString
	| imageFile previousFullImageName |
	
	imageFile := aString  asFileReference.
	imageFile parent ensureDirectory.
	previousFullImageName := self imagePath.
	
	^self
		changeImagePathTo: imageFile fullName;
		closeSourceFiles;
		openSourceFiles;  "so SNAPSHOT appears in new changes file"
		snapshot: true andQuit: false! !

!SmalltalkImage methodsFor: 'saving' stamp: 'PavelKrivanek 6/14/2011 09:59'!
saveImageInNewContext

	" This method is used for saving the Smalltalk image in a new process so the current process with all stack information may be forgotten.
	
	Do not use this method for common image saving. This method is designated for saving the image during the shrinking process in headless images. "

	[ self privSaveImageWithCleanup ] forkAt: 40.! !

!SmalltalkImage methodsFor: 'saving' stamp: 'StephaneDucasse 11/9/2010 18:55'!
saveSession
	"Save the current image without quitting and return it resuming status (whether it was restarted or in the same flow)."
	
	^ self snapshot: true andQuit: false! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
select: aBlock
	"Obsoleted."
	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals select: aBlock
! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'CamilloBruni 2/13/2012 23:35'!
send: startUpOrShutDown toClassesNamedIn: startUpOrShutDownList with: argument
	"Send the message #startUp: or #shutDown: to each class named in the list.
	The argument indicates if the system is about to quit (for #shutDown:) or if
	the image is resuming (for #startUp:).
	If any name cannot be found, then remove it from the list."

	| removals errors |
	removals := OrderedCollection new.
	errors := OrderedCollection new.
	startUpOrShutDownList do: [ :name | 
			| class |
			class := self at: name ifAbsent: [ nil ].
			class isNil
				ifTrue: [ removals add: name ]
				ifFalse: [ 
					self 
						logStartUpErrorDuring: [ class perform: startUpOrShutDown with: argument ] 
						into: errors 
						tryDebugger: false]].
			
	"Remove any obsolete entries, but after the iteration"
	startUpOrShutDownList removeAll: removals.
	^ errors! !

!SmalltalkImage methodsFor: 'miscellaneous' stamp: 'SeanDeNigris 7/12/2012 08:44'!
setMacFileInfoOn: aString
	"On Mac, set the file type and creator (noop on other platforms)"
	aString asFileReference setMacType: 'STch' creator: 'FAST'.! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 01:51'!
shortImageName
	"Answer the current image name but only that name and not the full path as with imageName."
	"SmalltalkImage current shortImageName"

	^ self imageFile basename! !

!SmalltalkImage methodsFor: 'PharoCore' stamp: 'PavelKrivanek 8/15/2011 19:03'!
shrinkToCore

	| packagesToUnload |
	
	packagesToUnload := (ScriptLoader new allCurrentPackages collect: [:p | p package name]) copyWithoutAll: (Smalltalk kernelPackageNames, Smalltalk corePackageNames).
	
	packagesToUnload do: [:packageName |
		ScriptLoader  new unloadPackageNamed: packageName ].
	
	self privCleanCore.! !

!SmalltalkImage methodsFor: 'PharoKernel' stamp: 'PavelKrivanek 6/14/2011 22:07'!
shrinkToKernel

	"
	WARNING:	THIS METHOD WILL DESTROY YOUR IMAGE

	Use with cation and only on fresh Pharo-Core images 
	"

	[ self privShrinkingProcess ] forkAt: 40.

	Processor terminateActive.
	
! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: ''!
shutDown
	^ Smalltalk closeSourceFiles! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'GuillermoPolito 4/23/2012 14:52'!
shutDownImage: quitting
	Object flushDependents.
	Object flushEvents.

	self processShutDownList: quitting.! !

!SmalltalkImage methodsFor: 'memory space' stamp: ''!
signalLowSpace
	"Signal the low-space semaphore to alert the user that space is running low."

	LowSpaceSemaphore signal.! !

!SmalltalkImage methodsFor: 'to clean later' stamp: ''!
size
	"Obsoleted."

	self flag: #tobeprecated.
	"self deprecated: 'Use Smalltalk globals'."
	^globals size! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'GuillermoPolito 7/6/2012 11:03'!
snapshot: save andQuit: quit
	"Mark the changes file and close all files as part of #processShutdownList.
	If save is true, save the current state of this Smalltalk in the image file.
	If quit is true, then exit to the outer OS shell.
	The latter part of this method runs when resuming a previously saved image. This resume logic checks for a document file to process when starting up."

	| snapshotResult resuming startupErrors |
	self addSnapshotRecord: save andQuit: quit.
	self shutDownImage: quit.
									
	save
		ifTrue: [ 
			snapshotResult := self snapshotPrimitive.	"<-- PC frozen here on image file"
			resuming := snapshotResult == true ]
		ifFalse: [ resuming := false ].
	(quit and: [ resuming not ])
		ifTrue: [ self quitPrimitive ].
	
	self startupImage: resuming snapshotWorked: snapshotResult.
	
	"We return the resuming state, which may be useful for users to know the state of the image"
	^resuming! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: ''!
snapshotPrimitive
	"Primitive. Write the current state of the object memory on a file in the
	same format as the Smalltalk-80 release. The file can later be resumed,
	returning you to this exact state. Return normally after writing the file.
	Essential. See Object documentation whatIsAPrimitive."

	<primitive: 97>
	^nil "indicates error writing image file"! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: ''!
sourceFileVersionString

	^ SourceFileVersionString! !

!SmalltalkImage methodsFor: 'private' stamp: ''!
sourceFileVersionString: aString

	SourceFileVersionString := aString! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 01:57'!
sourcesFile
	"Answer the full path to the version-stable source code"
	^ self vmDirectory /  SourceFileVersionString , self sourcesSuffix! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 01:58'!
sourcesName
	"Answer the full path to the version-stable source code"
	^ self sourcesFile fullName! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 5/7/2012 01:57'!
sourcesSuffix
	^ 'sources'! !

!SmalltalkImage methodsFor: 'memory space' stamp: 'BernardoContreras 10/27/2011 19:16'!
spaceLeftAfterGarbageCollection
	"Return a string describing the amount of memory available"
	| availInternal availPhysical availTotal |
	self garbageCollect.
	availInternal := self primBytesLeft.
	availPhysical := self bytesLeft: false.
	availTotal := self bytesLeft: true.
	(availTotal > (availInternal + 10000)) "compensate for mini allocations inbetween"
		ifFalse:[^availInternal asStringWithCommas, ' bytes available'].
	^String streamContents:[:s|
		s nextPutAll: availInternal asStringWithCommas, 	' bytes (internal) '; cr.
		s nextPutAll: availPhysical asStringWithCommas,	' bytes (physical) '; cr.
		s nextPutAll: availTotal asStringWithCommas, 	' bytes (total)     '].! !

!SmalltalkImage methodsFor: 'special objects' stamp: ''!
specialNargsAt: anInteger 
	"Answer the number of arguments for the special selector at: anInteger."

	^ (self specialObjectsArray at: 24) at: anInteger * 2! !

!SmalltalkImage methodsFor: 'special objects' stamp: 'GuillermoPolito 5/3/2012 11:13'!
specialObjectsArray
	^specialObjectsArray ifNil: [ specialObjectsArray := self primitiveGetSpecialObjectsArray ].! !

!SmalltalkImage methodsFor: 'special objects' stamp: 'GuillermoPolito 5/3/2012 11:13'!
specialObjectsArray: anArray
	specialObjectsArray := anArray! !

!SmalltalkImage methodsFor: 'special objects' stamp: ''!
specialSelectorAt: anInteger 
	"Answer the special message selector from the interleaved specialSelectors array."

	^ (self specialObjectsArray at: 24) at: anInteger * 2 - 1! !

!SmalltalkImage methodsFor: 'special objects' stamp: ''!
specialSelectorSize
	"Answer the number of special selectors in the system."

	^ (self specialObjectsArray at: 24) size // 2! !

!SmalltalkImage methodsFor: 'special objects' stamp: ''!
specialSelectors
	"Used by SystemTracer only."

	^SpecialSelectors! !

!SmalltalkImage methodsFor: 'snapshot and quit' stamp: 'GuillermoPolito 4/24/2012 16:57'!
startupImage: resuming snapshotWorked: snapshotResult
	| startupErrors |
	self vm setGCParameters.	
	deferredStartupActions := nil.
	resuming ifTrue: [ Smalltalk clearExternalObjects ].
	
	startupErrors := OrderedCollection new.
	
	UIManager default boot: resuming during: [
		startupErrors addAll: (self processStartUpList: resuming).
		resuming ifTrue: [ self recordStartupStamp ]
		].
	
	startupErrors addAll: (self executeDeferredStartupActions: resuming).
	
	startupErrors isEmpty 
		ifFalse: [ self handleStartupErrors: startupErrors ].
		
	snapshotResult == nil
		ifTrue: [ self error: 'Failed to write image file (disk full?)' ].! !

!SmalltalkImage methodsFor: '*System-ObjectStorage' stamp: 'tk 3/7/2000 18:40'!
storeDataOn: aDataStream
	"I don't get stored.  Use a DiskProxy"

	self error: 'use a DiskProxy to store me'! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: ''!
systemInformationString
	^ String cr, SystemVersion current version, String cr, self lastUpdateString, String cr, String cr, String cr, self licenseString! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: ''!
timeStamp: aStream 
	"Writes system version and current time on stream aStream."

	| dateTime |
	dateTime := Time dateAndTimeNow.
	aStream nextPutAll: 'From ', self datedVersion, ' [', self  lastUpdateString, '] on ', (dateTime at: 1) printString,
						' at ', (dateTime at: 2) printString! !

!SmalltalkImage methodsFor: 'class and trait names' stamp: ''!
traitNames
	"Answer a SortedCollection of all traits (not including class-traits) names."
	
	^ globals traitNames! !

!SmalltalkImage methodsFor: 'external' stamp: ''!
unbindExternalPrimitives
	"Primitive. Force all external primitives to be looked up again afterwards. Since external primitives that have not found are bound for fast failure this method will force the lookup of all primitives again so that after adding some plugin the primitives may be found."
	<primitive: 570>
	"Do nothing if the primitive fails for compatibility with older VMs"
! !

!SmalltalkImage methodsFor: 'modules' stamp: ''!
unloadModule: aString
	"Primitive. Unload the given module.
	This primitive is intended for development only since some
	platform do not implement unloading of DLL's accordingly.
	Also, the mechanism for unloading may not be supported
	on all platforms."
	<primitive: 571>
	^self primitiveFailed! !

!SmalltalkImage methodsFor: 'special objects' stamp: ''!
unregisterExternalObject: anObject
	"Unregister the given object in the external objects array. Do nothing if it isn't registered."

	ExternalSemaphoreTable unregisterExternalObject: anObject! !

!SmalltalkImage methodsFor: 'housekeeping' stamp: ''!
verifyChanges		
	"Smalltalk verifyChanges"
	"Recompile all methods in the changes file."
	self systemNavigation allBehaviorsDo: [:class | class recompileChanges].
! !

!SmalltalkImage methodsFor: 'sources, change log' stamp: ''!
version
	"Answer the version of this release."

	^SystemVersion current version! !

!SmalltalkImage methodsFor: 'copying' stamp: ''!
veryDeepCopyWith: deepCopier
	"Return self.  I can't be copied.  Do not record me."! !

!SmalltalkImage methodsFor: 'accessing' stamp: 'IgorStasenko 8/16/2011 17:52'!
vm
	"Answer the object to query about virtual machine."
	
	^ VirtualMachine! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 7/10/2012 21:54'!
vmBinary
	^ self vm binary! !

!SmalltalkImage methodsFor: 'image, changes names' stamp: 'CamilloBruni 7/10/2012 21:51'!
vmDirectory
	^ self vm directory! !

!SmalltalkImage methodsFor: 'image' stamp: 'IgorStasenko 8/17/2011 15:04'!
wordSize
	"Answer the size in bytes of an object pointer or word in the object memory.
	The value does not change for a given image, but may be modified by a SystemTracer
	when converting the image to another format. The value is cached in WordSize to
	avoid the performance overhead of repeatedly consulting the VM."

	"this method is deprecated" 
	^ self vm wordSize! !

Object subclass: #SystemNavigation
	instanceVariableNames: 'browserClass environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Support'!

!SystemNavigation commentStamp: 'LucFabresse 10/10/2010 19:54' prior: 0!
I support the navigation of the system. 
I act as a facade but as I could require some state or different way of navigating the system all my behavior are on the instance side.
I should (it is not complety done yet) be parametrized by an environment (aSystemDictionary) that scopes my queries.!

!SystemNavigation class methodsFor: 'accessing' stamp: 'StephaneDucasse 10/23/2010 15:40'!
default
	^ self new! !

!SystemNavigation methodsFor: 'message sends' stamp: 'StephaneDucasse 7/26/2011 12:50'!
addSelectorsReferingTo: aSymbol in: class to: sortedSenders special: special byte: byte 
	{class. class class} do: [:behavior| (behavior 
		thoroughWhichSelectorsReferTo: aSymbol
		special: special
		byte: byte) do: [ :sel | 
			sortedSenders add: (self createMethodNamed: sel realParent: behavior) ]]! !

!SystemNavigation methodsFor: 'query' stamp: 'AlexandreBergel 12/29/2010 11:39'!
allBehaviorsDo: aBlock 
	"Execute a block on each class, metaclass, trait and trait class"
	
	self environment allClassesDo: [ :aClass | 
		aBlock value: aClass ; 
			value: aClass class
	].
	
	self environment allTraitsDo: [ :aTrait |
		aBlock value:  aTrait ;
			value: aTrait classTrait
	]
! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allCallsOn: aSymbol 
	"Answer a Collection of all the methods that call on the provided symbol even deeply embedded in literal array."
	"self new allCallsOn: #allCallsOn:"
	
	^ self allReferencesTo: aSymbol! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 7/26/2011 12:45'!
allCallsOn: firstLiteral and: secondLiteral
	"Answer a SortedCollection of all the methods that call on both aLiteral 
	and secondLiteral."

	| aCollection firstSpecial secondSpecial firstByte secondByte |
	self flag: #ShouldUseAllCallsOn:. "sd"
	aCollection := SortedCollection new.
	firstSpecial := Smalltalk hasSpecialSelector: firstLiteral ifTrueSetByte: [:b | firstByte := b].
	secondSpecial := Smalltalk hasSpecialSelector: secondLiteral ifTrueSetByte: [:b | secondByte := b].
	self allBehaviorsDo: [:class | | secondArray |
			secondArray := class whichSelectorsReferTo: secondLiteral special: secondSpecial byte: secondByte.
			((class whichSelectorsReferTo: firstLiteral special: firstSpecial byte: firstByte) select: [:aSel | 
				(secondArray includes: aSel)]) do: [:sel | 
					aCollection add: (self createMethodNamed: sel realParent: class)]].
	^aCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 7/26/2011 12:45'!
allCallsOn: aSymbol from: aClass
	"Answer a SortedCollection of all the methods that call on aSymbol."

	| aSortedCollection special byte |
	aSortedCollection := SortedCollection new.
	special := Smalltalk hasSpecialSelector: aSymbol ifTrueSetByte: [:b | byte := b ].
	aClass withAllSubclassesDo: [ :class |
		(class whichSelectorsReferTo: aSymbol special: special byte: byte) do: [:sel |
				aSortedCollection add: (self createMethodNamed: sel realParent: class)]].
	^aSortedCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'MarcusDenker 12/11/2011 15:35'!
allCallsOn: aSymbol localTo: aClass
	"Answer a Set of RGMethodDefinition for all the methods
	 that call on aSymbol in, above or below the given class."

	| aSet special byte enum |
	aSet := Set new.
	special := Smalltalk hasSpecialSelector: aSymbol ifTrueSetByte: [:b | byte := b].
	enum := [:behavior|
			(behavior whichSelectorsReferTo: aSymbol special: special byte: byte) do:
				[:sel | aSet add: (RGMethodDefinition realClass: behavior  selector: sel)]].
	aClass theNonMetaClass withAllSuperAndSubclassesDo: enum.
		aClass theNonMetaClass class withAllSuperAndSubclassesDo: enum.
	^aSet! !

!SystemNavigation methodsFor: 'query' stamp: 'LucFabresse 10/11/2010 19:57'!
allClasses
	"currently returns all the classes in the current environment"

	^ self environment allClasses

	! !

!SystemNavigation methodsFor: 'query' stamp: 'AlexandreBergel 12/29/2010 11:38'!
allClassesAndTraits
	"Answer all the classes and traits present in the image"
	"Note that metaclasses are not part of the answer"
	
	^ Smalltalk globals allClassesAndTraits

	! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 11/9/2010 15:21'!
allClassesDefiningClassVarNamed: aString  
	"Answer an Array of all classes that define the class variables named aString."

	| aCollection |
	aCollection := ReadWriteStream on: Array new.
	self allClassesDo: [:class | (class hasClassVarNamed: aString) ifTrue: [aCollection nextPut: class]].
	^ aCollection contents! !

!SystemNavigation methodsFor: 'query' stamp: 'MarcusDenker 4/18/2011 10:07'!
allClassesDo: aBlock

	^ self environment allClassesDo: aBlock

	! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allClassesImplementing: aSelector  
	"Answer an Array of all classes that implement the message aSelector."

	| aCollection |
	aCollection := ReadWriteStream on: Array new.
	self allBehaviorsDo: [:class | (class includesSelector: aSelector) ifTrue: [aCollection nextPut: class]].
	^ aCollection contents! !

!SystemNavigation methodsFor: 'packages' stamp: ''!
allClassesInPackageNamed: packageName
	"Returns the list of classes that belongs to a Monticello package."
	
	"It would be probably better to delegate that query to the package itself"
	| classes organization |
	organization := Smalltalk globals organization.
	classes := (organization classesInCategory: packageName) asSet.
	(organization categoriesMatching: packageName, '-*') do: [:category| 
		classes addAll: (organization classesInCategory: category)].
	^classes asArray! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 11/9/2010 14:48'!
allClassesUsingSharedPool: aString  
	"Answer an Array of all classes that uses the shared pool named aString."

	| aCollection |
	aCollection := ReadWriteStream on: Array new.
	self allClassesDo: [:class | (class includesSharedPoolNamed: aString) ifTrue: [aCollection nextPut: class]].
	^ aCollection contents! !

!SystemNavigation methodsFor: 'query' stamp: 'GuillermoPolito 4/26/2012 11:21'!
allClassesWithUnimplementedCalls
	"Answer an Array of classes that have messages with calls to methods that aren't implemented
	anywhere in the system"
	| all dict |
	dict := Dictionary new.
	all := self systemNavigation allImplementedMessages.
	self systemNavigation allBehaviorsDo: [:cl | 
		cl selectorsAndMethodsDo: [:sel :meth | 
			meth isPrimitive ifFalse: [
				meth messages do: [:m | 
					(all includes: m) ifFalse: [
						((dict at: cl ifAbsentPut: [ Dictionary new ])
							at: sel ifAbsentPut: [ OrderedCollection new])
								add: m
					]
				]
			]
		]
	].
	^ dict! !

!SystemNavigation methodsFor: 'author' stamp: 'MarcusDenker 7/12/2012 18:06'!
allContributors
	"Return the list of initials and names of method authors."
	
	"I am wondering why a bag is used instead of a set. AlexandreBergel 12/29/2010 11:37"

	"SystemNavigation new allContributors"
	| contributorsNames |
	contributorsNames := Bag new.
	self allBehaviorsDo: [ :behavior |
		behavior methodsDo: [ :compiledMethod | | stamp |
			stamp := compiledMethod timeStamp.
			stamp notEmpty ifTrue: [
				contributorsNames add: compiledMethod timeStamp substrings first ]]].
	^contributorsNames! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allGlobalRefs
	"Answer a set of symbols that may be refs to Global names.  In some sense we should only need the associations, but this will also catch, eg, HTML tag types."

	^ self allGlobalRefsWithout: {{}. {}}! !

!SystemNavigation methodsFor: 'query' stamp: 'GuillermoPolito 3/18/2012 21:28'!
allGlobalRefsWithout: classesAndMessagesPair
	"Answer a set of symbols that may be refs to Global names. In some  
	sense we should only need the associations, but this will also catch, eg,  
	HTML tag types. This method computes its result in the absence of  
	specified classes and messages."

	"may be a problem if namespaces are introduced as for the moment  
	only Smalltalk is queried. sd 29/4/03"

	| globalRefs absentClasses absentSelectors |
	globalRefs := IdentitySet new: CompiledMethod instanceCount.
	absentClasses := classesAndMessagesPair first.
	absentSelectors := classesAndMessagesPair second.
	self flag: #shouldBeRewrittenUsingSmalltalkAllClassesDo:.	"sd 29/04/03"
			Smalltalk classNames
				do: [ :cName | 
					((absentClasses includes: cName)
						ifTrue: [ {} ]
						ifFalse: [ 
							{(Smalltalk globals at: cName).
							((Smalltalk globals at: cName) class)} ])
						do: [ :cl | 
							(absentSelectors isEmpty
								ifTrue: [ cl selectors ]
								ifFalse: [ cl selectors copyWithoutAll: absentSelectors ])
								do: [ :sel | 
									"Include all capitalized symbols for good 
									measure"
									(cl compiledMethodAt: sel)
										literalsDo: [ :m | 
											(m isSymbol and: [ m size > 0 and: [ m first canBeGlobalVarInitial ] ])
												ifTrue: [ globalRefs add: m ].
											(m isMemberOf: Array)
												ifTrue: [ 
													m
														do: [ :x | 
															(x isSymbol and: [ x size > 0 and: [ x first canBeGlobalVarInitial ] ])
																ifTrue: [ globalRefs add: x ] ] ].
											m isVariableBinding
												ifTrue: [ m key ifNotNil: [ globalRefs add: m key ] ] ] ] ] ].
	^ globalRefs! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allImplementedMessages
	"Answer a Set of all the messages that are implemented in the system."
	^ self allImplementedMessagesWithout: {{}. {}}! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allImplementedMessagesWithout: classesAndMessagesPair
	"Answer a Set of all the messages that are implemented in the system,  
	computed in the absence of the supplied classes and messages. Note this  
	reports messages that are in the absent selectors set."

	| messages absentClasses |
	messages := IdentitySet new: CompiledMethod instanceCount.
	absentClasses := classesAndMessagesPair first.
	self flag: #shouldBeRewrittenUsingSmalltalkAllClassesDo:.	"sd 29/04/03"
	Smalltalk classNames , Smalltalk traitNames
		do: [ :name | 
			((absentClasses includes: name)
				ifTrue: [ {} ]
				ifFalse: [ 
					{(Smalltalk globals at: name).
					((Smalltalk globals at: name) classSide)} ]) do: [ :each | messages addAll: each selectors ] ].
	^ messages! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 7/26/2011 11:33'!
allImplementorsOf: aSelector 
	"Answer a SortedCollection of all the methods that implement the message 
	aSelector."
	
	| aCollection |
	aCollection := SortedCollection new.
	self allBehaviorsDo: [:class | 
		(class includesSelector: aSelector)
			ifTrue: [aCollection add: (self createMethodNamed: aSelector realParent: class)]].
	^ aCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 7/26/2011 11:33'!
allImplementorsOf: aSelector localTo: aClass 
	"Answer a SortedCollection of all the methods that implement the message  
	aSelector in, above, or below the given class."
	| cls aCollection |
	aCollection := SortedCollection new.
	cls := aClass theNonMetaClass.
	cls withAllSuperAndSubclassesDo: [:class | (class includesSelector: aSelector)
						ifTrue: [ aCollection
									add: (self createMethodNamed: aSelector realParent: class) ]].
	cls class withAllSuperAndSubclassesDo: [:class | (class includesSelector: aSelector)
						ifTrue: [ aCollection
									add: (self createMethodNamed: aSelector realParent: class) ]].
	^ aCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 8/12/2011 14:58'!
allLocalCallsOn: aSymbol ofClass: aClass
	"Answer a SortedCollection of all the methods that call on aSymbol, anywhere in my class hierarchy."
	
	| aSet special byte cls |
	aSet := Set new.
	cls := aClass theNonMetaClass.
	special := Smalltalk hasSpecialSelector: aSymbol
					ifTrueSetByte: [:b | byte := b ].
	cls withAllSuperAndSubclassesDo: [ :class |
		(class whichSelectorsReferTo: aSymbol special: special byte: byte)
			do: [:sel |
				aSet add: (self createMethodNamed: sel realParent: class) ]].
		
	cls class withAllSuperAndSubclassesDo: [ :class |
		(class whichSelectorsReferTo: aSymbol special: special byte: byte)
			do: [:sel |
				aSet add: (self createMethodNamed: sel realParent: class) ]].
	^aSet! !

!SystemNavigation methodsFor: 'accessing' stamp: 'MarcusDenker 7/12/2012 18:04'!
allMethodsInCategory: category 
	"Answer the collection of methods that are contained in a particular method category"
	
	"SystemNavigation new allMethodsInCategory: 'printing'"

	| aCollection |
	aCollection := Set new.
			self allBehaviorsDo: [ :behavior | ((category = ClassOrganizer allCategory
					ifTrue: [ behavior organization allMethodSelectors]
					ifFalse: [ behavior organization listAtCategoryNamed: category])) do:
				[ :sel | aCollection add: (self createMethodNamed: sel realParent: behavior)]].
	^aCollection.
! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 7/26/2011 11:35'!
allMethodsSelect: aBlock 
	"Answer a SortedCollection of each method that, when used as the block  
	argument to aBlock, gives a true result."
	| aCollection |
	aCollection := SortedCollection new.
	self allBehaviorsDo: [:class | class
						selectorsAndMethodsDo: [:sel :m | (aBlock value: m)
								ifTrue: [aCollection
										add: (self createMethodNamed: sel realParent: class)]]].
	^ aCollection! !

!SystemNavigation methodsFor: 'query' stamp: 'SeanDeNigris 6/21/2012 09:13'!
allMethodsWithSourceString: aString matchCase: caseSensitive
	"Answer a SortedCollection of all the methods that contain, in source code, aString as a substring.  Search the class comments also"

	| list addMethod addComment |
	list := Set new.
	
	addMethod := [ :mrClass :mrSel | list add: (self createMethodNamed: mrSel realParent:  mrClass)].									
	addComment := [ :mrClass | list add: (RGCommentDefinition realClass: mrClass)].
										
	'Searching all source code...'
		displayProgressFrom: 0 to: Smalltalk globals classNames size
		during: [:bar | | count |
			count := 0.
			self allBehaviorsDo: [:each |
				bar current: (count := count + 1).
					each selectorsDo: [:sel | 
						((each sourceCodeAt: sel) includesSubstring: aString caseSensitive: caseSensitive) ifTrue: [
								 addMethod value: each value: sel]].
					(each organization classComment asString includesSubstring: aString caseSensitive: caseSensitive) ifTrue: [
								addComment value: each]	]].
			^ list asSortedCollection! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allObjectsDo: aBlock 
	"Evaluate the argument, aBlock, for each object in the system 
	excluding SmallIntegers."
	| object endMarker |
	object := self someObject.
	endMarker := Object new.
	[endMarker == object]
		whileFalse: [aBlock value: object.
			object := object nextObject]! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allObjectsSelect: aBlock 
	"Evaluate the argument, aBlock, for each object in the system excluding 
	SmallIntegers. Return a collection af all objects for whom the value is 
	true. "
	^ Array
		streamContents: [:s | self
				allObjectsDo: [:object | (aBlock value: object)
						ifTrue: [s nextPut: object]]]! !

!SystemNavigation methodsFor: 'query' stamp: 'GuillermoPolito 4/26/2012 11:21'!
allPrimitiveMethods
	"Answer an OrderedCollection of all the methods that are implemented by primitives."
	| aColl |
	aColl := OrderedCollection new: 200.
	self allBehaviorsDo: [:class | class
		selectorsAndMethodsDo: [:sel :method | 
			method isPrimitive
				ifTrue: [aColl addLast: class name , ' ' , sel , ' ' , method primitive printString]]].
	^ aColl! !

!SystemNavigation methodsFor: 'query' stamp: 'GuillermoPolito 4/26/2012 11:21'!
allPrimitiveMethodsInCategories: aList 
	"Answer an OrderedCollection of all the methods that are implemented by 
	primitives in the given categories. 1/26/96 sw"
	"SystemNavigation new allPrimitiveMethodsInCategories:  
		#('Collections-Streams' 'Files-Streams' 'Files-Abstract' 'Files-Macintosh')"

	| aColl |
	aColl := OrderedCollection new: 200.
	self
		allBehaviorsDo: [:aClass | (aList includes: (SystemOrganization categoryOfElement: aClass theNonMetaClass name asString) asString)
				ifTrue: [aClass
						selectorsAndMethodsDo: [:sel :method | 
							method isPrimitive
								ifTrue: [aColl addLast: aClass name , ' ' , sel , ' ' , method primitive printString]]]].
	^ aColl! !

!SystemNavigation methodsFor: 'query' stamp: 'MarcusDenker 7/12/2012 18:04'!
allReferencesTo: aLiteral 
	"Answer a Collection of all the methods that refer to aLiteral even deeply embedded in literal array."

	"SystemNavigation new allReferencesTo: #+"
	"SystemNavigation new allReferencesTo: Point binding"
	
	| result special byte symbol |
	result := OrderedCollection new.
	special := Smalltalk
		hasSpecialSelector: aLiteral
		ifTrueSetByte: [ :value | byte := value ].
	symbol := aLiteral isSymbol.
	self allBehaviorsDo: [ :class |
		| selectors |
		selectors := symbol
			ifFalse: [ class whichSelectorsReferTo: aLiteral special: special byte: byte ]
			ifTrue: [ class thoroughWhichSelectorsReferTo: aLiteral special: special byte: byte ].
		selectors do: [ :selector |
			result addLast: (self createMethodNamed: selector realParent: class) ] ].
	^ result! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 7/26/2011 13:05'!
allReferencesToPool: aPool
	"Answer all the references to variable aPool"
	
	| list |
	list := OrderedCollection new.
	self allClassesDo: [:cls|
				cls selectorsAndMethodsDo: [:sel :meth| | ref |
					ref := meth literals detect:
							[:lit|
								(lit isVariableBinding 
									and: [(lit key) notNil]) 
										and: [(aPool bindingOf: lit key) notNil ]] ifNone: [nil].
					ref ifNotNil: [ list add: (self createMethodNamed: sel realParent: cls) ]]].
	^list! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 7/26/2011 11:39'!
allReferencesToPool: aPool from: aClass
	"Answer all the references to variables from aPool"
	
	| list |
	list := OrderedCollection new.
	aClass withAllSubclassesDo: [ :cls |
		cls selectorsAndMethodsDo: [ :sel :meth| | ref |
			ref := meth literals detect: [ :lit |
				lit isVariableBinding and:[(aPool bindingOf: lit key) notNil]] ifNone: [nil].
			ref ifNotNil: [ list add: (self createMethodNamed: sel realParent: aClass) ]]].
	^list! !

!SystemNavigation methodsFor: 'query' stamp: 'GuillermoPolito 3/18/2012 21:29'!
allSelect: aBlock 
	"Answer a SortedCollection of each method that, when used as 
	the block  
	argument to aBlock, gives a true result."
	| aCollection |
	aCollection := SortedCollection new.
	self allBehaviorsDo: [:class | class
					selectorsAndMethodsDo: [:sel :meth | (aBlock value: meth)
								ifTrue: [aCollection add: class name , ' ' , sel]]].
	^ aCollection! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allSelectorsWithAnyImplementorsIn: selectorList 
	"Answer the subset of the given list which represent method selectors 
	which have at least one implementor in the system."
	| good |
	good := OrderedCollection new.
	self allBehaviorsDo: [:class | 
			selectorList do: [:aSelector | (class includesSelector: aSelector) ifTrue: [good add: aSelector]]].
	^ good asSet asArray sort! !

!SystemNavigation methodsFor: 'message sends' stamp: 'VeronicaUquillas 8/31/2011 18:41'!
allSendersOf: selector 
	|  sortedSenders special byte |
	sortedSenders := SortedCollection new.
	special := Smalltalk 
				hasSpecialSelector: selector
				ifTrueSetByte: [ :b | byte := b ].
	self allBehaviorsDo: [ :behavior | 
		(behavior
			thoroughWhichSelectorsReferTo: selector
			special: special
			byte: byte) do: [ :sel| sortedSenders add: (self createMethodNamed: sel realParent: behavior) ] ].
	^ sortedSenders! !

!SystemNavigation methodsFor: 'message sends' stamp: ''!
allSendersOf: selector inClass: aClass 
	| sortedSenders special byte |
	sortedSenders := SortedCollection new.
	special := Smalltalk hasSpecialSelector: selector ifTrueSetByte: [:b | byte := b ].
	self 
		addSelectorsReferingTo: selector
		in: aClass
		to: sortedSenders
		special: special
		byte: byte.
	^sortedSenders! !

!SystemNavigation methodsFor: 'message sends' stamp: 'LucFabresse 10/12/2010 08:23'!
allSendersOf: selector inClassCategory: category 
	| classes sortedSenders special byte |
	classes := self environment organization classesInCategory: category.
	sortedSenders := SortedCollection new.
	classes ifEmpty: [ ^ sortedSenders ].
	special := Smalltalk
				hasSpecialSelector: selector
				ifTrueSetByte: [ :b | byte := b ].
	classes do: 
		[ :class | 
		self 
			addSelectorsReferingTo: selector
			in: class
			to: sortedSenders
			special: special
			byte: byte ].
	^ sortedSenders! !

!SystemNavigation methodsFor: 'message sends' stamp: ''!
allSendersOf: selector inPackageNamed: packageName
	^(self categoriesInPackageNamed: packageName) inject: SortedCollection new into: [:sortedSenders :category|
		sortedSenders, (self allSendersOf: selector inClassCategory: category)]! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allSentMessages
	"Answer the set of selectors which are sent somewhere in the system."
	^ self  allSentMessagesWithout: {{}. {}}! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allSentMessagesWithout: classesAndMessagesPair 
	"Answer the set of selectors which are sent somewhere in the system,  
	computed in the absence of the supplied classes and messages."
	| sent absentClasses absentSelectors |
	sent := IdentitySet new: CompiledMethod instanceCount.
	absentClasses := classesAndMessagesPair first.
	absentSelectors := classesAndMessagesPair second.
	self allBehaviorsDo: [:class |
		((absentClasses includes: class)
			ifTrue: [{}]
			ifFalse: [{class. class classSide}]) do: [:each | 
					(absentSelectors isEmpty ifTrue: [each selectors] ifFalse: [each selectors copyWithoutAll: absentSelectors])
					do: [:sel | "Include all sels, but not if sent by self"
						(each compiledMethodAt: sel) literalsDo: [:m | 
								(m isSymbol) ifTrue: ["might be sent"
										m == sel ifFalse: [sent add: m]].
								(m isMemberOf: Array) ifTrue: ["might be performed"
										m do: [:x | (x isSymbol) ifTrue: [x == sel ifFalse: [sent add: x]]]]]]]].
		"The following may be sent without being in any literal frame"
		1
			to: Smalltalk specialSelectorSize
			do: [:index | sent
					add: (Smalltalk specialSelectorAt: index)].
	^ sent! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allUnSentMessagesWithout: classesAndMessagesPair 
	"Answer the set of selectors that are implemented but not sent, computed  
	in the absence of the supplied classes and messages."
	^ (self  allImplementedMessagesWithout: classesAndMessagesPair)
		copyWithoutAll: (self  allSentMessagesWithout: classesAndMessagesPair)! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allUnimplementedCalls
	"Answer an Array of each message that is sent by an expression in a  
	method but is not implemented by any object in the system."
	| aStream all |
	all := self allImplementedMessages.
	aStream := (Array new: 50) writeStream.
	self
		allBehaviorsDo: [:cl | cl
				selectorsAndMethodsDo: [:sel :meth |
					| secondStream | 
					secondStream := (String new: 5) writeStream.
					meth messages
						do: [:m | (all includes: m)
								ifFalse: [secondStream nextPutAll: m;
										 space]].
					secondStream position = 0
						ifFalse: [aStream nextPut: cl name , ' ' , sel , ' calls: ' , secondStream contents]]].
	^ aStream contents! !

!SystemNavigation methodsFor: 'query' stamp: 'GuillermoPolito 4/26/2012 11:21'!
allUnimplementedNonPrimitiveCalls
	"Answer an Array of each message that is sent by an expression in a  
	method but is not implemented by any object in the system."
	| aStream all |
	all := self systemNavigation allImplementedMessages.
	aStream := (Array new: 50) writeStream.
	self systemNavigation
		allBehaviorsDo: [:cl | cl
				selectorsAndMethodsDo: [:sel :meth |
					| secondStream | 
					secondStream := (String new: 5) writeStream.
					meth isPrimitive isFalse: [
						meth messages
							do: [:m | (all includes: m)
									ifFalse: [secondStream nextPutAll: m;
											 space]].
						secondStream position = 0
							ifFalse: [aStream nextPut: cl name , ' ' , sel , ' calls: ' , secondStream contents]]]].
	^ aStream contents! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allUnreferencedClassVariablesOf: aClass
	"Answer a list of the names of all the receiver's unreferenced class  
	vars, including those defined in superclasses"
	| aList |
	aList := OrderedCollection new.
	aClass withAllSuperclasses
		reverseDo: [:aSuperClass | aSuperClass classVarNames
				do: [:var | (self allReferencesTo: (aSuperClass classPool associationAt: var)) isEmpty
						ifTrue: [aList add: var]]].
	^ aList! !

!SystemNavigation methodsFor: 'query' stamp: ''!
allUnsentMessages
	"SystemNavigation new allUnSentMessages"
	"Answer the set of selectors that are implemented by some object in the  
	system but not sent by any."
	^ self allUnSentMessagesWithout: {{}. {}}! !

!SystemNavigation methodsFor: 'query' stamp: 'LucFabresse 10/12/2010 08:39'!
allUnsentMessagesIn: selectorSet 
	"Answer the subset of selectorSet which are not sent anywhere in the 
	system. As allSentMessages does not recurse into nested Arrays, filter
	with #allCallsOn, which does"
	
	^ (selectorSet copyWithoutAll: self allSentMessages) select: [:each | 			(self allCallsOn: each) isEmpty]   ! !

!SystemNavigation methodsFor: 'query' stamp: 'CamilloBruni 5/7/2012 01:02'!
allUnusedClassesWithout: classesAndMessagesPair
	"Enumerates all classes in the system and returns a list of those that are 
	apparently unused. A class is considered in use if it (a) has subclasses  
	or (b) is referred to by some method or (c) has its name in use as a  
	literal."

	"SystemNavigation new unusedClasses"

	| unused |
	self flag: #rewriteMePlease.
	unused := Smalltalk globals classNames asIdentitySet copyWithoutAll: (self allGlobalRefsWithout: classesAndMessagesPair).
	^ unused
		reject: [ :cName | 
			| cl |
			cl := Smalltalk globals at: cName.
			cl subclasses isEmpty not  ]! !

!SystemNavigation methodsFor: 'accessing' stamp: 'LucFabresse 10/11/2010 21:31'!
categoriesInPackageNamed: packageName
	^(self environment organization categoriesMatching: packageName), (self environment organization categoriesMatching: (packageName, '-*'))! !

!SystemNavigation methodsFor: 'author' stamp: 'MarcusDenker 7/12/2012 18:04'!
contributionsOf: aString
	"SystemNavigation new contributionsOf: 'alain.plantec'"
	
	| answer |
	answer := OrderedCollection new.
	self allBehaviorsDo: [ :behavior |
		behavior methodsDo: [ :compiledMethod | | stamp initials |
			stamp := compiledMethod timeStamp.
			stamp notEmpty ifTrue: [
				initials := compiledMethod timeStamp substrings first.
				aString = initials ifTrue: [
					answer add: (compiledMethod selector -> compiledMethod methodClass)]]]].
	^answer! !

!SystemNavigation methodsFor: 'author' stamp: 'MarcusDenker 7/12/2012 18:04'!
contributorsNotSignatories
	"SystemNavigation new contributorsNotSignatories"
	
	| signatories answer |
	answer := Dictionary new.
	signatories := self signatories.
	self allBehaviorsDo: [ :behavior |
		behavior methodsDo: [ :compiledMethod | | stamp initials |
			stamp := compiledMethod timeStamp.
			stamp notEmpty ifTrue: [
				initials :=  compiledMethod timeStamp substrings first.
				(signatories includes: initials) ifFalse: [
					(answer at: initials ifAbsentPut: [OrderedCollection new])
						add: (compiledMethod selector -> compiledMethod methodClass)]]]].
	^answer! !

!SystemNavigation methodsFor: 'private' stamp: 'VeronicaUquillas 9/1/2011 09:30'!
createMethodNamed: aSelector realParent: class

	^ RGMethodDefinition realClass: class selector: aSelector! !

!SystemNavigation methodsFor: 'accessing' stamp: 'LucFabresse 10/11/2010 19:09'!
environment
	^environment ! !

!SystemNavigation methodsFor: 'accessing' stamp: 'LucFabresse 10/11/2010 19:08'!
environment: aSystemDictionary
	^environment :=  aSystemDictionary! !

!SystemNavigation methodsFor: 'private' stamp: 'BenjaminVanRyseghem 5/16/2011 14:15'!
headingAndAutoselectForLiteral: aLiteral do: binaryBlock
	"Evaluate aBlock with either Users of ... or Senders of ... plus the auto-select string
	 for the given literal.  aLiteral can be a Symbol, a VariableBinding or an arbitrary object."

	| autoSelect |
	^aLiteral isSymbol
		ifTrue: [binaryBlock value: 'Senders of ', aLiteral value: aLiteral ]
		ifFalse:
			[autoSelect := aLiteral isVariableBinding
							ifTrue: [aLiteral key]
							ifFalse: [aLiteral printString].
			binaryBlock value: 'Users of ', autoSelect value: autoSelect]! !

!SystemNavigation methodsFor: 'query' stamp: 'MarcusDenker 7/12/2012 18:04'!
hierarchyOfClassesSurrounding: aClass
	"Answer a list of classes in the hierarchy both above and below the given class"
	"SystemNavigation new hierarchyOfClassesSurrounding: StringHolder"
	
	| list aClassNonMeta isMeta |
	aClass ifNil: [^ OrderedCollection new].
	aClass ifNil: [^ self].
	aClassNonMeta := aClass theNonMetaClass.
	isMeta := aClassNonMeta ~~ aClass.
	list := OrderedCollection new.
	aClass allSuperclasses reverseDo:
		[:cl | list addLast: cl].
	aClassNonMeta allSubclassesWithLevelDo:
		[:cl :level | | theClassOrMeta |
		theClassOrMeta := isMeta ifTrue: [cl class] ifFalse: [cl].
		list addLast: theClassOrMeta]
	 	startingLevel: 0.
	^ list

! !

!SystemNavigation methodsFor: 'query' stamp: 'MarcusDenker 7/12/2012 18:04'!
hierarchyOfImplementorsOf: aSelector forClass: aClass
	"Answer a list of classes in the hierarchy both above and below the given class which implement the given selector."
	"SystemNavigation new hierarchyOfImplementorsOf: #contents forClass: StringHolder"

	^ (self hierarchyOfClassesSurrounding: aClass) select:
		[:cl | cl includesSelector: aSelector]
! !

!SystemNavigation methodsFor: 'initialize' stamp: 'StephaneDucasse 10/23/2010 15:14'!
initialize

	super initialize.
	self environment: Smalltalk globals! !

!SystemNavigation methodsFor: 'message sends' stamp: ''!
isMessage: selector sentInClass: aClass 
	^(self allSendersOf: selector inClass: aClass) notEmpty ! !

!SystemNavigation methodsFor: 'message sends' stamp: ''!
isMessage: selector sentInClassCategory: category 
	^(self allSendersOf: selector inClassCategory: category) notEmpty ! !

!SystemNavigation methodsFor: 'message sends' stamp: ''!
isMessage: selector sentInPackageNamed: packageName 
	^(self allSendersOf: selector inPackageNamed: packageName) notEmpty ! !

!SystemNavigation methodsFor: 'query' stamp: ''!
isThereAnImplementorOf: aSelector 
	"Answer true if there is at least one implementor of the selector found  
	in the system, false if there are no implementors"
	"self new isThereAnImplementorOf: #contents.  
	self new isThereAnImplementorOf: #nobodyImplementsThis."
	self 
		allBehaviorsDo: [:class | (class includesSelector: aSelector)
				ifTrue: [^ true]].
	^ false! !

!SystemNavigation methodsFor: 'message sends' stamp: ''!
isUnsentMessage: selector

	| special byte |
	special := Smalltalk 
				hasSpecialSelector: selector
				ifTrueSetByte: [ :b | byte := b ].
	self allBehaviorsDo: 
		[ :behavior | 
			(behavior 
				thoroughHasSelectorReferringTo: selector
				special: special
				byte: byte) ifTrue: [^false] ].
	^true! !

!SystemNavigation methodsFor: 'utilities' stamp: 'MarcusDenker 7/12/2012 18:05'!
methodsWithInitials: targetInitials
	"Based on a do-it contributed to the Squeak mailing list by Goran Hultgen:
 Browse methods whose initials (in the time-stamp, as logged to disk) match the given initials.
 Print out the complete time-stamp table to the Transcript.
 Answer a list of (initials -> count) associations.

CAUTION: It may take several minutes for this to complete."

	"Time millisecondsToRun: [SystemNavigation new methodsWithInitials: 'bf'] "

	| initials timeStamp  allSubmitters |
	initials := ''.
	timeStamp := ''.
	allSubmitters := Bag new.
	self browseAllSelect:
			[:cm |
				timeStamp := cm timeStamp.
				initials := timeStamp isEmpty
					ifTrue:
						['']
					ifFalse:
						[timeStamp substrings first].
				initials := initials isEmpty
					ifTrue:
						['<no time stamp>']
					ifFalse:
						[initials first isDigit
							ifTrue:
								['<date>']
							ifFalse:
								[initials]].
				allSubmitters add: initials.
				(initials = targetInitials)]
		name: ('Methods with initials ', targetInitials)
		autoSelect: nil.

	allSubmitters sortedCounts do: [:elem | Transcript cr; show: elem asString].
	^ allSubmitters
! !

!SystemNavigation methodsFor: 'query' stamp: 'mada 5/4/2012 19:54'!
methodsWithUnboundGlobals
	"This says that for any global, it should match either the class's notion of what bindingOf: the key is, or bindingOf: should be nil and the binding should be in Undeclared. If the class answers a different binding through bindingOf: or answers no binding and the binding is not in Undeclared then the variable in the method is wrong.
	For a clean image the result should be empty. If it is not empty, evaluating Compiler recompileAll probably solves the problem. However, we should investigate why the method gets an incorrect state."

	"SystemNavigation new methodsWithUnboundGlobals"

	^self allSelect:
		[:m|
		m literals anySatisfy:
			[:l|
			l isVariableBinding
			and: [l key isSymbol "avoid class-side methodClass literals"
			and: [(m methodClass bindingOf: l key)
					ifNil: [(Undeclared associationAt: l key ifAbsent: []) ~~ l]
					ifNotNil: [:b| b ~~ l]]]]]! !

!SystemNavigation methodsFor: 'query' stamp: ''!
numberOfImplementorsOf: aSelector 
	"Answer a count of the implementors of the given selector found in the  
	system"
	"self new numberOfImplementorsOf: #contents.  
	self new numberOfImplementorsOf: #nobodyImplementsThis.  
	self new numberOfimplementorsOf: #numberOfImplementorsOf:."
	| aCount |
	aCount := 0.
	self
		allBehaviorsDo: [:class | (class includesSelector: aSelector)
				ifTrue: [aCount := aCount + 1]].
	^ aCount! !

!SystemNavigation methodsFor: 'query' stamp: 'MarcusDenker 7/12/2012 18:05'!
obsoleteBehaviors
	"SystemNavigation new obsoleteBehaviors inspect"
	"Find all obsolete behaviors including meta classes"

	| obs |
	obs := OrderedCollection new.
	Smalltalk garbageCollect.
	self 
		allObjectsDo: [:cl | (cl isBehavior
					and: [cl isObsolete])
				ifTrue: [obs add: cl]].
	^ obs asArray! !

!SystemNavigation methodsFor: 'query' stamp: 'MarcusDenker 7/12/2012 18:05'!
obsoleteClasses   

	"SystemNavigation new obsoleteClasses inspect"
	"NOTE:  Also try inspecting comments below"
	| obs |
	obs := OrderedCollection new.  Smalltalk garbageCollect.
	Metaclass allInstancesDo:
		[:m | | c | c := m soleInstance.
		(c ~~ nil and: ['AnOb*' match: c name asString])
			ifTrue: [obs add: c]].
	^ obs asArray

"Likely in a ClassDict or Pool...
(Association allInstances select: [:a | (a value isKindOf: Class) and: ['AnOb*' match: a value name]]) asArray
"
"Obsolete class refs or super pointer in last lit of a method...
| n l found |
Smalltalk browseAllSelect:
	[:m | found := false.
	1 to: m numLiterals do:
		[:i | (((l := m literalAt: i) isMemberOf: Association)
				and: [(l value isKindOf: Behavior)
				and: ['AnOb*' match: l value name]])
			ifTrue: [found := true]].
	found]
"! !

!SystemNavigation methodsFor: 'query' stamp: 'MarcusDenker 7/12/2012 18:05'!
obsoleteMethodReferences
	"SystemNavigation new obsoleteMethodReferences"

	"Open a browser on all referenced behaviors that are obsolete"

	| obsClasses references |
	references := Array new writeStream.
	obsClasses := self obsoleteBehaviors.
	'Scanning for methods referencing obsolete classes' 
		displayProgressFrom: 0 to: obsClasses size
		during: 
			[:bar | 
			obsClasses keysAndValuesDo: 
					[:index :each | | obsRefs | 
					bar current: index.
					obsRefs := each pointersToExcept: obsClasses.
					obsRefs do: 
							[:ref | 
							"Figure out if it may be a global"

							(ref isVariableBinding and: [ref key isString	"or Symbol"]) 
								ifTrue: 
									[(ref pointersTo) do: 
											[:meth | 
											(meth isKindOf: CompiledMethod) 
												ifTrue: [meth methodReference ifNotNil: [:mref | references nextPut: mref]]]]]]].
	^references contents! !

!SystemNavigation methodsFor: 'packages' stamp: 'CamilleTeruel 7/29/2012 18:49'!
removeAllButPackages: packagesList
  "Remove all classes and methods that do not belong to some of listed packages. Skip #unload. Do not use this method if you can use Monticello!! This method is used by some low-level shrinking process. Do not use it. This is quite hackish"

  | keep remove oldRemove extensionCategories log |
  keep := Set new.      
  packagesList do: [:pkg | keep addAll: (self allClassesInPackageNamed: pkg)].
  oldRemove := Set new.
  [ 
    remove := Smalltalk globals allClassesAndTraits reject: [:c | keep includes: c].
    remove := remove collect: [:c | c name].
    (remove asSortedCollection: [:a :b | a  <= b ]) do: [:className |
      | class |
      class := (Smalltalk globals classOrTraitNamed: className).
      class allSubclasses ifEmpty: [
        SystemAnnouncer uniqueInstance unsubscribe: class.
        class isTrait ifFalse: [ 
          (class class includesSelector: #unload) ifTrue: [ class removeSelector: #unload ] ].
        class removeFromSystem. 
        ] ].

    
    oldRemove ifNotEmpty: [
      oldRemove = remove ifTrue: [          
         self error: 'Some classes still have subclasses: ', (remove collect: [:c | c -> (Smalltalk classOrTraitNamed: c) allSubclasses]) asString ] ].
    oldRemove := remove.
    remove notEmpty] whileTrue.

  keep do: [: class | 
    extensionCategories := (class organization ownCategories select: [:category | 
      category first = $*]) asOrderedCollection.
    extensionCategories removeAllSuchThat: [:category |
      packagesList anySatisfy: [:package |
        ((category asLowercase allButFirst = package asLowercase) or: [package asLowercase, '-*' match: category asLowercase allButFirst])]].
    extensionCategories do: [:category | 
      class removeCategory: category ]. 
    class isTrait ifFalse: [
      extensionCategories := (class class organization ownCategories select: [:category | 
        category first = $*]) asOrderedCollection.
      extensionCategories removeAllSuchThat: [:category |
        packagesList anySatisfy: [:package |
          ((category asLowercase allButFirst = package asLowercase) or: [package asLowercase, '-*' match: category asLowercase allButFirst])]].
      extensionCategories do: [:category | 
        class class removeCategory: category ]]].
      
! !

!SystemNavigation methodsFor: 'query' stamp: 'StephaneDucasse 7/26/2011 12:42'!
selectAllMethods: aBlock 
	"Answer a SortedCollection of each method that, when used as the block  
	argument to aBlock, gives a true result."
	
	| aCollection |
	aCollection := SortedCollection new.
	self allBehaviorsDo: [:class | class
				selectorsAndMethodsDo: [ :sel :m | (aBlock value: m)
						ifTrue: [ aCollection add: (self createMethodNamed: sel realParent: class)]]].
	^ aCollection! !

!SystemNavigation methodsFor: 'author' stamp: 'MarcusDenker 7/12/2012 18:05'!
signatories
	"Answer the initials corresponding to the people who have signed the license"
	"SystemNavigation new signatories"
	
	^(self signatoriesString subStrings: {Character cr})
		collect: [ :each | each substrings first]! !

!SystemNavigation methodsFor: 'author' stamp: 'AlexandreBergel 12/29/2010 11:40'!
signatoriesString
	^'AB               Alexandre Bergel
ack              Alan Kay
ads              Adam Spitz
ajh              Anthony Hannan
aka              Mark Guzdial
al               Adrian Lienhard
aoy              Andres Otaduy
apb              Andrew P. Black
apl              Alain Plantec
ar               Andreas Raab
asm              Alejandro Magistrello
avi              Avi Bryant
bf               Bert Freudenberg
BG               Boris Gaertner
bh               Bob Hartwig
BJP              Bijan Parsia
bkv              Brent Vukmer
bolot            Bolot Kerimbaev
bootstrap        Pavel Krivanek
BP               Bijan Parsia
brp              Brent Pinkney
btr              Brian Rice
cbc              Chris Cunningham
ccn              Chris Norton
CdG              Cees de Groot
cds              C. David Shaffer
cmm              Chris Muller
crl              Craig Latta
cwp              Colin Putney
daf              Dave Faught
dao              danil osipchuk
DAS              David A Smith
dc               Damien Cassou
dd               Dominique Dutoit
dew              Doug Way
DF               Diego Fernandez
dgd              Diego Gomez Deck
dhhi             Dan Ingalls
di               Dan Ingalls
DM               Duane Maxwell
DSM              Duane Maxwell
dtl              Dave Lewis
dvf              Daniel Vainsencher
edc              Edgar DeCleene
efc              Eddie Cottongim
efo              Emilio Oca
em               Ernest Micklei?
emm              Ernest Micklei
es               Enrico Spinielli
FBS              Frank Shearar
fc               Frank Caggiano
fcs              Frank Sergeant
fm               Florin Mateoc
gh               Goran Krampe (nee Hultgren)
gk               Goran Krampe (nee Hultgren)
gm               German Morales
go               Georg Gollmann
gsa              German Arduino
HEG              Henrik Ekenberg
HilaireFernandes Hilaire Fernandes
HK               Herbert Konig
hmm              Hans-Martin Mosner
hpt              Hernan Tylim
huma             Lyndon Tremblay
ich.             Yuji Ichikawa
ikp              Ian Piumarta
jaf              Jan Fietz
jam              Javier Musa
jcg              Joshua Gargus
jdf              David Farber
jdr              Javier Diaz-Reinoso
je               Jorn Eyrich
je77             Jochen Rick
JF               Julian Fitzell
jhm              John Maloney
jlb              Jim Benson
jmb              Hans Baveco
JMM              John McIntosh
JMV              Juan Manuel Vuletich
jon              Jon Hylands
jrm              John-Reed Maffeo
jrp              John Pierce
jsp              Jeff Pierce
ka               Kazuhiro Abe
kfr              Karl Ramberg
KLC              Ken Causey
KR               korakurider
KTT              Kurt Thams
kwl              Klaus D. Witzel
ky               Koji Yokokawa
laza             Alexander Lazarevic
LB               Leo Burd
LC               Leandro Caniglia
LEG              Gerald Leeb
len              Luciano Esteban Notarfrancesco
lr               Lukas Renggli
lrs              Lorenzo Schiavina
ls               Lex Spoon
m3r              Maurice Rabb
mas              Mark Schwenk
MD               Markus Denker
mdr              Mike Rutenberg
mga              Markus Galli
miki             Mikael Kindborg
mikki            Mikael Kindborg
mir              Michael Rueger
mist             Michal Starke
mjg              Mark Guzdial
mjr              Mike Roberts
mjt              Mike Thomas
mk               Matej Kosik
MPW              Marcel Weiher
mrm              Martin McClure
ms               math
MU               Masashi Umezawa
mw               Martin Wirblat
nb               Naala Brewer
nice             Nicolas Cellier
nk               Ned Konz
Noury            Noury Bouraqadi
NS               Nathanael Schaerli
PHK              Peter Keeler
pk               Pavel Krivanek
pm               Patrick Mauritz
pnm              Paul McDonough
RAA              Bob Arning
raok             Richard A. O"Keefe
rbb              Brian Brown
rca              Russell Allen
reThink          Paul McDonough
rew              Roger Whitney
rhi              Robert Hirschfeld
Rik              Rik Fischer SmOOdy
RJT              Ron Teitelbaum
rr               Romain Robbes
rw               Roel Wuyts
rww              Robert Withers
sbw              Stephan B. Wessels
SD               Stephane Ducasse
sge              Steve Elkins
shrink           Pavel Krivanek
slg              Steve Gilbert
sm               Simon Michael
sma              Stefan Matthias Aust
sn               Suslov Nikolay
spfa             Stephane Rollandin
sps              Steven Swerling
SqR              Andres Valloud
SqR!!!!            Andres Valloud
SqR!!!!!!!!          Andres Valloud
sr               Stephan Rudlof
ssa              Sam S. Adams
st               Samuel Tardieu
stephaneducasse  Stephane Ducasse
stp              Stephen Travis Pope
sumim            Masato Sumi
svp              Stephen Vincent Pair
sw               Scott Wallace
T2               Toshiyuki Takeda
tak              Takashi Yamamiya
tao              Tim Olson
tb               Todd Blanchard
TBn              Torsten Bergmann
tetha            Tetsuya Hayashi
tfei             The Fourth Estate, Inc.
th               Torge Husfeldt
ti               Tobias Isenberg
TJ               TJ Leone
tk               Ted Kaehler
tk               Thomas Kowark
tlk              Tom Koenig
TN               korakurider
TPR              Tim Rowledge
TRee             Trygve Reenskaug
Tsutomu          Tsutomu Hiroshima
tween            Andy Tween
vb               Vassili Bykov
vj               Vladimir Janousek
wiz              Jerome Peace
wod              Bill Dargel
ykoubo           Koji Yokokawa
yo               Yoshiki Ohshima
zz               Serge Stinckwich
jmv
jm
jmm
md 			Marcus Denker
sd  			Stephane Ducasse
tpr'! !

!SystemNavigation methodsFor: 'query' stamp: ''!
unimplemented
	"Answer an Array of each message that is sent by an expression in a method but is not implemented by any object in the system."
	
	| all unimplemented |
	all := IdentitySet new: Symbol instanceCount * 2.
	self allBehaviorsDo: [:cl | cl selectorsDo: [:aSelector | all add: aSelector]].

	unimplemented := IdentityDictionary new.
	self allBehaviorsDo: [:cl |
		 cl selectorsAndMethodsDo: [:sel :meth |
			meth messages do: [:m | | entry |
				(all includes: m) ifFalse: [
					entry := unimplemented at: m ifAbsent: [Array new].
					entry := entry copyWith: (cl name, '>', sel).
					unimplemented at: m put: entry]]]].
	^ unimplemented
! !

!SystemNavigation methodsFor: 'message sends' stamp: 'LucFabresse 10/12/2010 08:25'!
unsentMessagesInCategory: category
	^(self environment organization classesInCategory: category) inject: Set new into: [:unsentMessages :class|
		unsentMessages, (self unsentMessagesInClass: class)]
	! !

!SystemNavigation methodsFor: 'message sends' stamp: 'StephaneDucasse 7/26/2011 15:14'!
unsentMessagesInClass: aClass 

	| methReferences |
	methReferences := Set new.
	aClass selectorsDo: [:selector|
		(self isUnsentMessage: selector) ifTrue: [
			methReferences add: (self createMethodNamed: selector realParent: aClass)]].
	^methReferences! !

!SystemNavigation methodsFor: 'message sends' stamp: 'LucFabresse 10/12/2010 08:28'!
unsentMessagesInPackageNamed: packageName
	| unsentMessages |
	unsentMessages := self unsentMessagesInCategory: packageName.	
	(self categoriesInPackageNamed: packageName) do: [:category|
		unsentMessages addAll: (self unsentMessagesInCategory: category)].
	^unsentMessages 
	! !

Object subclass: #SystemVersion
	instanceVariableNames: 'version date highestUpdate updates type major minor suffix'
	classVariableNames: 'Current'
	poolDictionaries: ''
	category: 'System-Support'!

!SystemVersion commentStamp: 'StephaneDucasse 12/8/2010 13:50' prior: 0!
I am responsible for maintaining what version of Squeak and the VM is running.  I also track all of the update items that have been included in the image.

I'm invoked at auto start to get the latest plugins, etc.

Some queries are
	SystemVersion current major
	SystemVersion current minor
	SystemVersion current suffix
	SystemVersion current	highestUpdate!

!SystemVersion class methodsFor: 'accessing' stamp: ''!
current
	Current ifNil: [Current := SystemVersion new].
	^Current! !

!SystemVersion class methodsFor: 'instance creation' stamp: ''!
fromShortVersionString: aString
	"Create a new SystemVersion from the given short version string. Useful for testing."

	| parts |
	parts := self parseShortVersionString: aString.
	^self new
		type: parts first;
		major: parts second;
		minor: parts third;
		suffix: parts fourth;
		yourself! !

!SystemVersion class methodsFor: 'instance creation' stamp: ''!
newVersion: versionName
	"Install a new SystemVersion as the current version. Only preserve the higest update recorded.
	This method is a candidate for being removed, I am not sure it ever makes sense to do this."
	
	| newVersion |
	newVersion := self fromShortVersionString: versionName.
	newVersion
		highestUpdate: self current highestUpdate.
	Current := newVersion
! !

!SystemVersion class methodsFor: 'accessing' stamp: ''!
parseShortVersionString: aString 
	"Answer the parts of aString, expected in short format <type><major>.<minor><suffix>."

	| stream length type major minor suffix |

	stream := aString readStream.
	[stream peek isDigit] whileFalse: [stream next].
	length := stream position.
	stream reset.
	type := (stream next: length).
	major := (stream upTo: $.) asInteger.
	stream := stream upToEnd readStream.
	[stream peek notNil and: [stream peek isDigit]] whileTrue: [stream next].
	length := stream position.
	stream reset.
	minor := (stream next: length) asInteger.
	suffix := stream upToEnd.
	^{type. major. minor. suffix}! !

!SystemVersion class methodsFor: 'accessing' stamp: ''!
parseVersionString: versionString 
	"Answer the version of this release as version, date, update."
	"SystemVersion parseVersionString: 'Squeak3.1alpha of 28 February 2001 [latest update: #3966]' "
	| version date update |
	
	[ | stream | stream := versionString readStream.
	version := stream upToAll: ' of '.
	date := Date readFrom: stream.
	stream upToAll: ' #'.
	update := Number readFrom: stream ] 
		on: Error
		do: [ ^ nil ].
	^ {  version. date. update  }! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
date
	^date! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
date: newDate
	date := newDate! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
datedVersion
	"Answer the version of this release."

	^ self version asString , ' of ' , self date printString! !

!SystemVersion methodsFor: 'accessing' stamp: 'StephaneDucasse 2/1/2011 07:11'!
dottedMajorMinor
	"Return the major/minor version number of the form X.Y, without any 'alpha' or 'beta' or other suffix. 
	If the receiver needs upgrading, do it. Remove the guard in 1.3."
	"SystemVersion current majorMinor"
	
	self upgrade.
	^String streamContents: [:str |
		str 
			nextPutAll: self major printString; nextPut: $.;
			nextPutAll: self minor printString]! !

!SystemVersion methodsFor: 'accessing' stamp: 'StephaneDucasse 12/8/2010 13:50'!
highestUpdate
	"Returns the highest number of available updates"

	| sortedUpdates |
	highestUpdate ifNil: [
		sortedUpdates := self updates asSortedCollection.
		highestUpdate := (sortedUpdates isEmpty
			ifTrue: [0]
			ifFalse: [sortedUpdates last])].
	^highestUpdate! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
highestUpdate: anInteger
	highestUpdate := anInteger! !

!SystemVersion methodsFor: 'printing' stamp: ''!
imageVersionString
	"Print the version compatibile with naming the image."
	
	^String streamContents: [:str |
		str nextPutAll: self type;
			nextPut: $-;
			nextPutAll: self major printString;
			nextPut: $.;
			nextPutAll: self minor printString;
			nextPut: $-;
			nextPutAll: self highestUpdate printString;
			nextPut: $-;
			nextPutAll: self suffix]! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
includesUpdate: anUpdate
	^self updates includes: anUpdate! !

!SystemVersion methodsFor: 'initialize' stamp: ''!
initialize
	super initialize.
	date := Date today.
	updates := Set new.
! !

!SystemVersion methodsFor: 'accessing' stamp: 'StephaneDucasse 12/8/2010 13:49'!
major
	"Answer the major number of a version. 1 in '1.2'"

	^ major! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
major: anObject
	"Set the value of major"

	major := anObject! !

!SystemVersion methodsFor: 'accessing' stamp: 'StephaneDucasse 2/1/2011 07:16'!
majorMinor
	"Return the major/minor version number of the form X.Y, without any 'alpha' or 'beta' or other suffix. 
	If the receiver needs upgrading, do it. Remove the guard in 1.3."
	"SystemVersion current majorMinor"
	
	^ self majorMinor: '.'! !

!SystemVersion methodsFor: 'accessing' stamp: 'StephaneDucasse 2/1/2011 07:15'!
majorMinor: aSeparatingString
	"Return the major/minor version number of the form XaSeparatingStringY, without any 'alpha' or 'beta' or other suffix. If the receiver needs upgrading, do it. Remove the guard in 1.3."
	"SystemVersion current majorMinor: '-'"
	"SystemVersion current majorMinor: ''"
	
	self upgrade.
	^String streamContents: [:str |
		str 
			nextPutAll: self major printString; 
			nextPutAll: aSeparatingString;
			nextPutAll: self minor printString]! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
majorMinorVersion
	"Return the major/minor version number of the form X.Y, without any 'alpha' or 'beta' or other suffix. 
	If the receiver needs upgrading, do it. Remove the guard in 1.3."
	"SystemVersion current majorMinorVersion"
	
	self upgrade.
	^String streamContents: [:str |
		str 
			nextPutAll: self type;
			nextPutAll: self major printString; nextPut: $.;
			nextPutAll: self minor printString]! !

!SystemVersion methodsFor: 'accessing' stamp: 'StephaneDucasse 12/8/2010 13:50'!
minor
	"Answer the minor number of a version. 2 in '1.2'"

	^ minor! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
minor: anObject
	"Set the value of minor"

	minor := anObject! !

!SystemVersion methodsFor: 'printing' stamp: ''!
printOn: stream
	stream
		nextPutAll: self datedVersion;
		nextPutAll: ' update ' , self highestUpdate printString! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
registerUpdate: update
	self updates add: update.
	self resetHighestUpdate! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
resetHighestUpdate
	highestUpdate := nil! !

!SystemVersion methodsFor: 'printing' stamp: ''!
shortVersionString
	"Print a short version string.  Compatibile with senders that concatenate this with 
	#lastUpdateString or #highestUpdate. Essentially the old version format. Guard 
	against no state being set."
	
	self type ifNil: [^'No version set'].
	^String streamContents: [:str |
		str 
			nextPutAll: self type;
			nextPutAll: self major printString;
			nextPut: $.;
			nextPutAll: self minor printString;
			nextPutAll: self suffix]! !

!SystemVersion methodsFor: 'accessing' stamp: 'StephaneDucasse 12/8/2010 13:50'!
suffix
	"Answer the suffix (alpha, beta, rc  of a version. beta in '1.2beta'"

	^ suffix! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
suffix: anObject
	"Set the value of suffix"

	suffix := anObject! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
type
	"Answer the value of type"

	^ type! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
type: anObject
	"Set the value of type"

	type := anObject! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
unregisterUpdate: update
	self updates remove: update ifAbsent: []! !

!SystemVersion methodsFor: 'accessing' stamp: ''!
updates
	^updates! !

!SystemVersion methodsFor: 'upgrading' stamp: ''!
upgrade

	| parts |
	version ifNil: [^self].
	parts := self class parseShortVersionString: version.
	self type: parts first.
	self major: parts second.
	self minor: parts third.
	self suffix: parts fourth.
	version := nil.! !

!SystemVersion methodsFor: 'accessing' stamp: 'MarcusDenker 4/14/2011 14:09'!
version
	"Answer the short version string. If the receiver needs upgrading, do it."
	
	^self shortVersionString! !

Object subclass: #TextDiffBuilder
	instanceVariableNames: 'xLines yLines'
	classVariableNames: 'IgnoreLineEndings InsertTextAttributes NormalTextAttributes RemoveTextAttributes'
	poolDictionaries: ''
	category: 'System-FilePackage'!

!TextDiffBuilder commentStamp: 'HenrikSperreJohansen 5/21/2010 02:00' prior: 0!
I implement the diff algorithm. I can show the differences between two texts. See my method comments for further information.

Instance Variables
	xLines:		<Array>
	yLines:		<Array>

xLines
	- an Array of DiffElement which is created from the first input text

yLines
	- an Array of DiffElement which is created from the second input text!

TextDiffBuilder subclass: #ClassDiffBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-FilePackage'!

!ClassDiffBuilder commentStamp: 'HenrikSperreJohansen 5/21/2010 02:06' prior: 0!
I'm like TextDiffBuilder, but I split the input text by Character >> #separators, instead of new lines. 
I'm probably ment to create diffs of class definitions.!

!ClassDiffBuilder methodsFor: 'private' stamp: 'HenrikSperreJohansen 5/21/2010 02:05'!
print: aString withAttributes: attributes on: stream

	stream
		withAttributes: attributes 
		do: [ stream nextPutAll: aString ]! !

!ClassDiffBuilder methodsFor: 'private' stamp: 'HenrikSperreJohansen 5/21/2010 02:04'!
split: aString
	"I return an array with aString splitted by Character >> #separators."

	^Array streamContents: [ :stream |
		| out |
		out := aString copy writeStream.
		aString do: [ :c |
			out nextPut: c.
			c isSeparator ifTrue:[
				stream nextPut: out contents.
				out reset ] ].
		out position = 0 ifFalse: [ 
			stream nextPut: out contents ] ]! !

TextDiffBuilder subclass: #PrettyTextDiffBuilder
	instanceVariableNames: 'sourceClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-FilePackage'!

!PrettyTextDiffBuilder commentStamp: 'HenrikSperreJohansen 5/21/2010 01:42' prior: 0!
I'm like TextDiffBuilder, but I use the pretty-printed version of the source code if available.

Instance Variables
	sourceClass:		<Class>

sourceClass
	- this class provides the pretty-printer
!

!PrettyTextDiffBuilder class methodsFor: 'instance creation' stamp: 'nk 10/29/2000 12:35'!
from: srcString to: dstString inClass: srcClass 
	^ (self new sourceClass: srcClass) from: srcString to: dstString
		! !

!PrettyTextDiffBuilder methodsFor: 'initialize' stamp: 'nk 10/29/2000 12:16'!
sourceClass: aClass
	sourceClass := aClass.! !

!PrettyTextDiffBuilder methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 1/8/2012 14:47'!
split: aString 
	| formatted trimmed |
	trimmed := aString asString trimBoth.
	trimmed isEmpty ifTrue: [ ^super split: '' ].
	formatted := [ sourceClass prettyPrinterClass
				format: trimmed
				in: sourceClass
				notifying: nil] on: Error do: [ :ex | trimmed ].
	^ super split: formatted! !

!TextDiffBuilder class methodsFor: 'instance creation' stamp: 'HenrikSperreJohansen 5/21/2010 01:38'!
buildDisplayPatchFrom: sourceText to: destinationText

	^(self from: sourceText to: destinationText) buildDisplayPatch! !

!TextDiffBuilder class methodsFor: 'instance creation' stamp: 'HenrikSperreJohansen 5/21/2010 01:38'!
buildDisplayPatchFrom: sourceText to: destinationText inClass: sourceClass 

	^self 
		buildDisplayPatchFrom: sourceText 
		to: destinationText
		inClass: sourceClass
		prettyDiffs: CodeHolder diffsWithPrettyPrint
		! !

!TextDiffBuilder class methodsFor: 'instance creation' stamp: 'HenrikSperreJohansen 5/21/2010 01:39'!
buildDisplayPatchFrom: srcString to: dstString inClass: srcClass prettyDiffs: prettyDiffs 
	"Build a display patch for mapping via diffs from srcString to dstString
	in the given class. If prettyDiffs is true, do the diffing for pretty-printed forms"

	| differ |
	differ := prettyDiffs
				ifTrue: [PrettyTextDiffBuilder]
				ifFalse: [self].
	^ (differ
		from: srcString
		to: dstString
		inClass: srcClass) buildDisplayPatch! !

!TextDiffBuilder class methodsFor: 'instance creation' stamp: 'HenrikSperreJohansen 5/21/2010 01:39'!
from: srcString to: dstString

	^self new from: srcString to: dstString! !

!TextDiffBuilder class methodsFor: 'instance creation' stamp: 'HenrikSperreJohansen 5/21/2010 01:39'!
from: srcString to: dstString inClass: srcClass
	"srcClass not used when not pretty printing, but have it for polymorphism with PrettyTextDiffBuilder"

	^self new from: srcString to: dstString! !

!TextDiffBuilder class methodsFor: 'settings' stamp: 'HenrikSperreJohansen 5/21/2010 01:21'!
ignoreLineEndings
	"Answer a boolean telling if line endings differences should be ignored or emphasized"
	
	^IgnoreLineEndings ifNil: [ true ]! !

!TextDiffBuilder class methodsFor: 'settings' stamp: 'HenrikSperreJohansen 5/21/2010 01:21'!
ignoreLineEndings: aBoolean
	"Setting for  if line ending differences should be ignored or emphasized"
	
	IgnoreLineEndings := aBoolean! !

!TextDiffBuilder class methodsFor: 'class initialization' stamp: 'HenrikSperreJohansen 5/21/2010 01:20'!
initialize

	self initializeTextAttributes! !

!TextDiffBuilder class methodsFor: 'class initialization' stamp: 'HenrikSperreJohansen 5/21/2010 01:20'!
initializeTextAttributes

	InsertTextAttributes :=  {TextEmphasis bold. TextColor color: Color green muchDarker}.
	RemoveTextAttributes := {TextEmphasis bold. TextColor color: Color red darker} .
	NormalTextAttributes :={ TextEmphasis normal }! !

!TextDiffBuilder class methodsFor: 'settings' stamp: 'HenrikSperreJohansen 5/21/2010 02:38'!
insertedTextAttributes

	^InsertTextAttributes! !

!TextDiffBuilder class methodsFor: 'settings' stamp: 'HenrikSperreJohansen 5/21/2010 02:40'!
insertedTextAttributes: anArrayOfTextAttributes
	"{ TextEmphasis. TextColor ]"

	InsertTextAttributes := anArrayOfTextAttributes! !

!TextDiffBuilder class methodsFor: 'settings' stamp: 'HenrikSperreJohansen 5/21/2010 02:40'!
removedTextAttributes

	^RemoveTextAttributes! !

!TextDiffBuilder class methodsFor: 'settings' stamp: 'HenrikSperreJohansen 5/21/2010 02:40'!
removedTextAttributes: anArrayOfTextAttributes
	"{ TextEmphasis. TextColor ]"

	RemoveTextAttributes := anArrayOfTextAttributes! !

!TextDiffBuilder methodsFor: 'creating patches' stamp: 'HenrikSperreJohansen 5/21/2010 01:54'!
buildDisplayPatch

	^Text streamContents: [ :stream |
		self 
			patchSequenceDoIfMatch: [ :string |
				self print: string withAttributes: NormalTextAttributes on: stream ]
			ifInsert: [ :string |
				self print: string withAttributes: InsertTextAttributes on: stream ]
			ifRemove: [ :string |
				self print: string withAttributes: RemoveTextAttributes on: stream ] ]! !

!TextDiffBuilder methodsFor: 'creating patches' stamp: 'HenrikSperreJohansen 5/21/2010 01:54'!
buildPatchSequence
	"This method is only implemented for backwards compatibility and testing."

	^Array streamContents: [ :stream |
		self 
			patchSequenceDoIfMatch: [ :string |
				stream nextPut: #match -> (string copyWithout: Character cr) ]
			ifInsert: [ :string | 
				stream nextPut: #insert -> (string copyWithout: Character cr) ]
			ifRemove: [ :string | 
				stream nextPut: #remove -> (string copyWithout: Character cr) ] ]! !

!TextDiffBuilder methodsFor: 'private' stamp: 'ul 2/8/2011 03:04'!
findMatches
	"I find the matching pairs of xLines and yLines. First I filter out all lines that can't have a pair, then I find the longest common subsequence of the remaining elements. Finally I mark the matching pairs."

	| temp lcs xFilteredLines yFilteredLines xNumbers yNumbers |
	"Filter out all lines that can't have a pair."
	temp := yLines asSet.
	xFilteredLines := xLines select: [ :each |
		temp includes: each ].
	xFilteredLines size = 0 ifTrue: [ ^self ].
	temp := xLines asSet.
	yFilteredLines := yLines select: [ :each |
		temp includes: each ].
	yFilteredLines size = 0 ifTrue: [ ^self ].
	"Map all lines to SmallIntegers, because they can be compared faster."
	temp := Dictionary new.
	xNumbers := xFilteredLines collect: [ :each |
		temp at: each ifAbsentPut: [ temp size ] ].
	yNumbers := yFilteredLines collect: [ :each |
		temp at: each ifAbsentPut: [ temp size ] ].
	temp := nil.
	"Find the longest common subsequence."
	lcs := self lcsFor: xNumbers and: yNumbers.
	"Mark the matching pairs."
	[ lcs == nil ] whileFalse: [
		(xFilteredLines at: (lcs at: 1)) matches: (yFilteredLines at: (lcs at: 2)).
		lcs := lcs at: 3 ]! !

!TextDiffBuilder methodsFor: 'initialize' stamp: 'HenrikSperreJohansen 5/21/2010 01:46'!
from: xString to: yString

	xLines := (self split: xString asString) replace: [ :each | DiffElement string: each ].
	yLines := (self split: yString asString) replace: [ :each | DiffElement string: each ].
	self findMatches! !

!TextDiffBuilder methodsFor: 'private' stamp: 'ul 1/10/2011 19:23'!
lcsFor: xFilteredLines and: yFilteredLines
	"I find one of the longest common subsequences of my the arguments. I assume that none of my arguments are empty. I return nil or an Array which represents a list. The first two elements are the matching line numbers, the last is the next node in the list or nil if there are no more elements. The list containts the longest common subsequence. I'm a modified version of the greedy lcs algorithm from the 6th page of 'An O(ND) Difference Algorithm and Its Variations (1986)' by Eugene W. Myers"

	| n m v lcss max |
	n := xFilteredLines size.
	m := yFilteredLines size.
	max := m + n.
	v := Array new: 2 * max + 1.
	v at: max + 2 put: 0.
	lcss := Array new: 2 * max + 1.
	0 to: max do: [ :d |
		d negated to: d by: 2 do: [ :k |
			| index lcs x y |
			index := max + k.
			(k + d = 0 or: [ k ~= d and: [ (v at: index) < (v at: index + 2) ] ])
				ifTrue: [ x := v at: (index := index + 2) ]
				ifFalse: [ x := (v at: index) + 1 ].
			y := x - k.
			lcs := lcss at: index.
			[ x < n and: [ y < m and: [ (xFilteredLines at: x + 1) = (yFilteredLines at: y + 1) ] ] ]
				whileTrue: [
					lcs := { x := x + 1. y := y + 1. lcs } ].
			(x >= n and: [ y >= m ]) ifTrue: [ ^lcs ].
			v at: max + k + 1 put: x.
			lcss at: max + k + 1 put: lcs ] ].
	self error! !

!TextDiffBuilder methodsFor: 'creating patches' stamp: 'HenrikSperreJohansen 5/21/2010 01:53'!
patchSequenceDoIfMatch: matchBlock ifInsert: insertBlock ifRemove: removeBlock
	"I'm the general purpose method to iterate through the patch sequence. See my senders to learn how to use me."

	| xLine xLineStream |
	xLineStream := xLines readStream.
	yLines do: [ :yLine | 
		yLine hasMatch 
			ifFalse: [ insertBlock value: yLine string  ]
			ifTrue: [
				[ (xLine := xLineStream next) == nil or: [ xLine == yLine match  ] ]
					whileFalse: [ removeBlock value: xLine string ].
				matchBlock value: yLine string ] ].
	[ (xLine := xLineStream next) == nil ] whileFalse: [
		removeBlock value: xLine string ]! !

!TextDiffBuilder methodsFor: 'private' stamp: 'HenrikSperreJohansen 5/21/2010 01:45'!
print: aString withAttributes: attributes on: stream

	stream
		withAttributes: attributes 
		do: [ 
			stream nextPutAll: aString.
			(aString notEmpty and: [ 
				aString last = Character cr or: [
					aString endsWith: String crlf ] ]) 
						ifFalse: [ stream cr ] ]! !

!TextDiffBuilder methodsFor: 'private' stamp: 'HenrikSperreJohansen 5/21/2010 01:44'!
split: aString
	"I return an Array of strings which are the lines extracted from aString. All lines contain the line separator characters, or not depending on preference."

	^Array streamContents: [ :stream |
		self class ignoreLineEndings
			ifTrue: [aString lineIndicesDo: [ :start :endWithoutSeparators :end |
				stream nextPut: (aString copyFrom: start to: endWithoutSeparators) ] ]
			ifFalse: [aString lineIndicesDo: [ :start :endWithoutSeparators :end |
				stream nextPut: (aString copyFrom: start to: end) ] ] ]! !

Object subclass: #ThirtyTwoBitRegister
	instanceVariableNames: 'hi low'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Hashing-Core'!

!ThirtyTwoBitRegister commentStamp: '<historical>' prior: 0!
I represent a 32-bit register. An instance of me can hold any non-negative integer in the range [0..(2^32 - 1)]. Operations are performed on my contents in place, like a hardware register, and results are always modulo 2^32.

This class is primarily meant for use by the SecureHashAlgorithm class.
!

!ThirtyTwoBitRegister class methodsFor: 'instance creation' stamp: 'jm 12/14/1999 16:05'!
new
	"Answer a new instance whose initial contents is zero."

	^ super new load: 0
! !

!ThirtyTwoBitRegister methodsFor: 'accumulator ops' stamp: 'jm 12/7/1999 15:36'!
+= aThirtTwoBitRegister
	"Replace my contents with the sum of the given register and my current contents."

	| lowSum |
	lowSum := low + aThirtTwoBitRegister low.
	hi := (hi + aThirtTwoBitRegister hi + (lowSum bitShift: -16)) bitAnd: 16rFFFF.
	low := lowSum bitAnd: 16rFFFF.
! !

!ThirtyTwoBitRegister methodsFor: 'converting' stamp: 'len 8/7/2002 17:37'!
asByteArray
	^ ByteArray with: (low bitAnd: 16rFF) with: (low bitShift: -8) with: (hi bitAnd: 16rFF) with: (hi bitShift: -8)! !

!ThirtyTwoBitRegister methodsFor: 'accessing' stamp: 'jm 12/14/1999 16:03'!
asInteger
	"Answer the integer value of my current contents."

	^ (hi bitShift: 16) + low
! !

!ThirtyTwoBitRegister methodsFor: 'converting' stamp: 'DSM 1/20/2000 17:17'!
asReverseInteger
	"Answer the byte-swapped integer value of my current contents."

	^ ((low bitAnd: 16rFF) bitShift: 24) +
       ((low bitAnd: 16rFF00) bitShift: 8) +
	  ((hi bitAnd: 16rFF) bitShift: 8) +
       (hi bitShift: -8)
! !

!ThirtyTwoBitRegister methodsFor: 'accumulator ops' stamp: 'jm 12/7/1999 15:41'!
bitAnd: aThirtTwoBitRegister
	"Replace my contents with the bitwise AND of the given register and my current contents."

	hi := hi bitAnd: aThirtTwoBitRegister hi.
	low := low bitAnd: aThirtTwoBitRegister low.
! !

!ThirtyTwoBitRegister methodsFor: 'accumulator ops' stamp: 'jm 12/7/1999 15:40'!
bitInvert
	"Replace my contents with the bitwise inverse my current contents."

	hi := hi bitXor: 16rFFFF.
	low := low bitXor: 16rFFFF.
! !

!ThirtyTwoBitRegister methodsFor: 'accumulator ops' stamp: 'jm 12/7/1999 15:40'!
bitOr: aThirtTwoBitRegister
	"Replace my contents with the bitwise OR of the given register and my current contents."

	hi := hi bitOr: aThirtTwoBitRegister hi.
	low := low bitOr: aThirtTwoBitRegister low.
! !

!ThirtyTwoBitRegister methodsFor: 'accumulator ops' stamp: 'RJT 10/28/2005 15:42'!
bitShift: anInteger
	"Replace my contents with the bitShift of anInteger."
	self load: (self asInteger bitShift: anInteger). ! !

!ThirtyTwoBitRegister methodsFor: 'accumulator ops' stamp: 'jm 12/7/1999 15:38'!
bitXor: aThirtTwoBitRegister
	"Replace my contents with the bitwise exclusive OR of the given register and my current contents."

	hi := hi bitXor: aThirtTwoBitRegister hi.
	low := low bitXor: aThirtTwoBitRegister low.
! !

!ThirtyTwoBitRegister methodsFor: 'accessing' stamp: 'adrian_lienhard 7/21/2009 19:49'!
byte1: hi1 byte2: hi2 byte3: low1 byte4: low2
	hi := (hi1 bitShift: 8) + hi2.
	low := (low1 bitShift: 8) + low2.! !

!ThirtyTwoBitRegister methodsFor: 'accessing' stamp: 'len 8/15/2002 01:34'!
byteAt: anInteger
	anInteger = 1 ifTrue: [^ hi bitShift: -8].
	anInteger = 2 ifTrue: [^ hi bitAnd: 16rFF].
	anInteger = 3 ifTrue: [^ low bitShift: -8].
	anInteger = 4 ifTrue: [^ low bitAnd: 16rFF]! !

!ThirtyTwoBitRegister methodsFor: 'copying' stamp: 'jm 12/7/1999 15:26'!
copy
	"Use the clone primitive for speed."

	<primitive: 148>
	^ super copy
! !

!ThirtyTwoBitRegister methodsFor: 'accessing' stamp: 'jm 12/7/1999 15:26'!
hi

	^ hi
! !

!ThirtyTwoBitRegister methodsFor: 'accumulator ops' stamp: 'jm 12/7/1999 23:09'!
leftRotateBy: bits
	"Rotate my contents left by the given number of bits, retaining exactly 32 bits."
	"Details: Perform this operation with as little LargeInteger arithmetic as possible."

	| bitCount s1 s2 newHi |
	"ensure bitCount is in range [0..32]"
	bitCount := bits \\ 32.
	bitCount < 0 ifTrue: [bitCount := bitCount + 32].

	bitCount > 16
		ifTrue: [
			s1 := bitCount - 16.
			s2 := s1 - 16.
			newHi := ((low bitShift: s1) bitAnd: 16rFFFF) bitOr: (hi bitShift: s2).
			low := ((hi bitShift: s1) bitAnd: 16rFFFF) bitOr: (low bitShift: s2).
			hi := newHi]
		ifFalse: [
			s1 := bitCount.
			s2 := s1 - 16.
			newHi := ((hi bitShift: s1) bitAnd: 16rFFFF) bitOr: (low bitShift: s2).
			low := ((low bitShift: s1) bitAnd: 16rFFFF) bitOr: (hi bitShift: s2).
			hi := newHi]
! !

!ThirtyTwoBitRegister methodsFor: 'accessing' stamp: 'jm 12/14/1999 16:07'!
load: anInteger
	"Set my contents to the value of given integer."

	low := anInteger bitAnd: 16rFFFF.
	hi := (anInteger bitShift: -16) bitAnd: 16rFFFF.
	self asInteger = anInteger
		ifFalse: [self error: 'out of range: ', anInteger printString].
! !

!ThirtyTwoBitRegister methodsFor: 'accessing' stamp: 'jm 12/14/1999 16:07'!
loadFrom: aByteArray at: index
	"Load my 32-bit value from the four bytes of the given ByteArray starting at the given index. Consider the first byte to contain the most significant bits of the word (i.e., use big-endian byte ordering)."

	hi := ((aByteArray at: index) bitShift: 8) + ( aByteArray at: index + 1).
	low := ((aByteArray at: index + 2) bitShift: 8) + ( aByteArray at: index + 3).
! !

!ThirtyTwoBitRegister methodsFor: 'accessing' stamp: 'jm 12/7/1999 15:26'!
low

	^ low! !

!ThirtyTwoBitRegister methodsFor: 'printing' stamp: 'laza 3/29/2004 12:22'!
printOn: aStream
	"Print my contents in hex with a leading 'R' to show that it is a register object being printed."

	aStream nextPutAll: 'R:'.
	self asInteger storeOn: aStream base: 16.
! !

!ThirtyTwoBitRegister methodsFor: 'accessing' stamp: 'adrian_lienhard 7/21/2009 19:49'!
reverseLoadFrom: aByteArray at: index
	"Load my 32-bit value from the four bytes of the given ByteArray
starting at the given index. Consider the first byte to contain the most
significant bits of the word (i.e., use big-endian byte ordering)."

	hi := ((aByteArray at: index + 3) bitShift: 8) + ( aByteArray at: index + 2).
	low := ((aByteArray at: index + 1) bitShift: 8) + ( aByteArray at: index).
! !

!ThirtyTwoBitRegister methodsFor: 'accessing' stamp: 'len 8/15/2002 01:29'!
storeInto: aByteArray at: index
	"Store my 32-bit value into the four bytes of the given ByteArray starting at the given index. Consider the first byte to contain the most significant bits of the word (i.e., use big-endian byte ordering)."

	aByteArray at: index put: (hi bitShift: -8).
	aByteArray at: index + 1 put: (hi bitAnd: 16rFF).
	aByteArray at: index + 2 put: (low bitShift: -8).
	aByteArray at: index + 3 put: (low bitAnd: 16rFF)! !

Object subclass: #ToolRegistry
	instanceVariableNames: 'tools inspectorsMap'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Tools'!

!ToolRegistry commentStamp: 'BernardoContreras 1/18/2012 21:44' prior: 0!
I am serving as a system-wide hub for various tools.
Instead of using direct class names for tools, which is not always nice because can introduce inter-package
dependencies, i provide a uniform way for accessing tools, via Smalltalk global:

Smalltalk tools someToolName,

where 'someToolName' is a name of the tool , under which some tool are registered.

Tools are adding themselves to registry by implementing #registerToolsOn: message at class side, for example:

MyClass>>registerToolsOn: registry
   registry register: self as: #myTool

Registers a MyClass as a tool under name #myTool, and can be accessed via:

Smalltalk tools myTool 

and avoid putting 'MyClass' reference into code.

You can add  ' self registerToolsOn: Smalltalk tools ' in class #initialize,
so your new tool(s) can be registered during package loading.

To reset tool registry to defaults , use: 
Smalltalk resetTools.

Also registry storing a map of inspector types. 
This is to replace an old scheme which using #inspectorClass , and introducing dependencies.
Now, an inspecting class dont needs to have any knowledge about its specialized inspector.
Instead, a specialized inspector could tell registry that it is available for inspecting instances of given class (see #registerInspector:for:  senders).
This allows to avoid dependency from instance class to its inspector class or using extension methods.

WARNING: The point of the tools registry is NOT to be a facade full of protocols but to be a registration for tools that get loaded. So, the tools registry should NOT hold specific tool's protocol.  the tool's client should invoke the correct methods of the tool.

----------- 
The category 'menu' is for backward compatibility with ToolSet. We should remove it.

!

!ToolRegistry methodsFor: 'misc' stamp: 'MarcusDenker 7/12/2012 18:02'!
browseDirectToolReferences
	"Smalltalk tools browseDirectToolReferences"
	"Browse all direct references of all of my registered tools.
	Useful for converting calls like:

		MyTool foo
	into:
		Smalltalk tools myTool foo 	"
	
	| bindings |
	
	bindings := tools values select: [:each | each value isBehavior ]
		thenCollect: [:each | each value binding ].
	bindings := bindings copyWith: (Smalltalk bindingOf: #ToolSet).

	^  SystemNavigation new
		browseMessageList: (
			SystemNavigation new allMethodsSelect: [:m |
				(m literals allButLast select: [:lit| lit isVariableBinding ]) includesAnyOf: bindings ] )
		name: 'All direct references to tools to fix'! !

!ToolRegistry methodsFor: 'misc' stamp: 'MarcusDenker 5/27/2011 14:24'!
codeCompletionAround: aBlock textMorph: aTextMorph keyStroke: evt
	^(self hasToolNamed: #codeCompletion)
	ifTrue: [ self codeCompletion codeCompletionAround: aBlock textMorph: aTextMorph keyStroke: evt ]
	ifFalse: [ aBlock value ]! !

!ToolRegistry methodsFor: 'convenience-debugging' stamp: 'IgorStasenko 10/29/2008 03:02'!
debugError: anError
	"Handle an otherwise unhandled error"
	^Processor activeProcess
		debug: anError signalerContext
		title: anError description! !

!ToolRegistry methodsFor: 'convenience-debugging' stamp: 'CamilloBruni 2/9/2012 12:28'!
debugSyntaxError: aSynataxError
	"Handle a syntax error.
	
	Note, if there is no #syntaxError registered, it will be silently ignored"
	
	self using: #syntaxError do: [:tool | 
			tool open: (tool syntaxError: aSynataxError)]! !

!ToolRegistry methodsFor: 'handling DNU' stamp: 'IgorStasenko 2/19/2011 03:15'!
doesNotUnderstand: aMessage
	"Return a tool identified by a message selector.
	We need to handle special case, when no debugger registered.
	"
	| tool |
	tool := tools at: aMessage selector ifAbsent: nil.
	tool ifNotNil: [ ^ tool value ].

	^ (aMessage selector == #debugger) 
		ifTrue: [ 
			Smalltalk 
				logError: 'No debugger installed to handle debug request.' 
				inContext: thisContext.
		
			nil. "Smalltalk snapshot: false andQuit: true."
		]
		ifFalse: [ super doesNotUnderstand: aMessage ]
! !

!ToolRegistry methodsFor: 'testing' stamp: 'IgorStasenko 2/19/2011 03:18'!
hasToolNamed: aToolName
	^ tools includesKey: aToolName ! !

!ToolRegistry methodsFor: 'misc' stamp: 'IgorStasenko 10/29/2008 02:09'!
informNoDebugger
	(self includes: #uiManager) 
		ifTrue: [
			self uiManager inform: 'Cannot open debugger' ]
		ifFalse: [
			"If we throw an exception here, this can lead to recursion.
			Any idea what else we can do here?"
			Processor activeProcess terminate ]! !

!ToolRegistry methodsFor: 'initialize-release' stamp: 'IgorStasenko 8/12/2011 16:37'!
initDefaultToolSet
	
	"Query the system for registering tools"
	"we cannot afford to show warnings because not all tools e.g Debugger, Inspector are registered yet"
	
	[	
	Object withAllSubclassesDo: [:ea | 
		
	"evaluate only for classes which directly implement this message,
	not via subclassing"
		(ea isMeta  and: [
			ea includesSelector: #registerToolsOn: ]) ifTrue: [
			 ea theNonMetaClass registerToolsOn: self  ]].
	] on: Warning do: [:ex | ex resume ]! !

!ToolRegistry methodsFor: 'initialize-release' stamp: 'IgorStasenko 2/20/2011 14:58'!
initialize

	"symbols used as keys "
	tools := IdentityDictionary new.  
	inspectorsMap := IdentityDictionary new.
	
	self initDefaultToolSet! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 15:03'!
inspect: anObject
	"Open an inspector on the given object. 
	
	You can register custom inspector for own class.
	Use:
	
	Smalltalk tools registerInspector: #MyaIspectorClass for: #MyInstancesClass "
	
	| class |
	
	class := (self inspectorClassOf: anObject)
		ifNil: [ tools at: #inspector ifAbsent: [ self error: 'no inspector found' ]].
	
	class ifNotNil: [ ^ class openOn: anObject ]
	
! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'GaryChambers 12/12/2011 11:20'!
inspect: anObject label: aString
	"Open an inspector on the given object. 
	
	You can register custom inspector for own class.
	Use:
	
	Smalltalk tools registerInspector: #MyaIspectorClass for: #MyInstancesClass 
	"
	| class |
	
	class := self inspectorClassOf: anObject. 
	class ifNil: [ class := tools at: #inspector ifAbsent: []].
	
	class ifNotNil: [ ^ class openOn: anObject withEvalPane: true withLabel: aString ]
! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 15:02'!
inspectorClassOf: anObject
	"Answer the inspector class for the given object. The tool set must know which inspector type to use for which object - the object cannot possibly know what kind of inspectors the toolset provides."
		
	anObject class withAllSuperclassesDo: [:cls | 
		inspectorsMap at: cls name ifPresent: [:inspectorName |
			^ Smalltalk classNamed: inspectorName
		]].

	^ nil
	
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:26'!
mainMenuItems
	"Answer the most important menu items available for this tool set"
	^#(
		('System Browser' 			#openClassBrowser)
		('Workspace'				#openWorkspace)
		('Test Runner'				#openTestRunner)
		('Monticello Browser'		#openMonticelloBrowser)
	)
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 15:32'!
menuItems
	"Answer the menu items available for this tool set"

	^#(
		('System Browser' 			#openClassBrowser)
		('Message Names'			#openMessageNames)
		-
		('Workspace'				#openWorkspace)
		('Transcript' 				#openTranscript)
		('File Browser'				#openFileList)
		-
		('Test Runner'				#openTestRunner)
		('Process Browser' 			#openProcessBrowser)
		-
		('Monticello Browser'		#openMonticelloBrowser)
		"('Monticello Configurations' #openMonticelloConfigurations)"
		('Recover lost changes...'	#openRecentChangesLog)
		"('Simple Change Sorter'		#openChangeSorter)"
		('Change Sorter'				#openDualChangeSorter)
	)
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:57'!
openChangeSorter
	^ self changeSorter open
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:55'!
openClassBrowser
	^ self browser open! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 14:42'!
openDualChangeSorter
	self dualChangeSorter open! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 14:38'!
openFileList
	
	self fileList open! !

!ToolRegistry methodsFor: 'menu' stamp: 'BernardoContreras 8/14/2011 02:23'!
openFinder
	^ self finder open! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 10/29/2008 03:42'!
openMessageNames
	"Bring a MessageNames tool to the front"
	
	self using: #messageNames do: [:tool | tool openMessageNames ]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'sd 6/5/2011 19:05'!
openMonticelloBrowser
	
	self monticelloBrowser open
	! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 10/29/2008 03:43'!
openProcessBrowser
	self using: #processBrowser do: [:tool |
		tool open
		]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/20/2011 14:39'!
openRecentChangesLog
	self changeList browseRecentLog! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 10/29/2008 03:46'!
openSelectorBrowser
	self using: #selectorBrowser do: [:tool |
		tool new morphicWindow openInWorld
	]
! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 04:02'!
openTestRunner
	self using: #testRunner do: [:tool | tool open ]! !

!ToolRegistry methodsFor: 'menu' stamp: 'FernandoOlivero 5/9/2011 09:38'!
openTranscript

	^ UIManager default openTranscript! !

!ToolRegistry methodsFor: 'menu' stamp: 'IgorStasenko 2/19/2011 03:58'!
openWorkspace
	^ self workspace open! !

!ToolRegistry methodsFor: 'registration' stamp: 'BenjaminVanRyseghem 2/22/2012 14:53'!
register: anObject as: aNameSelector
	"Register a tool class under given name"
	tools at: aNameSelector put: anObject.
	WorldState defaultWorldMenu! !

!ToolRegistry methodsFor: 'convenience-inspecting' stamp: 'IgorStasenko 2/20/2011 14:58'!
registerInspector: anInspectorClass for: anInstanceClass

	"Register inspector class for given instance class" 

	inspectorsMap at: anInstanceClass name put: anInspectorClass name
! !

!ToolRegistry methodsFor: 'registration' stamp: 'IgorStasenko 10/29/2008 01:31'!
remove: aName
	"remove tool with given name"
	tools removeKey: aName ifAbsent: []! !

!ToolRegistry methodsFor: 'tools' stamp: 'IgorStasenko 2/20/2011 14:36'!
transcript
	"No need to register transcript. But for uniformity,
	implement this method, so transcript can be accessed via
	Smalltalk tools transcript , as any other tool "
	^ Transcript! !

!ToolRegistry methodsFor: 'misc' stamp: 'IgorStasenko 2/19/2011 04:03'!
using: aToolName do: aBlock
	"Evaluate a block for tool found under corresponding name or silently ignore if its not found or nil"
	| tool |
	
	tool := tools at: aToolName ifAbsent: [ ^ self ].
	tool ifNotNil: [ ^ aBlock value: tool value ]! !

Object subclass: #UpdateStreamer
	instanceVariableNames: 'applyChanges silent upToNumber saveLocally path pharoServer'
	classVariableNames: 'UpdateDownloader'
	poolDictionaries: ''
	category: 'System-Changes'!

!UpdateStreamer commentStamp: 'StephaneDucasse 9/1/2010 21:59' prior: 0!
I'm a little class inspired heavily from Utilities that download update.list and launch update.

you can tell me to update throw a given number of update or to be verbose.


UpdateStreamer new 
	beVerbose;
	updateFromServer
	
This will update all the updates available and show up dialogs
	!

!UpdateStreamer methodsFor: 'accessing' stamp: 'StephaneDucasse 9/1/2010 16:43'!
beSilent

	silent := true! !

!UpdateStreamer methodsFor: 'accessing' stamp: 'StephaneDucasse 9/1/2010 16:43'!
beVerbose

	silent := false! !

!UpdateStreamer methodsFor: 'private actions' stamp: 'StephaneDucasse 12/25/2011 22:43'!
elementaryReadServerUpdates
	"Scan the update server(s) for unassimilated updates. If the instance variable upToNumber is not nil, it represents the highest-numbered update to load.  This makes it possible to update only up to a particular point."

	"UpdateStreamer new elementaryReadServerUpdates"

	| str urls failed loaded this nextDoc updateName |
	(Smalltalk globals includesKey: #ScriptLoader) ifFalse: [
		^ self feedback: 'ScriptLoader is not present (this may happen if you have run cleanUpForProduction). ' ].
	Cursor wait showWhile: 
		[ | docQueue docQueueSema |
		urls := self newUpdates.
		loaded := 0.
		failed := nil.
		"send downloaded documents throuh this queue"
		docQueue := SharedQueue new.
		"this semaphore keeps too many documents from beeing queueed up at a time"
		docQueueSema := Semaphore new.
		5 timesRepeat: [ docQueueSema signal ].

		"fork a process to download the updates"
		self retrieveUrls: urls ontoQueue: docQueue withWaitSema: docQueueSema.

		"process downloaded updates in the foreground"

		[ this := docQueue next.
		nextDoc := docQueue next.  
		nextDoc = #failed ifTrue: [ failed := this ].
		(failed isNil and: [ nextDoc ~= #finished ])
		] whileTrue: [
			failed ifNil: [
				nextDoc size = 0 ifTrue: [ failed := this ]. ].
			failed ifNil: [
				nextDoc peek asciiValue = 4	"pure object file"
					ifTrue: [failed := this]].	"Must be fileIn, not pure object file"
			failed ifNil: [
				self shouldApplyChanges
					ifTrue: [
						updateName := (this findTokens: '/') last.
						ChangeSet newChangesFromStream: nextDoc named: updateName.
						SystemVersion current registerUpdate: updateName initialIntegerOrNil].
			self saveLocally ifTrue:
				[self saveUpdate: nextDoc onFile: (this findTokens: '/') last].
			].
		loaded := loaded + 1.
		docQueueSema signal].
	].

	failed ~~ nil & (urls size - loaded > 0) ifTrue: [
		str := loaded printString ,' new update file(s) processed.'.
		str := str, '\Could not load ' withCRs, 
			(urls size - loaded) printString ,' update file(s).',
			'\Starting with "' withCRs, failed, '".'.
		self feedback: str].
	^ Array with: failed with: loaded
! !

!UpdateStreamer methodsFor: 'private actions' stamp: ''!
extractVersion: aVersion from: updateList
	"Pull out the part of the list that applies to a version."
	"self new extractVersion: 'Pharo1.1' from: ..."
	
	| listContents version versIndex |
	self flag: #shouldUseADictionary.
	listContents := self groupUpdatesByVersion: updateList.
	versIndex := (listContents collect: [ :pair | pair first]) indexOf: aVersion.
	versIndex = 0 ifTrue: [^ Array new].
	^ (listContents at: versIndex) last! !

!UpdateStreamer methodsFor: 'private actions' stamp: 'StephaneDucasse 9/1/2010 16:46'!
feedback: aString

	self silent 
		ifFalse: [self inform: aString]
		ifTrue: [Transcript show: aString ; cr]! !

!UpdateStreamer methodsFor: 'private actions' stamp: ''!
groupUpdatesByVersion: updateContents
	"Parse the contents of updates.list into {{vers. {fileNames*}}*}, and return it."
	
	| sections vers strm line fileNames |
	self flag: #shouldUseADictionary.
	sections := OrderedCollection new.
	fileNames := OrderedCollection new: 1000.
	strm := updateContents readStream.
	[ strm atEnd ] whileFalse: 
		[ line := strm nextLine.
		line size > 0 ifTrue: 
			[ line first = $# 
				ifTrue: 
					[ vers ifNotNil: [ sections addLast: {  vers. (fileNames asArray)  } ].
					"Start a new section"
					vers := line allButFirst.
					fileNames resetTo: 1 ]
				ifFalse: [ line first = $* ifFalse: [ fileNames addLast: line ] ] ] ].
	vers ifNotNil: [ sections addLast: {  vers. (fileNames asArray)  } ].
	^ sections 
	! !

!UpdateStreamer methodsFor: 'private actions' stamp: 'CamilloBruni 12/16/2011 14:07'!
newUpdates
	"Return a list of fully formed URLs of update files we do not yet have up to the upToNumber instance variable if set." 

	"self new newUpdates"

	| existing out list result doc raw char |
	out := OrderedCollection new.
	existing := SystemVersion current updates.
	result := ZnClient new
		systemPolicy;
		beOneShot;
		accept: ZnMimeType text;
		url: self updateListFileURLString;
		ifFail: [ :exception | self error: 'Cannot access update server: ' , exception printString ];
		get.
	"one file name per line"
	list := self extractVersion: SystemVersion current version from: result.
	list reverseDo: [:fileName | | itsNumber |
		itsNumber := fileName initialIntegerOrNil. 
		(existing includes: itsNumber)
			ifFalse: [ (itsNumber isNil or: [itsNumber <= self upToNumber])
							ifTrue: [out addFirst: self updateFolderURL, fileName]]
			ifTrue: [^ out]].
	"("
	(out size > 0) "or: 
			[char := doc reset; skipSeparators; next.
			(char == $*) | (char == $#)])"  ifTrue:
				[^ out "we have our list"].	
	self feedback: 'All code update servers seem to be unavailable'.
	^ out! !

!UpdateStreamer methodsFor: 'private actions' stamp: 'StephaneDucasse 9/1/2010 17:32'!
newUpdatesUpToNumber: aNumber
	"Return a list of fully formed URLs of update files we do not yet have.  Look at the file 'updates.list'.
	No updates numbered higher than aNumber (if it is not nil) are returned " 

	"self new newUpdatesToNumber: 14000"

	self upToNumber: aNumber.
	^ self newUpdates.! !

!UpdateStreamer methodsFor: 'accessing' stamp: ''!
path
	"Part ... in the following  http:// server / path /updates.list"
	
	^ path ifNil: [ path := 'updates/pharo'].! !

!UpdateStreamer methodsFor: 'accessing' stamp: ''!
path: aString
	"http:// server / aString /updates.list"
	
	^ path ifNil: [ path := 'updates/pharo'].! !

!UpdateStreamer methodsFor: 'private actions' stamp: 'StephaneDucasse 9/1/2010 19:15'!
retrieveUrls: urls ontoQueue: queue withWaitSema: waitSema 
	"download the given list of URLs. The queue will be loaded alternately  
	with url's and with the retrieved contents. If a download fails, the  
	contents will be #failed. If all goes well, a special pair with an empty  
	URL and the contents #finished will be put on the queue. waitSema is  
	waited on every time before a new document is downloaded; this keeps 
	the downloader from getting too far  ahead of the main process"
	"kill the existing downloader if there is one"
	
	self silent 
		ifTrue: [self silentRetrieveUrls: urls ontoQueue: queue withWaitSema: waitSema ]
		ifFalse: [self verboseRetrieveUrls: urls ontoQueue: queue withWaitSema: waitSema ]
! !

!UpdateStreamer methodsFor: 'accessing' stamp: ''!
saveLocally

	^ saveLocally ifNil: [saveLocally := false]! !

!UpdateStreamer methodsFor: 'accessing' stamp: ''!
saveLocally: aBoolean

	saveLocally := aBoolean! !

!UpdateStreamer methodsFor: 'untested' stamp: 'SeanDeNigris 7/12/2012 08:45'!
saveUpdate: doc onFile: fileName
	"Save the update on a local file.  With or without the update number on the front"

	| file fName pos updateDirectory |
	updateDirectory := 'updates' asFileReference.
	updateDirectory ensureDirectory.

	fName := fileName.
	(self updateRemoveSequenceNum) ifTrue:
		[pos := fName findFirst: [:c | c isDigit not].
		fName := fName copyFrom: pos to: fName size].
	
	doc reset; ascii.
	
	file := updateDirectory / fName.
	file exists ifTrue: [ ^ self ].
	
	file writeStreamDo: [ :s|
		s nextPutAll: doc contents].
! !

!UpdateStreamer methodsFor: 'accessing' stamp: ''!
server

	pharoServer ifNil: [pharoServer := 'pharo.gforge.inria.fr/'].
	^ pharoServer! !

!UpdateStreamer methodsFor: 'accessing' stamp: ''!
server: aStringWithHTTPColon

	pharoServer := aStringWithHTTPColon
! !

!UpdateStreamer methodsFor: 'accessing' stamp: 'StephaneDucasse 9/1/2010 16:49'!
shouldApplyChanges
	"when true the changes are downloaded and executed in the image"
	
	^ applyChanges ifNil: [applyChanges := true ]! !

!UpdateStreamer methodsFor: 'accessing' stamp: 'StephaneDucasse 9/1/2010 16:50'!
shouldApplyChanges: aBoolean
	"when true the changes are downloaded and executed in the image"
	
	applyChanges := aBoolean ! !

!UpdateStreamer methodsFor: 'accessing' stamp: 'MarcusDenker 11/29/2010 21:54'!
silent
	
	^ silent ifNil: [self beVerbose.false.]! !

!UpdateStreamer methodsFor: 'private actions' stamp: 'CamilloBruni 12/16/2011 14:28'!
silentRetrieveUrls: urls ontoQueue: queue withWaitSema: waitSema 
	"download the given list of URLs. The queue will be loaded alternately  
	with url's and with the retrieved contents. If a download fails, the  
	contents will be #failed. If all goes well, a special pair with an empty  
	URL and the contents #finished will be put on the queue. waitSema is  
	waited on every time before a new document is downloaded; this keeps 
	the downloader from getting too far  ahead of the main process"
	"kill the existing downloader if there is one"
	
	self zapUpdateDownloader.
	"fork a new downloading process"
	UpdateDownloader := [
		urls do: [ :url | | doc | 
			waitSema wait.
			queue nextPut: url.
			doc := ZnClient new
				systemPolicy;
				beOneShot;
				ifFail: [ :exception | 
					queue nextPut: #failed.
					UpdateDownloader := nil.
					Processor activeProcess terminate ];
				get: url.
			queue nextPut: doc readStream ].		
		queue nextPut: ''.
		queue nextPut: #finished.
		UpdateDownloader := nil] newProcess.
	UpdateDownloader priority: Processor userInterruptPriority.
	"start the process running"
	UpdateDownloader resume! !

!UpdateStreamer methodsFor: 'accessing' stamp: 'StephaneDucasse 9/1/2010 17:30'!
upToNumber
	"when specified the update will stop at this number"
	
	^ upToNumber ifNil: [ upToNumber := 1000000].! !

!UpdateStreamer methodsFor: 'accessing' stamp: 'StephaneDucasse 9/1/2010 16:57'!
upToNumber: aNumber
	"when specified the update will stop at this number"
	
	upToNumber := aNumber. ! !

!UpdateStreamer methodsFor: 'accessing' stamp: 'StephaneDucasse 1/31/2011 22:26'!
updateFolderURL
	"in the future it may be wiser to use SystemVersion to avoid to duplicate the logic"
	"SystemVersion current major"
	^ 'http://', self server, self path, (Smalltalk globals at: #ScriptLoader) currentMajorVersionNumber asString, '/'! !

!UpdateStreamer methodsFor: 'public' stamp: 'BenjaminVanRyseghem 7/6/2012 20:07'!
updateFromServer
	"Scan the update server for unassimilated updates. If upToNumber is not nil, it represents the highest-numbered update to load.  This makes it possible to update only up to a particular point.   If saveLocally is true, then save local copies of the update files on disc.  If shouldApplyChanges is true, then absorb the updates into the current image.

A file on the server called updates.list has the names of the last N update files.  We look backwards for the first one we do not have, and start there"
"* To add a new update:  Name it starting with a new two-digit code.  
* Do not use %, /, *, space, or more than one period in the name of an update file.
* The update name does not need to have any relation to the version name.
* Figure out which versions of the system the update makes sense for.
* Add the name of the file to each version's category below.
* Put this file and the update file on all of the servers.
*
* To make a new version of the system:  Pick a name for it (no restrictions)
* Put # and exactly that name on a new line at the end of this file.
* During the release process, fill in exactly that name in the dialog box.
* Put this file on the server."

"When two sets of updates need to use the same directory, one of them has a * in its 
serverUrls description.  When that is true, the first word of the description is put on
the front of 'updates.list', and that is the index file used."

	[[[
		| pair failed loaded |
		pair := self elementaryReadServerUpdates.
		failed := pair first.
		loaded := pair second.
		failed ifNil: ["is OK"
			 self feedback:  (loaded printString ,' new update file(s) processed.')]] 
	on: (Smalltalk at: #MCMergeOrLoadWarning ifAbsent: [ nil ]) do: [ :e| e resume: false ]] 
	on: (Smalltalk at: #MCNoChangesException ifAbsent: [ nil ]) do: [ :e| e resume]]
 	on: MCMergeResolutionRequest do: [ :request |
			request merger conflicts isEmpty
				ifTrue: [ request resume: true ]
				ifFalse: [ request pass ]].! !

!UpdateStreamer methodsFor: 'updating' stamp: 'Integrator 6/21/2012 16:27'!
updateFromServerSilently
	"Scan the update server for unassimilated updates. If upToNumber is not nil, it represents the highest-numbered update to load.  This makes it possible to update only up to a particular point.   If saveLocally is true, then save local copies of the update files on disc.  If shouldApplyChanges is true, then absorb the updates into the current image.

A file on the server called updates.list has the names of the last N update files.  We look backwards for the first one we do not have, and start there"
"* To add a new update:  Name it starting with a new two-digit code.  
* Do not use %, /, *, space, or more than one period in the name of an update file.
* The update name does not need to have any relation to the version name.
* Figure out which versions of the system the update makes sense for.
* Add the name of the file to each version's category below.
* Put this file and the update file on all of the servers.
*
* To make a new version of the system:  Pick a name for it (no restrictions)
* Put # and exactly that name on a new line at the end of this file.
* During the release process, fill in exactly that name in the dialog box.
* Put this file on the server."

"When two sets of updates need to use the same directory, one of them has a * in its 
serverUrls description.  When that is true, the first word of the description is put on
the front of 'updates.list', and that is the index file used."

	| pair failed loaded |
	pair := self elementaryReadServerUpdates.
	failed := pair first.
	loaded := pair second.
	^ true! !

!UpdateStreamer methodsFor: 'accessing' stamp: 'StephaneDucasse 1/31/2011 22:24'!
updateListFileURLString
	"in the future it may be wiser to use VersionNumber to avoid to duplicate the logic"
	
	^ self updateFolderURL, self updatesListFileName! !

!UpdateStreamer methodsFor: 'accessing' stamp: 'SeanDeNigris 7/12/2012 08:44'!
updatesListFile
	^  self updatesListFileName asFileReference! !

!UpdateStreamer methodsFor: 'accessing' stamp: 'StephaneDucasse 2/10/2011 15:39'!
updatesListFileName

	^ 'updates', (SystemVersion current majorMinor:''), '.list'! !

!UpdateStreamer methodsFor: 'private actions' stamp: 'SeanDeNigris 6/21/2012 09:16'!
verboseRetrieveUrls: urls ontoQueue: queue withWaitSema: waitSema 
	"download the given list of URLs. The queue will be loaded alternately  
	with url's and with the retrieved contents. If a download fails, the  
	contents will be #failed. If all goes well, a special pair with an empty  
	URL and the contents #finished will be put on the queue. waitSema is  
	waited on every time before a new document is downloaded; this keeps 
	the downloader from getting too far  ahead of the main process"
	"kill the existing downloader if there is one"
	
	self zapUpdateDownloader.
	"fork a new downloading process"
	UpdateDownloader := [ | updateCounter |
		updateCounter := 0.
		'Downloading updates' displayProgressFrom: 0 to: urls size during: [:bar |
			urls do: [ :url | | doc | 
				waitSema wait.
				queue nextPut: url.
				doc := ZnClient new
					systemPolicy;
					beOneShot;
					ifFail: [ :exception | 
						queue nextPut: #failed.
						UpdateDownloader := nil.
						Processor activeProcess terminate ];
					get: url.
				queue nextPut: doc readStream. 
				updateCounter := updateCounter + 1. 
				bar current: updateCounter ] ].
			queue nextPut: ''.
			queue nextPut: #finished.
			UpdateDownloader := nil ] newProcess.
	UpdateDownloader priority: Processor userInterruptPriority.
	"start the process running"
	UpdateDownloader resume ! !

!UpdateStreamer methodsFor: 'untested' stamp: ''!
writeList: listContents toStream: strm
	"Write a parsed updates.list out as text.
	This is the inverse of groupUpdatesByVersion:"

	"so far did not tested it - sd"
	[strm reset.
	listContents do:
		[:pair | | fileNames version | 
				version := pair first.  
				fileNames := pair last.
		strm nextPut: $#; nextPutAll: version; cr.
		fileNames do: [:fileName | strm nextPutAll: fileName; cr]]]
	ensure: [strm close]! !

!UpdateStreamer methodsFor: 'private actions' stamp: ''!
zapUpdateDownloader

	UpdateDownloader ifNotNil: [UpdateDownloader terminate].
	UpdateDownloader := nil.! !

Object subclass: #VirtualMachine
	instanceVariableNames: ''
	classVariableNames: 'LastStats WordSize'
	poolDictionaries: ''
	category: 'System-Support'!

!VirtualMachine commentStamp: 'IgorStasenko 8/16/2011 17:41' prior: 0!
I am exposing a various attributes of smalltalk virtual machine.


The direct use of this class is not recommended. 

All attributes are normally should be accessed via

Smalltalk vm someAttribute

instead of 

VirtualMachine someAttribute

!

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 15:08'!
allocationsBetweenGC
	"allocations between GCs (read-write)"
	^ self parameterAt: 5! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 15:08'!
allocationsBetweenGC: anInteger
	"allocations between GCs (read-write)"
	^ self parameterAt: 5 put: anInteger! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'CamilloBruni 7/10/2012 21:54'!
binary
	"Answer the a file reference to the virtual machine binary."
	"SmalltalkImage current vmBinary"

	^ FileLocator vmBinary
! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'IgorStasenko 8/17/2011 17:06'!
buildDate			
	"Return a String reflecting the build date of the VM"
	"Smalltalk buildDate"

	^self getSystemAttribute: 1006! !

!VirtualMachine class methodsFor: 'accessing' stamp: 'IgorStasenko 8/17/2011 16:57'!
cogitClass
	"Return the jit class that is currently executing the system (Cog VM only)"
	^self getSystemAttribute: 1008! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'CamilloBruni 7/10/2012 21:53'!
directory
	"Answer the a file reference to the directory containing the Smalltalk virtual machine."
	"SmalltalkImage current directory"

	^ FileLocator vmDirectory
! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'IgorStasenko 8/17/2011 16:59'!
documentPath
	"Answer the absolute path of the document passed to the vm or nil if none."
	"Smalltalk commandLine documentPath"
	
	^self getSystemAttribute: 2! !

!VirtualMachine class methodsFor: 'accessing' stamp: 'IgorStasenko 8/16/2011 18:08'!
extraVMMemory
	"Answer the current setting of the 'extraVMMemory' VM parameter. See the comment in extraVMMemory: for details."

	^ self parameterAt: 23
! !

!VirtualMachine class methodsFor: 'accessing' stamp: 'IgorStasenko 8/16/2011 18:09'!
extraVMMemory: extraBytesToReserve
	"Request that the given amount of extra memory be reserved for use by the virtual machine to leave extra C heap space available for things like plugins, network and file buffers, and so on. This request is stored when the image is saved and honored when the image is next started up. Answer the previous value of this parameter."

	extraBytesToReserve < 0
		ifTrue: [self error: 'VM memory reservation must be non-negative'].
	^ self parameterAt: 23 put: extraBytesToReserve
! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'MarcusDenker 7/13/2012 14:35'!
fileName

	^ (FilePath pathName: (self fullPath) isEncoded: true) asSqueakPathName! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'IgorStasenko 8/17/2011 14:55'!
flagInterpretedMethods: aBoolean
	"The Cog VM can be instructed to set the flag bit of CompiledMethods that it executes but will only interpret.  This can be used e.g. to profile startup. See CompiledMethod>>#flag & CompiledMethod>>#clearFlag. This flag persists across snapshots, stored in the image header."
	
	self parameterAt: 48 put: ((self parameterAt: 48) bitClear: 2) + (aBoolean ifTrue: [2] ifFalse: [0])! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 16:04'!
fullGCCount
	"Answer the total number of full GCs since startup (read-only)."

	^ self parameterAt: 7
! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'CamilloBruni 5/13/2012 19:19'!
fullPath
	"Return the full vm path as a string"
	"Ex: '/Applications/Squeak/Squeak 4.2.5beta1U.app/Contents/MacOS/Squeak VM Opt'"
	"SmalltalkImage current vmFullPath"

	^self getSystemAttribute: 0! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/16/2011 18:12'!
gcBiasToGrow: aBool
	"Tell the VM to grow after tenures instead of running in a tight loop where it does IGCs over and over. For some weird reason the primitive takes an integer not a bool but oh well..."
	
	self primitiveGCBiasToGrow: (aBool ifTrue: [1] ifFalse: [0])! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/16/2011 18:13'!
gcBiasToGrowLimit: arg
	"Tell the VM the grow limit if the GC logic has bias to grow."
	
	<primitive: 'primitiveSetGCBiasToGrowGCLimit'>
	^self primitiveFailed! !

!VirtualMachine class methodsFor: 'vm parameters' stamp: 'IgorStasenko 8/16/2011 17:50'!
getParameters	
	"Answer an Array containing the current values of the VM's internal
	parameter/metric registers.  Each value is stored in the array at the
	index corresponding to its VM register.  (See #vmParameterAt: and
	#vmParameterAt:put:.)"
	"Smalltalk vm getParameters"
	
	<primitive: 254>
	self primitiveFailed! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'IgorStasenko 8/16/2011 17:45'!
getSystemAttribute: attributeID 
	"Optional. Answer the string for the system attribute with the given 
	integer ID. Answer nil if the given attribute is not defined on this 
	platform. On platforms that support invoking programs from command 
	lines (e.g., Unix), this mechanism can be used to pass command line 
	arguments to programs written in Pharo.

	By convention, the first command line argument that is not a VM
	configuration option is considered a 'document' to be filed in. Such a
	document can add methods and classes, can contain a serialized object,
	can include code to be executed, or any combination of these.

	Currently defined attributes include: 
	-1000   1000th command line argument that specify VM options
   	...
	-1              first command line argument that specify VM options
      0               the full path name for currently executing VM
                       (or, on some platforms, just the path name of the VM's directory)
       1               full path name of this image (better use primImageName instead)
       2               a Squeak document to open, if any
       3               first command line argument for Squeak programs
       ...
       1000    1000th command line argument for Squeak programs
       1001    this platform's operating system 'Mac OS', 'Win32', 'unix', ...
       1002    operating system version
       1003    this platform's processor type
       1004    vm version
       1005    window system name
       1006    vm build id
       1007    Interpreter class (Cog VM only)
       1008    Cogit class (Cog VM only)
       1201    max filename length (Mac OS only)
       1202    file last error (Mac OS only)
       10001   hardware details (Win32 only)
       10002   operating system details (Win32 only)
       10003   graphics hardware details (Win32 only)"

	<primitive: 149>
	^ nil! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'MarcusDenker 7/10/2012 10:40'!
imageFile
	"Image file "
	^ Smalltalk image imageFile! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'MarcusDenker 7/10/2012 10:40'!
imagePath
	"Image file path
	Ex:  '/Users/foo/Pharo/Pharo-1.1.1-dev10.09.1.image'"
	^ Smalltalk image imagePath! !

!VirtualMachine class methodsFor: 'vm parameters' stamp: 'StephaneDucasse 1/9/2012 17:11'!
imageVersionNumber
	"Image version number (6505 means the Squeak V3 format with BlockClosure support (but _without_ BlockContext support)"
		
	^ self parameterAt: 41! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 16:05'!
incrementalGCCount
	"Answer the total number of incremental GCs since startup (read-only)."

	^ self parameterAt: 9
! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'IgorStasenko 8/17/2011 17:02'!
interpreterClass
	"Return the interpreter class that is currently executing the system (Cog VM only)"
	^ self getSystemAttribute: 1007! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'CamilloBruni 7/18/2012 13:45'!
interpreterSourceDate
	"Return the date of changes given by `self interpreterSourceVersion`"
	| dateString parts | 
	
	dateString := (self interpreterSourceVersion splitOn: 'Date: ') second.
	
	dateString first isDigit
		ifTrue: [ 
			"most probably the date is in ISO 8601 Format" 
			^ dateString asDateAndTime ].
		
	"Otherwise assume the old format: DDD MMM DD HH:MM:SS YYYY +TTTT ..."
	parts := dateString substrings first: 6.
	"create a more reasonable string.."
	dateString := String streamContents: [ :s |
		s 
			nextPutAll: (parts at: 5 ); space; "year"
			nextPutAll: (parts at: 2 ); space; "month name"
			nextPutAll: (parts at: 3 ); space; "day of month"
			nextPutAll: (parts at: 4 ); space; "time"
			nextPutAll: (parts last); space    "timezone"].
	^ dateString asDateAndTime ! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'CamilloBruni 7/18/2012 12:51'!
interpreterSourceVersion
	"The use of this primitive not recommended. Not all VMs providing that"

	"Answer a string corresponding to the version of the interpreter source.
	This represents the version level of the Smalltalk source code (interpreter
	and various plugins) that is translated to C by a CCodeGenerator, as distinct
	from the external platform source code, typically written in C and managed
	separately for each platform. An optional primitive is invoked that may not
	be available on all virtual machines."

	"Smalltalk vm interpreterSourceVersion"

	<primitive: 'primitiveInterpreterSourceVersion'>
	^ self getSystemAttribute: 1009! !

!VirtualMachine class methodsFor: 'testing' stamp: 'IgorStasenko 8/16/2011 19:14'!
isRunningCog
	"Returns true if we're running on a Cog VM (JIT or StackInterpreter)
	 (parameterAt: 42 is the number of stack pages)"

	^[(self parameterAt: 42) > 0] on: Error do:[:ex| ex return: false]! !

!VirtualMachine class methodsFor: 'testing' stamp: 'IgorStasenko 8/17/2011 14:56'!
isRunningCogit
	"Returns true if we're running on the Cog JIT
	 (vmParameterAt: 46 is the size of the machine code zone)"

	^[(self parameterAt: 46) > 0] on: Error do:[:ex| ex return: false]! !

!VirtualMachine class methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 8/19/2011 11:00'!
maxExternalSemaphores
	"The size of array in some VM's where external signals for
	semaphores in externalObjects are handled.
	Essentially, if a semaphore is registered in externalObjects outside
	its bounds, they will not be signalled."
	^ [self parameterAt: 49]
		on: PrimitiveFailed
		do: [:ex | ex return: nil]! !

!VirtualMachine class methodsFor: 'accessing' stamp: 'StephaneDucasse 11/7/2011 22:45'!
maxExternalSemaphores: aSize 
	"This method should never be called as result of normal program
	execution. If it is however, handle it differently:
	- In development, signal an error to promt user to set a bigger size
	at startup immediately.
	- In production, accept the cost of potentially unhandled interrupts,
	but log the action for later review.
	
	See comment in maxExternalObjectsSilently: why this behaviour is
	desirable, "
	"Can't find a place where development/production is decided.
	Suggest Smalltalk image inProduction, but use an overridable temp
	meanwhile. "
	| inProduction |
	self maxExternalSemaphores
		ifNil: [^ 0].
	inProduction := false.
	^ inProduction
		ifTrue: [self maxExternalSemaphoresSilently: aSize.
			self crTrace: 'WARNING: Had to increase size of semaphore signal handling table due to many external objects concurrently in use';
				 crTrace: 'You should increase this size at startup using #maxExternalObjectsSilently:';
				 crTrace: 'Current table size: ' , self maxExternalSemaphores printString]
		ifFalse: ["Smalltalk image"
			self error: 'Not enough space for external objects, set a larger size at startup!!'
			"Smalltalk image"]! !

!VirtualMachine class methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 8/19/2011 11:02'!
maxExternalSemaphoresSilently: aSize 
	"Changes the size of array where external signals for semaphores in
	externalObjects are handled.. 
	The size can only grow, and will always be the next power of two
	larger than the parameter.
	
	The intended use is to set the table size to some adequate
	maximum as part of a non-resuming image startUp.
	
	Setting this at any time other than start-up can potentially result in
	lost signals during reallocation.
	i.e. Requests handled during copying og signals from old to new array
	won't be seen if they occur to indices already copied, before
	pointers to the new and old arrays are switched."
	self maxExternalSemaphores
		ifNil: [^ 0].
	"The vm-header field where the size is stored is a short, maximum
	64k entries. 
	Well, on most platforms anyways"
	(aSize < 0
			or: [aSize > 65535])
		ifTrue: [^ DomainError signalFrom: 0 to: 65535].
	^ self parameterAt: 49 put: aSize! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'CamilloBruni 5/13/2012 19:18'!
maxFilenameLength
	"Return the maximal filename length (only under MacOS)"
	"Smalltalk vm maxFilenameLength"

	^ self getSystemAttribute: 1201! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 16:08'!
memoryEnd
	"end of memory"
		
	^ self parameterAt: 3
! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 16:08'!
oldSpaceEnd
	"end of old-space (0-based, read-only)"
		

	^ self parameterAt: 1
! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'IgorStasenko 8/17/2011 17:04'!
optionAt: i
	"Answer the i-th option of the command line, or nil if not so many options."
	
	^self getSystemAttribute: i negated! !

!VirtualMachine class methodsFor: 'vm parameters' stamp: 'StephaneDucasse 1/9/2012 17:11'!
parameterAt: parameterIndex
	"parameterIndex is a positive integer corresponding to one of the VM's internal
	parameter/metric registers.  Answer with the current value of that register.
	Fail if parameterIndex has no corresponding register.
	VM parameters are numbered as follows:
		1	end of old-space (0-based, read-only)
		2	end of young-space (read-only)
		3	end of memory (read-only)
		4	allocationCount (read-only)
		5	allocations between GCs (read-write)
		6	survivor count tenuring threshold (read-write)
		7	full GCs since startup (read-only)
		8	total milliseconds in full GCs since startup (read-only)
		9	incremental GCs since startup (read-only)
		10	total milliseconds in incremental GCs since startup (read-only)
		11	tenures of surving objects since startup (read-only)
		12-20 specific to the translating VM
		21	root table size (read-only)
		22	root table overflows since startup (read-only)
		23	bytes of extra memory to reserve for VM buffers, plugins, etc.

		24	memory threshold above which shrinking object memory (rw)
		25	memory headroom when growing object memory (rw)
		26  interruptChecksEveryNms - force an ioProcessEvents every N milliseconds, in case the image  is not calling getNextEvent often (rw)
		27	number of times mark loop iterated for current IGC/FGC (read-only) includes ALL marking
		28	number of times sweep loop iterated  for current IGC/FGC (read-only)
		29	number of times make forward loop iterated for current IGC/FGC (read-only)
		30	number of times compact move loop iterated for current IGC/FGC (read-only)
		31	number of grow memory requests (read-only)
		32	number of shrink memory requests (read-only)
		33	number of root table entries used for current IGC/FGC (read-only)
		34	number of allocations done before current IGC/FGC (read-only)
		35	number of survivor objects after current IGC/FGC (read-only)
		36  millisecond clock when current IGC/FGC completed (read-only)
		37  number of marked objects for Roots of the world, not including Root Table entries for current IGC/FGC (read-only)
		38  milliseconds taken by current IGC  (read-only)
		39  Number of finalization signals for Weak Objects pending when current IGC/FGC completed (read-only)
		40  VM word size - 4 or 8 (read-only)
		41 Image version number (6505 means the Squeak V3 format with BlockClosure support (but _without_ BlockContext support)"

	<primitive: 254>
	self primitiveFailed! !

!VirtualMachine class methodsFor: 'vm parameters' stamp: 'IgorStasenko 8/16/2011 17:50'!
parameterAt: parameterIndex put: newValue
	"parameterIndex is a positive integer corresponding to one of the VM's internal
	parameter/metric registers.  Store newValue (a positive integer) into that
	register and answer with the previous value that was stored there.
	Fail if newValue is out of range, if parameterIndex has no corresponding
	register, or if the corresponding register is read-only."

	<primitive: 254>
	self primitiveFailed! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'CamilloBruni 5/13/2012 19:19'!
path
	"Answer the path for the directory containing the Smalltalk virtual machine. Return the 	empty string if this primitive is not implemented."
	"SmalltalkImage current vmPath"

	^ (FilePath pathName: (self primVmPath) isEncoded: true) asSqueakPathName.
! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'IgorStasenko 8/16/2011 17:55'!
platformSourceVersion
	"The use of this primitive not recommended. Not all VMs providing that"


	"Answer a string corresponding to the version of the external platform source
	code, typically written in C and managed separately for each platform. This
	invokes an optional primitive that may not be available on all virtual machines."

	"Smalltalk vm platformSourceVersion"

	<primitive: 'primitivePlatformSourceVersion'>
	self notify: 'This virtual machine does not support the optional primitive #primitivePlatformSourceVersion' translated.
	^''! !

!VirtualMachine class methodsFor: 'primitives' stamp: 'CamilloBruni 5/13/2012 19:14'!
primVmPath
	"Answer the path for the directory containing the Smalltalk virtual machine. Return the empty string if this primitive is not implemented."
	"Smalltalk vmPath"

	<primitive: 142>
	^ ''! !

!VirtualMachine class methodsFor: 'primitives' stamp: 'IgorStasenko 8/16/2011 18:11'!
primitiveGCBiasToGrow: arg
	"Tell the VM to grow after tenures instead of running in a tight loop where it does IGCs over and over. For some weird reason the primitive takes an integer not a bool but oh well..."
	
	<primitive: 'primitiveSetGCBiasToGrow'>
	^self primitiveFailed! !

!VirtualMachine class methodsFor: 'accessing' stamp: 'IgorStasenko 8/17/2011 14:57'!
processHasThreadIdInstVar: aBoolean
	"The threaded VM needs to know if the 4th inst var of Process
	 is threadId which it uses to control process-to-thread binding.
	 This flag persists across snapshots, stored in the image header."
	aBoolean ifTrue: [self assert: (Process instVarNames at: 4) ='threadId'].
	self parameterAt: 48 put: ((self parameterAt: 48) bitClear: 1) + (aBoolean asBit)
	
! !

!VirtualMachine class methodsFor: 'testing' stamp: 'IgorStasenko 8/17/2011 14:59'!
processPreemptionYields
	"Answer whether the VM causes a process to yield on process preemption, i.e. to put a preempted process at the back of its run queue.  If the parameter is unavailable (non-Cog VMs) or bit 2 (4) is 0 then preemption yields."

	^(([self parameterAt: 48]
			on: Error
			do: [:ex| ^true]) allMask: 2r100) not! !

!VirtualMachine class methodsFor: 'accessing' stamp: 'IgorStasenko 8/17/2011 15:00'!
processPreemptionYields: aBoolean
	"The Cog VM can be instructed not to yield on process preemption, i.e. not to put a preempted process at the back of its run queue.  By default preempting a process causes it to yield (Blue Book semantics) which can have unfortunate effects. This flag persists across snapshots, stored in the image header."
	
	self parameterAt: 48 put: ((self parameterAt: 48) bitClear: 4) + (aBoolean ifTrue: [0] ifFalse: [2r100])! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 15:12'!
setGCBiasToGrow: aNumber
	"Primitive. Indicate that the GC logic should be bias to grow"
	<primitive: 'primitiveSetGCBiasToGrow'>
	^self primitiveFailed
"Example:
	Smalltalk vm  setGCBiasToGrowGCLimit: 16*1024*1024.
	Smalltalk vm setGCBiasToGrow: 1.
"! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 15:11'!
setGCBiasToGrowGCLimit: aNumber
	"Primitive. Indicate that the bias to grow logic should do a GC after aNumber Bytes"
	<primitive: 'primitiveSetGCBiasToGrowGCLimit'>
	^self primitiveFailed
"Example:
	Smalltalk vm setGCBiasToGrowGCLimit: 16*1024*1024.
"! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 15:17'!
setGCParameters
	"Adjust the VM's default GC parameters to avoid premature tenuring."

	self allocationsBetweenGC: 4000.  "do an incremental GC after this many allocations"
	self tenuringThreshold: 2000.  "tenure when more than this many objects survive the GC"
	self setGCBiasToGrowGCLimit: 16*1024*1024.
	self setGCBiasToGrow: 1.
! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 15:22'!
setGCSemaphore: semaIndex
	"Primitive. Indicate the GC semaphore index to be signaled on GC occurance."
	<primitive: 'primitiveSetGCSemaphore'>
	^self primitiveFailed
"Example:

	| index sema process |
	sema := Semaphore new.
	index := Smalltalk registerExternalObject: sema.
	Smalltalk setGCSemaphore: index.
	process := [
		[[true] whileTrue:[
			sema wait.
			Smalltalk beep.
		]] ensure:[
			Smalltalk setGCSemaphore: 0.
			Smalltalk unregisterExternalObject: sema.
		].
	] fork.
	process inspect.
"! !

!VirtualMachine class methodsFor: 'vm statistics' stamp: 'IgorStasenko 8/17/2011 16:23'!
statisticsReport
	"Workspace openContents: (Smalltalk vm statisticsReport) label: 'VM Statistics'"

	| params oldSpaceEnd youngSpaceEnd memoryEnd fullGCs fullGCTime incrGCs incrGCTime tenureCount mcMisses mcHits icHits upTime sendCount upTime2 fullGCs2 fullGCTime2 incrGCs2 incrGCTime2 tenureCount2 str |
	params := self getParameters.
	oldSpaceEnd		:= self oldSpaceEnd.
	youngSpaceEnd		:= self youngSpaceEnd.
	tenureCount			:= self tenureCount.
	memoryEnd			:= self memoryEnd.
	fullGCs				:= self fullGCCount.
	fullGCTime			:= self totalFullGCTime.
	incrGCs				:= self incrementalGCCount.
	incrGCTime			:= self totalIncrementalGCTime.
 
	upTime := Time millisecondClockValue.
	str := (String new: 1000) writeStream.
	str	<< 'uptime			';
		print: (upTime / 1000 / 60 // 60); nextPut: $h;
		print: (upTime / 1000 / 60 \\ 60) asInteger; nextPut: $m;
		print: (upTime / 1000 \\ 60) asInteger; nextPut: $s; cr.

	str	<< 'memory			';
		<< memoryEnd asStringWithCommas; << ' bytes'; cr.
	str	<<	'	old			';
		<< oldSpaceEnd asStringWithCommas; << ' bytes (';
		print: ((oldSpaceEnd / memoryEnd * 100) roundTo: 0.1); << '%)'; cr.
	str	<< '	young		';
		<< (youngSpaceEnd - oldSpaceEnd) asStringWithCommas; << ' bytes (';
		print: ((youngSpaceEnd - oldSpaceEnd / memoryEnd * 100) roundTo: 0.1); << '%)'; cr.
	str	<< '	used		';
		<< youngSpaceEnd asStringWithCommas; << ' bytes (';
		print: ((youngSpaceEnd / memoryEnd * 100) roundTo: 0.1); << '%)'; cr.
	str	<< '	free		';
		<< (memoryEnd - youngSpaceEnd) asStringWithCommas; << ' bytes (';
		print: ((memoryEnd - youngSpaceEnd / memoryEnd * 100) roundTo: 0.1); << '%)'; cr.

	str	<< 'GCs				';
		<< (fullGCs + incrGCs) asStringWithCommas.
	fullGCs + incrGCs > 0 ifTrue: [
		str
			<< ' ('; 
			print: ((upTime / (fullGCs + incrGCs)) roundTo: 1); 
			<< 'ms between GCs)'
	].
	str cr.
	str	<< '	full			';
		print: fullGCs; << ' totalling '; << fullGCTime asStringWithCommas; << 'ms (';
		print: ((fullGCTime / upTime * 100) roundTo: 0.1);
		<< '% uptime)'.
	fullGCs = 0 ifFalse:
		[str	<< ', avg '; print: ((fullGCTime / fullGCs) roundTo: 0.1); << 'ms'].
	str	cr.
	str	<< '	incr		';
		print: incrGCs; << ' totalling '; << incrGCTime asStringWithCommas; << 'ms (';
		print: ((incrGCTime / upTime * 100) roundTo: 0.1);
		<< '% uptime), avg '; print: ((incrGCTime / incrGCs) roundTo: 0.1); << 'ms'; cr.
	str	<< '	tenures		';
		<< tenureCount asStringWithCommas.
	tenureCount = 0 ifFalse:
		[str << ' (avg '; print: (incrGCs / tenureCount) asInteger; << ' GCs/tenure)'].
	str	cr.

LastStats ifNil: [LastStats := Array new: 6]
ifNotNil: [
	upTime2 := upTime - (LastStats at: 1).
	fullGCs2 := fullGCs - (LastStats at: 2).
	fullGCTime2 := fullGCTime - (LastStats at: 3).
	incrGCs2 := incrGCs - (LastStats at: 4).
	incrGCTime2 := incrGCTime - (LastStats at: 5).
	tenureCount2 := tenureCount - (LastStats at: 6).

	str	<<  'Since last view	';
		<< (fullGCs2 + incrGCs2) asStringWithCommas.
	fullGCs2 + incrGCs2 > 0 ifTrue: [
		str
			<< ' ('; 
			print: ((upTime2 / (fullGCs2 + incrGCs2)) roundTo: 1); 
			<< 'ms between GCs)'.
	].
	str cr.
	str	<< '	uptime		'; print: ((upTime2 / 1000.0) roundTo: 0.1); << 's'; cr.
	str	<< '	full			';
		print: fullGCs2; << ' totalling '; << fullGCTime2 asStringWithCommas; << 'ms (';
		print: ((fullGCTime2 / upTime2 * 100) roundTo: 0.1);
		<< '% uptime)'.
	fullGCs2 = 0 ifFalse:
		[str	<< ', avg '; print: ((fullGCTime2 / fullGCs2) roundTo: 0.1); << 'ms'].
	str	cr.
	str	<< '	incr		';
		print: incrGCs2; << ' totalling '; << incrGCTime2 asStringWithCommas; << 'ms (';
		print: ((incrGCTime2 / upTime2 * 100) roundTo: 0.1);
		<< '% uptime), avg '.
	incrGCs2 > 0 ifTrue: [
		 str print: ((incrGCTime2 / incrGCs2) roundTo: 0.1); << 'ms'
	].
	str cr.
	str	<< '	tenures		';
		<< tenureCount2 asStringWithCommas.
	tenureCount2 = 0 ifFalse:
		[str << ' (avg '; print: (incrGCs2 / tenureCount2) asInteger; << ' GCs/tenure)'].
	str	cr.
].
	LastStats at: 1 put: upTime.
	LastStats at: 2 put: fullGCs.
	LastStats at: 3 put: fullGCTime.
	LastStats at: 4 put: incrGCs.
	LastStats at: 5 put: incrGCTime.
	LastStats at: 6 put: tenureCount.

	^ str contents
! !

!VirtualMachine class methodsFor: 'vm statistics' stamp: 'IgorStasenko 8/17/2011 16:25'!
statisticsShortReport
	"Convenience item for access to recent statistics only"
	
	"
	Workspace openContents: (Smalltalk vm statisticsShortReport) label: 'VM Recent Statistics'
	"

	^ self statisticsReport readStream
		upToAll: 'Since';
		nextLine;
		upToEnd! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 16:06'!
tenureCount
	"tenures of surving objects since startup (read-only)."

	^ self parameterAt: 11
! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 15:09'!
tenuringThreshold
	"survivor count tenuring threshold (read-write)"
	^ self parameterAt: 6! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 15:16'!
tenuringThreshold: anInteger
	"survivor count tenuring threshold (read-write).
	tenure when more than this many objects survive the GC"
	
	^ self parameterAt: 6 put: anInteger! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 14:52'!
totalFullGCTime
	"Answer the total time in milliseconds spent in full GCs since startup (read-only)."

	^ self parameterAt: 8
! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 14:53'!
totalGCTime
	^ self totalFullGCTime + self totalIncrementalGCTime! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 14:52'!
totalIncrementalGCTime
	"Answer the total time in milliseconds spent in incremental GCs since startup (read-only)."

	^ self parameterAt: 10
! !

!VirtualMachine class methodsFor: 'GC' stamp: 'CamilloBruni 7/18/2012 12:25'!
version
	"Return the version number string of the Virtual machine built"
	"
	Smalltalk vm version
	"
	^ String streamContents: [ :s |
		1007 to: 1009 do: [:n| 
			s nextPutAll: (self getSystemAttribute: n); cr ]]! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'IgorStasenko 8/16/2011 17:55'!
versionLabel
	"The use of this primitive not recommended. Not all VMs providing that"

	"Answer a string corresponding to the version of virtual machine. This
	represents the version level of the Smalltalk source code (interpreter
	and various plugins) that is translated to C by a CCodeGenerator,  in
	addition to the external platform source code, typically written in C and
	managed separately for each platform.
	
	This invokes an optional primitive that may not be available on all virtual
	machines. See also vmVersion, which answers a string identifying the image
	from which virtual machine sources were generated."

	"Smalltalk vm versionLabel"

	<primitive: 'primitiveVMVersion'>
	self notify: 'This virtual machine does not support the optional primitive #primitiveVMVersion' translated.
	^''! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'CamilloBruni 5/13/2012 19:19'!
vmDirectory 
	^ self directory
! !

!VirtualMachine class methodsFor: 'attributes' stamp: 'CamilloBruni 5/13/2012 19:20'!
vmFileName
	^ self fileName! !

!VirtualMachine class methodsFor: 'cleaning' stamp: 'MarianoMartinezPeck 4/27/2012 20:02'!
voidCogVMState
	"Void any internal caches the VM maintains other than the method lookup caches.
	 These comprise
		- the stack zone, where method activations are stored, and
		- the machine code zone, where the machine code form of CompiledMethods is held."
	<primitive: 214>
	^self primitiveFailed! !

!VirtualMachine class methodsFor: 'accessing' stamp: 'IgorStasenko 8/17/2011 15:03'!
wordSize
	^ WordSize ifNil: [WordSize := [self parameterAt: 40] on: Error do: [4]]! !

!VirtualMachine class methodsFor: 'GC' stamp: 'IgorStasenko 8/17/2011 16:08'!
youngSpaceEnd
	"end of young-space"
		
	^ self parameterAt: 2
! !
SmalltalkImage initialize!
PrettyPrinting initialize!
ExternalSemaphoreTable initialize!
ISOLanguageDefinition initialize!
TextDiffBuilder initialize!
MD5NonPrimitive initialize!
DummySoundSystem initialize!
ExternalDropHandler initialize!
DigitalSignatureAlgorithm initialize!
Clipboard initialize!
MczInstaller initialize!
DefaultCommandLineHandler initialize!
Locale initialize!
SecureHashAlgorithm initialize!
FileServices initialize!
OSPlatform initialize!
SHA1 initialize!
DefaultExternalDropHandler initialize!
SystemAnnouncer initialize!
ChangeSet initialize!
